using System;
using System.Collections;
using RepPP.Toolkit;
using System.ComponentModel;

namespace RepPP.Framework.Window {
	//******************************************************************
	//
	/// <summary>
	/// Implement a search method for direct search
	/// </summary>
	//
	//******************************************************************
	public class DirectSearchMethod : ISearchMethod {
	    /// <summary>Where Clause</summary>
        private string                                  m_strWhereClause;
	    /// <summary>Join Clause</summary>
        private string                                  m_strJoinClause;
        /// <summary> Event triggered when the search method is about to be executed</summary>
        public event BeforeExecuteEventHandler          BeforeExecute;
        /// <summary> Event triggered when the validation of the search method's parameters is done</summary>
        public event CancelEventHandler                 AfterValidate;

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    //
	    //******************************************************************
        public DirectSearchMethod() {
            bool bIgnore;

            m_strWhereClause = string.Empty;
            m_strJoinClause  = string.Empty;
            bIgnore          = (AfterValidate == null);  //Prevent compilation warning
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Name:                       Gets the Name of the search method.
		/// </summary>
		//
	    //******************************************************************
        public string Name {
            get {
               return(string.Empty); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// AccelerationKey:            Gets the Acceleration key of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public int AccelerationKey {
            get {
               return(0); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// SqlCommand:                 Gets the Sql Command of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public string SqlCommand {
            get {
               return(string.Empty); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Description:                Gets the Description of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public string Description {
            get {
               return(string.Empty); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// HasArguments:               Checks if the search method has arguments
		/// </summary>
		//
	    //******************************************************************
        public bool HasArguments {
            get {
               return(false); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// WhereClause:                Gets/Sets the where clause of the direct 
        ///                             search.
		/// </summary>
		//
	    //******************************************************************
        public string WhereClause {
            get {
               return(m_strWhereClause); 
            }
            set {
               m_strWhereClause = value; 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// JoinClause:                 Gets/Sets the inner join clause of 
        ///                             the direct search.
		/// </summary>
		//
	    //******************************************************************
        public string JoinClause {
            get {
               return(m_strJoinClause); 
            }
            set {
               m_strJoinClause = value; 
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnBeforeExecute                  Triggers the BeforeExecute event
        /// </summary>
        /// <param name="e">                 Event argument</param>
        //
        //******************************************************************
        protected virtual void OnBeforeExecute(BeforeExecuteEventArgs e) {
            if (BeforeExecute != null) {
                BeforeExecute(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeUsingSearchMethod Read the rowset tree using a search method
        /// </summary>
        /// <param name="rowsetTree">        RowsetTreeDef instance</param>
        /// <param name="dataAccess">   Data access object</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //******************************************************************
        public bool ReadRowsetTreeUsingSearchMethod(RowsetTree rowsetTree, ISBDataAccess dataAccess) {
            bool                    bRetVal              = true;  
            RowsetTreeDefNode               node;
            string                  strWhereClause       = String.Empty;
            string                  strOriginalSelectCmd;
            BeforeExecuteEventArgs  e;

            node                 = rowsetTree.RowsetTreeDef.Nodes[0];
            strOriginalSelectCmd = node.SelectSqlCommand;
            try {
                if (WhereClause != string.Empty) {
                    strWhereClause = " WHERE " + WhereClause; 
                }
                if (JoinClause != string.Empty && strOriginalSelectCmd.IndexOf("DISTINCT") <= 0) {
                    node.SelectSqlCommand = strOriginalSelectCmd.Insert(7, "DISTINCT ");
                    strWhereClause        = JoinClause + " " + strWhereClause;
                }
                e              = new BeforeExecuteEventArgs(strOriginalSelectCmd, strWhereClause, rowsetTree, dataAccess);
                OnBeforeExecute(e);
                if (e.Status == SearchMethodExecutionStatusE.SMES_Cancel) {
                    bRetVal = false;
                } else {
                    if (e.Status == SearchMethodExecutionStatusE.SMES_Continue) {
                        strWhereClause = e.SqlWhereClause;
                        node.SetSelectSqlWhere(strWhereClause);
                        if (dataAccess != null) {
                            dataAccess.LoadSelectionBufferData(rowsetTree, string.Empty);
                        } else {
                            rowsetTree.ReadFromDb();
                        }
                    }
                }
            }
            finally {
                node.SelectSqlCommand = strOriginalSelectCmd;
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeUsingSearchMethod   Read the card using a search method
        /// </summary>
        /// <param name="rowsetTree">        RowsetTreeDef instance</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //******************************************************************
        public bool ReadRowsetTreeUsingSearchMethod(RowsetTree rowsetTree) {
            return ReadRowsetTreeUsingSearchMethod(rowsetTree, null);
        }
    } //Class
} // Namespace
