//*********************************************************
//
// QueryEditor:    SQL Command Editor User Control
//
//
// (c) 2006 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Windows.Forms;
using System.Xml;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Miscellaneous {
    //******************************************************************        
    //
    /// <summary>
	/// Graphic SQL Command Editor User Control. It shows the list of 
	/// tables, columns, rowsets, fields and function available and lets 
	/// the user select them by double-clicking on an item. 
	/// </summary>
	/// <remarks>
	/// The list of available functions must be stored in a file called
	/// RepPP_Functions.xml in RepPP.DirType.sdDirBin. The user may specify
	/// an additional file that contains additional functions.
	/// </remarks>
    //
    //******************************************************************        
	public class QueryEditor : System.Windows.Forms.UserControl, IUIConfiguration {
        private const string m_strDefaultWhere         = "WHERE (APPSYSTNAME = '{0}')";
        private const string m_strSelectTabsAndCols    = "SELECT APPSYSTNAME,SECTFULLPATH,TABLENAME,SD_NULLIF(COLPHYSNAME, COLNAME) as COLNAME FROM SYSTTABLECOL {0} ORDER BY APPSYSTNAME,SECTFULLPATH,TABLENAME,COLPOS";
        private const string m_strSelectRowsetsAndFlds = "SELECT APPSYSTNAME,SECTFULLPATH,MODNAME,GRPNAME,QUNAME FROM APPQUEST {0} ORDER BY APPSYSTNAME,SECTFULLPATH,MODNAME,GRPNAME,QUINDEX";
        /// <summary> Rep++ Application </summary>
        private IApplication                            m_app   = null; 
        /// <summary> Where clause that indicates the accessible sections</summary>
        private string                                  m_strSectWhere = null;
        /// <summary>Configruration Helper</summary>
        private ConfigHelper                            m_configHelper = null;
        private System.ComponentModel.IContainer        components;
        private System.Windows.Forms.TabControl         m_tabMain;
        private System.Windows.Forms.TabPage            m_pageQuery;
        private System.Windows.Forms.TabPage            m_pageOptions;
        private System.Windows.Forms.ImageList m_imageLis16x16;
        private System.Windows.Forms.Panel              m_pnlBottom;
        private System.Windows.Forms.Label              m_lblTabCol;
        private System.Windows.Forms.Label              m_lblRowsetFld;
        private System.Windows.Forms.Label              m_lblFunc;
        private RepPP.Miscellaneous.QueryEditor.TVCInst m_tvTablesAndCols;
        private RepPP.Miscellaneous.QueryEditor.TVCInst m_tvRowsetsAndFlds;
        private System.Windows.Forms.ListView           m_lvFunctions;
        private System.Windows.Forms.ColumnHeader       m_colName;
        private System.Windows.Forms.TextBox            m_txtFuncFile;
        private System.Windows.Forms.Label              m_lblLoadFuncFrom;
        private System.Windows.Forms.CheckBox           m_chkAddOnDblClk;
        private System.Windows.Forms.CheckBox           m_chkPrefixColNames;
        private System.Windows.Forms.OpenFileDialog     m_dlgOpenFile;
        private System.Windows.Forms.Panel              m_pnlTop;
        private System.Windows.Forms.Label              m_lblQuery;
        private System.Windows.Forms.TextBox            m_txtQuery;
        private Label                                   m_lblTemplate;
        private Button                                  m_btnTemplate;
        private ComboBox                                m_cbxTemplates;
        private System.Windows.Forms.Button             m_btnOpenFile;

        //******************************************************************        
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************        
		public QueryEditor() {
            InitializeComponent();
            m_pnlBottom.Height           = Height / 3;
            m_cbxTemplates.SelectedIndex = 0;
        }

        //******************************************************************        
        //
		/// <summary> 
		/// Dispose                     Clean up any resources being used.
		/// </summary>
        //
        //******************************************************************        
		protected override void Dispose(bool disposing) {
			if(disposing) {
				if (m_configHelper != null){
                    m_configHelper.SaveState(this);
                    m_configHelper.SaveToFile();
                    m_configHelper = null;
                }
                if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

        //******************************************************************        
        //
        /// <summary>
        /// Query                       Gets/Sets the text of the query 
        /// </summary>
        //
        //******************************************************************        
        public string Query {
            get {
                return(m_txtQuery.Text);
            }
            set {
                m_txtQuery.Text = value;
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// QueryLabel                  Gets/Sets the text of the query label
        /// </summary>
        //
        //******************************************************************        
        public string QueryLabel {
            get {
                return(m_lblQuery.Text);
            }
            set {
                m_lblQuery.Text = value;
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// PrefixColNames                  Gets a value that indicates if 
        ///                                 column names should be prefixed
        /// </summary>
        //
        //******************************************************************        
        private bool PrefixColNames {
            get {
                return(m_chkPrefixColNames.Checked);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// AddOnDblClk                     Gets a value that indicates if 
        ///                                 double-clicking on an item will
        ///                                 add it to the query.
        /// </summary>
        //
        //******************************************************************        
        private bool AddOnDblClk {
            get {
                return(m_chkAddOnDblClk.Checked);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// ShowTemplates                   Sets a value that indicates if 
        ///                                 the controls that enable the selection
        ///                                 of a template are showns
        /// </summary>
        //
        //******************************************************************        
        public bool ShowTemplates {
            set {
                m_btnTemplate.Visible  = value;
                m_cbxTemplates.Visible = value;
                m_lblTemplate.Visible  = value;
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// SetControlsText                 Sets the text of controls depending
        ///                                 on the current language
        /// </summary>
        /// <param name="translator">       Translator for custom User Translation</param>
        //
        //******************************************************************        
        private void SetControlsText(ControlTextTranslator translator) {
            if (m_app != null && m_app.LanguageCode == LanguageCode.sdFrench) {
                m_lblTabCol.Text         = "Tables et colonnes"; 
                m_lblRowsetFld.Text      = "Rowsets et champs";
                m_lblFunc.Text           = "Fonctions";
                m_lblQuery.Text          = "Requte";
                m_colName.Text           = "Nom";
                m_pageOptions.Text       = "Options";
                m_pageQuery.Text         = "Requte";
                m_chkAddOnDblClk.Text    = "Ajouter les Tables, colonnes, champs et fonctions en double cliquant sur l'item correspondant.";
                m_chkPrefixColNames.Text = "Prfixer les noms des colonnes avec les noms de table.";
                m_lblLoadFuncFrom.Text   = "Charger la liste des fonctions  partir du fichier suivant :";
                m_btnTemplate.Text       = "Inserer";
                m_lblTemplate.Text       = "Commandes frquement utilises";
                m_cbxTemplates.Items[1]  = "SELECT (Liste de choix)";
            }
            //User override translation !
            if (translator != null) {
                translator.TranslateControl(this);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// ReadTvContent                   Reads the content of a TreeView from
        ///                                 the database
        /// </summary>
        /// <param name="strPartialSqlCmd"> The partial SqlCommand</param>
        /// <param name="tv">               The TreeView control</param>
        //
        //******************************************************************        
        private void ReadTvContent(string strPartialSqlCmd, TreeView tv) {
            Connection conn;
            SqlCursor  cursor;
            TreeNode   nodeCurrParent        = null;
            TreeNode   nodeNew;
            string     strSqlCmd;
            string     strLastParentFullKey  = String.Empty;
            string     strCurrParent;
            string     strCurrParentFullKey;
            string     strSystem;
            string     strSection;
            string     strModule             = String.Empty;
            string     strLeaf;
            string     strText;
            bool       bRowsets;
            int        iImageIndex;


            if (m_app != null) {
                bRowsets    = (strPartialSqlCmd.IndexOf("MODNAME") != -1);
                iImageIndex = (bRowsets ? 2 : 0);
                if (m_strSectWhere == null || m_strSectWhere == String.Empty) {
                    m_strSectWhere = String.Format(m_strDefaultWhere, m_app.SystemName);
                }
                strSqlCmd = String.Format(strPartialSqlCmd, m_strSectWhere);
                conn      = m_app.RepositoryConnection;
                if (conn.Execute(strSqlCmd, out cursor) == 0) {
                    while (cursor.Next() == 0) {
                        strSystem = cursor.GetColumnValue("APPSYSTNAME");
                        strSection = cursor.GetColumnValue("SECTFULLPATH");
                        if (bRowsets) {
                            strModule     = cursor.GetColumnValue("MODNAME");
                            strCurrParent = cursor.GetColumnValue("GRPNAME");
                            strLeaf       = cursor.GetColumnValue("QUNAME");
                        } else {
                            strCurrParent = cursor.GetColumnValue("TABLENAME");
                            strLeaf       = cursor.GetColumnValue("COLNAME");
                            if (conn.IsReservedKeyword(strLeaf)) {
                                strLeaf   = conn.DecorateReservedKeyword(strLeaf);
                            }
                        }
                        if (bRowsets) {
                           strCurrParentFullKey = String.Format("{0}:{1}:{2}:{3}", 
                                                                strSystem, 
                                                                strSection, 
                                                                strModule,
                                                                strCurrParent);
                        } else {
                           strCurrParentFullKey = String.Format("{0}:{1}:{2}", 
                                                                strSystem, 
                                                                strSection,
                                                                strCurrParent);
                        }
                        if (strCurrParentFullKey != strLastParentFullKey){
                            strText                    = (bRowsets ?  strModule + ": " + strCurrParent : strCurrParent);
                            nodeCurrParent             = tv.Nodes.Add(strText);
                            nodeNew                    = nodeCurrParent;
                            nodeNew.ImageIndex         = iImageIndex;
                            nodeNew.SelectedImageIndex = iImageIndex;
                            nodeNew.Tag                = strCurrParentFullKey + ":" + strLeaf;
                            strLastParentFullKey       = strCurrParentFullKey;
                        }
                        nodeNew                    = nodeCurrParent.Nodes.Add(strLeaf);
                        nodeNew.ImageIndex         = iImageIndex + 1;
                        nodeNew.SelectedImageIndex = iImageIndex + 1;
                        nodeNew.Tag                = strCurrParentFullKey + ":" + strLeaf;
                    }
                    cursor.Close();
                }
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// ReadFunctions               Reads the list of functions from an
        ///                             XML file
        /// </summary>
        /// <param name="strFileName">  Path to the function file</param>
        /// <returns>
        /// True if the file was read successfully, False otherwise
        /// </returns>
        //
        //******************************************************************        
        private bool ReadFunctions(string strFileName) {
            XmlTextReader reader;
            ListViewItem  item;
            string        strLang;
            string        strTextTag  = "Text";
            string        strLabelTag = "Label";
            bool          bRetVal     = true;

            strLang      = ((int)((m_app.LanguageCode == LanguageCode.sdFrench) ? LanguageCode.sdFrench : LanguageCode.sdEnglish)).ToString();
            strTextTag  += strLang;
            strLabelTag += strLang;
            if (m_app != null) {            
                if (File.Exists(strFileName)) {
                    try {
                        using (reader = new XmlTextReader(strFileName)) {
                            while (reader.Read()) {
                                reader.MoveToContent();
                                if (reader.LocalName == "Function") {
                                    item = new ListViewItem(reader["Name"], 4);
                                    item.ToolTipText = reader[strLabelTag];
                                    item.Tag         = reader[strTextTag];
                                    m_lvFunctions.Items.Add(item);
                                }
                            }
                        }
                    }
                    catch {
                        MessageBox.Show(m_app.GetAtomText("errCannotReadFile", RepPP.AtomType.sdAtomErr), 
                                        m_app.GetAtomText("txtPrgTitle",       RepPP.AtomType.sdAtomMsg),
                                        MessageBoxButtons.OK, 
                                        MessageBoxIcon.Error);
                        bRetVal = false;
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************        
        //
        /// <summary>
        /// ReadFunctions               Reads the Normalized functions from an
        ///                             XML file named RepPP_Functions.xml
        ///                             that is located in RepPP.DirType.sdDirBin
        ///                             and optionally from a user-defined file
        /// </summary>
        //
        //******************************************************************        
        private void ReadFunctions() {

            m_lvFunctions.Items.Clear();
            ReadFunctions(Path.Combine(m_app.Tool.GetDir(RepPP.DirType.sdDirBin),"RepPP_Functions.xml"));
            if (m_txtFuncFile.Text != string.Empty) {
                if (!ReadFunctions(m_txtFuncFile.Text)) {
                    m_txtFuncFile.Text = string.Empty;    
                }
            }
        }

        //******************************************************************        
        //
        /// <summary>
        ///  Initialize                 Initializes the control and loads
        ///                             the available tables, columns, rowsets,
        ///                             fields and functions.
        ///                                                                 
        /// </summary>
        /// <param name="app">          Rep++ Application</param>
        /// <param name="strSectWhere"> Where clause set the accessible sections</param>
        /// <param name="strQuery">     The initial query text</param>
        /// <param name="translator">   Translator for custom User Translation</param>
        //
        //******************************************************************  
        public void Initialize(RepPP.IApplication app, string strSectWhere, string strQuery, ControlTextTranslator translator) {
            m_app          = app;
            m_strSectWhere = strSectWhere;
            Query          = strQuery;
            ReadTvContent(m_strSelectTabsAndCols, m_tvTablesAndCols);
            ReadTvContent(m_strSelectRowsetsAndFlds, m_tvRowsetsAndFlds);
			frmParams.InitAvailableColsAndFlds(m_tvTablesAndCols, m_tvRowsetsAndFlds);
            ReadFunctions();
            SetControlsText(translator);
        }

        //******************************************************************        
        //
        /// <summary>
        ///  Initialize                 Initializes the control and loads
        ///                             the available tables, columns, rowsets,
        ///                             fields and functions.
        ///                                                                 
        /// </summary>
        /// <param name="app">          Rep++ Application</param>
        /// <param name="strSectWhere"> Where clause set the accessible sections</param>
        /// <param name="strQuery">     The initial query text</param>
        //
        //******************************************************************        
        public void Initialize(RepPP.IApplication app, string strSectWhere, string strQuery) {
            Initialize(app, strSectWhere, strQuery, null);
        }

        //******************************************************************        
        //
        /// <summary>
        ///  CreateConfigHelper         Creates the configuration helper and
        ///                             loads the previous config (if any)
        ///                                                                 
        /// </summary>
        //
        //******************************************************************        
        private void CreateConfigHelper() {
            string  strCfgFile;

            strCfgFile = Path.Combine(m_app.Tool.GetDir(RepPP.DirType.sdDirBin),
                                      "RepPP_QueryEditor.cfg");
            m_configHelper = new ConfigHelper(strCfgFile);
            m_configHelper.LoadState(this);
        }

        //******************************************************************        
        //
        /// <summary>
        ///  OnLoad                     Called when the control is being 
        ///                             loaded
        ///                                                                 
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //******************************************************************        
        protected override void OnLoad(EventArgs e) {
            base.OnLoad (e);
            if (!DesignMode) {
                CreateConfigHelper();
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// GetFldText                  Extacts the "GROUP.FIELD" name from 
        ///                             "MODULE: GROUP.FIELD"
        /// </summary>
        /// <param name="strText">      Raw field name</param>
        //
        //******************************************************************        
        private string GetFldText(string strText){
            string strRetVal;
            int    iIndex;

            iIndex = strText.IndexOf(':');
            if (iIndex == -1) {
                throw new Exception("Invalid Field Name");
            }
            strRetVal = ":" + strText.Substring(iIndex + 2);
            return(strRetVal);
        }

        //******************************************************************        
        //
        /// <summary>
        /// InsertText                  Inserts a text at the current cursor
        ///                             location in the query TextBox
        /// </summary>
        /// <param name="strText">      Text to be inserted</param>
        //
        //******************************************************************        
        private void InsertText(string strText){
            int     iSelStart;
            string  strQuery;

            iSelStart = m_txtQuery.SelectionStart;
            strQuery  = m_txtQuery.Text;
            if (iSelStart > 0 && strQuery.Length >= iSelStart){
                if (strText.StartsWith(":") && strQuery[iSelStart - 1] == ':') {
                    strText = strText.Substring(1);
                } else if (!Char.IsWhiteSpace(strQuery, iSelStart - 1) ) {
                    strText = " " + strText;
                }
            }
            m_txtQuery.Select();
            //Issue: Can't undo if we just change the Text property. So we just past the new content to undoable
            m_txtQuery.SelectAll();   
            m_txtQuery.Paste(m_txtQuery.Text.Insert(iSelStart, strText));
            m_txtQuery.SelectionStart  = iSelStart;
            m_txtQuery.SelectionLength = strText.Length;
            m_txtQuery.ScrollToCaret();
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_tvTablesAndCols_DoubleClick   Called a table or column node is
        ///                                 double-clicked.
        /// </summary>
        /// <param name="sender">           Treeview</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************        
        private void m_tvTablesAndCols_DoubleClick(object sender, System.EventArgs e) {
            if (AddOnDblClk && m_tvTablesAndCols.SelectedNode != null) {
                if (PrefixColNames) {
                    InsertText(m_tvTablesAndCols.SelectedNode.FullPath.Replace('\\','.'));
                } else {
                    InsertText(m_tvTablesAndCols.SelectedNode.Text);
                }
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_tvRowsetsAndFlds_DoubleClick  Called when a rowset or field node is
        ///                                 double-clicked.
        /// </summary>
        /// <param name="sender">           Treeview</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************        
        private void m_tvRowsetsAndFlds_DoubleClick(object sender, System.EventArgs e) {
            if (AddOnDblClk && m_tvRowsetsAndFlds.SelectedNode != null) {
                if (m_tvRowsetsAndFlds.SelectedNode.Parent != null) {
                    InsertText(GetFldText(m_tvRowsetsAndFlds.SelectedNode.FullPath.Replace('\\','.')));
                }
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_lvFunctions_DoubleClick       Called a function is double-clicked.
        /// </summary>
        /// <param name="sender">           ListView</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************        
        private void m_lvFunctions_DoubleClick(object sender, System.EventArgs e) {
            if (AddOnDblClk) {
                InsertFunction(m_lvFunctions.SelectedItems[0]);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// InsertFunction                  Inserts a function
        /// </summary>
        /// <param name="listViewItem">     ListViewItem of the selected function</param>
        //
        //******************************************************************        
        private void InsertFunction(ListViewItem listViewItem) {
            string strText;
            
			strText =  frmParams.Show(this.ParentForm, 
                                      listViewItem.SubItems[0].Text, 
                                      listViewItem.Tag as string); 
            InsertText(strText);
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_txtQuery_DragDrop             Called following a drop .
        /// </summary>
        /// <param name="sender">           TextBox</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************        
        private void m_txtQuery_DragDrop(object sender, System.Windows.Forms.DragEventArgs e) {
            if (e.Data.GetDataPresent(DataFormats.Text)) {
                InsertText(e.Data.GetData(DataFormats.Text).ToString());
            } else if (e.Data.GetDataPresent(typeof(ListViewItem))) {
                InsertFunction(e.Data.GetData(typeof(ListViewItem)) as ListViewItem);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_txtQuery_DragEnter            Called following a drop .
        /// </summary>
        /// <param name="sender">           TextBox</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************        
        private void m_txtQuery_DragEnter(object sender, System.Windows.Forms.DragEventArgs e) {
            if (e.Data.GetDataPresent(DataFormats.Text) || e.Data.GetDataPresent(typeof(ListViewItem))) {
                e.Effect = DragDropEffects.Copy;
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_txtQuery_DragOver             Called following a drop .
        /// </summary>
        /// <param name="sender">           TextBox</param>
        /// <param name="e">                Event Argument</param>
        //
        //******************************************************************  
        private void m_txtQuery_DragOver(object sender, DragEventArgs e) {
            Point   pt;
            Point   ptIn;
            int     iPosition;
            string  strLast;
            Size    size;

            m_txtQuery.Focus();
            ptIn      = m_txtQuery.PointToClient(new Point(e.X, e.Y));
            iPosition = m_txtQuery.GetCharIndexFromPosition(ptIn);
            if (iPosition == m_txtQuery.Text.Length - 1) {
                if (ptIn.X < 0) {
                    iPosition = 0;
                } else {
                    strLast = m_txtQuery.Text.Substring(m_txtQuery.Text.Length - 1, 1);
                    size    = TextRenderer.MeasureText(strLast, m_txtQuery.Font);
                    pt      = m_txtQuery.GetPositionFromCharIndex(m_txtQuery.Text.Length - 1);
                    if (ptIn.X > (pt.X + (size.Width/4))) {
                        iPosition = m_txtQuery.Text.Length;
                    }
                }
            }
            m_txtQuery.SelectionStart  = iPosition;
            m_txtQuery.SelectionLength = 0;
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_tvTablesAndCols_ItemDrag      Handles the ItemDrag event for the
        ///                                 two tree views and the list view
        /// </summary>
        /// <param name="sender">           Event Sender</param>
        /// <param name="e">                Event Arguments</param>
        //
        //******************************************************************        
        private void m_tvTablesAndCols_ItemDrag(object sender, System.Windows.Forms.ItemDragEventArgs e) {
            TreeNode      node;
            string        strText;
            bool          bIsCol;
            bool          bIsRowset;
            bool          bIsFld;

            if (sender is TreeView) {
                node      = e.Item as TreeNode;
                bIsCol    = (sender == m_tvTablesAndCols && node.Parent != null);
                bIsRowset = (sender == m_tvRowsetsAndFlds && node.Parent == null);
                bIsFld    = (sender == m_tvRowsetsAndFlds && node.Parent != null);
                if (node != null && !bIsRowset) {
                    if(!PrefixColNames && bIsCol) {
                        strText = node.Text;
                    } else {
                        strText = node.FullPath.Replace('\\','.');
                    }
                    if (bIsFld) {
                        strText = GetFldText(strText);
                    }
                    this.DoDragDrop(strText, DragDropEffects.Copy);
                }
            } else if (sender is ListView) {
                this.DoDragDrop(e.Item, DragDropEffects.Copy);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// LoadState                       Implements IUIConfiguration.LoadState
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        //
        //******************************************************************        
        private void m_btnOpenFile_Click(object sender, System.EventArgs e) {
            if (m_dlgOpenFile.ShowDialog(this) == DialogResult.OK) {
                m_txtFuncFile.Text = m_dlgOpenFile.FileName;
                ReadFunctions();
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// m_tabMain_SelectedIndexChanged  Handles the SelectedIndexChanged 
        ///                                 event for the main tab control
        /// </summary>
        /// <param name="sender">           Event Sender</param>
        /// <param name="e">                Event Arguments</param>
        //
        //******************************************************************        
        private void m_tabMain_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (m_tabMain.SelectedTab == m_pageQuery) {
                ReadFunctions();
            }
        }



        //******************************************************************        
        //
        /// <summary>
        /// m_tabMain_SelectedIndexChanged  Handles the SelectedIndexChanged 
        ///                                 event for the main tab control
        /// </summary>
        /// <param name="sender">           Event Sender</param>
        /// <param name="e">                Event Arguments</param>
        //
        //******************************************************************        
        private void m_btnTemplate_Click(object sender, EventArgs e) {
            string strQuery;
            string strText;

            switch (m_cbxTemplates.SelectedIndex) {
            case 0 /* SELECT */:
                strQuery = "SELECT \r\nINTO \r\nFROM \r\nWHERE \r\nORDER BY ";
                break;
            case 1  /* SELECT for choice list  */:
                strQuery = "SELECT \r\nINTO :$CODE, :$VAL, :$DESC \r\nFROM \r\nWHERE \r\nORDER BY ";
                break;
            case 2  /* INSERT */:
                strQuery = "INSERT INTO \r\nVALUES ()";
                break;
            case 3  /* UPDATE */:
                strQuery = "UPDATE \r\nSET \r\nWHERE ";
                break;
            case 4  /* DELETE */:
                strQuery = "DELETE FROM \r\nWHERE ";
                break;
            default:
                throw new Exception("Invalid Template Type!!!");
            }
            if (m_app.LanguageCode == LanguageCode.sdFrench) {
                strText = "Cette opration va craser le texte de votre requte. Voulez-vous continuer?";
            } else {
                strText = "This operation will overwrite the text of your query. Do you want to continue?";
            }
            if (m_txtQuery.Text.Equals(string.Empty) ||
                MessageBox.Show(strText, 
                                m_app.GetAtomText("txtPrgTitle", RepPP.AtomType.sdAtomMsg), 
                                MessageBoxButtons.YesNo, 
                                MessageBoxIcon.Question, 
                                MessageBoxDefaultButton.Button2) == DialogResult.Yes) {
                m_txtQuery.SelectAll();
                m_txtQuery.Paste(strQuery);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// LoadState                       Implements IUIConfiguration.LoadState
        /// </summary>
        /// <param name="helper">           Configuration Helper</param>
        /// <param name="node">             Configuration Node</param>
        //
        //******************************************************************        
        public void LoadState(UIConfigurationHelper helper, XmlNode node) {
            string          strSelectedTabOrCol    = string.Empty;
            string          strSelectedRowsetOrFld = string.Empty;
            string          strVersion;
            int             iSelectedFunc          = -1;
            TreeNode        treeNode;
            ListViewItem    item;
            
            strVersion = helper.GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {

                strSelectedTabOrCol = helper.GetNodeAttribute(node, "SELECTED_TAB_OR_COL");
                if (strSelectedTabOrCol != String.Empty) {
                    treeNode = m_tvTablesAndCols.FindNodeUsingFullKey(strSelectedTabOrCol);
                    if (treeNode != null) {
                        m_tvTablesAndCols.SelectedNode = treeNode;
                        treeNode.EnsureVisible();
                    }
                }
                strSelectedRowsetOrFld = helper.GetNodeAttribute(node, "SELECTED_GRP_OR_FLD");
                if (strSelectedRowsetOrFld != String.Empty) {
                    treeNode = m_tvRowsetsAndFlds.FindNodeUsingFullKey(strSelectedRowsetOrFld);
                    if (treeNode != null) {
                        m_tvRowsetsAndFlds.SelectedNode = treeNode;
                        treeNode.EnsureVisible();
                    }
                }
                m_txtFuncFile.Text = helper.GetNodeAttribute(node, "OPTION_FUNC_FILE");
                if (m_txtFuncFile.Text != String.Empty){
                    if (!ReadFunctions(m_txtFuncFile.Text)) {
                        m_txtFuncFile.Text = string.Empty;
                    }
                }
                iSelectedFunc = helper.GetNodeAttribute(node, "SELECTED_FUNC", -1);
                if (iSelectedFunc > -1 && m_lvFunctions.Items.Count > iSelectedFunc) {
                    item = m_lvFunctions.Items[iSelectedFunc];
                    item.Selected = true;                
                    item.EnsureVisible();
                }
                m_chkPrefixColNames.Checked = (helper.GetNodeAttribute(node, "OPTION_PREFIX_CLOUMNS") == bool.TrueString);
                m_chkAddOnDblClk.Checked    = (helper.GetNodeAttribute(node, "OPTION_ADD_DBLCLK") == bool.TrueString);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// SaveState                       Implements IUIConfiguration.SaveState
        /// </summary>
        /// <param name="helper">           Configuration Helper</param>
        /// <param name="node">             Configuration Node</param>
        //
        //******************************************************************        
        public void SaveState(UIConfigurationHelper helper, XmlNode node) {
            string strSelectedTabOrCol    = string.Empty;
            string strSelectedRowsetOrFld = string.Empty;
            int    iSelectedFunc          = -1;

            if (m_tvTablesAndCols.SelectedNode != null) {
                strSelectedTabOrCol = m_tvTablesAndCols.GetCurrentlySelectedNodeFullKey(); 
            }
            if (m_tvRowsetsAndFlds.SelectedNode != null) {
                strSelectedRowsetOrFld = m_tvRowsetsAndFlds.GetCurrentlySelectedNodeFullKey(); 
            }
            if (m_lvFunctions.SelectedIndices.Count > 0) {
                iSelectedFunc = m_lvFunctions.SelectedIndices[0];
            }

            node.RemoveAll();
            helper.SetNodeAttribute(node, "VERSION",                "1");
            helper.SetNodeAttribute(node, "SELECTED_TAB_OR_COL",    strSelectedTabOrCol);
            helper.SetNodeAttribute(node, "SELECTED_GRP_OR_FLD",    strSelectedRowsetOrFld);
            helper.SetNodeAttribute(node, "SELECTED_FUNC",          iSelectedFunc);
            helper.SetNodeAttribute(node, "OPTION_PREFIX_CLOUMNS",  m_chkPrefixColNames.Checked.ToString());
            helper.SetNodeAttribute(node, "OPTION_ADD_DBLCLK",      m_chkAddOnDblClk.Checked.ToString());
            helper.SetNodeAttribute(node, "OPTION_FUNC_FILE",       m_txtFuncFile.Text);
        }

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(QueryEditor));
            this.m_tabMain = new System.Windows.Forms.TabControl();
            this.m_pageQuery = new System.Windows.Forms.TabPage();
            this.m_pnlTop = new System.Windows.Forms.Panel();
            this.m_cbxTemplates = new System.Windows.Forms.ComboBox();
            this.m_lblTemplate = new System.Windows.Forms.Label();
            this.m_btnTemplate = new System.Windows.Forms.Button();
            this.m_lblQuery = new System.Windows.Forms.Label();
            this.m_txtQuery = new System.Windows.Forms.TextBox();
            this.m_pnlBottom = new System.Windows.Forms.Panel();
            this.m_lblTabCol = new System.Windows.Forms.Label();
            this.m_lblRowsetFld = new System.Windows.Forms.Label();
            this.m_lblFunc = new System.Windows.Forms.Label();
            this.m_tvTablesAndCols = new RepPP.Miscellaneous.QueryEditor.TVCInst();
            this.m_imageLis16x16 = new System.Windows.Forms.ImageList(this.components);
            this.m_tvRowsetsAndFlds = new RepPP.Miscellaneous.QueryEditor.TVCInst();
            this.m_lvFunctions = new System.Windows.Forms.ListView();
            this.m_colName = new System.Windows.Forms.ColumnHeader();
            this.m_pageOptions = new System.Windows.Forms.TabPage();
            this.m_btnOpenFile = new System.Windows.Forms.Button();
            this.m_txtFuncFile = new System.Windows.Forms.TextBox();
            this.m_lblLoadFuncFrom = new System.Windows.Forms.Label();
            this.m_chkAddOnDblClk = new System.Windows.Forms.CheckBox();
            this.m_chkPrefixColNames = new System.Windows.Forms.CheckBox();
            this.m_dlgOpenFile = new System.Windows.Forms.OpenFileDialog();
            this.m_tabMain.SuspendLayout();
            this.m_pageQuery.SuspendLayout();
            this.m_pnlTop.SuspendLayout();
            this.m_pnlBottom.SuspendLayout();
            this.m_pageOptions.SuspendLayout();
            this.SuspendLayout();
            // 
            // m_tabMain
            // 
            this.m_tabMain.Controls.Add(this.m_pageQuery);
            this.m_tabMain.Controls.Add(this.m_pageOptions);
            this.m_tabMain.Dock = System.Windows.Forms.DockStyle.Fill;
            this.m_tabMain.ImageList = this.m_imageLis16x16;
            this.m_tabMain.Location = new System.Drawing.Point(0, 0);
            this.m_tabMain.Name = "m_tabMain";
            this.m_tabMain.SelectedIndex = 0;
            this.m_tabMain.Size = new System.Drawing.Size(859, 488);
            this.m_tabMain.TabIndex = 0;
            this.m_tabMain.SelectedIndexChanged += new System.EventHandler(this.m_tabMain_SelectedIndexChanged);
            // 
            // m_pageQuery
            // 
            this.m_pageQuery.Controls.Add(this.m_pnlTop);
            this.m_pageQuery.Controls.Add(this.m_pnlBottom);
            this.m_pageQuery.ImageIndex = 5;
            this.m_pageQuery.Location = new System.Drawing.Point(4, 23);
            this.m_pageQuery.Name = "m_pageQuery";
            this.m_pageQuery.Size = new System.Drawing.Size(851, 461);
            this.m_pageQuery.TabIndex = 0;
            this.m_pageQuery.Text = "Query";
            // 
            // m_pnlTop
            // 
            this.m_pnlTop.AllowDrop = true;
            this.m_pnlTop.Controls.Add(this.m_cbxTemplates);
            this.m_pnlTop.Controls.Add(this.m_lblTemplate);
            this.m_pnlTop.Controls.Add(this.m_btnTemplate);
            this.m_pnlTop.Controls.Add(this.m_lblQuery);
            this.m_pnlTop.Controls.Add(this.m_txtQuery);
            this.m_pnlTop.Dock = System.Windows.Forms.DockStyle.Fill;
            this.m_pnlTop.Location = new System.Drawing.Point(0, 0);
            this.m_pnlTop.Name = "m_pnlTop";
            this.m_pnlTop.Padding = new System.Windows.Forms.Padding(8);
            this.m_pnlTop.Size = new System.Drawing.Size(851, 171);
            this.m_pnlTop.TabIndex = 5;
            // 
            // m_cbxTemplates
            // 
            this.m_cbxTemplates.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.m_cbxTemplates.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.m_cbxTemplates.FormattingEnabled = true;
            this.m_cbxTemplates.Items.AddRange(new object[] {
            "SELECT",
            "SELECT (Choice List)",
            "INSERT",
            "UPDATE",
            "DELETE"});
            this.m_cbxTemplates.Location = new System.Drawing.Point(626, 5);
            this.m_cbxTemplates.Name = "m_cbxTemplates";
            this.m_cbxTemplates.Size = new System.Drawing.Size(160, 21);
            this.m_cbxTemplates.TabIndex = 2;
            // 
            // m_lblTemplate
            // 
            this.m_lblTemplate.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.m_lblTemplate.AutoSize = true;
            this.m_lblTemplate.Location = new System.Drawing.Point(456, 8);
            this.m_lblTemplate.Name = "m_lblTemplate";
            this.m_lblTemplate.Size = new System.Drawing.Size(136, 13);
            this.m_lblTemplate.TabIndex = 4;
            this.m_lblTemplate.Text = "Frequently used commands";
            // 
            // m_btnTemplate
            // 
            this.m_btnTemplate.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.m_btnTemplate.Location = new System.Drawing.Point(792, 5);
            this.m_btnTemplate.Name = "m_btnTemplate";
            this.m_btnTemplate.Size = new System.Drawing.Size(51, 21);
            this.m_btnTemplate.TabIndex = 3;
            this.m_btnTemplate.Text = "Insert";
            this.m_btnTemplate.UseVisualStyleBackColor = true;
            this.m_btnTemplate.Click += new System.EventHandler(this.m_btnTemplate_Click);
            // 
            // m_lblQuery
            // 
            this.m_lblQuery.Location = new System.Drawing.Point(8, 8);
            this.m_lblQuery.Name = "m_lblQuery";
            this.m_lblQuery.Size = new System.Drawing.Size(424, 16);
            this.m_lblQuery.TabIndex = 1;
            this.m_lblQuery.Text = "Query";
            this.m_lblQuery.TextAlign = System.Drawing.ContentAlignment.BottomLeft;
            // 
            // m_txtQuery
            // 
            this.m_txtQuery.AcceptsReturn = true;
            this.m_txtQuery.AcceptsTab = true;
            this.m_txtQuery.AllowDrop = true;
            this.m_txtQuery.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.m_txtQuery.Location = new System.Drawing.Point(8, 32);
            this.m_txtQuery.Multiline = true;
            this.m_txtQuery.Name = "m_txtQuery";
            this.m_txtQuery.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.m_txtQuery.Size = new System.Drawing.Size(835, 131);
            this.m_txtQuery.TabIndex = 0;
            this.m_txtQuery.DragDrop += new System.Windows.Forms.DragEventHandler(this.m_txtQuery_DragDrop);
            this.m_txtQuery.DragEnter += new System.Windows.Forms.DragEventHandler(this.m_txtQuery_DragEnter);
            this.m_txtQuery.DragOver += new System.Windows.Forms.DragEventHandler(this.m_txtQuery_DragOver);
            // 
            // m_pnlBottom
            // 
            this.m_pnlBottom.AutoScroll = true;
            this.m_pnlBottom.Controls.Add(this.m_lblTabCol);
            this.m_pnlBottom.Controls.Add(this.m_lblRowsetFld);
            this.m_pnlBottom.Controls.Add(this.m_lblFunc);
            this.m_pnlBottom.Controls.Add(this.m_tvTablesAndCols);
            this.m_pnlBottom.Controls.Add(this.m_tvRowsetsAndFlds);
            this.m_pnlBottom.Controls.Add(this.m_lvFunctions);
            this.m_pnlBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.m_pnlBottom.Location = new System.Drawing.Point(0, 176);
            this.m_pnlBottom.Name = "m_pnlBottom";
            this.m_pnlBottom.Size = new System.Drawing.Size(851, 285);
            this.m_pnlBottom.TabIndex = 2;
            // 
            // m_lblTabCol
            // 
            this.m_lblTabCol.Location = new System.Drawing.Point(8, 8);
            this.m_lblTabCol.Name = "m_lblTabCol";
            this.m_lblTabCol.Size = new System.Drawing.Size(232, 16);
            this.m_lblTabCol.TabIndex = 1;
            this.m_lblTabCol.Text = "Tables && Columns";
            // 
            // m_lblRowsetFld
            // 
            this.m_lblRowsetFld.Location = new System.Drawing.Point(304, 8);
            this.m_lblRowsetFld.Name = "m_lblRowsetFld";
            this.m_lblRowsetFld.Size = new System.Drawing.Size(256, 16);
            this.m_lblRowsetFld.TabIndex = 3;
            this.m_lblRowsetFld.Text = "Rowsets && Fields";
            // 
            // m_lblFunc
            // 
            this.m_lblFunc.Location = new System.Drawing.Point(600, 8);
            this.m_lblFunc.Name = "m_lblFunc";
            this.m_lblFunc.Size = new System.Drawing.Size(243, 16);
            this.m_lblFunc.TabIndex = 5;
            this.m_lblFunc.Text = "Functions";
            // 
            // m_tvTablesAndCols
            // 
            this.m_tvTablesAndCols.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)));
            this.m_tvTablesAndCols.HideSelection = false;
            this.m_tvTablesAndCols.ImageIndex = 0;
            this.m_tvTablesAndCols.ImageList = this.m_imageLis16x16;
            this.m_tvTablesAndCols.Location = new System.Drawing.Point(8, 32);
            this.m_tvTablesAndCols.Name = "m_tvTablesAndCols";
            this.m_tvTablesAndCols.SelectedImageIndex = 0;
            this.m_tvTablesAndCols.Size = new System.Drawing.Size(288, 246);
            this.m_tvTablesAndCols.TabIndex = 2;
            this.m_tvTablesAndCols.DoubleClick += new System.EventHandler(this.m_tvTablesAndCols_DoubleClick);
            this.m_tvTablesAndCols.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.m_tvTablesAndCols_ItemDrag);
            // 
            // m_imageLis16x16
            // 
            this.m_imageLis16x16.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("m_imageLis16x16.ImageStream")));
            this.m_imageLis16x16.TransparentColor = System.Drawing.Color.Fuchsia;
            this.m_imageLis16x16.Images.SetKeyName(0, "");
            this.m_imageLis16x16.Images.SetKeyName(1, "");
            this.m_imageLis16x16.Images.SetKeyName(2, "");
            this.m_imageLis16x16.Images.SetKeyName(3, "");
            this.m_imageLis16x16.Images.SetKeyName(4, "");
            this.m_imageLis16x16.Images.SetKeyName(5, "Note16x16.bmp");
            this.m_imageLis16x16.Images.SetKeyName(6, "Advanced16x16.bmp");
            // 
            // m_tvRowsetsAndFlds
            // 
            this.m_tvRowsetsAndFlds.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)));
            this.m_tvRowsetsAndFlds.HideSelection = false;
            this.m_tvRowsetsAndFlds.ImageIndex = 0;
            this.m_tvRowsetsAndFlds.ImageList = this.m_imageLis16x16;
            this.m_tvRowsetsAndFlds.Location = new System.Drawing.Point(304, 32);
            this.m_tvRowsetsAndFlds.Name = "m_tvRowsetsAndFlds";
            this.m_tvRowsetsAndFlds.SelectedImageIndex = 0;
            this.m_tvRowsetsAndFlds.Size = new System.Drawing.Size(288, 246);
            this.m_tvRowsetsAndFlds.TabIndex = 4;
            this.m_tvRowsetsAndFlds.DoubleClick += new System.EventHandler(this.m_tvRowsetsAndFlds_DoubleClick);
            this.m_tvRowsetsAndFlds.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.m_tvTablesAndCols_ItemDrag);
            // 
            // m_lvFunctions
            // 
            this.m_lvFunctions.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.m_lvFunctions.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.m_colName});
            this.m_lvFunctions.FullRowSelect = true;
            this.m_lvFunctions.GridLines = true;
            this.m_lvFunctions.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
            this.m_lvFunctions.HideSelection = false;
            this.m_lvFunctions.Location = new System.Drawing.Point(600, 32);
            this.m_lvFunctions.MinimumSize = new System.Drawing.Size(243, 0);
            this.m_lvFunctions.MultiSelect = false;
            this.m_lvFunctions.Name = "m_lvFunctions";
            this.m_lvFunctions.ShowItemToolTips = true;
            this.m_lvFunctions.Size = new System.Drawing.Size(243, 246);
            this.m_lvFunctions.SmallImageList = this.m_imageLis16x16;
            this.m_lvFunctions.Sorting = System.Windows.Forms.SortOrder.Ascending;
            this.m_lvFunctions.TabIndex = 5;
            this.m_lvFunctions.UseCompatibleStateImageBehavior = false;
            this.m_lvFunctions.View = System.Windows.Forms.View.Details;
            this.m_lvFunctions.DoubleClick += new System.EventHandler(this.m_lvFunctions_DoubleClick);
            this.m_lvFunctions.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.m_tvTablesAndCols_ItemDrag);
            // 
            // m_colName
            // 
            this.m_colName.Text = "Name";
            this.m_colName.Width = 220;
            // 
            // m_pageOptions
            // 
            this.m_pageOptions.Controls.Add(this.m_btnOpenFile);
            this.m_pageOptions.Controls.Add(this.m_txtFuncFile);
            this.m_pageOptions.Controls.Add(this.m_lblLoadFuncFrom);
            this.m_pageOptions.Controls.Add(this.m_chkAddOnDblClk);
            this.m_pageOptions.Controls.Add(this.m_chkPrefixColNames);
            this.m_pageOptions.ImageIndex = 6;
            this.m_pageOptions.Location = new System.Drawing.Point(4, 23);
            this.m_pageOptions.Name = "m_pageOptions";
            this.m_pageOptions.Size = new System.Drawing.Size(851, 461);
            this.m_pageOptions.TabIndex = 1;
            this.m_pageOptions.Text = "Options";
            // 
            // m_btnOpenFile
            // 
            this.m_btnOpenFile.Location = new System.Drawing.Point(823, 64);
            this.m_btnOpenFile.Name = "m_btnOpenFile";
            this.m_btnOpenFile.Size = new System.Drawing.Size(24, 20);
            this.m_btnOpenFile.TabIndex = 8;
            this.m_btnOpenFile.Text = "...";
            this.m_btnOpenFile.Click += new System.EventHandler(this.m_btnOpenFile_Click);
            // 
            // m_txtFuncFile
            // 
            this.m_txtFuncFile.Location = new System.Drawing.Point(311, 64);
            this.m_txtFuncFile.Name = "m_txtFuncFile";
            this.m_txtFuncFile.Size = new System.Drawing.Size(512, 20);
            this.m_txtFuncFile.TabIndex = 7;
            // 
            // m_lblLoadFuncFrom
            // 
            this.m_lblLoadFuncFrom.Location = new System.Drawing.Point(8, 64);
            this.m_lblLoadFuncFrom.Name = "m_lblLoadFuncFrom";
            this.m_lblLoadFuncFrom.Size = new System.Drawing.Size(320, 20);
            this.m_lblLoadFuncFrom.TabIndex = 6;
            this.m_lblLoadFuncFrom.Text = "Load the list of functions from the following file:";
            // 
            // m_chkAddOnDblClk
            // 
            this.m_chkAddOnDblClk.Location = new System.Drawing.Point(8, 32);
            this.m_chkAddOnDblClk.Name = "m_chkAddOnDblClk";
            this.m_chkAddOnDblClk.Size = new System.Drawing.Size(864, 24);
            this.m_chkAddOnDblClk.TabIndex = 5;
            this.m_chkAddOnDblClk.Text = "Add Tables, Columns, Fields and Functions by double clicking on the corresponding" +
                " item.";
            // 
            // m_chkPrefixColNames
            // 
            this.m_chkPrefixColNames.Location = new System.Drawing.Point(8, 8);
            this.m_chkPrefixColNames.Name = "m_chkPrefixColNames";
            this.m_chkPrefixColNames.Size = new System.Drawing.Size(864, 24);
            this.m_chkPrefixColNames.TabIndex = 4;
            this.m_chkPrefixColNames.Text = "Prefix column names with table names";
            // 
            // m_dlgOpenFile
            // 
            this.m_dlgOpenFile.DefaultExt = "xml";
            this.m_dlgOpenFile.Filter = "XML Files|*.xml";
            // 
            // QueryEditor
            // 
            this.Controls.Add(this.m_tabMain);
            this.Name = "QueryEditor";
            this.Size = new System.Drawing.Size(859, 488);
            this.m_tabMain.ResumeLayout(false);
            this.m_pageQuery.ResumeLayout(false);
            this.m_pnlTop.ResumeLayout(false);
            this.m_pnlTop.PerformLayout();
            this.m_pnlBottom.ResumeLayout(false);
            this.m_pageOptions.ResumeLayout(false);
            this.m_pageOptions.PerformLayout();
            this.ResumeLayout(false);

        }
		#endregion


        //*********************************************************
        //
	    /// <summary>
	    /// Implements a configuration helper that accepts a control
	    /// instead of a form.
	    /// </summary>
	    //
        //*********************************************************
        private class ConfigHelper : UIConfigurationHelper {

            //*********************************************************
            //
	        /// <summary>
	        /// ctor:                       Class Constructor
	        /// </summary>
	        /// <param name="strFileName">  Config file name</param>
	        //
            //*********************************************************
            public ConfigHelper(string strFileName) : base(strFileName){
            }

            //*********************************************************
            //
	        /// <summary>
	        /// LoadState:                      Load the state of a controls
	        ///                                 container
	        /// </summary>
	        /// <param name="ctl">              Control to load the state</param>
	        //
            //*********************************************************
            public virtual void LoadState(System.Windows.Forms.Control ctl) {
                XmlNode             node;           
                IUIConfiguration    uiConfig;
                
                node = FindNode(null, "CONTROL" + ctl.Name);
                if (ctl is IUIConfiguration) {
                    uiConfig = (IUIConfiguration)ctl;
                    uiConfig.LoadState(this, node);
                }
                foreach (System.Windows.Forms.Control c in ctl.Controls) {
                    LoadControlState(node, c);
                }
            }

            //*********************************************************
            //
	        /// <summary>
	        /// SaveState:                  Save the state of a control 
	        ///                             container
	        /// </summary>
	        /// <param name="ctl">          Control to save the state</param>
	        //
            //*********************************************************
            public virtual void SaveState(System.Windows.Forms.Control ctl) {
                XmlNode             node;
                IUIConfiguration    uiConfig;
                
                node = FindNode(null, "CONTROL" + ctl.Name);
                node.RemoveAll();
                if (ctl is IUIConfiguration) {
                    uiConfig = (IUIConfiguration)ctl;
                    uiConfig.SaveState(this, node);
                }
                foreach (System.Windows.Forms.Control c in ctl.Controls) {
                    SaveControlState(node, c);
                }
            }

        }

        //******************************************************************        
        //
        /// <summary>
        /// Implements a tree view that does not expand/collapse after a
        /// double click
        /// </summary>
        //
        //******************************************************************        
        private class TVCInst : TreeView {
            private const int   WM_LBUTTONDBLCLK        = 0x203;
            private bool        bIgnoreExpandOrCollapse = false;

            //******************************************************************        
            //
            /// <summary>
            /// OnBeforeExpand              Called before a node is expanded
            /// </summary>
            /// <param name="e">            Event Handler</param>
            //
            //******************************************************************        
            protected override void OnBeforeExpand(TreeViewCancelEventArgs e) {
                if (bIgnoreExpandOrCollapse) {
                    bIgnoreExpandOrCollapse = false;
                    e.Cancel = true;
                }
                base.OnBeforeExpand (e);
            }

            //******************************************************************        
            //
            /// <summary>
            /// OnBeforeCollapse            Called before a node is collapsed
            /// </summary>
            /// <param name="e">            Event Handler</param>
            //
            //******************************************************************        
            protected override void OnBeforeCollapse(TreeViewCancelEventArgs e) {
                if (bIgnoreExpandOrCollapse) {
                    bIgnoreExpandOrCollapse = false;
                    e.Cancel = true;
                }
                base.OnBeforeCollapse (e);
            }

            //******************************************************************        
            //
            /// <summary>
            /// OnMouseMove                 Called when the mouse moves to clear
            ///                             the flag that ignores Expand/Collapse
            /// </summary>
            /// <param name="e">            Event Handler</param>
            //
            //******************************************************************        
            protected override void OnMouseMove(MouseEventArgs e) {
                bIgnoreExpandOrCollapse = false;
                base.OnMouseMove (e);
            }

            //******************************************************************        
            //
            /// <summary>
            /// OnItemDrag                  Called when an item is dragged to 
            ///                             select it.
            /// </summary>
            /// <param name="e">            Event Handler</param>
            //
            //******************************************************************        
            protected override void OnItemDrag(ItemDragEventArgs e) {
                this.SelectedNode = (e.Item as TreeNodeRowset);
                base.OnItemDrag (e);
            }

            //******************************************************************        
            //
            /// <summary>
            /// WndProc                     Message loop that sets the flag that 
            ///                             ignores Expand/Collapse after a 
            ///                             double-click 
            /// </summary>
            /// <param name="m">            Message</param>
            //
            //******************************************************************        
            protected override void WndProc(ref Message m) {
                if (m.Msg == WM_LBUTTONDBLCLK) {
                    bIgnoreExpandOrCollapse = true;
                }
                base.WndProc (ref m);
            }

            //******************************************************************        
            //
            /// <summary>
            /// GetCurrentlySelectedNodeFullKey     Returns the current node's
            ///                                     Full Key
            /// </summary>
            /// <returns>
            /// The node's full key or an empty string
            /// </returns>
            //
            //******************************************************************        
            public string GetCurrentlySelectedNodeFullKey() {
                string strRetVal = String.Empty;

                if (SelectedNode != null) {
                    strRetVal = SelectedNode.Tag as string;
                }
                return(strRetVal); 
            }

            //******************************************************************        
            //
            /// <summary>
            /// FindNodeUsingFullKey            Finds a node using its full key
            /// </summary>
            /// <param name="strKey">           Full Key</param>
            /// <returns>
            /// The matching node or Null
            /// </returns>
            //
            //******************************************************************        
            public TreeNode FindNodeUsingFullKey(string strKey) {
                return(FindNodeUsingFullKey(Nodes, strKey)); 
            }

            //******************************************************************        
            //
            /// <summary>
            /// FindNodeUsingFullKey            Finds a node using its full key
            /// </summary>
            /// <param name="nodes">            Nodes to search</param>
            /// <param name="strKey">           Full Key</param>
            /// <returns>
            /// The matching node or Null
            /// </returns>
            //
            //******************************************************************        
            private TreeNode FindNodeUsingFullKey(TreeNodeCollection nodes, string strKey) {
                TreeNode nodeRetVal = null;

                foreach(TreeNode node in nodes) {
                    if(strKey.Equals(node.Tag) ) {
                        nodeRetVal = node;
                    } else {
                        nodeRetVal = FindNodeUsingFullKey(node.Nodes, strKey);
                    }
                    if (nodeRetVal != null) {
                        break;
                    }
                }
                return(nodeRetVal); 
            }
        } //Class TVCInst

        //******************************************************************        
        //
        /// <summary>
        /// Implements a form that displays a variable number of standard 
        /// function arguments
        /// </summary>
        //
        //******************************************************************        
        private class frmParams : Form {

            //******************************************************************        
            //
            /// <summary>
            /// Class representing the properties of a standard function argument
            /// </summary>
            //
            //******************************************************************        
            private class Param {
                /// <summary> Name of the argument </summary>
                public string Name              = string.Empty;
                /// <summary> Indication of whether the parameter is required </summary>
                public bool   Required          = true;
                /// <summary> Indication of whether the parameter should be enclosed in quotes</summary>
                public bool   EncloseInQuotes   = false;
            }
            /// <summary>List of columns and fields</summary>
            private static string [] m_arrColsAndFlds = null;
            /// <summary>The table panel control</summary>
            private TableLayoutPanel m_pnlTable;
            /// <summary>The ok button</summary>
            private Button           m_btnOK;
            /// <summary>The main panel</summary>
            private Panel            m_pnlMain;
            /// <summary>The list of controls created for the arguments</summary>
            private ArrayList		m_lstParamsCtls;

            //******************************************************************        
            //
            /// <summary>
            /// Class constructor
            /// </summary>
            /// <param name="strCaption">   Form caption</param>
            /// <param name="arrParams">    List of arguments</param>
            //
            //******************************************************************        
            private frmParams(string strCaption, Param[] arrParams) {
                int iParamCount;

                m_lstParamsCtls = new ArrayList();
                iParamCount     = ((arrParams == null) ? 0 : arrParams.Length);
                if (iParamCount > 0) {
                    CreateContainer(iParamCount);
                    for(int i = 0; i < iParamCount; i++) {
                        CreateParameter(arrParams[i], i); 
                    }
                }
                Initialize();
                this.Text = strCaption;
            }

            //******************************************************************
            //
            /// <summary>
            /// Displays the function arguments form
            /// </summary>
            /// <param name="frmParent">        The parent form</param>
            /// <param name="strCaption">       The caption of the form</param>
            /// <param name="strArguments">     The list of arguments</param>
            /// <returns>
            /// The complete function header with all the arguments
            /// </returns>
            //
            //******************************************************************        
            public static string Show(Form frmParent, string strCaption, string strArguments) {
                frmParams frm;
                string    strRetVal = strCaption;
                string[]  arrParamNames;
                Param[]   arrParams;
                Param     param;

                if (!string.IsNullOrEmpty(strArguments)) {
                    arrParamNames = strArguments.Split(',');
                    if (arrParamNames.Length > 0) {
                        arrParams = new Param[arrParamNames.Length];
                        for (int i = 0; i < arrParamNames.Length; i++) {
                            param = new Param();
                            param.Name = arrParamNames[i];
                            if (param.Name.Contains("[")) {
                                param.Name     = param.Name.Substring(1, param.Name.Length - 2);
                                param.Name    += " (optional)";
                                param.Required = false;
                            }
                            if (param.Name.Contains("'")) {
                                param.Name            = param.Name.Substring(1, param.Name.Length - 2);
                                param.EncloseInQuotes = true;
                            }
                            arrParams[i] = param;
                        }
                        using(frm = new frmParams(strCaption, arrParams)) {
                            if (frm.ShowDialog(frmParent) == DialogResult.OK) {
                                strRetVal = frm.CompleteFunctionName;
                            }
                        }
                    }
                }
                return(strRetVal);
            }

            //******************************************************************        
            //
            /// <summary>
            /// Initializes the list of columns and fields
            /// </summary>
            /// <param name="tvCols">   TreeView conatining the tables and columns</param>
            /// <param name="tvFlds">   TreeView containing the rowsets and fields</param>
            //
            //******************************************************************        
            public static void InitAvailableColsAndFlds(TVCInst tvCols, TVCInst tvFlds) {
                ArrayList lst = new ArrayList(16);

                foreach (TreeNode nodeTable in tvCols.Nodes) {
                    foreach (TreeNode nodeColumn in nodeTable.Nodes) {
                        lst.Add(nodeColumn.FullPath.Replace('\\','.'));
                    }
                }
                foreach (TreeNode nodeRowset in tvFlds.Nodes) {
                    foreach (TreeNode nodeFld in nodeRowset.Nodes) {
                        lst.Add(":" + nodeFld.FullPath.Replace('\\','.'));
                    }
                }
                m_arrColsAndFlds = (string[])lst.ToArray(typeof(string));
            }

            //******************************************************************        
            //
            /// <summary>
            /// Returns the complete function name including the argument values
            /// </summary>
            //
            //******************************************************************        
            public string CompleteFunctionName {
                get {
                    string strRetVal    = string.Empty;
                    Param  param;
                    int    iNumOfParams = m_pnlTable.RowCount;

                    foreach (ComboBox cbx in m_lstParamsCtls) {
                        param = cbx.Tag as Param;
                        if (strRetVal != string.Empty) {
                            if (cbx.Text != string.Empty || param.Required) {
                                strRetVal += ",";
                            }
                        }
                        if (cbx.SelectedIndex == -1) {
                            if (cbx.Text != string.Empty) {
                                if (param.EncloseInQuotes) {
                                    strRetVal += "'" + cbx.Text + "'";
                                } else {
                                    strRetVal += cbx.Text;
                                }
                            }
                        } else {
                            strRetVal += cbx.Text;
                        }
                    }
                    strRetVal = Text.Substring(0, Text.IndexOf('(') + 1) + strRetVal + ")";
                    return(strRetVal);
                }
            }

            //******************************************************************        
            //
            /// <summary>
            /// Creates the controls required to display/get an argument's value
            /// </summary>
            /// <param name="param">        Parameter to be displayed</param>
            /// <param name="iPosition">    The position of the parameter</param>
            //
            //******************************************************************        
            private void CreateParameter(Param param, int iPosition) {
                ComboBox cbx;
                Label    lbl;
           
                cbx                     = new ComboBox();
                cbx.Dock                = System.Windows.Forms.DockStyle.Fill;
                cbx.FormattingEnabled   = true;
                cbx.Location            = new System.Drawing.Point(149, 32);
                cbx.Size                = new System.Drawing.Size(287, 21);
                cbx.TabIndex            = iPosition;
                cbx.Tag                 = param;
                cbx.Items.AddRange(m_arrColsAndFlds);
                m_lstParamsCtls.Add(cbx);
                this.m_pnlTable.Controls.Add(cbx, 1, iPosition);
                // 
                // label1
                // 
                lbl             = new Label();
                lbl.AutoSize    = true;
                lbl.Dock        = System.Windows.Forms.DockStyle.Fill;
                lbl.Location    = new System.Drawing.Point(4, 1);
                lbl.Size        = new System.Drawing.Size(138, 27);
                lbl.Text        = param.Name;
                lbl.TextAlign   = System.Drawing.ContentAlignment.MiddleLeft;
                this.m_pnlTable.Controls.Add(lbl, 0, iPosition);
            }

            //******************************************************************        
            //
            /// <summary>
            /// Creates the the controls container
            /// </summary>
            /// <param name="iRowCount"></param>
            //
            //******************************************************************        
            private void CreateContainer(int iRowCount) {
                this.m_pnlTable = new TableLayoutPanel();
                this.m_pnlMain  = new Panel();
                this.m_btnOK    = new Button();
                // 
                // m_pnlTable
                // 
                this.m_pnlTable.AutoSize        = true;
                this.m_pnlTable.CellBorderStyle = System.Windows.Forms.TableLayoutPanelCellBorderStyle.None;
                this.m_pnlTable.ColumnCount     = 2;
                this.m_pnlTable.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 33F));
                this.m_pnlTable.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 67F));
                this.m_pnlTable.Location    = new System.Drawing.Point(6, 6);
                this.m_pnlTable.Margin      = new System.Windows.Forms.Padding(1);
                this.m_pnlTable.Name        = "m_pnlTable";
                this.m_pnlTable.RowCount    = iRowCount + 1;
                this.m_pnlTable.Size        = new Size(m_pnlTable.Width, 15);
                this.m_pnlTable.MaximumSize = new Size(450, 5000);
                this.m_pnlTable.TabIndex = 0;
                for (int i = 0; i <= iRowCount; i++) {
                    this.m_pnlTable.RowStyles.Add(new System.Windows.Forms.RowStyle());
                }
                // 
                // m_btnOK
                // 
                this.m_pnlTable.SetColumnSpan(this.m_btnOK, 2);
                this.m_btnOK.DialogResult            = DialogResult.OK;
                this.m_btnOK.Location                = new System.Drawing.Point(182, 91);
                this.m_btnOK.Margin                  = new System.Windows.Forms.Padding(181, 3, 3, 3);
                this.m_btnOK.Name                    = "m_btnOK";
                this.m_btnOK.Size                    = new System.Drawing.Size(75, 22);
                this.m_btnOK.TabIndex                = 100;
                this.m_btnOK.Text                    = "OK";
                this.m_btnOK.UseVisualStyleBackColor = true;

                this.m_pnlTable.Controls.Add(this.m_btnOK, 1, iRowCount);
                // 
                // m_pnlMain
                // 
                this.m_pnlMain.AutoSize = true;
                this.m_pnlMain.Controls.Add(this.m_pnlTable);
                this.m_pnlMain.Dock     = System.Windows.Forms.DockStyle.Fill;
                this.m_pnlMain.Location = new System.Drawing.Point(0, 0);
                this.m_pnlMain.Name     = "m_pnlMain";
                this.m_pnlMain.Padding  = new System.Windows.Forms.Padding(5);
                this.m_pnlMain.Size     = new System.Drawing.Size(453, 21);

                this.Controls.Add(this.m_pnlMain);

            }

            //******************************************************************        
            //
            /// <summary>
            /// Initializes the form
            /// </summary>
            //
            //******************************************************************        
            private void Initialize() {
                this.SuspendLayout();
                this.AcceptButton           = m_btnOK;
                this.AutoScaleDimensions    = new System.Drawing.SizeF(6F, 13F);
                this.AutoScaleMode          = System.Windows.Forms.AutoScaleMode.Font;
                this.AutoSize               = true;
                this.ClientSize             = new System.Drawing.Size(453, 21);
                this.Cursor                 = System.Windows.Forms.Cursors.Default;
                this.FormBorderStyle        = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
                this.Name                   = "frmParams";
                this.StartPosition          = FormStartPosition.CenterScreen;
                this.Text                   = "frmParams";
                this.ResumeLayout(false);
                this.PerformLayout();
            }
        }
    } // Class QueryEditor
} // Namespace RepPP.Miscellaneous
