//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {    
    /// <summary>Different modes to Open Items</summary>
    public enum OpenItemModeE {
        /// <summary>Open the item to edit it</summary>
        OI_Edit         =   1,
        /// <summary>Open a new item</summary>
        OI_New          =   2,
        /// <summary>Duplicate the item</summary>
        OI_Duplicate    =   3,
        /// <summary>No state</summary>
        OI_None         =   4
    };
        
    /// <summary>
    /// Handle a card with a selection buffer
    /// </summary>
    public class RowsetTreeDefNodeHandlerRootForSB : RowsetTreeDefNodeHandlerRootBase, IRowsetTreeDefNodeHandlerRootForSB {
	    /// <summary>Selection buffer</summary>
	    private ISelectionBuffer                m_selBuf;
        /// <summary>Interface to ask for a primary key</summary>
        private IAskForPrimaryKey               m_askForPrimaryKey;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// 
        /// <param name="rowsetTreeDef">        Rowset tree definition to attach</param>
        /// <param name="bSeeDelLine">          true to see deleted line
        ///                                     false not to see it</param>
        /// <param name="bUndo">                true to support undo operation
        ///                                     false to not support undo operation</param>
        /// <param name="askForPrimaryKey">     Interface to ask for a primary key</param>
        /// <param name="selectionBuffer">      Interface to update selection buffer</param>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandlerRootForSB(RowsetTreeDef      rowsetTreeDef,
                                                 bool               bSeeDelLine,
                                                 bool               bUndo,
                                                 IAskForPrimaryKey  askForPrimaryKey,
                                                 ISelectionBuffer   selectionBuffer) : base(rowsetTreeDef, bSeeDelLine, bUndo) {
            m_askForPrimaryKey = askForPrimaryKey;
            m_selBuf           = selectionBuffer;
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                Clean the class
        /// </summary>
        //  
        //*********************************************************     
        public override void Dispose() {
            Empty();
            base.Dispose();
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectionBuffer:                Selection buffer
        /// </summary>
        //  
        //*********************************************************     
		public virtual ISelectionBuffer SelectionBuffer {
		    get {
		        return(m_selBuf);
		    }
		    set {
		        m_selBuf = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// Empty:                  Empty the content of the rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public virtual void Empty() {
            Rowset              rowset;
            ISelectionBuffer    selBuf;
            
            rowset = Rowset;
            if (rowset != null) {
                if (rowset.UndelLineCount != 0 && rowset.LineState == LineState.sdLineNew) {
                    selBuf = SelectionBuffer;
                    selBuf.DeleteItemFromSelectionBuffer(false);
                }
                rowset.RowsetTree.Empty(true);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildWhereClause:                   Build the where clause
        /// </summary>
        /// <param name="node">                 Rowset tree definition node</param>
        /// <param name="bHierarchical">        true to use the father primary key</param>
        /// <returns>
        /// WHERE clause using the PK to select an item
        /// </returns>
        //
        //******************************************************************
        public static string BuildWhereClause(RowsetTreeDefNode node, bool bHierarchical) {
            string              strRetVal = "";
            FieldDefs           fldDefs;
            ArrayList           arrAncester;
            RowsetTreeDefNode   nodeFather;
            
            arrAncester = new ArrayList();
            nodeFather  = node;
            while (nodeFather != null) {
                arrAncester.Insert(0, nodeFather);
                nodeFather = (bHierarchical ? nodeFather.Parent as RowsetTreeDefNode : null);
            }
            foreach (RowsetTreeDefNode nodeNode in arrAncester) {
                fldDefs = nodeNode.RowsetDef.FieldDefs;
                foreach (RepPP.FieldDef fldDef in fldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        if (strRetVal.Length == 0) {
                            strRetVal = "WHERE ";
                        } else {
                            strRetVal += " AND ";
                        }
                        strRetVal += fldDef.ColName + "=:" + fldDef.RowsetDef.Name + "." + fldDef.Name;
                    }
                }
            }
            return(strRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public override int SBCurrentLine {
            get {
                ISelectionBuffer    selBuf;
                
                selBuf = SelectionBuffer;
                return((selBuf.CurrentRowset != null) ? selBuf.CurrentRowset.CurrentLine : -1);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the line count in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public override int SBLineCount {
            get {
                ISelectionBuffer    selBuf;
                
                selBuf = SelectionBuffer;
                return((selBuf.CurrentRowset != null) ? selBuf.CurrentRowset.LineCount : 0);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectItem:                         Use by selection buffer to select an item using its primary key
        /// </summary>
        /// <param name="arrPK">                Primary key values</param>
        /// <returns>
        ///  true:                              Set openned
        ///  false:                             Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool SelectItem(string[] arrPK) {
            bool    bRetVal;
            
            if (arrPK == null) {
                Empty();
                bRetVal = true;
            } else {
                bRetVal = (ReadFromDb(BuildWhereClause(RowsetTreeDefNode, false), arrPK) == 0);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************           
        public override bool AskToSave(bool bEnsureFatherExist) {
            bool                bRetVal;
            DialogResult        eVal;
            string              strMsg;
            string              strTitle;
            Rowset       rowset;

            CommitAllEdit(this);
            if (IsDirty()) {
                rowset  = Rowset;
                strMsg = m_app.GetAtomText("$FCH2", RepPP.AtomType.sdAtomMsg);
                if (m_app.IsAtomExist(m_app.ProgramName, AtomType.sdAtomMsg)) {
                    strTitle = m_app.GetAtomText(m_app.ProgramName, AtomType.sdAtomMsg);
                } else {
                    strTitle = m_app.ProgramName; 
                }
                eVal   = MessageBox.Show(strMsg,
                                         strTitle,
                                         (bEnsureFatherExist && rowset.LineState == LineState.sdLineNew) ?
                                            MessageBoxButtons.OKCancel : MessageBoxButtons.YesNoCancel,
                                         MessageBoxIcon.Question);
                switch(eVal) {
                case DialogResult.Cancel:
                    bRetVal = false;
                    break;
                case DialogResult.Yes:
                case DialogResult.OK:
                    bRetVal = SaveLine();
                    break;
                default:
                    if (Rowset.LineState == RepPP.LineState.sdLineNew) {
                        bRetVal = DeleteCurrentLine(true);
                    } else {
                        bRetVal = true;
                    }
                    break;
                }
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current line
        /// </summary>
        /// <param name="iLine">        Line position</param>
        /// <param name="bAsk">         Flag to ask to save unsaved</param>
        /// <returns>
        ///     true                    New line selected
        ///     false                   Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SetCurrentLine(int iLine, bool bAsk) {
            bool                bRetVal;
            ISelectionBuffer    selBuf;

            bRetVal = (bAsk) ? AskToSave() : true;
            if (bRetVal) {
                selBuf  = SelectionBuffer;
                bRetVal = selBuf.SelectItem(iLine);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current line
        /// </summary>
        /// <param name="iLine">        Line position</param>
        /// <returns>
        ///     true                    New line selected
        ///     false                   Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SetCurrentLine(int iLine) {
            return(SetCurrentLine(iLine, true));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetDefValue:                        Set the default value of the specified field
        /// </summary>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        //
        //*********************************************************     
        public virtual void SetDefValue(FieldDef[] arrFldDef, string[] arrValue) {
            int     iIndex = 0;
            
            if (arrFldDef != null && arrValue != null) {
                foreach (FieldDef fldDef in arrFldDef) {
                    fldDef.Value = arrValue[iIndex++];
                }
                SelectionBuffer.RefreshItemInSelectionBuffer(Rowset);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="rowsetSB">             Selection buffer rowset</param>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">            Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool InsertNewLine(Rowset rowsetSB, bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            bool                bRetVal;
            ISelectionBuffer    selBuf;

            RowsetTree.Empty();
            bRetVal = base.InsertNewLine(bAtEnd);
            if (bRetVal) {
                selBuf = SelectionBuffer;                
                selBuf.AddItemToSelectionBuffer(rowsetSB, false);
                SetDefValue(arrFldDef, arrValue);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">            Array of field for which the default value must be set</param>
        /// <param name="arrValue">             Array of default values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool InsertNewLine(bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            return(InsertNewLine(SelectionBuffer.CurrentRowset, bAtEnd, arrFldDef, arrValue));
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public override bool InsertNewLine(bool bAtEnd) {
            bool            bRetVal;
            RepPP.Rowset    rowsetSB;
            FieldDef[]      arrFldDef;
            string[]        arrValue;
            int             iSavedLinePos;
            
            bRetVal = AskToSave();
            if (bRetVal) {
                rowsetSB      = SelectionBuffer.CurrentRowset;
                iSavedLinePos = rowsetSB.CurrentLine;
                rowsetSB.AddBlankLine(bAtEnd);
                if (m_askForPrimaryKey == null) {
                    arrFldDef   = null;
                    arrValue    = null;
                } else {
                    bRetVal = m_askForPrimaryKey.AskForPrimaryKey(out arrFldDef, out arrValue);
                }
                if (bRetVal) {
                    bRetVal = InsertNewLine(bAtEnd, arrFldDef, arrValue);
                } else {
                    rowsetSB.RemoveLine();
                    rowsetSB.SelectLine(iSavedLinePos, false, true);
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:                      Duplicate the current item
        /// </summary>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public override bool DuplicateLine() {
            bool                bRetVal;
            ISelectionBuffer    selBuf;
            FieldDef[]          arrFldDef;
            string[]            arrValue;
            Rowset              rowset;
            Rowset              rowsetSB;
            int                 iSavedLinePos;
            
            bRetVal = AskToSave();
            if (bRetVal) {
                rowset         = Rowset;
                rowsetSB       = SelectionBuffer.CurrentRowset;
                iSavedLinePos = rowsetSB.CurrentLine;
                rowsetSB.DuplicateLine();
                // Path for indexer position : Must implement DuplicateLine with duplicate to end
                rowsetSB.MoveLineTo(rowsetSB.LineCount - 1);
                if (m_askForPrimaryKey == null) {
                    arrFldDef   = null;
                    arrValue    = null;
                } else {
                    bRetVal  = m_askForPrimaryKey.AskForPrimaryKey(out arrFldDef, out arrValue);
                }
                if (bRetVal) {
                    rowset.RowsetTree.ReadUnreadChild(rowset, null, false);
                    rowset.LineState = LineState.sdLineNew;
                    Util.SetRowsetChildrenState(rowset, LineState.sdLineNew);
                    selBuf           = SelectionBuffer;
                    selBuf.AddItemToSelectionBuffer(selBuf.CurrentRowset, true);
                    SetDefValue(arrFldDef, arrValue);
					OnDataConversion(new DataConversionEventArgs(this, DataConversionTypeE.DC_Decode));
					OnLineSelected(new LineSelectedEventArgs(0, 0));
                } else {
                    rowsetSB.RemoveLine();
                    rowsetSB.SelectLine(iSavedLinePos, false, true);
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskForDeleteB:          Ask before deleting a row
        /// </summary>
        /// <returns>
        ///  true:                  Accept
        ///  false:                 Refused
        /// </returns>
        //
        //*********************************************************     
        public virtual bool AskForDelete() {
            bool                bRetVal;
            DialogResult        eVal;
            string              strMsg;

            strMsg  = m_app.GetAtomText("$COD", RepPP.AtomType.sdAtomMsg);
            eVal    = MessageBox.Show(strMsg, m_app.ProgramName, MessageBoxButtons.YesNo);
            bRetVal = (eVal == DialogResult.Yes);
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:          Deletes the current line
        /// </summary>
        /// <param name="bReselect">    true to select another row</param>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DeleteCurrentLine(bool bReselect) {
            bool                bRetVal;
            Rowset              rowset;
            ISelectionBuffer    selBuf;
            
            rowset = Rowset;
            switch(rowset.LineState) {
            case LineState.sdLineNew:
                bRetVal = base.DeleteCurrentLine();
                if (bRetVal) {
                    RowsetTree.ResetState();
                }
                break;
            case LineState.sdLineDeleted:
                bRetVal = false;
                break;
            default:
                bRetVal = AskForDelete();
                if (bRetVal) {
                    bRetVal = base.DeleteCurrentLine();
                    if (bRetVal) {
                        bRetVal = SaveAllLines();
                        if (!bRetVal) {
                            Undo();
                        }
                    }
                }
                break;
            }
            if (bRetVal) {
                selBuf = SelectionBuffer;
                selBuf.DeleteItemFromSelectionBuffer(bReselect);
                SetCommandHolderState();
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:          Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public override bool DeleteCurrentLine() {
            return(DeleteCurrentLine(true));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:               Save a line (Same as SaveAllLines)
        /// </summary>
        /// <returns>
        /// true:                   A new line has been inserted
        /// false:                  No line inserted
        /// </returns>
        //  
        //*********************************************************
        public override bool SaveLine() {
            bool bRetVal;

            bRetVal = SaveAllLines();
            if (bRetVal) {
                SelectionBuffer.RefreshItemInSelectionBuffer(Rowset);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                  Yes
        ///  false:                 No
        /// </returns>
        //
        //*********************************************************     
        public override bool IsDirty() {
            bool                bRetVal;
            RepPP.RowsetTree    rowsetTree;
            
            bRetVal = base.IsDirty();
            if (!bRetVal) {
                rowsetTree = RowsetTree;
                if (rowsetTree != null && rowsetTree.RootRowset.LineState == LineState.sdLineNew) {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }
    } // class
} // namespace