using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	//******************************************************************
	//
	/// <summary>
	/// Implement a filter for a selection buffer
	/// </summary>
	//
	//******************************************************************
	public class SBFilter : SBFilterBase, ISBFilterUI {
	    /// <summary>Array of combobox containing the operator</summary>
	    protected ComboBox[]                    m_arrComboBox;
	    /// <summary>Array of Rep++ static field controls created in the panel</summary>
	    protected RepPP.RepPPStat[]             m_arrStat;
	    /// <summary>Array of Rep++ field controls created in the panel</summary>
	    protected RepPP.RepPPField[]            m_arrField;
	    /// <summary>Array of Rep++ field controls created to handle the BETWEEN clause in the panel</summary>
        protected RepPP.RepPPField[]            m_arrBetweenField;
	    /// <summary>Original size of the field before they're resize for the BETWEEN clause</summary>
	    protected int[]                         m_arrOriginalFieldSize;
        /// <summary>Maximum X position a field can be in the panel</summary>
        protected int                           m_iMaxXPos;

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                               Class constructor
	    /// </summary>
		/// <param name="rowsetDefParam">             RowsetDef containing the field use as filter</param>
		/// <param name="bShowUserFlagField">   True to add user flag fields to the filter group</param>
	    //
	    //******************************************************************
		public SBFilter(RepPP.RowsetDef rowsetDefParam, bool bShowUserFlagField) : base(rowsetDefParam, bShowUserFlagField) {
            m_arrOriginalFieldSize  = null;
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
		/// <param name="rowsetDefParam">     RowsetDef containing the field use as filter</param>
	    //
	    //******************************************************************
		public SBFilter(RepPP.RowsetDef rowsetDefParam) : this(rowsetDefParam, true) {
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ResizeForm:                 Resize the form to show as much field as possible
		/// </summary>
		/// <param name="form">         Father form to resize</param>
		/// <param name="ctlFather">    Father control (panel)</param>
		//
	    //******************************************************************
        public virtual void ResizeForm(Form form, Control ctlFather) {
            int     iMaxYSize;
            int     iYDelta;
            int     iNewYSize;
            
            iMaxYSize = 0;
            foreach (Control ctl in ctlFather.Controls) {
                if (ctl.Visible) {
                    if (ctl.Bottom > iMaxYSize) {
                        iMaxYSize = ctl.Bottom;
                    }
                }
            }
            iMaxYSize       += 10;
            iYDelta          = iMaxYSize - ctlFather.Height;
            iNewYSize        = form.Height + iYDelta;
            if (iNewYSize > (System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height - 10)) {
                iNewYSize = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height - 10;
            }
            if (form.Height != iNewYSize) {
                form.Height = iNewYSize;
            }
            if (form.Bottom > System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height) {
                form.Top = 1;
            }
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// FillPanel:                          Fill the panel with the fields to use as filter
	    /// </summary>
		/// <param name="panelAllFields">       Panel to fill with fields</param>
		/// <param name="font">                 Font to use</param>
		/// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
		/// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
	    //
	    //******************************************************************
		public virtual void FillPanel(Panel                 panelAllFields,
		                              System.Drawing.Font   font,
		                              int                   iMaxPromptSizeInChr,
		                              int                   iMaxFieldSizeInChr) {
            FillPanel(null, panelAllFields, null, "", font, iMaxPromptSizeInChr, iMaxFieldSizeInChr);
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// FillPanel:                          Fill the panel with the fields to use as filter
	    /// </summary>
        /// <param name="panelSearchField">     The panel that will contain the search field</param>
        /// <param name="panelAllFields">       The panel that will contain the other fields</param>
        /// <param name="fldDefSearchField">    The search FieldDef</param>
        /// <param name="strSearchValue">       The search value</param>
		/// <param name="font">                 Font to use</param>
		/// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
		/// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
	    //
	    //******************************************************************
		public virtual void FillPanel(Panel                 panelSearchField,
                                      Panel                 panelAllFields,
                                      FieldDef              fldDefSearchField,
                                      string                strSearchValue,
		                              System.Drawing.Font   font,
		                              int                   iMaxPromptSizeInChr,
		                              int                   iMaxFieldSizeInChr) {
		    int     iIndex;
		    
		    m_rowsetParam.Empty();
            if(panelSearchField != null && fldDefSearchField != null) {
		        DynamicForm.CreateRowsetFieldsForDSearch(panelSearchField,
                                                        m_rowsetParam.RowsetDef.FieldDefs[fldDefSearchField.Name],
                                                        panelAllFields,
	                                                    font,
	                                                    iMaxPromptSizeInChr,
	                                                    iMaxFieldSizeInChr,
	                                                    8,
	                                                    2,
	                                                    DynamicForm.OptionE.OPT_NO_CHECK |
	                                                    DynamicForm.OptionE.OPT_NO_RADIO |
	                                                    DynamicForm.OptionE.OPT_ONLY_MAIN_TBL_FIELD |
	                                                    DynamicForm.OptionE.OPT_ADD_OPERATOR_COMBO |
	                                                    DynamicForm.OptionE.OPT_ADD_BETWEEN_FIELD |
                                                        (ShowUserFlagField ? DynamicForm.OptionE.OPT_NONE : DynamicForm.OptionE.OPT_NO_USER_FLAG_FIELD),
	                                                    out m_iMaxXPos,
	                                                    out m_arrComboBox,
	                                                    out m_arrStat,
	                                                    out m_arrField,
	                                                    out m_arrBetweenField);
                if (m_arrField.Length > 0) {
                    m_arrField[0].FieldDef.SetValue(strSearchValue, true, RepPP.SetFieldOpt.sdSetNormal);
                }
            } else {
		        DynamicForm.CreateRowsetFields(panelAllFields,
		                                      m_rowsetParam.RowsetDef,
		                                      font,
		                                      iMaxPromptSizeInChr,
		                                      iMaxFieldSizeInChr,
		                                      8,
		                                      2,
		                                      DynamicForm.OptionE.OPT_NO_CHECK |
		                                      DynamicForm.OptionE.OPT_NO_RADIO |
		                                      DynamicForm.OptionE.OPT_ONLY_MAIN_TBL_FIELD |
		                                      DynamicForm.OptionE.OPT_ADD_OPERATOR_COMBO |
		                                      DynamicForm.OptionE.OPT_ADD_BETWEEN_FIELD |
                                              (ShowUserFlagField ? DynamicForm.OptionE.OPT_NONE : DynamicForm.OptionE.OPT_NO_USER_FLAG_FIELD),
		                                      out m_iMaxXPos,
		                                      out m_arrComboBox,
		                                      out m_arrStat,
		                                      out m_arrField,
		                                      out m_arrBetweenField);
            }
	        m_arrOriginalFieldSize = new int[m_arrField.Length];
	        iIndex = 0;
	        foreach (RepPP.RepPPField fldDefCtl in m_arrField) {
	            m_arrOriginalFieldSize[iIndex++] = fldDefCtl.Width;
	        }
	        iIndex = 0;
	        foreach (ComboBox combo in m_arrComboBox) {
	            combo.Items.AddRange(new object[] {"=", "<", "<=", ">", ">=", "<>", m_strBetween, "is null", "is not null"} );
	            combo.SelectedIndex         = 0;
	            combo.DropDownStyle         = ComboBoxStyle.DropDownList;
                combo.MaxDropDownItems      = combo.Items.Count;
	            combo.SelectedIndexChanged += new EventHandler(combo_SelectedIndexChanged);
	            SetFieldStateForThisOperator(iIndex, "=");
	            iIndex++;
	        }
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// DisposeFormResource:        Must be called when the form is closed
	    /// </summary>
	    //
	    //******************************************************************
		public virtual void DisposeFormResource() {
	        if (m_arrComboBox != null) {
    	        foreach (ComboBox combo in m_arrComboBox) {
	                combo.SelectedIndexChanged -= new EventHandler(combo_SelectedIndexChanged);
	            }
	            m_arrComboBox = null;
	        }
	        m_arrBetweenField       = null;
	        m_arrField              = null;
	        m_arrStat               = null;
	        m_arrOriginalFieldSize  = null;
	    }

	    //******************************************************************
	    //
		/// <summary>
		/// BuildWhereClause:                   Build a WHERE clause using the values in the group
		/// </summary>
		/// <param name="eType">                AND or OR</param>
		/// <param name="strFilterClause">      Human readable WHERE clause</param>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
		protected override string BuildWhereClause(AndOrE eType, out string strFilterClause) {
		    string      strRetVal;
		    string[]    arrOp;
		    int         iCount;

		    
		    iCount          = m_arrComboBox.Length;
		    arrOp           = new string[iCount];
		    for (int iIndex = 0; iIndex < iCount; iIndex++) {
		        arrOp[iIndex] = m_arrComboBox[iIndex].SelectedItem as string;
		    }
            ReOrderOperators(arrOp);
		    strRetVal = BuildWhereClause(eType, arrOp, out strFilterClause);
		    return(strRetVal);
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ReOrderOperators:                   Re-orders the operators based
        ///                                     on the real order of fields in
        ///                                     the group - instead of the display
        ///                                     order.
		/// </summary>
        /// <param name="arrOp">                array of operators</param>
        /// <remarks>
        /// This method assumes that the combo boxes are named as follows:
        /// comboFIELDNAME
        /// </remarks>
	    //
	    //******************************************************************
        private void ReOrderOperators(string[] arrOp){
            string  temp;
            string  strFld;
            string  strSearchFldName;
            FieldDef   fldDefDef; 
            FieldDef   fldDefSearch;
            bool    bDone   = false;
            int     i       = 1;
            int     iCount  = m_arrComboBox.Length;

            if (iCount > 0) {
                strSearchFldName = m_arrComboBox[0].Name.Replace("combo", "");
                fldDefSearch        = m_rowsetDefParam.FieldDefs[strSearchFldName];
                while (i < iCount && !bDone) {
                    strFld = m_arrComboBox[i].Name.Replace("combo", "");
                    fldDefDef = m_rowsetDefParam.FieldDefs[strFld];
                    if (fldDefSearch.LogicalPos > fldDefDef.LogicalPos) {
                        temp        = arrOp[i - 1];
                        arrOp[i -1] = arrOp[i];
                        arrOp[i]    = temp;
                    } else {
                        bDone = true;
                    }
                    i++;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadState               Load the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public virtual void LoadState(RepPP.Toolkit.Window.UIConfigurationHelper helper, System.Xml.XmlNode node) {
            string              strVersion;
            string              strWhere;
            string              strFilter;
            string              strRowsetName;
            RepPP.RowsetDef     rowsetDef;
            
            strVersion  = helper.GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                strWhere   = helper.GetNodeAttribute(node, "WHERECLAUSE");
                strRowsetName = helper.GetNodeAttribute(node, "GROUPNAME");
                if (strRowsetName != null && strRowsetName != String.Empty) {
                    rowsetDef             = m_app.RowsetDefs[strRowsetName];
                    TemplateRowsetDef   = rowsetDef;
                }
                if (strWhere != null) {
                    m_strWhereClause = strWhere;
                }
                strFilter = helper.GetNodeAttribute(node, "FILTERCLAUSE");
                if (strFilter != null) {
                    m_strFilterClause = strFilter;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveState               Save the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public virtual void SaveState(RepPP.Toolkit.Window.UIConfigurationHelper helper, System.Xml.XmlNode node) {
            helper.SetNodeAttribute(node, "VERSION",        "1");
            helper.SetNodeAttribute(node, "GROUPNAME",      m_rowsetDefTemplate.Name);
            helper.SetNodeAttribute(node, "WHERECLAUSE",    m_strWhereClause);
            helper.SetNodeAttribute(node, "FILTERCLAUSE",   m_strFilterClause);
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// SetFieldStateForThisOperator:   Set the field state for a given operator
	    /// </summary>
        /// <param name="iFldIndex">        FieldDef index</param>
        /// <param name="strOP">            Operation</param>
	    //
	    //******************************************************************
        protected virtual void SetFieldStateForThisOperator(int iFldIndex, string strOP) {
            RepPP.RepPPField    fldDefCtlFirst;
            RepPP.RepPPField    fldDefCtlSecond;
            int                 iNewWidth;
            bool                bBetween;
            bool                bNull;
            
            fldDefCtlFirst  = m_arrField[iFldIndex];
            fldDefCtlSecond = m_arrBetweenField[iFldIndex];
            bBetween     = (strOP == m_strBetween);
            bNull        = (strOP == "is null" || strOP == "is not null"); 
            if (bBetween) {
                iNewWidth = fldDefCtlSecond.Left - fldDefCtlFirst.Left - 4;
                if (fldDefCtlFirst.Width > iNewWidth) {
                    fldDefCtlFirst.Width = iNewWidth;
                }
                fldDefCtlFirst.FieldDef.IsRequired  = true;
                fldDefCtlFirst.Visible           = true;
                fldDefCtlSecond.FieldDef.IsRequired = true;
                fldDefCtlSecond.Visible          = true;
                fldDefCtlSecond.FieldDef.IsVisible  = true;
            } else if (bNull) {
                fldDefCtlFirst.FieldDef.IsRequired  = false;
                fldDefCtlSecond.FieldDef.IsRequired = false;
                if (fldDefCtlSecond.FieldDef.IsVisible) {
                    fldDefCtlSecond.FieldDef.IsVisible = false;
                    fldDefCtlSecond.Visible = false;
                    if (!fldDefCtlSecond.FieldDef.IsNull) {
                        fldDefCtlSecond.FieldDef.Value = string.Empty;
                    }
                }
                if (fldDefCtlFirst.FieldDef.IsVisible) {
                    fldDefCtlFirst.Visible = false;
                    if (!fldDefCtlFirst.FieldDef.IsNull) {
                        fldDefCtlFirst.FieldDef.Value = string.Empty;
                    }
                }
            } else {
                fldDefCtlFirst.FieldDef.IsRequired  = false;
                fldDefCtlFirst.Visible           = true;
                fldDefCtlSecond.FieldDef.IsRequired = false;
                if (fldDefCtlSecond.FieldDef.IsVisible) {
                    fldDefCtlSecond.FieldDef.IsVisible = false;
                    fldDefCtlSecond.Visible = false;
                    if (!fldDefCtlSecond.FieldDef.IsNull) {
                        fldDefCtlSecond.FieldDef.Value = string.Empty;
                    }
                    fldDefCtlFirst.Width = m_arrOriginalFieldSize[iFldIndex];
                }
            }
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// combo_SelectedIndexChanged:     Called when a combobox value change
	    /// </summary>
	    /// <param name="sender">           Sender object</param>
	    /// <param name="e">                Event argument</param>
	    //
	    //******************************************************************
        private void combo_SelectedIndexChanged(object sender, EventArgs e) {
            int                 iIndex;
            ComboBox            comboItem;
            string              strOP;
            
            comboItem = (ComboBox)sender;
            iIndex    = 0;
            foreach (ComboBox combo in m_arrComboBox) {
                if (combo == comboItem) {
                    break;
                }
                iIndex++;
            }
            strOP = comboItem.SelectedItem as String;
            SetFieldStateForThisOperator(iIndex, strOP);
        }
    }
}
