using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    /// <summary>
    /// Holds the content of a selection buffer
    /// </summary>
    public abstract class SBHolder : ISelectionBuffer, IDisposable {
        /// <summary>Form containing embedding this class</summary>
        private frmSBEditorBase         m_frmFather;
        /// <summary>Name of the rowset tree use as selection buffer</summary>
        private RepPP.RowsetTreeDef     m_rowsetTreeDef;
        /// <summary>Rowset tree of the selection buffer</summary>
        private RepPP.RowsetTree        m_rowsetTreeSelBuf;
        /// <summary>Filter use for the selection buffer</summary>
        private ISBFilter               m_filter;
        /// <summary>Search methods available for the selection buffer</summary>
        private ISearchMethod[]         m_arrSearchMethods;
        /// <summary>Index of the selection macro to execute when loading the selection buffer</summary>
        private int                     m_iNextSearchMethodIndex;
        /// <summary>Index of the selection macro that was used to load the selection buffer</summary>
        private int                     m_iActiveSearchMethodIndex;
        /// <summary>Method to access selection buffer data</summary>
        private ISBDataAccess           m_dataAccess;
        /// <summary>The selection buffer control</summary>
        protected ISelectionBufferUI    m_selBufUI;
	    /// <summary>true if adding item to the listview and don't want recursive call</summary>
	    protected bool                  m_bAddingItem;

        //******************************************************************
        //
        /// <summary>
        /// SBHolder:                       Class ctor
        /// </summary>
        /// <param name="selBufUI">         Selection buffer control</param>
        /// <param name="frmFather">        Father form</param>
        /// <param name="strRowsetTreeName">Name of the rowset tree representing the selection buffer</param>
        //
        //******************************************************************
        public SBHolder(ISelectionBufferUI selBufUI, frmSBEditorBase frmFather, string strRowsetTreeName) {
            m_selBufUI                      = selBufUI;
            m_frmFather                     = frmFather;
            m_rowsetTreeDef                 = frmFather.Application.RowsetTreeDefs[strRowsetTreeName];
            m_rowsetTreeSelBuf              = null;
            m_dataAccess                    = null;
            m_bAddingItem                   = false;
            if (m_rowsetTreeDef == null) {
                throw new System.ApplicationException("The rowset tree  '" + strRowsetTreeName + "' is not defined!");
            }
            m_filter = CreateFilter(m_rowsetTreeDef.Nodes[0].RowsetDef);
            LoadSearchMethods();
            m_iNextSearchMethodIndex   = -1;
            m_iActiveSearchMethodIndex = -1;
            m_rowsetTreeDef.BuildSqlCommand(true);
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        //
        //******************************************************************
        public virtual void Dispose() {
            if (m_rowsetTreeSelBuf != null) {
                m_rowsetTreeSelBuf.Dispose();
                m_rowsetTreeSelBuf = null;
            }
            if (m_filter != null) {
                m_filter.Dispose();
                m_filter = null;
            }
            m_selBufUI = null;
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectionBufferDataAccess:  Selection buffer data access
        /// </summary>
        //
        //******************************************************************
        public virtual RepPP.Toolkit.ISBDataAccess SelectionBufferDataAccess {
            get {
                return(m_dataAccess);
            }
            set {
                m_dataAccess = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Father:                     Father form editor
        /// </summary>
        //
        //******************************************************************
        public virtual frmSBEditorBase FatherEditor {
            get {
                return(m_frmFather);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RowsetTree:               Rowset tree holding the selection buffer data
        /// </summary>
        //
        //******************************************************************
        public virtual RepPP.RowsetTree RowsetTree {
            get {
                return(m_rowsetTreeSelBuf);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateFilter:               Creates a filter
        /// </summary>
        //
        //******************************************************************
        protected virtual ISBFilter CreateFilter(RepPP.RowsetDef rowsetDef) {
            return(new SBFilter(rowsetDef, true));
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadSearchMethods:          Loads the macros associated with 
        ///                             the selection buffer
        /// </summary>
        //
        //******************************************************************
        protected virtual void LoadSearchMethods() {
            ISearchMethod[] arrSearchMethods;

            if (m_arrSearchMethods != null) {
                foreach (ISearchMethod method in m_arrSearchMethods) {
                    method.BeforeExecute -= new BeforeExecuteEventHandler(m_frmFather.OnBeforeSearchMethodExecute);
                    method.AfterValidate -= new CancelEventHandler(m_frmFather.OnAfterSearchMethodValidate);
                }
            }
#if !(SD60 || SD70)
            arrSearchMethods = SearchMacro.Load(m_rowsetTreeDef);
#else
            arrSearchMethods = new ISearchMethod[0];
#endif
            m_arrSearchMethods    = new ISearchMethod[arrSearchMethods.Length + 1]; 
            m_arrSearchMethods[0] = new DirectSearchMethod();
            arrSearchMethods.CopyTo(m_arrSearchMethods, 1);
            foreach (ISearchMethod method in m_arrSearchMethods) {
                method.BeforeExecute += new BeforeExecuteEventHandler(m_frmFather.OnBeforeSearchMethodExecute);
                method.AfterValidate += new CancelEventHandler(m_frmFather.OnAfterSearchMethodValidate);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Fill:                           Fill the selection buffer
        /// </summary>
        /// <param name="bAddNewIfEmpty">   Indicates if a new item should be added if the selection buffer will be empty </param>
        //
        //******************************************************************
        public virtual void Fill(bool bAddNewIfEmpty) {
            ISBFilter               filter;
            ISBDataAccess           dataAccess;
            
            if (m_rowsetTreeSelBuf == null) {
                m_rowsetTreeSelBuf = m_rowsetTreeDef.RowsetTrees.Add();
            }
            filter                  = CurrentFilter;
            dataAccess              = SelectionBufferDataAccess;
            ActiveSearchMethodIndex = -1;
            if (NextSearchMethodIndex != -1) {
                SearchMethods[NextSearchMethodIndex].ReadRowsetTreeUsingSearchMethod(m_rowsetTreeSelBuf, dataAccess);
                ActiveSearchMethodIndex = NextSearchMethodIndex;
                NextSearchMethodIndex   = -1;
            } else if (filter != null) {
                filter.ReadRowsetTreeUsingFilterV(m_rowsetTreeSelBuf, "", dataAccess);
            } else {
                if (dataAccess != null) {
                    dataAccess.LoadSelectionBufferData(m_rowsetTreeSelBuf, "");
                } else {
                    m_rowsetTreeSelBuf.ReadFromDb();
                }
            }
            PopulateSBControl(bAddNewIfEmpty);
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:       Add an item to the selection buffer
        /// </summary>
        /// <param name="rowsetSB">         Rowset of the selection buffer</param>
        /// <param name="bDuplicate">       true if duplicate the item</param>
        //
        //******************************************************************
        public virtual void AddItemToSelectionBuffer(Rowset rowsetSB, bool bDuplicate) {
            m_bAddingItem = true;
            m_selBufUI.AddItemToSelectionBuffer(rowsetSB, bDuplicate);
            m_bAddingItem = false;
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh the item in the selection buffer
        /// </summary>
        /// <param name="rowset">           Rowset where to take the updated value</param>
        //
        //******************************************************************
        public virtual void RefreshItemInSelectionBuffer(Rowset rowset) {
            m_selBufUI.RefreshItemInSelectionBuffer(rowset);
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete the item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to reselect a new item after deletion</param>
        //
        //******************************************************************
        public virtual void DeleteItemFromSelectionBuffer(bool bReselect) {
            m_selBufUI.DeleteItemFromSelectionBuffer(bReselect);
        }

        //******************************************************************
        //
        /// <summary>
        /// ItemCount:                      Get the number of items if the current kind
        /// </summary>
        //
        //******************************************************************
        public virtual int ItemCount {
            get {
                return(m_selBufUI.GetItemCount());
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemIndex:                   Get the current item index for the current kind
        /// </summary>
        //
        //******************************************************************
        public virtual int CurrentItemIndex {
            get {
                return(m_selBufUI.GetCurrentItem());
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select an item in the selection buffer
        /// </summary>
        /// <param name="iPos">             Position of the item to select (0..n)</param>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //
        //******************************************************************
        public virtual bool SelectItem(int iPos) {
            return(m_selBufUI.SelectItem(iPos));
        }

        //******************************************************************
        //
        /// <summary>
        /// PopulateSBControl:                Populates the selection buffer
        ///                                   control using the selection buffer
        ///                                   data. Called automatically, once
        ///                                   the data is read.
        /// </summary>
        /// <param name="bAddNewIfEmpty">     Indicates if a new item should be added if the selection buffer is empty </param>
        //
        //******************************************************************
        public abstract void PopulateSBControl(bool bAddNewIfEmpty);

        //******************************************************************
        //
        /// <summary>
        /// CurrentRowset:           Returns the selection buffer rowset associated with the current item
        /// </summary>
        //
        //******************************************************************
        public abstract Rowset CurrentRowset {
            get;
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItemPK:               Gets the currently selected item primary key
        /// </summary>
        /// <returns>
        /// PK values
        /// </returns>
        //
        //******************************************************************
        public abstract string[] CurrentItemPK {
            get;
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemObject:              Gets the object currently selected in the selection buffer (node, listview item...)
        /// </summary>
        //
        //******************************************************************
        public abstract object CurrentItemObject {
            get;
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemType:                Gets the editor type of the currently selected item
        /// </summary>
        //
        //******************************************************************
        public abstract int CurrentItemType {
            get;
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentFilter:                  Gets the current filter
        /// </summary>
        //
        //******************************************************************
        public virtual ISBFilter CurrentFilter {
            get {
                return(m_filter);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SearchMethods:                  Gets the current search methods
        /// </summary>
        //
        //******************************************************************
        public virtual ISearchMethod[] SearchMethods {
            get {
                return(m_arrSearchMethods);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// NextSearchMethodIndex:          Gets the index of the current search
        ///                                 method to be executed
        /// </summary>
        //
        //******************************************************************
        public virtual int NextSearchMethodIndex {
            get {
                return(m_iNextSearchMethodIndex);
            }
            set{
                m_iNextSearchMethodIndex = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ActiveSearchMethodIndex:        Gets the index of the search
        ///                                 method used to load the SB
        /// </summary>
        //
        //******************************************************************
        public virtual int ActiveSearchMethodIndex {
            get {
                return(m_iActiveSearchMethodIndex);
            }
            set{
                m_iActiveSearchMethodIndex = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReloadSearchMethods:             Reloads the list of search methods
        /// </summary>
        //
        //******************************************************************
        public virtual void ReloadSearchMethods() {
            LoadSearchMethods();
            m_iNextSearchMethodIndex    = -1;
            m_iActiveSearchMethodIndex  = -1;
        }
    }
}
