using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Adapter of a ListViewRowset so it can be used as a Selection Buffer
	/// </summary>
	public class SBHolderListView : SBHolder, IDisposable {

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class ctor
        /// </summary>
        /// <param name="listView">         Listview representing the selection buffer</param>
        /// <param name="frmFather">        Father form</param>
        /// <param name="strRowsetTreeName">Name of the rowset tree representing the selection buffer</param>
        //
        //******************************************************************
        public SBHolderListView(ListViewRowset listView, frmSBEditorBase frmFather, string strRowsetTreeName) : base(listView, frmFather, strRowsetTreeName) {
            listView.BeforeSelect        += new ListViewRowset.BeforeSelectHandler(BeforeItemSelected);
            listView.AfterSelect         += new ListViewRowset.AfterSelectHandler(AfterItemSelected);
            listView.ColumnClick         += new ColumnClickEventHandler(OnColumnClick);
            listView.AutoSort             = true;
            listView.AllowColumnReorder   = true;
            //listView.ContextMenu          = frmFather.CreateSBContextMenu();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        //
        //******************************************************************
        public override void Dispose() {
            ListViewRowset    listView;

            listView = ListView;
            if (listView != null) {
                listView.BeforeSelect -= new ListViewRowset.BeforeSelectHandler(BeforeItemSelected);
                listView.AfterSelect  -= new ListViewRowset.AfterSelectHandler(AfterItemSelected);
                listView.ColumnClick  -= new ColumnClickEventHandler(OnColumnClick);
            }
            base.Dispose();
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(m_selBufUI as ListViewRowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// BeforeItemSelected:         Called before a item is selected in the listview
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        private void BeforeItemSelected(object sender, ListViewRowset.BeforeSelectEventArgs e) {
            if (!m_bAddingItem) {
                if (e.Reselect) {
                    e.Cancel = true;
                } else if (!FatherEditor.BeforeItemSelected()) {
                    e.Cancel = true;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterItemSelected:          Called after an item has been selected in the listview
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        private void AfterItemSelected(object sender, ListViewRowset.AfterSelectEventArgs e) {
            if (!m_bAddingItem) {
                FatherEditor.AfterItemSelected();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnColumnClick:              Called when a listview column is clicked
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        private void OnColumnClick(object sender, ColumnClickEventArgs e) {
            ListViewItem    item;
            ListViewRowset   listView;
            
            listView    = ListView;
            item        = listView.SelectedItem;
            if (item != null) {
                listView.EnsureVisible(item.Index);
            }
            FatherEditor.ChildAdjustState();
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentRowset:           Returns the selection buffer rowset associated with the current item
        /// </summary>
        //
        //******************************************************************
        public override Rowset CurrentRowset { 
            get {
                return((ListView == null) ? null : ListView.Rowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItemPK:               Gets the currently selected item primary key
        /// </summary>
        /// <returns>
        /// PK values
        /// </returns>
        //
        //******************************************************************
        public override string[] CurrentItemPK {
            get {
                string[]    arrRetVal;
                ArrayList   arrList;
                FieldDefs   fldDefs;
                
                arrList = new ArrayList();
                fldDefs = CurrentRowset.RowsetDef.FieldDefs;
                foreach (FieldDef fldDef in fldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        arrList.Add(fldDef.Value);
                    }
                }
                arrRetVal = new string[arrList.Count];
                arrList.CopyTo(arrRetVal);
                return(arrRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemObject:              Gets the object currently selected in the selection buffer (node, listview item...)
        /// </summary>
        //
        //******************************************************************
        public override object CurrentItemObject {
            get {
                return(ListView.SelectedItem);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemType:                Gets the editor type of the currently selected item
        /// </summary>
        //
        //******************************************************************
        public override int CurrentItemType {
            get {
                return(0);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// PopulateSBControl:                Populates the selection buffer
        ///                                   control using the selection buffer
        ///                                   data. Called automatically, once
        ///                                   the data is read.
        /// </summary>
        /// <param name="bAddNewIfEmpty">     Indicates if a new item should be added if the selection buffer is empty </param>
        //
        //******************************************************************
        public override void PopulateSBControl(bool bAddNewIfEmpty) {
            RepPP.Rowset    rowset;
            ListViewRowset  listView;
            
            listView                = ListView;
            rowset                  = RowsetTree.RootRowset;
            listView.MinColumnSize = 5;
            listView.MaxColumnSize = 20;
            listView.RowsetDef     = rowset.RowsetDef;
            listView.Fill(rowset, true);
            if (listView.Items.Count != 0) {
                if (listView.SelectedItems.Count == 0) {
                    listView.Items[0].Selected = true;
                }
                listView.EnsureVisible(listView.SelectedIndices[0]);
            } else if (bAddNewIfEmpty) {
                FatherEditor.InsertNewLine(0, false, true, "");
            }
        }
    }
}
