using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Adapter of a TreeViewRowsetTree so it can be used as a Selection Buffer
	/// </summary>
	public class SBHolderTreeView : SBHolder, IDisposable {

        //******************************************************************
        //
        /// <summary>
        /// SBTreeView:                     Class ctor
        /// </summary>
        /// <param name="treeView">         Treeview representing the selection buffer</param>
        /// <param name="frmFather">        Father form</param>
        /// <param name="strRowsetTreeName">Name of the rowset tree representing the selection buffer</param>
        //
        //******************************************************************
		public SBHolderTreeView(TreeViewRowsetTree treeView, frmSBEditorBase frmFather, string strRowsetTreeName) : base(treeView, frmFather, strRowsetTreeName) {
			treeView.BeforeSelect     += new TreeViewCancelEventHandler(BeforeNodeSelected);
			treeView.AfterSelect      += new TreeViewEventHandler(AfterNodeSelected);
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        //
        //******************************************************************
        public override void Dispose() {
            TreeView    treeView;

            treeView = TreeView;
            if (treeView != null) {
			    treeView.BeforeSelect -= new TreeViewCancelEventHandler(BeforeNodeSelected);
			    treeView.AfterSelect  -= new TreeViewEventHandler(AfterNodeSelected);
                treeView               = null;
            }
            base.Dispose();
        }

        //******************************************************************
        //
        /// <summary>
        /// TreeView:                   Selection buffer treeview
        /// </summary>
        //
        //******************************************************************
        public TreeViewRowsetTree TreeView {
            get {
                return(m_selBufUI as TreeViewRowsetTree);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// BeforeNodeSelected:         Called before a node is selected
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        protected virtual void BeforeNodeSelected(object sender, TreeViewCancelEventArgs e) {
            if (!m_bAddingItem) {
                m_bAddingItem = true;
                if (!FatherEditor.BeforeItemSelected()) {
                    e.Cancel = true;
                }
                m_bAddingItem = false;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterNodeSelected:          Called after a node has been selected
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        protected virtual void AfterNodeSelected(object sender, TreeViewEventArgs e) {
            if (!m_bAddingItem) {
                FatherEditor.AfterItemSelected();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentRowset:           Returns the selection buffer rowset 
        ///                          associated with the current item
        /// </summary>
        //
        //******************************************************************
        public override Rowset CurrentRowset { 
            get {
                Rowset          rowsetRetVal = null;
                TreeNodeRowset  nodeStd;
                
                nodeStd = TreeView.SelectedNode as TreeNodeRowset;
                if (nodeStd != null) {
                    rowsetRetVal = nodeStd.Rowset;
                } else {
                    rowsetRetVal = (RowsetTree != null) ? RowsetTree.RootRowset : null;
                }
                return(rowsetRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItemPK:               Gets the currently selected item primary key
        /// </summary>
        /// <returns>
        /// PK values
        /// </returns>
        //
        //******************************************************************
        public override string[] CurrentItemPK {
            get {
                string[]        arrRetVal;
                ArrayList       arrPK;
                Rowset          rowsetNode;
                TreeView        treeView;
                TreeNode        node;
                TreeNodeRowset  treeNode;
                
                treeView    = TreeView;
                arrPK       = new ArrayList();
                node        = treeView.SelectedNode;
                while (node != null && !(node is TreeNodeRowset)) {
                    node = node.Parent;
                } 
                while (node != null && node is TreeNodeRowset) {
                    treeNode  = node as TreeNodeRowset;
                    rowsetNode = treeNode.Rowset;
                    rowsetNode.Select();
                    foreach (FieldDef fldDef in rowsetNode.RowsetDef.FieldDefs) {
                        if (fldDef.IsPrimaryKey) {
                            arrPK.Insert(0, fldDef.Value);
                        }
                    }
                    node = node.Parent;
                }
                arrRetVal = new string[arrPK.Count];
                arrPK.CopyTo(arrRetVal);
                return(arrRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemObject:              Gets the object currently selected in the selection buffer (node, listview item...)
        /// </summary>
        //
        //******************************************************************
        public override object CurrentItemObject {
            get {
                return(TreeView.SelectedNode);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemType:                Gets the editor type of the currently selected item
        /// </summary>
        //
        //******************************************************************
        public override int CurrentItemType {
            get {
                int             iRetVal = 0;
                TreeNodeBase    node;

                node = TreeView.SelectedNode as TreeNodeBase;
                if (node != null) {
                    iRetVal = node.NodeType;
                }
                return(iRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// PopulateSBControl:                Populates the selection buffer
        ///                                   control using the selection buffer
        ///                                   data. Called automatically, once
        ///                                   the data is read.
        /// </summary>
        /// <param name="bAddNewIfEmpty">     Indicates if a new item should be added if the selection buffer is empty </param>
        //
        //******************************************************************
        public override void PopulateSBControl(bool bAddNewIfEmpty) {
            Fill(null, bAddNewIfEmpty);
        }

        //******************************************************************
        //
        /// <summary>
        /// Fill:                           Fill the selection buffer using a child node
        ///                                 factory
        /// </summary>
        /// <param name="nodeFactory">      Child node factory</param>
        /// <param name="bAddNewIfEmpty">   Flag to insert a new item if there is
        ///                                 no data</param>
        //
        //******************************************************************
        public virtual void Fill(TreeNodeRowset.IRowsetChildNodeFactory nodeFactory, bool bAddNewIfEmpty) {
            TreeViewRowsetTree   treeView;

            treeView = TreeView;
            treeView.FillNodes(RowsetTree.RootRowset, nodeFactory);
            if (treeView.Nodes.Count != 0) {
                if (treeView.SelectedNode == null) {
                    treeView.SelectedNode = treeView.Nodes[0];
                }
            } else if (bAddNewIfEmpty) {
                FatherEditor.InsertNewLine(0, false, true, "");
            }
        }
	}
}
