using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Adapter of a TreeViewRowsetTree and ListViewRowset so they can be used together as a Selection Buffer
	/// </summary>
	public class SBHolderTreeViewListView : SBHolderTreeView {
	    /// <summary>Listview attached to the selection buffer</summary>
	    private ListViewRowset   m_listView;
	    /// <summary>Hashlist of rowset tree created for the ListView</summary>
	    private Hashtable       m_hashRowsetTree;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class ctor
        /// </summary>
        /// <param name="treeView">         Treeview representing the selection buffer</param>
        /// <param name="listView">         ListView representing the selection buffer</param>
        /// <param name="frmFather">        Father form</param>
        /// <param name="strRowsetTreeName">Name of the rowset tree definining the content of the selection buffer</param>
        //
        //******************************************************************
		public SBHolderTreeViewListView(TreeViewRowsetTree treeView, ListViewRowset listView, frmSBEditorBase frmFather, string strRowsetTreeName) : base(treeView, frmFather, strRowsetTreeName) {
            m_listView                      = listView;
            m_listView.BeforeSelect        += new ListViewRowset.BeforeSelectHandler(BeforeListViewItemSelected);
            m_listView.AfterSelect         += new ListViewRowset.AfterSelectHandler(AfterListViewItemSelected);
            m_listView.ColumnClick         += new ColumnClickEventHandler(OnListViewColumnClick);
            m_hashRowsetTree                = new Hashtable();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        //
        //******************************************************************
        public override void Dispose() {
            if (m_listView != null) {
                m_listView.BeforeSelect -= new ListViewRowset.BeforeSelectHandler(BeforeListViewItemSelected);
                m_listView.AfterSelect  -= new ListViewRowset.AfterSelectHandler(AfterListViewItemSelected);
                m_listView.ColumnClick  -= new ColumnClickEventHandler(OnListViewColumnClick);
                m_listView               = null;
            }
            foreach (RowsetTree rowsetTree in m_hashRowsetTree.Values) {
                rowsetTree.Delete();
            }
            m_hashRowsetTree = null;
            base.Dispose();
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(m_listView);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// FillListView:               Fill the listview with the corresponding data
        /// </summary>
        /// <param name="node">         The current node</param>
        /// <param name="strItemKey">   Item to select if not null </param>
        //
        //******************************************************************
        public virtual void FillListView(TreeNodeListView node, string strItemKey) {
            TreeNodeRowset      nodeFather;
            TreeNode            nodeTmp;
            string              strWhereClause;
            string              strRowsetTreeName;
            RowsetTree          rowsetTree;
            Rowset              rowset;
            TreeViewRowsetTree  treeView;
            ListViewRowset      listView;
            ISBDataAccess       dataAccess;

            //SaveListViewConfigV();
            dataAccess              = SelectionBufferDataAccess;
            listView                = ListView;
            treeView                = TreeView;
            strWhereClause          = String.Empty;
            nodeTmp                 = node.Parent;
            listView.ContextMenuStrip = null;
            while ((nodeTmp != null) && !(nodeTmp is TreeNodeRowset)) {
                nodeTmp = nodeTmp.Parent;
            }
            nodeFather = nodeTmp as TreeNodeRowset;
            if (nodeFather != null) {
                strWhereClause = RowsetTreeDefNodeHandlerRootForSB.BuildWhereClause(nodeFather.Rowset.RowsetTreeDefNode, true);
            }
            strRowsetTreeName = node.SelectionBufferRowsetTreeName;
            if (m_hashRowsetTree.ContainsKey(strRowsetTreeName)) {
                rowsetTree = m_hashRowsetTree[strRowsetTreeName] as RowsetTree;
            } else {
                rowsetTree = FatherEditor.Application.RowsetTreeDefs[strRowsetTreeName].RowsetTrees.Add();
                m_hashRowsetTree.Add(strRowsetTreeName, rowsetTree);
            }
            if (rowsetTree.RowsetTreeDef.Nodes[0].SelectSqlCommand == String.Empty) {
                rowsetTree.RowsetTreeDef.BuildSqlCommand(true);
            }
            if (node.Filter != null) {
                node.Filter.ReadRowsetTreeUsingFilterV(rowsetTree, strWhereClause, dataAccess);
            } else {
                rowsetTree.RowsetTreeDef.Nodes[0].SetSelectSqlWhere(strWhereClause);
                if (dataAccess == null) {
                    rowsetTree.ReadFromDb();
                } else {
                    dataAccess.LoadSelectionBufferData(rowsetTree, strWhereClause);
                }
            }
            rowset                 = rowsetTree.RootRowset;
            listView.MinColumnSize = 5;
            listView.MaxColumnSize = 20;
            listView.RowsetDef         = rowset.RowsetDef;
            // LoadListViewConfigV();
            listView.Fill(rowset, true);
            if (listView.Items.Count != 0) {
                if (!listView.SelectItemFromKey(strItemKey)) {
                    listView.Items[0].Selected = true;
                }
                listView.EnsureVisible(m_listView.SelectedIndices[0]);
            }
            FatherEditor.AfterItemSelected();
            listView.ContextMenuStrip = treeView.ContextMenuStrip;
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterNodeSelected:          Called after a node has been selected
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        protected override void AfterNodeSelected(object sender, TreeViewEventArgs e) {
            ListViewRowset   listView;
            
            if (!m_bAddingItem) {
                listView = ListView;
                if (e.Node is TreeNodeListView) {
                    FillListView(e.Node as TreeNodeListView, null);
                    listView.Visible = true;
                } else {
                    listView.RowsetDef       = null;
                    listView.ContextMenu = null;
                    listView.Visible     = false;
                    FatherEditor.AfterItemSelected();
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// BeforeListViewItemSelected:     Called before a item is selected in the listview
        /// </summary>
        /// <param name="sender">           Sender object   </param>
        /// <param name="e">                Event arguments </param>
        //
        //******************************************************************
        private void BeforeListViewItemSelected(object sender, ListViewRowset.BeforeSelectEventArgs e) {
            if (!m_bAddingItem) {
                if (!FatherEditor.BeforeItemSelected()) {
                    e.Cancel = true;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterListViewItemSelected:  Called after an item has been selected in the listview
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        private void AfterListViewItemSelected(object sender, ListViewRowset.AfterSelectEventArgs e) {
            if (!m_bAddingItem) {
                FatherEditor.AfterItemSelected();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnListViewColumnClick:      Called when a listview column is clicked
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        private void OnListViewColumnClick(object sender, ColumnClickEventArgs e) {
            ListViewItem    item;
            ListViewRowset  listView;
            
            listView    = ListView;
            item        = listView.SelectedItem;
            if (item != null) {
                listView.EnsureVisible(item.Index);
            }
            FatherEditor.ChildAdjustState();
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:       Add an item to the selection buffer
        /// </summary>
        /// <param name="rowsetSB">          RowsetDef instance of the selection buffer</param>
        /// <param name="bDuplicate">       true if duplicate the item</param>
        //
        //******************************************************************
        public override void AddItemToSelectionBuffer(Rowset rowsetSB, bool bDuplicate) {
            TreeNode    node;
            
            node = TreeView.SelectedNode;
            m_bAddingItem = true;
            if (node != null && node is TreeNodeListView) {
                ListView.AddItemToSelectionBuffer(ListView.Rowset, bDuplicate);
            } else {
                base.AddItemToSelectionBuffer(rowsetSB, bDuplicate);
            }
            m_bAddingItem = false;
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh the item in the selection buffer
        /// </summary>
        /// <param name="rowset">            RowsetDef instance where to take the updated value</param>
        //
        //******************************************************************
        public override void RefreshItemInSelectionBuffer(Rowset rowset) {
            TreeNode    node;
            
            node = TreeView.SelectedNode;
            if (node is TreeNodeListView) {
                ListView.RefreshItemInSelectionBuffer(rowset);
            } else {
                base.RefreshItemInSelectionBuffer(rowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete the item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to reselect a new item after deletion</param>
        //
        //******************************************************************
        public override void DeleteItemFromSelectionBuffer(bool bReselect) {
            TreeNode    node;
            
            node = TreeView.SelectedNode;
            if (node is TreeNodeListView) {
                ListView.DeleteItemFromSelectionBuffer(bReselect);
            } else {
                base.DeleteItemFromSelectionBuffer(bReselect);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ItemCount:                      Get the number of items if the current kind
        /// </summary>
        //
        //******************************************************************
        public override int ItemCount {
            get {
                int         iRetVal;
                TreeNode    node;
                
                node = TreeView.SelectedNode;
                if (node is TreeNodeListView) {
                    iRetVal = ListView.GetItemCount();
                } else {
                    iRetVal = base.ItemCount;
                }
                return(iRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemIndex:                   Get the current item index for the current kind
        /// </summary>
        //
        //******************************************************************
        public override int CurrentItemIndex {
            get {
                int         iRetVal;
                TreeNode    node;
                
                node = TreeView.SelectedNode;
                if (node is TreeNodeListView) {
                    iRetVal = ListView.GetCurrentItem();
                } else {
                    iRetVal = base.CurrentItemIndex;
                }
                return(iRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select an item in the selection buffer
        /// </summary>
        /// <param name="iPos">             Position of the item to select (0..n)</param>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //
        //******************************************************************
        public override bool SelectItem(int iPos) {
            bool        bRetVal;
            TreeNode    node;
            
            node = TreeView.SelectedNode;
            if (node is TreeNodeListView) {
                bRetVal = ListView.SelectItem(iPos);
            } else {
                bRetVal = base.SelectItem(iPos);
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentRowset:           Returns the selection buffer rowset associated with the current item
        /// </summary>
        //
        //******************************************************************
        public override Rowset CurrentRowset { 
            get {
                Rowset          rowsetRetVal = null;
                TreeNodeText    nodeExt;
                
                nodeExt = TreeView.SelectedNode as TreeNodeListView;
                if (nodeExt != null) {
                    rowsetRetVal = ListView.Rowset;
                } else {
                    rowsetRetVal = base.CurrentRowset;
                }
                return(rowsetRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItemPK:               Gets the currently selected item primary key
        /// </summary>
        /// <returns>
        /// PK values
        /// </returns>
        //
        //******************************************************************
        public override string[] CurrentItemPK {
            get {
                string[]    arrRetVal;
                TreeNode    node;
                ArrayList   arrList;
                FieldDefs   fldDefs;
                
                node = TreeView.SelectedNode;
                if (node is TreeNodeListView) {
                    arrList = new ArrayList();
                    fldDefs = ListView.Rowset.RowsetDef.FieldDefs;
                    foreach (FieldDef fldDef in fldDefs) {
                        if (fldDef.IsPrimaryKey) {
                            arrList.Add(fldDef.Value);
                        }
                    }
                    arrRetVal = new string[arrList.Count];
                    arrList.CopyTo(arrRetVal);
                } else {
                    arrRetVal = base.CurrentItemPK;
                }
                return(arrRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemObject:              Gets the object currently selected in the selection buffer (node, listview item...)
        /// </summary>
        //
        //******************************************************************
        public override object CurrentItemObject {
            get {
                object      objRetVal;
                TreeNode    node;
                
                node = TreeView.SelectedNode;
                if (node is TreeNodeListView) {
                    objRetVal = ListView.SelectedItem;
                } else {
                    objRetVal = base.CurrentItemObject;
                }
                return(objRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentFilter:              Get thye current filter
        /// </summary>
        //
        //******************************************************************
        public override ISBFilter CurrentFilter {
            get {
                ISBFilter   filterRetVal;
                TreeNode    node;
                
                node = TreeView.SelectedNode;
                if (node is TreeNodeListView) {
                    filterRetVal = (node as TreeNodeListView).Filter;
                } else {
                    filterRetVal = base.CurrentFilter;
                }
                return(filterRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Fill:                           Fill the selection buffer
        /// </summary>
        /// <param name="bAddNewIfEmpty">   Indicates if a new item should be added if the selection buffer will be empty </param>
        //
        //******************************************************************
        public override void Fill(bool bAddNewIfEmpty) {
            TreeNode    node;
            
            node = TreeView.SelectedNode;
            if (node is TreeNodeListView) {
                FillListView(node as TreeNodeListView, null);
            } else {
                base.Fill(bAddNewIfEmpty);
            }            
        }

        //******************************************************************
        //
        /// <summary>
        /// TreeNodeListView:   Class defining a treenode to which a listview is attached
        /// </summary>
        //
        //******************************************************************
        public class TreeNodeListView : TreeNodeText {
            /// <summary>Name of the rowset tree used to fill the listview</summary>
            private string              m_strSBRowsetTreeName;
            /// <summary>Filter object</summary>
            private ISBFilterUI         m_filter;

            //*********************************************************	    
            //
            /// <summary>
            /// ctor:                                   Class constructor
            /// </summary>
            /// <param name="treeFather">               Containing tree control</param>
            /// <param name="strText">                  Text to associate to the node</param>
            /// <param name="iNodeType">                Type to associate to the node</param>
            /// <param name="iImageIndex">              Index of the image for the unselected node</param>
            /// <param name="iSelectedImageIndex">      Index of the image for the selected node</param>
            /// <param name="strSBRowsetTreeName">      Name of the rowset tree used to fill the listview</param>
            /// <param name="childNodeFactory">         Node factory used to create child. Can be NULL</param>
            //
            //*********************************************************	    
            public TreeNodeListView(TreeViewRowsetTree                       treeFather,
                                    string                                   strText,
                                    int                                      iNodeType,
                                    int                                      iImageIndex,
                                    int                                      iSelectedImageIndex,
                                    string                                   strSBRowsetTreeName,
                                    TreeNodeRowset.IRowsetChildNodeFactory     childNodeFactory) : base(treeFather, 
                                                                                                      strText, 
                                                                                                      iNodeType,
                                                                                                      iImageIndex,
                                                                                                      iSelectedImageIndex,
                                                                                                      null, 
                                                                                                      childNodeFactory) {
                m_strSBRowsetTreeName = strSBRowsetTreeName;
            }

            //*********************************************************	    
            //
            /// <summary>
            /// ctor:                                   Class constructor
            /// </summary>
            /// <param name="treeFather">               Father treeview</param>
            /// <param name="strText">                  Text of the node or null to get the rowset values</param>
            /// <param name="strSBRowsetTreeName">      Name of the rowset tree used to fill the listview</param>
            /// <param name="childNodeFactory">         Node factory used to create child. Can be NULL</param>
            //
            //*********************************************************	    
            public TreeNodeListView(TreeViewRowsetTree                       treeFather,
                                    string                                   strText,
                                    string                                   strSBRowsetTreeName,
                                    TreeNodeRowset.IRowsetChildNodeFactory   childNodeFactory) : base(treeFather, strText, null, childNodeFactory) {
                m_strSBRowsetTreeName = strSBRowsetTreeName;
            }

            //******************************************************************
            //
            /// <summary>
            /// Dispose:        Dispose the node
            /// </summary>
            //
            //******************************************************************
            public override void Dispose() {
                if (m_filter != null) {
                    m_filter.Dispose();
                    m_filter = null;
                }
                base.Dispose();
            }

            //*********************************************************	    
            //
            /// <summary>
            /// CreateFilter:                   Create the filter object
            /// </summary>
            /// <param name="rowsetDefParam">   Filter rowset parameter</param>
            //
            //*********************************************************	    
            protected virtual SBFilter CreateFilter(RowsetDef rowsetDefParam) {
                return(new SBFilter(rowsetDefParam));
            }

            //*********************************************************	    
            //
            /// <summary>
            /// InitializeFilter:           Create and initialize the filter object
            /// </summary>
            /// <param name="treeFather">   Father treeview</param>
            //
            //*********************************************************	    
            protected virtual void InitializeFilter(TreeViewRowsetTree treeFather) {
                RepPP.IApplication  app;
                RepPP.RowsetTreeDef rowsetTreeDef;

                app = RepPP.Application.GetApplication();
                if (app != null) {
                    rowsetTreeDef = app.RowsetTreeDefs[m_strSBRowsetTreeName];          
                    m_filter      = CreateFilter(rowsetTreeDef.Nodes[0].RowsetDef);
                    if (m_filter != null) {
                        LoadState(treeFather.ConfigurationHelper, treeFather.UIConfigurationNode);
                    }
                } else {
                    m_filter = null;
                }
            }

            //*********************************************************	    
            //
            /// <summary>
            /// SelectionBufferRowsetTreeName:  The name of the rowset tree used to fill the listview
            /// </summary>
            //
            //*********************************************************	    
            public string SelectionBufferRowsetTreeName {
                get {
                    return(m_strSBRowsetTreeName);
                }
                set {
                    m_strSBRowsetTreeName = value;
                }
            }

            //*********************************************************	    
            //
            /// <summary>
            /// Filter:     The listview filter
            /// </summary>
            //
            //*********************************************************	    
            public ISBFilterUI Filter {
                get {
                    if (m_filter == null) {
                        InitializeFilter(TreeView as TreeViewRowsetTree);
                    }
                    return(m_filter);
                }
                set {
                    if (m_filter != null) {
                        m_filter.Dispose();
                    }
                    m_filter = value;
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// LoadState               Load the UI configuration state
            /// </summary>
            /// <param name="helper">   Helper object</param>
            /// <param name="node">     Node containing the serialization</param>
            //
            //******************************************************************
            public override void LoadState(RepPP.Toolkit.Window.UIConfigurationHelper helper, System.Xml.XmlNode node) {
                TreeViewRowsetTree  treeFather;
                XmlNode             nodeChild;
                string              strKey;
                string              strVersion;
                
                treeFather = TreeView as TreeViewRowsetTree;
                if (treeFather != null) {
                    strKey          = treeFather.GetNodeFullKey(this);
                    nodeChild       = helper.FindNodeUsingAttribute(node, "NODEINFO", "VALUE", strKey, true);
                    strVersion      = helper.GetNodeAttribute(nodeChild, "VERSION");
                    if (strVersion != null && strVersion == "1" && m_filter != null) {
                        m_filter.LoadState(helper, nodeChild);
                    }
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// SaveState               Save the UI configuration state
            /// </summary>
            /// <param name="helper">   Helper object</param>
            /// <param name="node">     Node containing the serialization</param>
            //
            //******************************************************************
            public override void SaveState(RepPP.Toolkit.Window.UIConfigurationHelper helper, System.Xml.XmlNode node) {
                TreeViewRowsetTree  treeFather;
                XmlNode             nodeChild;
                string              strKey;
                
                treeFather = TreeView as TreeViewRowsetTree;
                if (treeFather != null) {
                    if (m_filter != null) {
                        strKey    = treeFather.GetNodeFullKey(this);
                        nodeChild = helper.FindNodeUsingAttribute(node, "NODEINFO", "VALUE", strKey, true);
                        helper.SetNodeAttribute(nodeChild, "VERSION", "1");
                        m_filter.SaveState(helper, nodeChild);
                    }
                }
            }
        } //SBHolderTreeViewListView
	}
}
