using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Base implementation of the interface for a non dockable selection buffer editor. 
    /// </summary>
    //
    //********************************************************************************    
    public class frmEditor : System.Windows.Forms.Form {
        /// <summary>Required designer variable.</summary>
        private System.ComponentModel.Container             components = null;
        /// <summary>Hold all the commands of the top editor</summary>
        private CmdHolder                                   m_cmdHolder;
        /// <summary>Child form container</summary>
        private Control                                     m_ctlChildContainer;
        /// <summary>Object implementing the selection buffer</summary>
        private RepPP.Application                           m_app;
        /// <summary>Text translator use by the base framework</summary>
        private ControlTextTranslator                       m_ctlTxtTranslatorBase;
        /// <summary>Text translator use by this specified editor</summary>
        private ControlTextTranslator                       m_ctlTxtTranslatorTop;
        /// <summary>Name of the atom used by the top translator</summary>
        private string                                      m_strTopTranslatorAtomName;
        /// <summary>Name of the atom used by the base translator</summary>
        private string                                      m_strBaseTranslatorAtomName;
        /// <summary>Object serializing the user interface settings</summary>
        private UIConfigurationHelper                       m_configHelper;
        /// <summary>Flag indicating if the fw saves the UI configuration automatically</summary>
        private bool                                        m_bAutoSaveConfig;
        

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmEditor() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator</param>
        //
        //******************************************************************
        protected virtual void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            m_app                       = app;
            m_strTopTranslatorAtomName  = strTopTranslatorAtomName;
            m_strBaseTranslatorAtomName = strBaseTranslatorAtomName;
            m_ctlChildContainer         = null;
            m_cmdHolder                 = new CmdHolder();
            m_ctlTxtTranslatorBase      = null;
            m_ctlTxtTranslatorTop       = null;
            m_configHelper              = configHelper;
            m_bAutoSaveConfig           = true;
            if (configHelper != null) {
                configHelper.RegisterForm(this);
            }
            if (app != null) {
                CreateControlTextTranslator();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the object
        /// </summary>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing)	{
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// Application:                        Rep++ Application object
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RepPP.Application Application {
            get {
                return(m_app);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CommandHolder:                    
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual CmdHolder CommandHolder {
            get {
                return(m_cmdHolder);
            }
            set {
                m_cmdHolder = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ChildContainer:                     Control containing the child editor
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Control ChildContainer {
            get {
                return(m_ctlChildContainer);
            }
            set {
                m_ctlChildContainer = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ConfigurationHelper:                Object responsible to keep user settings of the user interface
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual UIConfigurationHelper ConfigurationHelper {
            get {
                return(m_configHelper);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AutoSaveConfiguration:          Gets\Sets a flag indicating if the 
        ///                                 framework saves the UI configuration 
        ///                                 automatically
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool AutoSaveConfiguration {
            get {
                return (m_bAutoSaveConfig);
            }
            set {
                m_bAutoSaveConfig = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateControlTextTranslator:            Creates a control text translator
        /// </summary>
        /// <param name="ctlTxtTranslatorFather">   Parent Translator</param>
        /// <param name="strAtomName">              Atom name of the translator</param>
        /// <returns>
        /// a reference to the created ControlTextTranslator
        /// </returns>
        //
        //******************************************************************
        protected virtual ControlTextTranslator CreateControlTextTranslator(ControlTextTranslator ctlTxtTranslatorFather, string strAtomName) {
            return(new ControlTextTranslator(Application, new string[] {strAtomName}, ctlTxtTranslatorFather));
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateControlTextTranslator:        Creates a control text translator
        /// </summary>
        //
        //******************************************************************
        protected void CreateControlTextTranslator() {
            if (m_ctlTxtTranslatorBase == null) {
                m_ctlTxtTranslatorBase  = CreateControlTextTranslator((ControlTextTranslator)null, m_strBaseTranslatorAtomName);
            }
            if (m_strTopTranslatorAtomName != null) {
                m_ctlTxtTranslatorTop = CreateControlTextTranslator(m_ctlTxtTranslatorBase, m_strTopTranslatorAtomName);
            } else {
                if (m_ctlTxtTranslatorTop == null) {
                    m_ctlTxtTranslatorTop = m_ctlTxtTranslatorBase;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// TranslateControlText:           Translate the controls text using the specified atom
        /// </summary>
        //
        //******************************************************************
        protected virtual void TranslateControlText() {
            m_ctlTxtTranslatorTop.TranslateControl(this);
        }

        //******************************************************************
        //
        /// <summary>
        /// ControlTextTranslator:          Returns the control text translator
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public ControlTextTranslator ControlTextTranslator {
            get {
                return(m_ctlTxtTranslatorTop);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLoad:                             Called when the form is loaded
        /// </summary>
        /// <param name="e">                    Event argument</param>
        //  
        //*********************************************************     
        protected override void OnLoad(EventArgs e) {
            base.OnLoad (e);
            if (ControlTextTranslator != null) {
                ControlTextTranslator.TranslateControl(this);
            }
        }

		//******************************************************************
		//
		/// <summary>
		/// OnClosed:           Handle the form close event
		/// </summary>
		/// <param name="e">    Event arguments</param>
		// 
		//******************************************************************
		protected override void OnClosed(EventArgs e) {
			base.OnClosed(e);
            if (ConfigurationHelper != null && AutoSaveConfiguration) {
                ConfigurationHelper.SaveToFile();
			}
		}
        
        #region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "frmEditor";
		}
		#endregion
	}
}
