using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Base implementation of the interface for a non dockable selection buffer editor. 
    /// </summary>
    //
    //********************************************************************************    
	public class frmSBEditorBase : frmEditor, ISelectionBuffer, ISupportDirectSearch {    //TODO: Must we implement ISelectionBuffer??? I don't think these functions are connected yet
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container             components = null;
	    /// <summary>Current child editor</summary>
	    private frmSBChildEditor                            m_childEditorCurrent;
	    /// <summary>Child editor factory</summary>
	    private ISBChildEditorFactory                       m_childEditorFactory;
        /// <summary>Object implementing the selection buffer</summary>
        private ISelectionBuffer                            m_selBuf;
        /// <summary>Flag to ignore additional selections while there is a selection in progress</summary>
        protected bool                                      m_bSelecting;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorBase() {
            m_childEditorFactory    = null;
            m_selBuf                = null;
            m_childEditorCurrent    = null;
            m_bSelecting            = false;
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the object
        /// </summary>
        //
        //******************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
                CloseChildEditor();
                m_selBuf = null;
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app,
                            configHelper,
                            strTopTranslatorAtomName,
                            (strBaseTranslatorAtomName != null) ? strBaseTranslatorAtomName : "txtfrmTopBaseEditor");
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">          Name of the rowset tree representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected virtual void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            ChildEditorFactory = childEditorFactory;
        }

        //******************************************************************
        //
        /// <summary>
        /// OnLoad:                     Called when the form is loaded
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //******************************************************************
        protected override void OnLoad(EventArgs e) {
            Fill(true);
            base.OnLoad(e);
        }

        //******************************************************************
        //
        /// <summary>
        /// ChildEditorFactory:                 Factory class use to create the child editors
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ISBChildEditorFactory ChildEditorFactory {
            get {
                return(m_childEditorFactory);
            }
            set {
                m_childEditorFactory = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectionBuffer:                    Selection buffer
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ISelectionBuffer SelectionBuffer {
            get {
                return(m_selBuf);
            }
            set {
                m_selBuf = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentChildEditor:         Current child editor
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual frmSBChildEditor CurrentChildEditor {
            get {
                return(m_childEditorCurrent);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentChildEditorType:                  Current editor type or -1 if none selected
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public int CurrentChildEditorType {
            get {
                return((m_childEditorCurrent != null) ? m_childEditorCurrent.EditorType : -1);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:       Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">          Framework rowset tree node manager</param>
        //
        //******************************************************************
        public virtual void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateSBContextMenu:            Creates a context menu
        /// </summary>
        //
        //******************************************************************
        public virtual ContextMenuStrip CreateSBContextMenu() {

            return (null);
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateEditor:               Creates an editor of the specified type
        /// </summary>
        //
        //******************************************************************
        private frmSBChildEditor CreateEditor(int iEditorType) {
            frmSBChildEditor    frmRetVal;
            
            frmRetVal = m_childEditorFactory.CreateChildEditor(iEditorType);
            if (frmRetVal == null) {
                throw new System.ApplicationException("Unable to create a child editor of type " + iEditorType + "\n" +
                                                      "Look in the factory class " + m_childEditorFactory.GetType().ToString());
            }
            return(frmRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// CloseChildEditor:           Close the current child editor
        /// </summary>
        //
        //******************************************************************
        public virtual void CloseChildEditor() {
            if (m_childEditorCurrent != null) {
                m_childEditorCurrent.DetachChildEditor();
                m_childEditorCurrent.Dispose();
                m_childEditorCurrent = null;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AttachChildEditor:          Attach the child editor to the top editor
        /// </summary>
        //
        //******************************************************************
        private void AttachChildEditor() {
            frmSBChildEditor    currentEditor;
            
            currentEditor = CurrentChildEditor;
            if (currentEditor != null) {
                currentEditor.AttachChildEditor(ChildContainer);
                RegisterTriggerToChildEditor(currentEditor.Handler as RowsetTreeDefNodeHandlerRootBase);
                currentEditor.InitChildEditor();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectEditorType:           Select the specific editor
        /// </summary>
        //
        //******************************************************************
        private frmSBChildEditor SelectEditorType(int iEditorType) {
            frmSBChildEditor    frmRetVal;
            
            frmRetVal = CurrentChildEditor;
            if (frmRetVal == null || frmRetVal.EditorType != iEditorType) {
                CloseChildEditor();
                frmRetVal               = CreateEditor(iEditorType);
                m_childEditorCurrent    = frmRetVal;
                AttachChildEditor();
            }
            return(frmRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// AskToSave:                          Ask to save if data is dirty
        /// </summary>
        /// <param name="bEnsureFatherExist">   true to ensure father exist</param>
        /// <returns>
        /// true = Ok to continue, false = error or user refused
        /// </returns>
        //
        //******************************************************************
        public virtual bool AskToSave(bool bEnsureFatherExist) {
            frmSBChildEditor    childEditor;
            
            childEditor = CurrentChildEditor;
            return((childEditor == null) ? true : childEditor.AskToSave(bEnsureFatherExist));
        }

        //*********************************************************
        //
        /// <summary>
        /// ChildAdjustState:                   Adjust the state of the child editor
        /// </summary>
        //  
        //*********************************************************
        public virtual void ChildAdjustState() {
            frmSBChildEditor    frmChildEditor;

            frmChildEditor = CurrentChildEditor;
            if (frmChildEditor != null) {
                frmChildEditor.RaiseAdjustState();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ChildRefresh:                       Refresh the child editor
        /// </summary>
        //  
        //*********************************************************
        public virtual void ChildRefresh() {
            frmSBChildEditor    frmChildEditor;

            frmChildEditor = CurrentChildEditor;
            if (frmChildEditor != null) {
                frmChildEditor.Refresh();

            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectCurrentNodeItem:      Select the item of the current node.
        /// </summary>
        //
        //******************************************************************
        public virtual void SelectCurrentNodeItem() {
            frmSBChildEditor    childEditor;
            string[]            arrPK;
            int                 iEditorType;
            
            arrPK       = CurrentItemPK;
            iEditorType = CurrentItemType;
            childEditor = SelectEditorType(iEditorType);
            childEditor.SelectItem(arrPK);
        }

        //******************************************************************
        //
        /// <summary>
        /// InsertNewLine:                  Insert a new line
        /// </summary>
        /// <param name="iEditorType">      Type of the editor</param>
        /// <param name="bChild">           true if the new editor is a child of the current one</param>
        /// <param name="bAtEnd">           true to insert at the end</param>
        /// <param name="strNodeName">      Rowset tree definition node name of the child selection buffer</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //******************************************************************
        public virtual bool InsertNewLine(int iEditorType, bool bChild, bool bAtEnd, string strNodeName) {
            bool                bRetVal;
            frmSBChildEditor    childEditor;
            frmSBChildEditor    childEditorCurrent;
            FieldDef[]          arrFldDef;
            string[]            arrValue;
            Rowset              rowsetSB;
            
            bRetVal = AskToSave(bChild);
            if (bRetVal) {
                childEditorCurrent = CurrentChildEditor;
                if (childEditorCurrent != null && childEditorCurrent.EditorType == iEditorType) {
                    bRetVal = childEditorCurrent.InsertNewLine(bAtEnd);
                } else {
                    childEditor = CreateEditor(iEditorType);
                    bRetVal = childEditor.AskForPrimaryKey(out arrFldDef, out arrValue);
                    if (bRetVal) {
                        if (childEditorCurrent != null) {
                            childEditorCurrent.DetachChildEditor();
                            childEditorCurrent.Dispose();
                        }
                        m_childEditorCurrent = childEditor;
                        AttachChildEditor();
                        if (bChild && strNodeName != null && strNodeName.Length > 0) {
                            rowsetSB = SelectionBuffer.CurrentRowset.ChildRowsets[strNodeName];
                            bRetVal = childEditor.InsertNewLine(rowsetSB, bAtEnd, arrFldDef, arrValue);
                        } else {
                            bRetVal = childEditor.InsertNewLine(bAtEnd, arrFldDef, arrValue);
                        }
                    } else {
                        childEditor.Dispose();
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// InsertNewLine:              Insert a new line
        /// </summary>
        /// <param name="bAtEnd">       true to insert at the end</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //******************************************************************
        public virtual bool InsertNewLine(bool bAtEnd) {
            int     iEditorType;
            
            iEditorType = CurrentItemType;
            return(InsertNewLine(iEditorType, false, bAtEnd, ""));
        }

        //*********************************************************
        //
        /// <summary>
        /// CallDirectSearchDialog:       Popup the direct search dialog
        /// </summary>
        /// <param name="node">           Current search RowsetTreeDefNode</param>
        /// <param name="fldDef">         Current search field</param>
        /// <param name="strSearchVal">   Current search value</param>
        /// <param name="trans">          Control text translator</param>
        /// <param name="configHelper">   Configuration helper</param>
        /// <param name="strJoinClause">  Inner Join Clause</param>
        /// <param name="strWhereClause"> Where Clause</param>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //  
        //*********************************************************
        public virtual bool CallDirectSearchDialog(RowsetTreeDefNode        node, 
                                                   FieldDef                 fldDef, 
                                                   string                   strSearchVal, 
                                                   ControlTextTranslator    trans, 
                                                   UIConfigurationHelper    configHelper,
                                                   out string               strJoinClause,
                                                   out string               strWhereClause) {
            return frmSBDirectSearch.Show(node, fldDef, strSearchVal, trans, out strJoinClause, out strWhereClause);
        }

        //*********************************************************
        //
        /// <summary>
        /// DirectSearch:               Set filter for the selection buffer
        ///                             using a direct search
        /// </summary>
        /// <param name="node">         Current search RowsetTreeDefNode</param>
        /// <param name="fldDef">       Current search Field definition</param>
        /// <param name="strSearchVal"> Current search value</param>
        /// <param name="trans">        Control text translator</param>
        /// <param name="configHelper"> Configuration helper</param>
        //  
        //*********************************************************
        protected virtual void DirectSearch(RowsetTreeDefNode node, FieldDef fldDef, string strSearchVal, ControlTextTranslator trans, UIConfigurationHelper configHelper) {
            DirectSearchMethod  dsMethod;
            string              strJoinClause;
            string              strWhereClause;

            if (CurrentChildEditor != null) {
                if (AskToSave(false)) {
                    if (CallDirectSearchDialog(node, fldDef, strSearchVal, trans, configHelper, out strJoinClause, out strWhereClause)) {
                        dsMethod              = (DirectSearchMethod)SearchMethods[0];
                        dsMethod.WhereClause  = strWhereClause;
                        dsMethod.JoinClause   = strJoinClause;
                        NextSearchMethodIndex = 0;
                        Refresh(CurrentFilter, true);
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DirectSearch:               Set filter for the selection buffer
        ///                             using a direct search
        /// </summary>
        //  
        //*********************************************************
        public void DirectSearch() {
            FieldDef fldDef;
            FieldDef fldDefAlt;

            fldDef    = CurrentChildEditor.CurrentFieldDef;
            fldDefAlt = SBFilterBase.GetAlternateFilterField(fldDef);
            if (fldDefAlt != null) {
                fldDef = fldDefAlt;
            }
            ExecuteDirectSearch(fldDef, fldDef.ExtValue);
        }

        //******************************************************************
        //
        /// <summary>
        /// EnsureThatRequiredModuleIsIncluded  Ensures that the required modules are included
        /// </summary>
        /// <returns>
        /// True if successfull, False otherwise.
        /// </returns>
        //
        //******************************************************************
        private bool EnsureThatRequiredModuleIsIncluded(){
            bool    bRetVal;
            string  strMsg;
            
            bRetVal = Application.IsAtomExist("IsMacroNameUnique", AtomType.sdAtomSql);
            if (!bRetVal) {
                if(Application.LanguageCode == LanguageCode.sdFrench) {
                    strMsg = "Le module public FWDETHICKV1 n'est pas inclus dans votre programme. Veuillez ajouter ce module  votre programme.";
                } else {
                    strMsg = "The public module FWDETHICKV1 is not included in the program. Please add this module to your program.";
                }
                MessageBox.Show(strMsg, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return(bRetVal);
        }

#if !(SD60 || SD70)

        //******************************************************************
        //
        /// <summary>
        /// IsMacroManipulationAllowed:     Indicates if the manipulation of 
        ///                                 macros at runtime is allowed
        /// </summary>
        /// <returns>
        /// True to indicate that the users can create delete macros
        /// </returns>
        //
        //******************************************************************
        protected virtual bool IsMacroManipulationAllowed() {
            return(true);
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateMacro:                Creates a selection macro
        /// </summary>
        //  
        //*********************************************************
        public virtual void CreateMacro() {
            frmEditMacro frm;
            
            if(EnsureThatRequiredModuleIsIncluded()) {
                using (frm = new frmEditMacro(Application, this.RowsetTree.RowsetTreeDef.Name, ControlTextTranslator)) {
                    if (frm.ShowDialog() == DialogResult.OK) {
                        ReloadSearchMethods();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EditMacro:                  Edits a selection macro
        /// </summary>
        //  
        //*********************************************************
        public virtual void EditMacro() {
            frmSelectMacro frm;
            
            if(EnsureThatRequiredModuleIsIncluded()) {
                using (frm = new frmSelectMacro(this, ControlTextTranslator, false)) {
                    if (frm.ShowDialog() == DialogResult.OK) {
                        ReloadSearchMethods();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DeleteMacro:                Deletes a selection macro
        /// </summary>
        //  
        //*********************************************************
        public virtual void DeleteMacro() {
            frmSelectMacro frm;
            
            if(EnsureThatRequiredModuleIsIncluded()) {
                using (frm = new frmSelectMacro(this, ControlTextTranslator, true)) {
                    if (frm.ShowDialog() == DialogResult.OK) {
                        ReloadSearchMethods();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnExecuteSearch:             executes a search using a macro
        /// </summary>
        //  
        //*********************************************************
        protected virtual void OnExecuteSearch(object sender, EventArgs e) {
            ISearchMethod   searchMethod;
            int             iNextSearchMethodIndex;
            bool            bContinue = true;

            if (AskToSave(false)) {
                iNextSearchMethodIndex = ((int)((ToolStripMenuItem)sender).Tag);
                searchMethod           = SearchMethods[iNextSearchMethodIndex];
                if (searchMethod.HasArguments) {
                    bContinue = GetSearchMethodArgsValues(searchMethod);
                }
                if (bContinue) {
                    NextSearchMethodIndex = iNextSearchMethodIndex;
                    Refresh(true);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetSearchMethodArgsValues:  Gets the values of the search
        ///                             method arguments
        /// </summary>
        /// <param name="searchMethod"> Search methos</param>
        /// <returns>
        /// False if the user cancels, True otherwise.
        /// </returns>
        //  
        //*********************************************************
        protected virtual bool GetSearchMethodArgsValues(ISearchMethod searchMethod) {
            return frmSearchMacro.Show(searchMethod, ControlTextTranslator);
        }
#endif

        //*********************************************************
        //
        /// <summary>
        /// OnBeforeSearchMethodExecute:    Called before a search method
        ///                                 executes
        /// </summary>
        /// <param name="method">           The search method</param>
        /// <param name="e">                Event argument</param>
        //  
        //*********************************************************
        protected virtual void OnBeforeSearchMethodExecute(ISearchMethod method, BeforeExecuteEventArgs e) {
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAfterSearchMethodValidate:    Called before a search method
        ///                                 executes
        /// </summary>
        /// <param name="method">           The search method</param>
        /// <param name="e">                Event argument</param>
        //  
        //*********************************************************
        protected virtual void OnAfterSearchMethodValidate(ISearchMethod method, CancelEventArgs e) {
        }

        //*********************************************************
        //
        /// <summary>
        /// OnBeforeSearchMethodExecute:    Called before a search method
        ///                                 executes
        /// </summary>
        /// <param name="sender">           The search method</param>
        /// <param name="e">                Event argument</param>
        //  
        //*********************************************************
        internal void OnBeforeSearchMethodExecute(object sender, BeforeExecuteEventArgs e) {
            OnBeforeSearchMethodExecute(sender as ISearchMethod, e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAfterSearchMethodValidate:    Called before a search method
        ///                                 executes
        /// </summary>
        /// <param name="sender">           The search method</param>
        /// <param name="e">                Event argument</param>
        //  
        //*********************************************************
        internal void OnAfterSearchMethodValidate(object sender, CancelEventArgs e) {
            OnAfterSearchMethodValidate(sender as ISearchMethod, e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsDirectSearchField:            Checks if a direct search can be performed on
        ///                                 the specified field
        /// </summary>
        /// <param name="fldDef">           Field definition to be checked</param>
        /// <returns>
        /// True if a direct search can be performed on the field, false otherwise
        /// </returns>
        /// <remarks>
        /// ISupportDirectSearch Member
        /// </remarks>
        //
        //********************************************************************************
        public virtual bool IsDirectSearchField(FieldDef fldDef) {
            return RepPP.Toolkit.Util.IsDirectSearchField(fldDef);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteDirectSearch             Executes a direct search using the specified 
        ///                                 field and value
        /// </summary>
        /// <param name="fldDef">           Direct search field</param>
        /// <param name="strSearchVal">     Direct search value</param>
        /// <remarks>
        /// ISupportDirectSearch Member
        /// </remarks>
        //
        //********************************************************************************
        public virtual void ExecuteDirectSearch(FieldDef fldDef, string strSearchVal) {
            RowsetTreeDefNode   node;

            if (IsDirectSearchField(fldDef)) {
                node = CurrentChildEditor.RowsetTreeDef.FindNode(fldDef.RowsetDef.Name);
                DirectSearch(node, fldDef, strSearchVal, ControlTextTranslator, ConfigurationHelper);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CallFilterDialog:           Popup the filter dialog
        /// </summary>
        /// <param name="filter">       Filter to adapt</param>
        /// <param name="trans">        Control text translator</param>
        /// <param name="configHelper"> Configuration helper</param>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //  
        //*********************************************************
        public virtual bool CallFilterDialog(ISBFilter filter, ControlTextTranslator trans, UIConfigurationHelper configHelper) {
            bool        bRetVal = false;
            frmSBFilter frmFilter;

            using(frmFilter = new frmSBFilter(filter as SBFilter, trans, null, false)) {
                if (frmFilter.ShowDialog() == DialogResult.OK) {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFilter:                  Set filter for the selection buffer
        /// </summary>
        /// <param name="filter">       Filter to adapt</param>
        /// <param name="trans">        Control text translator</param>
        /// <param name="configHelper"> Configuration helper</param>
        //  
        //*********************************************************
        public virtual void SetFilter(ISBFilter filter, ControlTextTranslator trans, UIConfigurationHelper configHelper) {
            if (filter != null            &&
                CurrentChildEditor != null) {
                if (AskToSave(false)) {
                    if (CallFilterDialog(filter, trans, configHelper)) {
                        Refresh(filter, true);
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFilter:                  Set filter for the selection buffer
        /// </summary>
        //  
        //*********************************************************
        public virtual void SetFilter() {
            SetFilter(CurrentFilter, ControlTextTranslator, ConfigurationHelper);
        }

        //*********************************************************
        //
        /// <summary>
        /// ResetFilter:                Reset the current filter for the selection buffer
        /// </summary>
        /// <param name="filter">       Filter to adapt</param>
        //  
        //*********************************************************
        public virtual void ResetFilter(ISBFilter filter) {
            if ((filter != null && CurrentChildEditor != null && filter.WhereClause != String.Empty) || 
                (SelectionBuffer.ActiveSearchMethodIndex != -1)) {
                if (AskToSave(false)) {
                    if (SelectionBuffer.ActiveSearchMethodIndex != -1) {
                        SelectionBuffer.ActiveSearchMethodIndex = -1;
                    } else {
                        filter.WhereClause  = String.Empty;
                        filter.FilterClause = String.Empty;
                    }
                    Refresh(filter, true);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ResetFilter:                Reset the current filter for the selection buffer
        /// </summary>
        //  
        //*********************************************************
        public virtual void ResetFilter() {
            ResetFilter(CurrentFilter);
        }

        //*********************************************************
        //
        /// <summary>
        /// Refresh:                        Refresh the selection buffer editor
        /// </summary>
        /// <param name="filter">           Filter to use for refreshing the selection buffer</param>
        /// <param name="bForceToClose">    True to close the current item, and false to ask to save</param>
        //  
        //*********************************************************
        public virtual void Refresh(ISBFilter filter, bool bForceToClose) {
            RepPP.Rowset rowset;
            string[]            arrPKValue;
            string[]            arrPKName;
            object[]            arrSearch;
            int                 iIndex;
            bool                bContinue = true;

            if (filter != null) {
                //JH: It is ok not to close because the IsDirty has already been called
                //    when a search method menu is selected
                if (NextSearchMethodIndex == -1) {
                    if (bForceToClose) {
                        CloseChildEditor();
                    } else {
                        bContinue = AskToSave(true);
                        if (bContinue && CurrentChildEditor != null && CurrentChildEditor.IsDirty()) {
                            CloseChildEditor();
                        }
                    }
                }
                if (bContinue) {
                    arrPKValue = CurrentItemPK;
                    Fill(false);
                    rowset = CurrentRowset;
                    if (rowset != null && rowset.LineCount > 0) {
                        if (arrPKValue.Length > 0) {
                            arrPKName   = RepPP.Toolkit.Util.BuildPrimaryKeyFieldNames(rowset.RowsetDef);
                            arrSearch   = new object[arrPKName.Length * 2];
                            for (iIndex = 0; iIndex < arrPKName.Length; iIndex++) {
                                arrSearch[iIndex * 2]     = arrPKName[iIndex];
                                arrSearch[iIndex * 2 + 1] = arrPKValue[iIndex];
                            }
                            if (rowset.FindValue(false, arrSearch) == 0) {
                                SelectItem(rowset.CurrentLine);
                            }
                        }
                    }
                    SelectCurrentNodeItem();
                    ChildRefresh();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Refresh:                        Refresh the selection buffer editor
        /// </summary>
        /// <param name="bForceToClose">    True to close the current item, and false to ask to save</param>
        //  
        //*********************************************************
        public void Refresh(bool bForceToClose) {
            Refresh(CurrentFilter, bForceToClose);
        }       

        //*********************************************************
        //
        /// <summary>
        /// BeforeItemSelected:         Call before an item is selected
        /// </summary>
        //  
        //*********************************************************
        public virtual bool BeforeItemSelected() {
            return(!m_bSelecting && AskToSave(false));
        }

        //*********************************************************
        //
        /// <summary>
        /// AfterItemSelected:         Call after an item is selected
        /// </summary>
        //  
        //*********************************************************
        public virtual void AfterItemSelected() {
            try {
                m_bSelecting = true;
                SelectCurrentNodeItem();
            } finally {
                m_bSelecting = false;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:       Add an item to the selection buffer
        /// </summary>
        /// <param name="rowsetSB">         Rowset of the selection buffer</param>
        /// <param name="bDuplicate">       true if duplicate the item</param>
        //
        //******************************************************************
        public virtual void AddItemToSelectionBuffer(Rowset rowsetSB, bool bDuplicate) {
            SelectionBuffer.AddItemToSelectionBuffer(rowsetSB, bDuplicate);
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh the item in the selection buffer
        /// </summary>
        /// <param name="rowset">           Rowset where to take the updated value</param>
        //
        //******************************************************************
        public virtual void RefreshItemInSelectionBuffer(Rowset rowset) {
            SelectionBuffer.RefreshItemInSelectionBuffer(rowset);
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete the item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to reselect a new item after deletion</param>
        //
        //******************************************************************
        public virtual void DeleteItemFromSelectionBuffer(bool bReselect) {
            SelectionBuffer.DeleteItemFromSelectionBuffer(bReselect);
        }

        //******************************************************************
        //
        /// <summary>
        /// ItemCount:                      Get the number of items if the current kind
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int ItemCount {
            get {
                return(SelectionBuffer.ItemCount);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemIndex:                   Get the current item index for the current kind
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int CurrentItemIndex {
            get {
                return(SelectionBuffer.CurrentItemIndex);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select an item in the selection buffer
        /// </summary>
        /// <param name="iPos">             Position of the item to select (0..n)</param>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //
        //******************************************************************
        public virtual bool SelectItem(int iPos) {
            return(SelectionBuffer.SelectItem(iPos));
        }

        //******************************************************************
        //
        /// <summary>
        /// Fill:                           Fill the selection buffer
        /// </summary>
        /// <param name="bAddNewIfEmpty">   Indicates if a new item should be added if the selection buffer will be empty </param>
        //
        //******************************************************************
        public virtual void Fill(bool bAddNewIfEmpty) {
            if (SelectionBuffer != null) {
                SelectionBuffer.Fill(bAddNewIfEmpty);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentRowset:           Returns the selection buffer rowset associated with the current item
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Rowset CurrentRowset { 
            get {
                return(SelectionBuffer.CurrentRowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RowsetTree:           Returns the selection buffer rowset tree
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTree RowsetTree { 
            get {
                return(SelectionBuffer.RowsetTree);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemPK:                  Gets the currently selected item primary key
        /// </summary>
        /// <returns>
        /// PK values
        /// </returns>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string[] CurrentItemPK {
            get {
                return(SelectionBuffer.CurrentItemPK);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemObject:              Gets the object currently selected in the selection buffer (node, listview item...)
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual object CurrentItemObject {
            get {
                return(SelectionBuffer.CurrentItemObject);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentItemType:                Gets the editor type of the currently selected item
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int CurrentItemType {
            get {
                return(SelectionBuffer.CurrentItemType);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentFilter:                  Gets the current filter
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ISBFilter CurrentFilter {
            get {
                return(SelectionBuffer.CurrentFilter);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SearchMethods:                  Gets the current search methods
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ISearchMethod[] SearchMethods {
            get {
                return(SelectionBuffer.SearchMethods);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// NextSearchMethodIndex:          Gets the index of the search method
        ///                                 to execute
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int NextSearchMethodIndex {
            get {
                return(SelectionBuffer.NextSearchMethodIndex);
            }
            set {
                SelectionBuffer.NextSearchMethodIndex = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ActiveSearchMethodIndex:        Gets the index of the search method
        ///                                 that was used to load the SB
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int ActiveSearchMethodIndex {
            get {
                return(SelectionBuffer.ActiveSearchMethodIndex);
            }
            set {
                SelectionBuffer.ActiveSearchMethodIndex = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReloadSearchMethods:             Reloads the list of search methods
        /// </summary>
        //
        //******************************************************************
        public virtual void ReloadSearchMethods() {
            SelectionBuffer.ReloadSearchMethods();
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "frmSBEditorBase";
		}
		#endregion

    }
}