using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Implements the interface for a Rep++ Macro that has one or more substitution
    /// parameters.
    /// </summary>
    //
    //********************************************************************************    
    public class frmSearchMacro : Form {
        /// <summary>Cancel button</summary>
        protected Button            butCancel;
        /// <summary>OK Button</summary>
        protected Button            butOk;
        /// <summary>Panel holding the Macro's Parameters</summary>
        protected Panel             pnlMacroParams;
        /// <summary>Description label</summary>
        protected Label             lblDescription;
        /// <summary>Description GroupBox</summary>
        protected GroupBox          grpDescription;
        /// <summary>Parameter Values GroupBox</summary>
        protected GroupBox          grpValues;
        /// <summary>Error provider for the form</summary>
        protected FldErrorProvider  m_errorProvider;
        /// <summary>Rep++ Macro object</summary>
        protected Macro             m_macro;
        /// <summary>Search Method</summary>
        protected SearchMacro       m_searchMacro;

        //******************************************************************
	    //
		/// <summary>
		/// ctor:                       Form default constructor. Used by designer
		/// </summary>
	    //
	    //******************************************************************
        public frmSearchMacro() {
            InitializeComponent();
        }

	    //******************************************************************
	    //
		/// <summary>
		/// ctor:                           Form constructor.
		/// </summary>
        /// <param name="searchMacro">      Search macro object</param>
        /// <param name="ctlTranslator">    Control text translator</param>
	    //
	    //******************************************************************
        public frmSearchMacro(SearchMacro searchMacro, ControlTextTranslator ctlTranslator): this() {
            m_errorProvider            = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
            m_errorProvider.BlinkStyle = ErrorBlinkStyle.NeverBlink;
            if (ctlTranslator != null) {
                ctlTranslator.TranslateControl(this);
            }
            m_searchMacro       = searchMacro;
            m_macro             = searchMacro.Macro;
            Text                = m_macro.Name;
            lblDescription.Text = m_macro.Description;
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Show:                           Displays the Macro form
		/// </summary>
        /// <param name="searchMethod">     Search Method object</param>
        /// <param name="ctlTranslator">    Control text translator</param>
        /// <returns>
        /// True if the form was displayed and the user hit OK, false otherwise.
        /// </returns>
	    //
	    //******************************************************************
        public static bool Show(ISearchMethod searchMethod, ControlTextTranslator ctlTranslator) {
            bool            bRetVal = false;
            frmSearchMacro  frm;

            using(frm = new frmSearchMacro(((SearchMacro)searchMethod), ctlTranslator)) {
                if (frm.ShowDialog() == DialogResult.OK) {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// ResizeForm:                 Resize the form to show as much field as possible
		/// </summary>
		//
	    //******************************************************************
        protected virtual void ResizeForm() {
            int iMaxX = 0;
            int iMaxY = 0;
            int iTmp;

            foreach(Control ctl in pnlMacroParams.Controls) {
                iTmp = ctl.Location.X + ctl.Size.Width;
                if (iTmp > iMaxX) {
                    iMaxX = iTmp;
                }
                iTmp = ctl.Location.Y + ctl.Size.Height;
                if (iTmp > iMaxY) {
                    iMaxY = iTmp;
                }
            }
            Width   -= pnlMacroParams.Width - iMaxX - 10;
            Height  -= pnlMacroParams.Height - iMaxY - 10;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLoad:                         Create dynamic form when form is loading
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            int                 iMaxXPos;
            ComboBox[]          arrComboBox;
            RepPP.RepPPStat[]   arrStat;
            RepPP.RepPPField[]  arrField;
            RepPP.RepPPField[]  arrBetweenField;

            if (m_macro.Arguments.Count <= 0) {
                throw new ApplicationException("The number of macro arguments must be greater than zero");
            }
            SetDefaultValues();
	        DynamicForm.CreateRowsetFields(pnlMacroParams,
	                                      m_macro.Arguments[0].FieldDef.RowsetDef,
	                                      Font,
	                                      40,
	                                      40,
	                                      0,
	                                      0,
	                                      DynamicForm.OptionE.OPT_NO_CHECK |
	                                      DynamicForm.OptionE.OPT_NO_RADIO,
	                                      out iMaxXPos,
	                                      out arrComboBox,
	                                      out arrStat,
	                                      out arrField,
	                                      out arrBetweenField);
            ResizeForm();
            base.OnLoad(e);

        }

        //********************************************************************************
        //
        /// <summary>
        /// SetDefaultValues:               Sets the defaut values of the macro parameters
        /// </summary>
        //
        //********************************************************************************
        private void SetDefaultValues() {
            FieldDef fldDef;

            foreach(MacroArg arg in m_macro.Arguments) {
                fldDef = arg.FieldDef;
                if (fldDef.Value == string.Empty && fldDef.DefaultValue != string.Empty) {
                    fldDef.SetValue(fldDef.DefaultValue);    
                }
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// OnOK:                       Called when the OK button is clicked
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
	    //******************************************************************
        private void OnOK(object sender, EventArgs e) {
            m_errorProvider.Reset();
            if (IsValid() && m_searchMacro.ValidateArguments()) {
                DialogResult = DialogResult.OK;
                Close();
            }
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// IsValid:            Checks if the value are valids
	    /// </summary>
	    //
	    //******************************************************************
		public bool IsValid() {
            MacroArgs   args;
		    bool        bRetVal = true;
            string      strErrorText;
		    		    
            args = m_macro.Arguments;
		    foreach (MacroArg arg in args) {
		        if (arg.FieldDef.Validate(out strErrorText) != 0) {
                    m_errorProvider.AddError(this, arg.FieldDef, strErrorText);
		            bRetVal = false;
		        }
		    }
		    return(bRetVal);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.butCancel = new System.Windows.Forms.Button();
            this.butOk = new System.Windows.Forms.Button();
            this.pnlMacroParams = new System.Windows.Forms.Panel();
            this.lblDescription = new System.Windows.Forms.Label();
            this.grpDescription = new System.Windows.Forms.GroupBox();
            this.grpValues = new System.Windows.Forms.GroupBox();
            this.grpDescription.SuspendLayout();
            this.grpValues.SuspendLayout();
            this.SuspendLayout();
            // 
            // butCancel
            // 
            this.butCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butCancel.Location = new System.Drawing.Point(99, 122);
            this.butCancel.Name = "butCancel";
            this.butCancel.Size = new System.Drawing.Size(80, 24);
            this.butCancel.TabIndex = 4;
            this.butCancel.Text = "Cancel";
            // 
            // butOk
            // 
            this.butOk.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butOk.Location = new System.Drawing.Point(13, 122);
            this.butOk.Name = "butOk";
            this.butOk.Size = new System.Drawing.Size(80, 24);
            this.butOk.TabIndex = 3;
            this.butOk.Text = "Ok";
            this.butOk.Click += new System.EventHandler(this.OnOK);
            // 
            // pnlMacroParams
            // 
            this.pnlMacroParams.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlMacroParams.Location = new System.Drawing.Point(6, 19);
            this.pnlMacroParams.Name = "pnlMacroParams";
            this.pnlMacroParams.Size = new System.Drawing.Size(155, 11);
            this.pnlMacroParams.TabIndex = 5;
            // 
            // lblDescription
            // 
            this.lblDescription.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.lblDescription.AutoEllipsis = true;
            this.lblDescription.Location = new System.Drawing.Point(6, 16);
            this.lblDescription.Name = "lblDescription";
            this.lblDescription.Size = new System.Drawing.Size(155, 35);
            this.lblDescription.TabIndex = 6;
            this.lblDescription.Text = "This is a description of the macro";
            // 
            // grpDescription
            // 
            this.grpDescription.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.grpDescription.Controls.Add(this.lblDescription);
            this.grpDescription.Location = new System.Drawing.Point(12, 12);
            this.grpDescription.Name = "grpDescription";
            this.grpDescription.Size = new System.Drawing.Size(167, 62);
            this.grpDescription.TabIndex = 7;
            this.grpDescription.TabStop = false;
            this.grpDescription.Text = "Description: ";
            // 
            // grpValues
            // 
            this.grpValues.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.grpValues.Controls.Add(this.pnlMacroParams);
            this.grpValues.Location = new System.Drawing.Point(12, 80);
            this.grpValues.Name = "grpValues";
            this.grpValues.Size = new System.Drawing.Size(167, 36);
            this.grpValues.TabIndex = 8;
            this.grpValues.TabStop = false;
            this.grpValues.Text = "Values: ";
            // 
            // frmMacro
            // 
            this.AcceptButton = this.butOk;
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.CancelButton = this.butCancel;
            this.ClientSize = new System.Drawing.Size(191, 158);
            this.ControlBox = false;
            this.Controls.Add(this.grpValues);
            this.Controls.Add(this.grpDescription);
            this.Controls.Add(this.butCancel);
            this.Controls.Add(this.butOk);
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "frmMacro";
            this.ShowIcon = false;
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
            this.Text = "TODO Put Name of Macro Here";
            this.grpDescription.ResumeLayout(false);
            this.grpValues.ResumeLayout(false);
            this.ResumeLayout(false);

        }

        #endregion

    } // Class
} // Namespace