using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.Threading;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Handle the Rep++ Application object lifetime.
	/// </summary>
	//
    //********************************************************************************
	public class ApplicationHolder : IDisposable {
	    private const string                            m_strEventViewerErrorMsg =
	                                                    "Information has been logged in the event viewer. Please check the event viewer.";
        /// <summary>Event sent before initializing rep++</summary>
        public event EventHandler	                    BeforeInitializeRepPP;
        /// <summary>Event sent while Initializing rep++</summary>
        public event ProgInitEventHandler	            ProgInit;
        /// <summary>Event sent after Initializing rep++</summary>
        public event EventHandler	                    AfterInitializeRepPP;
        /// <summary>Holder for a Rep++ Application object</summary>
        private Application                             m_app;
        /// <summary>Father page</summary>
        private PagePreInit                             m_pageFather;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    /// <param name="pageFather">   Father page</param>
	    //
        //********************************************************************************
        public ApplicationHolder(PagePreInit pageFather) {
            m_app                   = null;
            m_pageFather            = pageFather;
            m_pageFather.PreInit   += new EventHandler(m_pageFather_PreInit);
            m_pageFather.Unload    += new EventHandler(m_pageFather_Unload);
            m_pageFather.Error     += new EventHandler(m_pageFather_Error);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor. Use to manually initialize Rep++.
	    /// </summary>
	    //
        //********************************************************************************
        public ApplicationHolder() : this(null) {
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Dispose:                    Dispose
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void Dispose() {
            if (m_app != null) {
                m_app.Release();
                m_app = null;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetSessionLanguage:         Language used to initialize Rep++ and switch the
	    ///                             languages. May be overriden by the ForcedLanguage
	    /// </summary>
	    /// <param name="page">         Page doing the request</param>
	    /// <returns>
	    /// Language code
	    /// </returns>
	    //
        //********************************************************************************
        public static LanguageCode GetSessionLanguage(Page page) {
            LanguageCode    eRetVal;
            string          str = null;
            
            if (!page.IsPostBack) {
                str = page.Request.Params["_LANG"];
            }
            if (str != null) {
                page.Session["LANGUAGE"] = str;
            } else {
                str = page.Session["LANGUAGE"] as string;
            }
            if (str == null || str == String.Empty) {
                eRetVal = LanguageCode.sdDefault;
            } else {
                eRetVal = (LanguageCode)Int32.Parse(str);
            }
            return(eRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetSessionLanguage:         Language used to initialize Rep++ and switch the
	    ///                             languages. May be overriden by the ForcedLanguage
	    /// </summary>
	    /// <param name="page">         Page doing the request</param>
	    /// <param name="eLang">        Language to set</param>
	    //
        //********************************************************************************
        public static void SetSessionLanguage(Page page, LanguageCode eLang) {
            page.Session["LANGUAGE"] = ((int)eLang).ToString();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SessionLanguage:            Language used to initialize Rep++ and switch the
	    ///                             languages. May be overriden by the ForcedLanguage
	    /// </summary>
	    //
        //********************************************************************************
        public virtual LanguageCode SessionLanguage {
            get {
                return(GetSessionLanguage(m_pageFather));
            }
            set {
                SetSessionLanguage(m_pageFather, value);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitTypedInstanceFactory:   Initialize the typed instance factory. Create the 
	    ///                             typed instance factory object and register all the
	    ///                             typed instances of the entry assembly.
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void InitTypedInstanceFactory() {
            m_app.TypedInstance.RegisterAll(true);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitThreadCulture:          Initialize the culture of the thread according to 
	    ///                             the current language of Rep++.
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void InitThreadCulture() {
            CultureInfo     culture;
            string          strCultureCode;

            switch(m_app.LanguageCode) {
            case LanguageCode.sdFrench:     
                strCultureCode = "fr-CA";
                break;
            case LanguageCode.sdGerman:     
                strCultureCode = "de-DE";
                break;
            case LanguageCode.sdItalian:    
                strCultureCode = "it-IT";
                break;
            case LanguageCode.sdSpanish:    
                strCultureCode = "es-ES";
                break;
            default:                        
                strCultureCode = "en-CA";
                break;
            }
            culture                                = CultureInfo.CreateSpecificCulture(strCultureCode);
            Thread.CurrentThread.CurrentCulture    = culture;
            Thread.CurrentThread.CurrentUICulture  = culture;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// CreateApplication:          Creates a Rep++ application
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void CreateApplication() {
            LanguageCode            eLang;
            RepPP.Application       app;
            ProgInitEventHandler    progInitHandler;
 
            app = RepPP.Application.GetApplication();
            if (app != null) {
#if DEBUG
				throw new FWException(FWException.EditorExceptionE.RepPPAlreadyInitialized);
#else
                EventLog eventLog;
                try {
                    if (!EventLog.SourceExists("Rep++ .Net")) {
                        EventLog.CreateEventSource("Rep++ .Net", "Application");
                    }
                    using (eventLog = new EventLog()) {
                        eventLog.Source = "Rep++ .Net";
                        eventLog.WriteEntry("A Rep++ application object is already initialized for this thread. It means an previous application on this process didn't dispose it correctly. It can also mean someone initialize the Rep++ object before the page.", EventLogEntryType.Error);
                    }
                } catch {
                }
                app.Dispose();
                if (m_app != null) {
                    m_app = null;
                }
#endif
            }
            try {
                OnBeforeInitializeRepPP(System.EventArgs.Empty);
                eLang           = SessionLanguage;
                progInitHandler = new ProgInitEventHandler(OnProgInit);
                m_app           = CreateApplication(eLang, null, progInitHandler);
                if (m_app == null) {
    	            m_app = RepPP.Application.CreateFromRes(eLang, null, progInitHandler);
                }
                if (m_app != null) {
                    InitTypedInstanceFactory();
                    InitThreadCulture();
                    OnAfterInitializeRepPP(System.EventArgs.Empty);
                }
            } catch(SystemException exc) {
                throw new ErrorInEventViewerException(exc);
            }
	        if (m_app == null) {
    	        throw new ErrorInEventViewerException(new ApplicationException("Unable to initialize the Rep++ application object - Check the event viewer for more information."));
	        } else if (m_app.SessionId == -1) {
    	        throw new FWException(FWException.EditorExceptionE.RepPPRequiresResourceDispencer);
    		} else if (!Utility.IsWebCodeGenIncluded(m_app)) {
                throw new FWException(FWException.EditorExceptionE.WebCodeGenRequired);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// m_pageFather_PreInit:           Function called before the Init is send to the hierarchy.
	    /// </summary>
	    /// <param name="sender">           Sender object</param>
	    /// <param name="e">                Event argument</param>
	    //
        //********************************************************************************
        private void m_pageFather_PreInit(object sender, EventArgs e) {
            CreateApplication();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GenDisplayAlert:            Display an alert
	    /// </summary>
	    /// <param name="writer">       Stream</param>
        /// <param name="strMsg">       Message to display in the alert</param>
	    //
        //********************************************************************************
        protected virtual void GenDisplayAlert(HtmlTextWriter writer, string strMsg) {
            writer.WriteLine("<script language=\"javascript\" type=\"text/javascript\">window.alert('" +
                                strMsg.Replace('\'', ' ') + "');</script>");
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// CheckIfErrorInEventViewer:  Checks if information has been logged in the event viewer.
	    /// </summary>
	    /// <param name="writer">       Stream</param>
	    //
        //********************************************************************************
        public virtual void CheckIfErrorInEventViewer(HtmlTextWriter writer) {
            Application   app;
            
            app = Application;
            if (app != null && app.IsEventLogged) {
                app.IsEventLogged = false;
                if (Utility.IsLocal) {
                    GenDisplayAlert(writer, m_strEventViewerErrorMsg);
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// CheckIfErrorInEventViewer:  Checks if information has been logged in the event viewer.
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual void CheckIfErrorInEventViewer() {
            System.Exception    exc;
            RepPP.Application   app;
            
            exc = m_pageFather.Server.GetLastError();
            app = Application;
            if (app != null && app.IsEventLogged) {
                app.IsEventLogged = false;
                if (!(exc is ErrorInEventViewerException) && Utility.IsLocal) {
                    throw new ErrorInEventViewerException(exc);
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// m_pageFather_Error:     Trigger events if an exception occur in the page
	    /// </summary>
	    /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event argument</param>
	    //
        //********************************************************************************
        private void m_pageFather_Error(object sender, EventArgs e) {
            CheckIfErrorInEventViewer();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// m_pageFather_Unload:            Called before the form is unload
	    /// </summary>
	    /// <param name="sender">           Sender object</param>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
        private void m_pageFather_Unload(object sender, EventArgs e) {
		    Dispose();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Application:                    Rep++ Application
	    /// </summary>
	    //
        //********************************************************************************
        public RepPP.Application Application {
            get {
                return(m_app);
            }
        }

		//*********************************************************
		//
		/// <summary>
		///  CreateApplication:       Allows the user to create a custom
		///                           application
		/// </summary>
		/// <param name="eLang">      Language of the application</param>
		/// <param name="oldApp">     Old Application</param>
		/// <param name="handler">    ProgInit Event Handler</param>
		/// <returns>
		/// Application or null
		/// </returns>
		//
		//*********************************************************
		protected virtual RepPP.Application CreateApplication(LanguageCode          eLang, 
                                                              RepPP.Application     oldApp, 
                                                              ProgInitEventHandler  handler) {
		    HttpApplicationFW   appFW;
            RepPP.Application   appRetVal = null;
		    
		    appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
		    if (appFW != null) {
		        appRetVal = appFW.CreateApplication(eLang, null, handler);
		    }
            return (appRetVal);
        }

		//*********************************************************
		//
		/// <summary>
		///  OnBeforeInitializeRepPP:       Called before the initialization of rep++
		/// </summary>
		/// <param name="e">                Event arguments</param>
		//
		//*********************************************************
		protected virtual void OnBeforeInitializeRepPP(EventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
		    if (appFW != null) {
		        appFW.BeforeInitializeRepPP(this.m_pageFather);
		    }
		    if (BeforeInitializeRepPP != null) {
		        BeforeInitializeRepPP(this, e);
		    }
        }

		//*********************************************************
		//
		/// <summary>
		///  OnProgInit:            Called during the initialization
		///                         of rep++
		/// </summary>
		/// <param name="sender">   Event sender</param>
		/// <param name="e">        Event arguments</param>
		//
		//*********************************************************
        protected virtual void OnProgInit(object sender, ApplicationEventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
		    if (appFW != null) {
		        appFW.ProgInit((RepPP.Application)sender);
		    }
		    if (ProgInit != null) {
		        ProgInit(sender, e);
		    }
        }

		//*********************************************************
		//
		/// <summary>
		///  OnAfterInitializeRepPP:        Called after the initialization of rep++
		/// </summary>
		/// <param name="e">                Event arguments</param>
		//
		//*********************************************************
		protected virtual void OnAfterInitializeRepPP(EventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
		    if (appFW != null) {
		        appFW.AfterInitializeRepPP(this.m_pageFather);
		    }
		    if (AfterInitializeRepPP != null) {
		        AfterInitializeRepPP(this, e);
		    }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Exception wrapper to indicate to check the event viewer
	    /// </summary>
	    //
        //********************************************************************************
        public class ErrorInEventViewerException : System.Exception {

            //********************************************************************************
            //
	        /// <summary>
	        /// ctor:                   Class constructor
	        /// </summary>
            /// <param name="excInner"> Inner exception</param>
	        //
            //********************************************************************************
            public ErrorInEventViewerException(System.Exception excInner) : base(m_strEventViewerErrorMsg, excInner) {}

            //********************************************************************************
            //
	        /// <summary>
	        /// ctor:                   Class constructor
	        /// </summary>
            /// <param name="strMsg">   Message of the exception</param>
	        //
            //********************************************************************************
            public ErrorInEventViewerException(string strMsg) : base(m_strEventViewerErrorMsg + ((strMsg == null) ? String.Empty : " - " + strMsg)) {}
        } // Class ErrorInEventViewer
    } // Class ApplicationHolder
} // Namespace
