using System;
using System.Drawing;
using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;
using System.Globalization;
using RepPP.Toolkit;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Implement the standard context menu for the framework. 
	/// All menu items are injected.
	/// </summary>
    //
    //********************************************************************************
    [ParseChildren(false)]
    public class ContextMenuFW : ContextMenu {
	    /// <summary>Name of each item in the ContextMenu</summary>
	    public enum ItemNameE {
	        /// <summary>New</summary>
	        New            =   0,
	        /// <summary>Duplicate</summary>
	        Duplicate      =   1,
	        /// <summary>Delete</summary>
	        Delete         =   2,
	    };

	    private RepPP.Toolkit.ControlUtil   m_controlUtil;
        private string                      m_strImagesFolder;
        private string                      m_strParentID;
        private const int                   m_iMaximumNumberOfItems = 2;


        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:           Class constructor
	    /// </summary>
        //
        //********************************************************************************
		public ContextMenuFW() {
            m_strImagesFolder   = "Images/";
            m_strParentID       = String.Empty;
		    m_controlUtil       = new RepPP.Toolkit.ControlUtil(this, new RepPP.Toolkit.Web.WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Dispose:        Dispose the control
	    /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            if (m_controlUtil != null) {
                m_controlUtil.Dispose();
                m_controlUtil = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnInit:                     Called to initialize the page
	    /// </summary>
	    /// <param name="e">            Event Argument</param>
        //
        //********************************************************************************
        protected override void OnInit(EventArgs e) {
            ArrayList   stdItems;
 
            EnableViewState = false;
		    stdItems      = BuildStdItems();
		    foreach (ContextMenuItem item in stdItems) {
                Items.Add(item);
		    }
            base.OnInit (e);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitItem:                   Initialize an item
	    /// </summary>
	    /// <param name="item">         Menu item</param>
	    /// <param name="strId">        Menu id</param>
	    /// <param name="strTitle">     Menu item title</param>
	    /// <param name="strTooltip">   Menu item tooltip</param>
	    /// <param name="strImg">       Image name</param>
	    /// <param name="strCmd">       Command name</param>
	    /// <param name="strScript">    Script to attach to the menu item</param>
        //
        //********************************************************************************
        protected virtual void InitItem(ContextMenuItem   item, 
                                        string            strId,
                                        string            strTitle,
                                        string            strTooltip,
                                        string            strImg,
                                        string            strCmd,
                                        string            strScript) {
            item.Id                         = strId;
            item.Text                       = strTitle;
            item.Tooltip                    = strTooltip;
            item.CommandName                = strCmd;
            item.JavaScriptCommand          = strScript;
            item.ImageURL                   = ImagesFolder + strImg + ".gif";
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitLink:                   Initialize a link
	    /// </summary>
	    /// <param name="item">         Menu item</param>
	    /// <param name="translator">   Translator</param>
	    /// <param name="strId">        Menu id</param>
	    /// <param name="strTitle">     Default title</param>
	    /// <param name="strImg">       Image name</param>
	    /// <param name="strCmd">       Command name</param>
	    /// <param name="strScript">    Script to attach to the menu item</param>
        //
        //********************************************************************************
        protected virtual void InitLink(ContextMenuItem           item, 
                                        ControlTextTranslatorBase translator, 
                                        string                    strId,
                                        string                    strTitle,
                                        string                    strImg,
                                        string                    strCmd,
                                        string                    strScript) {
            string strText;
            string strTooltip;

            strTooltip = strTitle;
            if (translator != null) {
                strText = translator.GetItemText(strId, "Text");
                if (strText != null && strText != String.Empty) {
                    strTitle = strText;
                }
                strTooltip = translator.GetItemText(strId, "Tooltip");
                if (strTooltip == null || strTooltip != String.Empty) {
                    strTooltip = strTitle;
                }
            }
            InitItem(item, strId, strTitle, strTooltip, strImg, strCmd, strScript);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// BuildStdItems:  Build the ilst of standard items
	    /// </summary>
        //
        //********************************************************************************
        protected virtual ArrayList BuildStdItems() {
            ContextMenuItem[]           itemsRetVal;
            RepPP.Application           app;
            ControlTextTranslatorBase   translator = null;
            
            app = m_controlUtil.Application;
            if (app != null && app.IsAtomExist("Editor_STD", RepPP.AtomType.sdAtomMsg)) {
                translator = new ControlTextTranslatorBase(app, new string[] {"Editor_STD"});
            }
            itemsRetVal     = new ContextMenuItem[m_iMaximumNumberOfItems];
            for (int iIndex = 0; iIndex < itemsRetVal.Length; iIndex++) {
                itemsRetVal[iIndex]  = new ContextMenuItem();
            }
            InitLink(itemsRetVal[0],  translator, "New",       "New",          "New",       "",    ParentID + "_DoCmd('new', caller);");
            InitLink(itemsRetVal[1],  translator, "Duplicate", "Duplicate",    "Duplicate", "",    ParentID + "_DoCmd('duplicate', caller);");
            //InitLink(itemsRetVal[2],  translator, "Delete",    "Delete",       "Delete",    "",    ParentID + "_DoCmd('delete', caller);");
            return(new ArrayList(itemsRetVal));
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ImagesFolder:   The path to the folder containing the menu's images
	    /// </summary>
        //
        //********************************************************************************
        [Category("Appearance"),
         NotifyParentProperty(true),
         Description("The path to the folder containing the menu's images"),
         Editor(typeof(System.Web.UI.Design.UrlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public virtual string ImagesFolder {
            get {
                return(m_strImagesFolder);
            }
            set {
                int     iOriSize;
                string  strOriURL;
                string  strURL;
                
                if (m_strImagesFolder != value) {
                    strOriURL         = (m_strImagesFolder == null) ? String.Empty : m_strImagesFolder;
                    iOriSize          = strOriURL.Length;
                    m_strImagesFolder = (value == null) ? String.Empty : value;
                    if (m_strImagesFolder != String.Empty && !m_strImagesFolder.EndsWith("/")) {
                        m_strImagesFolder += "/";
                    }
                    foreach (ContextMenuItem item in Items) {
                        strURL = item.ImageURL;
                        if (strURL.Substring(0, iOriSize) == strOriURL) {
                            item.ImageURL = value + strURL.Substring(iOriSize);
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ParentID:   The ID of the hosting control
	    /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        public virtual string ParentID {
            get {
                return(m_strParentID);
            }
            set {
                ArrayList   stdItems;

                if (m_strParentID != value) {
                    Items.Clear();
                    m_strParentID = value;
       		        stdItems      = BuildStdItems();
    		        foreach (ContextMenuItem item in stdItems) {
                        Items.Add(item);
		            }
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Items:          Override the standard property to make it invisible in the designer
	    ///                 and not serializable.
	    /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public override RepPP.Toolkit.Web.ContextMenuItemCollection Items {
            get {
                return base.Items;
            }
        }

	}
}
