using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.IO;
using RepPP;
using RepPP.Toolkit; 
using RepPP.Toolkit.Web; 

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// DataGridRowsetSelect:   Implement a datagrid which can be used to select item
	/// </summary>
    //
    //********************************************************************************
	public class DataGridRowsetSelect : RepPP.Toolkit.Web.DataGridRowset {
	    /// <summary>Position of the automatic button</summary>
	    public enum AutoButtonE {
	        /// <summary>No automatic button</summary>
	        None                =       0,
	        /// <summary>Delete and new button before context menu</summary>
	        BeforeContextMenu   =       1,
	        /// <summary>Delete and new button after context menu</summary>
	        AfterContextMenu    =       2
	    }
	
        /// <summary>Button column which implement the actions</summary>
        protected DataGridColumn                m_butColOp          = null;
        /// <summary>context menu link column</summary>
        protected TemplateColumn                m_ctxMenuLinkCol    = null;
        /// <summary>context menu control</summary>
        protected ContextMenu                   m_ctxMenu           = null;
        /// <summary>Rep++ command name</summary>
        public const string                     REPPP_CMD           = "_REPPPCMD";
        /// <summary>Refresh command name</summary>
        public const string                     REFRESH_CMD         = "_REFRESH";
        /// <summary>Filter command name</summary>
        public const string                     FILTER_CMD          = "_FILTER";
        /// <summary>Raised when a user executes a command on the datagrid</summary>
        [Browsable(true)]
        [Category("Action")]
        [Description("Event raised when a user executes a command on the datagrid")] 
        public event CommandEventHandler        CommandExecuted;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                           Class constructor
	    /// </summary>
	    //
        //********************************************************************************
		public DataGridRowsetSelect() {
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// CreateOpColumns:                Creates the column generating the operations
	    /// </summary>
	    //
        //********************************************************************************
		protected virtual DataGridColumn CreateOpColumns() {
		    ButtonColumnNP  butCol;
            string          strOnClickScript;
		    
		    butCol                      = new ButtonColumnNP();
		    butCol.ButtonType           = ButtonColumnType.LinkButton;
            butCol.HeaderStyle.Width    = new Unit(OperationsColumnSize);
            butCol.ItemStyle.Width      = new Unit(OperationsColumnSize);

            if (AllowNew && (AllowDelete || AllowDuplicate)) {
		        butCol.ButtonPosition = ButtonPositionE.ItemAndHeader;
            } else if (AllowNew){
		        butCol.ButtonPosition = ButtonPositionE.Header;
            } else if (AllowDelete || AllowDuplicate) {
		        butCol.ButtonPosition = ButtonPositionE.Item;
            }
	        butCol.JavaScriptCommand = " ";
            butCol.Text              = "";
            if (AllowDelete) {
                // Not implemented
		        butCol.Text = "<img border='no' src='" + DeleteImageUrl +"/>";
            }
            if (AllowDuplicate) {
		        if (butCol.Text != String.Empty) {
                    butCol.Text += "&nbsp;";
                }
                strOnClickScript  = ID + "_OnDuplicate(this.parentNode.parentNode.parentNode.attributes['RowKey'].value)";
                butCol.Text      += "<img border='no' src='" + DuplicateImageUrl + "' onclick=\"" + strOnClickScript + "\" />";
            }
            if (AllowNew) {
                butCol.HeaderJavaScriptCommand  = ID + "_OnNew()";
		        butCol.HeaderText               = "<img border='no' src='" + NewImageUrl + "' />";
            }
		    return(butCol);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// CreateCtxMenuLinkColumn:    Create a context menu link column
	    /// </summary>
	    //
        //********************************************************************************
		protected virtual TemplateColumn CreateCtxMenuLinkColumn() {
            TemplateColumn      colRetVal;

		    colRetVal                   = new TemplateColumn();
            colRetVal.ItemTemplate      = new TemplateContextMenuLink(MenuImageUrl, ContextMenu);
            colRetVal.HeaderStyle.Width = new Unit(ContextMenuColumnSize);
            colRetVal.ItemStyle.Width   = new Unit(ContextMenuColumnSize);
		    return(colRetVal);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// CreateColumnSet:                Creates the column generating the operations
	    /// </summary>
	    /// <param name="dataSource">       Data source</param>
	    /// <param name="bUseDataSource">   true if using data source</param>
	    //
        //********************************************************************************
        protected override ArrayList CreateColumnSet(PagedDataSource dataSource, bool bUseDataSource) {
            ArrayList       arrRetVal;
            AutoButtonE     eAutoButton;

            m_butColOp       = CreateOpColumns();
            m_ctxMenuLinkCol = CreateCtxMenuLinkColumn();
            
            eAutoButton = AutoButton;
            if (eAutoButton == AutoButtonE.AfterContextMenu) {
                Columns.AddAt(0, m_butColOp);
            }
            if (ContextMenu != String.Empty) {
                Columns.AddAt(0, m_ctxMenuLinkCol);
            }
            if (eAutoButton == AutoButtonE.BeforeContextMenu) {
                Columns.AddAt(0, m_butColOp);
            }
            arrRetVal = base.CreateColumnSet(dataSource, bUseDataSource);
            if (eAutoButton != AutoButtonE.None) {
                Columns.Remove(m_butColOp);
            }
            if (ContextMenu != String.Empty) {
                Columns.Remove(m_ctxMenuLinkCol);
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ComputeColSize:                 Compute the size of the operation button column
	    /// </summary>
        /// <param name="rowsetDef">        Rowset definition attached to the grid</param>
        /// <param name="fldDef">           Field definition attached to the cell</param>
        /// <param name="col">              Datagrid column</param>
        /// <param name="iMinColSize">      Minimum size of a column</param>
        /// <param name="iMaxColSize">      Maximum size of a column</param>
        /// <returns>
        /// Column size
        /// </returns>
	    //
        //********************************************************************************
        protected override int ComputeColSize(RowsetDef rowsetDef, FieldDef fldDef, DataGridColumn col, int iMinColSize, int iMaxColSize) {
            int             iRetVal;
            ButtonColumnNP  butCol;
            TemplateColumn  templateCol;
            
            iRetVal = base.ComputeColSize(rowsetDef, fldDef, col, iMinColSize, iMaxColSize);
            butCol  = col as ButtonColumnNP;
            if (butCol != null &&
                (String.Compare(butCol.Text, 0, "<img ", 0, 5, true) == 0 ||
                 String.Compare(butCol.HeaderText, 0, "<img ", 0, 5, true) == 0) ) {
                iRetVal = 4;
            } else if (col is TemplateColumn) {
                templateCol = (TemplateColumn)col;
                if (templateCol.ItemTemplate is TemplateContextMenuLink) {
                    iRetVal = 3;
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsSelectionBuffer:      Determines if the control is used as a selection buffer
        /// </summary>
        //
        //********************************************************************************
        [Description("Determines if the control is used as a selection buffer.")]
        [Category("Behavior")]
        [DefaultValue("")]
        [Bindable(true)]
        public virtual bool IsSelectionBuffer {
            get {
                bool    bRetVal;
                object  obj;

                obj     = ViewState["IsSelectionBuffer"];
                bRetVal = (obj != null) ? (bool)obj : false;
                return(bRetVal);
            }
            set {
                ViewState["IsSelectionBuffer"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuToOpen:      Get or set the context menu to open.
        /// </summary>
        //
        //********************************************************************************
        [Description("Gets or sets the Id of the ContextMenu to associate with the datagrid.")]
        [Category("Layout")]
        [DefaultValue("")]
        [Bindable(true)]
        [TypeConverter(typeof(ContextMenuLink.MenuControlConverter))]
        public virtual string ContextMenu {
            get {
                string                  strRetVal;
                object                  obj;

                obj         = ViewState["ContextMenu"];
                strRetVal   = (obj != null) ? (string)obj : String.Empty;
                return(strRetVal);
            }
            set {
                TemplateContextMenuLink templateCtxMenu;

                ViewState["ContextMenu"] = value;
                templateCtxMenu = (m_ctxMenuLinkCol != null) ? (m_ctxMenuLinkCol.ItemTemplate as TemplateContextMenuLink) : null;
                if (templateCtxMenu != null) {
                    templateCtxMenu.ContextMenu = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AutoButton:         Position of the automatic button (delete and new)
        /// </summary>
        //
        //********************************************************************************
        [Description("Position of the automatic button (delete and new).")]
        [Category("Layout")]
        [DefaultValue("")]
        [Bindable(true)]
        public virtual AutoButtonE AutoButton {
            get {
                AutoButtonE eRetVal;
                object      obj;

                obj     = ViewState["AutoButton"];
                eRetVal = (obj != null) ? (AutoButtonE)obj : AutoButtonE.None;
                return(eRetVal);
            }
            set {
                ViewState["AutoButton"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ChildEditorPage:       Page in charge of the next command
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Description("Page in charge of the next command.")]
        [Category("Behavior")]
        [DefaultValue("")]
        [Bindable(true)]
        public virtual string ChildEditorPage {
            get {
                string  strRetVal;
                object  obj;

                obj = ViewState["ChildEditorPage"];
                if (obj != null) {
                    strRetVal = (string)obj;
                } else {
                    strRetVal = String.Empty;
                }
                return(strRetVal);
            }
            set {
                ViewState["ChildEditorPage"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteImageUrl:         The image that will appear in the cells of the 
        ///                         operations column
        /// </summary>
        //
        //********************************************************************************
        //[Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        //[Category("Appearance")]
        //[Description("The image that will appear in the cells of the operations column")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string DeleteImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["DeleteImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = "images/Delete.gif";
                }
                return(strRetVal);
            }
            set {
                ViewState["DeleteImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateImageUrl:      The image that will appear in the cells of the 
        ///                         operations column
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Category("Appearance")]
        [Description("The image that will appear in the cells of the operations column")]
        public virtual string DuplicateImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["DuplicateImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = "images/Duplicate.gif";
                }
                return(strRetVal);
            }
            set {
                ViewState["DuplicateImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// NewImageUrl:            The image that will appear in the header of the 
        ///                         operations column
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Category("Appearance")]
        [Description("The image that will appear in the header of the operations column")]
        public virtual string NewImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["NewImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = "images/New.gif";
                }
                return(strRetVal);
            }
            set {
                ViewState["NewImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// MenuImageUrl:           The image that will appear in the cells of the 
        ///                         menu column
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Category("Appearance")]
        [Description("The image that will appear in the cells of the menu column")]
        public virtual string MenuImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["MenuImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = "images/menublip.gif";
                }
                return(strRetVal);
            }
            set {
                ViewState["MenuImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowDelete:           Whether the delete command will be accessible from the DataGrid
        /// </summary>
        //
        //********************************************************************************
        //[Category("Behavior")]
        //[Description("Whether the delete command will be accessible from the DataGrid")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        private bool AllowDelete {
            get {
                return(false);
            }
            set {
                // do nothing
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowNew:           Whether the delete command will be accessible from the DataGrid
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Description("Whether the New command will be accessible from the DataGrid")]
        public virtual bool AllowNew {
            get {
                bool    bRetVal;
                object  objAuto;
                
                objAuto = ViewState["AllowNew"];
                if (objAuto != null) {
                    bRetVal = (bool)objAuto;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AllowNew"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowDuplicate:           Whether the delete command will be accessible from the DataGrid
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Description("Whether the Duplicate command will be accessible from the DataGrid")]
        public virtual bool AllowDuplicate {
            get {
                bool    bRetVal;
                object  objAuto;
                
                objAuto = ViewState["AllowDuplicate"];
                if (objAuto != null) {
                    bRetVal = (bool)objAuto;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AllowDuplicate"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowEdit:           Whether the delete command will be accessible from the DataGrid
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Description("Whether the Edit command will be accessible from the DataGrid")]
        public virtual bool AllowEdit {
            get {
                bool    bRetVal;
                object  objAuto;
                
                objAuto = ViewState["AllowEdit"];
                if (objAuto != null) {
                    bRetVal = (bool)objAuto;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AllowEdit"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuColumnSize:          Size of the context menu column
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(20)]
        [Description("Size of the context menu column")]
        public virtual int ContextMenuColumnSize {
            get {
                int     iRetVal;
                object  objAuto;
                
                objAuto = ViewState["ContextMenuColumnSize"];
                if (objAuto != null) {
                    iRetVal = (int)objAuto;
                } else {
                    iRetVal = 20;
                }
                return(iRetVal);
            }
            set {
                ViewState["ContextMenuColumnSize"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OperationsColumnSize:          Size of the operations column
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(20)]
        [Description("Size of the operations column")]
        public virtual int OperationsColumnSize {
            get {
                int     iRetVal;
                object  objAuto;
                
                objAuto = ViewState["OperationsColumnSize"];
                if (objAuto != null) {
                    iRetVal = (int)objAuto;
                } else {
                    iRetVal = 20;
                }
                return(iRetVal);
            }
            set {
                ViewState["OperationsColumnSize"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLoad:             Initialize settings needed during the lifetime of the
        ///                     incoming Web request. Overriden to set a reference to the
        ///                     row that initiates a context menu and acctivate its items
        ///                     based on the capabilities of the grid
        /// </summary>
        /// <param name="e">    Event argument </param>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            Control                 currentContainer;
            
            base.OnLoad(e);
            if (ContextMenu != null && ContextMenu != string.Empty){
                currentContainer = this;
                do{
                    m_ctxMenu        = (ContextMenu)currentContainer.FindControl(ContextMenu);
                    currentContainer = currentContainer.Parent;
                }while( currentContainer != null && m_ctxMenu == null);
                if (m_ctxMenu == null) {
                    throw new InvalidOperationException("Cannot find a ContextMenu with the id '" + ContextMenu + "'");
                }
                m_ctxMenu.CallerReference = "this.parentNode.parentNode.attributes['RowKey'].value";
                if (m_ctxMenu is ContextMenuFW) {
                    (m_ctxMenu as ContextMenuFW).ParentID = ID;
                }
                if (m_ctxMenu is ContextMenuFW) {
                    m_ctxMenu.Items[(int)ContextMenuFW.ItemNameE.New].Visible       = AllowNew;
                    m_ctxMenu.Items[(int)ContextMenuFW.ItemNameE.Duplicate].Visible = AllowDuplicate;
                    //m_ctxMenu.Items[(int)ContextMenuFW.ItemNameE.Delete].Visible    = AllowDelete;
                }
            }
            if (IsGridPostBack) {
                this.DataBind();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Called when the databinding occurs
        /// </summary>
        /// <param name="e">    Event argument </param>
        //
        //********************************************************************************
        protected override void OnDataBinding(EventArgs e) {
            PageWithSelectionBuffer     pageFather;
            RowsetTree                  rowsetTree;

            if (!DesignMode && IsSelectionBuffer) {
                pageFather = Page as PageWithSelectionBuffer;
                if (pageFather != null && (!(pageFather is PageEditor) || pageFather.IsSelectionBufferCreated)) {
                    // Load selection buffer data only if the Transaction is on the stack
                    // or there is no transaction (Only a Page with a selection buffer)
                    if (pageFather.SelectionBuffer != null) {
                        rowsetTree = pageFather.SelectionBuffer.SBRowsetTree;
                    }
                }
            }
            base.OnDataBinding(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsColumnEditable:       Return whether or not the column is editable
        /// </summary>
        /// <param name="col">      Datarid column</param>
        /// <returns>
        /// True:       The column is editable
        /// False:      The column is not editable
        /// </returns>
        //
        //********************************************************************************
        private bool IsColumnEditable(DataGridColumn col) {
            bool            bRetVal = false;
            TemplateColumn  colTemplate;

            colTemplate = col as TemplateColumn;
            if (colTemplate != null) {
                bRetVal = (colTemplate.ItemTemplate is TemplateFldControl);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetCellOnClickScript:       Get cell Onclick script
        /// </summary>
        /// <param name="strRowKey">    Row key of the cell</param>
        /// <returns>
        /// OnClick script
        /// </returns>
        //
        //********************************************************************************
        protected override string GetCellOnClickScript(string strRowKey) {
            string  strRetVal;
            string  strCmdArgs;
                
            strCmdArgs = SELECT_ITEM_CMD + "=Edit,Item," + ChildEditorPage + "," + GetStackOperation() + "," + strRowKey;
            if (IsSelectionBuffer) {
                strRetVal = WebUtil.GetPostBackEventReference(Page, this, strCmdArgs);
            } else {
                strRetVal = WebUtil.GetClientValidatedPostback(Page, this, strCmdArgs);
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddCellAttributes:      Add attributes to a cell
        /// </summary>
        /// <param name="cell">     Cell</param>
        /// <param name="row">      Datagrid item (row)</param>
        /// <param name="col">      Column attached to the cell</param>
        /// <param name="rowset">    Group instance attached to the table</param>
        /// <param name="rowsetDef">      RowsetDef attached to the table</param>
        /// <param name="fldDef">      FieldDef attached to the cell</param>
        //
        //********************************************************************************
        protected override void AddCellAttributes(TableCell cell, DataGridItem row, DataGridColumn col, Rowset rowset, RowsetDef rowsetDef, FieldDef fldDef) {
            string      strOnClickScript = string.Empty;

            if (fldDef != null) {
                cell.Attributes.Add("FieldName", fldDef.Name);
            }
            if (!DesignMode              && 
                AllowEdit                &&
                !(col is TemplateColumn) &&
                !(col is ButtonColumnNP) &&
                (col is BoundColumn)     &&
                !IsColumnEditable(col)   ) {
                switch(row.ItemType) {
                case ListItemType.AlternatingItem:
                case ListItemType.Item:
                case ListItemType.SelectedItem:
                    cell.Attributes.Add("OnClick", ID + "_OnSelect('" + row.Attributes["RowKey"] + "')");
                    break;
                default:
                    break;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddTableAttributes:     Add attributes to a table
        /// </summary>
        /// <param name="tbl">      Table control</param>
        /// <param name="rowset">    RowsetDef instance attached to the table</param>
        /// <param name="rowsetDef">      RowsetDef attached to the table</param>
        //
        //********************************************************************************
        protected override void AddTableAttributes(Table tbl, Rowset rowset, RowsetDef rowsetDef) {
            bool    bIsSelectionBuffer;
            
            bIsSelectionBuffer = IsSelectionBuffer;
            base.AddTableAttributes(tbl, rowset, rowsetDef);
            tbl.Attributes.Add("IsSelectionBuffer", bIsSelectionBuffer ? "true" : "false");
            tbl.Attributes.Add("ChildEditorPage", ChildEditorPage);
            if (!bIsSelectionBuffer) {
                tbl.Attributes.Remove("CurrentKey");
                tbl.Attributes.Add("CurrentKey", "");
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetStackOperation:      Get the stack operation
        /// </summary>
        /// <returns>
        /// The stack operation
        /// </returns>
        //
        //********************************************************************************
        protected virtual string GetStackOperation() {
            string  strRetVal;

            if (IsSelectionBuffer) {
                if (Page is PageEditor) {
                    strRetVal = "TopTransaction";
                } else {
                    strRetVal = "PushTransaction";
                }
            } else {
               strRetVal = "PushPage";
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnSelectScript:      Get the script code which can be executed to select an item
        /// </summary>
        /// <returns>
        /// Select script code
        /// </returns>
        //
        //********************************************************************************
        protected virtual string GetOnSelectScript() {
            string      strRetVal;
            string      strIsDirtyScript;
            PageEditor  pageEditor;

            pageEditor = Page as PageEditor;
            if (IsSelectionBuffer && (pageEditor != null)) {
                strIsDirtyScript = pageEditor.CheckIsDirtyScript;
            } else {
                strIsDirtyScript = String.Empty;
            }
            strRetVal = strIsDirtyScript + GetCellOnClickScript("$$$");
            strRetVal = strRetVal.Replace("$$$'", "' + strRowKey");
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnSelectScript:          Get the script code which can be executed to select an item
        /// </summary>
        /// <param name="strRowKey">    Key of the item to be selected</param>
        /// <returns>
        /// Select script code
        /// </returns>
        //
        //********************************************************************************
        public virtual string GetOnSelectScript(string strRowKey) {
            return(GetOnSelectScript().Replace("' + strRowKey", strRowKey + "'"));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnNewScript:      Get the script code which can be executed to create a new item
        /// </summary>
        /// <returns>
        /// New script code
        /// </returns>
        //
        //********************************************************************************
        public virtual string GetOnNewScript() {
            string      strRetVal;
            string      strIsDirtyScript;
            string      strCmdArgs;
            PageEditor  pageEditor;

            pageEditor = Page as PageEditor;
            if (IsSelectionBuffer && (pageEditor != null)) {
                strIsDirtyScript = pageEditor.CheckIsDirtyScript;
            } else {
                strIsDirtyScript = String.Empty;
            }
            strCmdArgs = REPPP_CMD + "=New,Item," + ChildEditorPage + "," + GetStackOperation();
            if (IsSelectionBuffer) {
                strRetVal = strIsDirtyScript + WebUtil.GetPostBackEventReference(Page, this, strCmdArgs);
            } else {
                strRetVal = strIsDirtyScript + WebUtil.GetClientValidatedPostback(Page, this, strCmdArgs);
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnDuplicateScript:      Get the script code which can be executed to duplicate an item
        /// </summary>
        /// <returns>
        /// Duplicate script code
        /// </returns>
        //
        //********************************************************************************
        protected virtual string GetOnDuplicateScript() {
            string      strRetVal;
            string      strIsDirtyScript;
            string      strCmdArgs;
            PageEditor  pageEditor;

            pageEditor = Page as PageEditor;
            if (IsSelectionBuffer && (pageEditor != null)) {
                strIsDirtyScript = pageEditor.CheckIsDirtyScript;
            } else {
                strIsDirtyScript = String.Empty;
            }
            strCmdArgs = REPPP_CMD + "=Duplicate,Item," + ChildEditorPage + "," + GetStackOperation() + ",";
            strCmdArgs = "'" + strCmdArgs + "' + strRowKey";
            if (IsSelectionBuffer) {
                strRetVal = strIsDirtyScript + WebUtil.GetPostBackEventReference(Page, this, "$$$");
            } else {
                strRetVal = strIsDirtyScript + WebUtil.GetClientValidatedPostback(Page, this, "$$$");
            }
            strRetVal = strRetVal.Replace("'$$$'", strCmdArgs);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnDuplicateScript:       Get the script code which can be executed to duplicate an item
        /// </summary>
        /// <param name="strRowKey">    Key of the item to be duplicated</param>
        /// <returns>
        /// Duplicate script code
        /// </returns>
        //
        //********************************************************************************
        public virtual string GetOnDuplicateScript(string strRowKey) {
            return(GetOnDuplicateScript().Replace("' + strRowKey", strRowKey + "'"));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnRefreshScript:         Get the script code which can be executed to refresh the
        ///                             selection buffer
        /// </summary>
        /// <returns>
        /// Refresh script code
        /// </returns>
        //
        //********************************************************************************
        public virtual string GetOnRefreshScript() {
            string      strRetVal = String.Empty;
            string      strIsDirtyScript;
            PageEditor  pageEditor;

            if (IsSelectionBuffer) {
                pageEditor       = Page as PageEditor;
                strIsDirtyScript = (pageEditor != null) ? pageEditor.CheckIsDirtyScript : String.Empty;
                strRetVal        = strIsDirtyScript + WebUtil.GetPostBackEventReference(Page, this, REFRESH_CMD);
                strRetVal        = "function DataGridRowsetSelect_OnRefresh() {\r\n" + strRetVal + "\r\n}";
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetOnFilterScript:      Get the script code which can be executed to filter the
        ///                         selection buffer
        /// </summary>
        /// <returns>
        /// Filter script code
        /// </returns>
        //
        //********************************************************************************
        public virtual string GetOnFilterScript() {
            string strRetVal = String.Empty;
            string strCmdArgs;

            if (IsSelectionBuffer) {
                strCmdArgs = "'" + FILTER_CMD + "=' + strFilterClause";
                strRetVal  = WebUtil.GetPostBackEventReference(Page, this, "$$$");
                strRetVal  = strRetVal.Replace("'$$$'", strCmdArgs);
                strRetVal  = "function DataGridRowsetSelect_OnFilter(strFilterClause) {\r\n" + strRetVal + "\r\n}";
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:            Add javascript before rendering occurs
	    /// </summary>
	    /// <param name="e">        Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            StreamReader            reader;
            string                  strScript;
            PageEditor              pageEditor;
            SelectionBufferHolder   sbHolder            = null;
            Rowset                  rowsetTransRoot;
            RowsetDef               rowsetDefTransRoot;
            bool                    bSelectCurrentLine  = true;

            base.OnPreRender(e);
            if (IsSelectionBuffer && ChildEditorPage == String.Empty && !DesignMode) {
                throw new System.ArgumentException("The ChildEditorPage property must be set when the control is used as a selection buffer");
            }
            if (ID != null) {
                reader    = new StreamReader(typeof(DataGridRowsetSelect).Assembly.GetManifestResourceStream("RepPP.Framework.Web.DataGridRowsetSelectScript.js"));
                strScript = String.Format(reader.ReadToEnd(),
                                          ID,
                                          GetOnSelectScript(),
                                          GetOnNewScript(),
                                          GetOnDuplicateScript(),
                                          GetOnRefreshScript(),
                                          GetOnFilterScript());
                WebUtil.RegisterClientScriptBlock(Page, ID, strScript);
            }
            if (Controls != null && Controls.Count > 0){
                Controls[0].EnableViewState = false;
            }
            //Clear the selection if the editor contains an item not in the selection buffer
            if (!DesignMode && IsSelectionBuffer) {
                pageEditor = Page as PageEditor;
                if (pageEditor != null){
                    rowsetTransRoot = pageEditor.TransactionRowsetTree.RootRowset;
                    if (rowsetTransRoot.LineState == LineState.sdLineNew) {
                        bSelectCurrentLine = false;
                    } else {
                        rowsetDefTransRoot = rowsetTransRoot.RowsetDef; 
                        if (sbHolder != null && sbHolder.SelectItem(WebUtil.BuildPrimaryKey(rowsetDefTransRoot, true), false) == false) {
                            bSelectCurrentLine = false;
                        }                
                    }
                }
            }
            if (!IsSelectionBuffer || !bSelectCurrentLine) {
                SelectedIndex = -1;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Render:                 Render the control
	    /// </summary>
        /// <param name="writer">   HTML text writer</param>
	    //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            AutoButtonE     eAutoButton;

            if (m_butColOp == null) {
                m_butColOp = CreateOpColumns();
            }
            if (m_ctxMenuLinkCol == null) {
                m_ctxMenuLinkCol = CreateCtxMenuLinkColumn();
            }
            eAutoButton = AutoButton;
            if (eAutoButton == AutoButtonE.AfterContextMenu) {
                Columns.AddAt(0, m_butColOp);
            }
            if (ContextMenu != String.Empty) {
                Columns.AddAt(0, m_ctxMenuLinkCol);
            }
            if (eAutoButton == AutoButtonE.BeforeContextMenu) {
                Columns.AddAt(0, m_butColOp);
            }
            base.Render(writer);
            if (eAutoButton != AutoButtonE.None) {
                Columns.Remove(m_butColOp);
            }
            if (ContextMenu != String.Empty) {
                Columns.Remove(m_ctxMenuLinkCol);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnCommandExecuted:          Raise the CommandExecuted event
	    /// </summary>
        /// <param name="e">            Event argumentsr</param>
	    //
        //********************************************************************************
        protected virtual void OnCommandExecuted(CommandEventArgs e) {
            PageEditor editor;
            switch(e.CommandName) {
            case REPPP_CMD:
            case SELECT_ITEM_CMD:
                editor = Page as PageEditor;
                if (editor != null) {
                    if (e.CommandArgument.Equals(String.Empty)) {
                        editor.ExecuteCmds();
                    } else {
                        editor.ExecuteCmds((string)e.CommandArgument);
                    }
                } else if (Page is PageWithSelectionBuffer) {
                    (Page as PageWithSelectionBuffer).Transfer(ChildEditorPage, (string)e.CommandArgument);
                }
                break;
            case REFRESH_CMD:
                Refresh();
                break;
            case FILTER_CMD:
                Filter((string)e.CommandArgument);
                editor = Page as PageEditor;
                if (editor != null) {
                    editor.ExecuteCmds("Edit,First," + ChildEditorPage + "," + GetStackOperation());
                }
                break;
            default:
                break;
            }
            if (CommandExecuted != null) {
                CommandExecuted(this, e);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnItemSelected:         Raise the ItemSelected event
	    /// </summary>
        /// <param name="e">        Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnItemSelected(CommandEventArgs e) {
            OnCommandExecuted(e);
            base.OnItemSelected(e);
        }

        //********************************************************************************
        //
		/// <summary>
		/// RaisePostBackEvent:     Called to raise an event
		/// </summary>
		//
        //********************************************************************************
        public override void RaisePostBackEvent(string strEventArgument) {
            string  strCmdArgs = String.Empty;
            string  strCmd;
            bool    bIsEditorCommand;
            bool    bCmdSet = false;;
            
            // setting the command early will set Cmd.UpdateValue 
            // which is needed by Page.Validate()
            strCmd = strEventArgument.ToUpper();
            if (strCmd.StartsWith(SELECT_ITEM_CMD + "=")) {
                strCmdArgs = strEventArgument.Substring(SELECT_ITEM_CMD.Length + 1);
            } else if (strCmd.StartsWith(REPPP_CMD + "=")) {
                strCmdArgs = strEventArgument.Substring(REPPP_CMD.Length + 1);
            }
            bIsEditorCommand = !strCmdArgs.Equals(String.Empty) ;
            if (bIsEditorCommand) {
                if (Page is PageEditor) {
                    (Page as PageEditor).SetCommands(strCmdArgs);
                    bCmdSet = true;
                }
            }
            // The selection buffer does not trigger the validation.
            if (!IsSelectionBuffer) {
                Page.Validate();
            }
            if (bIsEditorCommand) {
                OnCommandExecuted(new CommandEventArgs(REPPP_CMD, bCmdSet ? String.Empty : strCmdArgs));
            } else if (strCmd == REFRESH_CMD) {
                OnCommandExecuted(new CommandEventArgs(REFRESH_CMD, String.Empty));
            } else if (strCmd.StartsWith(FILTER_CMD + "=")) {
                strCmdArgs = strEventArgument.Substring(FILTER_CMD.Length + 1);
                OnCommandExecuted(new CommandEventArgs(FILTER_CMD, strCmdArgs));
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Refresh:            Refresh the selection buffer
		/// </summary>
		//
        //********************************************************************************
        public virtual void Refresh() {
            PageWithSelectionBuffer  page;

            page = Page as PageWithSelectionBuffer;
            if (page != null) {
                page.SelectionBuffer.Refresh();
                if (page is PageEditor) {
                    (page as PageEditor).SetSBCurrentSelection();
                }
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Filter:                     Filter the selection buffer
		/// </summary>
		/// <param name="strClause">    Filter clause</param>
		//
        //********************************************************************************
        public virtual void Filter(string strClause) {
            PageWithSelectionBuffer  page;
            DataSourceRowset         dataSourceRowset;

            page = Page as PageWithSelectionBuffer;
            if (page != null) {
                if (strClause != String.Empty) {
                    strClause = " WHERE " + strClause;
                }
                page.SelectionBuffer.FilterClause = strClause;
                dataSourceRowset                  = DataSource as DataSourceRowset;
                if (dataSourceRowset != null) {
                    dataSourceRowset.Refresh();
                }                
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ApplySortExpression:        Applies the sort expression to the current 
        ///                             DataSource
        /// </summary>
        /// <param name="bDataBinding"> Indicates that the page is in the databinding phase</param>
        //
        //********************************************************************************
        protected override bool ApplySortExpression(bool bDataBinding) {
            object[]                arrSortParam;
            DataSourceRowset        dsRowset;
            PageWithSelectionBuffer parentPage;
            PageEditor              parentEditor;
            SelectionBufferHolder   sbHolder;
            Rowset                  rowsetDataSource      = null;
            bool                    bRowsetAlreadySorted = false;
            bool                    bRetVal              = false;

            bRetVal = base.ApplySortExpression(bDataBinding);
            if (bRetVal && (m_strSortColumn != null) && (m_strSortDirection != null)) {
                arrSortParam    = new object[2];
                arrSortParam[0] = m_strSortColumn;
                arrSortParam[1] = (m_strSortDirection.EndsWith("ASC")) ? RepPP.Order.sdAscending : RepPP.Order.sdDescending;
                dsRowset        = DataSource == null ? null : (DataSource as DataSourceRowset);
                parentEditor    = Page as PageEditor;
                parentPage      = Page as PageWithSelectionBuffer;
                if (dsRowset != null) {
                    rowsetDataSource     = dsRowset.Rowset;
                    bRowsetAlreadySorted = dsRowset.SortRowset;
                }
                if (parentEditor != null && rowsetDataSource != null &&  parentEditor.TransactionRowsetTree == rowsetDataSource.RowsetTree) {
                    if (bDataBinding){
                        if (!bRowsetAlreadySorted) {
                            rowsetDataSource.Sort(arrSortParam);
                        }
                        parentEditor.SaveToCache();
                    }
                } else if (IsSelectionBuffer && parentPage != null){
                    parentPage.SetSBInitialSort(arrSortParam);
                    if (bDataBinding){
                        sbHolder = parentPage.SelectionBuffer;
                        if (sbHolder != null){
                            sbHolder.EnsureSortedOrder();
                        }
                    }
                }
            }
            return(bRetVal);
        }
	} // Class DataGridRowsetSelect
} // Namespace
