using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.Caching;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    
	/// <summary>
	/// Implements the the ISessionDataHolder and ICacheDataHolder interfaces to handle
	/// saving the data in the session and cache.
	/// </summary>
	public class DataHolder: ISessionDataHolder, ICacheDataHolder {

		private const string        m_strUniqueIdKeyName = "UniqueIdProvider";
		private static object       m_objectLock = new object();
        private HttpSessionState    m_session = null;
        
        /// <summary>Event sent when a request realizes that the session has timed out</summary>
	    public event EventHandler	SessionTimeout;
		
        //********************************************************************************
        //
        /// <summary>
        /// Session:					Gets the ASP.NET Session
        /// </summary>
        //
        //********************************************************************************
		public virtual HttpSessionState Session {
			get {
                HttpSessionState sessionRetVal;

                if (m_session != null) {
                    sessionRetVal = m_session;
                } else {
                    sessionRetVal = HttpContext.Current.Session;
                }
				return(sessionRetVal);
			}
            set {
                m_session = value;
            }
		}


        //********************************************************************************
        //
        /// <summary>
        /// Cache:						Gets the ASP.NET Cache
        /// </summary>
        //
        //********************************************************************************
		protected virtual Cache Cache {
			get {
				return(HttpRuntime.Cache);
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// SaveToSession:              Save an object in the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <param name="obj">          Object to save</param>
        //
        //********************************************************************************
        public virtual void SaveToSession(string strName, object obj) {
            Session[strName] = obj;
		}

        //********************************************************************************
        //
        /// <summary>
        /// IsExistInSession:           Check if an object exist in the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// true if yes, false if no
        /// </returns>
        //
        //********************************************************************************
        public virtual bool IsExistInSession(string strName) {
			bool bRetVal;

			EnsureSessionDidNotTimeOut();
            bRetVal = Session[strName] != null;
			return(bRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromSession:            Load an object from the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// object or null if not in the session
        /// </returns>
        //
        //********************************************************************************
        public virtual object LoadFromSession(string strName) {
			object objRetVal;

			EnsureSessionDidNotTimeOut();
            objRetVal = Session[strName];
			return(objRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// DeleteFromSession:          Delete an object from the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        //
        //********************************************************************************
        public virtual void DeleteFromSession(string strName) {
			Session.Remove(strName);
		}

        //********************************************************************************
        //
        /// <summary>
        /// GetUniqueId:                Get a unique id for the session
        /// </summary>
        /// <returns>
        /// Unique id
        /// </returns>
        //
        //********************************************************************************
		public virtual int GetUniqueId() {
			int iRetVal;


			lock(SessionSyncRoot) {
				if (Session[m_strUniqueIdKeyName] == null) {
					iRetVal = 0;
				} else {
					iRetVal = (int)Session[m_strUniqueIdKeyName];
				}
				Session[m_strUniqueIdKeyName] = iRetVal + 1;
			}
			return(iRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// SessionSyncRoot:	        Lock the session to sync access
        /// </summary>
        //
        //********************************************************************************
        public virtual object SessionSyncRoot {
            get {
                return(Session.SyncRoot);
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// SaveToCache:                Save an object in the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <param name="obj">          Object to save</param>
        /// <param name="bPermanent">   true if permanent, false if it can be discarded</param>
        //
        //********************************************************************************
        public virtual void SaveToCache(string strName, object obj, bool bPermanent) {
			lock(CacheSyncRoot) {
				Cache.Insert(strName,
							obj,
							null,
							Cache.NoAbsoluteExpiration,
							Cache.NoSlidingExpiration,
							bPermanent ? CacheItemPriority.NotRemovable : CacheItemPriority.Default,
							null);
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// IsExistInCache:             Check if an object exist in the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// true if yes, false if no
        /// </returns>
        //
        //********************************************************************************
        public virtual bool IsExistInCache(string strName) {
			bool bRetVal;

			bRetVal = Cache[strName] != null;
			return(bRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromCache:              Load an object from the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// object or null if not in the cache
        /// </returns>
        //
        //********************************************************************************
        public virtual object LoadFromCache(string strName) {
			object objRetVal;

			objRetVal = Cache[strName];
			return(objRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// DeleteFromCache:            Delete an object from the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        //
        //********************************************************************************
        public virtual void DeleteFromCache(string strName) {
			lock(CacheSyncRoot) {
				Cache.Remove(strName);
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// CacheSyncRoot:			    Lock the cache to sync access
        /// </summary>
        //
        //********************************************************************************
        public virtual object CacheSyncRoot {
            get {
                return(m_objectLock);
            }
		}

        //********************************************************************************
        //
        /// <summary>
        ///  OnSessionTimeout:              Called when a session time out is detected
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnSessionTimeout(EventArgs e) {
            HttpApplicationFW appFW;

            appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
            if (appFW != null) {
                appFW.SessionTimeout(this);
            }
            if (SessionTimeout != null) {
                SessionTimeout(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EnsureSessionDidNotTimeOut:         Makes sure that the session did not time out
        /// </summary>
        /// <remarks>
        /// Throws a clear exception indicating that a session has timed out.
        /// </remarks>
        //
        //********************************************************************************
        protected virtual void EnsureSessionDidNotTimeOut() {
            string strCookieHeader;
            if (Session != null && Session.IsNewSession) {
                strCookieHeader= HttpContext.Current.Request.Headers["Cookie"];
                if ((strCookieHeader != null) && (strCookieHeader.IndexOf("ASP.NET_SessionId") >= 0)) {
                    OnSessionTimeout(EventArgs.Empty);
                }
            }
		}

	} // class DataHolder
} // Namespace