using System;
using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;
using System.Globalization;
using RepPP.Toolkit;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Implement the standard toolbar for the framework. All button items are injected.
	/// </summary>
    //
    //********************************************************************************
	[ToolboxData("<{0}:DynamicFormTableForFilter runat=server></{0}:DynamicFormTableForFilter>")]
    public class DynamicFormTableForFilter : DynamicFormTable {
        private string                  m_strBetween;
	    private SBFilter                m_filter;
	    private RadioButtonList         m_ctlRadioButtonList;
        private string                  m_strPermanentWhereClause;  // Where clause to be applied all the time
	    private bool                    m_bRefresh;                 // Generates code to refresh the selection buffer
	    private bool                    m_bExit;                    // Generates code to close the filter window

        //********************************************************************************
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        //
        //********************************************************************************
        public DynamicFormTableForFilter() {
            m_strBetween                = "between";
            Option                      = OptionE.OPT_ADD_BETWEEN_FIELD |
                                          OptionE.OPT_ADD_OPERATOR_COMBO |
                                          OptionE.OPT_ALWAYS_SELECTABLE |
                                          OptionE.OPT_NO_CHECK |
                                          OptionE.OPT_NO_RADIO |
                                          OptionE.OPT_ONLY_MAIN_TBL_FIELD;
            m_filter                    = null;
            m_ctlRadioButtonList        = null;
            m_strPermanentWhereClause   = String.Empty;
            m_bRefresh                  = false;
            m_bExit                     = false;
        }

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_filter != null) {
                m_filter.Dispose();
                m_filter = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// PermanentWhereClause:         Where clause applied in addition to whatever filter value
        /// </summary>
        //
        //********************************************************************************
        //DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(true)]
		public virtual string PermanentWhereClause {
		    get {
		        return(m_strPermanentWhereClause);
		    }
		    set {
		        m_strPermanentWhereClause = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Option:         Option to use for field generations
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public override OptionE Option {
		    get {
		        return(base.Option);
		    }
		    set {
		        base.Option = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Between:            Word use for BETWEEN in the combobox
        /// </summary>
        //
        //********************************************************************************
        public virtual string Between {
            get {
                return(m_strBetween);
            }
            set {
                SBFilter    filter;
                
                m_strBetween = (value == null || value == "") ? "between" : value;
                filter       = Filter;
                if (filter != null) {
                    filter.Between = Between;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateOpCombo:          Creates the dropdown list used for operation
        /// </summary>
        /// <param name="fldDef">   Field definition attached to the row</param>
        /// <param name="cell">     Father cell</param>
        /// <returns>
        /// A dropdown list control
        /// </returns>
        //
        //********************************************************************************
        protected override DropDownList CreateOpCombo(FieldDef fldDef, TableCell cell) {
            DropDownList    ctlRetVal;
            
            ctlRetVal = base.CreateOpCombo(fldDef, cell);
            ctlRetVal.Items.AddRange(new ListItem[] { new ListItem("="),
                                                      new ListItem("<"),
                                                      new ListItem("<="),
                                                      new ListItem(">"),
                                                      new ListItem(">="),
                                                      new ListItem("<>"),
                                                      new ListItem(m_strBetween, "between") });
            ctlRetVal.SelectedIndex = 0;
            ctlRetVal.Attributes.Add("onchange", "DynamicFormTableForFilter_OnListChg(this);");
            return(ctlRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateControl:              Creates a control
        /// </summary>
        /// <param name="eCtlType">     Type of control</param>
        /// <param name="fldDef">       Field definition attached to the cell</param>
        /// <param name="strCtlName">   Name of the control or null if using field name</param>
        /// <param name="cell">         Father cell</param>
        /// <param name="eType">        Cell type</param>
        //
        //********************************************************************************
        protected override RepPP.Toolkit.FldControlUtil.IFldControl CreateControl(CtlType eCtlType, FieldDef fldDef, string strCtlName, TableCell cell, CellTypeE eType) {
            RepPP.Toolkit.FldControlUtil.IFldControl    ctlRetVal;
            
            ctlRetVal = base.CreateControl(eCtlType, fldDef, strCtlName, cell, eType);
            if (eType == CellTypeE.Between) {
                ((WebControl)ctlRetVal).Style["visibility"] = "hidden";
            }
            return(ctlRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateAndOrRadioButton:     Creates the AND/OR radio button control list
        /// </summary>
        /// <param name="row">          Row in which the button list is created</param>
        //
        //********************************************************************************
        protected virtual void CreateAndOrRadioButton(TableRow row) {
            RadioButtonList ctl;
            TableCell       cell;
            
            cell    = CreateCell(null, CellTypeE.Other);
            cell.Style["HorizontalAlign"] = "Center";
            row.Cells.Add(cell);
            cell.HorizontalAlign = HorizontalAlign.Center;
            ctl     = new RadioButtonList();
            ctl.ID  = "AndOr";
            ctl.Items.Add(new ListItem("And", "And"));
            ctl.Items.Add(new ListItem("Or", "Or"));
            ctl.SelectedIndex = 0;
            ctl.RepeatDirection = RepeatDirection.Horizontal;
            cell.Controls.Add(ctl);
            m_ctlRadioButtonList = ctl;
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateCmdButton:            Create the command button
        /// </summary>
        /// <param name="row">          Row in which the command button is created</param>
        //
        //********************************************************************************
        protected virtual void CreateCmdButton(TableRow row) {
            Button          ctl;
            TableCell       cell;
            
            cell                    = CreateCell(null, CellTypeE.Other);
            row.Cells.Add(cell);
            cell.HorizontalAlign    = HorizontalAlign.Center;
            ctl                     = new Button();
            ctl.Text                = "Ok";
            ctl.ID                  = "butOK";
            ctl.Click              += new EventHandler(ctlButton_Click);
            cell.Controls.Add(ctl);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateGroupFields:                  Create field controls in the specified container. Override to
        ///                                     add a And/Or control and a command button
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="eOption">              Generation options</param>
        /// <param name="arrComboBox">          Array of combo box generated if option specified</param>
        /// <param name="arrLabel">             Array of created static fields</param>
        /// <param name="arrFieldCtl">          Array of created fields</param>
        /// <param name="arrBetweenFieldCtl">   Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        protected override void CreateRowsetDefFields(RowsetDef rowsetDef, RepPP.Toolkit.Web.DynamicFormTable.OptionE eOption, out DropDownList[] arrComboBox, out RepPP.Toolkit.FldControlUtil.IFldControl[] arrLabel, out RepPP.Toolkit.FldControlUtil.IFldControl[] arrFieldCtl, out RepPP.Toolkit.FldControlUtil.IFldControl[] arrBetweenFieldCtl) {
            TableRow    row;
            
            row     = CreateRow(null);
            Rows.Add(row);
            CreateAndOrRadioButton(row);
            base.CreateRowsetDefFields (rowsetDef, eOption, out arrComboBox, out arrLabel, out arrFieldCtl, out arrBetweenFieldCtl);
            row     = CreateRow(null);
            Rows.Add(row);
            CreateCmdButton(row);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateFilter:               Creates the filter
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition use as a template to create the filter</param>
        //
        //********************************************************************************
        protected virtual SBFilter CreateFilter(RowsetDef rowsetDef) {
            return(new SBFilter(rowsetDef));
        }

        //********************************************************************************
        //
        /// <summary>
        /// Filter:             Selection buffer filter
        /// </summary>
        //
        //********************************************************************************
        public virtual SBFilter Filter {
            get {
                return(m_filter);
            }
            set {
                m_filter = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SourceRowsetDef:            RowsetDef use to define the form
        /// </summary>
        //
        //*********************************************************	    
        protected override RowsetDef SourceRowsetDef {
            get {
                RowsetDef   rowsetDefRetVal;
                SBFilter    filter;
                
                filter          = Filter;
                rowsetDefRetVal = (filter == null) ? null : filter.ParamRowset.RowsetDef;
                return(rowsetDefRetVal);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateChildControls:        Fill the table
        /// </summary>
        //
        //*********************************************************	    
         protected override void CreateChildControls() {
            RowsetDef   rowsetDefTemplate;
            
            rowsetDefTemplate = RowsetDef;
            if (rowsetDefTemplate != null) {
                Filter          = CreateFilter(rowsetDefTemplate);
                Filter.Between  = Between;
            }
            base.CreateChildControls();
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetAndOrValue:      Gets the value of the AND/OR switch
        /// </summary>
        //
        //********************************************************************************
        protected virtual SBFilterBase.AndOrE GetAndOrValue() {
            SBFilterBase.AndOrE eRetVal = SBFilterBase.AndOrE.AND;
            ListItem            itemSelected;
        
            itemSelected    = m_ctlRadioButtonList.SelectedItem;
            if (itemSelected != null) {
                switch(itemSelected.Value.ToUpper()) {
                case "AND":
                    eRetVal = SBFilterBase.AndOrE.AND;
                    break;
                case "OR":
                    eRetVal = SBFilterBase.AndOrE.OR;
                    break;
                default:
                    break;
                }
            }
            return(eRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFilter:          Set the selection buffer filter
        /// </summary>
        //
        //********************************************************************************
        protected virtual void SetFilter() {
            DropDownList[]      arrDropDown;
            SBFilterBase.AndOrE eAndOr;
            SBFilter            filter;
            
            filter      = Filter;
            arrDropDown = DropDownListArray;
            if (filter != null && arrDropDown != null) {
                eAndOr          = GetAndOrValue();
                filter.BuildOpArray(arrDropDown);
                filter.SetFilter(eAndOr);
                m_bRefresh  = true;
                m_bExit     = true;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetBetweenFieldState:           Set the visibility state of the BETWEEN field depending on
	    ///                                 the currently selected operation
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual void SetBetweenFieldState() {
            DropDownList[]                  arrDropDown;
            FldControlUtil.IFldControl[]    arrBetweenFldControl;
            DropDownList                    ctlDrop;
            WebControl                      ctlBetween;
            string                          strAttrVal;
            int                             iCount;

            arrDropDown             = DropDownListArray;
            arrBetweenFldControl    = BetweenFieldControlArray;
            iCount                  = arrDropDown.Length;
            for (int iIndex = 0; iIndex < iCount; iIndex++) {
                ctlDrop     = arrDropDown[iIndex];
                ctlBetween  = arrBetweenFldControl[iIndex] as WebControl;
                strAttrVal  = (ctlDrop.SelectedIndex != -1 && ctlDrop.SelectedValue == "between") ? "visible" : "hidden";
                ctlBetween.Style["Visibility"] = strAttrVal;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:                    Called before the page is rendered
	    /// </summary>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            StreamReader        reader;
            const string        strKey = "DynamicFormTableForFilter";
            string              strExec;
            string              strWhereClause;

            base.OnPreRender(e);
            SetBetweenFieldState();
	        if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                strExec = "<script language='javascript'>\r\n<!--\r\n";
                if (m_bRefresh) {
                    if (PermanentWhereClause != String.Empty && Filter.WhereClause != String.Empty) {
                        strWhereClause = "( " + PermanentWhereClause + " ) AND ( " + Filter.WhereClause + " )";
                    } else if (PermanentWhereClause != String.Empty) {
                        strWhereClause = PermanentWhereClause;
                    } else {
                        strWhereClause = Filter.WhereClause;
                    }
                    strExec += "DynamicFormTableForFilter_FilterSelectionBuffer(\"" + strWhereClause + "\");\r\n";
                }
                if (m_bExit) {
                    strExec += "DynamicFormTableForFilter_Exit();\r\n";
                }
                strExec += "-->\r\n</script>\r\n";
                reader = new StreamReader(typeof(DynamicFormTableForFilter).Assembly.GetManifestResourceStream("RepPP.Framework.Web.DynamicFormTableForFilterScript.js"));
	            WebUtil.RegisterClientScriptBlock(Page,
                                                  strKey,
                                                  reader.ReadToEnd() + strExec);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnUnload:           Called before the page is unloaded
        /// </summary>
        /// <param name="e">    Argument to pass to the event</param>
        //
        //********************************************************************************
        protected override void OnUnload(EventArgs e) {
            SBFilter    filter;
            
            filter = Filter;
            if (filter != null) {
                filter.Dispose();
                filter = null;
            }
            base.OnUnload(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctlButton_Click:        Click event
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Argument to pass to the event</param>
        //
        //********************************************************************************
        private void ctlButton_Click(object sender, EventArgs e) {
            SetFilter();
        }
    } // class DynamicFormTableForFilter
} // Namespace