using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.IO;
using RepPP;
using RepPP.Toolkit; 
using RepPP.Toolkit.Web; 

namespace RepPP.Framework.Web
{
	/// <summary>
	/// HyperLinkFilter: Hyper link control to call the filter page
	/// </summary>
	public class HyperLinkFilter : RepPP.Toolkit.Web.AtmHyperLink {
        /// <summary>
        /// Define the display mode of the filter clause
        /// </summary>
        public enum FilterClauseDisplayModeE {
            /// <summary>Do not diplay the filter clause</summary>
            None    = 0,
            /// <summary>Diplay the filter clause as the text of the control</summary>
            Text    = 1,
            /// <summary>Diplay the filter clause as the title of the control</summary>
            Title   = 2
        }

        //********************************************************************************
        //
        /// <summary>
        /// FilterPage:       Page in charge of the filter data
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Description("Page to show on click event")]
        [Category("Behavior")]
        [DefaultValue("")]
        [Bindable(true)]
        public virtual string FilterPage {
            get {
                string  strRetVal;
                object  obj;

                obj = ViewState["FilterPage"];
                if (obj != null) {
                    strRetVal = (string)obj;
                } else {
                    strRetVal = String.Empty;
                }
                return(strRetVal);
            }
            set {
                ViewState["FilterPage"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FilterClauseDisplayMode:        Display the filter clause   
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(FilterClauseDisplayModeE.Title)]
        [Description("Indicates whether or not to display the filter clause")]
        public virtual FilterClauseDisplayModeE FilterClauseDisplayMode {
            get {
                FilterClauseDisplayModeE   eRetVal;
                object                      objAuto;
                
                objAuto = ViewState["FilterClauseDisplayMode"];
                if (objAuto != null) {
                    eRetVal = (FilterClauseDisplayModeE)objAuto;
                } else {
                    eRetVal = FilterClauseDisplayModeE.Title;
                }
                return(eRetVal);
            }
            set {
                ViewState["FilterClauseDisplayMode"] = FilterClauseDisplayModeE.Title;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetFilterClause:            Get the selection buffer filter clause
	    /// </summary>
	    /// <returns>
	    /// Filter clause
	    /// </returns>
	    //
        //********************************************************************************
        protected virtual string  GetFilterClause() {
            string          strRetVal = String.Empty;
            PageEditor      page;

            page = Page as PageEditor;
            if (page != null) {
                strRetVal = page.SelectionBuffer.FilterClause;
                if (strRetVal != String.Empty) {
                    strRetVal = strRetVal.Replace("WHERE ", "");
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:            Add javascript before rendering occurs
	    /// </summary>
	    /// <param name="e">        Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            StreamReader    reader;
            const string    strKey = "HyperLinkFilter";
            string          strFilterClause;

            if (FilterClauseDisplayMode == FilterClauseDisplayModeE.Text) {
                strFilterClause = GetFilterClause();
                if (strFilterClause != String.Empty) {
                    FormattedText = Text + ": " + strFilterClause;
               } else {
                    FormattedText = String.Empty;
                }
            }
            base.OnPreRender(e);
            if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                reader = new StreamReader(typeof(DataGridRowsetSelect).Assembly.GetManifestResourceStream("RepPP.Framework.Web.HyperLinkFilterScript.js"));
                WebUtil.RegisterClientScriptBlock(Page, ID, reader.ReadToEnd());
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddAttributesToRender:  Add the attributes to render
        /// </summary>
        /// <param name="writer">   Stream</param>
	    //
        //********************************************************************************
        protected override void AddAttributesToRender(HtmlTextWriter writer) {
            string strIsDirtyScript;
            string strFilterPage;
            string strOnClickCmd;
		    bool   bDesignMode;
		    ISite  site;
		        
		    site        = Site;
		    bDesignMode = (site == null) ? false : site.DesignMode;
            if (!bDesignMode) {
                strFilterPage    = FilterPage;
                strIsDirtyScript = (Page is PageEditor) ? (Page as PageEditor).CheckIsDirtyScript : string.Empty;
                if (strFilterPage != String.Empty) {
                    strOnClickCmd = strIsDirtyScript + " OpenWindow('" + strFilterPage + "', 'FilterWindow', true, 100, 100)";
                    writer.AddAttribute(HtmlTextWriterAttribute.Onclick, strOnClickCmd);
                    if (FilterClauseDisplayMode == FilterClauseDisplayModeE.Title) {
                        writer.AddAttribute(HtmlTextWriterAttribute.Title, GetFilterClause());
                    }
                }
            }
            base.AddAttributesToRender(writer);
        }
	} // HyperLinkFilter
} // Namesapce
