using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;

namespace RepPP.Framework.Web {
	/// <summary>
	/// Handle navigation between pages and transactions
	/// </summary>
	[Serializable]
	public class NavStack {
		/// <summary>a string literal to build the name of the session item that will hold the stack</summary>
		private const string        m_strStackSessionNamePrefix = "Stack_";
		/// <summary>The id of the stack</summary>
		private int                 m_iStackId;
		/// <summary>a continuously incrementing id used for ensuring the order of requests</summary>
		private int                 m_iOpSequenceId;
		/// <summary>The array holding entries and frames</summary>
		private Stack               m_stackEntries;
		/// <summary>Object handling session data</summary>
		[NonSerialized]
		private ISessionDataHolder  m_sessionDataHolder;
		/// <summary>Object handling cache data</summary>
		[NonSerialized]
		private ICacheDataHolder    m_cacheDataHolder;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor
        /// </summary>
        /// <param name="sessionDataHolder">    Session data holder</param>
        /// <param name="cacheDataHolder">      Cache data holder</param>
        /// <param name="iStackId">		        The Stack ID</param>
        //
        //********************************************************************************
		private NavStack(ISessionDataHolder sessionDataHolder, ICacheDataHolder cacheDataHolder, int iStackId) {
		    m_sessionDataHolder = sessionDataHolder;
		    m_cacheDataHolder   = cacheDataHolder;
			m_iStackId          = iStackId;
			m_iOpSequenceId     = 0;
			m_stackEntries      = new Stack();
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor. Use for serialization.
        /// </summary>
        //
        //********************************************************************************
		private NavStack() : this(null, null, -1) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Destroy:                            Destroy the stack and all its component removing it from session
        /// </summary>
        //
        //********************************************************************************
		public virtual void Destroy() {
		    foreach (NavEntry entry in m_stackEntries) {
		        entry.Destroy();
		    }
		    SessionDataHolder.DeleteFromSession(StackSessionName);
            m_stackEntries.Clear();
            m_stackEntries = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// Clear:                            Clears the stack without destroying it in the session
        /// </summary>
        //
        //********************************************************************************
		public virtual void Clear() {
		    foreach (NavEntry entry in m_stackEntries) {
		        entry.Destroy();
		    }
            m_stackEntries.Clear();
		}

        //********************************************************************************
        //
        /// <summary>
        /// SetFather:              Set the father for all include entries
        /// </summary>
        //
        //********************************************************************************
		private void SetFather() {
			foreach (NavEntry entry in m_stackEntries) {
				entry.SetFather(this);
			}
		}

		//********************************************************************************
        //
        /// <summary>
        /// SessionDataHolder:					Object handling the data hold in session
        /// </summary>
        //
        //********************************************************************************
		public virtual ISessionDataHolder SessionDataHolder {
			get {
				return(m_sessionDataHolder);
			}
			set {
			    m_sessionDataHolder = value;
			}
		}

		//********************************************************************************
        //
        /// <summary>
        /// CacheDataHolder:				    Object handling the data hold in cache
        /// </summary>
        //
        //********************************************************************************
		public virtual ICacheDataHolder CacheDataHolder {
			get {
				return(m_cacheDataHolder);
			}
			set {
			    m_cacheDataHolder = value;
			}
		}

		//********************************************************************************
        //
        /// <summary>
        /// StackId:					Gets the Id of the stack
        /// </summary>
        //
        //********************************************************************************
		public virtual int StackId {
			get {
				return(m_iStackId);
			}
		}

		//********************************************************************************
        //
        /// <summary>
        /// Count:					Gets the number of entries on the stack
        /// </summary>
        //
        //********************************************************************************
		public virtual int Count {
			get {
				return(m_stackEntries.Count);
			}
		}

		//********************************************************************************
        //
        /// <summary>
        /// OpSequenceId:			    Operation sequence Id to make sure that requests are received
        ///								in order.
        /// </summary>
        //
        //********************************************************************************
		public int OpSequenceId {
			get {
				return(m_iOpSequenceId);
			}
			set {
			    if (m_iOpSequenceId != value) {
				    m_iOpSequenceId = value;
				}
			}
		}
        //********************************************************************************
        //
        /// <summary>
        /// GetStackSessionName:			Gets the name of the session item that will hold
        ///									the stack in the session.
        /// </summary>
        //
        //********************************************************************************
		private static string GetStackSessionName(int iStackId) {
			return(m_strStackSessionNamePrefix + iStackId.ToString());
		}

		//********************************************************************************
        //
        /// <summary>
        /// StackSessionName:			Gets the name of the session item that will hold 
        ///								a serialized version of the stack in the session
        /// </summary>
        //
        //********************************************************************************
		protected virtual string StackSessionName {
			get {
				return(GetStackSessionName(StackId));
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// Create:					            Creates and returns a stack with a unique stack id
        /// </summary>
        /// <param name="sessionDataHolder">    Object handling session data</param>
        /// <param name="cacheDataHolder">      Object handling cache data</param>
        /// <returns>
        /// The newly created stack object
        /// </returns>
        //
        //********************************************************************************
		public static NavStack Create(ISessionDataHolder sessionDataHolder, ICacheDataHolder cacheDataHolder) {
            NavStack    stackRetVal;
            int         iStackId;
			
    	    iStackId    = sessionDataHolder.GetUniqueId();
			stackRetVal = new NavStack(sessionDataHolder, cacheDataHolder, iStackId);
            sessionDataHolder.SaveToSession(stackRetVal.StackSessionName, stackRetVal);
            return(stackRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// CreateFromSession			        Loads the stack having the specified stack id from
        ///								        the session.
        /// </summary>
        /// <param name="sessionDataHolder">    Object handling session data</param>
        /// <param name="cacheDataHolder">      Object handling cache data</param>
        /// <param name="iStackId">		        The id of the stack to loaded from the session</param>
        /// <returns>
        /// The stack object or null if no stack with the specified id is found
        /// </returns>
        //
        //********************************************************************************
		public static NavStack CreateFromSession(ISessionDataHolder sessionDataHolder, ICacheDataHolder cacheDataHolder, int iStackId) {
			NavStack    stackRetVal;
			string      strObjName;

            strObjName = GetStackSessionName(iStackId);
            if (sessionDataHolder.IsExistInSession(strObjName)) {
			    stackRetVal = sessionDataHolder.LoadFromSession(strObjName) as NavStack;
			    stackRetVal.SetFather();
			    stackRetVal.SessionDataHolder = sessionDataHolder;
			    stackRetVal.CacheDataHolder   = cacheDataHolder;
			} else {
			    stackRetVal = null;
			}
			return(stackRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// FindObj:                            Finds an object in the stack using its name
        /// </summary>
        /// <param name="strName">              Name of the object</param>
        /// <returns>
        /// Object or null if not found
        /// </returns>
        //
        //********************************************************************************
		public NavEntryObject FindObj(string strName) {
		    NavEntryObject  objRetVal = null;
		    
		    foreach (NavEntry entry in m_stackEntries) {
		        objRetVal = entry[strName];
		        if (objRetVal != null) {
		            break;
		        }
		    }
		    return(objRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// PushEntry:                          Push an entry in the stack
        /// </summary>
        /// <param name="entry">                Entry to push in the stack</param>
        //
        //********************************************************************************
		public virtual void PushEntry(NavEntry entry) {
		    m_stackEntries.Push(entry);
		}

        //********************************************************************************
        //
        /// <summary>
        /// DeleteTopEntry:                     Deletes the top entry of the specified type
        /// </summary>
        /// <param name="type">                 Type of the entry</param>
        //
        //********************************************************************************
		public virtual void DeleteTopEntry(Type type) {
		    NavEntry        entry = null;
            bool            bEnd;

		    
		    bEnd = false;
		    while (m_stackEntries.Count != 0 && !bEnd) {
		        entry   = m_stackEntries.Pop() as NavEntry;
		        bEnd    = (entry.GetType() == type);
		        entry.Destroy();
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// DeleteTopEntry:                     Deletes the top entry.
        /// </summary>
        //
        //********************************************************************************
		public virtual void DeleteTopEntry() {
		    NavEntry  entry;
		    
		    entry = m_stackEntries.Pop() as NavEntry;
		    if (entry != null) {
		        entry.Destroy();
    		}
		}

        //********************************************************************************
        //
        /// <summary>
        /// DeleteToTopEntry:					Deletes all entries up to the specified entry type.
        /// </summary>
        /// <param name="type">                 Type of the entry</param>
        //
        //********************************************************************************
		public virtual void DeleteToTopEntry(Type type) {
		    NavEntry        entry = null;
		    bool            bEnd;
		    
		    bEnd = false;
		    while (m_stackEntries.Count != 0 && !bEnd) {
		        entry = m_stackEntries.Peek() as NavEntry;
		        if (entry.GetType() == type) {
		            bEnd = true;
		        } else {
		            entry = m_stackEntries.Pop() as NavEntry;
		            entry.Destroy();
		        }
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// GetCurrentEntry:                    Gets the current entry of the specified type
        /// </summary>
        /// <param name="type">                 Type of the entry</param>
        //
        //********************************************************************************
		public virtual NavEntry GetCurrentEntry(Type type) {
		    NavEntry    entryRetVal = null;
		    
		    foreach (NavEntry entry in m_stackEntries) {
		        if (entry.GetType() == type) {
		            entryRetVal = entry;
		            break;
		        }
		    }
		    return(entryRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// GetCurrentEntry:                    Gets the current entry of the specified type
        /// </summary>
        //
        //********************************************************************************
		public virtual NavEntry GetCurrentEntry() {
		    NavEntry    entryRetVal = null;
		    
            if (m_stackEntries.Count > 0) {
                entryRetVal = m_stackEntries.Peek() as NavEntry;
            }
            return(entryRetVal);
		}
	} // Class NavStack
} // Namespace
