using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {
    /// <summary>Base class use to implement all editors</summary>
    public class PageEditor : PageWithSelectionBuffer {

        /// <summary>Determine the caching mode of the transaction</summary>
        public enum TransactionCacheModeE {
            /// <summary>Cache the data and commit the update one line at the time</summary>
            CacheAndCommit  =   0,
            /// <summary>Cache the data and update only when explicitly asked</summary>
            CacheAndDelay   =   1
        };

        /// <summary>Operation which can be done on the navigation stack</summary>
        public enum NavStackOpE {
            /// <summary>Do nothing</summary>
            NONE         = 0,
            /// <summary>Creates a new stack</summary>
            NEW          = 1,
            /// <summary>Push a transaction</summary>
            PUSHTRANS    = 2,
            /// <summary>Push a page</summary>
            PUSHPAGE     = 3,
            /// <summary>Go to top of current transaction</summary>
            TOPTRANS     = 4,
            /// <summary>Pop an entry</summary>
            POP          = 5,
            /// <summary>Pop a transaction</summary>
            POPTRANS     = 6
            
        };

        /// <summary>Position in the current list of items</summary>
        public enum PositionE {
            /// <summary>First item</summary>
            FIRST           = 0,
            /// <summary>Previous item</summary>
            PREVIOUS        = 1,
            /// <summary>Next item</summary>
            NEXT            = 2,
            /// <summary>Last item</summary>
            LAST            = 3
        };

        /// <summary>Position in the current hierarch of editors and transations</summary>
        public enum UpPositionE {
            /// <summary>Go up to the previous editor</summary>
            PREVIOUSEDITOR              = 0,
            /// <summary>Go up to the top of the current transaction</summary>
            TOPOFTRANSACTION            = 1,
            /// <summary>Go up to the previous transaction</summary>
            PREVIOUSTRANSACTION         = 2,
        };

        /// <summary>Script that verifies the is dirty</summary>
        private const string                m_strCheckIsDirtyScript             = "{if (!PageEditor_CheckIsDirty()) return false;} ";
        /// <summary>Script that asks for confirmation before a delete </summary>
        private const string                m_strConfirmDeleteScript            = "{if (!PageEditor_ConfirmDelete()) return false;} ";
        /// <summary>Name of the hidden field containing the command to execute</summary>
        public const string                 REPPP_CMD_FIELD_NAME                = "_RepPPCmd";
        /// <summary>Name of the hidden field containing the card dirty state</summary>
        public const string                 REPPP_ISROWSETTREEDIRTY_FIELD_NAME  = "_RepPPIsRowsetTreeDirty";
        /// <summary>Name of the transaction rowset tree object name kept in the transaction entry object list</summary>
        public const string                 REPPP_TRANSACTION_OBJECT_NAME       = "_RepPPTransactionRowsetTree";

        /// <summary>Event sent before data validation</summary>
        public event CancelEventHandler     BeforeValidate;
        /// <summary>Event sent after data validation</summary>
        public event CancelEventHandler     AfterValidate;
        /// <summary>Event sent before an item is duplicated</summary>
        public event CancelEventHandler     BeforeDuplicateItem;
        /// <summary>Event sent after an item is duplicated</summary>
        public event EventHandler           AfterDuplicateItem;
        /// <summary>Event sent before an item is created</summary>
        public event CancelEventHandler     BeforeNewItem;
        /// <summary>Event sent after an item is created</summary>
        public event EventHandler           AfterNewItem;
        /// <summary>Event sent before an item is deleted</summary>
        public event CancelEventHandler     BeforeDeleteItem;
        /// <summary>Event send after an item is deleted</summary>
        public event EventHandler           AfterDeleteItem;
        /// <summary>Event sent before the transaction data is read from the database</summary>
        public event CancelEventHandler     BeforeLoadFromDB;
        /// <summary>Event sent after the transaction data is read from the database</summary>
        public event CancelEventHandler     AfterLoadFromDB;
        /// <summary>Event sent before the transaction data is saved in the database</summary>
        public event CancelEventHandler     BeforeSaveToDB;
        /// <summary>Event sent after the transaction data is saved in the database</summary>
        public event CancelEventHandler     AfterSaveToDB;
        
        /// <summary>Standard command handler</summary>
        private StdCmdHandler               m_stdCmdHandler;
        /// <summary>Array of commands to execute</summary>
        protected Cmd[]                     m_arrCmds;
        /// <summary>Current command to execute</summary>
        protected int                       m_iCurrentCmdIndex;
        /// <summary>Name of the rowset tree use by the transaction</summary>
        private string                      m_strTransRowsetTreeName;
        /// <summary>Name of the rowset tree node use by the transaction</summary>
        private string                      m_strTransNodeName;
        /// <summary>Rowset tree holding the transaction</summary>
        private RowsetTree                  m_rowsetTreeTransaction;
        /// <summary>Original line state</summary>
        private LineState                   m_eOriginalLineState;
        /// <summary>Indicates if the page is valid</summary>
        private bool                        m_bIsPageValid;
        /// <summary>Cache mode</summary>
        private TransactionCacheModeE       m_eCacheMode;
        /// <summary>Array of save command handler</summary>
        private ArrayList                   m_arrSaveHandler;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor for the root transaction
        /// </summary>
        /// <param name="strTransRowsetTreeName">   Transaction name</param>
        /// <param name="strTransNodeName">         Transaction node name</param>
        /// <param name="eCacheMode">               Cache mode use for the transaction</param>
        /// <param name="strSBRowsetTreeName">      Name of the rowset tree definiton use for the selection buffer</param>
        /// <param name="strSBFilterClause">        Selection buffer filter clause</param>
        /// <param name="eSBCacheMode">             Selection buffer cache mode</param>
        //
        //********************************************************************************
        protected PageEditor(string                 strTransRowsetTreeName, 
                             string                 strTransNodeName, 
                             TransactionCacheModeE  eCacheMode, 
                             string                 strSBRowsetTreeName, 
                             string                 strSBFilterClause, 
                             SBCacheModeE           eSBCacheMode): base(strSBRowsetTreeName, 
                                                                        strSBFilterClause, 
                                                                        eSBCacheMode) {
            if (strTransRowsetTreeName != String.Empty && String.Compare(strTransRowsetTreeName, strSBRowsetTreeName, true) == 0 ) {
                throw new FWException(FWException.EditorExceptionE.BadParameter, 
                                      "The rowset tree use for the transaction and for the selection buffer must not be the same");
            }
            m_strTransRowsetTreeName    = strTransRowsetTreeName;
            m_strTransNodeName          = strTransNodeName;
            m_rowsetTreeTransaction     = null;
            m_eOriginalLineState        = LineState.sdLineOld;
            m_eCacheMode                = eCacheMode;
            m_bIsPageValid              = true;
            m_arrSaveHandler            = new ArrayList();
            m_arrCmds                   = null;
            m_iCurrentCmdIndex          = -1;
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor for the root transaction
        /// </summary>
        /// <param name="strTransRowsetTreeName">   Transaction name</param>
        /// <param name="strTransNodeName">         Transaction node name</param>
        /// <param name="eCacheMode">               Cache mode use for the transaction</param>
        /// <param name="strSBRowsetTreeName">      Name of the rowset tree use for the selection buffer</param>
        /// <param name="strSBFilterClause">        Selection buffer filter clause</param>
        //
        //********************************************************************************
        protected PageEditor(string                 strTransRowsetTreeName, 
                             string                 strTransNodeName, 
                             TransactionCacheModeE  eCacheMode, 
                             string                 strSBRowsetTreeName, 
                             string                 strSBFilterClause): this(strTransRowsetTreeName, 
                                                                             strTransNodeName, 
                                                                             eCacheMode, 
                                                                             strSBRowsetTreeName, 
                                                                             strSBFilterClause, 
                                                                             SBCacheModeE.NonPermanentCache) {
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor for the child transaction
        /// </summary>
        /// <param name="strTransRowsetTreeName">   Name of the rowset tree use for the transaction</param>
        /// <param name="strTransNodeName">         Name of the node use for this page</param>
        //
        //********************************************************************************
        protected PageEditor(string strTransRowsetTreeName, 
                             string strTransNodeName) : this(strTransRowsetTreeName, 
                                                             strTransNodeName, 
                                                             TransactionCacheModeE.CacheAndCommit, 
                                                             String.Empty, 
                                                             String.Empty) {
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public PageEditor() : this("", "", TransactionCacheModeE.CacheAndCommit, "", "") {
            if (!RepPP.Toolkit.Web.WebUtil.IsWebDesignMode()) {
                throw new FWException(FWException.EditorExceptionE.MissingParam, "Transaction Rowset Tree Name & Node Name");
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DisposeCtl:                         Dispose a control
        /// </summary>
        /// <param name="ctl">                  Control to dispose</param>
        //
        //********************************************************************************
        private void DisposeCtl(Control ctl) {
            foreach (Control ctlChild in ctl.Controls) {
                DisposeCtl(ctlChild);
            }
            ctl.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FlushContent:                       Remove gracefully the content of the page
        /// </summary>
        //
        //********************************************************************************
        public override void FlushContent() {
            if (m_rowsetTreeTransaction != null) {
                m_rowsetTreeTransaction.Dispose();
                m_rowsetTreeTransaction = null;
            }
            base.FlushContent();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// StandardCommandHandler: Standard command handler
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
	    public virtual StdCmdHandler StandardCommandHandler {
	        get {
	            if (m_stdCmdHandler == null) {
	                m_stdCmdHandler = new StdCmdHandler(this);
	            }
	            return(m_stdCmdHandler);
	        }
	    }

        //********************************************************************************
        //
        /// <summary>
        /// CurrentCommand:                         Command currently executed
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Cmd CurrentCommand {
            get {
                Cmd cmdRetVal = null;

                if (m_iCurrentCmdIndex < m_arrCmds.Length) {
                    cmdRetVal = m_arrCmds[m_iCurrentCmdIndex];
                }
                return(cmdRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CurrentTransactionEntry:            Returns the current transaction entry
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual NavEntryTransaction CurrentTransactionEntry {
            get {
                NavStack                navStack;
                NavEntryTransaction     trans;
                
                navStack = NavigationStack;
                trans    = navStack.GetCurrentEntry(typeof(NavEntryTransaction)) as NavEntryTransaction;
                return(trans);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionRowsetTreeName:                Returns the transaction rowset tree name
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string TransactionRowsetTreeName {
            get {
                return(m_strTransRowsetTreeName);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionNodeName:           Returns the transaction node name
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string TransactionNodeName {
            get {
                return(m_strTransNodeName);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRootNode:                    Returns if we're the root node of the transaction
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool IsRootNode {
            get {
                RowsetTreeDefNode   node;
                
                node = TransactionNode;
                return(node.RowsetTreeDef.Nodes[0] == node);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsValid:            Replace the page validation property which cannot be called if the
        ///                     validate method has not been called.
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public new bool IsValid {
            get {
                return(m_bIsPageValid);
            }
            set {
                m_bIsPageValid = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionCacheMode:               Mode use to cache the transaction data
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual TransactionCacheModeE TransactionCacheMode {
            get {
                TransactionCacheModeE   eRetVal;
                NavEntryTransaction     trans;
                
                if (IsRootNode) {
                    eRetVal = m_eCacheMode;
                } else {
                    trans = CurrentTransactionEntry;
                    if (trans == null) {
                        throw new FWException(FWException.EditorExceptionE.CannotFindTransactionEntry);
                    }
                    eRetVal = trans.TransactionCacheMode;
                }
                return(eRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionRowsetTreeDef:   Returns the transaction rowset tree definition
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDef TransactionRowsetTreeDef {
            get {
                RepPP.Application   app;
                
                app = RepApplication;
                return(app.RowsetTreeDefs[TransactionRowsetTreeName]);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionNode:           Returns the transaction node
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNode TransactionNode {
            get {
                RowsetTreeDef   rowsetTreeDef;
                
                rowsetTreeDef = TransactionRowsetTreeDef;
                return(rowsetTreeDef.FindNode(TransactionNodeName));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionRowsetTree:            Returns the transaction rowset tree
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTree TransactionRowsetTree {
            get {
                return(m_rowsetTreeTransaction);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TransactionRowset:           Returns the transaction rowset
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Rowset TransactionRowset {
            get {
                RowsetTreeDefNode node;
                int       iRetCode;

                node = TransactionNode;
                if (node.DelayReading && node.SelectSqlCommand == String.Empty) {
                    iRetCode = node.RowsetTreeDef.BuildSqlCommand();
                    if (iRetCode != 0) {
                        throw new FWException(FWException.EditorExceptionE.CannotBuildSqlCmds, 
                                              " Faulty RowsetTree: " + node.RowsetTreeDef.Name +
                                              " Return Code:       " + iRetCode);
                    }
                }
                return(m_rowsetTreeTransaction.GetCurrentRowset(TransactionNode));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CheckIsDirtyScript:           Script that verifies the is dirty
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        internal string CheckIsDirtyScript {
            get {
                return(m_strCheckIsDirtyScript);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ConfirmDeleteScript:            Script that asks for confirmation before a delete
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        internal string ConfirmDeleteScript {
            get {
                return(m_strConfirmDeleteScript);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// DefaultCommand:              Gets the default command of the editor when 
	    ///                              no command is specified in the query string  
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        protected virtual Cmd DefaultCommand {
            get {
                Cmd cmdRetVal;

                cmdRetVal = new Cmd();
                if (!IsPostBack && (PreviousPage == null || !(PreviousPage is PageWithStack))) {
                    cmdRetVal.StackOp = NavStackOpE.PUSHTRANS;
                    cmdRetVal.Name    = "EDIT";
                    cmdRetVal.Params  = new string[1]{"FIRST"};
                }
                return(cmdRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeValidate:              Called before the validation
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnBeforeValidate(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeValidate(this);
                }
                if (!e.Cancel && BeforeValidate != null) {
                    BeforeValidate(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterValidate:               Called after the validation
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnAfterValidate(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.AfterValidate(this);
                }
                if (!e.Cancel && AfterValidate != null) {
                    AfterValidate(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeDuplicateItem:         Called before an item is duplicated
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnBeforeDuplicateItem(CancelEventArgs e){
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeDuplicateItem(this);
                }
                if (!e.Cancel && BeforeDuplicateItem != null) {
                    BeforeDuplicateItem(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterDuplicateItem:          Called after an item is duplicated 
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnAfterDuplicateItem(EventArgs e) {
            HttpApplicationFW appFW;

            appFW = Context.ApplicationInstance as HttpApplicationFW;
            if (appFW != null) {
                appFW.AfterDuplicateItem(this);
            }
            if (AfterDuplicateItem != null) {
                AfterDuplicateItem(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeNewItem:               Called before an new item is created
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnBeforeNewItem(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeNewItem(this);
                }
                if (!e.Cancel && BeforeNewItem != null) {
                    BeforeNewItem(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterNewItem:                Called after a new item is created
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnAfterNewItem(EventArgs e) {
            HttpApplicationFW appFW;

            appFW = Context.ApplicationInstance as HttpApplicationFW;
            if (appFW != null) {
                appFW.AfterNewItem(this);
            }
            if (AfterNewItem != null) {
                AfterNewItem(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeDeleteItem:            Called before an item is deleted 
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnBeforeDeleteItem(CancelEventArgs e){
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeDeleteItem(this);
                }
                if (!e.Cancel && BeforeDeleteItem != null) {
                    BeforeDeleteItem(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterDeleteItem:             Called after an item is deleted
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnAfterDeleteItem(EventArgs e) {
            HttpApplicationFW appFW;

            appFW = Context.ApplicationInstance as HttpApplicationFW;
            if (appFW != null) {
                appFW.AfterDeleteItem(this);
            }
            if (AfterDeleteItem != null) {
                AfterDeleteItem(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeLoadFromDB:            Called before the transaction data is read 
        ///                                 from the database 
        /// </summary>
        /// <param name="e">                Event arguments</param>
        /// <returns>
        /// False if the user cancelled the event, True otherwise
        /// </returns>
        //
        //********************************************************************************
        public virtual bool OnBeforeLoadFromDB(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeLoadFromDB(this);
                }
                if (!e.Cancel && BeforeLoadFromDB != null){
                    BeforeLoadFromDB(this, e);
                }
            }
            return (!e.Cancel);
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterLoadFromDB:             Called after the transaction data is read 
        ///                                 from the database
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        public virtual void OnAfterLoadFromDB(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.AfterLoadFromDB(this);
                }
                if (!e.Cancel && AfterLoadFromDB != null) {
                    AfterLoadFromDB(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnBeforeSaveToDB:              Called before the transaction data is saved
        ///                                 in the database
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        public virtual void OnBeforeSaveToDB(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.BeforeSaveToDB(this);
                }
                if (!e.Cancel && BeforeSaveToDB != null) {
                    BeforeSaveToDB(this, e);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnAfterSaveToDB:               Called after the transaction data is saved 
        ///                                 in the database 
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        public virtual void OnAfterSaveToDB(CancelEventArgs e) {
            HttpApplicationFW appFW;

            if (!e.Cancel) {
                appFW = Context.ApplicationInstance as HttpApplicationFW;
                if (appFW != null) {
                    e.Cancel = !appFW.AfterSaveToDB(this);
                }
                if (!e.Cancel && AfterSaveToDB != null) {
                    AfterSaveToDB(this, e);
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// RegisterIsDirtyControl:             Makes a control check the IsDirty state of
	    ///                                     the editor before triggerring a postback
	    /// </summary>
	    /// <param name="ctl">                  Web Control to register</param>
	    //
        //********************************************************************************
        public virtual void RegisterIsDirtyControl(WebControl ctl) {
            string  strOriginalOnClick;

            if (ctl != null) {
                strOriginalOnClick = ctl.Attributes["onclick"];
                if (strOriginalOnClick == null || strOriginalOnClick == String.Empty) {
                    ctl.Attributes["onclick"] = CheckIsDirtyScript;
                } else if (!strOriginalOnClick.StartsWith(CheckIsDirtyScript)) {
                    ctl.Attributes["onclick"] = CheckIsDirtyScript + strOriginalOnClick;
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// RegisterSaveStateHandler:       Register a save command handler
	    /// </summary>
	    /// <param name="strID">            Control ID</param>
	    //
        //********************************************************************************
        public void RegisterSaveCmdHandler(string strID) {
            if (m_arrSaveHandler != null) {
                m_arrSaveHandler.Add(strID);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// UpdateSelectionBuffer:          Updates the selection buffer data in the cache 
        ///                                 using the transaction data and state
        /// </summary>
        /// <param name="arrKey">           Key of the item that changed</param>
        /// <param name="stateOriginal">    LineState of the Rowset before it is saved</param>
        /// <param name="rowsetTrans">      Transaction data used to update the selection buffer</param>
        /// <returns>
        /// True if successful, False otherwise
        /// </returns>
        //
        //********************************************************************************
        public virtual bool UpdateSelectionBuffer(string[] arrKey, LineState stateOriginal, Rowset rowsetTrans) {
            bool bRetVal = true;

            if (SelectionBuffer == null) {
                bRetVal = false;
            } else if (stateOriginal == LineState.sdLineDeleted) {
                bRetVal = SelectionBuffer.RemoveItem(arrKey);
            } else if (stateOriginal == LineState.sdLineNew) {
                if ( SBCacheMode == SBCacheModeE.NoCache) {
                    SetSBCurrentSelection();
                } else {
                    SelectionBuffer.AddItem(rowsetTrans);
                }
            } else {
                bRetVal = SelectionBuffer.UpdateItem(rowsetTrans);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveToCache:                    Save the current item to the cache
        /// </summary>
        //
        //********************************************************************************
        public virtual void SaveToCache() {
            NavEntryTransaction     entryTrans;
            NavEntryObjectInstance  objTrans;
            
            entryTrans  = CurrentTransactionEntry;
            if (entryTrans == null) {
                throw new FWException(FWException.EditorExceptionE.CannotFindTransactionEntry);
            }
            objTrans = entryTrans[REPPP_TRANSACTION_OBJECT_NAME] as NavEntryObjectInstance;
            if (objTrans == null) {
                objTrans = new NavEntryObjectInstance(entryTrans, REPPP_TRANSACTION_OBJECT_NAME);
                entryTrans[REPPP_TRANSACTION_OBJECT_NAME] = objTrans;
            }
            objTrans.SaveToSession(TransactionRowsetTree);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveToDB:                   Save the current item to the database
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public virtual bool SaveToDB() {
            bool            bRetVal;
            RowsetTree    rowsetTreeTrans;
            Rowset   rowsetTrans;
            int             iErrCode;
            string          strError;
            string[]        arrKey;
            LineState       stateOriginal;
            
            rowsetTreeTrans = TransactionRowsetTree;
            if (rowsetTreeTrans.IsModified) {
                rowsetTrans = rowsetTreeTrans.RootRowset;
                arrKey        = WebUtil.BuildPrimaryKey(rowsetTrans.RowsetDef, true);
                stateOriginal = rowsetTrans.LineState;
                iErrCode      = DataAccess.SaveTransactionData(rowsetTreeTrans, out strError);
                if (iErrCode == 0) {
                    UpdateSelectionBuffer(arrKey, stateOriginal, rowsetTrans);
                    bRetVal = true;
                } else {
                    bRetVal = false;
                    if (iErrCode != Int32.MinValue) {
                        strError = GetErrorMessage(iErrCode, strError);
                        DisplayErrorInValidationSummary(iErrCode, strError);
                    }
                }
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditRootItem:                       Edit a root item
        /// </summary>
        /// <param name="arrKey">               Key of the item</param>
        /// <param name="bSaveToCache">         Save to cache if true</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        private bool EditRootItem(string[] arrKey, bool bSaveToCache) {
            bool            bRetVal;
            RowsetTree    rowsetTreeTrans;
            int             iErrCode;
            string          strMsg;
            
            rowsetTreeTrans  = TransactionRowsetTree;
            iErrCode         = DataAccess.LoadTransactionData(rowsetTreeTrans, arrKey);
		    if (iErrCode == 0) {
                if ((TransactionRowset.LineCount == 0) ||
                    (SelectionBuffer != null && SelectionBuffer.SelectItem(arrKey, true) == false)) {
                    throw new FWException(FWException.EditorExceptionE.KeyNotFound);
                }                
    		    FWTrace.WriteTrace(String.Format("Read rowset tree {0} from the database", TransactionRowsetTreeName));
    		    bRetVal = true;
		    } else {
		        rowsetTreeTrans.Empty();
		        bRetVal = false;
		        if (iErrCode == int.MinValue) {
    		        FWTrace.WriteTrace(String.Format("User cancelled the Pre/Post Read rowset tree {0} from the database", TransactionRowsetTreeName));
		        } else {
		            strMsg =string.Format("Error reading the rowset tree {0} from database. Code = {1}",
		                                  TransactionRowsetTreeName,
		                                  iErrCode.ToString());
                    if (RepApplication.IsEventLogged) {
                        throw new ApplicationHolder.ErrorInEventViewerException(strMsg);
                    } else {
                        throw new ApplicationException(strMsg);
                    }
		        }
            }
            if (bSaveToCache) {
                SaveToCache();
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditRootItem:                       Edit a root item
        /// </summary>
        /// <param name="strKey">               Key of the item</param>
        /// <param name="bSaveToCache">         Save to cache if true</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        private bool EditRootItem(string strKey, bool bSaveToCache) {
            bool            bRetVal;
            string[]        arrKey;
            
            arrKey  = Utility.DecodeKey(strKey);
            bRetVal = EditRootItem(arrKey, bSaveToCache);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditCurrent:                        Edit the currently selected item
        /// </summary>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool EditCurrent() {
            SaveToCache();
            return(true);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditItem:                           Edit an item
        /// </summary>
        /// <param name="arrKey">               Key of the item</param>
        /// <param name="bSaveToCache">         Save root item data to cache if true</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool EditItem(string[] arrKey, bool bSaveToCache) {
            bool            bRetVal;
            Rowset   rowsetTrans;
            
            if (IsRootNode) {
                bRetVal = EditRootItem(arrKey, bSaveToCache);
            } else {
                rowsetTrans  = TransactionRowset;
                if (rowsetTrans.FindUsingPrimaryKey(0, true, true, true, arrKey) == 0) {
                    bRetVal = EditCurrent();
                } else {
                    throw new FWException(FWException.EditorExceptionE.KeyNotFound);
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditItem:                           Edit an item
        /// </summary>
        /// <param name="strKey">               Key of the item</param>
        /// <param name="bSaveToCache">         Save root item data to cache if true</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public bool EditItem(string strKey, bool bSaveToCache) {
            bool            bRetVal;
            string[]        arrKey;
            
            arrKey  = Utility.DecodeKey(strKey);
            bRetVal = EditItem(arrKey, bSaveToCache);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditItem:                           Edit an item
        /// </summary>
        /// <param name="strKey">               Key of the item</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool EditItem(string strKey) {
            return(EditItem(strKey, true));
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditPosition:                       Edit the item at the specified position
        /// </summary>
        /// <param name="ePos">                 Position to edit</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool EditPosition(PositionE ePos) {
            bool                    bRetVal;
            string[]                arrKeys;
            Rowset                  rowsetTrans;
            int                     iCurLine;
            int                     iLineCount;
            SelectionBufferHolder   selBuf;
            bool                    bNonSelected = false;
            
            rowsetTrans = TransactionRowset;
            if (IsRootNode) {
                selBuf = SelectionBuffer;
                if (selBuf == null) {
                    throw new FWException(FWException.EditorExceptionE.CannotFindSB);
                } else if (selBuf.ItemCount == 0) {
                    bRetVal = NewItem();
                } else {
                    if (ePos == PositionE.NEXT || ePos == PositionE.PREVIOUS) {
                        if (!selBuf.SelectItem(RepPP.Toolkit.Util.BuildPrimaryKey(rowsetTrans.RowsetDef, true), false)) {
                            // If a new item has not been saved and we move from it, it will not be found!
                            selBuf.SelectedIndex = selBuf.ItemCount - 1;
                            bNonSelected         = true;
                        }
                    }
                    switch(ePos){
                    case PositionE.FIRST:
                        arrKeys = selBuf.SelectFirstItem();
                        break;
                    case PositionE.PREVIOUS:
                        if (bNonSelected) {
                            arrKeys = selBuf.SelectedKey;
                        } else {
                            arrKeys = selBuf.SelectPreviousItem();
                        }
                        break;
                    case PositionE.NEXT:
                        arrKeys = selBuf.SelectNextItem();
                        break;
                    case PositionE.LAST:
                        arrKeys = selBuf.SelectLastItem();
                        break;
                    default:
                        throw new FWException(FWException.EditorExceptionE.InvalidPosition);
                    }
                    if (arrKeys == null) {
                        throw new FWException(FWException.EditorExceptionE.DevError);
                    }
                    bRetVal = EditRootItem(arrKeys, true);
                }
            } else {
                iCurLine    = rowsetTrans.CurrentUndelLine;
                iLineCount  = rowsetTrans.UndelLineCount;
                if (iLineCount == 0) {
                    throw new FWException(FWException.EditorExceptionE.CannotNavigateEmptyRowset);
                }
                switch(ePos) {
                case PositionE.FIRST:
                    rowsetTrans.SelectUndelLine(0);
                    break;
                case PositionE.PREVIOUS:
                    iCurLine--;
                    if (iCurLine >= 0) {
                        rowsetTrans.SelectUndelLine(iCurLine);
                    }
                    break;
                case PositionE.NEXT:
                     iCurLine++;
                     if (iCurLine < iLineCount) {
                        rowsetTrans.SelectUndelLine(iCurLine);
                     }
                    break;
                case PositionE.LAST:
                    rowsetTrans.SelectUndelLine(iLineCount - 1);
                    break;
                default:
                    throw new FWException(FWException.EditorExceptionE.InvalidPosition);
                }
                bRetVal = EditCurrent();
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// NewItem:                            Create an item
        /// </summary>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool NewItem() {
            Rowset          rowsetTrans;
            CancelEventArgs eventArg;
            
            eventArg = new CancelEventArgs(false);
            OnBeforeNewItem(eventArg);
            if (!eventArg.Cancel) {
                rowsetTrans = TransactionRowset;
                if (IsRootNode) {
                    rowsetTrans.Empty();
                }
                rowsetTrans.AddBlankLine(true);
                OnAfterNewItem(EventArgs.Empty);
            }
            SaveToCache();
            return(true);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteCurrent:                      Deletes current item
        /// </summary>
        //
        //********************************************************************************
        public virtual bool DeleteCurrent() {
            Rowset          rowsetTrans;
            CancelEventArgs eventArg;
            LineState       eLineState;
            int             iCurrentLine;
            int             iTry;
            int             iLineCount;
            string[]        arrKey;
            bool            bRetVal     = true;
            
            eventArg = new CancelEventArgs(false);
            OnBeforeDeleteItem(eventArg);
            if (!eventArg.Cancel) {
                rowsetTrans = TransactionRowset;
                if (SelectionBuffer != null) {
                    SelectionBuffer.SelectItem(WebUtil.BuildPrimaryKey(rowsetTrans.RowsetDef, true), false);
                }
                eLineState = rowsetTrans.LineState;
                rowsetTrans.DeleteLine();
                if (IsRootNode || TransactionCacheMode == TransactionCacheModeE.CacheAndCommit) {
                    bRetVal = SaveToDB();
                }
                if (bRetVal) {
                    OnAfterDeleteItem(EventArgs.Empty);
                    if (IsRootNode) {
                        if (SelectionBuffer != null) {
                            iCurrentLine    = SelectionBuffer.SelectedIndex;
                            arrKey          = (iCurrentLine == -1) ? null : SelectionBuffer.SelectedKey;
                        } else {
                            iCurrentLine    = -1;
                            arrKey          = null;
                        }
                    } else {
                        iCurrentLine = rowsetTrans.CurrentLine;
                        if (iCurrentLine != -1 && rowsetTrans.LineState == LineState.sdLineDeleted) {
                            if (rowsetTrans.UndelLineCount == 0) {
                                iCurrentLine = -1;
                            } else {
                                iLineCount = rowsetTrans.LineCount;
                                iTry = iCurrentLine + 1;
                                while (iTry < iLineCount && rowsetTrans.ThisLineState(iTry) == LineState.sdLineDeleted) {
                                    iTry++;
                                }
                                if (iTry >= iLineCount) {
                                    iTry = iCurrentLine - 1;
                                    while (iTry >= 0 && rowsetTrans.ThisLineState(iTry) == LineState.sdLineDeleted) {
                                        iTry--;
                                    }
                                }
                                iCurrentLine = iTry;
                            }
                        }
                        if (iCurrentLine == -1) {
                            arrKey = null;
                        } else {
                            rowsetTrans.SelectLine(iCurrentLine);
                            arrKey = WebUtil.BuildPrimaryKey(rowsetTrans.RowsetDef, true);
                        }
                    }
                    if (iCurrentLine != -1) {
                        EditItem(arrKey, true);
                    } else if (IsRootNode) {
                        NewItem();
                    } else {
                        SaveToCache();
                        AppendCommand(new Cmd());
                        GoUp(UpPositionE.PREVIOUSEDITOR);  // Does no return
                    }
                } else {
                    rowsetTrans.LineState = eLineState;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateCurrent:                   Duplicates current item
        /// </summary>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool DuplicateCurrent() {
            Rowset   rowsetTrans;
            CancelEventArgs eventArg;

            //TODO: Autonum which are null???
            eventArg = new CancelEventArgs(false);
            OnBeforeDuplicateItem(eventArg);
            if (!eventArg.Cancel) {
                rowsetTrans = TransactionRowset;
                if (IsRootNode) {
                    rowsetTrans.LineState = LineState.sdLineNew;
                } else {
                    rowsetTrans.DuplicateLine();
                }
                OnAfterDuplicateItem(EventArgs.Empty);
            }
            SaveToCache();
            return(true);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateItem:                      Duplicates the specified item
        /// </summary>
        /// <param name="strKey">               Key of the item to duplicate</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool DuplicateItem(string strKey) {
            bool    bRetVal;
            Rowset  rowsetTrans;
            
            bRetVal     = EditItem(strKey, false);
            rowsetTrans = TransactionRowset;
            if (bRetVal) {
                bRetVal = DuplicateCurrent();
            } else {
                SaveToCache();
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveCurrent:                        Save the current item
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public virtual bool SaveCurrent() {
            bool bRetVal;

            bRetVal = SaveToDB();
            SaveToCache();
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteStack:                        Delete the specified stack
        /// </summary>
        /// <param name="iStackId">             Stack id</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool DeleteStack(int iStackId) {
            NavStack    navStack;
            
            navStack        = NavigationStack;
            NavigationStack = null;
            navStack.Destroy();
            Response.Clear();
            Response.End(); // Don't return!
            return(true);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetCommandsString:              Gets the list of commands as a string 
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual string GetCommandsString() {
            string strRetVal = null;

            if (!IsPostBack) {
                strRetVal = Request[REPPP_CMD_FIELD_NAME];
            } else {
                strRetVal = Request.Form[REPPP_CMD_FIELD_NAME];
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetSBCurrentSelection:          Selects the current item in the selection buffer
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void SetSBCurrentSelection(){
            Rowset      rowsetRoot;
            RowsetDef   rowsetDefRoot;

            if (SelectionBuffer != null) {
                rowsetRoot = TransactionRowsetTree.RootRowset;
                if (rowsetRoot.LineState != LineState.sdLineNew) {
                    rowsetDefRoot = TransactionRowsetTree.RootRowset.RowsetDef; 
                    if (SelectionBuffer.SelectItem(WebUtil.BuildPrimaryKey(rowsetDefRoot, true), false) == false) {
                        if (SelectionBuffer.FilterClause == String.Empty) {
                            throw new FWException(FWException.EditorExceptionE.KeyNotFound);
                        }
                    }                
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// LoadCurrentData:            Load the transaction rowset tree
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void LoadCurrentData() {
            string                  strSessionRowsetTreeName;
            NavEntryTransaction     transEntry;
            RowsetTreeDef           rowsetTreeDef;
            NavEntryObjectInstance  objInst;
            Rowset                  rowsetTrans;
            bool                    bDataFound   = false;
            
            rowsetTreeDef   = TransactionRowsetTreeDef;
            transEntry      = CurrentTransactionEntry;
            if (transEntry != null) {
                strSessionRowsetTreeName = transEntry.TransactionRowsetTreeName;
                if (rowsetTreeDef.Name == strSessionRowsetTreeName) {
                    bDataFound = true;
                }
            }
            if (bDataFound) {
                m_rowsetTreeTransaction  = rowsetTreeDef.RowsetTrees.Add();
                objInst                  = transEntry[REPPP_TRANSACTION_OBJECT_NAME] as NavEntryObjectInstance;
                if (objInst == null) {
                    throw new FWException(FWException.EditorExceptionE.CannotFindTransRowsetTree);
                }
                if (!objInst.LoadFromSession(m_rowsetTreeTransaction)) {
                    throw new FWException(FWException.EditorExceptionE.CannotLoadData);
                }
                DataAccess.Initialize(m_rowsetTreeTransaction);
                m_rowsetTreeTransaction.RootRowset.SelectLine(0, true, true);
                rowsetTrans = TransactionRowset;
                if (rowsetTrans != null && rowsetTrans.LineCount != 0) {
                    m_eOriginalLineState = rowsetTrans.LineState;
                }
                SetSBCurrentSelection();
            } else if (IsRootNode) {
                m_rowsetTreeTransaction = rowsetTreeDef.RowsetTrees.Add();
            } else {
                throw new FWException(FWException.EditorExceptionE.CannotFindTransactionEntry);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GoUp:                               Goes back to the item at the specified up position
        /// </summary>
        /// <param name="ePos">                 Position to edit</param>
        /// <remarks>
        /// Does not return!
        /// </remarks>
        //
        //********************************************************************************
        public virtual void GoUp(UpPositionE ePos) {
            switch (ePos) {
            case UpPositionE.PREVIOUSEDITOR:
                GoUp(null, false);
                break;
            case UpPositionE.PREVIOUSTRANSACTION:
                GoUp(typeof(NavEntryTransaction), false);
                break;
            case UpPositionE.TOPOFTRANSACTION:
                GoUp(typeof(NavEntryTransaction), true);
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.DevError);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// PushTransaction:                    Pushes a transaction into the navigation stack  
	    /// </summary>
        /// <param name="strPageUrl">           Url of the page</param>
        /// <param name="strRowsetTreeName">    Transaction rowset tree name</param>
        /// <param name="eCacheMode">           Transaction cache mode</param>
        /// <param name="strSBRowsetTreeName">  Selection buffer rowset tree name</param>
        /// <param name="strSBFilterClause">    Selection buffer filter where clause</param>
        /// <returns>
        /// The newly added navigation stack entry
        /// </returns>
	    //
        //********************************************************************************
        public virtual NavEntryTransaction PushTransaction(string                  strPageUrl, 
                                                           string                  strRowsetTreeName, 
                                                           TransactionCacheModeE   eCacheMode,
                                                           string                  strSBRowsetTreeName,
                                                           string                  strSBFilterClause) {
            NavEntryTransaction entryRetVal;

            entryRetVal = new NavEntryTransaction(NavigationStack,
                                                  strPageUrl,
                                                  strRowsetTreeName,
                                                  eCacheMode);                                                                        
            NavigationStack.PushEntry(entryRetVal);
            if (strSBRowsetTreeName != null && strSBRowsetTreeName != String.Empty) {
                entryRetVal[m_strSBRefObjName] = new NavEntryObjectSBRef(entryRetVal, 
                                                                         m_strSBRefObjName, 
                                                                         strSBRowsetTreeName, 
                                                                         strSBFilterClause,
                                                                         null,
                                                                         SBCacheMode);
            }     
            return(entryRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ExecuteNavStackOp:              Execute the navigation stack operation if any
	    /// </summary>
	    /// <param name="cmd">              Cmd object</param>
	    //
        //********************************************************************************
        protected virtual void ExecuteNavStackOp(Cmd cmd) {
            NavStackOpE eStackOp;
            string      strNextPage;

            eStackOp    = cmd.StackOp;
            switch(eStackOp) {
            case NavStackOpE.TOPTRANS:                    
            case NavStackOpE.POP:
            case NavStackOpE.POPTRANS:
                cmd.StackOp = NavStackOpE.NONE;
                switch(eStackOp) {    
                case NavStackOpE.TOPTRANS:                    
                    GoUp(UpPositionE.TOPOFTRANSACTION);
                    break;
                case NavStackOpE.POP:
                    GoUp(UpPositionE.PREVIOUSEDITOR);
                    break;
                case NavStackOpE.POPTRANS:
                    GoUp(UpPositionE.PREVIOUSTRANSACTION);
                    break;
                default:
                    break;
                }
                break;
            case NavStackOpE.NEW:
            case NavStackOpE.PUSHTRANS:
            case NavStackOpE.PUSHPAGE:
                if (!IsCurrentPage(cmd.NextPage)) {
                    strNextPage     = cmd.NextPage;
                    cmd.NextPage    = String.Empty;
                    Transfer(strNextPage, null); // Does Not Return
                } else {
                    cmd.StackOp = NavStackOpE.NONE;
                    switch(eStackOp) {
                    case NavStackOpE.NEW:
                        if (!IsRootNode) {
                            throw new FWException(FWException.EditorExceptionE.NotAllowedToCreateTransaction);
                        }
                        NavigationStack = NavStack.Create(SessionDataHolder, CacheDataHolder);
                        PushTransaction(CurrentPage, TransactionRowsetTreeName, TransactionCacheMode, SBRowsetTreeName, SBFilterClause);
                        break;
                    case NavStackOpE.PUSHTRANS:
                        if (!IsRootNode) {
                            throw new FWException(FWException.EditorExceptionE.NotAllowedToCreateTransaction);
                        }
                        PushTransaction(CurrentPage, TransactionRowsetTreeName, TransactionCacheMode, SBRowsetTreeName, SBFilterClause);
                        break;
                    case NavStackOpE.PUSHPAGE:
                        PushPage(CurrentPage);
                        break;
                    default:
                        throw new FWException(FWException.EditorExceptionE.DevError);
                    }
                }
                break;
            case NavStackOpE.NONE:
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidStackOperation);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CheckPrimaryKey:        Checks if the primary key has illegaly changed
        /// </summary>
        //
        //********************************************************************************
        protected virtual void CheckPrimaryKey(Rowset rowsetTrans) {
            if (m_eOriginalLineState != LineState.sdLineNew && rowsetTrans.LineState == LineState.sdLineNew) {
                FWTrace.WriteWarning(String.Format("The primary key has changed."));
                throw new FWException(FWException.EditorExceptionE.PKChanged);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// DisplayErrorInValidationSummary:    Display a rowset tree error
	    /// </summary>
	    /// <param name="iErrorCode">           Error Code</param>
	    /// <param name="strErrorMsg">          Error text</param>
	    //
        //********************************************************************************
        public virtual void DisplayErrorInValidationSummary(int iErrorCode, string strErrorMsg) {
            ValidationSummary   validatorSumm;
            BaseValidator       validatorTmp;    

            if (strErrorMsg != String.Empty) {
                validatorSumm = (ValidationSummary)WebUtil.FindControlUsingType(Controls, typeof(ValidationSummary));
                if (validatorSumm != null) {
                    if (validatorSumm.ShowMessageBox == false && validatorSumm.ShowSummary == false) {
                        validatorSumm.ShowSummary = true;
                    }
                    validatorTmp              = new ValidatorNotValid();
                    validatorTmp.ErrorMessage = strErrorMsg;
                    validatorTmp.IsValid      = false;
                    Validators.Add(validatorTmp);
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetErrorMessage:                    Get rep++  error message
	    /// </summary>
	    /// <param name="iErrorCode">           Error code</param>
	    /// <param name="strDBErrMessage">      Database error message</param>
	    //
        //********************************************************************************
        protected virtual string GetErrorMessage(int iErrorCode, string strDBErrMessage) {
            RepPP.Application   app;
            string              strRetVal   = String.Empty;
            string              strAtomName = String.Empty;

            app = RepApplication;
            switch (iErrorCode) {
            case 56:    // ERR_REF_INTEGRITY
                strAtomName = "$REF";
                break;
            case 31:    // ERR_ALREADY_DEFINED:
                strAtomName = "$RAD";
                break;
            case 14:    // ERR_BAD_VAL:
                strAtomName = "$IAN";
                break;
            default:
                break;
            }
            if (strAtomName != String.Empty && app.IsAtomExist(strAtomName, AtomType.sdAtomErr)) {
                strRetVal = app.GetAtomText(strAtomName, AtomType.sdAtomErr);
            } else {
                strRetVal = "Unknown Error: Code = " + iErrorCode + " Text = " + strDBErrMessage;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetCommands:                    Set the editor commands
	    /// </summary>
	    /// <param name="strCmds">          Commands to be executed</param>
	    //
        //********************************************************************************        
        public virtual void SetCommands(string strCmds) {
            m_arrCmds           = StandardCommandHandler.ParseCmds(strCmds);
            m_iCurrentCmdIndex  = 0;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// AppendCommands:                 Appends one or more commands to the editor commands
	    /// </summary>
	    /// <param name="strCmds">          Commands to be appended</param>
	    //
        //********************************************************************************        
        public virtual void AppendCommands(string strCmds) {
            Cmd[] arrCmds;

            arrCmds = StandardCommandHandler.ParseCmds(strCmds);
            AppendCommands(arrCmds);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AppendCommands:                     Appends one or more commands to the list 
        ///                                     of commands to be executed by the editor
        /// </summary>
        /// <param name="Cmds">                 List of commands to be appended</param>
        //
        //********************************************************************************
        protected virtual void AppendCommands(Cmd[] Cmds){
            ArrayList   arrCmds;
            int         iLastCmdIndex = -1;

            if (m_arrCmds == null) {
                arrCmds         = new ArrayList();
            } else {
                iLastCmdIndex   = m_arrCmds.Length - 1;
                arrCmds         = new ArrayList(m_arrCmds);
            }
            if (iLastCmdIndex >= 0 &&
                String.Compare((arrCmds[iLastCmdIndex] as Cmd).Name, "NONE", true) == 0) {
                arrCmds.RemoveAt(iLastCmdIndex);
            }
            arrCmds.AddRange(Cmds);
            m_arrCmds = (Cmd[])arrCmds.ToArray(typeof(Cmd));
            if (m_iCurrentCmdIndex == -1) {
                m_iCurrentCmdIndex  = 0;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AppendCommand:                      Appends a command to the list of commands
        ///                                     to be executed by the editor
        /// </summary>
        /// <param name="cmd">                  Command to be appended</param>
        //
        //********************************************************************************
        protected virtual void AppendCommand(Cmd cmd){
            AppendCommands(new Cmd[1]{cmd});
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// UpdatePostBackData:                 Update the cache with the post back data
	    /// </summary>
	    /// <returns>
	    /// true if succeed, false if failed.
	    /// </returns>
	    //
        //********************************************************************************
        public virtual bool UpdatePostBackData() {
            bool                    bRetVal;
            
            SaveToCache();
            if (TransactionCacheMode == TransactionCacheModeE.CacheAndCommit &&
                !IsRootNode) {
                bRetVal = SaveCurrent();
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnInit:                         Called when the page is initialized
	    /// </summary>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
        protected override void OnInit(EventArgs e) {
            Cmd             cmd;
            string          strCmds;
            PageEditor      editorFather;
            
            base.OnInit(e);
            if (!DesignMode) {
                strCmds = Context.Items[REPPP_CMD_FIELD_NAME] as string;
                if (PreviousPage == null && strCmds == null) {
                    strCmds             = GetCommandsString();
                    m_arrCmds           = StandardCommandHandler.ParseCmds(strCmds);
                    m_iCurrentCmdIndex  = 0;
                } else {
                    if (PreviousPage is PageEditor) {
                        editorFather        = PreviousPage as PageEditor;
                        m_arrCmds           = editorFather.m_arrCmds;
                        m_iCurrentCmdIndex  = editorFather.m_iCurrentCmdIndex;
                    } else {
                        if (strCmds != null && strCmds != String.Empty) {
                            m_arrCmds           = StandardCommandHandler.ParseCmds(strCmds);
                            m_iCurrentCmdIndex  = 0;
                        }
                    }
                    if (Command != null && Command != String.Empty) {
                        AppendCommands(Command);
                    }
                }
                if (m_arrCmds == null) {
                    cmd                 = DefaultCommand;
                    m_arrCmds           = new Cmd[1] { cmd };
                    m_iCurrentCmdIndex  = 0;
                }
                if (m_iCurrentCmdIndex >= m_arrCmds.Length) {
                    throw new IndexOutOfRangeException("The command index points to an invalid command");
                }
                LoadCurrentData();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Validate:                   Validates the page
        /// </summary>
        //
        //********************************************************************************
        public override void Validate() {
            CancelEventArgs eventArgs;
            Rowset          rowsetTrans;
            
            if (CurrentCommand.UpdateValue) {
                rowsetTrans  = TransactionRowset;
                if (rowsetTrans == null || rowsetTrans.LineCount == 0 || rowsetTrans.LineState == LineState.sdLineOld) {
                    IsValid = true;
                } else {
                    eventArgs = new CancelEventArgs(false);
                    OnBeforeValidate(eventArgs);
                    if (eventArgs.Cancel) {
                        IsValid = false;
                    } else {
                        base.Validate();
                        IsValid = base.IsValid;
                        if (IsValid) {
                            CheckPrimaryKey(rowsetTrans);
                            OnAfterValidate(eventArgs);
                            if (eventArgs.Cancel) {
                                IsValid = false;
                            }
                        }
                    }
                }
            } else {
                IsValid = true;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ExecuteCmds:                    Executes one or more commands
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void ExecuteCmds() {
            Cmd                 cmd;
            bool                bUpdateData = true;
            bool                bNoError;
            ApplicationHolder   appHolder;
            StdCmdHandler       stdCmdHandler;

            appHolder = RepApplicationHolder;
            if (IsValid) {            
                bNoError        = true;
                stdCmdHandler   = StandardCommandHandler;
                while (m_iCurrentCmdIndex < m_arrCmds.Length && bNoError) {
                    cmd = CurrentCommand;
                    if (IsPostBack && cmd.UpdateValue && bUpdateData) {
                        bNoError        = UpdatePostBackData();
                        bUpdateData     = false;
                        cmd.UpdateValue = false;
                    }
                    if (bNoError) {
                        ExecuteNavStackOp(cmd); // May not return!
                        m_iCurrentCmdIndex++;
                        bNoError        = stdCmdHandler.ExecuteCmd(cmd);    // May not return too...
                    }
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ExecuteCmds:                    Executes one or more commands
	    /// </summary>
	    /// <param name="strCmds">          commands to be executed</param>
	    //
        //********************************************************************************
        public virtual void ExecuteCmds(string strCmds) {
            SetCommands(strCmds);
            ExecuteCmds();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetClientSideParameters:            Sets the values that will be written in the client
	    ///                                     side script
	    /// </summary>
        /// <param name="strSaveMsg">           Message displayed to ask if changes will be saved</param>
        /// <param name="strDeleteMsg">         Message displayed to confirm the deletion of a root item</param>
        /// <param name="strNewState">          Message to indicate that the current item is new</param>
        /// <param name="strModifiedState">     Message to indicate that the current item is modified</param>
        /// <param name="strUnchangedState">    Message to indicate that the current item is already saved</param>
        /// <param name="iCurrentPosition">     The current position of the item in the list</param>
        /// <param name="iItemCount">           Number of items in the selection buffer or group instance</param>
	    //
        //********************************************************************************
        protected virtual void GetClientSideParameters(out string strSaveMsg,
                                                       out string strDeleteMsg,
                                                       out string strNewState,
                                                       out string strModifiedState,
                                                       out string strUnchangedState,
                                                       out int    iCurrentPosition,
                                                       out int    iItemCount) {
            RepPP.Application   app;

            app = RepApplication;
            if (app.IsAtomExist("IsRowsetTreeDirty", AtomType.sdAtomMsg)) {
                strSaveMsg = app.GetAtomText("IsRowsetTreeDirty", AtomType.sdAtomMsg);
                strSaveMsg = strSaveMsg.Replace("\r", "");
                strSaveMsg = strSaveMsg.Replace("\n", "");
            } else {
                strSaveMsg = " You have modified your card. Do you want to lose your modifications ?";
            }
            if (app.IsAtomExist("AskRootDelete", AtomType.sdAtomMsg)) {
                strDeleteMsg = app.GetAtomText("AskRootDelete", AtomType.sdAtomMsg);
                strDeleteMsg = strDeleteMsg.Replace("\r", "");
                strDeleteMsg = strDeleteMsg.Replace("\n", "");
            } else {
                strDeleteMsg = " Are you sure that you want to delete this item? ";
            }
            if (app.IsAtomExist("MODIFIED_STATE", AtomType.sdAtomMsg)) {
                strModifiedState = app.GetAtomText("MODIFIED_STATE", AtomType.sdAtomMsg);
                strModifiedState = strModifiedState.Replace("\r", "");
                strModifiedState = strModifiedState.Replace("\n", "");
            } else {
                strModifiedState = "Modified";
            }
            if (app.IsAtomExist("NEW_STATE", AtomType.sdAtomMsg)) {
                strNewState = app.GetAtomText("NEW_STATE", AtomType.sdAtomMsg);
                strNewState = strNewState.Replace("\r", "");
                strNewState = strNewState.Replace("\n", "");
            } else {
                strNewState = "New";
            }
            if (app.IsAtomExist("UNCHANGED_STATE", AtomType.sdAtomMsg)) {
                strUnchangedState = app.GetAtomText("UNCHANGED_STATE", AtomType.sdAtomMsg);
                strUnchangedState = strUnchangedState.Replace("\r", "");
                strUnchangedState = strUnchangedState.Replace("\n", "");
            } else {
                strUnchangedState = "Unchanged";
            }
            if (IsRootNode){
                if (SelectionBuffer != null) {
                    iItemCount          = SelectionBuffer.ItemCount;
                    iCurrentPosition    = SelectionBuffer.SelectedIndex + 1;
                } else {
                    iItemCount          = 1;
                    iCurrentPosition    = 1;
                }
            } else {
                iItemCount          = TransactionRowset.UndelLineCount;
                iCurrentPosition    = TransactionRowset.CurrentUndelLine + 1;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// RegisterPageEditorScript:       Register the editor script
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual void RegisterPageEditorScript() {
            Application     app;
            StreamReader    reader;
            const string    strKey          = "PageEditor";
            ArrayList       arrFldCtlID;
            string          strSaveMsg;
            string          strDeleteMsg;
            string          strStateNew;
            string          strStateModified;
            string          strStateUnchanged;
            string          strScript;
            int             iCurrentLine;
            int             iItemCount;

	        if (!WebUtil.IsClientScriptBlockRegistered(this, strKey)) {
                app         = RepApplication;
                arrFldCtlID = new ArrayList();
                WebUtil.FindAllFldControlsID(Controls, ref arrFldCtlID);
                GetClientSideParameters(out strSaveMsg,
                                        out strDeleteMsg,
                                        out strStateNew,
                                        out strStateModified,
                                        out strStateUnchanged,
                                        out iCurrentLine,
                                        out iItemCount);
                reader = new StreamReader(typeof(PageEditor).Assembly.GetManifestResourceStream("RepPP.Framework.Web.PageEditorScript.js"));
	            WebUtil.RegisterClientScriptBlock(this,
                                                  strKey,
                                                  String.Format(reader.ReadToEnd(), 
                                                                Utility.GetFormName(this),
                                                                RepPP.Toolkit.Util.FormatStringList(arrFldCtlID, "document.getElementById(\"", "\")"),
                                                                RepPP.Toolkit.Util.FormatStringList(m_arrSaveHandler, "\"", "\""),
                                                                strSaveMsg,
                                                                strDeleteMsg,
                                                                strStateNew,
                                                                strStateModified,
                                                                strStateUnchanged,
                                                                iCurrentLine,
                                                                iItemCount));
            }
	        if (!WebUtil.IsClientScriptBlockRegistered(this, "PageEdior_InitializeFldControls")) {
                strScript = "PageEdior_InitializeFldControls();\r\n" + 
                            "PageEditor_SetSaveState(PageEditor_IsRowsetTreeDirty());";
                strScript = "<script language='javascript'>\r\n<!--\r\n" + strScript + "\r\n-->\r\n</script>";
	            WebUtil.RegisterStartupScript(this, "PageEdior_InitializeFldControls", strScript);
            }

        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:                    Called before rendering the page
	    /// </summary>
	    /// <param name="e">                Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {

            ExecuteCmds();
            WebUtil.RegisterHiddenField(this, REPPP_CMD_FIELD_NAME, "");
            WebUtil.RegisterHiddenField(this, REPPP_ISROWSETTREEDIRTY_FIELD_NAME, TransactionRowsetTree.IsModified.ToString());
            RegisterPageEditorScript();
            base.OnPreRender(e);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Render:                         Render the page
	    /// </summary>
	    /// <param name="writer">           Stream writer</param>
	    //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            RepPP.Toolkit.Util.FldSelectableState   fldSelectableState;
            Rowset                                  rowsetTrans;
            
            rowsetTrans = TransactionRowset;
            using(fldSelectableState = new RepPP.Toolkit.Util.FldSelectableState(rowsetTrans.RowsetDef)) {
                if (rowsetTrans.LineState == LineState.sdLineNew) {
                    fldSelectableState.State = RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All;
                } else {
                    fldSelectableState.State = RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_NoPK;
                }
                base.Render(writer);
            }
            RepApplicationHolder.CheckIfErrorInEventViewer(writer);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnUnload:                       Called at page unload
	    /// </summary>
	    /// <param name="e">                Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnUnload(EventArgs e) {
            if (m_rowsetTreeTransaction != null) {
                m_rowsetTreeTransaction.Dispose();
                m_rowsetTreeTransaction = null;
            }
            base.OnUnload(e);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ValidatorNotValid:             Class defining a not valid validator
	    /// </summary>
        //
        //********************************************************************************
        public class ValidatorNotValid : CustomValidator {
            //********************************************************************************
            //
            /// <summary>
            /// EvaluateIsValid:        Determine if the current value is valid
            /// </summary>
            /// <returns>
            /// true if yes, false if not
            ///</returns>
            //
            //********************************************************************************
            protected override bool EvaluateIsValid() {
                return(false);
            }
        } // Class ValidatorNotValid

        /// <summary>Hold a command to execute</summary>
        public class Cmd {
            /// <summary>true to update instance with postback values, false to ignore postback values</summary>
            public bool         UpdateValue;
            /// <summary>Navigation stack operation</summary>
            public NavStackOpE  StackOp;
            /// <summary>Page to transfer to</summary>
            public string       NextPage;
            /// <summary>Command code</summary>
            public string       Name;
            /// <summary>Command parameters</summary>
            public string[]     Params;

            //********************************************************************************
            //
	        /// <summary>
	        /// ctor:                           Class constructor
	        /// </summary>
	        //
            //********************************************************************************
            public Cmd() {
                SetToNone();
            }

            //********************************************************************************
            //
	        /// <summary>
	        /// SetToNone:                      Set the command to None
	        /// </summary>
	        //
            //********************************************************************************
            public void SetToNone() {
                UpdateValue     = false;
                StackOp         = NavStackOpE.NONE;
                NextPage        = String.Empty;
                Params          = new string[0];
                Name            = "NONE";
            }
        } // Class Cmd
    } // Class PageEditor
};