using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    /// <summary>
    /// Base class used to implement pages that need both the navigation
    /// stack and the selection buffer
    /// </summary>
    public class PageWithSelectionBuffer : PageWithStack {

        /// <summary>Determine the caching mode of the Selection Buffer</summary>
        public enum SBCacheModeE {
            /// <summary>Do not cache the data. always read from db</summary>
            NoCache             =   0,
            /// <summary>Cache the data in a non permanent cache location</summary>
            NonPermanentCache   =   1
        };

        /// <summary>Name of the selection buffer reference object kept in the entry object list</summary>
        public const string                 m_strSBRefObjName               = "_RepPPSelectionBufferRef";

        /// <summary>Rep++ Application holder</summary>
        private ApplicationHolder           m_appHolder;
        /// <summary>Selection Buffer data holder</summary>
        private SelectionBufferHolder       m_sbHolder;
        /// <summary>Data access object </summary>
        private   IDataAccess               m_dataAccess;
        /// <summary>Name of the rowset tree use by the selection buffer</summary>
        private string                      m_strSBRowsetTreeName;
        /// <summary>Filter clause use by the selection buffer</summary>
        private string                      m_strSBFilterClause;
        /// <summary>Used to set the initial sort params without creating the actual sb</summary>
        private object[]                    m_arrInitialSortParams;
        /// <summary>Indicates the if the SB data will be cached and how.</summary>
        private SBCacheModeE                m_eSBCacheMode;

        /// <summary>Event sent before loading the selection buffer data from DB</summary>
	    public event CancelEventHandler	    BeforeLoadSelectionBufferFromDB;
	    /// <summary>Event sent after reading the selection buffer data from DB</summary>
	    public event CancelEventHandler	    AfterLoadSelectionBufferFromDB;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="strSBRowsetTreeName">      Name of the rowset tree use for the selection buffer</param>
        /// <param name="strSBFilterClause">        Selection buffer filter clause</param>
        /// <param name="eSBCacheMode">             Selection buffer cache mode</param>
        //
        //********************************************************************************
        protected PageWithSelectionBuffer(string strSBRowsetTreeName, string strSBFilterClause, SBCacheModeE eSBCacheMode) {
            m_appHolder             = new ApplicationHolder(this);
            m_sbHolder              = null;
            m_dataAccess            = null;
            m_strSBRowsetTreeName   = strSBRowsetTreeName;
            m_strSBFilterClause     = strSBFilterClause;
            m_arrInitialSortParams  = null;
            m_eSBCacheMode          = eSBCacheMode;
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="strSBRowsetTreeName">      Selection buffer card name</param>
        /// <param name="strSBFilterClause">        Selection buffer filter clause</param>
        //
        //********************************************************************************
        protected PageWithSelectionBuffer(string strSBRowsetTreeName, string strSBFilterClause) : this(strSBRowsetTreeName, strSBFilterClause, SBCacheModeE.NonPermanentCache) {
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public PageWithSelectionBuffer() : this("", "", SBCacheModeE.NonPermanentCache) {
        }

        //********************************************************************************
        //
        /// <summary>
        /// FlushContent:                       Remove gracefully the content of the page
        /// </summary>
        //
        //********************************************************************************
        public override void FlushContent() {
            base.FlushContent();
            if (m_appHolder != null) {
                m_appHolder.Dispose();
                m_appHolder = null;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RepApplicationHolder:               Returns the application holder object
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ApplicationHolder RepApplicationHolder {
            get {
                return(m_appHolder);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// DataAccess:          Object handling the data access
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
	    public virtual IDataAccess DataAccess {
	        get {
	            if (m_dataAccess == null) {
	                m_dataAccess = new DataAccess(this);
	            }
	            return(m_dataAccess);
	        }
	        set {
	            m_dataAccess = value;
	        }
	    }

        //********************************************************************************
        //
	    /// <summary>
	    /// RepApplication:                 Rep++ Application
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public RepPP.Application RepApplication {
            get {
                return((RepApplicationHolder != null) ? RepApplicationHolder.Application : null);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SBRowsetTreeName:                Returns the Selection Buffer card name
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string SBRowsetTreeName {
            get {
                return(m_strSBRowsetTreeName);
            }
            set {
                if (m_sbHolder != null){
                    throw new FWException(FWException.EditorExceptionE.BadParameter, 
                                          "Cannot change the SB rowset tree name after the SB is loaded");
                } else {
                    m_strSBRowsetTreeName = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SBFilterClause:            Returns the Selection Buffer filter clause
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string SBFilterClause {
            get {
                return(m_sbHolder == null ? m_strSBFilterClause : m_sbHolder.FilterClause);
            } 
            set {
                if (m_sbHolder != null){
                    m_sbHolder.FilterClause = value;
                } else {
                   m_strSBFilterClause = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SBCacheMode:                Selection Buffer Cache Mode
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual SBCacheModeE SBCacheMode {
            get {
                return(m_sbHolder == null ? m_eSBCacheMode : m_sbHolder.SBCacheMode);
            } 
            set {
                if (m_sbHolder == null) {
                    m_eSBCacheMode = value;
                } else {
                    throw new FWException(FWException.EditorExceptionE.DevError,
                                          "The SB cache mode cannot be changed once the SB is loaded");
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsSelectionBufferCreated:           Returns the selection buffer holder
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool IsSelectionBufferCreated {
            get {
                return (m_sbHolder != null);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectionBuffer:                    Returns the selection buffer holder
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual SelectionBufferHolder SelectionBuffer {
            get {
                NavEntryObjectSBRef     sbRef           = null;
                NavEntry                entry;
                bool                    bCreateSBRef    = false;

                if (m_sbHolder == null) {
                    if (m_strSBRowsetTreeName != null) {
                        sbRef = NavigationStack.FindObj(m_strSBRefObjName) as NavEntryObjectSBRef;
                        if (sbRef == null) {
                            if (m_strSBRowsetTreeName == null || m_strSBRowsetTreeName == String.Empty) {
                                bCreateSBRef = false; // return null value
                            } else {
                                bCreateSBRef = true;
                            }
                        } else if (m_strSBRowsetTreeName != String.Empty){
                            if (sbRef.SBRowsetTreeName != m_strSBRowsetTreeName /*|| sbRef.FilterClause != m_strSBFilterClause*/) {
                                bCreateSBRef = true;
                            }
                        }
                        if (bCreateSBRef) {
                            entry = new NavEntry(NavigationStack, CurrentPage);
                            NavigationStack.PushEntry(entry);
                            entry[m_strSBRefObjName] = new NavEntryObjectSBRef(entry, 
                                                                               m_strSBRefObjName, 
                                                                               m_strSBRowsetTreeName, 
                                                                               m_strSBFilterClause,
                                                                               null,
                                                                               SBCacheMode);
                            sbRef                    = entry[m_strSBRefObjName] as NavEntryObjectSBRef;
                        }
                    }
                    if (sbRef == null) {
                        m_sbHolder = null;
                    } else {
                        m_sbHolder = new SelectionBufferHolder(RepApplication, sbRef, DataAccess);
                        if (m_arrInitialSortParams != null) {
                            m_sbHolder.SaveSortParam(m_arrInitialSortParams);
                        }
                    }
                }
                return(m_sbHolder);
            }
        }

		//********************************************************************************
		//
		/// <summary>
		///  SetSBInitialSort:                      Sets the initial sort parameters of the
		///                                         selection buffer data without creating the 
		///                                         selection buffer holder. Prevents the actions
		///                                         triggerred by the creation of the selection
		///                                         buffer holder like (adding an entry with an
		///                                         SBRef) in the stack
		/// </summary>
		/// <param name="arrSortParam">             The sort parameters</param>
		//
		//********************************************************************************
        public void SetSBInitialSort(object[] arrSortParam) {
            if (m_sbHolder == null) { 
                m_arrInitialSortParams  = arrSortParam;
            } else {
                m_sbHolder.SaveSortParam(arrSortParam);
            }
        }

		//********************************************************************************
		//
		/// <summary>
		///  OnBeforeLoadSelectionBufferFromDB:     Called before loading the selection buffer
		///                                         data from database
		/// </summary>
		/// <param name="e">                        Event arguments</param>
		//
		//********************************************************************************
	    public virtual void OnBeforeLoadSelectionBufferFromDB(CancelEventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    if (!e.Cancel) {
		        appFW = Context.ApplicationInstance as HttpApplicationFW;
		        if (appFW != null) {
		            e.Cancel = !appFW.BeforeLoadSelectionBufferFromDB(this);
		        }
		        if (!e.Cancel && BeforeLoadSelectionBufferFromDB != null) {
		            BeforeLoadSelectionBufferFromDB(this, e);
		        }
		    }
	    }

		//********************************************************************************
		//
		/// <summary>
		///  OnAfterLoadSelectionBufferFromDB:      Called after loading the selection buffer
		///                                         data from the database
		/// </summary>
		/// <param name="e">                        Event arguments</param>
		//
		//********************************************************************************
	    public virtual void OnAfterLoadSelectionBufferFromDB(CancelEventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    if (!e.Cancel) {
		        appFW = Context.ApplicationInstance as HttpApplicationFW;
		        if (appFW != null) {
		            e.Cancel = !appFW.AfterLoadSelectionBufferFromDB(this);
		        }
		        if (!e.Cancel && AfterLoadSelectionBufferFromDB != null) {
		            AfterLoadSelectionBufferFromDB(this, e);
		        }
		    }
	    }
  } // Class PageWithSelectionBuffer
} // Namespace