using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    /// <summary>Base class used to implement all pages that need the navigation stack</summary>
    public class PageWithStack : PagePreInit {
        /// <summary>Name of the hidden field containing the stack id</summary>
        public const string                 REPPP_STACKID_FIELD_NAME      = "_RepPPStackId";
        /// <summary>Name of the hidden field containing the sequence id</summary>
        public const string                 REPPP_SEQUENCEID_FIELD_NAME   = "_RepPPSequenceId";

        /// <summary>Navigation stack</summary>
        private NavStack                    m_navStack;
        /// <summary>Default data holder (session and cache)</summary>
        private DataHolder                  m_dataHolder;
        /// <summary>Trace object</summary>
        private FWTrace                     m_trace;
        /// <summary>Used to pass commands between pages during a transfer</summary>
        private string                      m_strCommand;
        /// <summary>Used to pass indicate if the page can be cached on the browser</summary>
        private bool                        m_bAllowBrowserCaching;
	    /// <summary>Event sent when an out of sequence command is received</summary>
	    public event EventHandler	        OutOfSequence;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public PageWithStack() {
            m_navStack              = null;
            m_dataHolder            = null;
            m_trace                 = null;
            m_strCommand            = "";
            m_bAllowBrowserCaching  = false;
        }

        //********************************************************************************
        //
        /// <summary>
        /// DisposeCtl:                         Dispose a control
        /// </summary>
        /// <param name="ctl">                  Control to dispose</param>
        //
        //********************************************************************************
        private void DisposeCtl(Control ctl) {
            foreach (Control ctlChild in ctl.Controls) {
                DisposeCtl(ctlChild);
            }
            ctl.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FlushContent:                       Remove gracefully the content of the page
        /// </summary>
        //
        //********************************************************************************
        public virtual void FlushContent() {
            foreach (Control ctl in Controls) {
                DisposeCtl(ctl);
            }
            if (!Controls.IsReadOnly) {
                Controls.Clear();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// NavigationStack:                    Navigation stack
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual NavStack NavigationStack {
            get {
                PageWithStack   pageFather;
                string          strStackId;
                int             iStackId;

                if (m_navStack == null) {
            
                    pageFather = PreviousPage as PageWithStack;
                    if (pageFather != null) {
                       m_navStack = pageFather.NavigationStack;
                    } else {
                        strStackId = GetParam(Request, REPPP_STACKID_FIELD_NAME);
                        try {
                            iStackId = (strStackId == null || strStackId == String.Empty) ? -1 : Int32.Parse(strStackId);
                        } catch {
                            iStackId = -1;
                        }
                        if (iStackId == -1) {
                            m_navStack = NavStack.Create(SessionDataHolder, CacheDataHolder);
                        } else {
                            m_navStack = NavStack.CreateFromSession(SessionDataHolder, CacheDataHolder, iStackId);
                        }
                    }
                }
                return(m_navStack);
            }
            set {
                m_navStack = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FWTrace:                            Framework trace object
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual FWTrace FWTrace  {
            get {
                if (m_trace == null) {
                    m_trace = new FWTrace(this);
                }
                return(m_trace);
            }
            set {
                m_trace = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SessionDataHolder:                  Object handling session data
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ISessionDataHolder SessionDataHolder {
            get {
                if (m_dataHolder == null) {
                    m_dataHolder = new DataHolder();
                }
                return(m_dataHolder);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CacheDataHolder:                    Object handling cache data
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ICacheDataHolder CacheDataHolder {
            get {
                if (m_dataHolder == null) {
                    m_dataHolder = new DataHolder();
                }
                return(m_dataHolder);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// CurrentPage:                    Currently executing page
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        protected virtual string CurrentPage {
            get {
                return(Request.CurrentExecutionFilePath);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SequenceID:                  Gets/Sets the sequence id of the page
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        protected virtual int SequenceID {
            get {
                int iRetVal;
                
                iRetVal = NavigationStack.OpSequenceId;
                return(iRetVal);
            }
            set {
                NavigationStack.OpSequenceId = value;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// AllowBrowserCaching:            Gets/Sets whether the page may be cached on the browser
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        protected virtual bool AllowBrowserCaching {
            get {
                return(m_bAllowBrowserCaching);
            }
            set {
                m_bAllowBrowserCaching = value;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// IsCurrentPage:                  Checks if the specified page is the current one
	    /// </summary>
	    /// <param name="strPage">          Page url</param>
	    //
        //********************************************************************************
        protected virtual bool IsCurrentPage(string strPage) {
            bool    bRetVal = true;
            string  strCurPage;
            string  strApplicationPath;
            
            strCurPage = CurrentPage;
            if (strPage != null && strPage != String.Empty) {
                if (strPage.IndexOf('/') == -1) {
                    strApplicationPath  = strCurPage.Substring(0, strCurPage.LastIndexOf('/') + 1);
                    strPage             = strApplicationPath + strPage;
                }
                if (String.Compare(strPage, strCurPage, true) != 0) {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Command:                        Gets/Sets string commands sent from one page to
	    ///                                 an other following a transfer.
	    /// </summary>
	    //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual string Command {
            get {
                return(m_strCommand);
            }
            set {
                m_strCommand = value;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetLanguage:                    Sets the language of the application
	    /// </summary>
	    /// <param name="eLanguage">        Language code</param>
	    //
        //********************************************************************************
        public virtual void SetLanguage(LanguageCode eLanguage) {
            if (eLanguage != LanguageCode.sdDefault && eLanguage != ApplicationHolder.GetSessionLanguage(this)) {
                ApplicationHolder.SetSessionLanguage(this, eLanguage);
                Transfer(CurrentPage, null);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Transfer:                       Sends a command to an editor for execution  
	    /// </summary>
	    /// <param name="strEditorUrl">     Url of the target editor</param>
	    /// <param name="strCmd">           Command to be executed</param>
	    //
        //********************************************************************************
        public virtual void Transfer(string  strEditorUrl, string strCmd) {
            if (strCmd != null) {
                Command = strCmd;
            }
            FlushContent(); 
            Server.Transfer(strEditorUrl, false);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// PushPage:                       Pushes a page url into the navigation stack  
	    /// </summary>
	    /// <param name="strPageUrl">       Url of the page</param>
	    //
        //********************************************************************************
        public virtual NavEntry PushPage(string strPageUrl) {
            NavEntry entryRetVal;

            entryRetVal = new NavEntry(NavigationStack, strPageUrl);
            NavigationStack.PushEntry(entryRetVal);
            return(entryRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GoUp:                           Goes to the url in the top navigation stack entry
	    ///                                 of the specified type.
	    /// </summary>
	    /// <param name="type">             Type of the navigation stack entry</param>
	    /// <param name="bToEntry">         Indicates if we pop the matching entry or leave it at the top</param>
	    //
        //********************************************************************************
        public virtual void GoUp(Type type, bool bToEntry) {
            NavEntry entry;

            if (bToEntry) {
                NavigationStack.DeleteToTopEntry(type);
            } else if (type != null) {
                NavigationStack.DeleteTopEntry(type);
            } else {
                NavigationStack.DeleteTopEntry();            
            }
            entry = NavigationStack.GetCurrentEntry();
            if (entry == null) {
                throw new FWException(FWException.EditorExceptionE.NavStackEmpty);
            }
            if (!IsCurrentPage(entry.Url)) {
                Transfer(entry.Url, null);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GoUp:                           Goes to the url in the top navigation stack entry
	    /// </summary>
	    //
        //********************************************************************************
        public virtual void GoUp() {
            GoUp(null, false);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GoDown:                           Goes to the specified page url aftre pushing
	    ///                                   it as a top entry of the navigation stack
	    /// </summary>
	    /// <param name="strPageUrl">         The url of the page to go to</param>
	    //
        //********************************************************************************
        public virtual void GoDown(string strPageUrl) {
            if (!IsCurrentPage(strPageUrl)) {
                PushPage(strPageUrl);
                Transfer(strPageUrl, null);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// AttachObject:                     Adds an object to the top entry of the navigation stack
	    /// </summary>
	    /// <param name="obj">                The object to be attached</param>
	    //
        //********************************************************************************
        public virtual void AttachObject(NavEntryObject obj) {
            NavEntry entry;

            entry = NavigationStack.GetCurrentEntry();
            if (entry == null) {
                throw new FWException(FWException.EditorExceptionE.NavStackEmpty);
            }
            obj.Save();
            entry[obj.Name] = obj;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetObject:                          Adds an object to the top entry of the navigation stack
	    /// </summary>
	    /// <param name="strObjectName">        Name of the attached object to be returned</param>
	    /// <returns>
	    /// The object or null
	    /// </returns>
	    //
        //********************************************************************************
        public virtual NavEntryObject GetObject(string strObjectName) {
            NavEntryObject objRetVal;

            objRetVal = NavigationStack.FindObj(strObjectName);
            if (objRetVal != null) {
                objRetVal.Load();
            }
            return(objRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetParam:                       Returns the value of a parameter in either the
	    ///                                 QueryString, Form, Cookie or Server Variables.
	    ///                                 Uses the postback to skip the QueryString params
	    /// </summary>
	    /// <param name="httpRequest">      The HTTP Request</param>
	    /// <param name="strKey">           The name of the parameter to be returned</param>
	    //
        //********************************************************************************
        protected string GetParam(System.Web.HttpRequest httpRequest, string strKey) {
            string      strRetVal = null;
            HttpCookie  cookie;

            if (!IsPostBack) {
                strRetVal = httpRequest[strKey];
            } else {
                strRetVal = httpRequest.Form[strKey];
                if (strRetVal == null){
                    cookie = httpRequest.Cookies[strKey];
                    if (cookie != null){
                        strRetVal = cookie.Value;
                    }
                    if (strRetVal == null){
                        strRetVal = httpRequest.ServerVariables[strKey];
                    }
                }
            }
            return strRetVal;
        }        

        //********************************************************************************
        //
	    /// <summary>
	    /// CheckSequence:                  Ensures a correct sequence of commands
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual void CheckSequence(){
            int                 iLastSequenceId     = 0;
            int                 iCurrentSequenceId  = 0;

            if (!IsCallback) {
                iLastSequenceId     = SequenceID;
                iCurrentSequenceId  = Convert.ToInt32(Request[REPPP_SEQUENCEID_FIELD_NAME]);
			    if ((iCurrentSequenceId > iLastSequenceId) || 
                    (iCurrentSequenceId == iLastSequenceId && iCurrentSequenceId == 0)) {
	    		    SequenceID = iCurrentSequenceId;
			    } else if (IsPostBack){
                    if (AllowBrowserCaching) {
                        EnsureCurrentPageIsTop();
                    } else {
	    		        SequenceID = 0;
                        OnOutOfSequence(EventArgs.Empty);
                    }
			    }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// EnsureCurrentPageIsTop:         Ensures that the entry at the top of the stack 
	    ///                                 refers to the current page 
	    /// </summary>
	    //
        //********************************************************************************
        protected virtual void EnsureCurrentPageIsTop() {
            NavEntry entry;

            entry = NavigationStack.GetCurrentEntry();
            while (entry != null && entry.Url != CurrentPage) {
                NavigationStack.DeleteTopEntry();
                entry = NavigationStack.GetCurrentEntry();
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnInit:                         Called when the page is initialized
	    /// </summary>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
        protected override void OnInit(EventArgs e) {
            PageWithStack   pageFather;
            
            if (!DesignMode) {
                pageFather = PreviousPage as PageWithStack;
                if (pageFather != null) {
                    FWTrace             = pageFather.FWTrace;
                    Command             = pageFather.Command;
                } else {
                    CheckSequence();
                }
            }
            base.OnInit(e);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:                    Called before rendering the page
	    /// </summary>
	    /// <param name="e">                Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            base.OnPreRender(e);
            WebUtil.RegisterHiddenField(this, REPPP_STACKID_FIELD_NAME,     NavigationStack.StackId.ToString());
            WebUtil.RegisterHiddenField(this, REPPP_SEQUENCEID_FIELD_NAME,  (SequenceID + 1).ToString());
            if (!AllowBrowserCaching) {
                Response.Cache.SetCacheability(HttpCacheability.NoCache);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  OnOutOfSequence:               Called when an out of sequence command is received
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnOutOfSequence(EventArgs e) {
            HttpApplicationFW appFW;

            appFW = Context.ApplicationInstance as HttpApplicationFW;
            if (appFW != null) {
                appFW.OutOfSequence(this);
            }
            if (OutOfSequence != null) {
                OutOfSequence(this, e);
            }
        }
    } // Class PageWithStack
} // Namesapce