using System;
using System.Web;

namespace RepPP.Framework.Web {
	/// <summary>
	/// Class holding the selection buffer data
	/// </summary>
	public class SelectionBufferHolder : IDisposable {
	    /// <summary>Rep++ application object</summary>
	    private RepPP.Application                       m_app;
	    /// <summary>Selection buffer reference</summary>
	    private NavEntryObjectSBRef                     m_sbRef;
	    /// <summary>Rowset tree holding the selection buffer data</summary>
	    private RowsetTree                              m_rowsetTreeSB;
	    /// <summary>Data access object</summary>
	    private IDataAccess                             m_dataAccess;
	    /// <summary>Indicates if the data has been sorted</summary>
        private bool                                    m_bSorted;
        /// <summary>Event sent when the selection buffer data or current selection change</summary>
        public event EventHandler	                    SBChange;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                           Constructor of the class
	    /// </summary>
		/// <param name="app">              Rep++ application object</param>
		/// <param name="sbRef">            Selection buffer reference navigation entry object</param>
		/// <param name="dataAccess">       Object handling data access</param>
        //
        //********************************************************************************
		public SelectionBufferHolder(RepPP.Application app, NavEntryObjectSBRef sbRef, IDataAccess dataAccess) {
		    m_app               = app;
		    m_sbRef             = sbRef;
		    m_rowsetTreeSB      = null;
		    m_dataAccess        = dataAccess;
            m_bSorted           = false;
            if (m_app == null || m_sbRef == null || m_dataAccess == null) {
                throw new FWException(FWException.EditorExceptionE.BadParameter);
            }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Dispose:        Dispose the object
	    /// </summary>
        //
        //********************************************************************************
		public virtual void Dispose() {
		    if (m_rowsetTreeSB != null) {
		        m_rowsetTreeSB.Dispose();
		        m_rowsetTreeSB = null;
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SBRowsetTreeName:     Name of the rowset tree holding the selection buffer
	    /// </summary>
        //
        //********************************************************************************
		public virtual string SBRowsetTreeName {
		    get {
		        return(m_sbRef.SBRowsetTreeName);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// LoadData:                   Load the data from the cache or from the database
	    /// </summary>
	    /// <param name="rowsetTree">   Rowset tree to hold the data</param>
        //
        //********************************************************************************
		protected virtual void LoadData(RowsetTree rowsetTree) {
            rowsetTree.Empty();
            m_bSorted = false;
		    if (SBCacheMode == PageWithSelectionBuffer.SBCacheModeE.NoCache || !m_sbRef.LoadSBFromCache(rowsetTree)) {
		        m_dataAccess.LoadSelectionBufferData(rowsetTree, FilterClause);
		        SaveData(rowsetTree);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SaveData:                   Saves the data to the cache
	    /// </summary>
	    /// <param name="rowsetTree">   RowsetTree holding the data to be saved</param>
        //
        //********************************************************************************
		protected virtual void SaveData(RowsetTree rowsetTree) {
            if (SBCacheMode == PageWithSelectionBuffer.SBCacheModeE.NonPermanentCache) {
                if (m_bSorted){
                    throw new FWException(FWException.EditorExceptionE.NotAllowedToSaveSortedSB);
                }
                m_sbRef.SaveSBToCache(rowsetTree);
            }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// InternalSBRowsetTree:     Gets the selection buffer rowset tree (but don't load it)
	    /// </summary>
        //
        //********************************************************************************
		protected virtual RowsetTree InternalSBRowsetTree {
		    get {
		        RowsetTreeDef    rowsetTreeDef;
		        string           strSBRowsetTreeName;
		        
		        if (m_rowsetTreeSB == null) {
                    strSBRowsetTreeName = SBRowsetTreeName;
                    rowsetTreeDef       = m_app.RowsetTreeDefs[strSBRowsetTreeName];
		            if (rowsetTreeDef == null) {
                        throw new FWException(FWException.EditorExceptionE.RowsetTreeNotFound, strSBRowsetTreeName);
		            }
		            m_rowsetTreeSB = rowsetTreeDef.RowsetTrees.Add();
		        }
		        return(m_rowsetTreeSB);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SBRowsetTree: Gets the selection buffer rowset tree
	    /// </summary>
        //
        //********************************************************************************
		public virtual RowsetTree SBRowsetTree {
		    get {
		        if (m_rowsetTreeSB == null) {
		            LoadData(InternalSBRowsetTree);
		        }
		        return(m_rowsetTreeSB);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Refresh:   ReLoad the data from the database and updates the cache
	    /// </summary>
        //
        //********************************************************************************
		public virtual void Refresh() {
            RowsetTree rowsetTree;

            rowsetTree = InternalSBRowsetTree;
            rowsetTree.Empty();
            m_bSorted = false;
		    m_dataAccess.LoadSelectionBufferData(rowsetTree, FilterClause);
		    SaveData(rowsetTree);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SBRootRowset:    Root rowset of the selection buffer
	    /// </summary>
        //
        //********************************************************************************
		public virtual Rowset SBRootRowset {
		    get {
		        return(SBRowsetTree.RootRowset);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectFirstItem:                Select the first item of the selection buffer
	    /// </summary>
	    /// <returns>
	    /// Keys or null if error
	    /// </returns>
	    //
        //********************************************************************************
		public virtual string[] SelectFirstItem() {
		    int     iCurLine;
		    
            EnsureSortedOrder();
		    iCurLine = ItemCount;
		    if (iCurLine != 0) {
		        SelectedIndex = 0;
		    }
		    return(SelectedKey);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectPreviousItem:             Select the previous item of the selection buffer
	    /// </summary>
	    /// <returns>
	    /// Keys or null if error
	    /// </returns>
	    //
        //********************************************************************************
		public virtual string[] SelectPreviousItem() {
		    int             iCurLine;
		    
            EnsureSortedOrder();
		    iCurLine    = SelectedIndex;
		    if (iCurLine != 0) {
		        SelectedIndex = iCurLine - 1;
		    }
		    return(SelectedKey);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectNextItem:                 Select the next item of the selection buffer
	    /// </summary>
	    /// <returns>
	    /// Keys or null if error
	    /// </returns>
	    //
        //********************************************************************************
		public virtual string[] SelectNextItem() {
		    int             iCurLine;
		    
            EnsureSortedOrder();
		    iCurLine    = SelectedIndex + 1;
		    if (iCurLine < ItemCount) {
		        SelectedIndex = iCurLine;
		    }
		    return(SelectedKey);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectLastItem:                 Select the last item of the selection buffer
	    /// </summary>
	    /// <returns>
	    /// Keys or null if error
	    /// </returns>
	    //
        //********************************************************************************
		public virtual string[] SelectLastItem() {
            EnsureSortedOrder();
		    SelectedIndex = ItemCount - 1;
		    return(SelectedKey);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectItem:                     Select the specified item of the selection buffer
	    /// </summary>
	    /// <param name="arrKeys">          Keys of the item</param>
	    /// <param name="bTrigEvent">       true to trigger event</param>
	    /// <returns>
	    /// true if succeed, false if not found
	    /// </returns>
	    //
        //********************************************************************************
		public virtual bool SelectItem(string[] arrKeys, bool bTrigEvent) {
		    bool            bRetVal;
		    Rowset   rowset;
		    
		    rowset   = SBRootRowset;
		    bRetVal = (rowset.FindUsingPrimaryKey(-1, true, true, true, arrKeys) == 0);
            if (bRetVal && bTrigEvent) {
                OnSBChange(EventArgs.Empty);
            }
            return(bRetVal);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// AddItem:                        Add an item to the selection buffer
	    /// </summary>
	    /// <param name="rowset">           Rowset use as a template for values</param>
	    //
        //********************************************************************************
		public virtual void AddItem(Rowset rowset) {
		    Rowset   rowsetSB;
		    
		    rowsetSB = SBRootRowset;
		    rowsetSB.AddBlankLine();
		    rowsetSB.RowsetDef.CopyFieldsFrom(rowset.RowsetDef, CopyFieldOpt.sdCopyState);
            SaveData(SBRowsetTree);
            OnSBChange(EventArgs.Empty);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// RemoveItem:                     Remove an item from the selection buffer
	    /// </summary>
	    /// <param name="arrKeys">          Keys of the item</param>
	    /// <returns>
	    /// true if succeed, false if not found
	    /// </returns>
	    //
        //********************************************************************************
		public virtual bool RemoveItem(string[] arrKeys) {
		    bool    bRetVal;
		    Rowset   rowset;
		    
		    rowset   = SBRootRowset;
		    bRetVal = (rowset.FindUsingPrimaryKey(-1, true, true, true, arrKeys) == 0);
		    if (bRetVal) {
		        rowset.LineState = LineState.sdLineNew;
		        rowset.DeleteLine();
                SaveData(SBRowsetTree);
                OnSBChange(EventArgs.Empty);
		    }
		    return(bRetVal);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// UpdateItem:                     Update an item of the selection buffer
	    /// </summary>
	    /// <param name="rowset">           Rowset use as a template for values</param>
	    /// <returns>
	    /// true if succeed, false if not found
	    /// </returns>
	    //
        //********************************************************************************
		public virtual bool UpdateItem(Rowset rowset) {
		    bool        bRetVal;
		    Rowset      rowsetSB;
		    string[]    arrKeys;
		    
		    rowsetSB = SBRootRowset;
		    arrKeys  = RepPP.Toolkit.Util.BuildPrimaryKey(rowset.RowsetDef, true);
		    bRetVal  = (rowsetSB.FindUsingPrimaryKey(-1, true, true, true, arrKeys) == 0);
		    if (bRetVal) {
    		    rowsetSB.RowsetDef.CopyFieldsFrom(rowset.RowsetDef, CopyFieldOpt.sdCopyState);
                SaveData(SBRowsetTree);
                OnSBChange(EventArgs.Empty);
    		}
    		return(bRetVal);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectedKey:                    Currently selected key
	    /// </summary>
	    //
        //********************************************************************************
		public virtual string[] SelectedKey {
		    get {
		        return(RepPP.Toolkit.Util.BuildPrimaryKey(SBRootRowset.RowsetDef, true));
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SelectedIndex:                  Currently selected index
	    /// </summary>
	    //
        //********************************************************************************
		public virtual int SelectedIndex {
		    get {
		        return(SBRootRowset.CurrentLine);
		    }
		    set {
		        Rowset   rowsetSB;
		        
		        rowsetSB = SBRootRowset;
		        if (value != rowsetSB.CurrentLine) {
		            rowsetSB.SelectLine(value);
                    OnSBChange(EventArgs.Empty);
                }
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// ItemCount:                      Number of items
	    /// </summary>
	    //
        //********************************************************************************
		public virtual int ItemCount {
		    get {
		        Rowset   rowsetSB;
		        
		        rowsetSB = SBRootRowset;
		        return(rowsetSB.LineCount);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// FilterClause:                   Current filter clause of the selection buffer
	    /// </summary>
	    //
        //********************************************************************************
		public virtual string FilterClause {
		    get {
		        return(m_sbRef.FilterClause);
		    }
            set {
                if (value != m_sbRef.FilterClause) {
                    m_sbRef.FilterClause = value;
                    LoadData(SBRowsetTree);
                    OnSBChange(EventArgs.Empty);
                }
            }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// SBCacheMode:              The Selection Buffer Caching mode
	    /// </summary>
	    //
        //********************************************************************************
		public virtual PageWithSelectionBuffer.SBCacheModeE SBCacheMode {
		    get {
		        return(m_sbRef.SBCacheMode);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// OnSBChange                      Called when the selection buffer data or 
        ///                                 current selection change
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //********************************************************************************
        protected virtual void OnSBChange(EventArgs e) {
		    HttpApplicationFW   appFW;
		    
		    appFW = HttpContext.Current.ApplicationInstance as HttpApplicationFW;
		    if (appFW != null) {
		        appFW.SBChange(this, e);
		    }
		    if (SBChange != null) {
		        SBChange(this, e);
		    }
        }

        //********************************************************************************
        ///<summary>
        /// SaveSortParam:                     Saves the sort criteria of the selection buffer data.
        ///</summary>
        ///<param name="arrSortParam">         Columns and order of the sort</param>
        ///
        //********************************************************************************
        public virtual void SaveSortParam (object[] arrSortParam) {
            m_sbRef.SortParams = arrSortParam;
        }

        //********************************************************************************
        ///<summary>
        /// EnsureSortedOrder:                  Sort the selection buffer data if it is 
        ///                                     not already sorted .
        ///</summary>
        /// <returns>
        /// True:           The sort is successfully performed.
        /// False:          An error occurred during the sort.
        /// </returns>
        //********************************************************************************
        public virtual bool EnsureSortedOrder () {
            object[]    arrSortParam;
            bool        bRetVal = false;

            arrSortParam = m_sbRef.SortParams;
            if (arrSortParam != null && EnsureValidFields(arrSortParam)) {
                SBRootRowset.Sort(arrSortParam);
                m_bSorted = true;
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        ///<summary>
        /// EnsureValidFields:                  Ensures that the sort fields exist in the group
        ///</summary>
        /// <param name="arrSortParam">         Sort Parameters</param>
        /// <returns>
        /// True if the sort params are valid, false otherwise.
        /// </returns>
        //********************************************************************************
        protected virtual bool EnsureValidFields(object[] arrSortParam) {
            bool  bRetVal = true;
            int   iCount;
            RowsetDef rowsetDef;
            FieldDef fldDef;

            rowsetDef    = SBRootRowset.RowsetDef;
            iCount = arrSortParam.Length;
            for (int i = 0; i < iCount; i += 2) {
                fldDef = rowsetDef.FieldDefs[(string)arrSortParam[i]];
                if (fldDef == null || !fldDef.IsVisible) {
                    bRetVal = false;
                    break;
                }
            }
            return(bRetVal);
        }
	}
}
