using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;

namespace RepPP.Framework.Web {
	/// <summary>
	/// Handle the parsing and the cracking of the parameters of the standard command (the one implemented by the PageEditor)
	/// </summary>
	//
	//  Cmds                ::=     Cmd { ';' Cmd }
	//  Cmd                 ::=     CmdName { ',' CmdParam }
	//  CmdName             ::=     Param
	//  CmdParam            ::=     Param
	//  Param               ::=     NonReservedChar { NonReservedChar }
	//  NonReservedChar     ::=     Any char except ',' ';'
	//
	//  StdCmd:             ::=     NoneCmd | EditCmd | NewCmd | DeleteCmd | DuplicateCmd | SaveCmd | DeleteStackCmd
	//  NoneCmd             ::=     'None'
	//  EditCmd             ::=     'Edit' ',' ( EditItemParam | EditPosParam )
	//  EditItemParam       ::=     'Item' ',' PageName ',' StackOp ',' Key
	//  EditPosParam        ::=     'First' | Previous' | 'Next' | 'Last' | 'Top' | 'Up' | 'UpTransaction' | 'Current' {',' PageName ',' StackOp}
	//  NewCmd              ::=     'New' ',' ( NewItemParam | 'Current' )
	//  NewItemParam        ::=     'Item' ',' PageName ',' StackOp
	//  DeleteCmd           ::=     'Delete' ',' 'Current'
	//  DuplicateCmd        ::=     'Duplicate' ',' ( DuplicateItemParam | 'Current' )
	//  DuplicateItemParam  ::=     'Item' ',' PageName ',' StackOp ',' Key
	//  SaveCmd             ::=     'Save'
	//  DeleteStackCmd      ::=     'DeleteStack' ',' StackId
	//  StackOp             ::=     'New' | 'PushTransaction' | 'PushPage' | 'TopTransaction'
	//  PageName            ::=     Param
	//  Key                 ::=     Param
	//  StackId             ::=     Digit { Digit }
	//
	public class StdCmdHandler {
        /// <summary>List of commands</summary>
        private Hashtable                   m_hashCmdList;
        /// <summary>PageEditor holding this command handler</summary>
        protected PageEditor                  m_pageFather;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor for the root transaction
        /// </summary>
        /// <param name="pageEditor">               PageEditor holding this command handler</param>
        //
        //********************************************************************************
		public StdCmdHandler(PageEditor pageEditor) {
		    m_pageFather    = pageEditor;
		    m_hashCmdList   = new Hashtable(8);
            m_hashCmdList.Add("NONE",        new CmdEntry(new CmdEntry.ParseCmdDel(ParseNoneCmd),       null));
            m_hashCmdList.Add("EDIT",        new CmdEntry(new CmdEntry.ParseCmdDel(ParseEditCmd),       new CmdEntry.ExecuteCmdDel(ExecuteEditCmd)));
            m_hashCmdList.Add("NEW",         new CmdEntry(new CmdEntry.ParseCmdDel(ParseNewCmd),        new CmdEntry.ExecuteCmdDel(ExecuteNewCmd)));
            m_hashCmdList.Add("DELETE",      new CmdEntry(new CmdEntry.ParseCmdDel(ParseDeleteCmd),     new CmdEntry.ExecuteCmdDel(ExecuteDeleteCmd)));
            m_hashCmdList.Add("DUPLICATE",   new CmdEntry(new CmdEntry.ParseCmdDel(ParseDuplicateCmd),  new CmdEntry.ExecuteCmdDel(ExecuteDuplicateCmd)));
            m_hashCmdList.Add("SAVE",        new CmdEntry(new CmdEntry.ParseCmdDel(ParseSaveCmd),       new CmdEntry.ExecuteCmdDel(ExecuteSaveCmd)));
            m_hashCmdList.Add("DELETESTACK", new CmdEntry(new CmdEntry.ParseCmdDel(ParseDeleteStackCmd),new CmdEntry.ExecuteCmdDel(ExecuteDeleteStackCmd)));
		}

        //********************************************************************************
        //
        /// <summary>
        /// CommandList:                        List of standard commands
        /// </summary>
        //
        //********************************************************************************
        public virtual Hashtable CommandList {
            get {
                return(m_hashCmdList);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseStackOp:                       Parse a Stack operation
        /// </summary>
        //
        //********************************************************************************
        public PageEditor.NavStackOpE ParseStackOp(string strStackOp) {
            PageEditor.NavStackOpE eRetVal;
            
            strStackOp = strStackOp.ToUpper();
            switch(strStackOp) {
            case "NEW":
                eRetVal = PageEditor.NavStackOpE.NEW;
                break;
            case "PUSHTRANSACTION":
                eRetVal = PageEditor.NavStackOpE.PUSHTRANS;
                break;
            case "PUSHPAGE":
                eRetVal = PageEditor.NavStackOpE.PUSHPAGE;
                break;
            case "TOPTRANSACTION":
                eRetVal = PageEditor.NavStackOpE.TOPTRANS;
                break;
            case "":
                eRetVal = PageEditor.NavStackOpE.NONE;
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidStackOperation);
            }
            return(eRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseNoneCmd:                       Parse the NONE command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseNoneCmd(PageEditor.Cmd cmd, string[] arrParam) {
            if (arrParam.Length != 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_None);
            }
            cmd.UpdateValue = false;
            cmd.NextPage    = "";
            cmd.StackOp     = PageEditor.NavStackOpE.NONE;
            cmd.Params      = new string[0];
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseEditCmd:                       Parse the EDIT command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseEditCmd(PageEditor.Cmd cmd, string[] arrParam) {
            string      strKey;
            string      strSubCommand;

            if (arrParam.Length == 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
            }
            strSubCommand = arrParam[0].ToUpper();
            switch(strSubCommand) {
            case "ITEM":
                if (arrParam.Length != 4) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
                }
                break;
            case "CURRENT":
            case "NEXT":
            case "PREVIOUS":
            case "FIRST":
            case "LAST":
                if (arrParam.Length != 1 && arrParam.Length != 3) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
                }
                break;
            case "UP":
            case "UPTRANSACTION":
            case "TOP":
                if (arrParam.Length != 1) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
                }
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
            }
            switch(strSubCommand) {
            case "ITEM":
                cmd.UpdateValue = true;
                cmd.NextPage    = arrParam[1];
                cmd.StackOp     = ParseStackOp(arrParam[2]);
                strKey          = arrParam[3];
                cmd.Params      = new string[2] { strSubCommand, strKey };
                break;
            case "CURRENT":
            case "NEXT":
            case "PREVIOUS":
            case "FIRST":
            case "LAST":
                cmd.UpdateValue = !m_pageFather.IsRootNode;
                if (arrParam.Length > 1) {
                    cmd.NextPage    = arrParam[1];
                    cmd.StackOp     = ParseStackOp(arrParam[2]);
                } else {
                    cmd.NextPage    = String.Empty;
                    cmd.StackOp     = PageEditor.NavStackOpE.NONE;
                }
                cmd.Params      = new string[1] { strSubCommand };
                break;
            case "UP":
                cmd.UpdateValue = true;
                cmd.NextPage    = "";
                cmd.StackOp     = PageEditor.NavStackOpE.POP;
                cmd.Params      = new string[1] { strSubCommand };
                break;
            case "UPTRANSACTION":
                cmd.UpdateValue = false;
                cmd.NextPage    = "";
                cmd.StackOp     = PageEditor.NavStackOpE.POPTRANS;
                cmd.Params    = new string[1] { strSubCommand };
                break;
            case "TOP":
                cmd.UpdateValue = true;
                cmd.NextPage    = "";
                cmd.StackOp     = PageEditor.NavStackOpE.TOPTRANS;
                cmd.Params      = new string[1] { strSubCommand };
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.DevError);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseNewCmd:                        Parse the NEW command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseNewCmd(PageEditor.Cmd cmd, string[] arrParam) {
            string      strSubCommand;

            if (arrParam.Length == 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_New);
            }
            strSubCommand = arrParam[0].ToUpper();
            switch(strSubCommand) {
            case "ITEM":
                if (arrParam.Length != 3) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_New);
                }
                cmd.UpdateValue = true;
                cmd.NextPage    = arrParam[1];
                cmd.StackOp     = ParseStackOp(arrParam[2]);
                cmd.Params      = new string[1] { strSubCommand };
                break;
            case "CURRENT":
                if (arrParam.Length != 1) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_New);
                }
                cmd.UpdateValue = true;
                cmd.NextPage    = String.Empty;
                cmd.StackOp     = PageEditor.NavStackOpE.NONE;
                cmd.Params      = new string[1] { strSubCommand };
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_New);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseDeleteCmd:                     Parse the DELETE command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseDeleteCmd(PageEditor.Cmd cmd, string[] arrParam) {
            string      strSubCommand;

            if (arrParam.Length == 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Delete);
            }
            strSubCommand = arrParam[0].ToUpper();
            if (strSubCommand != "CURRENT") {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Delete);
            }
            if (arrParam.Length != 1) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Delete);
            }
            cmd.UpdateValue = false;
            cmd.NextPage    = String.Empty;
            cmd.StackOp     = PageEditor.NavStackOpE.NONE;
            cmd.Params      = new string[1] { strSubCommand };
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseDuplicateCmd:                  Parse the DUPLICATE command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseDuplicateCmd(PageEditor.Cmd cmd, string[] arrParam) {
            string      strKey;
            string      strSubCommand;

            if (arrParam.Length == 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Duplicate);
            }
            strSubCommand = arrParam[0].ToUpper();
            switch(strSubCommand) {
            case "ITEM":
                if (arrParam.Length != 4) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_Duplicate);
                }
                cmd.UpdateValue = true;
                cmd.NextPage    = arrParam[1];
                cmd.StackOp     = ParseStackOp(arrParam[2]);
                strKey          = arrParam[3];
                cmd.Params      = new string[2] { strSubCommand, strKey };
                break;
            case "CURRENT":
                if (arrParam.Length != 1) {
                    throw new FWException(FWException.EditorExceptionE.InvalidCmd_Duplicate);
                }
                cmd.UpdateValue = true;
                cmd.NextPage    = String.Empty;
                cmd.StackOp     = PageEditor.NavStackOpE.NONE;
                cmd.Params      = new string[1] { strSubCommand };
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Duplicate);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseSaveCmd:                       Parse the SAVE command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseSaveCmd(PageEditor.Cmd cmd, string[] arrParam) {
            if (arrParam.Length != 0) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Save);
            }
            cmd.UpdateValue = true;
            cmd.NextPage    = String.Empty;
            cmd.StackOp     = PageEditor.NavStackOpE.NONE;
            cmd.Params      = new string[0];
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseDeleteStackCmd:                Parse the DELETESTACK command
        /// </summary>
        /// <param name="cmd">                  Command to parse</param>
        /// <param name="arrParam">             Parameters to the command</param>
        //
        //********************************************************************************
        protected virtual void ParseDeleteStackCmd(PageEditor.Cmd cmd, string[] arrParam) {
            string      strStackId;

            if (arrParam.Length != 1) {
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_DeleteStack);
            }
            strStackId      = arrParam[0];
            cmd.UpdateValue = false;
            cmd.NextPage    = String.Empty;
            cmd.StackOp     = PageEditor.NavStackOpE.NONE;
            cmd.Params      = new string[1] { strStackId };
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseCmd:                           Parse a command to a Cmd object
        /// </summary>
        /// <param name="strCmdName">           Command to parse</param>
        /// <param name="arrParams">            Parameters to the command</param>
        /// <returns>
        /// Cmd object or null if command not found
        /// </returns>
        //
        //********************************************************************************
        public virtual PageEditor.Cmd ParseCmd(string strCmdName, string[] arrParams) {
            PageEditor.Cmd  cmdRetVal;
            Hashtable       hashCmdList;
            CmdEntry        cmdEntry;
            
            strCmdName  = strCmdName.ToUpper();
            hashCmdList = CommandList;
            cmdEntry    = hashCmdList[strCmdName] as CmdEntry;
            if (cmdEntry == null) {
                cmdRetVal = null;
            } else {
                cmdRetVal           = new PageEditor.Cmd();
                cmdRetVal.Name      = strCmdName;
                cmdEntry.Parse(cmdRetVal, arrParams);
            }
            return(cmdRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseCmd:                           Parse a command to a Cmd object
        /// </summary>
        /// <param name="strCmd">               Command to parse</param>
        /// <returns>
        /// Command object
        /// </returns>
        //
        //********************************************************************************
        protected virtual PageEditor.Cmd ParseCmd(string strCmd) {
            PageEditor.Cmd  cmdRetVal;
            string          strCmdName;
            string[]        arrParams;
            int             iIndex;
            
            iIndex = strCmd.IndexOf(',');
            if (iIndex == -1) {
                strCmdName = strCmd;
                arrParams  = new string[0];
            } else {
                strCmdName = strCmd.Substring(0, iIndex);
                arrParams  = strCmd.Substring(iIndex + 1).Split(',');
            }
            cmdRetVal = ParseCmd(strCmdName, arrParams);
            if (cmdRetVal == null) {
                throw new FWException(FWException.EditorExceptionE.InvalidCommand);
            }
            return(cmdRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseCmds:                          Parse the list of commands
        /// </summary>
        /// <param name="strCmds">              Commands to parse</param>
        /// <returns>
        /// Array of command object
        /// </returns>
        //
        //********************************************************************************
        public virtual PageEditor.Cmd[] ParseCmds(string strCmds) {
            PageEditor.Cmd[]    arrRetVal = null;
            string[]            arrCmd;
            int                 iIndex;
            
            if (strCmds != null && strCmds != String.Empty) {
                arrCmd      = strCmds.Split(';');
                arrRetVal   = new PageEditor.Cmd[arrCmd.Length];
                iIndex      = 0;
                foreach (string strCmd in arrCmd) {
                    arrRetVal[iIndex] = ParseCmd(strCmd);
                    iIndex++;
                }
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteEditCmd:                     Execute the EDIT command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteEditCmd(PageEditor.Cmd cmd) {
            bool    bRetVal;
            
            switch(cmd.Params[0]) {
            case "ITEM":
                bRetVal = m_pageFather.EditItem(cmd.Params[1]);
                break;
            case "NEXT":
                bRetVal = m_pageFather.EditPosition(PageEditor.PositionE.NEXT);
                break;
            case "PREVIOUS":
                bRetVal = m_pageFather.EditPosition(PageEditor.PositionE.PREVIOUS);
                break;
            case "FIRST":
                bRetVal = m_pageFather.EditPosition(PageEditor.PositionE.FIRST);
                break;
            case "LAST":
                bRetVal = m_pageFather.EditPosition(PageEditor.PositionE.LAST);
                break;
            case "CURRENT":
            case "UP":
            case "UPTRANSACTION":
            case "TOP":
                bRetVal = m_pageFather.EditCurrent();
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Edit);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteNewCmd:                      Execute the NEW command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteNewCmd(PageEditor.Cmd cmd) {
            return(m_pageFather.NewItem());
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteDeleteCmd:                   Execute the DELETE command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteDeleteCmd(PageEditor.Cmd cmd) {
            return(m_pageFather.DeleteCurrent());
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteDuplicateCmd:                Execute the DUPLICATE command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteDuplicateCmd(PageEditor.Cmd cmd) {
            bool        bRetVal;
            string      strSubCommand;

            strSubCommand = cmd.Params[0];
            switch(strSubCommand) {
            case "ITEM":
                bRetVal = m_pageFather.DuplicateItem(cmd.Params[1]);
                break;
            case "CURRENT":
                bRetVal = m_pageFather.DuplicateCurrent();
                break;
            default:
                throw new FWException(FWException.EditorExceptionE.InvalidCmd_Duplicate);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteSaveCmd:                     Execute the SAVE command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteSaveCmd(PageEditor.Cmd cmd) {
            return(m_pageFather.SaveCurrent());
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteDeleteStackCmd:              Execute the DELETESTACK command
        /// </summary>
        /// <param name="cmd">                  Command to execute</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool ExecuteDeleteStackCmd(PageEditor.Cmd cmd) {            
            int     iStackId;
            
            try {
                iStackId = Int32.Parse(cmd.Params[0]);
            } catch {
                throw new FWException(FWException.EditorExceptionE.BadParameter);
            }
            return(m_pageFather.DeleteStack(iStackId));
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExecuteCmd:                         Execute the command find in the command object
        /// </summary>
        /// <param name="cmd">                  Command object</param>
        /// <returns>
        /// true if succeed, false if error
        /// </returns>
        //
        //********************************************************************************
        public virtual bool ExecuteCmd(PageEditor.Cmd cmd) {
            bool            bRetVal;
            Hashtable       hashCmdList;
            CmdEntry        cmdEntry;
            
            hashCmdList = CommandList;
            cmdEntry    = hashCmdList[cmd.Name] as CmdEntry;
            if (cmdEntry == null) {
                throw new FWException(FWException.EditorExceptionE.InvalidCommand, cmd.Name);
            }
            bRetVal = (cmdEntry.Execute != null) ? cmdEntry.Execute(cmd) : true;
            return(bRetVal);
        }

        /// <summary>
        /// Defines a command parser/executer
        /// </summary>
        protected class CmdEntry {
            /// <summary>Defines the delegate type for the command parser</summary>
            public delegate void                ParseCmdDel(PageEditor.Cmd cmd, string[] arrParams);
            /// <summary>Defines the delegate type for the method executing the command</summary>
            public delegate bool                ExecuteCmdDel(PageEditor.Cmd cmd);
            /// <summary>Delegate to the command parser</summary>
            public  ParseCmdDel                 Parse;
            /// <summary>Delegate to the method excecuting the command</summary>
            public  ExecuteCmdDel               Execute;

            //********************************************************************************
            //
	        /// <summary>
	        /// ctor:                           Class constructor
	        /// </summary>
            /// <param name="parse">            Delegate to the parser of the command</param>
            /// <param name="exec">             Delegate to the method executing the command</param>
	        //
            //********************************************************************************
            public CmdEntry(ParseCmdDel parse, ExecuteCmdDel exec) {
                Parse = parse;
                Execute = exec;
            }
        } // Class CmdEntry
	} // Class StdCmdHandler
} // Namespace
