using System;
using System.Drawing.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports atom property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class AtmControlUtil : ControlUtil {
	    /// <summary>Common interface for all controls attached to a Rep++ Atom</summary>
	    public interface IAtmControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Field name</summary>
	        AtmFullName                 AtomFullName {  get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>Atom text attached to the control</summary>
	    protected string                m_strAtomText;
	    /// <summary>Full name of the atom</summary>
	    protected AtmFullName           m_atmFullName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="ctrlHelper">       Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public AtmControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly) : base(parent, ctrlHelper, bResDispenserOnly) {
		    m_strAtomText   = null;
		    m_atmFullName   = new AtmFullName();
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPInitialized:           Called when the Rep++ is initialized
		/// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        public override void RepPPInitialized(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_strAtomText = null;
                break;
            }
            base.RepPPInitialized(sender, e);
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPResetting:             Called when the Rep++ is reset. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public override void RepPPResetting(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_strAtomText = "<" + AtomFullName.ToString() + ">";
                break;
            default:
                break;
            }
            base.RepPPResetting(sender, e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AtomFullName:          Full name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public AtmFullName AtomFullName {
		    get {
		        return(m_atmFullName);
		    }
		    set {
		        m_atmFullName   = value;
		        m_strAtomText   = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// AtomText:               Atom text attached to the control
		/// </summary>
        //
        //********************************************************************************
		public string AtomText {
		    get {
		        RepPP.Application   app;
		        
		        if (m_strAtomText == null) {
		            app = Application;
                    if (app == null || m_atmFullName.AtomName == String.Empty) {
                        m_strAtomText = "<" + m_atmFullName.AtomName + ">";
                    } else if (!app.IsAtomExist(m_atmFullName.AtomName, m_atmFullName.AtomType)) {
                        m_strAtomText = "<?" + m_atmFullName + "?>";
                    } else {
		                m_strAtomText = app.GetAtomText(m_atmFullName.AtomName, m_atmFullName.AtomType);
		            }
		        }
		        return(m_strAtomText);
		    }
		}
   
    } // Class AtmControlUtil
} // Namespace