//*********************************************************
//
// CheckIntegrity:          Checks integrity between tables
//
// Author:	                Consyst SQL inc.
//
// (c) 2005 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Collections;

namespace RepPP.Toolkit {
	/// <summary>
	/// Checks integrity between tables
	/// </summary>
	public class CheckIntegrity {
        /// <summary>Rep++ Connection</summary>
        RepPP.Connection        m_connection;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public CheckIntegrity(RepPP.Connection connection) {
            m_connection = connection;
		}

        //*********************************************************
        //
        /// <summary>
        /// FindFldUsingCol :           Find Field by one Column
        /// </summary>
        /// <param name="fldDefs">      List of fields</param>
        /// <param name="strColName">   Column Name</param>
        /// <returns>
        /// Found field
        /// </returns>
        //
        //*********************************************************
        public RepPP.FieldDef FindFldUsingCol(FieldDefs fldDefs, string strColName) {
            RepPP.FieldDef fldRetVal = null;

            foreach (RepPP.FieldDef fld in fldDefs) {
                if (fld.ColName == strColName) {
                    fldRetVal = fld;
                    break;
                }
            }
            return(fldRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildSqlCmd :                   Build Sql Command from parameters
        /// </summary>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="strWhereClause">   Where clause to applied to SQL</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <returns>
        /// Sql Command
        /// </returns>
        //
        //********************************************************************************
        protected virtual string BuildSqlCmd(string                strTableName, 
                                             string                strWhereClause, 
                                             string[]              arrRetColList) {
            string          strRetVal;
            string          strRetColList;

            strRetColList = String.Join(",", arrRetColList);
            strRetVal     = String.Format("SELECT DISTINCT {0} FROM {1} WHERE {2} ORDER BY {0}", strRetColList, strTableName, strWhereClause);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildIsItemExistSqlCmd :        Build Sql Command to verify if item 
        ///                                 is referenced somewhere
        /// </summary>
        /// <param name="rowset">           Rowset to check</param>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="arrPKColList">     Reference Columns where to check</param>
        /// <param name="arrPKFldDefList">  FieldDef containing the value to check. If null, the field attached to arrPKColList are used.</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <returns>
        /// Sql Command
        /// </returns>
        //
        //********************************************************************************
        protected virtual string BuildReferencedBySqlCmd(RepPP.Rowset       rowset, 
                                                         string             strTableName, 
                                                         string[]           arrPKColList, 
                                                         RepPP.FieldDef[]   arrPKFldDefList, 
                                                         string[]           arrRetColList) {
            string          strRetVal;
            string          strWhereClause;
            int             iIndex;
            RepPP.FieldDef  fld;

            if (arrPKFldDefList != null && arrPKFldDefList.Length != arrPKColList.Length) {
                throw new System.ApplicationException("IsReferencedBy : The primary keys list doesn't fit with the foreign primary keys list");
            }
            iIndex         = 0;
            strWhereClause = "";
            foreach (string strColName in arrPKColList) {
                fld = (arrPKFldDefList != null) ? arrPKFldDefList[iIndex++] : 
                                                  FindFldUsingCol(rowset.RowsetDef.FieldDefs, strColName.ToUpper());
                if (fld == null) {
                    throw new System.ApplicationException(String.Format("IsReferencedBy : Cannot match a field from RowsetDef '{0}' with the Column : '{1}'", rowset.RowsetDef.Name, strColName));
                }
                if (strWhereClause != String.Empty) {
                    strWhereClause += " AND ";
                }
                strWhereClause += strColName + "=:" + fld.RowsetDef.Name + "." + fld.Name;
            }
            strRetVal = BuildSqlCmd(strTableName, strWhereClause, arrRetColList);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsIsReferencedByExec :          Execute the check if item in Rowset is referenced by one or more foreign tables
        /// </summary>
        /// <param name="strSqlCmd">        SQL Cmd to check item</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="arrReferencedBy">  Array of array of referenced values</param>                            
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool IsIsReferencedByExec(string          strSqlCmd, 
                                                    string[]        arrRetColList,
                                                    out string[][]  arrReferencedBy) {
            RepPP.SqlCursor         sqlCursor;
            ArrayList               arrColList;
            string[]                arrColValue;
            int                     iColCount;
            int                     i;
            bool                    bRetVal;

            arrReferencedBy = null;
            if (m_connection.Execute(strSqlCmd, RepPP.SqlOptimization.sdOptMultiRow, out sqlCursor) != 0) {
                throw new System.ApplicationException("CheckIntegrityUI - IsReferencedBy : Unable to execute SQL Command, check your parameters");
            }
            using (sqlCursor) {
                arrColList = new ArrayList();
                iColCount = arrRetColList.Length;
                while (sqlCursor.Next() == 0) {
                    arrColValue = new string[iColCount];
                    for (i = 0; i < iColCount; i++) {
                        arrColValue[i] = sqlCursor.GetColumnValue(i);
                    }
                    arrColList.Add(arrColValue);
                }
            }
            if (arrColList.Count != 0) {
                arrReferencedBy = new string[arrColList.Count][];
                arrColList.CopyTo(arrReferencedBy);
                bRetVal = true;
            } else {
                bRetVal = false;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Rowset is referenced by one or more foreign tables
        /// </summary>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="strWhereClause">   where to check</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="arrReferencedBy">  Array of array of referenced values</param>                            
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldDefList
        //  SELECT QUNAME FROM SYSTQUEST WHERE myWhereClause
        //
        //********************************************************************************
        public virtual bool IsReferencedBy(string                strTableName, 
                                           string                strWhereClause,
                                           string[]              arrRetColList,
                                           out string[][]        arrReferencedBy) {
            string                  strSqlCmd;
            bool                    bRetVal;

            strSqlCmd = BuildSqlCmd(strTableName,
                                    strWhereClause,
                                    arrRetColList);
            bRetVal   = IsIsReferencedByExec(strSqlCmd, arrRetColList, out arrReferencedBy);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Rowset is referenced by one or more foreign tables
        /// </summary>
        /// <param name="rowset">            Rowset to check</param>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="arrPKColList">     Reference Columns where to check</param>
        /// <param name="arrPKFldDefList">     FieldDef containing the value to check. If null, the field attached to arrPKColList are used.</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="arrReferencedBy">  Array of array of referenced values</param>                            
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldDefList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        public virtual bool IsReferencedBy(RepPP.Rowset         rowset, 
                                           string               strTableName, 
                                           string[]             arrPKColList, 
                                           RepPP.FieldDef[]     arrPKFldDefList,
                                           string[]             arrRetColList,
                                           out string[][]       arrReferencedBy) {
            string  strSqlCmd;
            bool    bRetVal;

            strSqlCmd = BuildReferencedBySqlCmd(rowset, 
                                                strTableName, 
                                                arrPKColList, 
                                                arrPKFldDefList,
                                                arrRetColList);
            bRetVal   = IsIsReferencedByExec(strSqlCmd, arrRetColList, out arrReferencedBy);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Rowset is referenced by one or more foreign tables
        /// </summary>
        /// <param name="rowset">           Rowset to check</param>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="arrPKColList">     Reference Columns where to check</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="arrReferencedBy">  Array of array of referenced values</param>                            
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldDefList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        public virtual bool IsReferencedBy(RepPP.Rowset     rowset, 
                                           string           strTableName, 
                                           string[]         arrPKColList, 
                                           string[]         arrRetColList,
                                           out string[][]   arrReferencedBy) {
            return(IsReferencedBy(rowset, strTableName, arrPKColList, null, arrRetColList, out arrReferencedBy));
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildIsItemExistSqlCmd :            Build Sql Command to verify if an item is already defined
        /// </summary>
        /// <param name="rowset">               Rowset to check</param>
        /// <param name="strFKTableName">       Name of the table where we check for item existance</param>
        /// <param name="arrFKTablePK">         Primary key of the foreign table.</param>
        /// <param name="arrFldDefToCheck">     FieldDef where to check in case</param>
        /// <param name="strUserWhereClause">   Clause to add to the WHERE clause</param>
        /// <returns>
        /// Sql Command
        /// </returns>
        //
        //********************************************************************************
        protected virtual string BuildIsItemExistSqlCmd(RepPP.Rowset        rowset, 
                                                        string              strFKTableName, 
                                                        string[]            arrFKTablePK,
                                                        RepPP.FieldDef[]    arrFldDefToCheck,
                                                        string              strUserWhereClause) {
            RepPP.FieldDef  fld;
            string          strRetVal;
            string          strWhereClause;
            int             iIndex;

            if (arrFKTablePK.Length != arrFldDefToCheck.Length) {
                throw new System.ApplicationException("IsItemExist : The primary keys list doesn't fit with the foreign primary keys list");
            }
            strWhereClause = "";
            iIndex         = 0;
            foreach (string strColName in arrFKTablePK) {
                fld = arrFldDefToCheck[iIndex++];
                if (strWhereClause != String.Empty) {
                    strWhereClause += " AND ";
                }
                strWhereClause += strColName + "=:" + fld.RowsetDef.Name + "." + fld.Name;
            }
            if (strUserWhereClause != null && strUserWhereClause != string.Empty) {
                strWhereClause += " " + strUserWhereClause;
            }
            strRetVal = string.Format("SELECT {0} FROM {1} WHERE {2} FOR UPDATE",
                                      String.Join(",", arrFKTablePK),
                                      strFKTableName,
                                      strWhereClause);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddPKToArray:                   Build a list of PK of a rowset definition (with ancestors)
        /// </summary>
        /// <param name="arr">              Array to fill</param>
        /// <param name="node">             Node of the rowset tree definition</param>
        /// <param name="rowsetDef">        RowsetDef containing the PK.</param>
        //
        //********************************************************************************
        private void AddPKToArray(ArrayList arr, RepPP.RowsetTreeDefNode node, RepPP.RowsetDef rowsetDef) {
            if (node != null) {
                node = node.Parent as RowsetTreeDefNode;
                if (node != null) {
                    AddPKToArray(arr, node, node.RowsetDef);
                }
            }
            foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                if (fld.IsPrimaryKey) {
                    arr.Add(fld);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                       Check if item exist in rowset (Referencing)
        /// </summary>
        /// <param name="rowset">               Rowset to check</param>
        /// <param name="strFKTableName">       Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">         Primary key of the foreign table.</param>
        /// <param name="arrFldDefToCheck">     FieldDef where to check in case. If null, the primary key is used</param>
        /// <param name="strUserWhereClause">   Clause to add to the WHERE clause</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        // SELECT GINST.arrFldDefToCheck FROM strFKTableName WHERE arrFKTablePK=:GINST.arrFldDefToCheck
        //
        //********************************************************************************
        public virtual bool IsItemExist(RepPP.Rowset        rowset, 
                                        string              strFKTableName, 
                                        string[]            arrFKTablePK,
                                        RepPP.FieldDef[]    arrFldDefToCheck,
                                        string              strUserWhereClause) {
            bool                bRetVal     = false;
            RepPP.SqlCursor     sqlCursor;
            ArrayList           arr;
            string              strSqlCmd;

            if (arrFldDefToCheck == null) {
                arr  = new ArrayList();
                AddPKToArray(arr, rowset.RowsetTreeDefNode, rowset.RowsetDef);
                arrFldDefToCheck = new RepPP.FieldDef[arr.Count];
                arr.CopyTo(arrFldDefToCheck);
            }
            strSqlCmd = BuildIsItemExistSqlCmd(rowset, strFKTableName, arrFKTablePK, arrFldDefToCheck, strUserWhereClause);
            if (m_connection.Execute(strSqlCmd, RepPP.SqlOptimization.sdOptMultiRow, out sqlCursor) != 0) {
                throw new System.ApplicationException("IsItemExist : Unable to execute SQL Command, check your parameters");
            }
            using(sqlCursor) {                
                while (sqlCursor.Next() == 0) {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                   Check if item in Rowset exist (Referencing)
        /// </summary>
        /// <param name="rowset">           Rowset to check</param>
        /// <param name="strFKTableName">   Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">     Primary key of the foreign table.</param>
        /// <param name="arrFldDefToCheck"> FieldDef where to check in case. If null, the primary key is used</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        // SELECT GINST.arrFldDefToCheck FROM strFKTableName WHERE arrFKTablePK=:GINST.arrFldDefToCheck
        //
        //********************************************************************************
        public virtual bool IsItemExist(RepPP.Rowset        rowset, 
                                        string              strFKTableName, 
                                        string[]            arrFKTablePK,
                                        RepPP.FieldDef[]    arrFldDefToCheck) {
            return(IsItemExist(rowset, strFKTableName, arrFKTablePK, arrFldDefToCheck, null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                       Check if item exist in Rowset (Referencing)
        /// </summary>
        /// <param name="rowset">               Rowset to check</param>
        /// <param name="strFKTableName">       Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">         Primary key of the foreign table.</param>
        /// <param name="strUserWhereClause">   Clause to add to the WHERE clause</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        // SELECT COUNT(*) FROM strFKTableName WHERE arrFKTablePK=:Field attached to arrFKTablePK
        //
        //********************************************************************************
        public virtual bool IsItemExist(RepPP.Rowset    rowset, 
                                        string          strFKTableName, 
                                        string[]        arrFKTablePK,
                                        string          strUserWhereClause) {
            return(IsItemExist(rowset, strFKTableName, arrFKTablePK, null, strUserWhereClause));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                   Check if item in Rowset exist (Referencing)
        /// </summary>
        /// <param name="rowset">           Rowset to check</param>
        /// <param name="strFKTableName">   Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">     Primary key of the foreign table.</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        // SELECT COUNT(*) FROM strFKTableName WHERE arrFKTablePK=:Field attached to arrFKTablePK
        //
        //********************************************************************************
        public virtual bool IsItemExist(RepPP.Rowset    rowset, 
                                        string          strFKTableName, 
                                        string[]        arrFKTablePK) {
            return(IsItemExist(rowset, strFKTableName, arrFKTablePK, null, null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsPKExist :                 Check if the PK exist
        /// </summary>
        /// <param name="rowset">       Rowset to check</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        //
        //********************************************************************************
        public virtual bool IsPKExist(RepPP.Rowset rowset) {
            bool            bRetVal;
            ArrayList       arr;
            string[]        arrTablePK;
            int             iIndex;

            arr = new ArrayList();
            AddPKToArray(arr, rowset.RowsetTreeDefNode, rowset.RowsetDef);
            arrTablePK = new string[arr.Count];
            iIndex     = 0;
            foreach (RepPP.FieldDef fld in arr) {
                arrTablePK[iIndex++] = fld.ColName;
            }
            bRetVal = IsItemExist(rowset, rowset.RowsetDef.TableName, arrTablePK);
            return(bRetVal);
        }
	} // Class CheckIntegrity
} // Namespace
