﻿using System;
using System.Collections;

namespace RepPP.Toolkit {
	/// <summary>
	/// Controls text translator using atoms
	/// </summary>
	public class ControlTextTranslatorBase {
        /// <summary>
        /// Rep++ application object
        /// </summary>
        protected RepPP.IApplication                m_app;
        /// <summary>
        /// Hash table to access atom items text
        /// </summary>
        protected Hashtable                         m_hash;
        /// <summary>
        /// Father translator
        /// </summary>
        protected ControlTextTranslatorBase         m_fatherTranslator;
        /// <summary>
        /// True to indicate case-sesitive matches
        /// </summary>
        protected bool                              m_bCaseSensitive;
#if REPDEBUG 
        /// <summary>
        /// Hash table containing the items text used
        /// </summary>
        protected Hashtable                         m_hashUsed;
        /// <summary>
        /// Atom list used to translate the control and its children.
        /// </summary>
        protected String                            m_strAtomListName   = String.Empty;
#endif

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="bCaseSensitive">   True to indicate case-sesitive matches</param>
        //
        //*********************************************************        
		public ControlTextTranslatorBase(RepPP.IApplication app, string [] arrAtomName, bool bCaseSensitive)	{
		    string      strText;
		    string      strKey;
		    string      strValue;
		    string[]    arrLine;
		    int         iPos;
		    
		    m_app              = app;
            m_bCaseSensitive   = bCaseSensitive;
            m_fatherTranslator = null;
		    if (arrAtomName.Length == 0) {
		        throw new System.ApplicationException("Atom array is empty");
		    }
            strText = "";
		    foreach (string strAtomText in arrAtomName) {
		        if (m_app.IsAtomExist(strAtomText, RepPP.AtomType.sdAtomMsg)) {
#if REPDEBUG 
                    m_strAtomListName += strAtomText + "; ";
#endif
                    if (strText != String.Empty) {
                        strText += "\r";
                    }
		            strText += m_app.GetAtomText(strAtomText, RepPP.AtomType.sdAtomMsg) + "\r\n";
                } else {
#if REPDEBUG 
                    System.Diagnostics.Debug.WriteLine(String.Format("Translation: This Atom {0} for Translation doesn't exist !", strAtomText));
#endif
                }
            }
            strText = strText.Replace("\n", "");
	        arrLine = strText.Split('\r');
	        m_hash  = new Hashtable(arrLine.Length);
	        foreach (string strLine in arrLine) {
                if (!strLine.Trim().StartsWith("--")) {
		            iPos  = strLine.IndexOf('=');
		            if (iPos != -1) {
		                strKey      = strLine.Substring(0, iPos);
		                strValue    = strLine.Substring(iPos + 1);
                        if (!m_bCaseSensitive) {
                            strKey = strKey.ToUpper();
                        }
		                if (!m_hash.ContainsKey(strKey)) {
		                    m_hash.Add(strKey, strValue);
		                } else {
		                    m_hash[strKey] = strValue;
		                }
                    }
                }
            }
#if REPDEBUG 
            m_hashUsed = (Hashtable)m_hash.Clone();
#endif
		}

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="app">          Rep++ application object</param>
        /// <param name="arrAtomName">  Array of atoms</param>
        //
        //*********************************************************        
        public ControlTextTranslatorBase(RepPP.IApplication app, string[] arrAtomName) : this(app, arrAtomName, true) {
        }

        //*********************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="fatherTranslator"> Father translator</param>
        //
        //*********************************************************        
		public ControlTextTranslatorBase(RepPP.IApplication           app, 
                                         string []                    arrAtomName, 
                                         ControlTextTranslatorBase    fatherTranslator) : this(app, arrAtomName, true) {
            m_fatherTranslator = fatherTranslator;
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateControlTextTranslator:    Creates a new control text translator using the same type as
        ///                                 the father.
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="fatherTranslator"> Father translator</param>
        /// <returns>
        /// New object
        /// </returns>
        //
        //*********************************************************        
		public static ControlTextTranslatorBase CreateControlTextTranslator(RepPP.IApplication           app, 
                                                                            string []                    arrAtomName, 
                                                                            ControlTextTranslatorBase    fatherTranslator) {
            ControlTextTranslatorBase           transRetVal;
            System.Reflection.ConstructorInfo   ctorInfo;
            
            if (fatherTranslator == null) {
                transRetVal = new ControlTextTranslatorBase(app, arrAtomName);
            } else {
                ctorInfo = fatherTranslator.GetType().GetConstructor(new Type[] {typeof(RepPP.IApplication),
                                                                                 typeof(string[]),
                                                                                 typeof(ControlTextTranslatorBase)});
                transRetVal = (ctorInfo == null) ? new ControlTextTranslatorBase(app, arrAtomName, fatherTranslator) :
                                                   ctorInfo.Invoke(new object[] {app, arrAtomName, fatherTranslator} ) as ControlTextTranslatorBase;
            }
            return(transRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// GetItemText:                Get the item text defined in the atom 
        /// </summary>
        /// <param name="strItemName">  Item name</param>
        /// <returns>
        /// Itenm text
        /// </returns>
        //
        //*********************************************************                
        public virtual string GetItemText(string strItemName) {
            return(GetItemText(strItemName, String.Empty));
        }

        //*********************************************************
        //
        /// <summary>
        /// GetItemText:                Get the item text defined in the atom 
        /// </summary>
        /// <param name="strItemName">  Item name</param>
        /// <param name="strAttrName">  Item attribute (Text, Tooltip, ...)</param>
        /// <returns>
        /// Itenm text
        /// </returns>
        //
        //*********************************************************                
        public virtual string GetItemText(string strItemName, string strAttrName) {
            string  strRetVal;
            string  strKey;

            strKey = (strAttrName == String.Empty) ? strItemName : strItemName + "." + strAttrName;
            if (!m_bCaseSensitive) {
                strKey = strKey.ToUpper();
            }
            if (m_hash.ContainsKey(strKey)) {
                strRetVal = (string)m_hash[strKey];
                strRetVal = strRetVal.Replace("\\r", "\r").Replace("\\n", "\n");
#if REPDEBUG 
                if (m_hashUsed.ContainsKey(strKey)) {
                    m_hashUsed.Remove(strKey);
                }
#endif
            } else if (m_fatherTranslator != null) { 
                strRetVal = m_fatherTranslator.GetItemText(strItemName, strAttrName);
            } else {
                strRetVal = null;
            }
            return(strRetVal);
        }        

	} // End class ControlTextTranslatorBase

}// End namespace
