using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class ControlUtil : IDisposable {
	    /// <summary>Interface that must be supported by all control imbedding this class</summary>
	    public interface IRepControl {
	        /// <summary>Rep++ is reinitialized in design mode</summary>
	        void            RepPPReInit();
            /// <summary>Rep++ is reset in design mode</summary>
            void            RepPPReset();
	    }

        /// <summary>Interface that must be supported by all control helper imbedding in this class</summary>
        public interface IControlUtilHelper {
            /// <summary>Control Util parent used in class where the interface is implemented</summary>
            ControlUtil     CtrlUtilParent { set; }
            /// <summary>Get component Site</summary>
            ISite           GetSite();
            /// <summary>Indicate if the control is in design mode</summary>
            Boolean         IsDesignMode();
            /// <summary>Used when the control need to update</summary>
            void            UpdateControl();
        }

	    /// <summary>Parent WEB control</summary>
	    private object                  m_parent;
	    /// <summary>Rep++ Application object</summary>
	    private Application             m_app;
        /// <summary>Control Util Helper used depending the control</summary>
        private IControlUtilHelper      m_ctrlUtilHelper;
	    /// <summary>Must use the Resource Dispenser only if true</summary>
	    private bool                    m_bResDispenserOnly;
	    /// <summary>true if application holder reference counting is disabled</summary>
	    private bool                    m_bNoRefCount;
	    /// <summary>Was unable to get the application object. Wait for Rep++ Reinitialization to try again</summary>
	    private bool                    m_bInError;
        /// <summary>true if control registred to the InitNotification</summary>
        private bool                    m_bRegistred;
        /// <summary>Cookie return at the registration</summary>
        private object                  m_cookieRegistration;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="parent">           Parent Web control</param>
        /// <param name="ctrlHelper">       Control Util Helper used depending the control</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        /// <param name="bNoRefCount">      true if parent cannot always call dispose. In this case, no notification will be sent when Rep++ is reinitialized</param>
        //
        //********************************************************************************
        public ControlUtil(object parent, IControlUtilHelper ctrlHelper, bool bResDispenserOnly, bool bNoRefCount) {
            m_parent                        = parent;
            m_ctrlUtilHelper                = ctrlHelper;
            m_ctrlUtilHelper.CtrlUtilParent = this;
		    m_app                           = null;
		    m_bResDispenserOnly             = bResDispenserOnly;
		    m_bNoRefCount                   = bNoRefCount;
            m_bInError                      = false;
            m_bRegistred                    = false;
            m_cookieRegistration            = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="parent">           Parent Web control</param>
        /// <param name="ctrlHelper">       Control Util Helper used depending the control</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public ControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly) : this(parent, ctrlHelper, bResDispenserOnly, false) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                Dispose the object
        /// </summary>
        //
        //********************************************************************************
        public virtual void Dispose() {
            RepPP.InitNotificationEventHandler  delInit;
            RepPP.InitNotificationEventHandler  delReset;

		    if (m_app != null) {
		        if (!m_bNoRefCount) {
		            m_app.Release();
		        }
		        m_app = null;
		    }
            if (m_bRegistred) {
                delInit  = new RepPP.InitNotificationEventHandler(this.RepPPInitialized);
                delReset = new RepPP.InitNotificationEventHandler(this.RepPPResetting);
                InitNotification.UnregisterControl(delInit, delReset, m_cookieRegistration);
                m_bRegistred         = false;
                m_cookieRegistration = null;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Site:               Returns the site object
		/// </summary>
        //
        //********************************************************************************
		public ISite Site {
		    get {
                return(m_ctrlUtilHelper.GetSite());
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Parent:                     Parent control
		/// </summary>
        //
        //********************************************************************************
		public object Parent {
            get {
                return(m_parent);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// IsDesignMode:               Indicates if the control is in design mode
		/// </summary>
        //
        //********************************************************************************
		public virtual bool IsDesignMode {
            get {
		        return(m_ctrlUtilHelper.IsDesignMode());
            }
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPInitialized:           Called when the Rep++ is initialized. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public virtual void RepPPInitialized(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_bInError = false;
                m_ctrlUtilHelper.UpdateControl();
                break;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPResetting:             Called when the Rep++ is reset. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public virtual void RepPPResetting(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                if (m_parent is IRepControl) {
                    ((IRepControl)m_parent).RepPPReset();
                }
                break;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Application:            Returns the Rep++ application object
		/// </summary>
        //
        //********************************************************************************
		public Application Application {
		    get {
                RepPP.Application                   appRetVal = null;
                RepPP.InitNotificationEventHandler  delInit;
                RepPP.InitNotificationEventHandler  delReset;
		        bool                                bDesignMode;
                ISite                               site;
		        
		        if (m_app != null) {
                    appRetVal = m_app;
                } else {
		            bDesignMode = IsDesignMode;
                    if (bDesignMode) {
                        site = Site;
                        //To speed the designer loading ...we wait until the site becomes available
                        // and only then we register.
                        if (!m_bRegistred && site != null) {
                            delInit              = new RepPP.InitNotificationEventHandler(this.RepPPInitialized);
                            delReset             = new RepPP.InitNotificationEventHandler(this.RepPPResetting);
                            m_cookieRegistration = InitNotification.RegisterControl(delInit, delReset, site);
                            m_bRegistred = true;
                        }
                        if (!m_bInError) {
                            try {
                                appRetVal  = RepPP.Application.GetApplicationForDesign(Site);
                            } finally {
                                m_bInError = (appRetVal == null);
                            }
                        }
                    } else {
		                if (m_bNoRefCount) {
		                    appRetVal  = Application.GetApplication();
		                } else {
		                    appRetVal = Application.CreateFromRes(LanguageCode.sdDefault, null);
		                    if (appRetVal != null && m_bResDispenserOnly && appRetVal.SessionId == -1) {
                                appRetVal.Release();
    	                        throw new ApplicationException("Rep++ must be started using the Resource Dispenser. Change the connection setting.");
    		                }
    		            }
                        m_app = appRetVal;
                    }
    		    }
		        return(appRetVal);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            IComponentChangeService     changeService;
            ISite                       site;

            site = Site;
            if (site != null) {
                changeService =  site.GetService(typeof(IComponentChangeService)) as IComponentChangeService;
                if (changeService != null) {
                    changeService.OnComponentChanging(m_parent, null);
                    changeService.OnComponentChanged(m_parent, null, "", "");
                }
            }
        }
    } // Class
} // Namespace
