using System;
using System.Collections;
using System.Text;

namespace RepPP.Toolkit {

    //******************************************************************
    //
    /// <summary>
    /// Synchronize the access to a transaction 
    /// </summary>
    /// <remarks>
    /// This class ensure no data has been modified by another user since the data
    /// has been read. To do it, this class use a field mark for auto date stamping
    /// at update. When the data is read, the datetime is kept. Before the card is
    /// update, the class READ FOR UPDATE the datetime and compare it with the kept
    /// value. If value is different, the DataChanged method is called which in the
    /// base implementation trigger an exception.
    /// </remarks>
    //
    //******************************************************************
    public class DataChangedDetector : IDisposable {
        /// <summary>Sent when the data change by another user since it has been read</summary>
        public event RepPP.DbBeforeUpdRowsetTreeEventHandler    DataChanged;
        /// <summary>Connection use to access the database</summary>
        private RepPP.Connection                                m_conn;
        /// <summary>Rowset tree monitored</summary>
        private RepPP.RowsetTree                                m_rowsetTree;
        /// <summary>Node in the rowset tree definition being monitored</summary>
        private RepPP.RowsetTreeDefNode                         m_node;
        /// <summary>SQL command use to read for update the last modification date</summary>
        private string                                          m_strReadForUpdate;
        /// <summary>FieldDef containing the date stamping</summary>
        private RepPP.FieldDef                                  m_fldDefLastUpdate;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="rowsetTree">           Rowset tree to monitor</param>
        /// <param name="node">                 Node in the rowset tree definition to monitor. Null for root.</param>
        /// <param name="conn">                 Connection to use to access the data</param>
        //
        //******************************************************************
        public DataChangedDetector(RepPP.RowsetTree rowsetTree, RepPP.RowsetTreeDefNode node, RepPP.Connection conn) {
            m_conn      = (conn == null) ? node.Application.DataConnection : conn;
            m_node  = node;
            if (m_node == null) {
                throw new System.ApplicationException("The rowset tree " + rowsetTree.RowsetTreeDef.Name + " must have a root node.");
            }
            RowsetTree          = rowsetTree;
            m_fldDefLastUpdate  = null;
            foreach (RepPP.FieldDef fldDef in m_node.RowsetDef.FieldDefs) {
                if ((fldDef.Options & RepPP.FieldOptions.sdFieldAutoUpdTimeStamp) != 0) {
                    m_fldDefLastUpdate = fldDef;
                    break;
                }
            }
            if (m_fldDefLastUpdate == null) {
                throw new System.ApplicationException("The rowset " + m_node.RowsetDef.Name + " must contains a field definition with automatic date stamping at update");
            }
            m_strReadForUpdate  = "SELECT " + m_fldDefLastUpdate.ColPhysName + " FROM " + m_node.RowsetDef.TableName + 
                                  " " + Util.BuildPKWhereClause(m_node, true) + 
                                  " FOR UPDATE";
            m_node.Application.DbBeforeUpdRowsetTree += new DbBeforeUpdRowsetTreeEventHandler(Application_DbBeforeUpdRowsetTree);
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="node">                 Node in the rowset tree definition to monitor. Null for root.</param>
        /// <param name="conn">                 Connection to use to access the data</param>
        //
        //******************************************************************
        public DataChangedDetector(RepPP.RowsetTreeDefNode node, RepPP.Connection conn) : this(null, node, conn) {
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                            Dispose the object
        /// </summary>
        //
        //******************************************************************
        public void Dispose() {
            if (m_node != null) {
                m_node.Application.DbBeforeUpdRowsetTree -= new DbBeforeUpdRowsetTreeEventHandler(Application_DbBeforeUpdRowsetTree);
                m_node = null;
            }
            m_rowsetTree = null;
        }

        //******************************************************************
        //
        /// <summary>
        /// RowsetTree:                         Rowset tree monitored by this class
        /// </summary>
        //
        //******************************************************************
        public RepPP.RowsetTree RowsetTree {
            get {
                return(m_rowsetTree);
            }
            set {
                if (value != null && value.RowsetTreeDef != m_node.RowsetTreeDef) {
                    throw new System.ApplicationException("The node " + m_node.RowsetDef.Name + " is not attached to the rowset tree " + m_rowsetTree.RowsetTreeDef.Name);
                }
                m_rowsetTree = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RowsetTreeDefNode:                     Node in the rowset tree monitored by this class
        /// </summary>
        //
        //******************************************************************
        public RepPP.RowsetTreeDefNode Node {
            get {
                return(m_node);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadForUpdateSqlCmd:                SQL command used to READ for update the last modification date
        /// </summary>
        //
        //******************************************************************
        public string ReadForUpdateSqlCmd {
            get {
                return(m_strReadForUpdate);
            }
            set {
                m_strReadForUpdate = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDataChanged:                      Called when data has been changed since it has been read
        /// </summary>
        /// <param name="e">                    Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnDataChanged(RepPP.RowsetTreeEventArgs e) {
            if (DataChanged != null) {
                DataChanged(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CheckActiveLine:                    Checks for the active line if the datetime has changed
        /// </summary>
        /// <param name="rowset">               Rowset to check</param>
        /// <returns>
        /// true if succeed, false if date time has changed
        /// </returns>
        //
        //******************************************************************
        private bool CheckActiveLine(RepPP.Rowset rowset) {
            bool                bRetVal = true;
            RepPP.SqlCommand    sqlCmd;
            RepPP.SqlCursor     sqlCur;
            RepPP.Field         fld;
            string              strDateTime;
            DateTime            dateTimeNew;
            DateTime            dateTimeOld;

            fld = rowset.Fields[m_fldDefLastUpdate.Name];
            if (fld.IsNull()) {
                throw new System.ApplicationException("The automatic date stamping field is null");
            }
            dateTimeOld = fld.GetValueAsDateTime(rowset.CurrentLine);
            sqlCmd      = m_conn.SqlCommands.Open(m_strReadForUpdate, RepPP.SqlOptimization.sdOptCursor);
            if (sqlCmd == null) {
                throw new System.ApplicationException(m_conn.ErrorMessage);
            }
            using (sqlCmd) {
                if (sqlCmd.Execute(out sqlCur) != 0) {
                    throw new System.ApplicationException(m_conn.ErrorMessage);
                }
                using(sqlCur) {
                    if (sqlCur.Next() != 0) {
                        throw new System.ApplicationException("The record doesn't exist anymore");
                    } else {
                        strDateTime = sqlCur.GetColumnValue(0);
                        if (strDateTime == String.Empty) {
                            throw new System.ApplicationException("The automatic date stamping field is null");
                        }
                        dateTimeNew = RepPP.FieldDef.ConvertValueToDateTime(strDateTime, m_fldDefLastUpdate.Type, m_fldDefLastUpdate.MaxSize);
                        if (dateTimeOld != dateTimeNew) {
                            bRetVal = false;
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// RecursiveCheck:                     Checks recursively for data changed by another user
        /// </summary>
        /// <param name="rowset">               Rowset to check</param>
        /// <returns>
        /// true if succeed, false if date time has changed
        /// </returns>
        //
        //******************************************************************
        private bool RecursiveCheck(RepPP.Rowset rowset) {
            bool            bRetVal = true;
            int             iLineIndex;
            int             iLineCount;
            int             iChildIndex;
            int             iChildCount;
            RepPP.Rowset    rowsetChild;
            
            iLineCount = rowset.LineCount;
            if (rowset.RowsetTreeDefNode == m_node) {
                for (iLineIndex = 0; iLineIndex < iLineCount; iLineIndex++) {
                    if (rowset.ThisLineState(iLineIndex) == LineState.sdLineChanged) {
                        rowset.SelectLine(iLineIndex, false, false);
                        bRetVal = CheckActiveLine(rowset);
                    }
                }
            } else {
                iLineIndex = 0;
                while (iLineIndex < iLineCount && bRetVal) {
                    if (rowset.ThisLineState(iLineIndex) == LineState.sdLineOld || rowset.ThisLineState(iLineIndex) == LineState.sdLineChanged) {
                        rowset.SelectLine(iLineIndex, false, false);
                        iChildCount = rowset.ChildRowsets.Count;
                        iChildIndex = 0;
                        while (iChildIndex < iChildCount && bRetVal) {
                            rowsetChild = rowset.GetChild(iChildIndex, false, false);
                            if (rowsetChild != null) {
                                rowsetChild.Select(false, false);
                                bRetVal = RecursiveCheck(rowsetChild);
                            }
                            iChildIndex++;
                        }
                    }
                    iLineIndex++;
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Application_DbBeforeUpdRowsetTree:  Called when data is being update in a card
        /// </summary>
        /// <param name="sender">               Sender object</param>
        /// <param name="e">                    Evenet argument</param>
        //
        //******************************************************************
        void Application_DbBeforeUpdRowsetTree(object sender, RowsetTreeEventArgs e) {
            RepPP.Rowset rowset;
            
            if (e.RowsetTree == m_rowsetTree) {
                rowset = m_rowsetTree.RootRowset;
                rowset.SaveLinePos();
                try {
                    if (!RecursiveCheck(m_rowsetTree.RootRowset)) {
                        e.EventState = RepPP.EventState.sdReject;
                        m_conn.Rollback();
                        OnDataChanged(e);
                    }
                } finally {
                    rowset.RestoreLinePos(true);
                }
            }
        }
    } // Class DataChangedDetector
} // Namespace
