using System;
using System.Data;
using System.Collections;
using System.ComponentModel;
using System.Runtime.Serialization;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// DataSource interface to permit access to rowset for data binding controls
	/// </summary>
	//
    //********************************************************************************
	[Serializable]
	public class DataSourceRowset : MarshalByValueComponent, IList, IBindingList, IComparer, ITypedList, ISerializable, RowsetTreeDefNodeControlUtil.IRowsetTreeDefNodeControl {
	
	    /// <summary>Determines if description or code is displayed for choice fields</summary>
	    public enum ShowDescriptionE {
	        /// <summary>Never show the description of choice field</summary>
	        Never           = 0,
	        /// <summary>Show description but not when the user edit it</summary>
	        NotOnEdit       = 1,
	        /// <summary>Show the description all the times</summary>
	        Always          = 2
	    };
        /// <summary>The types of an item change</summary>
        public enum ItemChangedTypeE {
            /// <summary>Item changed </summary>
            ItemChanged     = 0,
            /// <summary>Item deleted </summary>
            ItemDeleted     = 1,
            /// <summary>Item inserted </summary>
            ItemInserted    = 2
        };
	    
	    /// <summary>RowsetDef control utility class</summary>
	    protected RowsetTreeDefNodeControlUtil  m_nodeControlUtil;  // Utility class for control attached to node of a rowset tree def
	    private ArrayList                       m_arrayList;        // Array of rows (lines)
	    private FieldDef[]                      m_arrFieldDef;      // List of visible fields used as columns
		private bool                            m_bAllowNew;        // true to allow insertion in the source
		private bool                            m_bAllowEdit;       // true to allow modification in the source
		private bool                            m_bAllowRemove;     // true to remove a line from the source
		private bool                            m_bAllowSort;       // true to allow sort
		private int                             m_iSortedColumn;    // Column use to sort the list (-1 if none)
		private ListSortDirection               m_eSortDirection;   // Direction of the sort
		private Hashtable                       m_hashCurFldErrList;// Current list of field in errors
		private ShowDescriptionE                m_eShowDesc;        // Show description for choice
		private string                          m_strWhereClause;   // WHERE clause for autofill
		private bool                            m_bAutoFill;        // Autofill the datasource if true
        private bool                            m_bSortRowset;   // Sort the actutual Rowset if true
        private Rowset                          m_rowsetAutoFill;   // Autofill group instance if any
		internal DataRowRowset                  m_rowsetRowNewLine; // New line if in process.
		/// <summary>Event trigger when the data source changed</summary>
        public event ListChangedEventHandler    ListChanged;        // Event

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                               Private constructor
        /// </summary>
        /// <param name="parent">               Control embedding this class. Null except if the DataSourceRowset is also embedded.</param>
		/// <param name="strRowsetTreeName">    Rep++ RowsetTreeDef name</param>
		/// <param name="strRowsetName">        Rep++ RowsetTreeDefNode name</param>
		/// <param name="iLineCount">           Number of lines in the source</param>
        /// <param name="bNoRefCount">          true to cancel reference counting (component dispose is not always called)</param>
        //
        //********************************************************************************
		private DataSourceRowset(object parent, string strRowsetTreeName, string strRowsetName, int iLineCount, bool bNoRefCount) {
		    m_nodeControlUtil = new RowsetTreeDefNodeControlUtil((parent == null) ? this : parent,
                                                                  new ControlUtilHelperBase(),
		                                                          false,          // Not always used with resource dispenser
                                                                  bNoRefCount);   // The Dispose is not called by the Page default implementation. Must not use ref counting
		    m_arrayList                 = new ArrayList(iLineCount);
		    FieldDefs                   = null;
		    m_bAllowNew                 = true;
		    m_bAllowEdit                = true;
		    m_bAllowRemove              = true;
		    m_bAllowSort                = true;
            m_bSortRowset               = true;
		    m_eShowDesc                 = ShowDescriptionE.Always;
		    m_iSortedColumn             = -1;
		    m_hashCurFldErrList         = null;
		    m_eSortDirection            = ListSortDirection.Ascending;
            m_nodeControlUtil.RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName(strRowsetTreeName, strRowsetName);
		    m_strWhereClause            = "";
		    m_bAutoFill                 = false;
		    m_rowsetAutoFill            = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group instance. The source will contains
        ///                             all the row of the instance
        /// </summary>
		/// <param name="info">         Serialization object</param>
		/// <param name="context">      Serialization context</param>
        //
        //********************************************************************************
		protected DataSourceRowset(SerializationInfo info, StreamingContext context) : this(null, "", "", 0, true) {
		    string  strRowsetTreeName;
		    string  strRowsetName;
		    
		    strRowsetTreeName                   = info.GetString("RowsetTreeName");
		    strRowsetName                       = info.GetString("RowsetName");
		    m_eShowDesc                         = (ShowDescriptionE)info.GetInt32("ShowDesc");
    		m_bAllowNew                         = info.GetBoolean("AllowNew");
	    	m_bAllowEdit                        = info.GetBoolean("AllowEdit");
		    m_bAllowRemove                      = info.GetBoolean("AllowRemove");
		    m_bAllowSort                        = info.GetBoolean("AllowSort");
            m_bSortRowset                       = info.GetBoolean("SortRowset");
            m_nodeControlUtil.RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName(strRowsetTreeName, strRowsetName);
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
        //
        //********************************************************************************
		public DataSourceRowset() : this(null, "", "", 0, true) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
        /// <param name="parent">       Control embedding this class. Null except if the DataSourceRowset is also embedded.</param>
        /// <param name="bNoRefCount">  true to cancel reference counting (component dispose is not always called)</param>
        /// 
        //
        //********************************************************************************
		public DataSourceRowset(object parent, bool bNoRefCount) : this(parent, "", "", 0, bNoRefCount) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
        //
        //********************************************************************************
		public DataSourceRowset(System.ComponentModel.IContainer container) : this(null, "", "", 0, true) {
			container.Add(this);
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
		/// <param name="node">          Rep++ RowsetTreeDefNode</param>
        //
        //********************************************************************************
		public DataSourceRowset(RowsetTreeDefNode node) : this(null, node.RowsetTreeDef.Name, node.RowsetDef.Name, 0, true) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a rowset. The source will contains
        ///                             all the row of the instance
        /// </summary>
		/// <param name="rowset">       Rep++ Rowset</param>
        //
        //********************************************************************************
		public DataSourceRowset(Rowset rowset) : this(null, rowset.RowsetTree == null ? string.Empty : rowset.RowsetTree.RowsetTreeDef.Name, rowset.RowsetDef.Name, rowset.UndelLineCount, true) {
		    Rowset = rowset;
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the object
        /// </summary>
		/// <param name="bDisposing">   true if dispose, false if GC</param>
        //
        //********************************************************************************
        protected override void Dispose(bool bDisposing) {            
            if (m_rowsetAutoFill != null) {
                m_rowsetAutoFill.Delete();
                m_rowsetAutoFill = null;
            }
            if (m_nodeControlUtil != null) {
                m_nodeControlUtil.Dispose();
                m_nodeControlUtil = null;
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// GetObjectData:              Serialized the object
		/// </summary>
		/// <param name="info">         Information to serialize</param>
		/// <param name="context">      Context</param>
        //
        //********************************************************************************
		public virtual void GetObjectData(SerializationInfo info, StreamingContext context) { // ISerializable
		    info.AddValue("RowsetTreeName", m_nodeControlUtil.RowsetTreeDefNodeFullName.RowsetTreeName);
		    info.AddValue("RowsetName",     m_nodeControlUtil.RowsetTreeDefNodeFullName.RowsetName);
		    info.AddValue("ShowDesc",       (int)m_eShowDesc);
		    info.AddValue("AllowNew",       m_bAllowNew);
		    info.AddValue("AllowEdit",      m_bAllowEdit);
		    info.AddValue("AllowRemove",    m_bAllowRemove);
		    info.AddValue("AllowSort",      m_bAllowSort);
		    info.AddValue("SortRowset",     m_bSortRowset);
		}

        //********************************************************************************
        //
		/// <summary>
		/// FieldDefs:                     Array of fields
		/// </summary>
        //
        //********************************************************************************
		public FieldDef[] FieldDefs {
		    get {
                FieldDef[] arrRetFlds = null;

		        if (m_arrFieldDef == null) {
		            arrRetFlds = FillStruct();
                    if (m_nodeControlUtil != null && !m_nodeControlUtil.IsDesignMode) {
                       m_arrFieldDef = arrRetFlds;
                    }
		        } else {
                    arrRetFlds = m_arrFieldDef;              
                }
		        return(arrRetFlds);
		    }
		    set {
               if (m_nodeControlUtil != null && !m_nodeControlUtil.IsDesignMode) {
		            m_arrFieldDef = value;
               }
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_nodeControlUtil != null) {
                m_nodeControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetDef:              Attached group
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("RowsetDef which will be represented by this Data Source")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual RowsetDef RowsetDef {
            get {
                return(m_nodeControlUtil.RowsetDef);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetTreeDefNodeFullName:                  Name of the RowsetTreeDefNode attached to the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.RowsetTreeDefNodeControlDesigner+NodeEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]        
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Name of the ChildRowsetDef attached to the source")]
		public virtual RowsetTreeDefNodeFullName RowsetTreeDefNodeFullName {
		    get {
		        return(m_nodeControlUtil.RowsetTreeDefNodeFullName);
		    }
		    set {
		        m_nodeControlUtil.RowsetTreeDefNodeFullName = value;
		        FieldDefs = null;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// WhereClause:            WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("WHERE clause use to auto-fill the data source")]
        [DefaultValue("")]
		public virtual string WhereClause {
		    get {
		        return(m_strWhereClause);
		    }
		    set {
		        m_strWhereClause = (value == null) ? "" : value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// AutoFill:               Use to auto-fill the data source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Auto fill the data source")]
        [DefaultValue(false)]
		public virtual bool AutoFill {
		    get {
		        return(m_bAutoFill);
		    }
		    set {
		        m_bAutoFill = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// AutoAssociateRowset: Automaticaly associate a group instance to the data source
        /// </summary>
        //
        //********************************************************************************
        public virtual void AutoAssociateRowset(RowsetTree rowsetTree) {
            RowsetTreeDef       rowsetTreeDef;
            RowsetTreeDefNode   node;
            Rowset              rowsetRoot;
            
            node = m_nodeControlUtil.RowsetTreeDefNode;
            if (rowsetTree == null) {
                rowsetTreeDef   = node.RowsetTreeDef;
                rowsetRoot      = rowsetTreeDef.Nodes[0].RowsetDef.ActiveRowset;
                if (rowsetRoot != null && rowsetRoot.RowsetTree != null && rowsetRoot.RowsetTree.RowsetTreeDef == rowsetTreeDef) {
                    rowsetTree = rowsetRoot.RowsetTree;
                }
            }
            if (rowsetTree != null) {
                Rowset = rowsetTree.GetCurrentRowset(node);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Rowset:      Attached group instance
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("Rowset represented by this Data Source")]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual Rowset Rowset {
            get {
                CheckForAutoFill();
                return((m_rowsetAutoFill != null) ? m_rowsetAutoFill : m_nodeControlUtil.Rowset);
            }
            set {
                if (value == null                     || 
                    m_nodeControlUtil.Rowset != value ||
                    Count != value.UndelLineCount) {
                    if (value != null) {
                        if (value.RowsetDef != m_nodeControlUtil.RowsetDef) {
                            throw new ArgumentException("The group instance doesn't point to the same group");
                        }
                    }
                    if (m_rowsetAutoFill != null) {
                        m_rowsetAutoFill.Delete();
                        m_rowsetAutoFill = null;
                        AutoFill         = false;
                    }
                    m_nodeControlUtil.Rowset = value;
                    FillRow();
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_nodeControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// FillStruct:                 Fill structure
		/// </summary>
        //
        //********************************************************************************
        private FieldDef[] FillStruct() {

		    int         iVisibleField;
		    FieldDef[]  arrRetFlds      = null;
		    RowsetDef   rowsetDef;
		    
		    m_arrayList.Clear();
		    rowsetDef = RowsetDef;
		    if (rowsetDef != null) {
		        iVisibleField   = 0;
		        foreach (FieldDef fldDef in rowsetDef.FieldDefs) {
		            if (fldDef.IsVisible) {
		                iVisibleField++;
		            }
		        }
		        arrRetFlds    = new FieldDef[iVisibleField];
		        iVisibleField = 0;
		        foreach (FieldDef fldDef in rowsetDef.FieldDefs) {
		            if (fldDef.IsVisible) {
		                arrRetFlds[iVisibleField++] = fldDef;
		            }
		        }
		    }
            return (arrRetFlds);
        }

        //********************************************************************************
        //
		/// <summary>
		/// FillRow:            Fill the row with the content of the group instance
		/// </summary>
        //
        //********************************************************************************
        private void FillRow() {
            int             iLineCount;
            int             iUndelLineCount;
            DataRowRowset   rowsetRow;
            Rowset          rowset;
            FieldDef[]      arrFldDef;
            
            arrFldDef = FieldDefs;    // Ensure the field array is filled.
            rowset = Rowset;
            m_arrayList.Clear();
            if (rowset != null) {
                iUndelLineCount         = rowset.UndelLineCount;
                iLineCount              = rowset.LineCount;
                m_arrayList.Capacity    = iUndelLineCount;
                for (int iIndex = 0; iIndex < iLineCount; iIndex++) {
                    if (rowset.ThisLineState(iIndex) != LineState.sdLineDeleted) {
                        rowsetRow = new DataRowRowset(this, rowset.LineBookmark(iIndex));
                        m_arrayList.Add(rowsetRow);
                    }
                }
            }
            OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, 0));
        }

        //********************************************************************************
        //
		/// <summary>
        /// CreateRowset:          Create an autofill rowset
		/// </summary>
        //
        //********************************************************************************
        private void CreateRowset() {
            string      strSelect;
            string      strUpdate;
            string      strInsert;
            string      strDelete;
            RowsetDef   rowsetDef;

            rowsetDef = RowsetDef;
            if (rowsetDef != null) {
                if (m_rowsetAutoFill != null) {
                    m_rowsetAutoFill.Dispose();
                    m_rowsetAutoFill = null;
                }
                rowsetDef.BuildSqlCommand("", out strSelect, out strUpdate, out strInsert, out strDelete);
                strSelect = rowsetDef.Application.Tool.SetSqlWhere(strSelect, m_strWhereClause);
                m_rowsetAutoFill = rowsetDef.Rowsets.Create(RowsetType.sdHorizontal);
                if (m_rowsetAutoFill.ReadFromDb(strSelect) == 0) {
                    FillRow();
                }
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// CheckForAutoFill:       Check if we must auto-fill
		/// </summary>
        //
        //********************************************************************************
        private void CheckForAutoFill() {
            if (!DesignMode && AutoFill && m_rowsetAutoFill == null) {
                CreateRowset();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFieldNameIndex:          Gets the index of a field name in the list of columns
        /// </summary>
		/// <param name="strName">      Field name</param>
		/// <returns>
		/// Index or -1 if not found
		/// </returns>
        //
        //********************************************************************************
		private int GetFieldNameIndex(string strName) {
		    int         iRetVal = -1;
		    int         iIndex;
		    FieldDef[]  arrFldDef;
		    
		    iIndex = 0;
		    arrFldDef = FieldDefs;
		    if (arrFldDef != null) {
		        foreach (FieldDef fldDef in arrFldDef) {
		            if (String.Compare(fldDef.Name, strName, true) == 0) {
		                iRetVal = iIndex;
		                break;
		            } else {
		                iIndex++;
		            }
		        }
		    }
		    return(iRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application_FldOnError:     Intercept field error
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event parameter</param>
        //
        //********************************************************************************
        private void Application_FldOnError(object sender, FieldGenEventArgs e) {
            if (m_hashCurFldErrList != null) {
                if (e.FieldDef.RowsetDef == RowsetDef) {
                    m_hashCurFldErrList.Add(e.FieldDef.Name, e.ErrorText);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsLineValid:                    Check if line is valid. Returns the list of errors
        ///                                 in hashFldInError
        /// </summary>
        /// <param name="hashFldInError">   Hashtable to fill with the error</param>
        /// <returns>
        /// true:                           Line is valid
        /// false:                          Line is not
        /// </returns>
        //
        //********************************************************************************
        internal bool IsLineValid(Hashtable hashFldInError) {
            bool        bRetVal = true;
            RowsetDef   rowsetDef;
            FieldDef[]  arrFldDef;
            
		    rowsetDef                         = RowsetDef;
		    rowsetDef.Application.FldOnError += new FldOnErrorEventHandler(Application_FldOnError);
            m_hashCurFldErrList               = hashFldInError;
            m_hashCurFldErrList.Clear();
            arrFldDef = FieldDefs;
            foreach (FieldDef fldDef in arrFldDef) {
                if (fldDef.Validate() != 0) {
                    bRetVal = false;
                }
            }
            m_hashCurFldErrList               = null;
		    rowsetDef.Application.FldOnError -= new FldOnErrorEventHandler(Application_FldOnError);
            return(bRetVal);
        }		

        //********************************************************************************
        //
        /// <summary>
        /// OnListChanged:      Trigger the ListChanged event
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        public void OnListChanged(ListChangedEventArgs e) {
            if (ListChanged != null) {
                ListChanged(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:        Show description of choice if true, code if false
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("How to display a choice value")]
        public ShowDescriptionE ShowDescription {
            get {
                return(m_eShowDesc);
            }
            set {
                m_eShowDesc = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Sort:               Returns the colums use for sort (or "" if none)
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public string Sort {
            get {
                FieldDef[]  arrFldDef;
                string      strRetVal;
                
                arrFldDef    = FieldDefs;
                strRetVal = (m_iSortedColumn == -1) ? 
                            "" : 
                            arrFldDef[m_iSortedColumn].Name;
                return(strRetVal);
            }
            set {
                int                 iIndex;
                ListSortDirection   eDir;
                object[]            arrSortParam;
                
                if (!m_bAllowSort) {
                    throw new InvalidOperationException("Sort not allowed");
                }
                if (value != "") {
                    if (value.IndexOf(',') != -1) {
                        throw new InvalidOperationException("Sort accept only simple field name");
                    }
                    value = value.ToUpper();
                    if (value.EndsWith(" ASC")) {
                        eDir  = ListSortDirection.Ascending;
                        value = value.Substring(0, value.Length - 4).Trim();
                    } else if (value.EndsWith(" DESC")) {
                        eDir  = ListSortDirection.Descending;
                        value = value.Substring(0, value.Length - 5).Trim();
                    } else {
                        eDir  = ListSortDirection.Ascending;
                    }
                    iIndex = GetFieldNameIndex(value);
                    if (iIndex == -1) {
                        throw new ArgumentException("Column '" + value + "' is not a valid column name");
                    }
                    m_iSortedColumn  = iIndex;
                    m_eSortDirection = eDir;
                }
                if (SortRowset) {
                    arrSortParam    = new object[2];
                    arrSortParam[0] = FieldDefs[m_iSortedColumn].Name;
                    arrSortParam[1] = m_eSortDirection;
                    Rowset.Sort(arrSortParam);
                    FillRow();
                } else {
                    if (Count == 0) {
                       FillRow();
                    }
                    m_arrayList.Sort(this);
                }
                OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, 0));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsReadOnly:         Returns if the list is read-only
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool IsReadOnly {            // IList
            get {
                return(!m_bAllowNew && !m_bAllowEdit && !m_bAllowRemove);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Default indexer
        /// </summary>
        //
        //********************************************************************************
        public object this[int iIndex] {     // IList
            get {
                CheckForAutoFill();
                return(m_arrayList[iIndex]);
            }
            set {
                throw new InvalidOperationException("The Item property is read-only");
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  RemoveAt:               Remove the item from the list
        /// </summary>
        /// <param name="iIndex">    Index of the item to remove</param>
        //
        //********************************************************************************
        public void RemoveAt(int iIndex) {   // IList
            Rowset          rowset;
            DataRowRowset   rowsetRow;
            int             iLineToDel;
            
            if (!m_bAllowRemove) {
                throw new InvalidOperationException("Rows cannot be deleted");
            }
            rowsetRow = m_arrayList[iIndex] as DataRowRowset;
            if (rowsetRow == null) {
                throw new ArgumentException("Bad argument type");
            }
            rowset     = Rowset;
            iLineToDel = rowset.LineFromBookmark(rowsetRow.Bookmark);
            if (iLineToDel == -1) {
                throw new ArgumentException("Bad index");
            }
            if (iLineToDel == rowset.CurrentLine) {
                rowset.DeleteLine();
            } else {
                rowset.SaveLinePos();
                try {
                    rowset.SelectLine(iLineToDel, false, false);
                    rowset.DeleteLine();
                } finally {
                    rowset.RestoreLinePos(true);
                }
            }
            m_arrayList.RemoveAt(iIndex);
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, iIndex));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Insert:                Insert an item in the list
        /// </summary>
        /// <param name="index">    Position at which to insert the item</param>
        /// <param name="value">    Item to insert</param>
        //
        //********************************************************************************
        public void Insert(int index, object value) {   // IList
            throw new InvalidOperationException("The list is read-only. Use to AddNew method.");
        }

        //********************************************************************************
        //
        /// <summary>
        /// Remove:                 Remove an item from the list
        /// </summary>
        /// <param name="value">    Item to remove</param>
        //
        //********************************************************************************
        public void Remove(object value) {  // IList
            RemoveAt(m_arrayList.IndexOf(value));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Contains:              Check if the list contains the specified object.
        /// </summary>
        /// <param name="value">    Object to check</param>
        /// <returns>
        /// true if yes, false if not.
        /// </returns>
        //
        //********************************************************************************
        public bool Contains(object value) {    // IList
            CheckForAutoFill();
            return(m_arrayList.Contains(value));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Clear:                 Removes all items from the list
        /// </summary>
        //
        //********************************************************************************
        public void Clear() {   // IList
            Rowset   rowset;
            
            rowset = Rowset;
            if (rowset != null) {
                rowset.Empty(RowsetDelFlag.sdDelKey);
            }
            m_arrayList.Clear();
            OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, 0));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  IndexOf:               Returns the index of the object in the list
        /// </summary>
        /// <param name="value">    Object</param>
        /// <returns>
        /// Index or -1 if not found
        /// </returns>
        //
        //********************************************************************************
        public int IndexOf(object value) {      // IList
            CheckForAutoFill();
            return(m_arrayList.IndexOf(value));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Add:                   Adds a new item to the list. Use to AddNew method instead.
        /// </summary>
        /// <param name="value">    Object to add.</param>
        //
        //********************************************************************************
        public int Add(object value) {      // IList
            throw new InvalidOperationException("The list is read-only. Use to AddNew method.");
        }

        //********************************************************************************
        //
        /// <summary>
        ///  IsFixedSize:           Returns if the list has a fixed size.
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool IsFixedSize {   // IList
            get {
                return(!(m_bAllowNew || m_bAllowRemove));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  IsSynchronized:        Returns false. This list is not thread safe
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool IsSynchronized {    // ICollection
            get {
                return(false);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Count:                 Returns the number of lines (rows) in the source
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public int Count {              // ICollection
            get {
                CheckForAutoFill();
                return(m_arrayList.Count);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  CopyTo:                Copy the content of the list to the specified array
        /// </summary>
        /// <param name="array">    Array to copy to</param>
        /// <param name="iIndex">   Index to start with</param>
        //
        //********************************************************************************
        public void CopyTo(Array array, int iIndex) {    // ICollection
            CheckForAutoFill();
            m_arrayList.CopyTo(array, iIndex);
        }

        //********************************************************************************
        //
        /// <summary>
        ///  SyncRoot:              Returns the synchronize root. This list is not thread safe.
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public object SyncRoot {        // ICollection
            get {
                return(this);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  GetEnumerator:         Returns an enumerator for the list
        /// </summary>
        /// <returns>
        /// Enumerator for the list
        /// </returns>
        //
        //********************************************************************************
        public IEnumerator GetEnumerator() {    // IEnumerator
            CheckForAutoFill();
            return(m_arrayList.GetEnumerator());
        }

        //********************************************************************************
        //
        /// <summary>
        /// CompareValues:          Compare two objects value
        /// </summary>
        /// <param name="x">        First object</param>
        /// <param name="y">        Second object</param>
        /// <returns>
        /// -1 (x .lt. y), 0 (x == y), 1 (x .gt. y)
        /// </returns>
        //
        //********************************************************************************
        private int CompareValues(object x, object y) {
            int     iRetVal;
            string  strVal1;
            string  strVal2;
            
            if (x == DBNull.Value) {
                if (y == DBNull.Value) {
                    iRetVal = 0;
                } else {
                    iRetVal = -1;
                }
            } else {
                if (y == DBNull.Value) {
                    iRetVal = 1;
                } else {
                    strVal1 = x.ToString();
                    strVal2 = y.ToString();
                    iRetVal = String.Compare(strVal1, strVal2);
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Compare:               Compare two objects
        /// </summary>
        /// <param name="x">        First object</param>
        /// <param name="y">        Second object</param>
        /// <returns>
        /// -1 (x .lt. y), 0 (x == y), 1 (x .gt. y)
        /// </returns>
        //
        //********************************************************************************
        public int Compare(object x, object y) {
            int                 iRetVal;
            object              objVal1;
            object              objVal2;
            
            objVal1 = (x is DataRowRowset) ? (x as DataRowRowset)[m_iSortedColumn] : x;
            objVal2 = (y is DataRowRowset) ? (y as DataRowRowset)[m_iSortedColumn] : y;
            iRetVal = CompareValues(objVal1, objVal2);
            if (m_eSortDirection == ListSortDirection.Descending) {
                iRetVal = -iRetVal;
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddIndex:               Adds an index for the sort.
        /// </summary>
        /// <param name="property"> Property of the index</param>
        //
        //********************************************************************************
        public void AddIndex(PropertyDescriptor property) {     // IBindingList
            // Do nothing. Not supported.
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowNew:           Determines if the source allow insertion of rows
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("Allow the insertion of new rows")]
        public bool AllowNew {  // IBindingList
            get {
                bool    bRetVal;
                
                if (DesignMode) {
                    bRetVal = m_bAllowNew;
                } else {
                    bRetVal = m_bAllowNew && (Rowset != null);
                }
                return(bRetVal);
            }
            set {
                m_bAllowNew = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SortRowset:             Determines if the sort will be performed locally 
        ///                         or using the internal list
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("Sort the Rowset instead of the internal list")]
        public bool SortRowset {
            get {
                return(m_bSortRowset);
            }
            set {
                m_bSortRowset = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ApplySort:                  Apply the sort
        /// </summary>
        /// <param name="property">     Property use to sort</param>
        /// <param name="direction">    Direction of the sort</param>
        //
        //********************************************************************************
        public void ApplySort(PropertyDescriptor property, ListSortDirection direction) {   // IBindingList
            string  strName;
            
            strName = (direction == ListSortDirection.Ascending) ? property.Name : property.Name + " DESC";
            Sort    = strName;
        }

        //********************************************************************************
        //
        /// <summary>
        ///  SortProperty:          Returns the property which is actually sorted
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public PropertyDescriptor SortProperty {    // IBindingList
            get {
                PropertyDescriptor  propRetVal;
                FieldDef[]          arrFldDef;
                FieldDef            fldDef;
                
                if (m_iSortedColumn == -1) {
                    propRetVal = null;
                } else {
                    arrFldDef  = FieldDefs;
                    fldDef     = arrFldDef[m_iSortedColumn];
                    propRetVal = new DataRowRowsetPropertyDescriptor(fldDef.Name,
                                                                     fldDef.Prompt,
                                                                     m_iSortedColumn,
                                                                     typeof(string),
                                                                     !fldDef.IsModifiable);
                }
                return(propRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Find:                      Find a raw using the key of a property
        /// </summary>
        /// <param name="strColName">   Column to use for the search</param>
        /// <param name="key">          Key value</param>
        /// <returns>
        /// Index or -1 if not found
        /// </returns>
        //
        //********************************************************************************
        public int Find(string strColName, object key) {
            int     iRetVal = -1;
            int     iColIndex;
            int     iCount;
            object  objVal;

            iColIndex = GetFieldNameIndex(strColName);
            if (iColIndex == -1) {
                throw new ArgumentException("Column '" + strColName + "' does not exist in the source.");
            }
            if (iColIndex == m_iSortedColumn) {
                iRetVal = m_arrayList.BinarySearch(key, this);
            } else {
                iCount  = m_arrayList.Count;
                for (int iIndex = 0; iIndex < iCount; iIndex++) {
                    objVal = (m_arrayList[iIndex] as DataRowRowset)[iColIndex];
                    if (CompareValues(key, objVal) == 0) {
                        iRetVal = iIndex;
                        break;
                    }
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Find:                  Find a raw using the key of a property
        /// </summary>
        /// <param name="property"> Property use to find the item</param>
        /// <param name="key">      Key value</param>
        /// <returns>
        /// Index
        /// </returns>
        //
        //********************************************************************************
        public int Find(PropertyDescriptor property, object key) {  // IBindingList
            return(Find(property.Name, key));
        }

        //********************************************************************************
        //
        /// <summary>
        ///  SupportsSorting:       Returns true to indicate the source support sorting
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool SupportsSorting {   // IBindingList
            get {
                return(true);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsSorted:           Returns true if source is sorted
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool IsSorted {  // IBindingList
            get {
                return(m_iSortedColumn != -1);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowRemove:        Determines if the source allow deletion of rows
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("Allow removal of rows")]
        public bool AllowRemove {   // IBindingList
            get {
                return(m_bAllowRemove);
            }
            set {
                m_bAllowRemove = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SupportsSearching:      Returns true if the source support searching
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool SupportsSearching { // IBindingList
            get {
                return(true);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SortDirection:      Returns the sort direction
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public ListSortDirection SortDirection {    // IBindingList
            get {
                return(m_eSortDirection);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SupportsChangeNotification: Returns true if the source support change notification
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        public bool SupportsChangeNotification {    // IBindingList
            get {
                return(true);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RemoveSort:                 Remove a sort column
        /// </summary>
        //
        //********************************************************************************
        public void RemoveSort() {  // IBindingList
            Sort = "";
        }

        //********************************************************************************
        //
        /// <summary>
        ///  AddNew:                Adds a new row (line) to the source
        /// </summary>
        /// <returns>
        /// New line (row)
        /// </returns>
        //
        //********************************************************************************
        public object AddNew() {    // IBindingList
            IntPtr  iBmk;
            Rowset  rowset;
            
            rowset = Rowset;
            if (!m_bAllowNew || rowset == null) {
                throw new InvalidOperationException("AddNew is not allowed");
            }
            rowset.Select(false, true);
            rowset.AddBlankLine(true);
            iBmk = rowset.LineBookmark(rowset.CurrentLine);
            m_rowsetRowNewLine = new DataRowRowset(this, iBmk);
            m_arrayList.Add(m_rowsetRowNewLine);
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, m_arrayList.Count - 1));
            return(m_rowsetRowNewLine);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowEdit:          Determines if the source allow modification of items
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("Allow edition of a row")]
        public bool AllowEdit { // IBindingList
            get {
                return(m_bAllowEdit);
            }
            set {
                m_bAllowEdit = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        ///  AddDuplicate:                Adds a duplicate row (line) to the source
        /// </summary>
        /// <returns>
        /// New line (row)
        /// </returns>
        //
        //********************************************************************************
        public object AddDuplicate() {
            IntPtr  iBmk;
            Rowset  rowset;
            
            rowset = Rowset;
            if (!m_bAllowNew || rowset == null) {
                throw new InvalidOperationException("AddDuplicate is not allowed");
            }
            rowset.Select(false, true);
            rowset.DuplicateLine();
            iBmk = rowset.LineBookmark(rowset.CurrentLine);
            m_rowsetRowNewLine = new DataRowRowset(this, iBmk);
            m_arrayList.Add(m_rowsetRowNewLine);
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, m_arrayList.Count - 1));
            return(m_rowsetRowNewLine);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RemoveIndex:            Remove a sort index
        /// </summary>
        /// <param name="property"> Property to remove</param>
        //
        //********************************************************************************
        public void RemoveIndex(PropertyDescriptor property) {  // IBindingList
            // Do nothing.
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetItemProperties:              Returns a descriptor for each property (field)
        /// </summary>
        /// <param name="listAccessors">    List accessor (not used)</param>
        /// <returns>
        /// List of properties
        /// </returns>
        //
        //********************************************************************************
        public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] listAccessors) { // ITypedList
            PropertyDescriptorCollection        colRetVal = null;
            DataRowRowsetPropertyDescriptor[]    arrPropDesc;
            int                                 iIndex;
            FieldDef[]                          arrFldDef;
            
            arrFldDef = FieldDefs;
            if (arrFldDef == null) {
                arrPropDesc = new DataRowRowsetPropertyDescriptor[0];
            } else {
                arrPropDesc = new DataRowRowsetPropertyDescriptor[arrFldDef.Length];
                iIndex      = 0;
                foreach (FieldDef fldDef in arrFldDef) {
                    arrPropDesc[iIndex] = new DataRowRowsetPropertyDescriptor(fldDef.Name,
                                                                              fldDef.Prompt,
                                                                              iIndex,
                                                                              typeof(string),
                                                                              !fldDef.IsModifiable);
                    iIndex++;
                }
            }
            colRetVal = new PropertyDescriptorCollection(arrPropDesc);
            return(colRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetListName:                    Names use to identify the source (TableStyle)
        /// </summary>
        /// <param name="listAccessors">    List of accessor (not used)</param>
        /// <returns>
        /// Name of the datasource
        /// </returns>
        //
        //********************************************************************************
        public string GetListName(PropertyDescriptor[] listAccessors) {
            RowsetDef   rowsetDef;
            
            rowsetDef = RowsetDef;
            return((rowsetDef == null) ? "Source1" : rowsetDef.Name + "_Source");
        }

        //********************************************************************************
        //
        /// <summary>
        ///  Refresh:                 Refreshes the list of items in the list from the 
        ///                           attached Rowset
        /// </summary>
        //
        //********************************************************************************
        public void Refresh() {
            FillRow();
        }

        //********************************************************************************
        //
        /// <summary>
        ///  RefreshItem:                   Refreshes an item in the list from the attached group instance
        /// </summary>
        /// <param name="iItemIndex">       Item index</param>
        /// <param name="eItemChangeType">  The item change type</param>
        //
        //********************************************************************************
        public void RefreshItem(int iItemIndex, ItemChangedTypeE eItemChangeType) {
            IntPtr  iBmk;
            Rowset  rowset;

            switch (eItemChangeType) {
            case ItemChangedTypeE.ItemChanged:
                OnListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, iItemIndex));
                break;
            case ItemChangedTypeE.ItemDeleted:
                if (!m_bAllowRemove) {
                    throw new InvalidOperationException("Rows cannot be deleted");
                }
                m_arrayList.RemoveAt(iItemIndex);
                OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, iItemIndex));
                
                break;
            case ItemChangedTypeE.ItemInserted:
                rowset = Rowset;
                if (!m_bAllowNew || rowset == null) {
                    throw new InvalidOperationException("AddNew is not allowed");
                }
                iBmk = rowset.LineBookmark(rowset.CurrentLine);
                m_rowsetRowNewLine = new DataRowRowset(this, iBmk);
                m_arrayList.Insert(iItemIndex, m_rowsetRowNewLine);
                OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, iItemIndex));
                break;
            }
        }
    }
}
