//*********************************************************
//
// DataTableRowset:			    Overload the .NET DataTable to interact with rowset
//
// Author:						JFO
//
// (c) 2002 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Data;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit {
	/// <summary>
	/// Rep++ DataTable object
	/// </summary>
	public class DataTableRowset : System.Data.DataTable {
        //
        //  This class extend the DataTable class. 
        //  First, the constructor build the base datatable with the content of a rowset
        //  Second, each time something is changed in the datatable, the rowset is synchronized.
        //  with the change.
        //  The datatable can be access through a dataview (Datatable.DefaultView). 
        //  Datagrid and other control access the datatable via this dataview. The
        //  row in the view are not necessarely in the same order as the row in the datatable.
        //  More, deleted rows are not in the dataview (which make them quite close of a neutral pos).
        //  
        //  So, four position system are used in this class:
        //
        //  A.  Row index as defines in the DataTable (0..Count-1)
        //  B.  Rowset lines (1..LineCount)
        //  C.  Neutral position (0..undeleted line count - 1)
        //  D.  Dataview position (0..undeleted line count -1)
        //  The neutral position express the index of undeleted line in both the DataTable and the Rowset
        //  as both must be correlated.
        //
        //
        /// <summary>Category of trace</summary>
        public enum TraceCatE {
            /// <summary></summary>
            TC_DATACTOR     = 1,
            /// <summary></summary>
            TC_GRIDCTOR     = 2,
            /// <summary></summary>
            TC_DATAEVENT    = 4,
            /// <summary></summary>
            TC_GRIDEVENT    = 8,
            /// <summary></summary>
            TC_DATAMETHOD   = 16,
            /// <summary></summary>
            TC_GRIDMETHOD   = 32,
            /// <summary></summary>
            TC_DATASINK     = 64,
            /// <summary></summary>
            TC_GRIDSINK     = 128,
            /// <summary></summary>
            TC_COMBOEDIT    = 256,
            /// <summary></summary>
            TC_COMBOKEY     = 512
        }
        
        /// <summary>Type of trace</summary>
        public enum TraceTypeE {
            /// <summary></summary>
            TT_ENTER        = 1,
            /// <summary></summary>
            TT_EXIT         = 2,
            /// <summary></summary>
            TT_NONE         = 0
        };
                
        /// <summary>Trace level</summary>
        internal  static int        m_iTraceLevel = 0;
        /// <summary>Rowset attached to the datatable</summary>
	    protected RepPP.Rowset      m_rowset;
	    /// <summary>RowsetDef attached to the datatable</summary>
	    protected RepPP.RowsetDef   m_rowsetDef;
	    /// <summary>true if initialization in process</summary>
        protected bool              m_bInitializing;

        //*********************************************************	    
	    //
        /// <summary>
	    /// ctor:                       DataTable constructor
        /// </summary>
        /// <param name="rowsetDef">    RowsetDef to attach to the datatable</param>
	    //
        //*********************************************************	    
		public DataTableRowset(RepPP.RowsetDef rowsetDef) {
		    Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATACTOR, rowsetDef, null, "SCDataTable::SCDataTable");
		    m_rowset                 = null;
		    m_rowsetDef              = rowsetDef;
            TableName                = rowsetDef.Name;
            m_bInitializing          = true;
            BuildTableStructv();
		    m_bInitializing          = false;
		    DefaultView.ListChanged += new ListChangedEventHandler(this.ListChangedSink);
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATACTOR, rowsetDef, null, "SCDataTable::SCDataTable");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                   Datatable constructor
        /// </summary>
        /// <param name="rowset">   Rowset to attach to the datatable</param>
        //
        //*********************************************************	    
        public DataTableRowset(RepPP.Rowset rowset) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATACTOR, rowset.RowsetDef, rowset, "SCDataTable::SCDataTable");
            m_rowset                = rowset;
            m_rowsetDef             = rowset.RowsetDef;
            TableName               = m_rowsetDef.Name;
            m_bInitializing         = true;
            BuildTableStructv();
            Fillv();
            m_bInitializing         = false;
            DefaultView.ListChanged += new ListChangedEventHandler(this.ListChangedSink);
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATACTOR, rowset.RowsetDef, rowset, "SCDataTable::SCDataTable");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Trace:                  Display a trace message
        /// </summary>
        /// <param name="eType">    Type of trace</param>
        /// <param name="eCat">     Category of trace</param>
        /// <param name="rowsetDef">RowsetDef</param>
        /// <param name="rowset">   Rowset</param>
        /// <param name="str">      Message</param>
        //
        //*********************************************************	    
        public static void Trace(TraceTypeE eType, TraceCatE eCat, RepPP.RowsetDef rowsetDef, RepPP.Rowset rowset, string str) {
            if ((eCat & (TraceCatE)32768) != 0) {   // Put to 32767 to enable all trace
                string  strTmp;
                string  strMsg;
                               
                if (eType == TraceTypeE.TT_ENTER) {
                    strTmp = new String('+', m_iTraceLevel) + "<Enter>";
                    m_iTraceLevel++;
                } else if (eType == TraceTypeE.TT_EXIT) {
                    m_iTraceLevel--;
                    strTmp = new String('+', m_iTraceLevel) + "<Exit>";
                } else {
                    strTmp = new String('+', m_iTraceLevel);
                }
                strMsg = strTmp.PadRight(24, ' ');
                if (rowset == null) {
                    strTmp = "rowset:<null> rowsetDef:" + rowsetDef.Name;
                } else if (rowset.IsDetached) {
                    strTmp = "rowset:<Detached> rowsetDef:" + rowsetDef.Name;
                } else {
                    strTmp = "rowset:" + rowset.ToString();
                }
                strMsg += strTmp.PadRight(64, ' ') + str;
                System.Diagnostics.Trace.WriteLine(strMsg);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Trace:                  Output a trace message
        /// </summary>
        /// <param name="eType">    Trace type</param>
        /// <param name="eCat">     Trace category</param>
        /// <param name="str">      Message</param>
        //
        //*********************************************************	    
        public void Trace(TraceTypeE eType, TraceCatE eCat, string str) {
            Trace(eType, eCat, m_rowsetDef, m_rowset, str);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// BuildTableStructv:   Builds the table structure 
        /// </summary>
        //
        //*********************************************************	    
        private void BuildTableStructv() {
            DataColumn      col;
            Type            type;
            
            Columns.Clear();
            foreach(RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldDate:
                case RepPP.FieldType.sdFieldDateTime:
                case RepPP.FieldType.sdFieldTime:
                case RepPP.FieldType.sdFieldHexa:
                case RepPP.FieldType.sdFieldMemo:
                case RepPP.FieldType.sdFieldMoney:
                case RepPP.FieldType.sdFieldNumeric:
                case RepPP.FieldType.sdFieldNumericDec:
                case RepPP.FieldType.sdFieldPercentage:
                case RepPP.FieldType.sdFieldPercentageDec:
                case RepPP.FieldType.sdFieldFormatString:
                case RepPP.FieldType.sdFieldMultiAlphaChoice:
                case RepPP.FieldType.sdFieldMultiNumChoice:
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                case RepPP.FieldType.sdFieldString:
                case RepPP.FieldType.sdFieldUpperString:
                case RepPP.FieldType.sdFieldRaw:
                    type = System.Type.GetType("System.String");
                    break;
                default:
                    type = System.Type.GetType("System.String");
                    break;
                }
                col             = new System.Data.DataColumn(fldDef.Name, type, null, MappingType.Attribute);
                col.AllowDBNull = true; // fldDef.IsRequired ? false : true;
                col.Caption     = fldDef.Prompt;
                col.MaxLength   = fldDef.MaxSize;
                col.ReadOnly    = fldDef.IsSelectable ? false : true;
                Columns.Add(col);
            }
        }		

        //*********************************************************
        //
        /// <summary>
        /// Fillv:           Fill the dataset with value 
        /// </summary>
        //
        //*********************************************************	    
        private void Fillv() {
            int             iLineIndex;
            int             iLineCount;
            int             iColIndex;
            int             iCurLine;
            RepPP.FieldDefs fldDefs;
            DataRow         row;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATAMETHOD, "SCDataTable::Fillv");
            BeginLoadData();            
            Rows.Clear();
            if (m_rowset != null) {
                iLineCount = m_rowset.LineCount;
                iCurLine   = m_rowset.CurrentLine;
                fldDefs    = m_rowset.RowsetDef.FieldDefs;
                for (iLineIndex = 0; iLineIndex < iLineCount; iLineIndex++) {
                    m_rowset.SelectLine(iLineIndex, false, false);
                    iColIndex = 0;
                    row = NewRow();
                    foreach(RepPP.FieldDef fldDef in fldDefs) {
                        row[iColIndex] = fldDef.ExtValue;
                        iColIndex++;
                    }
                    Rows.Add(row);
                    switch(m_rowset.LineState) {
                    case RepPP.LineState.sdLineChanged:
                    case RepPP.LineState.sdLineOld:
                        row.AcceptChanges();
                        break;
                    case RepPP.LineState.sdLineDeleted:
                        row.AcceptChanges();
                        row.Delete();
                        break;
                    case RepPP.LineState.sdLineNew:
                        break;
                    }
                }
                if (iCurLine != m_rowset.CurrentLine && iCurLine != -1) {
                    m_rowset.SelectLine(iCurLine, false, false);
                }
            }
            EndLoadData();            
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATAMETHOD, "SCDataTable::Fillv");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// BuildPrimKeyValStr:  Build primary key value of the current line 
        /// </summary>
        /// <returns>
        /// Primary key
        /// </returns>
        //
        //*********************************************************	    
        protected string BuildPrimKeyValStr() {
            string  str = "";
            
            foreach(RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    str +=  fldDef.ExtValue + " ";
                }
            }
            return(str);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetRowNeutralPosI:      Gets the row neutral index in the row list
        /// </summary>
        /// <param name="row">      Row</param>
        /// <returns>
        /// Row neutral position: 0..max or -1 if not found
        /// </returns>
        //
        //*********************************************************	    
        protected int GetRowNeutralPosI(DataRow row) {
            int     iRetVal = -1;
            int     iPos = 0;
                        
            foreach(DataRow rowIndex in Rows) {
                if (row.Equals(rowIndex)) {
                    iRetVal = iPos;
                    break;
                } else {
                    if (rowIndex.RowState != DataRowState.Deleted) {
                        iPos++;
                    }
                }
            }
            return(iPos);
        }

        //*********************************************************	    
		//
		/// <summary>
		/// CnvRowPosToNeutralPosI:	Convert a row position to a neutral position
		/// </summary>
		/// <param name="iRowPos">  Row position</param>
		/// <returns>
		/// Neutral position:	Position in the list without deleted line and base on 0
		/// </returns>
		//
        //*********************************************************	    
		protected int CnvRowPosToNeutralPosI(int iRowPos) {
			int		iRetVal = iRowPos;
			int		iRowIndex;

			for (iRowIndex = 0; iRowIndex < iRowPos; iRowIndex++) {
				if (this.Rows[iRowIndex].RowState == DataRowState.Deleted) {
					iRetVal--;
				}
			}
			return(iRetVal-1);
		}

        //*********************************************************	    
		//
		/// <summary>
		/// CnvNeutralPosToRowPosI:		Convert a neutral position to a row position
		/// </summary>
		/// <param name="iPos">         Neutral position</param>
		/// <returns>
		/// Row position or -1 if not found
		/// </returns>
		//
        //*********************************************************	    
		protected int CnvNeutralPosToRowPosI(int iPos) {
			int		iRetVal = -1;
			int     iRowIndex;
			int     iRowCount;

			iRowCount = Rows.Count;
			iRowIndex = 0;
			while (iRowIndex < iRowCount && iRetVal == -1) {
				if (Rows[iRowIndex].RowState == DataRowState.Deleted) {
					iRowIndex++;
				} else {
				    if (iPos == 0) {
				        iRetVal = iRowIndex;
				    } else {
				        iRowIndex++;
				        iPos--;
				    }
				}
            }
			return(iRetVal);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// CnvLineToNeutralPosI:	Convert a group instance line position to a neutral position
        /// </summary>
        /// <param name="iLinePos"> Rowset line position</param>
        /// <returns>
        /// Neutral position:	Position in the list without deleted line and base on 0
        /// </returns>
		//
        //*********************************************************	    
        protected int CnvLineToNeutralPosI(int iLinePos) {
			int		iRetVal = -1;
			int		iLineIndex;

            if (m_rowset != null) {
                iRetVal = iLinePos;                
			    for (iLineIndex = 0; iLineIndex < iLinePos; iLineIndex++) {
				    if (m_rowset.ThisLineState(iLineIndex) == RepPP.LineState.sdLineDeleted) {
					    iRetVal--;
				    }
			    }
			}
            return(iRetVal);
        }

        //*********************************************************	    
		//
		/// <summary>
		/// CnvNeutralPosToLineI:		Convert a neutral position to a group instance line position
		/// </summary>
		/// <param name="iPos">         Neutral position</param>
		/// <returns>
		/// Rowset line position (0 = if unknown line)
		/// </returns>
		//
        //*********************************************************	    
		protected int CnvNeutralPosToLineI(int iPos) {
			int		iRetVal = -1;
			int     iLineCount;

            if (m_rowset != null && m_rowset.IsDetached == false) {
                iLineCount = m_rowset.LineCount;
			    while (iPos >= 0 && (iRetVal+1) < iLineCount) {
				    iRetVal++;
				    if (m_rowset.ThisLineState(iRetVal) != RepPP.LineState.sdLineDeleted) {
					    iPos--;
				    }
			    }
			    if (iPos >= 0) {
			        iRetVal = -1;
			    }
			}
			return(iRetVal);
		}

        //*********************************************************
        //
        /// <summary>
        /// CnvNeutralPosToViewPosI:    Transform a neutral position to a view position
        /// </summary>
        /// <param name="iNeutralPos">  Neutral position</param>
        /// <returns>
        /// View position
        /// </returns>
        //
        //*********************************************************
        protected int CnvNeutralPosToViewPosI(int iNeutralPos) {
            int         iRetVal = -1;
            int         iRowIndex;
            int         iViewIndex;
            int         iViewCount;
            DataView    dataView;
            DataRow     row;
            DataRow     rowTmp;
            
            iRowIndex = CnvNeutralPosToRowPosI(iNeutralPos);
            if (iRowIndex != -1) {
                row      = Rows[iRowIndex];
                dataView = DefaultView;
                iRowIndex  = 0;
                iViewIndex = 0;
                iViewCount = dataView.Count;
                while (iRowIndex < iViewCount && iRetVal == -1) {
                    rowTmp = dataView[iRowIndex].Row;
                    if (row.Equals(rowTmp)) {
                        iRetVal = iViewIndex;
                    } else {
                        iRowIndex++;
                        if (rowTmp.RowState != DataRowState.Deleted) {
                            iViewIndex++;
                        } else {
                            MessageBox.Show("Must never happens!");
                        }
                    }
                }
            }
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// CnvViewPosToNeutralPosI:    Transform a view position to a neutral position
        /// </summary>
        /// <param name="iViewPos">     View position</param>
        /// <returns>
        /// Neutral position
        /// </returns>
        //
        //*********************************************************
        protected int CnvViewPosToNeutralPosI(int iViewPos) {
            int     iRetVal = -1;
            int     iViewCount;
            DataRow row;

            iViewCount = DefaultView.Count;
            if (iViewPos < iViewCount) {
                row     = DefaultView[iViewPos].Row;            
                iRetVal = GetRowNeutralPosI(row);
            } else {
                // If not found, line is a new line not already inserted....
                iRetVal = iViewCount;
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnColumnChanging:   Called when a datatable column changed
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************	    
        protected override void OnColumnChanging(DataColumnChangeEventArgs e) {
			int				iRowNeutralPos;
            int             iCurLine;
            RepPP.FieldDef  fldDef;
            string          strValue;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATASINK, "SCDataTable::OnColumnChanging");
            try {
                if (!m_bInitializing && m_rowset != null) {
                    iRowNeutralPos = GetRowNeutralPosI(e.Row);
                    if (iRowNeutralPos == -1) {
                        MessageBox.Show("Unknown row???");
                    } else {
                        iCurLine = CnvNeutralPosToLineI(iRowNeutralPos);
                        Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATASINK, "SCDataTable::OnColumnChanging Grid Line = " + iRowNeutralPos + " New cur line = " + iCurLine + "Old cur line = " +  m_rowset.CurrentLine.ToString());
                        if (m_rowset.CurrentLine != iCurLine) {
                            m_rowset.SelectLine(iCurLine, true, true);
                        }
                        fldDef = m_rowset.RowsetDef.FieldDefs[e.Column.ColumnName];
                        if (fldDef == null) {
                            MessageBox.Show("Unknown field '" + e.Column.ColumnName + "'");
                        } else {
                            strValue = e.ProposedValue.ToString();
                            if (fldDef.ExtValue != strValue) {
                                fldDef.ExtValue = strValue;
                                fldDef.FormatValue();
                                if (fldDef.ExtValue != e.ProposedValue.ToString()) {
                                    e.ProposedValue = fldDef.ExtValue;
                                }
                            }
                        }
                    }
                }
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }
            base.OnColumnChanging(e);
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATASINK, "SCDataTable::OnColumnChanging");
        }

        //*********************************************************
        //
        /// <summary>
        /// ListChangedSink:        Called when a dataview list changes
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************
        protected virtual void ListChangedSink(object sender, ListChangedEventArgs e) {
            int     iViewPos;
            int     iCurLine;
            int     iLine;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATASINK, "SCDataTable::ListChangedSink. Event = " + e.ListChangedType.ToString());
            try {            
                if (!m_bInitializing && m_rowset != null && !m_rowset.IsDetached) {
                    switch(e.ListChangedType) {
                    case ListChangedType.ItemAdded:
                        // Patch: the event is sent two times... don't know why
                        // So, if we're inserting a new line only if the row index
                        // is >= # of lines in rowset
                        if (e.NewIndex >= (m_rowset.LineCount - m_rowset.DeletedLineCount)) {
                            iLine = m_rowset.CurrentLine;
                            m_rowset.AddBlankLine(true);
                            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATASINK, "SCDataTable::ListChangedSink Line added");
                            if (iLine != -1) {
                                m_rowset.SelectLine(iLine, true, false);
                            }
                        }
                        break;
                    case ListChangedType.ItemChanged:
                        break;
                    case ListChangedType.ItemDeleted:
                        iViewPos = e.NewIndex;
                        iLine    = CnvNeutralPosToLineI(CnvViewPosToNeutralPosI(iViewPos));
                        iCurLine = m_rowset.CurrentLine;
                        if (iLine != iCurLine) {
                            m_rowset.SelectLine(iLine, false, false);
                        }
                        if (m_rowset.LineState == RepPP.LineState.sdLineDeleted) {
                            MessageBox.Show("Deleting a deleted line!");
                        } else {
                            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATASINK, "SCDataTable::ListChangedSink Line deleted");
                            m_rowset.DeleteLine();
                            if (iLine != iCurLine) {
                                m_rowset.SelectLine(iCurLine, true, false);
                            }
                        }
                        break;
                    case ListChangedType.ItemMoved:
                        break;
                    case ListChangedType.Reset:
                        break;
                    default:
                        break;
                    }
                }
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATASINK, "SCDataTable::ListChangedSink");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Refresh:    Refresh the content of the dataset with the group instance
        /// </summary>
        //
        //*********************************************************	    
        public virtual void Refresh() {
            m_bInitializing = true;
            Fillv();
            m_bInitializing = false;                
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Rowset:      Get or set the attached group instance
        /// </summary>
        //
        //*********************************************************	    
        public virtual RepPP.Rowset Rowset {
            get {
                return(m_rowset);
            }
            set {
                m_rowset = value;
                Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATAMETHOD, "SCDataTable::Rowset");
                Refresh();
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// LineCount:      Returns the number of lines (not including deleted lined)
        /// </summary>
        //
        //*********************************************************	    
        public int LineCount {
            get {
                int     iRetVal;
                
                if (m_rowset == null) {
                    iRetVal = 0;
                } else {
                    iRetVal = m_rowset.LineCount - m_rowset.DeletedLineCount;
                    if (iRetVal != DefaultView.Count) {
                        MessageBox.Show("Rowset and dataset desynchronized!");
                    }
                }
                return(iRetVal);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InsertLine:             Insert a new row at the current line
        /// </summary>
        //
        //*********************************************************
        public virtual void InsertLine() {
            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATAMETHOD, "SCDataTable::InsertLine");
            if (m_rowset != null) {
                m_rowset.AddBlankLine(false);
                Refresh();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DuplicateCurrentLine:   Duplicate the current line
        /// </summary>
        //
        //*********************************************************
        public virtual void DuplicateCurrentLine() {
            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATAMETHOD, "SCDataTable::DuplicateCurrentLine");
            if (m_rowset != null) {
                m_rowset.DuplicateLine();
                Refresh();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        /// True if line is deleted
        /// </returns>
        //
        //*********************************************************
        public virtual bool DeleteCurrentLine() {
            bool    bRetVal = false;
            int     iCurLine;
            
            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_DATAMETHOD, "SCDataTable::DeleteCurrentLine");
            if (m_rowset != null) {
                iCurLine = m_rowset.CurrentLine;
                if (iCurLine != -1 && m_rowset.LineState != RepPP.LineState.sdLineDeleted) {
                    m_rowset.DeleteLine();
                    Refresh();
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }
    }
}
