using System;
using System.Drawing.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports fields property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class FldControlUtil : ControlUtil {
	    /// <summary>Common interface for controls attached to a field</summary>
	    public interface IFldControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Field name</summary>
	        FldFullName                 FieldFullName {  get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>FieldDef attached to the control</summary>
	    protected FieldDef              m_fldDef;
	    /// <summary>Full name of the field</summary>
	    protected FldFullName           m_fldFullName;
        /// <summary>Indicates whether or not the visible property of the control is bound to the field property </summary>
        protected bool                  m_bBindVisible;
        /// <summary>Indicates whether or not the Selectable (ReadOnly/Enabled) property of the control is bound to the field property </summary>
        protected bool                  m_bBindSelectable;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="ctrlHelper">       Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public FldControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly) : base(parent, ctrlHelper, bResDispenserOnly) {
		    m_fldDef            = null;
		    FieldFullName       = new FldFullName();
            m_bBindVisible      = true;
            m_bBindSelectable   = true;
		}

        //********************************************************************************
        //
        /// <summary>
        /// OnRefreshField:         Called whenever the field full name changes
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event argument</param>
        //
        //********************************************************************************
        private void OnRefreshField(object sender, EventArgs e) {
            m_fldDef = null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:          Full name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public FldFullName FieldFullName {
		    get {
		        return(m_fldFullName);
		    }
		    set {
                if (m_fldFullName != null && m_fldFullName != value){
                    m_fldFullName.NameChanged -= new EventHandler(OnRefreshField);
                }
		        m_fldFullName   = value;
		        m_fldDef        = null;
                if (m_fldFullName != null){
                    m_fldFullName.NameChanged += new EventHandler(OnRefreshField);
                }
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// FieldDef:                  Rep++ FieldDef attached to the control
		/// </summary>
        //
        //********************************************************************************
		public FieldDef FieldDef {
		    get {
                RepPP.FieldDef      fldDefRetVal = null;
		        RepPP.Application   app;
		        
		        if (m_fldDef != null && m_fldDef.IsDetached) {
		            m_fldDef = null;
		        }
		        if (m_fldDef != null) {
                    fldDefRetVal = m_fldDef;
                } else {
		            app          = Application;
	                fldDefRetVal = app != null ? app.FieldDef(FieldFullName.RowsetName, FieldFullName.FieldName) : null;
                    if (!IsDesignMode) {
                        m_fldDef = fldDefRetVal;
                    }
		        }
		        return(fldDefRetVal);
		    }
		    set {
		        if (value != m_fldDef) {
		            m_fldDef = null;
		            if (value == null) {
		                FieldFullName = new FldFullName();
		            } else {
		                FieldFullName = new FldFullName(value.RowsetDef.Name, value.Name);
		                m_fldDef      = value;
		            }
		        }
		    }
		}

        //********************************************************************************
        //
		/// <summary>
        /// Field:          Rep++ Field attached to the control
		/// </summary>
        //
        //********************************************************************************
        public Field Field {
            get {
                RepPP.Field      fldRetVal = null;
                RepPP.Rowset     rowset;

                if (FieldDef != null && m_fldDef != null) {
                    rowset = m_fldDef.RowsetDef.ActiveRowset;
                    if (rowset != null) {
                        fldRetVal = rowset.Fields[m_fldDef.Name];
                    }
                }
                return(fldRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        public bool BindVisible {
            get {
                return(m_bBindVisible);
            }
            set {
                m_bBindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindSelectable:         Indicates whether or not the Selectable (ReadOnly/Enabled) property
        ///                         of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        public bool BindSelectable {
            get {
                return(m_bBindSelectable);
            }
            set {
                m_bBindSelectable = value;
            }
        }
    
    } // Class FldControlUtil

} // End Namespace