using System;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;

namespace RepPP.Toolkit {

	//********************************************************************************
	//
	/// <summary>
	/// Generate report from a card instance using XSL template.
    ///
    ///	(c) 2002, Consyst SQL Inc, 
	///	All rights reserved
	/// </summary>
	//
	//********************************************************************************
    public class GenXSLReport {
        private string              m_strTempDir;
        private string              m_strGuid;
        
        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="strTempDir">   Temporary filename</param>
        //
        //******************************************************************
        public GenXSLReport(string strTempDir) {
            m_strTempDir = strTempDir;
            m_strGuid    = Guid.NewGuid().ToString();
        }

        //******************************************************************
        //
        /// <summary>
        /// Gets/Sets the temporary path where the Report will be generated
        /// </summary>
        //
        //******************************************************************
        public string TemporaryPath {
            get {
                return(m_strTempDir);
            }
            set {
                m_strTempDir = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        //
        //******************************************************************
        public GenXSLReport() : this(System.IO.Path.GetTempPath()) {
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateXSLFile:                  Create a XSL file using 
        /// </summary>
        /// <param name="strPrefixName">    File prefix Name</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <returns>
        /// XML Filename
        /// </returns>
        //
        //******************************************************************
        private string CreateXSLFile(string strPrefixName, string strXSLDocument) {
            const string    cstrFILENAME    = "xlsReport{0}{1}.xsl";
            StreamWriter    streamWriter    = null;
            string          strFilename;

            strFilename  = System.IO.Path.Combine(m_strTempDir, 
                                                  string.Format(cstrFILENAME, m_strGuid, strPrefixName));
            using(streamWriter = File.CreateText(strFilename)) {
                streamWriter.Write(strXSLDocument);
            } 
            return(strFilename);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateReport:                 Generates a report in a browser 
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree to generate Report</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <param name="bHaveChild">       Generate XLS for Child group instance</param>
        /// <returns>
        /// XML Filename
        /// </returns>
        //
        //******************************************************************
        public string GenerateReport(RepPP.RowsetTree   rowsetTree, 
                                     string             strXSLDocument, 
                                     bool               bHaveChild) {
            string  strRetVal = null;

            CreateXSLFile(rowsetTree.RowsetTreeDef.Name, strXSLDocument);
            strRetVal = SerializeToXml(rowsetTree, bHaveChild);
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateHTMLReport:             Generate HTML Report
        /// </summary>
        /// <param name="strPrefixName">    File prefix Name</param>
        /// <param name="strXmlFileName">   XLST Template source</param>
        /// <param name="strXSLDocument">   Generate XLS for Child group instance</param>
        /// <returns>
        /// HTML Filepath
        /// </returns>
        //
        //******************************************************************
        private string GenerateHTMLReport(string              strPrefixName,
                                          string              strXmlFileName,
                                          string              strXSLDocument) {
            const string    cstrXSLFILENAME     = "xlsReport{0}{1}.xsl";
            const string    cstrHTMLFILENAME    = "report{0}{1}.html";            
			XslCompiledTransform xlsTransform;
            XmlUrlResolver  resolver;
            string          strXslFilename;
            string          strRetVal           = null;
			xlsTransform		 = new XslCompiledTransform();
            resolver             = new XmlUrlResolver();            
            strXslFilename       = System.IO.Path.Combine(m_strTempDir, 
                                                          string.Format(cstrXSLFILENAME, m_strGuid, strPrefixName));
            strRetVal            = System.IO.Path.Combine(m_strTempDir, 
                                                          string.Format(cstrHTMLFILENAME, m_strGuid, strPrefixName));           
            resolver.Credentials = System.Net.CredentialCache.DefaultCredentials;
			XmlWriter			xmlWriter;
            XmlReader           xmlReaderXmlSrc;
            XmlReader           xmlReaderXlsSrc;
			XmlReaderSettings   readerSettings  = new XmlReaderSettings();
                 
#if (VC10)
            readerSettings.DtdProcessing = DtdProcessing.Ignore;
#else
            readerSettings.ProhibitDtd = false;
#endif
            using (xmlReaderXmlSrc = XmlReader.Create(strXmlFileName, readerSettings)) {
                using (xmlReaderXlsSrc = XmlReader.Create(strXslFilename, readerSettings)) {
                    try {
                        xlsTransform.Load(xmlReaderXlsSrc, XsltSettings.Default, resolver);                
                        using (FileStream stream1 = new FileStream(strRetVal, FileMode.Create, FileAccess.Write)) {
                            using (xmlWriter = XmlWriter.Create(stream1)) {
                                xlsTransform.Transform(xmlReaderXmlSrc, null, xmlWriter, resolver);
                            }
                        }
                    } catch(XsltException) { //XmlReader hide the Xslt Exception...
                        throw;
                    }
                }
            }
			return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateHTMLReport:             Generate HTML Report for a single
        ///                                 Rowset
        /// </summary>
        /// <param name="rowset">           Rowset to generate Report</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <returns>
        /// HTML Filepath
        /// </returns>
        //
        //******************************************************************
        public string GenerateHTMLReport(RepPP.Rowset        rowset, 
                                         string              strXSLDocument) {
            string      strXmlFileName;

            strXmlFileName = GenerateReport(rowset, strXSLDocument);  
            return(GenerateHTMLReport(rowset.RowsetDef.Name, strXmlFileName, strXSLDocument));
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateHTMLReport:             Generate HTML Report
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree to generate Report</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <param name="bHaveChild">       Generate XLS for Child group instance</param>
        /// <returns>
        /// HTML Filepath
        /// </returns>
        //
        //******************************************************************
        public string GenerateHTMLReport(RepPP.RowsetTree   rowsetTree, 
                                         string             strXSLDocument, 
                                         bool               bHaveChild) {
            string      strXmlFileName;

            strXmlFileName = GenerateReport(rowsetTree, strXSLDocument, bHaveChild);  
            return(GenerateHTMLReport(rowsetTree.RowsetTreeDef.Name, strXmlFileName, strXSLDocument));
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateReport:                 Generate XLS Report without Child
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree to generate Report</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <returns>
        /// XSL Filepath
        /// </returns>
        //
        //******************************************************************
        public string GenerateReport(RepPP.RowsetTree rowsetTree, string strXSLDocument) {
            return(GenerateReport(rowsetTree, strXSLDocument, false));
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateReport:                 Generate XLS Report from a single
        ///                                 Rowset
        /// </summary>
        /// <param name="rowset">           Rowset to generate Report</param>
        /// <param name="strXSLDocument">   XLST Template source</param>
        /// <returns>
        /// XSL Filepath
        /// </returns>
        //
        //******************************************************************
        public string GenerateReport(RepPP.Rowset rowset, string strXSLDocument) {
            string          strRetVal       = null;
            string          strPrefixName;

            strPrefixName = rowset.RowsetDef.Name;
            CreateXSLFile(strPrefixName, strXSLDocument);
            strRetVal = SerializeToXml(rowset, strPrefixName, false);
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetFieldDescription:            Get Description from ChoiceList FieldDef 
        /// </summary>
        /// <param name="fldDef">           FieldDef to get Description</param>
        /// <returns>
        /// Field Description
        /// </returns>
        //
        //******************************************************************
        private string GetFieldDescription(RepPP.FieldDef fldDef) {
            string[]  arrValue;
            string    strRetVal = "";
            int       iIndex;

            if (fldDef.IsChoiceAvailable) {
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice :
                case RepPP.FieldType.sdFieldSimpleNumChoice   :
                    iIndex = fldDef.ChoiceList.FindValue(fldDef.Value);
                    if (iIndex != -1) {
                        strRetVal = fldDef.ChoiceList.GetDesc(iIndex);
                    }
                    break;
                case RepPP.FieldType.sdFieldMultiAlphaChoice :
                case RepPP.FieldType.sdFieldMultiNumChoice   :
                    arrValue = fldDef.ExtValue.Split(',');
                    foreach(string strValue in arrValue) {
                        iIndex = fldDef.ChoiceList.FindCode(strValue);
                        if (iIndex != -1) {
                            strRetVal += (strRetVal != string.Empty) ? "," + fldDef.ChoiceList.GetDesc(iIndex) :
                                                                             fldDef.ChoiceList.GetDesc(iIndex); 
                        }
                    }
                    break;
                }
            }
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateXML:            Generate XML from a group Instance                
        /// </summary>
        /// <param name="xmlFather">XML DOM</param>
        /// <param name="rowset">   Current Rowset</param>
        /// <param name="bChild">   Generate XML for Child group instance</param>
        //
        //******************************************************************
        private void GenerateXML(XmlNode xmlFather, RepPP.Rowset rowset, bool bChild) {
            XmlDocument     xmlDoc;
            XmlNode         xmlGinst;
            XmlNode         xmlLines;
            XmlNode         xmlLine;
            XmlNode         xmlFields;
            XmlNode         xmlNode;
            XmlAttribute    xmlNewAttr;
            string          strFldDesc;
            int             iLineCount;            
            
            xmlDoc    = (xmlFather is XmlDocument) ? xmlFather as XmlDocument : xmlFather.OwnerDocument;
            xmlGinst  = xmlFather.AppendChild(xmlDoc.CreateElement(rowset.RowsetDef.Name));
            xmlFields = xmlGinst.AppendChild(xmlDoc.CreateElement("FIELDS"));
            xmlLines  = xmlGinst.AppendChild(xmlDoc.CreateElement("LINES"));
   			foreach (RepPP.FieldDef fldDef in rowset.RowsetDef.FieldDefs) {
                xmlNode           = xmlFields.AppendChild(xmlDoc.CreateElement(fldDef.Name));
                xmlNode.InnerText = fldDef.Prompt;
			}
			rowset.RowsetDef.SaveActiveRowset();
            rowset.SaveLinePos();
            rowset.Select(false, false);
            try {
                iLineCount = rowset.LineCount;
                for (int i=0; i < iLineCount; i++) {
                    rowset.SelectLine(i, false, false);
                    if (rowset.LineState != RepPP.LineState.sdLineDeleted) {
                        xmlLine = xmlLines.AppendChild(xmlDoc.CreateElement("LINE"));
   			            foreach (RepPP.FieldDef fldDef in rowset.RowsetDef.FieldDefs) {
                            xmlNode           = xmlLine.AppendChild(xmlDoc.CreateElement(fldDef.Name));
                            xmlNode.InnerText = fldDef.ExtValue;
                            xmlNewAttr        = xmlNode.Attributes.Append(xmlDoc.CreateAttribute("INTVALUE"));
                            xmlNewAttr.Value  = fldDef.Value;                    
                            if (fldDef.IsChoiceAvailable) {
                                strFldDesc = GetFieldDescription(fldDef);
                                if (strFldDesc != string.Empty) {
                                    xmlNewAttr       = xmlNode.Attributes.Append(xmlDoc.CreateAttribute("DESCRIPTION"));
                                    xmlNewAttr.Value = strFldDesc;                    
                                }
                            }
                        }
                        if (bChild) {
                            foreach(RepPP.Rowset rowsetChild in rowset.ChildRowsets) {
                                GenerateXML(xmlLine, rowsetChild, true);
                            }
                        }
                    }
                }
            } finally {
                rowset.RestoreLinePos(false);
                rowset.RowsetDef.RestoreActiveRowset(false);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SerializeToXml:          Save Rowset tree to XML File
        /// </summary>
        /// <param name="rowsetTree">Rowset tree to save</param>
        /// <param name="bChild">    Build XML Child Rowset</param>
        /// <returns>
        /// XML Filepath
        /// </returns>
        //
        //******************************************************************
        public string SerializeToXml(RepPP.RowsetTree rowsetTree, bool bChild) {
            return(SerializeToXml(rowsetTree.RootRowset, rowsetTree.RowsetTreeDef.Name, bChild));
        }

        //******************************************************************
        //
        /// <summary>
        /// SerializeToXml:         Save Group Rowset to XML File
        /// </summary>
        /// <param name="rowset">   Rowset to save</param>
        /// <param name="strPrefixName">    File prefix Name</param>
        /// <param name="bChild">   Build XML Rowset</param>
        /// <returns>
        /// XML Filepath
        /// </returns>
        //
        //******************************************************************
        public string SerializeToXml(RepPP.Rowset rowset, string strPrefixName, bool bChild) {
            const string    cstrENCODING     = "UTF-8";
            const string    cstrFILENAME_XML = "xlsReport{0}{1}.xml";
            const string    cstrPROCESSOR    = @"type=""text/xsl"" href=""xlsReport{0}{1}.xsl""";
            XmlDocument     xmlDoc;
            XmlTextWriter   xmlWriter        = null;
            string          strRetVal        = null;

            xmlDoc    = new XmlDocument();
            strRetVal = System.IO.Path.Combine(m_strTempDir, 
                                               string.Format(cstrFILENAME_XML, m_strGuid, strPrefixName)); 
            xmlWriter = new XmlTextWriter(strRetVal, System.Text.Encoding.GetEncoding(cstrENCODING));
            try {
                xmlWriter.Formatting = Formatting.Indented;
                xmlDoc.AppendChild(xmlDoc.CreateXmlDeclaration("1.0", cstrENCODING, null));
                xmlDoc.AppendChild(xmlDoc.CreateProcessingInstruction("xml-stylesheet", 
                                                                      string.Format(cstrPROCESSOR, m_strGuid, strPrefixName)));
                GenerateXML(xmlDoc, rowset, bChild);
                xmlDoc.Save(xmlWriter);
            } finally {
                xmlWriter.Close();
            }
            return(strRetVal);
        }

    } // End class GenXSLReport
} // End namespace
