using System;
using System.Collections;

namespace RepPP.Toolkit {
	/// <summary>
	/// Handle the historic for a transaction
	/// </summary>
	public abstract class HistoricBase : IDisposable {
        /// <summary>Type of operation to log</summary>
        public enum LogTypeE {
            /// <summary>No log</summary>
            None     =  0,
            /// <summary>Log only when row is inserted</summary>
            AtInsert =  1,
            /// <summary>Log only when row is updated</summary>
            AtUpdate =  2,
            /// <summary>Log only when row is deleted</summary>
            AtDelete =  4,
            /// <summary>Log when row is inserted, updated or deleted</summary>
            AtAll    =  7
        };
        
        /// <summary>Counter to give unique name to each created group</summary>
        static private int                  m_iCount = 0;
        /// <summary>Rep++ Application object</summary>
        protected RepPP.Application         m_app;
        /// <summary>Rep++ database connection to use</summary>
        protected RepPP.Connection          m_conn;
        /// <summary>Node of the rowset tree definition for which the historic is done</summary>
        protected RepPP.RowsetTreeDefNode   m_nodeTable;
        /// <summary>RowsetTreeDef which can be used to read the historic</summary>
        protected RepPP.RowsetTreeDef       m_rowsetTreeDefHistoric;
        /// <summary>Rowset tree which can be used to read the historic</summary>
        protected RepPP.RowsetTree          m_rowsetTreeHistoric;
        /// <summary>RowsetDef pointing to the historic table</summary>
        protected RepPP.RowsetDef           m_rowsetDefHistoric;
        /// <summary>Rowset for the historic table</summary>
        protected RepPP.Rowset              m_rowsetHistoric;
        /// <summary>SQL command object use to insert the line in the log</summary>
        protected RepPP.SqlCommand          m_sqlInsert;
        /// <summary>SQL command to insert into the historic table</summary>
        private string                      m_strInsSqlCmd;
        /// <summary>Type of log</summary>
        protected LogTypeE                  m_eLogType;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="conn">                 Database connection to use (null for DataConnection)</param>
        /// <param name="strHistTablePrefix">   Prefix to add to the table name to build the historic table name</param>
        /// <param name="strHistTableSuffix">   Suffix to add to the table name to build the historic table name</param>
        /// <param name="eLogType">             Type of event to log. More than one option can be specified</param>
        /// <param name="nodeTable">            Node of the rowset tree definition pointing to the table to check for historic</param>
        /// <param name="rowsetDefInfo">        RowsetDef containing the additional field to put in the historic</param>
		/// <param name="arrFldName">           List of fields coming from the base rowset which must be
		///                                     included in the historic rowset. null for all</param>
		/// <param name="bInfoBefore">          true to put the information before the other fields</param>
        //
        //********************************************************************************
		protected HistoricBase(RepPP.Connection         conn,
		                       string                   strHistTablePrefix,
		                       string                   strHistTableSuffix,
		                       LogTypeE                 eLogType,
		                       RepPP.RowsetTreeDefNode  nodeTable,
		                       RepPP.RowsetDef          rowsetDefInfo,
		                       string[]                 arrFldName,
		                       bool                     bInfoBefore) {
	        ArrayList   arrFldName2;
	        string      strNewTableName;
	        string      strDummy;
	        
	        m_app           = nodeTable.Application;
	        m_conn          = (conn == null) ? m_app.DataConnection : conn;
	        arrFldName2     = (arrFldName == null) ? null : new ArrayList(arrFldName);
	        strNewTableName = null;
	        foreach (RepPP.FieldDef fldDef in nodeTable.RowsetDef.FieldDefs) {
	            if (fldDef.IsPrimaryKey) {
	                strNewTableName = fldDef.TableName;
	                break;
	            }
	        }
	        if (strNewTableName == null) {
	            throw new System.ApplicationException("The rowset " + nodeTable.RowsetDef.Name + " must have a primary key and an attached table");
	        }
	        strNewTableName = BuildHistTableName(strNewTableName, strHistTablePrefix, strHistTableSuffix);
	        m_eLogType          = eLogType;
	        m_nodeTable         = nodeTable;
	        m_rowsetDefHistoric = BuildNewRowsetDef("$HIST" + (m_iCount++).ToString(),
	                                                strNewTableName,
	                                                nodeTable,
	                                                rowsetDefInfo,
	                                                arrFldName2,
	                                                bInfoBefore);
	        m_rowsetTreeDefHistoric   = m_app.RowsetTreeDefs.Add(m_rowsetDefHistoric.Name);
	        m_rowsetTreeDefHistoric.Nodes.Add(m_rowsetDefHistoric,
	                                          RepPP.RowsetType.sdHorizontal,
	                                          RepPP.RowsetFlag.sdNormal,
	                                          false,
	                                          0,
	                                          0,
	                                          m_conn,
	                                          RepPP.SqlOptimization.sdOptFull);
	        m_rowsetTreeDefHistoric.BuildSqlCommand();
	        m_rowsetTreeHistoric = m_rowsetTreeDefHistoric.RowsetTrees.Add();
            m_rowsetHistoric     = m_rowsetDefHistoric.Rowsets.Create(RepPP.RowsetType.sdVertical);
            m_rowsetDefHistoric.BuildSqlCommand("", out strDummy, out m_strInsSqlCmd, out strDummy, out strDummy);
            if ((eLogType & LogTypeE.AtInsert) != 0) {
                m_app.DbAfterInsLine += new RepPP.DbAfterInsLineEventHandler(m_app_DbAfterInsLine);
            }
            if ((eLogType & LogTypeE.AtUpdate) != 0) {
                m_app.DbAfterUpdLine += new RepPP.DbAfterUpdLineEventHandler(m_app_DbAfterUpdLine);
            }
            if ((eLogType & LogTypeE.AtDelete) != 0) {
                m_app.DbAfterDelLine += new RepPP.DbAfterDelLineEventHandler(m_app_DbAfterDelLine);
            }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:                Dispose the object
		/// </summary>
        //
        //********************************************************************************
		public virtual void Dispose() {
            m_app.DbAfterInsLine -= new RepPP.DbAfterInsLineEventHandler(m_app_DbAfterInsLine);
            m_app.DbAfterUpdLine -= new RepPP.DbAfterUpdLineEventHandler(m_app_DbAfterUpdLine);
            m_app.DbAfterDelLine -= new RepPP.DbAfterDelLineEventHandler(m_app_DbAfterDelLine);
		    if (m_rowsetTreeHistoric != null) {
		        m_rowsetTreeHistoric.Delete();
		        m_rowsetTreeHistoric = null;
		    }
		    if (m_rowsetTreeDefHistoric != null) {
		        m_rowsetTreeDefHistoric.Delete();
		        m_rowsetTreeDefHistoric = null;
		    }
		    if (m_sqlInsert != null) {
		        m_sqlInsert.Dispose();
		        m_sqlInsert = null;
		    }
		    if (m_rowsetHistoric != null) {
		        m_rowsetHistoric.Delete();
		        m_rowsetHistoric = null;
		    }
		    if (m_rowsetDefHistoric != null) {
		        m_rowsetDefHistoric.Delete();
		        m_rowsetDefHistoric = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// HistoricRowsetTree:   Rowset tree which can be used to read the historic
		/// </summary>
        //
        //********************************************************************************
		public RepPP.RowsetTree HistoricRowsetTree {
		    get {
		        return(m_rowsetTreeHistoric);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// InsertSqlCmd:           SQL command use to insert in the historic
		/// </summary>
        //
        //********************************************************************************
		public string InsertSqlCmd {
		    get {
		        return(m_strInsSqlCmd);
		    }
		    set {
		        m_strInsSqlCmd = value;
		        if (m_sqlInsert != null) {
		            m_sqlInsert.Dispose();
		            m_sqlInsert = null;
		        }
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// BuildHistTableName:                 Routine use to build the historic table name from the base
		///                                     table name. This method can be override to specify a new way to
		///                                     get the historic table name
		/// </summary>
		/// <param name="strTableName">         Base table name</param>
		/// <param name="strHistTablePrefix">   Prefix to add</param>
		/// <param name="strHistTableSuffix">   Suffix to add</param>
		/// <returns>
		/// Historic table name
		/// </returns>
        //
        //********************************************************************************
		protected virtual string BuildHistTableName(string  strTableName,
		                                            string  strHistTablePrefix,
		                                            string  strHistTableSuffix) {
		    string  strRetVal = "";
		    
		    if (strHistTablePrefix != null) {
		        strRetVal += strHistTablePrefix;
		    }
		    strRetVal += strTableName;
		    if (strHistTableSuffix != null) {
		        strRetVal += strHistTableSuffix;
		    }
		    return(strRetVal);
		}

        //********************************************************************************
        //
		/// <summary>
		/// AddField:                   Add a field to the group using another field as template
		/// </summary>
		/// <param name="rowsetDef">    RowsetDef</param>
		/// <param name="fldDefSrc">    Source field definition to use as template</param>
		/// <param name="strTableName"> Name of the table attached to the new field</param>
		/// <param name="iLogPos">      Position of the field in the rowset</param>
        //
        //********************************************************************************
        private void AddFieldDef(RepPP.RowsetDef rowsetDef, RepPP.FieldDef fldDefSrc, string strTableName, int iLogPos) {
		    RepPP.FieldOptions      eFldOptions;
		    RepPP.FieldDef          fldDefNew;
		    
		    eFldOptions = (fldDefSrc.IsPrimaryKey) ? RepPP.FieldOptions.sdFieldPK | RepPP.FieldOptions.sdFieldRequired : (RepPP.FieldOptions)0;
		    if (fldDefSrc.IsPassword) {
		        eFldOptions |= RepPP.FieldOptions.sdFieldNoEcho;
		    }
		    fldDefNew   = rowsetDef.FieldDefs.Add(fldDefSrc.Name,
		                                          iLogPos,
		                                          strTableName,
		                                          fldDefSrc.ColName,
		                                          fldDefSrc.Type,
		                                          eFldOptions,
		                                          fldDefSrc.MinSize,
                                                  fldDefSrc.MaxSize,
		                                          fldDefSrc.Precision);
		    if (fldDefSrc.IsChoiceAvailable && fldDefSrc.ChoiceList.List != null) {
		        fldDefNew.ChoiceList.List = fldDefSrc.ChoiceList.List;
		    }
		    fldDefNew.InputMask  = fldDefSrc.InputMask;
		    fldDefNew.Prompt     = fldDefSrc.Prompt;
		    fldDefNew.OutputMask = fldDefSrc.OutputMask;
		    fldDefNew.IsVisible  = fldDefSrc.IsVisible;
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddFatherPKField:               Add the primary key fields of the ancesters
        /// </summary>
        /// <param name="rowsetDef">        RowsetDef to add the field to</param>
		/// <param name="strNewTableName">  Historic table name</param>
        /// <param name="crsetDefFather">   Father child rowset definition</param>
        /// <param name="iLogPos">          Actual position</param>
        //
        //********************************************************************************
        private void AddFatherPKField(RepPP.RowsetDef rowsetDef, string strNewTableName, RepPP.RowsetTreeDefNode crsetDefFather, ref int iLogPos) {
            if (crsetDefFather != null) {
                AddFatherPKField(rowsetDef, strNewTableName, crsetDefFather.Parent as RepPP.RowsetTreeDefNode, ref iLogPos);
                foreach (RepPP.FieldDef fldDef in crsetDefFather.RowsetDef.FieldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        AddFieldDef(rowsetDef, fldDef, strNewTableName, iLogPos++);                        
                    }
                }
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// BuildNewRowsetDef:              Build a new rowset definition which represent the record to
		///                                 insert in the historic table
		/// </summary>
		/// <param name="strNewRowsetName"> Name of the new group</param>
		/// <param name="strNewTableName">  Historic table name</param>
		/// <param name="nodeTable">        Base node of the rowset tree definition</param>
		/// <param name="rowsetDefInfo">    RowsetDef containing the fields to the historic group</param>
		/// <param name="arrFldName">       List of fields coming from the base group which must be
		///                                 included in the historic group. null for all</param>
		/// <param name="bInfoBefore">      true to put the information before the other fields</param>
		/// <returns>
		/// New rowset definition
		/// </returns>
        //
        //********************************************************************************
		private RepPP.RowsetDef BuildNewRowsetDef(string                    strNewRowsetName,
		                                          string                    strNewTableName,
		                                          RepPP.RowsetTreeDefNode   nodeTable,
		                                          RepPP.RowsetDef           rowsetDefInfo,
		                                          ArrayList                 arrFldName,
		                                          bool                      bInfoBefore) {
		    RepPP.RowsetDef rowsetDefRetVal;
		    int             iLogPos = 1;
		    
		    rowsetDefRetVal = m_app.RowsetDefs.Add(strNewRowsetName, "", true);
		    if (bInfoBefore) {
		        iLogPos = 500;
		    }
		    AddFatherPKField(rowsetDefRetVal, strNewTableName, nodeTable.Parent as RepPP.RowsetTreeDefNode, ref iLogPos);
		    foreach (RepPP.FieldDef fldDef in nodeTable.RowsetDef.FieldDefs) {
		        if (fldDef.IsDatabaseLinked) {
		            if (arrFldName == null ||
		                arrFldName.IndexOf(fldDef.Name) != -1) {
		                AddFieldDef(rowsetDefRetVal, fldDef, strNewTableName, iLogPos++);
		            }
		        }
		    }
		    if (bInfoBefore) {
		        iLogPos = 1;
		    }
		    foreach (RepPP.FieldDef fldDef in rowsetDefInfo.FieldDefs) {
		        AddFieldDef(rowsetDefRetVal, fldDef, strNewTableName, iLogPos++);
		    }
		    return(rowsetDefRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillLogRecord:                  Fill the extra field in the historic group. This method
        ///                                 must be overriden.
        /// </summary>
        /// <param name="eType">            Type of log (insert, update, delete)</param>
        /// <param name="rowsetSrc">        Rowset containing the value of the base table which is inserted, updated or deleted</param>
        /// <param name="rowsetLogRecord">  Historic group instance that you must complete</param>
        /// <returns>
        /// true if ok, false if error.
        /// </returns>
        //
        //********************************************************************************
        protected abstract bool FillLogRecord(LogTypeE      eType,
                                              RepPP.Rowset  rowsetSrc,
                                              RepPP.Rowset  rowsetLogRecord);

        //********************************************************************************
        //
        /// <summary>
        /// InsertLogRecord:                Insert a log record in the database
        /// </summary>
        /// <param name="eType">            Type of log record (insert, update, delete)</param>
        /// <param name="rowsetSrc">        Rowset containing the value of the base table</param>
        /// <param name="rowsetLogRecord">  Historic group instance that you must complete</param>
        /// <returns>
        /// true if success, false if error.
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool InsertLogRecord(LogTypeE     eType,
                                               RepPP.Rowset rowsetSrc,
                                               RepPP.Rowset rowsetLogRecord) {
            bool            bRetVal;
            RepPP.Rowset    rowsetFather;
            
            m_rowsetHistoric.RowsetDef.SaveActiveRowset();
            try {
                m_rowsetHistoric.Select(false, false);
                m_rowsetHistoric.Empty();
                rowsetFather = rowsetSrc.FatherRowset;
                while (rowsetFather != null) {
                    m_rowsetDefHistoric.CopyFieldsFrom(rowsetFather.RowsetDef, RepPP.CopyFieldOpt.sdSetValid);
                    rowsetFather = rowsetFather.FatherRowset;
                }
                m_rowsetDefHistoric.CopyFieldsFrom(rowsetSrc.RowsetDef, RepPP.CopyFieldOpt.sdSetValid);
                bRetVal = FillLogRecord(eType, rowsetSrc, rowsetLogRecord);
                if (bRetVal) {
                    if (m_sqlInsert == null) {
                        m_sqlInsert = m_conn.SqlCommands.Open(m_strInsSqlCmd, RepPP.SqlOptimization.sdOptCursor);
                    }
                    if (m_sqlInsert.Execute() != 0) {
                        m_app.Errors.Add(0,
                                         RepPP.ErrorLevel.sdErrValidation,
                                         m_conn.ErrorMessage,
                                         m_conn.Name,
                                         RepPP.ObjType.sdRowset,
                                         rowsetLogRecord);
                        bRetVal = false;
                        m_conn.Rollback();
                    }
                }
            } finally {
                m_rowsetHistoric.RowsetDef.RestoreActiveRowset(false);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_app_DbAfterInsLine:   Called when a line is inserted
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //********************************************************************************
        private void m_app_DbAfterInsLine(object sender, RepPP.RowsetEventArgs e) {
            if (e.Rowset.RowsetDef == m_nodeTable.RowsetDef) {
                if (!InsertLogRecord(LogTypeE.AtInsert,
                                     e.Rowset,
                                     m_rowsetHistoric)) {
                    e.EventState = RepPP.EventState.sdReject;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_app_DbAfterUpdLine:   Called when a line is updated
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //********************************************************************************
        private void m_app_DbAfterUpdLine(object sender, RepPP.RowsetEventArgs e) {
            if (e.Rowset.RowsetDef == m_nodeTable.RowsetDef) {
                if (!InsertLogRecord(LogTypeE.AtUpdate,
                                     e.Rowset,
                                     m_rowsetHistoric)) {
                    e.EventState = RepPP.EventState.sdReject;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_app_DbAfterDelLine:   Called when a line is deleted
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //********************************************************************************
        private void m_app_DbAfterDelLine(object sender, RepPP.RowsetEventArgs e) {
            if (e.Rowset.RowsetDef == m_nodeTable.RowsetDef) {
                if (!InsertLogRecord(LogTypeE.AtDelete,
                                     e.Rowset,
                                     m_rowsetHistoric)) {
                    e.EventState = RepPP.EventState.sdReject;
                }
            }
        }
    }
}
