using System;
using System.Collections;
using System.Windows.Forms;
using RepPP;
using System.ComponentModel;
//
// Rep++ Search Method Interface
// Copyright (C) 2008 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
//
namespace RepPP.Toolkit {

    /// <summary>Enumeration of the possible search method execution states</summary>
    public enum SearchMethodExecutionStatusE {
        /// <summary>The execution of the search method was cancelled</summary>
        SMES_Cancel     = 0,
        /// <summary>The Where clause of the search method was modified but not yet executed</summary>
        SMES_Continue   = 1,
        /// <summary>The user has already performed the execution of the search method</summary>
        SMES_Skip       = 2,
    };

    //*********************************************************
    //
    /// <summary>
    /// BeforeExecuteEventArgs:  Arguments of the event when 
    ///                          a search method is about to be
    ///                          executed.
    /// </summary>
    //
    //*********************************************************
    public class BeforeExecuteEventArgs : System.EventArgs {
        /// <summary>Original Sql Command</summary>
        public  string                          OriginalSqlCmd;
        /// <summary>The Where Clause</summary>
        public  string                          SqlWhereClause;
        /// <summary>The Execution Status</summary>
        public  SearchMethodExecutionStatusE    Status;
        /// <summary>RowsetTree that will conatin the loaded data</summary>
        public  RowsetTree                      RowsetTree;
        /// <summary>The DataAccess that will be used to load the data</summary>
        public  ISBDataAccess                   DataAccess;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        /// <param name="strOriginalSqlCmd"> Original Sql Command</param>
        /// <param name="strSqlWhereClause"> Sql Where Clause</param>
        /// <param name="rowsetTree">        Container RowsetTree</param>
        /// <param name="dataAccess">        DataAccess object</param>
        //
        //*********************************************************
        public BeforeExecuteEventArgs(string        strOriginalSqlCmd, 
                                      string        strSqlWhereClause,
                                      RowsetTree    rowsetTree,
                                      ISBDataAccess dataAccess) : base() {
            OriginalSqlCmd = strOriginalSqlCmd; 
            SqlWhereClause = strSqlWhereClause;
            RowsetTree     = rowsetTree;
            DataAccess     = dataAccess;
            Status         = SearchMethodExecutionStatusE.SMES_Continue;
        }
    }; // Class BeforeExecuteEventArgs

    /// <summary>Delegate for the BeforeExecute event</summary>
    public delegate void BeforeExecuteEventHandler(object sender, BeforeExecuteEventArgs e);

    /// <summary>
    /// The ISearchMethod interface defines the contract a selection macro must implement
    /// </summary>
    public interface ISearchMethod {
        /// <summary> Event triggered when the search method is about to be executed</summary>
        event BeforeExecuteEventHandler         BeforeExecute;
        /// <summary> Event triggered when the validation of the search method's parameters is done</summary>
        event CancelEventHandler                AfterValidate;
        /// <summary>Name of the search method</summary>
        string                                  Name { get; }
        /// <summary>Acceleration Key of the search metyhod</summary>
        int                                     AccelerationKey { get; }
        /// <summary>Sql Command of the search method</summary>
        string                                  SqlCommand { get; }
        /// <summary>Description of the search method</summary>
        string                                  Description { get; }
        /// <summary>Indicates if the search method has arguments</summary>
        bool                                    HasArguments { get; }

        //********************************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeUsingSearchMethod:    Read a rowset tree using the search method
        /// </summary>
        /// <param name="rowsetTree">           RowsetTree</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //********************************************************************************
        bool    ReadRowsetTreeUsingSearchMethod(RepPP.RowsetTree rowsetTree);

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeUsingSearchMethod     Read a rowset tree using the search method
        /// </summary>
        /// <param name="rowsetTree">           Rowset tree</param>
        /// <param name="dataAccess">           Data access object</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //******************************************************************
        bool    ReadRowsetTreeUsingSearchMethod(RepPP.RowsetTree rowsetTree, ISBDataAccess dataAccess);
    }
}