using System;
using System.Drawing.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports group property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class RowsetControlUtil : ControlUtil {
	    /// <summary>Common interface for controls attached to a rowset</summary>
	    public interface IRowsetControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Rowset name</summary>
	        string                      RowsetName { get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>Rowset definition attached to the control</summary>
	    protected RowsetDef             m_rowsetDef;
	    /// <summary>Rowset attached to the control</summary>
	    protected Rowset                m_rowset;
	    /// <summary>Full name of the field</summary>
	    protected string                m_strRowsetName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="ctrlHelper">       Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        /// <param name="bNoRefCount">      true to disable reference counting for this instance</param>
        //
        //********************************************************************************
        public RowsetControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly, bool bNoRefCount) : base(parent, ctrlHelper, bResDispenserOnly, bNoRefCount) {
		    m_rowsetDef     = null;
		    m_rowset        = null;
		    m_strRowsetName = "";
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// /// <param name="ctrlHelper">   Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public RowsetControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly) : this(parent, ctrlHelper, bResDispenserOnly, false) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// RowsetName:                 Rowset name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public string RowsetName {
		    get {
		        return(m_strRowsetName);
		    }
		    set {
		        m_strRowsetName = (value == null) ? "" : value;
		        m_rowsetDef     = null;
		        m_rowset        = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// RowsetDef:                  Rep++ RowsetDef attached to the control
		/// </summary>
        //
        //********************************************************************************
		public RowsetDef RowsetDef {
		    get {
                RowsetDef           rowsetDefRetVal = null;
		        RepPP.Application   app;
		        
		        if (m_rowsetDef != null && m_rowsetDef.IsDetached) {
		            m_rowsetDef = null;
		            m_rowset    = null;
		        }
		        if (m_rowsetDef != null) {
                    rowsetDefRetVal = m_rowsetDef;
                } else {
		            app                 = Application;
	                rowsetDefRetVal     = (app == null || m_strRowsetName == String.Empty) ? null : app.RowsetDefs[m_strRowsetName];
                    if (!IsDesignMode) {
                        m_rowsetDef = rowsetDefRetVal;
                    }
		        }
		        return(rowsetDefRetVal);
		    }
		    set {
		        if (value != m_rowsetDef) {
		            m_rowsetDef   = null;
		            m_rowset = null;
		            if (value == null) {
		                m_strRowsetName = "";
		            } else {
		                m_strRowsetName = value.Name;
		                m_rowsetDef     = value;
		            }
		        }
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Rowset:              Rep++ Rowset attached to the control
		/// </summary>
        //
        //********************************************************************************
		public Rowset Rowset {
		    get {
		        if (m_rowset != null && m_rowset.IsDetached) {
		            m_rowsetDef = null;
		            m_rowset    = null;
		        }
		        return(m_rowset);
		    }
		    set {
		        if (value != m_rowset) {
		            m_rowsetDef = null;
		            m_rowset    = null;
		            if (value == null) {
		                m_strRowsetName = "";
		            } else {
		                m_strRowsetName = value.RowsetDef.Name;
		                m_rowsetDef     = value.RowsetDef;
		                m_rowset        = value;
		            }
		        }
		    }
		}

    } // End Class FldControlUtil
} // End Namespace