using System;
using System.Drawing.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a
    /// control and supports a node of a rowset tree definition property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class RowsetTreeDefNodeControlUtil : ControlUtil {
        /// <summary>Common interface for controls attached to a node of a rowset tree definition</summary>
        public interface IRowsetTreeDefNodeControl : IRepControl {
            /// <summary>Rep++ application object</summary>
            RepPP.Application           Application { get; }
            /// <summary>Name of the node of a rowset tree definition</summary>
            RowsetTreeDefNodeFullName   RowsetTreeDefNodeFullName { get; set; }
            /// <summary>Trigger an event to indicate a component property has changed</summary>
            void                        TriggerComponentChange();
        }
        
        /// <summary>RowsetTreeDefNode attached to the control</summary>
        protected RowsetTreeDefNode             m_node;
        /// <summary>Rowset attached to the control</summary>
        protected Rowset                        m_rowset;
        /// <summary>Full name of the RowsetTreeDefNode</summary>
        protected RowsetTreeDefNodeFullName     m_nodeFullName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="ctrlHelper">       Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        /// <param name="bNoRefCount">      true to disable reference counting for this instance</param>
        //
        //********************************************************************************
        public RowsetTreeDefNodeControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly, bool bNoRefCount) : base(parent, ctrlHelper, bResDispenserOnly, bNoRefCount) {
            m_node          = null;
            m_rowset        = null;
            m_nodeFullName  = new RowsetTreeDefNodeFullName();
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="ctrlHelper">       Control Util Helper</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public RowsetTreeDefNodeControlUtil(object parent, ControlUtil.IControlUtilHelper ctrlHelper, bool bResDispenserOnly) : this(parent, ctrlHelper, bResDispenserOnly, false) {
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetTreeDefNodeFullName:      The full name of the RowsetTreeDefNode attached to the control
        /// </summary>
        //
        //********************************************************************************
        public RowsetTreeDefNodeFullName RowsetTreeDefNodeFullName {
            get {
                return(m_nodeFullName);
            }
            set {
                m_nodeFullName  = value;
                m_node          = null;
                m_rowset        = null;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetTreeDefNode:              Rep++ RowsetTreeDefNode attached to the control
        /// </summary>
        //
        //********************************************************************************
        public RowsetTreeDefNode RowsetTreeDefNode {
            get {
                RepPP.RowsetTreeDefNode nodeDefRetVal = null;
                RepPP.RowsetTreeDef     rowsetTreeDef;
                RepPP.Application       app;
                
                if (m_node != null && m_node.IsDetached) {
                    m_node   = null;
                    m_rowset = null;
                }
                if (m_node != null) {
                    nodeDefRetVal = m_node;
                } else {
                    app = Application;
                    if (app != null && m_nodeFullName.RowsetTreeName != string.Empty && m_nodeFullName.RowsetName != string.Empty) {
                        rowsetTreeDef = app.RowsetTreeDefs[m_nodeFullName.RowsetTreeName];
                        if (rowsetTreeDef != null) {
                            nodeDefRetVal = rowsetTreeDef.FindNode(m_nodeFullName.RowsetName);
                            if (!IsDesignMode) {
                                m_node = nodeDefRetVal;
                            }
                        }
                    }
                }
                return(nodeDefRetVal);
            }
            set {
                if (value != m_node) {
                    m_node   = null;
                    m_rowset  = null;
                    if (value == null) {
                        RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName();
                    } else {
                        RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName(value.RowsetTreeDef.Name, value.Name);
                        m_node             = value;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetDef:                  Rep++ RowsetDef attached to the control
        /// </summary>
        //
        //********************************************************************************
        public RowsetDef RowsetDef {
            get {
                RowsetDef           rowsetDefRetVal = null;
                Application         app;
                RowsetTreeDefNode  node;

                node = this.RowsetTreeDefNode;
                if (node != null) {
                    rowsetDefRetVal = node.RowsetDef;
                } else {
                    app = Application;
                    if (app != null && m_nodeFullName.RowsetName != string.Empty) {
                        rowsetDefRetVal = app.RowsetDefs[m_nodeFullName.RowsetName];
                    }
                }
                return(rowsetDefRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Rowset:              Rep++ Rowset attached to the control
        /// </summary>
        //
        //********************************************************************************
        public Rowset Rowset {
            get {
                if (m_rowset != null && m_rowset.IsDetached) {
                    m_node      = null;
                    m_rowset    = null;
                }
                return(m_rowset);
            }
            set {
                RowsetTreeDefNode node;

                if (value != m_rowset) {
                    m_node  = null;
                    m_rowset    = null;
                    if (value != null) {
                        node  = value.RowsetTreeDefNode;
                        if (node != null) {
                            RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName(node.RowsetTreeDef.Name, node.Name);
                        } else {
                            RowsetTreeDefNodeFullName = new RowsetTreeDefNodeFullName("", value.RowsetDef.Name);
                        }
                        m_node  = node;
                        m_rowset    = value;
                    }
                }
            }
        }
      
    } // End Class RowsetTreeDefNodeControlUtil
} // End Namespace