using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;

namespace RepPP.Toolkit {
	//******************************************************************
	//
	/// <summary>
	/// Implement a base class for filter of a selection buffer
	/// </summary>
	//
	//******************************************************************
	public abstract class SBFilterBase : ISBFilter {
	    /// <summary>Specifies if we must use a AND or a OR between condition in the WHERE clause</summary>
	    public enum AndOrE {
	        /// <summary>Use AND expression</summary>
	        AND =   1,
	        /// <summary>Use OR expression</summary>
	        OR  =   2
	    };
	    
	    /// <summary>Counter to create unique group name</summary>
	    private static int                      m_iGlobCounter = 0;
	    /// <summary>Value that represents an empty choice in a choice list</summary>
        private const string                    m_strNoSelection = "*";
	    /// <summary>Rep++ Application object</summary>
	    protected RepPP.Application             m_app;
	    /// <summary>Temp rowset containing the filter values</summary>
	    protected RepPP.Rowset                  m_rowsetParam;
	    /// <summary>Rowset definition use as a template</summary>
	    protected RepPP.RowsetDef               m_rowsetDefTemplate;
	    /// <summary>Rowset definition containing the filter values</summary>
	    protected RepPP.RowsetDef               m_rowsetDefParam;
	    /// <summary>Actual WHERE clause</summary>
	    protected string                        m_strWhereClause;
	    /// <summary>Human readable WHERE clause</summary>
	    protected string                        m_strFilterClause;
	    /// <summary>JOIN clause</summary>
	    protected string                        m_strJoinClause;
        /// <summary>wild any char for like clause</summary>
        protected char                          m_chrWildCharAny;
        /// <summary>wild one char for like clause</summary>
        protected char                          m_chrWildCharOne;
        /// <summary>String shown for BETWEEN clause</summary>
        protected string                        m_strBetween;
        /// <summary>Array of generated fields</summary>
        protected FieldDef[]                    m_arrFldDef;
        /// <summary>Array of generated BETWEEN fields</summary>
        protected FieldDef[]                    m_arrBetweenFldDef;
        /// <summary>Indicateds whether or not to show the user flag field on the filter</summary>
        protected bool                          m_bShowUserFlagField;
        /// <summary> Event triggered when the search method is about to be executed</summary>
        public event BeforeExecuteEventHandler  BeforeExecute;
        /// <summary> Event triggered when a fieldDef is being added</summary>
        private event CancelEventHandler        AddingFieldDef;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //
        //******************************************************************
        public SBFilterBase() : this (null) {
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                               Class constructor
	    /// </summary>
		/// <param name="rowsetDefParam">       Rowset definition containing the field use as filter</param>
		/// <param name="bShowUserFlagField">   True to add user flag fields to the filter group</param>
	    //
	    //******************************************************************
		public SBFilterBase(RepPP.RowsetDef rowsetDefParam, bool bShowUserFlagField): this (rowsetDefParam, bShowUserFlagField, null) {
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="rowsetDefParam">       Rowset definition containing the field use as filter</param>
        /// <param name="bShowUserFlagField">   True to add user flag fields to the filter group</param>
        /// <param name="handler">              Handler called when a field is added to the filter</param>
        //
        //******************************************************************
        public SBFilterBase(RepPP.RowsetDef rowsetDefParam, bool bShowUserFlagField, CancelEventHandler handler) {
            try {
                if (handler != null) {
                    AddingFieldDef += handler;
                }
                m_arrFldDef        = null;
                m_arrBetweenFldDef = null;
                if (rowsetDefParam != null) {
                    m_app             = rowsetDefParam.Application;
                    m_rowsetParam     = null;
                    TemplateRowsetDef = rowsetDefParam;
                } else {
                    m_app = RepPP.Application.GetApplication();
                }
                m_bShowUserFlagField = bShowUserFlagField;
                m_strWhereClause     = "";
                m_strFilterClause    = "";
                m_strJoinClause      = "";
                m_chrWildCharAny     = '%';
                m_chrWildCharOne     = '_';
                m_strBetween         = "between";
            }
            finally {
                if (handler != null) {
                    AddingFieldDef -= handler;
                }
            }
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                           Class constructor
	    /// </summary>
		/// <param name="rowsetDefParam">   Rowset definition containing the field use as filter</param>
	    //
	    //******************************************************************
		public SBFilterBase(RepPP.RowsetDef rowsetDefParam) : this (rowsetDefParam, true) {
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// Dispose:                    Dispose the class
	    /// </summary>
	    //
	    //******************************************************************
		public virtual void Dispose() {
		    TemplateRowsetDef = null;
		}

        //********************************************************************************
        //
		/// <summary>
		/// AddFieldDef:                Add a field to the group using another field as template
		/// </summary>
		/// <param name="rowsetDef">    Rowset definition</param>
		/// <param name="fldDef">       Source field to use as template</param>
		/// <param name="strFldName">   Name of the field</param>
		/// <param name="bIsVisible">   Is the field visible?</param>
		/// <param name="iLogPos">      Position of the field in the group</param>
        //
        //********************************************************************************
        protected virtual FieldDef AddFieldDef(RepPP.RowsetDef  rowsetDef,
                                               RepPP.FieldDef   fldDef, 
                                               string           strFldName, 
                                               bool             bIsVisible,
                                               int              iLogPos) {
		    RepPP.FieldOptions  eFldOptions;
		    RepPP.FieldDef      fldDefNew;
            RepPP.FieldDef      fldDefSrc;

            fldDefSrc = GetAlternateFilterField(fldDef);
            if (fldDefSrc == null) {
                 fldDefSrc = fldDef;
            }
		    eFldOptions        = (fldDefSrc.IsPassword) ? RepPP.FieldOptions.sdFieldNoEcho : (RepPP.FieldOptions)0;
		    fldDefNew          = rowsetDef.FieldDefs.Add(strFldName,
		                                                 iLogPos,
		                                                 fldDefSrc.TableName,
		                                                 fldDefSrc.ColName,
		                                                 (fldDefSrc.Type != FieldType.sdFieldFormatString) ? fldDefSrc.Type : FieldType.sdFieldString,
		                                                 eFldOptions,
		                                                 fldDefSrc.MinSize,
                                                         fldDefSrc.MaxSize,
		                                                 fldDefSrc.Precision);
            fldDefNew.ControlType = (fldDefSrc.ControlType != CtlType.sdCtlFmt) ? fldDefSrc.ControlType : CtlType.sdCtlEdit;
            // Exception in case of a dynamic choice list
		    if (fldDefSrc.IsChoiceAvailable && fldDefSrc.ChoiceList.List != null) {
		        fldDefNew.ChoiceList.List = fldDefSrc.ChoiceList.List;
                fldDefNew.ChoiceList.AddChoice(m_strNoSelection, 
                                               m_strNoSelection, 
                                               m_strNoSelection, 
                                               0);
		    }
		    fldDefNew.Prompt       = Util.GetDirectSearchFieldPrompt(fldDefSrc);
		    fldDefNew.OutputMask   = fldDefSrc.OutputMask;
		    fldDefNew.IsVisible    = bIsVisible;
            fldDefNew.IsUserFlagOn = fldDefSrc.IsUserFlagOn;
            foreach (RepPP.Attribute att in fldDefSrc.Attributes) {
                fldDefNew.Attributes.Add(att.Name, att.Value);    
            }
		    return(fldDefNew);
        }

        //********************************************************************************
        //
		/// <summary>
		/// GetAlternateFilterField:    Gets the field that will be used in the filter
        ///                             instead of the specified field
		/// </summary>
		/// <param name="fldDef">       Source field</param>
        /// <returns>
        /// The alternate field or null
        /// </returns>
        //
        //********************************************************************************
        public static RepPP.FieldDef GetAlternateFilterField(RepPP.FieldDef fldDef) {
            RepPP.FieldDef  fldDefRetVal        = null;
            RepPP.Attribute attAlternateFldName;
            string          strAlternateFldName;
            string[]        arrFldFullName;

            attAlternateFldName = fldDef.Attributes["ALT_SEARCH_FLD"];
            if (attAlternateFldName != null) {
                strAlternateFldName = attAlternateFldName.Value;
                if (!string.IsNullOrEmpty(strAlternateFldName)) {
                    if (strAlternateFldName.Contains(".")) {
                        arrFldFullName = strAlternateFldName.Split('.');
                        if (arrFldFullName.Length == 2) {
                            fldDefRetVal = fldDef.Application.FieldDef(arrFldFullName[0], arrFldFullName[1]);
                        }
                    } else if (fldDef.RowsetDef.FieldDefs[strAlternateFldName] != null) {
                        fldDefRetVal = fldDef.RowsetDef.FieldDefs[strAlternateFldName];
                    }
                }
            }
            return(fldDefRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAddingFieldDef                 Triggers the AddingFieldDef event
        /// </summary>
        /// <param name="fldDef">            Field being added</param>
        /// <param name="e">                 Event argument</param>
        //
        //******************************************************************
        protected virtual void OnAddingFieldDef(FieldDef fldDef, CancelEventArgs e) {
            if (AddingFieldDef != null) {
                AddingFieldDef(fldDef, e);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// BuildNewRowsetDef:              Build a new rowset definition which represent the record to
		///                                 insert in the historic table
		/// </summary>
		/// <param name="strNewRowsetName"> Name of the new group</param>
		/// <param name="rowsetDefSrc">     Base rowset definition</param>
		/// <returns>
		/// New rowset definition
		/// </returns>
        //
        //********************************************************************************
		protected virtual RepPP.RowsetDef BuildNewRowsetDef(string          strNewRowsetName,
		                                                    RepPP.RowsetDef rowsetDefSrc) {
		    RepPP.RowsetDef         rowsetDefRetVal;
		    int                     iLogPos = 1;
		    int                     iCount = 0;
		    ArrayList               arrFldDef;
		    ArrayList               arrBetweenFldDef;
            CancelEventArgs         e;
		    
		    arrFldDef          = new ArrayList(rowsetDefSrc.FieldDefs.Count);
		    arrBetweenFldDef   = new ArrayList(rowsetDefSrc.FieldDefs.Count);
		    rowsetDefRetVal    = m_app.RowsetDefs.Add(strNewRowsetName, rowsetDefSrc.TableName, true);
		    foreach (RepPP.FieldDef fldDef in rowsetDefSrc.FieldDefs) {
		        if (fldDef.IsDatabaseLinked                    && 
                    fldDef.IsVisible                           && 
                    fldDef.Type != RepPP.FieldType.sdFieldMemo &&
                    (!fldDef.IsUserFlagOn || ShowUserFlagField)) {
                    e = new CancelEventArgs();
                    OnAddingFieldDef(fldDef, e);
                    if (!e.Cancel) {
                        arrFldDef.Add(AddFieldDef(rowsetDefRetVal, fldDef, fldDef.Name, true, iLogPos++));
                        arrBetweenFldDef.Add(AddFieldDef(rowsetDefRetVal, fldDef, "_" + (iCount++).ToString(), false, iLogPos++));
                    }
		        }
		    }
		    m_arrFldDef        = new FieldDef[arrFldDef.Count];
		    m_arrBetweenFldDef = new FieldDef[arrBetweenFldDef.Count];
		    arrFldDef.CopyTo(m_arrFldDef);
		    arrBetweenFldDef.CopyTo(m_arrBetweenFldDef);
		    return(rowsetDefRetVal);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// TemplateRowsetDef:          RowsetDef used as a template
		/// </summary>
		//
	    //******************************************************************
		public RepPP.RowsetDef TemplateRowsetDef {
		    get {
		        return(m_rowsetDefTemplate);
		    }
		    set {
		        if (m_rowsetDefTemplate != value) {
		            if (m_rowsetParam != null) {
		                m_rowsetParam.Delete();
		                m_rowsetParam = null;
		            }
		            if (m_rowsetDefParam != null) {
		                m_rowsetDefParam.Dispose();
		                m_rowsetDefParam = null;
		            }
		            if (value != null) {
                        if (m_app == null) {
                            m_app = value.Application;
                        }
		                m_rowsetDefParam = BuildNewRowsetDef("$HIDE_FILT" + (m_iGlobCounter++).ToString(), value);
		                m_rowsetParam    = m_rowsetDefParam.Rowsets.Create(RepPP.RowsetType.sdVertical);
		            } else {
		                m_rowsetDefParam  = null;
		                m_rowsetParam     = null;
		            }
		            m_rowsetDefTemplate = value;
		        }
		    }
		}

	    //******************************************************************
	    //
		/// <summary>
		/// Between:            String used to express the BETWEEN
		/// </summary>
		//
	    //******************************************************************
		public string Between {
		    get {
		        return(m_strBetween);
		    }
		    set {
                m_strBetween = value;
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// WildCharAny:           Wild any char for LIKE clause
		/// </summary>
		//
	    //******************************************************************
		public char WildCharAny {
		    get {
		        return(m_chrWildCharAny);
		    }
		    set {
                m_chrWildCharAny = value;
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// WildCharOne:           Wild one char for LIKE clause
		/// </summary>
		//
	    //******************************************************************
		public char WildCharOne {
		    get {
		        return(m_chrWildCharOne);
		    }
		    set {
                m_chrWildCharOne = value;
            }
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// ParamRowset:         Rowset containing the fields use as filter
	    /// </summary>
	    //
	    //******************************************************************
		public RepPP.Rowset ParamRowset {
		    get {
		        return(m_rowsetParam);
		    }
		}

	    //******************************************************************
	    //
	    /// <summary>
	    /// ShowUserFlagField:      Indicateds whether or not to show the user flag field
	    ///                         on the filter    
	    /// </summary>
	    //
	    //******************************************************************
        public bool ShowUserFlagField {
            get {
                return(m_bShowUserFlagField);
            }
            set {
                m_bShowUserFlagField = value;
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// BuildValue:                         Build the value of a field
		/// </summary>
		/// <param name="fldDef">                  FieldDef</param>
		/// <returns>
		/// Value to be used in the WHERE clause
		/// </returns>
	    //
	    //******************************************************************
		protected virtual string BuildValue(RepPP.FieldDef fldDef) {
		    string  strRetVal;
		    
		    switch(fldDef.Type) {
		    case RepPP.FieldType.sdFieldDate:
		        strRetVal = "SD_CHAR_TO_DATE('" + fldDef.Value + "')";
		        break;
		    case RepPP.FieldType.sdFieldDateTime:
		        strRetVal = "SD_CHAR_TO_TIMESTAMP('" + fldDef.Value + "')";
		        break;
		    case RepPP.FieldType.sdFieldTime:
		        strRetVal = "SD_CHAR_TO_TIME('" + fldDef.Value + "')";
		        break;
		    case RepPP.FieldType.sdFieldFormatString:
		    case RepPP.FieldType.sdFieldHexa:
		    case RepPP.FieldType.sdFieldMemo:
		    case RepPP.FieldType.sdFieldMultiAlphaChoice:
            case RepPP.FieldType.sdFieldSimpleAlphaChoice:
            case RepPP.FieldType.sdFieldString:
            case RepPP.FieldType.sdFieldUpperString:
                strRetVal = "'" + fldDef.Value.Replace("'", "''") + "'";
                break;
		    case RepPP.FieldType.sdFieldMoney:
		    case RepPP.FieldType.sdFieldMultiNumChoice:
		    case RepPP.FieldType.sdFieldNumeric:
		    case RepPP.FieldType.sdFieldNumericDec:
		    case RepPP.FieldType.sdFieldPercentage:
		    case RepPP.FieldType.sdFieldPercentageDec:
		    case RepPP.FieldType.sdFieldSimpleNumChoice:
                strRetVal = fldDef.Value;
                break;
            default:
                strRetVal = null;
                break;
		    }
		    return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildFieldWhereClause:          Build the WHERE clause for a field
        /// </summary>
        /// <param name="fldDef">              FieldDef</param>
        /// <param name="strValue">         Value specified for the field</param>
        /// <param name="strOperator">      The operator</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        protected virtual string BuildFieldWhereClause(RepPP.FieldDef fldDef, string strValue, string strOperator) {
            return(fldDef.RowsetDef.TableName + "." + fldDef.ColName + strOperator + strValue);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildFieldFilterClause:         Build the filter clause for a field
        /// </summary>
        /// <param name="fldDef">           Field definition</param>
        /// <param name="strValue">         Value specified for the field</param>
        /// <param name="strOperator">      The operator</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        protected virtual string BuildFieldFilterClause(RepPP.FieldDef fldDef, string strValue, string strOperator) {
            return(fldDef.RowsetDef.TableName + "." + fldDef.Name + strOperator + "'" + strValue + "'");
        }

	    //******************************************************************
	    //
		/// <summary>
		/// BuildWhereClause:                   Build a WHERE clause using the values in the rowset
		/// </summary>
		/// <param name="eType">                AND or OR</param>
		/// <param name="arrOp">                Array of operators</param>
		/// <param name="strFilterClause">      Human readable WHERE clause</param>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
		protected virtual string BuildWhereClause(AndOrE eType, string[] arrOp, out string strFilterClause) {
		    string  strWhereClause;
		    string  strVal;
		    string  strVal2;
		    string  strOP;
		    int     iPos = 0;
            bool    bNull;
		    
		    strWhereClause  = String.Empty;
		    strFilterClause = String.Empty;
		    foreach (FieldDef fldDef in m_arrFldDef) {
                bNull = (arrOp[iPos] == "is null" || arrOp[iPos] == "is not null");
		        if (bNull || (!fldDef.IsNull && (!fldDef.IsChoiceAvailable || fldDef.Value != m_strNoSelection))) {
		            strVal = (bNull ? "" : BuildValue(fldDef));
		            if (strVal != null) {
		                if (strWhereClause != String.Empty) {
		                    strWhereClause  += (eType == AndOrE.AND) ? " AND " : " OR ";
		                    strFilterClause += (eType == AndOrE.AND) ? " AND " : " OR ";
		                }
		                strOP = arrOp[iPos];
		                if (strOP == m_strBetween) {
		                    strVal2          = BuildValue(m_arrBetweenFldDef[iPos]);
                            strWhereClause  += BuildFieldWhereClause(fldDef, strVal + " AND " + strVal2, " BETWEEN ");
                            strFilterClause += BuildFieldFilterClause(fldDef, fldDef.ExtValue + "' AND '" + m_arrBetweenFldDef[iPos].ExtValue + "'", " BETWEEN ");
		                } else if (strOP == "is null" || strOP == "is not null") {
                            strWhereClause  += BuildFieldWhereClause(fldDef, "", " " + strOP.ToUpper());
                            strFilterClause += BuildFieldFilterClause(fldDef, "", " " + strOP.ToUpper());
		                } else {
		                    if (strOP == "=" || strOP == "<>") {
		                        switch(fldDef.Type) {
		                        case RepPP.FieldType.sdFieldString:
		                        case RepPP.FieldType.sdFieldUpperString:
		                            if (strVal.IndexOf(m_chrWildCharAny) != -1 ||
		                                strVal.IndexOf(m_chrWildCharAny) != -1) {
		                                strOP  = (strOP == "=") ? " LIKE " : " NOT LIKE ";
                                        if (m_chrWildCharAny != '%') {
                                            strVal = strVal.Replace(m_chrWildCharAny.ToString(), "%");
                                        }
                                        if (m_chrWildCharOne != '_') {
                                            strVal = strVal.Replace(m_chrWildCharOne.ToString(), "_");
                                        }
		                            }
		                            break;
		                        default:
		                            break;
		                        }
		                    }
                            strWhereClause   += BuildFieldWhereClause(fldDef, strVal, strOP);
                            strFilterClause  += BuildFieldFilterClause(fldDef, fldDef.ExtValue, strOP);
  		                }
  		            }
  		        }
  		        iPos++;
		    }
		    if (strWhereClause != String.Empty) {
		        strWhereClause = "(" + strWhereClause + ")";
		    }
		    return(strWhereClause);
		}

	    //******************************************************************
	    //
		/// <summary>
		/// BuildWhereClause:                   Build a WHERE clause using the values in the group
		/// </summary>
		/// <param name="eType">                AND or OR</param>
		/// <param name="strFilterClause">      Human readable WHERE clause</param>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
	    protected abstract string BuildWhereClause(AndOrE eType, out string strFilterClause);

	    //******************************************************************
	    //
		/// <summary>
		/// WhereClause:                    Actual WHERE clause
		/// </summary>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
		public virtual string WhereClause {
		    get {
		        return(m_strWhereClause);
		    }
		    set {
		        m_strWhereClause = value;
		    }
		}

	    //******************************************************************
	    //
		/// <summary>
		/// FilterClause:                    Human readable WHERE clause
		/// </summary>
		/// <returns>
		/// Filter clause
		/// </returns>
	    //
	    //******************************************************************
		public virtual string FilterClause {
		    get {
		        return(m_strFilterClause);
		    }
		    set {
		        m_strFilterClause = value;
		    }
		}

	    //******************************************************************
	    //
		/// <summary>
		/// JoinClause:                    Join clause
		/// </summary>
		/// <returns>
		/// JOIN clause
		/// </returns>
	    //
	    //******************************************************************
		public virtual string JoinClause {
		    get {
		        return(m_strJoinClause);
		    }
		    set {
		        if (value != null) {
                    m_strJoinClause = value;
                } else {
                    m_strJoinClause = string.Empty;
                }
		    }
		}

        //******************************************************************
        //
        /// <summary>
        /// OnBeforeExecute                  Triggers the BeforeExecute event
        /// </summary>
        /// <param name="e">                 Event argument</param>
        //
        //******************************************************************
        protected virtual void OnBeforeExecute(BeforeExecuteEventArgs e) {
            if (BeforeExecute != null) {
                BeforeExecute(this, e);
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// SetFilter:                      Set the filter
		/// </summary>
		/// <param name="eType">            Use AND or OR</param>
		//
	    //******************************************************************
		public virtual void SetFilter(AndOrE eType) {
		    m_strWhereClause = BuildWhereClause(eType, out m_strFilterClause);
		}

	    //******************************************************************
	    //
		/// <summary>
		/// AddFilter:                      Add a filter
		/// </summary>
		/// <param name="eType">            Use AND or OR</param>
		//
	    //******************************************************************
		public virtual void AddFilter(AndOrE eType) {
		    string  strWhereClause;
		    
		    strWhereClause = BuildWhereClause(eType, out m_strFilterClause);
		    if (m_strWhereClause == null || m_strWhereClause == String.Empty) {
		        m_strWhereClause = strWhereClause;
		    } else {
		        m_strWhereClause += " AND " + strWhereClause;
		    }
		}

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeUsingFilterV      Read the rowset tree using the filter
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree</param>
        /// <param name="strAddWhere">      Additionnal where clause used to filter the rowset tree</param>
        //
        //******************************************************************
        public virtual void ReadRowsetTreeUsingFilterV(RepPP.RowsetTree rowsetTree, string strAddWhere) {
            ReadRowsetTreeUsingFilterV(rowsetTree, strAddWhere, null);
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadFilter:                     Read the filter
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree</param>
        /// <param name="strWhereClause">   Where clause used to filter the rowset tree</param>
        /// <param name="dataAccess">       Data access object</param>
        //
        //******************************************************************
        protected virtual void ReadFilter(RepPP.RowsetTree rowsetTree, String strWhereClause, ISBDataAccess dataAccess) {
            RowsetTreeDefNode       node;

            node = rowsetTree.RowsetTreeDef.Nodes[0];
            node.SetSelectSqlWhere(strWhereClause);
            if (dataAccess != null) {
                if (dataAccess is ISBDataAccessWithDirectSearch) {
                    ((ISBDataAccessWithDirectSearch)dataAccess).LoadSelectionBufferData(rowsetTree, 
                                                                                        strWhereClause, 
                                                                                        JoinClause);
                } else  {
                    dataAccess.LoadSelectionBufferData(rowsetTree, strWhereClause);
                }
            } else {
                rowsetTree.ReadFromDb();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeDefUsingFilterV   Read the card using the filter
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree</param>
        /// <param name="strAddWhere">      Additionnal where clause used to filter the rowset tree</param>
        /// <param name="dataAccess">       Data access object</param>
        //
        //******************************************************************
        public virtual void ReadRowsetTreeUsingFilterV(RepPP.RowsetTree rowsetTree, 
                                                       string           strAddWhere,
                                                       ISBDataAccess    dataAccess) {
            RowsetTreeDefNode       node;
            BeforeExecuteEventArgs  e;
            string                  strWhereClause       = String.Empty;
            string                  strOriginalSelectCmd;

            if (strAddWhere != null && strAddWhere != String.Empty) {
                strWhereClause = strAddWhere;
            }
            if (WhereClause != String.Empty) {
                strWhereClause += (strWhereClause == String.Empty) ? "WHERE " + WhereClause :
                                                                    " AND (" + WhereClause + ")";
            }
            node                 = rowsetTree.RowsetTreeDef.Nodes[0];
            strOriginalSelectCmd = node.SelectSqlCommand;
            try {
                if (JoinClause != string.Empty) {
                    node.SelectSqlCommand = strOriginalSelectCmd.Insert(7, "DISTINCT ");
                    strWhereClause        = JoinClause + " " + strWhereClause;
                }
                e              = new BeforeExecuteEventArgs(strOriginalSelectCmd, strWhereClause, rowsetTree, dataAccess);
                OnBeforeExecute(e);
                if (e.Status == SearchMethodExecutionStatusE.SMES_Continue) {
                    ReadFilter(rowsetTree, strWhereClause, dataAccess);
                }
            }
            finally {
                node.SelectSqlCommand = strOriginalSelectCmd;
            }
        }

	    //******************************************************************
	    //
	    /// <summary>
	    /// IsValid:            Checks if the value are valids
	    /// </summary>
	    //
	    //******************************************************************
		public bool IsValid() {
		    bool    bRetVal = true;
		    		    
		    foreach (RepPP.FieldDef fldDef in m_rowsetDefParam.FieldDefs) {
		        if (fldDef.Validate() != 0) {
		            bRetVal = false;
		        }
		    }
		    return(bRetVal);
        }

	    //******************************************************************
	    //
        /// <summary>
        /// BuildDSInnerClause:         Builds the inner clause for a direct
        ///                             search
        /// </summary>
        /// <param name="node">         Current direct search Card</param>
        //
	    //******************************************************************
        public virtual string BuildDSInnerClause(RepPP.RowsetTreeDefNode node) {
            string                  strRetVal = "";
            string                  strInnerJoin;
            string                  strPKJoin;
            RepPP.RowsetTreeDefNode nodeAncester;
            RepPP.Connection        conn;
            ArrayList               arrAncester;

            conn            = (node.Connection == null) ? node.Application.DataConnection : node.Connection;
            arrAncester     = new ArrayList(16);
            nodeAncester    = node.Parent as RowsetTreeDefNode;
            while (nodeAncester != null) {
                if (nodeAncester.Parent == null || nodeAncester.IsPrimaryKeyBroken) {
                    arrAncester.Add(nodeAncester);
                }
                nodeAncester = nodeAncester.Parent as RowsetTreeDefNode;
            }
            foreach (RowsetTreeDefNode nodeTmp in arrAncester) {
                strInnerJoin = " INNER JOIN " + Util.DecorateWord(conn, node.RowsetDef.TableName) + " ON ";
                strPKJoin    = "";
                foreach (RepPP.FieldDef fldDef in nodeTmp.RowsetDef.FieldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        if (strPKJoin != String.Empty) {
                            strPKJoin += " AND ";
                        }
                        strPKJoin += Util.DecorateWord(conn, nodeTmp.RowsetDef.TableName) + "." + Util.DecorateWord(conn, fldDef.ColPhysName) + "=" +
                                     Util.DecorateWord(conn, node.RowsetDef.TableName) + "." + Util.DecorateWord(conn, node.RowsetDef.GetColPhysName(fldDef.ColName));
                    }
                }
                strInnerJoin += strPKJoin;
                strRetVal     = strInnerJoin + strRetVal;
                node          = nodeTmp;
            }
            return(strRetVal);
        }
    }
}
