using System;
using System.Collections;
using System.ComponentModel;

namespace RepPP.Toolkit {
	//******************************************************************
	//
	/// <summary>
	/// Implement a default search method
	/// </summary>
	//
	//******************************************************************
	public class SearchMacro : ISearchMethod {
	    /// <summary>Macro object</summary>
        private Macro                                  m_macro;
        /// <summary> Event triggered when the search method is about to be executed</summary>
        public event BeforeExecuteEventHandler         BeforeExecute;
        /// <summary> Event triggered when the validation of the search method's parameters is done</summary>
        public event CancelEventHandler                AfterValidate;

	    //******************************************************************
	    //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
		/// <param name="macro">        Rep++ Macro</param>
	    //
	    //******************************************************************
        public SearchMacro(Macro macro) {
            m_macro = macro;
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Name:                       Gets the Name of the search method.
		/// </summary>
		//
	    //******************************************************************
        public string Name {
            get {
               return(m_macro.Name); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// AccelerationKey:            Gets the Acceleration key of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public int AccelerationKey {
            get {
               return(m_macro.AcceleratorKey); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// SqlCommand:                 Gets the Sql Command of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public string SqlCommand {
            get {
               return(m_macro.SqlCommand); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Description:                Gets the Description of 
        ///                             the search method.
		/// </summary>
		//
	    //******************************************************************
        public string Description {
            get {
               return(m_macro.Description); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// HasArguments:               Checks if the search method has arguments
		/// </summary>
		//
	    //******************************************************************
        public bool HasArguments {
            get {
               return(m_macro.Arguments.Count > 0); 
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// InnerObject:                Gets/Sets the Inner Object of the search method.
		/// </summary>
		//
	    //******************************************************************
        public Macro Macro {
            get {
                return(m_macro);
            }
            set {
                m_macro = (Macro)value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnBeforeExecute                  Triggers the BeforeExecute event
        /// </summary>
        /// <param name="e">                 Event argument</param>
        //
        //******************************************************************
        protected virtual void OnBeforeExecute(BeforeExecuteEventArgs e) {
            if (BeforeExecute != null) {
                BeforeExecute(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeDefUsingSearchMethod   Read the card using a search method
        /// </summary>
        /// <param name="rowsetTree">   Rowset tree</param>
        /// <param name="dataAccess">   Data access object</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //******************************************************************
        public bool ReadRowsetTreeUsingSearchMethod(RowsetTree rowsetTree, ISBDataAccess dataAccess) {
            bool                    bRetVal  = true;
            RowsetTreeDefNode       node;
            string                  strSqlWhereClause;
            string                  strOriginalSelectCmd;
            BeforeExecuteEventArgs  e;

            node                 = rowsetTree.RowsetTreeDef.Nodes[0];
            strOriginalSelectCmd = node.SelectSqlCommand;
            try {
                strSqlWhereClause = BuildMacroWhereClause();
                e                 = new BeforeExecuteEventArgs(strOriginalSelectCmd, strSqlWhereClause, rowsetTree, dataAccess);
                OnBeforeExecute(e);
                if (e.Status == SearchMethodExecutionStatusE.SMES_Cancel) {
                    bRetVal = false;
                } else {
                    if (e.Status == SearchMethodExecutionStatusE.SMES_Continue) {
                        strSqlWhereClause = e.SqlWhereClause;
                        node.SetSelectSqlWhere(strSqlWhereClause);
                        if (dataAccess != null) {
                            dataAccess.LoadSelectionBufferData(rowsetTree, string.Empty);
                        } else {
                            rowsetTree.ReadFromDb();
                        }
                    }
                }
            }
            finally {
                node.SelectSqlCommand = strOriginalSelectCmd;
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadRowsetTreeDefUsingSearchMethod  Read the card using a search method
        /// </summary>
        /// <param name="rowsetTree">           Rowset tree</param>
        /// <returns>
        /// False if the execution was cancelled, True otherwise
        /// </returns>
        //
        //******************************************************************
        public bool ReadRowsetTreeUsingSearchMethod(RowsetTree rowsetTree) {
            return ReadRowsetTreeUsingSearchMethod(rowsetTree, null);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAfterValidate                  Triggers the AfterValidate event
        /// </summary>
        /// <param name="e">                 Event argument</param>
        //
        //******************************************************************
        protected virtual void OnAfterValidate(CancelEventArgs e) {
            if (AfterValidate != null) {
                AfterValidate(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ValidateArguments               Validate the argument values
        /// </summary>
        /// <returns>
        /// True if the argument values are valid, False otherwise
        /// </returns>
        //
        //******************************************************************
        public bool ValidateArguments() {
            bool            bRetVal;
            CancelEventArgs e        = new CancelEventArgs(false);
            
            OnAfterValidate(e);
            bRetVal = !e.Cancel;
            return (bRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
		/// BuildMacroWhereClause:              Builds the SQL WHERE clause of the selection macro
		/// </summary>
        /// <returns>
        /// The WHERE clause of the selection macro
        /// </returns>
        //
        //********************************************************************************
        protected virtual string BuildMacroWhereClause() {
            MacroArgs   arguments;
            string      strRetVal;
            string      strRowsetName;
            string      strOldParam;
            string      strNewParam;

            strRetVal = m_macro.SqlCommand;
            arguments = m_macro.Arguments;
            if (arguments.Count > 0) {
                strRowsetName = arguments[0].FieldDef.RowsetDef.Name;
                foreach(MacroArg arg in arguments){
                    strOldParam = ":" + arg.Name.Trim();
                    strNewParam = ":" + strRowsetName + "." + arg.FieldDef.Name;
                    strRetVal   = strRetVal.Replace(strOldParam, strNewParam);    
                }
            }
            return(strRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Load:                           Loads all macros associated with a rowset tree definition
        /// </summary>
        /// <param name="rowsetTreeDef">    Rowset tree definition</param>
        /// <returns>
        /// List of macros associated with the specified rowset tree.
        /// </returns>
        //  
        //*********************************************************     
        public static SearchMacro[] Load(RepPP.RowsetTreeDef rowsetTreeDef) {
            Application app;
            ArrayList   arrRetVal = new ArrayList();

            app          = rowsetTreeDef.Application;
            foreach(Macro macro in app.Macros) {
                if (macro.Type == MacroType.sdSelection && macro.RowsetTreeDef != null && macro.RowsetTreeDef.Name == rowsetTreeDef.Name) {
                    arrRetVal.Add(new SearchMacro(macro));
                }

            }
            return((SearchMacro[])(arrRetVal.ToArray(typeof(SearchMacro))));
        }
    } //Class
} // Namespace
