using System;
using System.Windows.Forms;

namespace RepPP.Toolkit {
	/// <summary>
	/// Implement some string utility functions
	/// </summary>
	public class StringUtility {
        /// <summary>Options for the search</summary>
        public enum SearchTypeE {
            /// <summary>
            /// No option
            /// </summary>
            SK_NONE                 = 0,
            /// <summary>
            /// Ignore Upper/Lower case
            /// </summary>
            SK_IGNORE_CASE          = 1,
            /// <summary>
            /// Don't look for keyword between ""
            /// </summary>
            SK_NOT_IN_DOUBLE_QUOTE  = 2,
            /// <summary>
            /// Don't look for keyword between ""
            /// </summary>
            SK_NOT_IN_SIMPLE_QUOTE  = 4,
            /// <summary>
            /// Don't look for keyword between ()
            /// </summary>
            SK_NOT_IN_PARENTHESES   = 8,
            /// <summary>
            /// Keyword must be preceded by a blank
            /// </summary>
            SK_BLANK_BEFORE         = 16,
            /// <summary>
            /// Keyword must be followed by a blank
            /// </summary>
            SK_BLANK_AFTER          = 32,
            /// <summary>
            /// Keyword must be followed by a non alpha, num or '_'
            /// </summary>
            SK_NON_ALNUM_AFTER      = 64,
            /// <summary>
            /// Don't look for keywork between <!-- -->
            /// </summary>
            SK_NOT_IN_COMMENT       = 128,
            /// <summary>
            /// An = must follow
            /// </summary>
            SK_EQUAL_AFTER          = 256,
            /// <summary>
            /// Keyword must be preceded by a non alpha, num or '_'
            /// </summary>
            SK_NON_ALNUM_BEFORE     = 512,
        };

        /// <summary>
        /// Class implementing a searched keyword
        /// </summary>
        public class SearchedKeyword {
            /// <summary>Keyword to search</summary>
            public  string          strKeyword;     // Keyword to find
            /// <summary>Type of search for the keyword (SK_???)</summary>
            public  SearchTypeE     eSearchType;    // Search type

            //*********************************************************	    
            //
            /// <summary>
            /// ctor:               Class constructor
            /// </summary>
            /// <param name="str">  Keyword</param>
            /// <param name="eType">Search options</param>
            //
            //*********************************************************	    
            public                  SearchedKeyword(string str, SearchTypeE eType) {strKeyword=str;eSearchType=eType;}
        };

        //*********************************************************	    
        //
        /// <summary>
        /// IsBlank:            Returns true if character is blank 
        /// </summary>
        /// <param name="cChr"> Character to check</param>
        /// <returns>
        /// false (no), true (yes)
        /// </returns>
        //
        //*********************************************************	    
        public static bool IsBlank(char cChr) {
            return(cChr == ' ' || cChr == (char)9 || cChr == '\n' || cChr =='\r');
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SkipBlank:              Skip the blank in a string
        /// </summary>
        /// <param name="str">      String</param>
        /// <param name="iStartPos">Starting position</param>
        /// <returns>
        /// New position
        /// </returns>
        //
        //*********************************************************	    
        public static int SkipBlank(string str, int iStartPos) {
            int     iStrSize;
            
            iStrSize = str.Length;
            while (iStartPos < iStrSize && IsBlank(str[iStartPos])) {
                iStartPos++;
            }
            return(iStartPos);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// StrFindKeyword:             Finds the specified keyword in the string
        /// </summary>
        /// <param name="str">          String to search</param>
        /// <param name="iStartPos">    Starting position</param>
        /// <param name="iKeywordPos">  Keyword position (returned)</param>
        /// <param name="strKeyword">   Keyword to find</param>
        /// <param name="eSearchType">  Type of search.
        ///                             SK_IGNORE_CASE              Ignore case.
        ///                             SK_NOT_IN_DOUBLE_QUOTE      Don't look for keyword between ""
        ///                             SK_NOT_IN_SIMPLE_QUOTE      Don't look for keyword between ''
        ///                             SK_NOT_IN_PARENTHESES       Don't look for keyword between ()
        ///                             SK_BLANK_BEFORE             Keyword must be preceded by a blank
        ///                             SK_BLANK_AFTER              Keyword must be followed by a blank
        ///                             SK_NON_ALNUM_AFTER          Keyword must be followed by a non letter or num
        /// </param>
        /// <returns>
        /// Location after the found keyword or -1 if not found
        /// </returns>
        /// <remarks>
        /// SK_NOT_IN_DOUBLE_QUOTE and SK_NOT_IN_SIMPLE_QUOTE have precedence
        /// on SK_NOT_IN_PARENTHESES. So, if on of the quote option is specified, a () in '' 
        /// or "" is not counted. A ' between "" or a " between '' is not considered.
        /// </remarks>
        //
        //*********************************************************	    
        public static int StrFindKeyword(string        str,
                                         int           iStartPos,
                                         out int       iKeywordPos,
                                         string        strKeyword,
                                         SearchTypeE   eSearchType) {                                    
            int     iRetVal        = -1;
            int     iPos;
            int     iStrSize;
            bool    bFirstChr        = true;
            bool    bDoubleQuote     = false;
            bool    bSimpleQuote     = false;
            bool    bComment         = false;
            int     iParentheseCount = 0;
            int     iKeywordSize;
            bool    bFound;

            iKeywordPos  = -1;
            iKeywordSize = strKeyword.Length;
            iStrSize     = str.Length;
            if ((eSearchType & SearchTypeE.SK_IGNORE_CASE) != 0) {
                strKeyword = strKeyword.ToUpper();
            }
            iPos = iStartPos;
            while (iRetVal == -1 && iPos <= iStrSize - iKeywordSize) {
                bFound = false;
                if (bDoubleQuote == false &&
                    bSimpleQuote == false &&
                    bComment     == false &&
                    iParentheseCount == 0) {
                    if ((eSearchType & SearchTypeE.SK_IGNORE_CASE) != 0) {
                        bFound = (str.Substring(iPos, iKeywordSize).ToUpper() == strKeyword);
                    } else {
                        bFound = (str.Substring(iPos, iKeywordSize) == strKeyword);
                    }
                    if (bFound) {
                        if ((eSearchType & SearchTypeE.SK_BLANK_BEFORE) != 0) {
                            if (bFirstChr) {
                                bFound = false;
                            } else {                                
                                if (IsBlank(str[iPos-1]) == false) {
                                    bFound = false;
                                }
                            }
                        }
                        if ((eSearchType & SearchTypeE.SK_BLANK_AFTER) != 0) {
                            if (iPos + iKeywordSize < iStrSize && IsBlank(str[iPos+iKeywordSize]) == false) {
                                bFound = false;
                            }
                        }
                        if ((eSearchType & SearchTypeE.SK_NON_ALNUM_AFTER) != 0) {
                            if (iPos + iKeywordSize < iStrSize && (Char.IsLetterOrDigit(str[iPos+iKeywordSize]) || str[iPos+iKeywordSize] == '_')) {
                                bFound = false;
                            }
                        }
                        if ((eSearchType & SearchTypeE.SK_NON_ALNUM_BEFORE) != 0) {
                            if (!bFirstChr) {
                                if (Char.IsLetterOrDigit(str[iPos-1]) || str[iPos-1] == '_') {
                                    bFound = false;
                                }
                            }
                        }
                    }
                }
                if (bFound) {
                    iKeywordPos = iPos;
                    iRetVal     = iPos + iKeywordSize;
                } else {
                    switch(str[iPos]) {                    
                    case '\'':
                        if ((eSearchType & SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE) != 0 && !bDoubleQuote) {
                            bSimpleQuote = !bSimpleQuote;
                        }
                        break;
                    case '"':
                        if ((eSearchType & SearchTypeE.SK_NOT_IN_DOUBLE_QUOTE) != 0 && !bSimpleQuote) {
                            bDoubleQuote = !bDoubleQuote;
                        }
                        break;
                    case '(':
                        if ((eSearchType & SearchTypeE.SK_NOT_IN_PARENTHESES) != 0 &&
                            !bDoubleQuote &&
                            !bSimpleQuote) {
                            iParentheseCount++;
                        }
                        break;
                    case ')':
                        if ((eSearchType & SearchTypeE.SK_NOT_IN_PARENTHESES) != 0 &&
                            !bDoubleQuote &&
                            !bSimpleQuote) {
                            if (iParentheseCount != 0) {
                                iParentheseCount--;
                            }
                        }
                        break;
                    default:
                        break;
                    }
                    if ((iPos + 4) <= str.Length) {
                        if (str.Substring(iPos,4) == "<!--") {
                            if ((eSearchType & SearchTypeE.SK_NOT_IN_COMMENT) != 0 &&
                                !bDoubleQuote &&
                                !bSimpleQuote) {
                                bComment = !bComment;
                            }                              
                        }
                    }
                    if ((iPos + 3) <= str.Length) {
                        if (str.Substring(iPos,3) == "-->") {
                            if ((eSearchType & SearchTypeE.SK_NOT_IN_COMMENT) != 0 &&
                                !bDoubleQuote &&
                                !bSimpleQuote) {
                                if (bComment) {
                                    bComment = !bComment;   
                                }
                            }                              
                        }
                    }
                }
                iPos++;
                bFirstChr = false;
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// StrFindFirstOfMultiKeyword:     Finds the keyword appearing first in the string
        /// </summary>
        /// <param name="str">              String to search in</param>
        /// <param name="iStartPos">        Starting position</param>
        /// <param name="iKeywordPos">      Returned the keyword position</param>
        /// <param name="iKeywordFound">    Argument position of the keyword found. Can be -1</param>
        /// <param name="keywordList">      List of keywords</param>
        /// <returns>
        /// Position after the first keyword found or -1 if not found
        /// </returns>
        //
        //*********************************************************	    
        public static int StrFindFirstOfMultiKeyword(string            str,
                                                     int               iStartPos,
                                                     out int           iKeywordPos,
                                                     out int           iKeywordFound,
                                                     SearchedKeyword[] keywordList) {
            int     iRetVal = -1;
            int     iTmp;
            int     iTmp2;
            int     iIndex;

            iKeywordFound = -1;
            iKeywordPos   = -1;
            for (iIndex = 0; iIndex < keywordList.Length; iIndex++) {
                iTmp = StrFindKeyword(str,
                                      iStartPos,
                                      out iTmp2,
                                      keywordList[iIndex].strKeyword,
                                      keywordList[iIndex].eSearchType);
                if (iTmp != -1) {
                    if (iRetVal == -1 || iTmp < iRetVal) {
                        iRetVal       = iTmp;
                        iKeywordPos   = iTmp2;
                        iKeywordFound = iIndex;
                    }
                }
            }
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SetControlText:             Set the text of the specified controls or forms
        /// </summary>
        /// <param name="app">          Rep++ Application object</param>
        /// <param name="ctl">          Array of objects</param>
        //
        //******************************************************************
        public static void SetControlText(RepPP.IApplication app, Control ctl) {
            string  strAtomName;
            
            strAtomName = "txt" + ctl.Name;
            if (app.IsAtomExist(strAtomName, RepPP.AtomType.sdAtomMsg)) {
                ctl.Text = app.GetAtomText("txt" + ctl.Name, RepPP.AtomType.sdAtomMsg);
            }
            foreach (Control ctlChild in ctl.Controls) {
                SetControlText(app, ctlChild);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SetControlText:             Set the text of the specified controls or forms
        /// </summary>
        /// <param name="app">          Rep++ Application object</param>
        /// <param name="frm">          Form object</param>
        //
        //******************************************************************
        public static void SetControlText(RepPP.IApplication app, Form frm) {
            string  strAtomName;
            
            strAtomName = "txt" + frm.Name;
            if (app.IsAtomExist(strAtomName, RepPP.AtomType.sdAtomMsg)) {
                frm.Text = app.GetAtomText("txt" + frm.Name, RepPP.AtomType.sdAtomMsg);
            }
            foreach (Control ctlChild in frm.Controls) {
                SetControlText(app, ctlChild);
            }
        }
	} /* class */
} /* namespace */
