using System;
using System.Collections;
using System.IO;
using System.Resources;
using System.Xml;
using System.Globalization;

namespace RepPP.Toolkit.CodeGen {
	/// <summary>
	/// TypedInstanceGenBase: Base class for typed instance generation
	/// </summary>
	public abstract class TypedInstanceGenBase {
        /// <summary>Prefis for the generated typed rowset tree class name </summary>
        public const string TYPED_ROWSET_TREE_PREFIX    = "Base";
        /// <summary>Prefix for the generated typed rowset class name </summary>
        public const string TYPED_ROWSET_PREFIX         = "Base";
        /// <summary>Prefix to use for web services generated class name</summary>
        public const string WEB_SERV_CLASS_PREFIX       = "WS";
        /// <summary>Namespace for the generated typed instances</summary>
        public const string TYPED_INST_NAMESPACE        = "Internal";
        /// <summary>
        /// Interface of a Log object
        /// </summary>
        public interface ILog {
            /// <summary>
            /// LogClashBetweenRowsetAndRowsetTreeName:     Log a clash between generated class for a rowset tree and a rowset
            /// </summary>
            /// <param name="rowsetTreeDef">    Rowset tree definition</param>
            /// <param name="rowsetDef">        Rowset definition</param>
            /// <param name="strName">          Name in clash</param>
            void LogClashBetweenRowsetAndRowsetTreeName(RepPP.RowsetTreeDef rowsetTreeDef, RepPP.RowsetDef rowsetDef, string strName);
            /// <summary>
            /// LogClashError:                  Log a clash between a generated property and a base member
            /// </summary>
            /// <param name="strName">          Name in clash</param>
            /// <param name="eBaseMemberType">  Type of the base member</param>
            /// <param name="ePropType">        Type of the property</param>
            void LogClashError(string strName, ClashObjectType eBaseMemberType, ClashObjectType ePropType);
        };

        /// <summary>
        /// Manager of Comments in Resources
        /// </summary>
        public class CommentResManager {
            /// <summary>Dotnet resource manager</summary>
            private ResourceManager        m_resManager;

            /// <summary>Culture informations used by the comments</summary>
            private CultureInfo            m_culture;

            //******************************************************************
            //
            /// <summary>
            /// Class constructor.
            /// </summary>
            //
            //******************************************************************
            public CommentResManager() {
                m_culture = null;
            }

            //******************************************************************
            //
            /// <summary>
            /// Get the Dotnet resource manager
            /// </summary>
            //
            //******************************************************************
            private ResourceManager ResourceManager {
                get {
                    if (m_resManager == null) {
                        m_resManager = new ResourceManager("RepPP.Toolkit.CommentsDictionary", typeof(CommentResManager).Assembly);
                    }
                    return (m_resManager);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Gets or sets the Culture informations used 
            /// when the would like change the language of the comments
            /// </summary>
            //
            //******************************************************************
            public CultureInfo Culture {
                get {
                    return (m_culture);
                }
                set {
                    m_culture = value;
                }
            }
            //******************************************************************
            //
            /// <summary>
            /// GetString:      Get a string from ressource in the 
            ///                 current culture
            /// </summary>
            /// <param name="arrKey">List of key to find out the string</param>
            /// <returns>
            /// String in the correct culture otherwise null value
            /// </returns>
            //
            //******************************************************************
            protected virtual String GetString(String[] arrKey) {
                String      strRetVal;
                String      strUniqueName;

                strUniqueName = String.Join(".", arrKey);
                strRetVal = ResourceManager.GetString(strUniqueName, m_culture);
#if DEBUG
                if (String.IsNullOrEmpty(strRetVal)) {
                    System.Diagnostics.Debug.WriteLine(String.Format("Missing '{0}' Key to get the Comment string in the ressource", strUniqueName));
                }
#endif
                return(strRetVal);
            }

            //******************************************************************
            //
            /// <summary>
            /// GetMisc:        Gets a miscellaneous value from the ressource
            /// </summary>
            /// <param name="strMiscKey">Miscelaneous key to find out the
            /// correct string in resource</param>
            /// <returns>
            /// The miscellaneous value found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public String GetMisc(String strMiscKey) {
                return (GetString(new String[] { strMiscKey }));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetMisc:        Gets a miscellaneous value from the ressource
            /// </summary>
            /// <param name="arrMiscKey">List of keys to find out the correct
            /// string in resource</param>
            /// <returns>
            /// The miscellaneous value if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public String GetMisc(String[] arrMiscKey) {
                return (GetString(arrMiscKey));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetClassDesc:   Gets a Class Description
            /// </summary>
            /// <param name="strClassNameKey">Class Name Key to find out 
            /// the class description</param>
            /// <returns>
            /// The class description if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public string GetClassDesc(string strClassNameKey) {
                return(GetString(new String[] { strClassNameKey }));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetMemberDesc:  Gets a class member Description
            /// </summary>
            /// <param name="strClassNameKey">Class Name Key to find out 
            /// the member description</param>
            /// <param name="strMemberNameKey">Member Name Key to find out 
            /// the member description</param>
            /// <returns>
            /// The member description if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public String GetMemberDesc(String strClassNameKey, String strMemberNameKey) {
                return (GetString(new String[] { strClassNameKey, strMemberNameKey }));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetClassDesc:   Gets a function Description
            /// </summary>
            /// <param name="strClassNameKey">Class Name Key to find out 
            /// the function description</param>
            /// <param name="strFunctionNameKey">Function Name Key to find out 
            /// the function description</param>
            /// <returns>
            /// The function description if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public String GetFuncDesc(String strClassNameKey, String strFunctionNameKey) {
                return(GetString(new String[] { strClassNameKey, strFunctionNameKey }));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetFuncParamDesc:   Gets a parameter of function Description
            /// </summary>
            /// <param name="strClassNameKey">Class Name Key to find out 
            /// the parameter of function description</param>
            /// <param name="strFunctionNameKey">Function Name Key to find out 
            /// the parameter of function description</param>
            /// <param name="strParameterNameKey">Parameter Name Key to find out 
            /// the parameter of function description</param>
            /// <returns>
            /// The parameter of function description if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public String GetFuncParamDesc(String strClassNameKey, String strFunctionNameKey, String strParameterNameKey) {
                return(GetString(new String[] { strClassNameKey, strFunctionNameKey, strParameterNameKey }));
            }

            //******************************************************************
            //
            /// <summary>
            /// GetFuncReturnDesc:  Gets the return function Description
            /// </summary>
            /// <param name="strClassNameKey">Class Name Key to find out 
            /// the return function description</param>
            /// <param name="strFunctionNameKey">Function Name Key to find out 
            /// the return function description</param>
            /// <returns>
            /// The return function description if found otherwise a null value
            /// </returns>
            //
            //******************************************************************
            public string GetFuncReturnDesc(string strClassNameKey, string strFunctionNameKey) {
                return (GetFuncParamDesc(strClassNameKey, strFunctionNameKey, "return"));
            }

        }// End class CommentResManager

	    /// <summary>Interface to inform project of file update</summary>
	    public interface IProjectFolder {

            //******************************************************************
            //
	        /// <summary>
	        /// Creates a file in the project
	        /// </summary>
	        /// <param name="strFileName">  FileName to add</param>
	        /// <returns>
	        /// Stream to be used to write to the file
	        /// </returns>
	        //
            //******************************************************************
	        StreamWriter CreateFile(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Open a file for reading
	        /// </summary>
	        /// <param name="strFileName">  Name of the file</param>
	        /// <returns>
	        /// Stream to be used to read the file
	        /// </returns>
	        //
            //******************************************************************
	        StreamReader OpenForReading(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Open a file for writing
	        /// </summary>
	        /// <param name="strFileName">  Name of the file</param>
	        /// <returns>
	        /// Stream to be used to write to the file
	        /// </returns>
	        //
            //******************************************************************
	        StreamWriter OpenForWriting(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Delete a file from project
	        /// </summary>
	        /// <param name="strFileName">  Name of the file</param>
	        //
            //******************************************************************
	        void DeleteFile(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Checks if the file exist in the project folder
	        /// </summary>
	        /// <param name="strFileName">  Name of the file</param>
	        //
            //******************************************************************
	        bool IsFileExistInFolder(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Checks if the file exist in the project
	        /// </summary>
	        /// <param name="strFileName">  Name of the file</param>
	        //
            //******************************************************************
	        bool IsFileExistInProject(string strFileName);

            //******************************************************************
            //
	        /// <summary>
	        /// Gets the list of files in the project
	        /// </summary>
	        /// <returns>
	        /// List of files
	        /// </returns>
	        //
            //******************************************************************
	        string[] GetFileList();
	    }
        
        /// <summary>Type of member which can cause name clash</summary>
        public enum ClashObjectType {
            /// <summary>Unknwown object type</summary>
            Unknown         = 0,
            /// <summary>Base field (.Net field)</summary>
            BaseField       = 1,
            /// <summary>Base property</summary>
            BaseProperty    = 2,
            /// <summary>Base method</summary>
            BaseMethod      = 3,
            /// <summary>Base constructor</summary>
            BaseCtor        = 4,
            /// <summary>Base event</summary>
            BaseEvent       = 5,
            /// <summary>A property giving access to a child group instance</summary>
            Rowset          = 6,
            /// <summary>A property giving access to a Rep++ field</summary>
            Field           = 7
        };
	    
	    /// <summary>Name of the configuration file for auto regeneration</summary>
	    private const string            m_strConfigFileName = "TypedConfig.xml";
        /// <summary>Signature for typed instance files</summary>
        private string                  m_strSignature      = " {0}\r\n\r\n";
        /// <summary>User Signature for typed instance files</summary>
        private string                  m_strUserSignature  = " {1}.\r\n\r\n";
        /// <summary>Web Services generator class</summary>
	    private WebServicesGenBase      m_wsGen;
        /// <summary>Manager of Comments in Resources</summary>
        protected CommentResManager     m_cmtResManager;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="wsGen">                Generator Web Services adapter class</param>
        //
        //******************************************************************
		protected TypedInstanceGenBase(WebServicesGenBase wsGen) {
		    m_wsGen            = wsGen;
            m_cmtResManager     = new CommentResManager();
            m_strSignature     = String.Format(m_strSignature, (m_cmtResManager).GetMisc("TypedInstanceAdvice"));
            m_strUserSignature = String.Format(m_strSignature, (m_cmtResManager).GetMisc("UserTypedInstanceAdvice"));
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //
        //******************************************************************
		protected TypedInstanceGenBase() : this(null) {
		}

        //******************************************************************
        //
        /// <summary>
        /// WSGenerator:                        Web services generator if any
        /// </summary>
        //
        //******************************************************************
		protected WebServicesGenBase WSGenerator {
		    get {
		        return(m_wsGen);
		    }
		}

        //******************************************************************
        //
        /// CommentManager:                     Manager of Comments
        ///                                     generated in Typed Instance
        //
        //******************************************************************
        public CommentResManager CommentManager {
            get {
                return(m_cmtResManager);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// 
        /// </summary>
        //
        //******************************************************************
        public string GetTypeOfFieldFromFieldType(RepPP.FieldType eType, int iMaxSize) {
	        string  strRetVal;
	        
	        switch(eType) {	            
	        case FieldType.sdFieldDate:
	        case FieldType.sdFieldDateTime:
	        case FieldType.sdFieldTime:
	            strRetVal = "FieldDateTime";
	            break;
	        case FieldType.sdFieldMoney:
	        case FieldType.sdFieldNumericDec:
	        case FieldType.sdFieldPercentageDec:
	            strRetVal = "FieldDecimal";
	            break;
	        case FieldType.sdFieldNumeric:
	            if (iMaxSize > 9) {
	                strRetVal = "FieldDecimal";
	            } else {
	                strRetVal = "FieldInt";
                }
	            break;
	        case FieldType.sdFieldMultiNumChoice:
	        case FieldType.sdFieldSimpleNumChoice:
	            strRetVal = "FieldInt";
	            break;
#if !SD70
            case FieldType.sdFieldBoolean:
                strRetVal = "FieldBoolean";
                break;
#endif                
	        default:
	            strRetVal = "FieldString";
	            break;
	        }
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetFieldName:               Gets the name of the a field
        /// </summary>
        /// <param name="fldDef">       Field definition object</param>
        /// <returns>
        /// Name of the field
        /// </returns>
        //
        //******************************************************************
        public virtual string GetFieldName(RepPP.FieldDef fldDef) {
            return(fldDef.AlternateName);
        }

        //******************************************************************
        //
        /// <summary>
        /// DecorateFieldPropertyName:  Decorate the name of the property for a field
        /// </summary>
        /// <param name="strPrefix">    Prefix to use</param>
        /// <param name="strFldName">   Field name</param>
        /// <returns>
        /// Property name for the field
        /// </returns>
        //
        //******************************************************************
        protected virtual string DecorateFieldPropertyName(string strPrefix, string strFldName) {
            return(strPrefix + strFldName.Replace("$", ""));
        }

        //******************************************************************
        //
        /// <summary>
        /// GetRowsetName:                  Get the name of a rowset
        /// </summary>
        /// <param name="rowsetDef">        RowsetDef</param>
        /// <returns>
        /// Name for the group
        /// </returns>
        //
        //******************************************************************
        public virtual string GetRowsetName(RepPP.RowsetDef rowsetDef) {
            return(rowsetDef.AlternateName);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetNodeRowsetName:          Get the name of the rowset attached to the node of a rowset tree definition
        /// </summary>
        /// <param name="node">     Child rowset definition</param>
        /// <returns>
        /// Name of a child rowset definition
        /// </returns>
        //
        //******************************************************************
        public virtual string GetNodeRowsetName(RepPP.RowsetTreeDefNode node) {
            return(node.AlternateName);
        }

        //******************************************************************
        //
        /// <summary>
        /// DecorateRowsetClassName:    Decorate the name of the generated class for a rowset
        /// </summary>
        /// <param name="strPrefix">    Prefix to use</param>
        /// <param name="strRowsetName">Rowset name</param>
        /// <returns>
        /// Class name for the rowset
        /// </returns>
        //
        //******************************************************************
        protected virtual string DecorateRowsetClassName(string strPrefix, string strRowsetName) {
            return(strPrefix + strRowsetName.Replace("$", ""));
        }

        //******************************************************************
        //
        /// <summary>
        /// DecorateRowsetPropertyName: Decorate the name of the generated property for a rowset
        /// </summary>
        /// <param name="strPrefix">    Prefix to use</param>
        /// <param name="strRowsetName">Rowset name</param>
        /// <returns>
        /// Property name for the rowset
        /// </returns>
        //
        //******************************************************************
        protected virtual string DecorateRowsetPropertyName(string strPrefix, string strRowsetName) {
            return(strPrefix + strRowsetName.Replace("$", ""));
        }

        //******************************************************************
        //
        /// <summary>
        /// GetRowsetTreeName:              Get the name of a rowset tree
        /// </summary>
        /// <param name="rowsetTree">       Rowset tree</param>
        /// <returns>
        /// Name for the rowset tree
        /// </returns>
        //
        //******************************************************************
        public virtual string GetRowsetTreeName(RepPP.RowsetTreeDef rowsetTree) {
            return(rowsetTree.AlternateName);
        }

        //******************************************************************
        //
        /// <summary>
        /// DecorateRowsetTreeClassName:    Decorates the name of the generated class for a rowset tree
        /// </summary>
        /// <param name="strPrefix">        Prefix to use</param>
        /// <param name="strCrdName">       Rowset tree name</param>
        /// <returns>
        /// Class name for the rowset tree
        /// </returns>
        //
        //******************************************************************
        protected virtual string DecorateRowsetTreeClassName(string strPrefix, string strCrdName) {
            return(strPrefix + strCrdName.Replace("$", ""));
        }

        //*********************************************************
        //
        /// <summary>
        /// FillHashWithName:       Fills the hash table with base member
        /// </summary>
        /// <param name="hashName"> Hash table to fill</param>
        /// <param name="type">     Type of the class</param>
        //
        //*********************************************************
        private void FillHashWithName(Hashtable hashName, Type type) {
            System.Reflection.MemberInfo[]  arrMI;
            string                          strName;
            ClashObjectType                 eType;
            bool                            bPrivate;

            arrMI = type.GetMembers(System.Reflection.BindingFlags.NonPublic |
                                    System.Reflection.BindingFlags.Instance  |
                                    System.Reflection.BindingFlags.Static);
            foreach (System.Reflection.MemberInfo mi in arrMI) {
                bPrivate = false;
                strName  = mi.Name;
                switch(mi.MemberType) {
                case System.Reflection.MemberTypes.Event:
                    eType    = ClashObjectType.BaseEvent;
                    break;
                case System.Reflection.MemberTypes.Field:
                    bPrivate = ((System.Reflection.FieldInfo)mi).IsPrivate;
                    eType    = ClashObjectType.BaseField;
                    break;
                case System.Reflection.MemberTypes.Constructor:
                    bPrivate = ((System.Reflection.ConstructorInfo)mi).IsPrivate;
                    eType    = ClashObjectType.BaseCtor;
                    break;
                case System.Reflection.MemberTypes.Method:
                    bPrivate = ((System.Reflection.MethodInfo)mi).IsPrivate;
                    eType    = ClashObjectType.BaseMethod;
                    break;
                case System.Reflection.MemberTypes.Property:
                    bPrivate = ((System.Reflection.PropertyInfo)mi).GetGetMethod().IsPrivate &&
                               ((System.Reflection.PropertyInfo)mi).GetSetMethod().IsPrivate;
                    eType    = ClashObjectType.BaseProperty;
                    break;
                default:
                    eType    = ClashObjectType.Unknown;
                    break;
                }
                if (!bPrivate && eType != ClashObjectType.Unknown && !hashName.ContainsKey(strName)) {
                    hashName.Add(strName, eType);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CheckRowsetNameClash:               Checks for name clash for the specified rowset
        /// </summary>
        /// <param name="log">                  Log object</param>
        /// <param name="typeRowsetBaseClass">  Base class used for generated rowset</param>
        /// <param name="strFldPropPrefix">     Prefix to use for property accessing field</param>
        /// <param name="rowsetDef">            Rowset definition to be checked</param>
        /// <returns>
        /// true:       Success
        /// false:      Name clash
        /// </returns>
        //
        //*********************************************************
        private bool CheckRowsetNameClash(ILog log, Type typeRowsetBaseClass, string strFldPropPrefix, RepPP.RowsetDef rowsetDef) {
            bool        bRetVal = true;
            Hashtable   hashName;
            string      strName;

            hashName = new Hashtable(100);
            FillHashWithName(hashName, typeRowsetBaseClass);
            foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                strName = DecorateFieldPropertyName(strFldPropPrefix, GetFieldName(fld));
                if (hashName.ContainsKey(strName)) {
                    log.LogClashError(strName, (ClashObjectType)hashName[strName], ClashObjectType.Field);
                } else {
                    hashName.Add(strName, ClashObjectType.Field);
                }
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// CheckNodeRowsetNameClash:           Checks for name clash for the rowset attached to the specified rowset tree definition
        /// </summary>
        /// <param name="log">                  Log object</param>
        /// <param name="typeRowsetBaseClass">  Base class used for generated rowset</param>
        /// <param name="strFldPropPrefix">     Prefix to use for property accessing field instance</param>
        /// <param name="node">                 Node of the rowset tree definition to be checked</param>
        /// <returns>
        /// true:       Success
        /// false:      Name clash
        /// </returns>
        //
        //*********************************************************
        private bool CheckNodeRowsetNameClash(ILog log, Type typeRowsetBaseClass, string strFldPropPrefix, RepPP.RowsetTreeDefNode node) {
            bool    bRetVal;

            bRetVal = CheckRowsetNameClash(log, typeRowsetBaseClass, strFldPropPrefix, node.RowsetDef);
            foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                if (!CheckNodeRowsetNameClash(log, typeRowsetBaseClass, strFldPropPrefix, nodeChild)) {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// FillNodeList:               Fill a list with all nodes of a rowset tree definition hierarchy
        /// </summary>
        /// <param name="node">         Node</param>
        /// <param name="arrNode">      Array to fill</param>
        //
        //*********************************************************
        private void FillNodeList(RepPP.RowsetTreeDefNode node, ArrayList arrNode) {
            arrNode.Add(node);
            foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                FillNodeList(nodeChild, arrNode);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CheckNameForClash:                      Checks if name clash exist with generated field and rowset name with base class
        /// </summary>
        /// <param name="log">                      Log object</param>
        /// <param name="arrRowsetTreeDefs">        Array of rowset tree definition to check</param>
        /// <param name="arrRowsetDef">             Array of rowset definition to check</param>
        /// <param name="typeBaseRowsetTreeClass">  Type of base class for generated rowset tree</param>
        /// <param name="typeBaseRowsetClass">      Type of base class for generated rowset</param>
        /// <param name="strRowsetTreeClsPrefix">   Prefix to use with rowset tree name</param>
        /// <param name="strRowsetClsPrefix">       Prefix to use with rowset name</param>
        /// <param name="strRowsetPropPrefix">      Prefix to use for property to access rowset</param>
        /// <param name="strFldPropPrefix">         Prefix to use for property to access field</param>
        /// <returns>
        /// true:   Ok
        /// false:  Problem. Name clash
        /// </returns>
        /// <remarks>
        /// 1.  Must ensure the name of all classes generated for rowset trees and rowsets don't clash.
        /// 2.  Must ensure that all properties accessing child rowset in rowset tree class don't clash with base class.
        /// 3.  Must ensure that all properties for each rowset class don't clash with base class.
        /// </remarks>
        //
        //*********************************************************        
        public bool CheckForNameClash(ILog                  log,
                                      RepPP.RowsetTreeDef[] arrRowsetTreeDefs,
                                      RepPP.RowsetDef[]     arrRowsetDef,
                                      Type                  typeBaseRowsetTreeClass,
                                      Type                  typeBaseRowsetClass,
                                      string                strRowsetTreeClsPrefix,
                                      string                strRowsetClsPrefix,
                                      string                strRowsetPropPrefix,
                                      string                strFldPropPrefix) {
            bool                    bRetVal = true;
            RepPP.RowsetTreeDefNode nodeRoot;
            string                  strName;
            Hashtable               hashName;
            ArrayList               arrNode;

            hashName = new Hashtable(arrRowsetTreeDefs.Length);
            foreach (RepPP.RowsetTreeDef rowsetTree in arrRowsetTreeDefs) {
                hashName.Add(DecorateRowsetTreeClassName(strRowsetTreeClsPrefix, GetRowsetTreeName(rowsetTree)), rowsetTree);
            }
            foreach (RepPP.RowsetDef rowsetDef in arrRowsetDef) {
                strName = DecorateRowsetClassName(strRowsetClsPrefix, GetRowsetName(rowsetDef));
                if (hashName.ContainsKey(strName)) {
                    bRetVal = false;
                    log.LogClashBetweenRowsetAndRowsetTreeName(hashName[strName] as RepPP.RowsetTreeDef, rowsetDef, strName);
                }
            }
            if (bRetVal) {
                foreach (RepPP.RowsetTreeDef rowsetTree in arrRowsetTreeDefs) {
                    hashName.Clear();
                    nodeRoot = rowsetTree.Nodes[0];
                    if (nodeRoot != null) {
                        arrNode = new ArrayList(32);
                        FillNodeList(nodeRoot, arrNode);
                        FillHashWithName(hashName, typeBaseRowsetTreeClass);
                        foreach (RepPP.RowsetTreeDefNode node in arrNode) {
                            strName = DecorateRowsetPropertyName(strRowsetPropPrefix, GetNodeRowsetName(node));
                            if (hashName.ContainsKey(strName)) {
                                log.LogClashError(strName, (ClashObjectType)hashName[strName], ClashObjectType.Rowset);
                                bRetVal = false;
                            } else {
                                hashName.Add(strName, ClashObjectType.Rowset);
                            }
                        }
                        if (!CheckNodeRowsetNameClash(log, typeBaseRowsetClass, strFldPropPrefix, nodeRoot)) {
                            bRetVal = false;
                        }
                    }
                }
                foreach (RepPP.RowsetDef rowsetDef in arrRowsetDef) {
                    if (!CheckRowsetNameClash(log, typeBaseRowsetClass, strFldPropPrefix, rowsetDef)) {
                        bRetVal = false;
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowset:                    Generate typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">                Rowset definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated property rowset name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated property field name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <param name="strBaseClassName">         Name of the base class</param>
        /// <param name="bGenWS">                   true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public abstract string GenerateTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                   string           strRowsetTreeClassPrefix,
                                                   string           strRowsetClassPrefix,
                                                   string           strRowsetPropertyPrefix,
                                                   string           strFldPropertyPrefix,
                                                   string           strNameSpace,
                                                   string           strRootNameSpace,
                                                   string           strBaseClassName,
                                                   bool             bGenWS,
                                                   bool             bUseWSDataContracts);

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowset:                Generate typed instance for a group
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <param name="strBaseClassName">     Name of the base class</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">  true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public string GenerateTypedRowset(RepPP.RowsetDef   rowsetDef, 
                                          string            strNameSpace,
                                          string            strRootNameSpace,
                                          string            strBaseClassName,
                                          bool              bGenWS,
                                          bool              bUseWSDataContracts) {
            return(GenerateTypedRowset(rowsetDef,
                                       "RTC",
                                       "RC",
                                       "rs",
                                       "fld",
                                       strNameSpace,
                                       strRootNameSpace,
                                       strBaseClassName,
                                       bGenWS,
                                       bUseWSDataContracts));
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowsetTree:                    Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTree">                   Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">         Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">      Prefix for generated property rowset name</param>
        /// <param name="strFldPropertyPrefix">         Prefix for generated property field name</param>
        /// <param name="strNameSpace">                 The namespace</param>
        /// <param name="strRootNamespace">             The root namespace of the project</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowset</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public abstract string GenerateTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTree, 
                                                       string               strRowsetTreeClassPrefix,
                                                       string               strRowsetClassPrefix,
                                                       string               strRowsetPropertyPrefix,
                                                       string               strFldPropertyPrefix,
                                                       string               strNameSpace,
                                                       string               strRootNamespace,
                                                       string               strRowsetTreeBaseClassName,
                                                       string               strRowsetBaseClassName,
                                                       bool                 bGenWS,
                                                       bool                 bUseWSDataContracts);

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowsetTree:                    Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTree">                   Rowset tree definition</param>
        /// <param name="strNameSpace">                 The namespace</param>
        /// <param name="strRootNamespace">             The root namespace of the project</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowset</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public string GenerateTypedRowsetTree(RepPP.RowsetTreeDef   rowsetTree, 
                                              string                strNameSpace,
                                              string                strRootNamespace,
                                              string                strRowsetTreeBaseClassName,
                                              string                strRowsetBaseClassName,
                                              bool                  bGenWS,
                                              bool                  bUseWSDataContracts) {
            return(GenerateTypedRowsetTree(rowsetTree,
                                           "RTC",
                                           "RC",
                                           "rs",
                                           "fld",
                                           strNameSpace,
                                           strRootNamespace,
                                           strRowsetTreeBaseClassName,
                                           strRowsetBaseClassName,
                                           bGenWS,
                                           bUseWSDataContracts));                                                
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowset:            Generate user typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition</param>
        /// <param name="strRowsetClassPrefix"> Prefix for generated rowset class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public abstract string GenerateUserTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                       string           strRowsetClassPrefix,
                                                       string           strNameSpace,
                                                       string           strRootNameSpace);

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowsetTree:            Generate user typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">            Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public abstract string GenerateUserTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTreeDef, 
                                                           string               strRowsetTreeClassPrefix,
                                                           string               strNameSpace,
                                                           string               strRootNameSpace);

        //******************************************************************
        //
		/// <summary>
		/// BuildRowsetTreeFileName:                    Build the name of a rowset tree typed instance
		/// </summary>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset tree class name</param>
		/// <param name="rowsetTree">                   Rep++ Rowset tree definition</param>
		/// <param name="strFileExt">                   File extension</param>
		/// <returns>
		/// Filename
		/// </returns>
        //
        //******************************************************************
		public virtual string BuildRowsetTreeFileName(string strRowsetTreeClassPrefix, RepPP.RowsetTreeDef rowsetTree, string strFileExt) {
		    return(DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, (rowsetTree == null) ? "" : GetRowsetTreeName(rowsetTree)) + strFileExt);
		}

        //******************************************************************
        //
		/// <summary>
		/// BuildRowsetFileName:                    Build the name of a typed rowset
		/// </summary>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
		/// <param name="rowsetDef">                Rep++ rowset definition</param>
		/// <param name="strFileExt">               File extension</param>
		/// <returns>
		/// Filename
		/// </returns>
        //
        //******************************************************************
		public virtual string BuildRowsetFileName(string strRowsetClassPrefix, RepPP.RowsetDef rowsetDef, string strFileExt) {
		    return(DecorateRowsetClassName(strRowsetClassPrefix, (rowsetDef == null) ? "" : GetRowsetName(rowsetDef)) + strFileExt);
		}

        //******************************************************************
        //
        /// <summary>
        /// GenerateSignature:              Generates the signature
        /// </summary>
        /// <param name="strRemark">        Command for remark (' or //)</param>
        /// <returns>
        /// Signature
        /// </returns>
        //
        //******************************************************************
		protected string GenerateSignature(string strRemark) {
		    return(strRemark + m_strSignature);
		}

        //******************************************************************
        //
        /// <summary>
        /// GenerateSignature:              Generates the user signature (without the "do not modify" comment)
        /// </summary>
        /// <param name="strRemark">        Command for remark (' or //)</param>
        /// <returns>
        /// Signature
        /// </returns>
        //
        //******************************************************************
        protected string GenerateUserSignature(string strRemark) {
            return (strRemark + m_strUserSignature);
        }

        //******************************************************************
        //
		/// <summary>
		/// IsSignaturePresent:             Checks if the signature is present
		/// </summary>
		/// 
		/// <param name="projFolder">       Interface with the project folder</param>
		/// <param name="strFileName">      File name</param>
		/// <returns>
		/// true if succeed, false if not
		/// </returns>
        //
        //******************************************************************
		private bool IsSignaturePresent(IProjectFolder projFolder, string strFileName) {
		    string          strContent;
		    StreamReader    streamReader;
		    
		    streamReader = projFolder.OpenForReading(strFileName);
		    using(streamReader) {
		        strContent = streamReader.ReadToEnd();
		    }
		    return(strContent.IndexOf(m_strSignature) != -1);
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateRowsets:                      Regenerates all rowset which need to be regenerated
		/// </summary>
		/// <param name="app">                      Rep++ application object</param>
		/// <param name="projFolder">               Interface with the project folder</param>
		/// <param name="strFileExt">               File extension (.vb, .cs)</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix    "> Prefix for generated property rowset name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated property field name</param>
		/// <param name="strNameSpace">             Namespace</param>
        /// <param name="strRootNameSpace">         Root Namespace of the project</param>
		/// <param name="strRowsetBaseClass">       Rowset base class</param>
		/// <param name="bGenWS">                   true to generates Web Services structures</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
        /// <param name="bFullSync">                true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		private void RegenerateRowsets(RepPP.Application  app,
		                               IProjectFolder     projFolder,
		                               string             strFileExt,
                                       string             strRowsetTreeClassPrefix,
                                       string             strRowsetClassPrefix,
                                       string             strRowsetPropertyPrefix,
                                       string             strFldPropertyPrefix,
		                               string             strNameSpace,
                                       string             strRootNameSpace,      
		                               string             strRowsetBaseClass,
		                               bool               bGenWS,
                                       bool               bUseWSDataContracts,
		                               bool               bFullSync) {
		    bool            bGenerate;
		    bool            bFileExist;
		    string          strFileName;
		    string          strNewContent;
		    string          strOldContent;
		    StreamReader    streamReader;
		    StreamWriter    streamWriter;
            string          strUserFileName;
		    
		    foreach (RepPP.RowsetDef rowsetDef in app.RowsetDefs) {
		        strFileName = BuildRowsetFileName(TYPED_ROWSET_PREFIX + strRowsetClassPrefix, rowsetDef, strFileExt);
		        bFileExist  = projFolder.IsFileExistInFolder(strFileName);
		        if (bFileExist || bFullSync) {
		            strNewContent = GenerateTypedRowset(rowsetDef,
                                                        strRowsetTreeClassPrefix,
                                                        strRowsetClassPrefix,
                                                        strRowsetPropertyPrefix,
                                                        strFldPropertyPrefix,
		                                                strNameSpace,
                                                        strRootNameSpace,
		                                                strRowsetBaseClass,
		                                                bGenWS,
                                                        bUseWSDataContracts);
		            if (bFileExist) {
		                using(streamReader = projFolder.OpenForReading(strFileName)) {
		                    strOldContent = streamReader.ReadToEnd();
		                }
		                bGenerate = (strOldContent != strNewContent);
		            } else {
		                bGenerate = bFullSync;
		            }
		            if (bGenerate) {
		                streamWriter = (bFileExist) ? projFolder.OpenForWriting(strFileName) :
		                                              projFolder.CreateFile(strFileName);
                        using(streamWriter) {
                            streamWriter.Write(strNewContent);
                        }
                    }
                    // Generate user typed instance class
                    strUserFileName = BuildRowsetFileName(strRowsetClassPrefix, rowsetDef, strFileExt);
                    if (!projFolder.IsFileExistInProject(strUserFileName)) {
		                strNewContent = GenerateUserTypedRowset(rowsetDef,
                                                                strRowsetClassPrefix,
		                                                        strNameSpace,
                                                                strRootNameSpace);
		                streamWriter = projFolder.CreateFile(strUserFileName);
                        using(streamWriter) {
                            streamWriter.Write(strNewContent);
                        }
                    }
		        }
		    }
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateRowsetTrees:                  Regenerates all rowset trees which need to be regenerated
		/// </summary>
		/// <param name="app">                      Rep++ application object</param>
		/// <param name="projFolder">               Interface with the project folder</param>
		/// <param name="strFileExt">               File extension (.vb, .cs)</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated property rowset name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated property field name</param>
		/// <param name="strNameSpace">             Namespace</param>
        /// <param name="strRootNameSpace">         Root namespace of the project</param>
		/// <param name="strRowsetTreeBaseClass">   Rowset tree base class if found</param>
		/// <param name="strRowsetBaseClass">       Rowset base class</param>
		/// <param name="bGenWS">                   true to generates Web Services structures</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
		/// <param name="bFullSync">                true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		private void RegenerateRowsetTrees(RepPP.Application  app,
		                                   IProjectFolder     projFolder,
		                                   string             strFileExt,
                                           string             strRowsetTreeClassPrefix,
                                           string             strRowsetClassPrefix,
                                           string             strRowsetPropertyPrefix,
                                           string             strFldPropertyPrefix,
		                                   string             strNameSpace,
                                           string             strRootNameSpace,
		                                   string             strRowsetTreeBaseClass,
		                                   string             strRowsetBaseClass,
		                                   bool               bGenWS,
                                           bool               bUseWSDataContracts,
		                                   bool               bFullSync) {
		    bool                    bGenerate;
		    bool                    bFileExist;
		    string                  strFileName;
		    string                  strNewContent;
		    string                  strOldContent;
		    RepPP.RowsetTreeDef[]   arrRowsetTree;
		    RepPP.RowsetDef[]       arrRowset;
		    StreamReader            streamReader;
		    StreamWriter            streamWriter;
            string                  strUserFileName;
		    
		    arrRowsetTree = new RepPP.RowsetTreeDef[1];
		    arrRowset     = new RepPP.RowsetDef[0];
		    foreach (RepPP.RowsetTreeDef rowsetTree in app.RowsetTreeDefs) {
                // Generate typed instance class (_cc)
		        strFileName     = BuildRowsetTreeFileName(TYPED_ROWSET_TREE_PREFIX + strRowsetTreeClassPrefix, rowsetTree, strFileExt);
		        bFileExist      = projFolder.IsFileExistInFolder(strFileName);
		        if (bFileExist || bFullSync) {
		            strNewContent = GenerateTypedRowsetTree(rowsetTree,
                                                            strRowsetTreeClassPrefix,
                                                            strRowsetClassPrefix,
                                                            strRowsetPropertyPrefix,
                                                            strFldPropertyPrefix,
		                                                    strNameSpace,
                                                            strRootNameSpace,
		                                                    strRowsetTreeBaseClass,
		                                                    strRowsetBaseClass,
		                                                    bGenWS,
                                                            bUseWSDataContracts);
		            if (bFileExist) {                                          
		                using(streamReader  = projFolder.OpenForReading(strFileName)) {
		                    strOldContent = streamReader.ReadToEnd();
		                }
		                bGenerate = (strOldContent != strNewContent);
		            } else {
		                bGenerate = bFullSync;
		            }
		            if (bGenerate) {
		                streamWriter = (bFileExist) ? projFolder.OpenForWriting(strFileName) :
		                                              projFolder.CreateFile(strFileName);
                        using(streamWriter) {
                            streamWriter.Write(strNewContent);
                        }
                    }
                    // Generate user typed instance class
                    strUserFileName = BuildRowsetTreeFileName(strRowsetTreeClassPrefix, rowsetTree, strFileExt);
                    if (!projFolder.IsFileExistInProject(strUserFileName)) {
		                strNewContent = GenerateUserTypedRowsetTree(rowsetTree,
                                                                    strRowsetTreeClassPrefix,
		                                                            strNameSpace,
                                                                    strRootNameSpace);
		                streamWriter = projFolder.CreateFile(strUserFileName);
                        using(streamWriter) {
                            streamWriter.Write(strNewContent);
                        }
                    }
		        }
		    }
		}

        //******************************************************************
        //
		/// <summary>
		/// CheckRegenerateForNameClash:            Checks files that we must regenerate for name clash
		/// </summary>
		/// <param name="app">                      Rep++ application object</param>
        /// <param name="log">                      Logging object</param>
		/// <param name="projFolder">               Interface with the project folder</param>
		/// <param name="strFileExt">               File extension (.vb, .cs)</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated property rowset name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated property field name</param>
		/// <param name="bFullSync">                true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		private bool CheckRegenerateForNameClash(RepPP.Application  app,
		                                         ILog               log,
		                                         IProjectFolder     projFolder,
		                                         string             strFileExt,
                                                 string             strRowsetTreeClassPrefix,
                                                 string             strRowsetClassPrefix,
                                                 string             strRowsetPropertyPrefix,
                                                 string             strFldPropertyPrefix,
		                                         bool               bFullSync) {
		    bool            bRetVal;
		    ArrayList       arrRowsetTrees;
		    ArrayList       arrRowsetDef;
		    bool            bFileExist;
		    string          strFileName;

		    arrRowsetTrees = new ArrayList(app.RowsetTreeDefs.Count);
		    arrRowsetDef   = new ArrayList(app.RowsetDefs.Count);
		    foreach (RepPP.RowsetTreeDef rowsetTree in app.RowsetTreeDefs) {
		        strFileName     = BuildRowsetTreeFileName(strRowsetTreeClassPrefix, rowsetTree, strFileExt);
		        bFileExist      = projFolder.IsFileExistInFolder(strFileName);
		        if (bFileExist || bFullSync) {
		            arrRowsetTrees.Add(rowsetTree);
		        }
		    }
		    foreach (RepPP.RowsetDef rowsetDef in app.RowsetDefs) {
		        strFileName = BuildRowsetFileName(strRowsetClassPrefix, rowsetDef, strFileExt);
		        bFileExist  = projFolder.IsFileExistInFolder(strFileName);
		        if (bFileExist || bFullSync) {
		            arrRowsetDef.Add(rowsetDef);
                }		    
		    }
		    bRetVal = CheckForNameClash(log,
		                                (RepPP.RowsetTreeDef[])arrRowsetTrees.ToArray(typeof(RepPP.RowsetTreeDef)),
		                                (RepPP.RowsetDef[])arrRowsetDef.ToArray(typeof(RepPP.RowsetDef)),
	                                    typeof(RepPP.TypedRowsetTreeBase),
	                                    typeof(RepPP.TypedRowsetBase),
	                                    strRowsetTreeClassPrefix,
	                                    strRowsetClassPrefix,
	                                    strRowsetPropertyPrefix,
	                                    strFldPropertyPrefix);
            return(bRetVal);
	    }

        //******************************************************************
        //
		/// <summary>
		/// AddInfo:                Add an information to the config file
		/// </summary>
		/// <param name="xmlNode">  XML node</param>
		/// <param name="strInfo">  Node name</param>
		/// <param name="strVal">   Node value</param>
        //
        //******************************************************************
		private static void AddInfo(XmlNode xmlNode, string strInfo, string strVal) {
		    XmlAttribute    attr;
		    
		    attr        = xmlNode.OwnerDocument.CreateAttribute(strInfo);
		    attr.Value  = strVal;
		    xmlNode.Attributes.Append(attr);
		}

        //******************************************************************
        //
        /// <summary>
        /// GetInfo:                    Gets the information from a config file
        /// </summary>
        /// <param name="xmlNode">      XML node</param>
        /// <param name="strInfo">      Node name</param>
        /// <param name="strDefVal">    Default value</param>
        /// <returns>
        /// Node value
        /// </returns>
        //
        //******************************************************************
		private static string GetInfo(XmlNode xmlNode, string strInfo, string strDefVal) {
		    string      strRetVal;
		    XmlNode     attr;
		    
		    attr      = xmlNode.Attributes.GetNamedItem(strInfo, "");
		    strRetVal = (attr == null) ? strDefVal : attr.Value;
		    return(strRetVal);
		}

        //******************************************************************
        //
        /// <summary>
        /// GetInfo:                Gets the information from a config file
        /// </summary>
        /// <param name="xmlNode">  XML node</param>
        /// <param name="strInfo">  Node name</param>
        /// <returns>
        /// Node value
        /// </returns>
        //
        //******************************************************************
		private static string GetInfo(XmlNode xmlNode, string strInfo) {
		    string      strRetVal;
		    XmlNode     attr;
		    
		    attr      = xmlNode.Attributes.GetNamedItem(strInfo, "");
		    strRetVal = (attr == null) ? "" : attr.Value;
		    return(strRetVal);
		}

        //******************************************************************
        //
		/// <summary>
		/// SaveConfigInfo:                         Save configuration information in the XML file
		/// </summary>
		/// <param name="projFolder">               Interface with the project folder</param>
        /// <param name="strSysName">               System name</param>
        /// <param name="strAppName">               Program name</param>
        /// <param name="strConName">               Connection name</param>
        /// <param name="strUsrName">               User name</param>
        /// <param name="eLangCode">                Language code</param>
        /// <param name="strDataConName">           Data connection name (used only for Rep++ 8 and more)</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
		/// <param name="strNameSpace">             Namespace</param>
        /// <param name="strRootNameSpace">         The root namespace</param>
		/// <param name="strRowsetTreeBaseClass">   Rowset tree base class if found</param>
		/// <param name="strRowsetBaseClass">       Rowset base class</param>
		/// <param name="bGenWS">                   true to generates Web Services structures</param>
		/// <returns>
		/// true if succeed, false if not
		/// </returns>
        //
        //******************************************************************
		public static bool SaveConfigInfo(IProjectFolder    projFolder,
		                                  string            strSysName,
		                                  string            strAppName,
		                                  string            strConName,
		                                  string            strUsrName,
		                                  LanguageCode      eLangCode,
                                          string            strDataConName,
                                          string            strRowsetTreeClassPrefix,
                                          string            strRowsetClassPrefix,
                                          string            strRowsetPropertyPrefix,
                                          string            strFldPropertyPrefix,
		                                  string            strNameSpace,
                                          string            strRootNameSpace,
		                                  string            strRowsetTreeBaseClass,
		                                  string            strRowsetBaseClass,
		                                  bool              bGenWS) {
		    bool                    bRetVal;
		    XmlDocument             xmlDoc;
		    XmlNode                 xmlNode;
		    StreamWriter            streamWriter;
		    
		    if (strRowsetTreeBaseClass == null) {
		        strRowsetTreeBaseClass = String.Empty;
		    }
		    if (strRowsetBaseClass == null) {
		        strRowsetBaseClass = String.Empty;
		    }
		    streamWriter = projFolder.IsFileExistInFolder(m_strConfigFileName) ?
		                   projFolder.OpenForWriting(m_strConfigFileName) :
		                   projFolder.CreateFile(m_strConfigFileName);
		    try {
		        using(streamWriter) {
		            xmlDoc  = new XmlDocument();
		            xmlNode = xmlDoc.CreateNode(XmlNodeType.Element, "GenInfo", "");
		            xmlDoc.AppendChild(xmlNode);
		            AddInfo(xmlNode, "SystemName",       strSysName);
		            AddInfo(xmlNode, "ApplicationName",  strAppName);
		            AddInfo(xmlNode, "ConnectionName",   strConName);
		            AddInfo(xmlNode, "UserName",         strUsrName);
		            AddInfo(xmlNode, "LanguageCode",     ((int)eLangCode).ToString());
                    if (RepPP.Version.SDVERNUM >= 800) {
                        AddInfo(xmlNode, "DataConnectionName", strDataConName);
                    }
		            AddInfo(xmlNode, "RowsetTreeClassPrefix",   strRowsetTreeClassPrefix);
		            AddInfo(xmlNode, "RowsetClassPrefix",       strRowsetClassPrefix);
		            AddInfo(xmlNode, "RowsetPropPrefix",        strRowsetPropertyPrefix);
		            AddInfo(xmlNode, "FieldPropPrefix",         strFldPropertyPrefix);
		            AddInfo(xmlNode, "NameSpace",               strNameSpace);
                    AddInfo(xmlNode, "RootNameSpace",           strRootNameSpace);
		            AddInfo(xmlNode, "RowsetTreeBaseClass",     strRowsetTreeBaseClass);
		            AddInfo(xmlNode, "RowsetBaseClass",         strRowsetBaseClass);
		            AddInfo(xmlNode, "GenerateWSStruct",        bGenWS.ToString());
		            xmlDoc.Save(streamWriter);
		            bRetVal = true;
		        }
		    } catch {
		        bRetVal = false;
		    }
		    return(bRetVal);
		}

        //******************************************************************
        //
		/// <summary>
		/// ExtractConfigInfo:                      Extract configuration information from the XML file
		/// </summary>
		/// <param name="projFolder">               Interface with the project folder</param>
        /// <param name="strSysName">               System name</param>
        /// <param name="strAppName">               Program name</param>
        /// <param name="strConName">               Connection name</param>
        /// <param name="strUsrName">               User name</param>
        /// <param name="eLangCode">                Language code</param>
        /// <param name="strDataConName">           Data connection name (used only for Rep++ 8 and more)</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
		/// <param name="strNameSpace">             Namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
		/// <param name="strRowsetTreeBaseClass">   Rowset tree base class if found</param>
		/// <param name="strRowsetBaseClass">       Rowset base class</param>
		/// <param name="bGenWS">                   true to generates Web Services structures</param>
        //
        //******************************************************************
		public static void ExtractConfigInfo(IProjectFolder    projFolder,
		                                     out string        strSysName,
		                                     out string        strAppName,
		                                     out string        strConName,
		                                     out string        strUsrName,
		                                     out LanguageCode  eLangCode,
                                             out string        strDataConName,
                                             out string        strRowsetTreeClassPrefix,
                                             out string        strRowsetClassPrefix,
                                             out string        strRowsetPropertyPrefix,
                                             out string        strFldPropertyPrefix,
		                                     out string        strNameSpace,
                                             out string        strRootNameSpace,
		                                     out string        strRowsetTreeBaseClass,
		                                     out string        strRowsetBaseClass,
		                                     out bool          bGenWS) {
		    XmlDocument             xmlDoc;
		    XmlNode                 xmlNode;
		    StreamReader            streamReader;
		    
		    strSysName          = "";
		    strAppName          = "";
		    strConName          = "";
		    strUsrName          = "";
		    eLangCode           = LanguageCode.sdDefault;
            strDataConName      = "";
		    strRootNameSpace    = "";
            strNameSpace        = "";
		    strRowsetTreeBaseClass    = "";
		    strRowsetBaseClass   = "";
		    bGenWS              = false;
		    if (projFolder.IsFileExistInFolder(m_strConfigFileName)) {
		        using(streamReader = projFolder.OpenForReading(m_strConfigFileName)) {
		            xmlDoc              = new XmlDocument();
		            xmlDoc.Load(streamReader);
		            xmlNode                 = xmlDoc.DocumentElement;
		            strSysName              = GetInfo(xmlNode, "SystemName");
		            strAppName              = GetInfo(xmlNode, "ApplicationName");
		            strConName              = GetInfo(xmlNode, "ConnectionName");
		            strUsrName              = GetInfo(xmlNode, "UserName");
		            eLangCode               = (LanguageCode)Int32.Parse(GetInfo(xmlNode, "LanguageCode"));
                    if (RepPP.Version.SDVERNUM >= 800) {
                        strDataConName = GetInfo(xmlNode, "DataConnectionName");;
                    }
		            strRowsetTreeClassPrefix    = GetInfo(xmlNode,  "RowsetTreeClassPrefix", "RTC");
		            strRowsetClassPrefix        = GetInfo(xmlNode,  "RowsetClassPrefix",     "RC");
		            strRowsetPropertyPrefix     = GetInfo(xmlNode,  "RowsetPropPrefix",      "rs");
		            strFldPropertyPrefix        = GetInfo(xmlNode,  "FieldPropPrefix",       "fld");
		            strNameSpace                = GetInfo(xmlNode,  "NameSpace");
                    strRootNameSpace            = GetInfo(xmlNode,  "RootNameSpace");
		            strRowsetTreeBaseClass      = GetInfo(xmlNode,  "RowsetTreeBaseClass");
		            strRowsetBaseClass          = GetInfo(xmlNode,  "RowsetBaseClass");
		            bGenWS                      = (GetInfo(xmlNode, "GenerateWSStruct").ToLower() != "false");
		        }
		    } else {
		        throw new System.ApplicationException("Unable to find the configuration file - " + m_strConfigFileName);
		    }
		}

        //******************************************************************
        //
		/// <summary>
		/// CleanObsoleteTypeFiles:                 Clean obsolete file
		/// </summary>
		/// <param name="app">                      Rep++ application object</param>
		/// <param name="projFolder">               Interface with the project folder</param>
        /// <param name="strFileExt">               File extension</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        //
        //******************************************************************
		private void CleanObsoleteTypeFiles(RepPP.Application   app,
		                                    IProjectFolder      projFolder,
		                                    string              strFileExt,
                                            string              strRowsetTreeClassPrefix,
                                            string              strRowsetClassPrefix) {
		    string[]    arrFileName;
		    string      strRowsetTreeDefFileName;
		    string      strRowsetDefFileName;
		    string      str;

		    strRowsetTreeDefFileName    = BuildRowsetTreeFileName(strRowsetTreeClassPrefix, null, strFileExt).ToUpper();
		    strRowsetDefFileName        = BuildRowsetFileName(strRowsetClassPrefix, null, strFileExt).ToUpper();
		    arrFileName                 = projFolder.GetFileList();
		    foreach (string strFileName in arrFileName) {
		        str = strFileName.ToUpper();
		        if (str.EndsWith(strRowsetTreeDefFileName)) {
		            if (IsSignaturePresent(projFolder, strFileName)) {
		                str = str.Substring(0, str.LastIndexOf(strRowsetTreeDefFileName));
		                if (app.RowsetTreeDefs[str] == null) {
		                    projFolder.DeleteFile(strFileName);
		                }
		            }
		        } else if (str.EndsWith(strRowsetDefFileName)) {
		            if (IsSignaturePresent(projFolder, strFileName)) {
		                str = str.Substring(0, str.LastIndexOf(strRowsetDefFileName));
		                if (app.RowsetDefs[str] == null) {
		                    projFolder.DeleteFile(strFileName);
		                }
		            }
		        }
		    }
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all rowset trees and rowsets which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="strFileExt">           File extension (.vb, .cs)</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //******************************************************************
        protected virtual bool RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, string strFileExt, bool bFullSync, bool bUseWSDataContracts) {
		    bool            bRetVal = true;
		    string          strSysName;
		    string          strAppName;
		    string          strConName;
		    string          strUsrName;
		    LanguageCode    eLangCode;
            string          strDataConName;
            string          strRowsetTreeClassPrefix;
            string          strRowsetClassPrefix;
            string          strRowsetPropertyPrefix;
            string          strFldPropertyPrefix;
		    string          strNameSpace;
            string          strRootNameSpace;
		    string          strRowsetTreeBaseClass;
		    string          strRowsetBaseClass;
		    bool            bGenWS;
		    
		    ExtractConfigInfo(projFolder,
		                      out strSysName,
		                      out strAppName,
		                      out strConName,
		                      out strUsrName,
		                      out eLangCode,
                              out strDataConName,
                              out strRowsetTreeClassPrefix,
                              out strRowsetClassPrefix,
                              out strRowsetPropertyPrefix,
                              out strFldPropertyPrefix,
		                      out strNameSpace,
                              out strRootNameSpace,
		                      out strRowsetTreeBaseClass,
		                      out strRowsetBaseClass,
		                      out bGenWS);
            if (strNameSpace == String.Empty && strRootNameSpace == String.Empty) {
                System.Windows.Forms.MessageBox.Show("The project does not have a root or a default namespace and no namespace is specified for the typed instances. \r\nPlease specify a namespace for the typed instances.",
                                                     "Rep++ Typed Instances Add-In",
                                                     System.Windows.Forms.MessageBoxButtons.OK,
                                                     System.Windows.Forms.MessageBoxIcon.Error);
            } else {
		        if (app == null) {
                    app = RepPP.Application.GetApplication();
                    if (app != null) {
                         RepPP.Application.Reset();
                    }
                    app = RepPP.Application.GetApplicationForDesign(null);
		            if (app == null) {
		                throw new System.ApplicationException("Unable to initialize Rep++");
		            }
		        }
		        if (log != null) {
		            bRetVal = CheckRegenerateForNameClash(app,
		                                                  log,
		                                                  projFolder,
		                                                  strFileExt,
		                                                  strRowsetTreeClassPrefix,
		                                                  strRowsetClassPrefix,
		                                                  strRowsetPropertyPrefix,
		                                                  strFldPropertyPrefix,
		                                                  bFullSync);
		        }
		        if (bRetVal) {
	                RegenerateRowsetTrees(app,
	                                      projFolder,
	                                      strFileExt,
                                          strRowsetTreeClassPrefix,
                                          strRowsetClassPrefix,
                                          strRowsetPropertyPrefix,
                                          strFldPropertyPrefix,
                                          strNameSpace,
                                          strRootNameSpace,
	                                      strRowsetTreeBaseClass,
	                                      strRowsetBaseClass,
	                                      bGenWS,
                                          bUseWSDataContracts,
	                                      bFullSync);
	                RegenerateRowsets(app,
	                                  projFolder,
	                                  strFileExt,
                                      strRowsetTreeClassPrefix,
                                      strRowsetClassPrefix,
                                      strRowsetPropertyPrefix,
                                      strFldPropertyPrefix,
	                                  strNameSpace,
                                      strRootNameSpace,
	                                  strRowsetBaseClass,
	                                  bGenWS,
                                      bUseWSDataContracts,
	                                  bFullSync);
                    if (bFullSync) {		                         
	                    CleanObsoleteTypeFiles(app, projFolder, strFileExt, strRowsetTreeClassPrefix, strRowsetClassPrefix);
	                }
	            }
            }
            return(bRetVal);
		}

        //******************************************************************
        //
		/// <summary>
        /// RegenerateTypedInstance:            Regenerates all RowsetTrees and Rowsets which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        //
        //******************************************************************
        public abstract void RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts);

        //******************************************************************
        //
		/// <summary>
        /// RegenerateTypedInstance:            Regenerates all RowsetTrees and Rowsets which need to be regenerated
		/// </summary>
        /// <param name="log">                  Log object for name clash</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        //
        //******************************************************************
		public abstract void RegenerateTypedInstance(ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts);
	} // class TypedInstanceGenBase
} // namespace
