using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.IO;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections.Generic;

namespace RepPP.Toolkit {

    //********************************************************************************
    //
	/// <summary>
	/// Utility functions generics to thick or web clients
	/// </summary>
	//
    //********************************************************************************
	public class Util {

        //********************************************************************************
        //
        /// <summary>
        /// GetChildRowsetDefHierarchy:  Gets the hierarchy of nodes from the root to the specified leaf.
        /// </summary>
        /// <param name="node">         Leaf node of the rowset tree definition</param>
        /// <returns>
        /// Array of RowsetTreeDefNode representing the hierarchy ending with the leaf node
        /// </returns>
        //
        //********************************************************************************
        static public RowsetTreeDefNode[] GetChildRowsetDefHierarchy(RowsetTreeDefNode node) {
            RowsetTreeDefNode[] arrRetVal;
            string[]            arrNodeName;
            int                 iCount;

            arrNodeName = node.GetHierarchy().Split(',');
            iCount      = arrNodeName.Length;
            arrRetVal   = new RowsetTreeDefNode[iCount];
            for (int iIndex = 0; iIndex < iCount; iIndex++) {
                arrRetVal[iIndex] = node.RowsetTreeDef.FindNode(arrNodeName[iIndex]);
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildPrimaryKey:            Build the primary key of the ancester of the specified rowset
        /// </summary>
        /// <param name="rowsetDef">    Rowset</param>
        /// <param name="bExtValue">    true to build it with external value</param>
        /// <returns>
        /// Array of primary keys of the rowset
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildPrimaryKey(RowsetDef rowsetDef, bool bExtValue) {
            string[]    arrRetVal;
            ArrayList   arrTmp;
            
            arrTmp = new ArrayList();
            foreach (FieldDef fld in rowsetDef.FieldDefs) {
                if (fld.IsPrimaryKey) {
                    arrTmp.Add(bExtValue ? fld.ExtValue : fld.Value);
                }
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildFlatHierarchyPrimaryKey:   Build the primary key of the ancester of the specified node
        /// </summary>
        /// <param name="node">             Leaf node of the key</param>
        /// <param name="bIncludeSelf">     true to also include the key of the leaft itself</param>
        /// <param name="bExtValue">        true to build it with external value</param>
        /// <returns>
        /// Array of primary key
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildFlatHierarchyPrimaryKey(RowsetTreeDefNode node, bool bIncludeSelf, bool bExtValue) {
            string[]            arrRetVal;
            RowsetTreeDefNode[] arrRowsetTreeDefNode;
            ArrayList           arrTmp;
            
            if (!bIncludeSelf) {
                node = node.Parent as RowsetTreeDefNode;
            }
            arrRowsetTreeDefNode = GetChildRowsetDefHierarchy(node);
            arrTmp               = new ArrayList();
            foreach (RowsetTreeDefNode nodeNode in arrRowsetTreeDefNode) {
                arrTmp.AddRange(BuildPrimaryKey(nodeNode.RowsetDef, bExtValue));
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildHierarchyPrimaryKey:   Build the primary key of the ancester of the specified node
        /// </summary>
        /// <param name="node">         Leaf node of the key</param>
        /// <param name="bIncludeSelf"> true to also include the key of the node itself</param>
        /// <param name="bExtValue">    true to build it with external value</param>
        /// <returns>
        /// Array of RowsetTreeDefNodes representing the hierarchy terminating with the leaf card group
        /// </returns>
        //
        //********************************************************************************
        static public string[][] BuildHierarchyPrimaryKey(RowsetTreeDefNode node, bool bIncludeSelf, bool bExtValue) {
            string[][]          arrRetVal;
            RowsetTreeDefNode[] arrRowsetTreeDefNode;
            ArrayList           arrTmp;
            int                 iIndex;
            
            if (!bIncludeSelf) {
                node = node.Parent as RowsetTreeDefNode;
            }
            arrRowsetTreeDefNode = GetChildRowsetDefHierarchy(node);
            arrRetVal            = new string[arrRowsetTreeDefNode.Length][];
            arrTmp               = new ArrayList();
            iIndex               = 0;
            foreach (RowsetTreeDefNode nodeNode in arrRowsetTreeDefNode) {
                arrTmp.AddRange(BuildPrimaryKey(nodeNode.RowsetDef, bExtValue));
                arrRetVal[iIndex] = new string[arrTmp.Count];
                arrTmp.CopyTo(arrRetVal[iIndex]);
                arrTmp.Clear();
                iIndex++;
            }
            return(arrRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildPKWhereClause:                 Build a WHERE clause which select an item using its primary key
        /// </summary>
        /// <param name="node">                 Node</param>
        /// <param name="bBuildAncestralKey">   true to include ancestor's key</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        public static string BuildPKWhereClause(RepPP.RowsetTreeDefNode node, bool bBuildAncestralKey) {
            string  strRetVal;
            
            if (bBuildAncestralKey && node.Parent != null) {
                strRetVal = BuildPKWhereClause(node.Parent as RepPP.RowsetTreeDefNode, true);
            } else {
                strRetVal = String.Empty;
            }
            foreach (RepPP.FieldDef fldDef in node.RowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    if (strRetVal.Length == 0) {
                        strRetVal = "WHERE ";
                    } else {
                        strRetVal += " AND ";
                    }
                    strRetVal += fldDef.ColName + "=:" + fldDef.RowsetDef.Name + "." + fldDef.Name;
                }
            }
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildPKWhereClause:                 Build a WHERE clause which select an item using its primary key
        /// </summary>
        /// <param name="rowset">               Rowset</param>
        /// <param name="bBuildAncestralKey">   true to include ancestor's key</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        public static string BuildPKWhereClause(RepPP.Rowset rowset, bool bBuildAncestralKey) {
            string  strRetVal;
            
            if (bBuildAncestralKey && rowset.FatherRowset != null) {
                strRetVal = BuildPKWhereClause(rowset.FatherRowset, true);
            } else {
                strRetVal = String.Empty;
            }
            foreach (RepPP.FieldDef fldDef in rowset.RowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    if (strRetVal.Length == 0) {
                        strRetVal = "WHERE ";
                    } else {
                        strRetVal += " AND ";
                    }
                    strRetVal += fldDef.ColName + "=:" + fldDef.RowsetDef.Name + "." + fldDef.Name;
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildArrayOfValues:             Build an array of value of the fields of the active rowset of a definition
        /// </summary>
        /// <param name="rowsetDef">        Rowset definition of the key</param>
        /// <param name="bExtValue">        true to build it with external value</param>
        /// <returns>
        /// Array of primary keys of the rowset
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildArrayOfValues(RowsetDef rowsetDef, bool bExtValue) {
            ArrayList       arrTemp;
            string[]        arrRetVal;
            
            arrTemp = new ArrayList(rowsetDef.FieldDefs.Count);
            foreach (FieldDef fldDef in rowsetDef.FieldDefs) {
                if (fldDef.IsVisible) {
                    arrTemp.Add((bExtValue) ? fldDef.ExtValue : fldDef.Value);
                }
            }
            arrRetVal =  (string[])arrTemp.ToArray(typeof(string));
            return(arrRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadItemUsingPK:                    Reads an item using the primary key of the item
        /// </summary>
        /// <param name="rowsetTree">           Rowset tree</param>
        /// <param name="arrKeys">              Array of primary keys</param>
        /// <param name="bAppend">              true to append</param>
        /// <returns>
        /// Returned code
        /// </returns>
        //
        //******************************************************************
        public static int ReadItemUsingPK(RowsetTree rowsetTree, string[] arrKeys, bool bAppend) {
            int                 iRetVal;
            string              strOldWhereClause;
            RowsetTreeDefNode   nodeRoot;
            Rowset              rowsetRoot;
            int                 iIndex;
            
            nodeRoot            = rowsetTree.RowsetTreeDef.Nodes[0];
            rowsetRoot          = rowsetTree.RootRowset;
            strOldWhereClause   = nodeRoot.GetSelectSqlWhere();
            nodeRoot.SetSelectSqlWhere(BuildPKWhereClause(rowsetRoot, false));
            rowsetRoot.Empty();
            rowsetRoot.AddBlankLine();
            iIndex = 0;
            foreach (FieldDef fldDef in rowsetRoot.RowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    fldDef.ExtValue = arrKeys[iIndex++];
                }
            }
            iRetVal = rowsetTree.ReadFromDb(bAppend);
            nodeRoot.SetSelectSqlWhere(strOldWhereClause);
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FormatStringList:           Format a list of string (prefix str1 suffix,prefix str2 suffix,...)
        /// </summary>
        /// <param name="arrStr">       Array of string</param>
        /// <param name="strPrefix">    Prefix</param>
        /// <param name="strSuffix">    Suffix</param>
        /// <returns>
        /// Formated string (prefix str1 suffix,prefix str2 suffix,...)
        /// </returns>
        //
        //********************************************************************************
        static public string FormatStringList(ArrayList arrStr, string strPrefix, string strSuffix) {
            string strRetVal = String.Empty;;

            foreach (string str in arrStr) {
                if (strRetVal != String.Empty) {
                    strRetVal += ",";
                }
                strRetVal += strPrefix + str + strSuffix;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
		/// IsNumeric:                  Verifies that an object is numeric 
		/// </summary>
		/// <returns>
		/// True if the object is numeric
		/// </returns>
		//
        //********************************************************************************
        static public bool IsNumeric(object num) {
            double result = 0;

            return(Double.TryParse(num.ToString(), 
                                   System.Globalization.NumberStyles.Any, 
                                   System.Globalization.NumberFormatInfo.InvariantInfo, 
                                   out result));
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildPrimaryKeyFieldNames:      Build an array of the primary key field names
        /// </summary>
        /// <param name="rowsetDef">        Rowset definition</param>
        /// <returns>
        /// Array of primary key field names of the rowset definition
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildPrimaryKeyFieldNames(RowsetDef rowsetDef) {
            string[]        arrRetVal;
            ArrayList       arrTmp;
            
            arrTmp = new ArrayList();
            foreach (FieldDef fldDef in rowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    arrTmp.Add(fldDef.Name);
                }
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
		/// CopyFieldValues:            Sets the values of the destination fields that do
		///                             exist as source fields.  
		/// </summary>
        /// <param name="fldDefsSrc">   Source field definitions</param>
        /// <param name="fldDefsDest">  Destination field definitions</param>
		//
        //********************************************************************************
        static public void CopyFieldValues(FieldDefs fldDefsSrc, FieldDefs fldDefsDest) {
            foreach (FieldDef fldDef in fldDefsSrc) {
                if (fldDefsDest[fldDef.Name] != null){
                    fldDefsDest[fldDef.Name].Value = fldDef.Value;
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ConvertStringToDateTime:    Convert a string to a DateTime
        /// </summary>
        /// <param name="strVal">       Value to convert</param>
        /// <returns>
        /// DateTime or DateTime.MinValue if error
        /// </returns>
        //
        //*********************************************************
        static public DateTime ConvertStringToDateTime(string strVal) {
            DateTime    dateTimeRetVal = DateTime.MinValue;
            string[]    arrDayTime;
            string[]    arrStr;
            int         iYear;
            int         iMonth;
            int         iDay;
            int         iHour;
            int         iMin;
            int         iSec;

            if (strVal.Length > 9) {
                arrDayTime = strVal.Split(' ');
                if (arrDayTime[0].Length == 10) {
                    arrStr = arrDayTime[0].Split(arrDayTime[0][4]);
                    if (arrStr.Length == 3) {
                        iYear  = Int32.Parse(arrStr[0]);
                        iMonth = Int32.Parse(arrStr[1]);
                        iDay   = Int32.Parse(arrStr[2]);
                        iHour  = 0;
                        iMin   = 0;
                        iSec   = 0;
                        if (arrDayTime.Length > 1) {
                            arrStr = arrDayTime[1].Split(arrDayTime[1][2]);
                            if (arrStr.Length == 3) {
                                iHour   = Int32.Parse(arrStr[0]);
                                iMin    = Int32.Parse(arrStr[1]);
                                iSec    = Int32.Parse(arrStr[2]);
                            }
                        }
                        dateTimeRetVal = new DateTime(iYear, iMonth, iDay, iHour, iMin, iSec);
                    }
                }
            }
            return(dateTimeRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsDirectSearchField:            Checks if a direct search can be performed on
        ///                                 the specified field
        /// </summary>
        /// <param name="fldDef">           Field definition</param>
        /// <returns>
        /// true/false
        /// </returns>
        //
        //********************************************************************************
        public static bool IsDirectSearchField(FieldDef fldDef) {
            RepPP.FieldDef  fldDefAlt;
            bool            bRetVal;
            
            bRetVal = (fldDef != null                       && 
                       fldDef.IsVisible                     && 
                       fldDef.IsDatabaseLinked              && 
                       fldDef.Type != FieldType.sdFieldMemo && 
                       (fldDef.Attributes["ISNOTDIRSRCHFLD"] == null || fldDef.Attributes["ISNOTDIRSRCHFLD"].Value != "0"));
            if (!bRetVal && fldDef != null) {
                fldDefAlt = SBFilterBase.GetAlternateFilterField(fldDef);
                if (fldDefAlt != null) {
                    bRetVal = IsDirectSearchField(fldDefAlt);    
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetDirectSearchFieldPrompt:     Gets the prompt of the specified direct search
        ///                                 field
        /// </summary>
        /// <param name="fldDef">           Field definition</param>
        /// <returns>
        /// Field prompt
        /// </returns>
        //
        //********************************************************************************
        public static string GetDirectSearchFieldPrompt(FieldDef fldDef) {
            RepPP.Attribute attPrompt;
            string          strRetVal = null;
            
            attPrompt = fldDef.Attributes[((int)fldDef.Application.LanguageCode).ToString()];
            if (attPrompt != null && !string.IsNullOrEmpty(attPrompt.Value)) {
                strRetVal = attPrompt.Value;
            } else {
                strRetVal = fldDef.Prompt;
            }
            return(strRetVal);
        }

	    //******************************************************************
	    //
        /// <summary>
        /// DecorateWord:               Decorate a word to prevent conflicts
        ///                             with reserved words
        /// </summary>
        /// <param name="conn">         Database connection</param>
        /// <param name="strWord">      Word</param>
        /// <returns>
        /// Decorated word
        /// </returns>
        //
	    //******************************************************************
        public static string DecorateWord(RepPP.Connection conn, string strWord) {
            if (conn.IsReservedKeyword(strWord)) {
                strWord = conn.DecorateReservedKeyword(strWord);
            }
            return(strWord);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetRowsetChildrenState      Sets the line state of all
        ///                             the child rowsets
        /// </summary>
        /// <param name="rowsetRoot">   Parent Rowset</param>
        /// <param name="eLineState">   Line state to set</param>
        //
        //*********************************************************     
        public static void SetRowsetChildrenState(Rowset rowsetRoot, LineState eLineState) {
            int iLineCount;

            foreach (Rowset rsChild in rowsetRoot.ChildRowsets) {
                SetRowsetChildrenState(rsChild, eLineState);
                try {
                    rsChild.SaveLinePos();
                    iLineCount = rsChild.LineCount;
                    for (int i = 0; i < iLineCount; i++) {
                        rsChild.SelectLine(i, false, false);    
                        rsChild.LineState = eLineState; 
                    }
                } finally {
                    rsChild.RestoreLinePos(false);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Handle selectable state of fields of a group
        /// </summary>
        //
        //********************************************************************************
        public class FldSelectableState : IDisposable {
            /// <summary>
            /// Represent the state of the fields of a group
            /// </summary>
            public enum SelStateE {
                /// <summary>No fields for this action should be enabled</summary>
                SS_None     = 0,
                /// <summary>All fields besided primary key for this action should be enabled</summary>
                SS_NoPK     = 1,
                /// <summary>All fields for this action should be enabled</summary>
                SS_All      = 2
            };

            static private Dictionary<String, bool[]>   m_dictOriginalStates;       // Dictionary of original selectable states per group
            private bool[]                              m_arrPrevFieldStates;       // Previous selectable state of the field
            private bool[]                              m_arrOriginalFieldStates;   // Original field states
            private RepPP.RowsetDef                     m_rowsetDef;                // Rowset definition
            private SelStateE                           m_eState;                   // Current state

            //********************************************************************************
            //
            /// <summary>
            /// Static constructor
            /// </summary>
            //
            //********************************************************************************
            static FldSelectableState() {
                m_dictOriginalStates = new Dictionary<string,bool[]>(32);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="rowsetDef">            RowsetDef containing the field</param>
            //
            //********************************************************************************
            public FldSelectableState(RepPP.RowsetDef rowsetDef) {
                int     iIndex;
                String  strName;
                bool[]  arrOriginalStates;
                
                strName = rowsetDef.Name;
                if (m_dictOriginalStates.ContainsKey(strName) && m_dictOriginalStates[strName].Length != rowsetDef.FieldDefs.Count) {
                    m_dictOriginalStates.Remove(strName);
                }
                if (m_dictOriginalStates.ContainsKey(strName)) {
                    arrOriginalStates = m_dictOriginalStates[strName];
                } else {
                    arrOriginalStates = new bool[rowsetDef.FieldDefs.Count];
                    iIndex            = 0;
                    foreach (RepPP.FieldDef fldDef in rowsetDef.FieldDefs) {
                        arrOriginalStates[iIndex++] = fldDef.IsSelectable;
                    }
                    m_dictOriginalStates.Add(strName, arrOriginalStates);
                }
                m_rowsetDef              = rowsetDef;
                m_arrPrevFieldStates     = new bool[rowsetDef.FieldDefs.Count];
                m_arrOriginalFieldStates = arrOriginalStates;
                iIndex                   = 0;
                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                    m_arrPrevFieldStates[iIndex]    = fldDef.IsSelectable;
                    fldDef.IsSelectable             = arrOriginalStates[iIndex];
                    iIndex++;
                }
                m_eState = SelStateE.SS_All;
            }

            //********************************************************************************
            //
            /// <summary>
            /// Dispose:                        Restore original field state
            /// </summary>
            //
            //********************************************************************************
            public void Dispose() {
                int     iIndex;
                
                iIndex = 0;
                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                    fldDef.IsSelectable = m_arrPrevFieldStates[iIndex++];
                }
                m_rowsetDef = null;
            }

            //********************************************************************************
            //
            /// <summary>
            /// State:                          Current state
            /// </summary>
            //
            //********************************************************************************
            public SelStateE State {
                get {
                    return(m_eState);
                }
                set {
                    int iIndex;
                    
                    if (m_eState != value) {
                        m_eState = value;
                        iIndex   = 0;
                        foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {                            
                            switch(m_eState) {
                            case SelStateE.SS_All:
                                fldDef.IsSelectable = m_arrOriginalFieldStates[iIndex];
                                break;
                            case SelStateE.SS_NoPK:
                                fldDef.IsSelectable = (fldDef.IsPrimaryKey) ? false : m_arrOriginalFieldStates[iIndex];
                                break;
                            default:
                                fldDef.IsSelectable = false;
                                break;
                            }
                            iIndex++;
                        }
                    }
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// SetFieldProtection:             Sets the field protection for one field
            /// </summary>
            /// <param name="fldDefToChange">   Field Def that will be changed</param>
            //
            //********************************************************************************
            public void SetFieldProtection(RepPP.FieldDef fldDefToChange) {
                int iIndex = 0;
                
                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {                            
                    if (fldDef == fldDefToChange) {
                        switch(m_eState) {
                        case SelStateE.SS_All:
                            fldDef.IsSelectable = m_arrOriginalFieldStates[iIndex];
                            break;
                        case SelStateE.SS_NoPK:
                            fldDef.IsSelectable = (fldDef.IsPrimaryKey) ? false : m_arrOriginalFieldStates[iIndex];
                            break;
                        default:
                            fldDef.IsSelectable = false;
                            break;
                        }
                    }
                    iIndex++;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// UpdateOriginalSelectableState:  Updates the original selectable states
            /// </summary>
            /// <param name="fldDefs">          List of fields to be updated</param>
            //
            //********************************************************************************
            public void UpdateOriginalSelectableState(params RepPP.FieldDef[] fldDefs) {
                int     iIndex = 0;

                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                    if (Array.IndexOf(fldDefs, fldDef) != -1) {
                        m_arrOriginalFieldStates[iIndex] = fldDef.IsSelectable;
                    }
                    iIndex++;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// SetOriginalFieldProtection:    Set the original field protection for a specified field
            /// </summary>
            /// <param name="strName">         FieldDef Name</param>
            //  
            //********************************************************************************
            public void SetOriginalFieldProtection(string strName) {
                FieldDefs   fldDefs;
                int         iFldCount;

                fldDefs      = m_rowsetDef.FieldDefs;  
                iFldCount    = fldDefs.Count;
                for (int i = 0; i < iFldCount ; i++) {
                    if (fldDefs[i].Name == strName) {
                        m_arrOriginalFieldStates[i] = fldDefs[i].IsSelectable;
                    }
                }
            }
        } // Class FldSelectableState
	} // Class Util
}
