using System;
using RepPP.Toolkit;
using RepPP.Toolkit.CodeGen;

namespace RepPP.Toolkit {

    //********************************************************************************
    //
	/// <summary>
	/// Base class for Web Services object generators
	/// </summary>
	//
    //********************************************************************************
	public abstract class WebServicesGenBase {
	    private string      m_strFldPropPrefix;
	    private string      m_strRowsetPropPrefix;
	    private string      m_strRowsetTreeClassPrefix;
        private string      m_strRowsetClassPrefix;
	    private bool        m_bAllPropAsString;
	    private bool        m_bAddLineState;

        /// <summary>Prefix for the generated methods and classes </summary>
        public const string                                 WEB_SERV_METHOD_PREFIX          = "WS_";
        /// <summary>Prefix for the generated methods and classes </summary>
        public const string                                 WEB_SERV_CARD_PREFIX            = "WSC";
        /// <summary>Prefix for the generated methods and classes </summary>
        public const string                                 WEB_SERV_GROUP_PREFIX           = "WSG";
        /// <summary>Web services class name </summary>
        public const string                                 WEB_SERV_CLASS_NAME             = "WebServices";
        /// <summary>Manager of Comments in Resources</summary>
        protected TypedInstanceGenBase.CommentResManager    m_cmtResManager;

        //*********************************************************	    
        //
	    /// <summary>
	    /// ctor:           Class constructor
	    /// </summary>
        //
        //*********************************************************
		public WebServicesGenBase() {
            m_cmtResManager         = new TypedInstanceGenBase.CommentResManager();
            RowsetTreeClassPrefix   = WEB_SERV_CARD_PREFIX;
            RowsetClassPrefix       = WEB_SERV_GROUP_PREFIX;
            FieldPropertyPrefix     = "fld";
            RowsetPropertyPrefix    = "gs";
            AllPropertiesAsString   = true;
            AddLineState            = true;            
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// RowsetTreeClassPrefix:  Prefix use for the name of a class representing a rowset tree
	    /// </summary>
        //
        //*********************************************************
		public virtual string RowsetTreeClassPrefix {
		    get {
		        return(m_strRowsetTreeClassPrefix);
		    }
		    set {
		        m_strRowsetTreeClassPrefix = (value == null) ? "" : value;
		    }
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// RowsetClassPrefix:    Prefix use for the name of a class representing a rowset
	    /// </summary>
        //
        //*********************************************************
		public virtual string RowsetClassPrefix {
		    get {
		        return(m_strRowsetClassPrefix);
		    }
		    set {
		        m_strRowsetClassPrefix = (value == null) ? "" : value;
		    }
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// FieldPropertyPrefix:    Prefix to use to generates property attached to Rep++ field
	    /// </summary>
        //
        //*********************************************************
		public virtual string FieldPropertyPrefix {
		    get {
		        return(m_strFldPropPrefix);
		    }
		    set {
		        m_strFldPropPrefix = (value == null) ? "" : value;
		    }
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// RowsetPropertyPrefix:    Prefix to use to generates property attached to Rep++ rowset
	    /// </summary>
        //
        //*********************************************************
		public virtual string RowsetPropertyPrefix {
		    get {
		        return(m_strRowsetPropPrefix);
		    }
		    set {
		        m_strRowsetPropPrefix = (value == null) ? "" : value;
		    }
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// AllPropertiesAsString:   true if all properties must be of type string.
	    /// </summary>
        //
        //*********************************************************
		public virtual bool AllPropertiesAsString {
		    get {
		        return(m_bAllPropAsString);
		    }
		    set {
		        m_bAllPropAsString = value;
		    }
		}

        //*********************************************************	    
        //
	    /// <summary>
	    /// AddLineState:           true to add a property/field for holding the line state
	    /// </summary>
        //
        //*********************************************************
		public virtual bool AddLineState {
		    get {
		        return(m_bAddLineState);
		    }
		    set {
		        m_bAddLineState = value;
		    }
		}

        //******************************************************************
        //
        /// CommentManager:                     Manager of Comments
        ///                                     generated in web services
        //
        //******************************************************************
        public TypedInstanceGenBase.CommentResManager CommentManager {
            get {
                return(m_cmtResManager);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateFieldPropName:  Generates the name of a property attached to a Rep++ field.
        /// </summary>
        /// <param name="fld">      Rep++ field</param>
        /// <returns>
        /// Property name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateFieldPropName(RepPP.FieldDef fld) {
            string  strFldName;

            strFldName = fld.AlternateName.Replace("$", "");
            return(FieldPropertyPrefix + strFldName);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateRowsetName:         Generates the name of a Rep++ rowset.
        /// </summary>
        /// <param name="rowsetDef">    Rep++ rowset definition</param>
        /// <returns>
        /// Property name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateRowsetName(RepPP.RowsetDef rowsetDef) {
            return(rowsetDef.AlternateName.Replace("$", ""));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateRowsetPropName:     Generates the name of a property attached to a Rep++ rowset.
        /// </summary>
        /// <param name="rowsetDef">    Rep++ rowset definition</param>
        /// <returns>
        /// Property name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateRowsetPropName(RepPP.RowsetDef rowsetDef) {
            string  strRowsetName;

            strRowsetName = GenerateRowsetName(rowsetDef);
            return(RowsetPropertyPrefix + strRowsetName);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateNodeRowsetName:     Generates the name of a Rep++ rowset tree definition node.
        /// </summary>
        /// <param name="node">         Rep++ rowset tree definition node</param>
        /// <returns>
        /// Name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateNodeRowsetName(RepPP.RowsetTreeDefNode node) {
            return(node.AlternateName.Replace("$", ""));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateChildRowsetPropName:    Generates the name of a property attached to a Rep++ rowset tree definition node.
        /// </summary>
        /// <param name="node">             Rep++ rowset tree definition node</param>
        /// <returns>
        /// Property name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateChildRowsetPropName(RepPP.RowsetTreeDefNode node) {
            string  strRowsetName;

            strRowsetName = GenerateNodeRowsetName(node);
            return(RowsetPropertyPrefix + strRowsetName);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateRowsetTreeClassName:    Generates the name of a class attached to a Rep++ rowset tree.
        /// </summary>
        /// <param name="rowsetTreeDef">    Rep++ RowsetTreeDef</param>
        /// <returns>
        /// Class name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateRowsetTreeClassName(RepPP.RowsetTreeDef rowsetTreeDef) {
            string  strRowsetTreeName;

            strRowsetTreeName = rowsetTreeDef.AlternateName.Replace("$", "");
            return(RowsetTreeClassPrefix + strRowsetTreeName);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateRowsetClassName:    Generates the name of a class attached to a Rep++ rowset.
        /// </summary>
        /// <param name="node">         RowsetTreeDef node if any</param>
        /// <param name="rowsetDef">    Rep++ rowset definition</param>
        /// <returns>
        /// Class name
        /// </returns>
        //
        //*********************************************************
        protected virtual string GenerateRowsetClassName(RepPP.RowsetTreeDefNode node, RepPP.RowsetDef rowsetDef) {
            string          strRetVal;
            //RepPP.RowsetTreeDefNode nodeParent;
            
            if (node != null) {
                strRetVal  = RowsetClassPrefix + GenerateNodeRowsetName(node);
            //    nodeParent = node.Parent as RepPP.RowsetTreeDefNode;
            //    while (nodeParent != null) {
            //        strRetVal  = GenerateNodeRowsetName(nodeParent) + "_" + strRetVal;
            //        nodeParent = nodeParent.Parent as RepPP.RowsetTreeDefNode;
            //    }
            //    strRetVal = GenerateRowsetTreeClassName(node.RowsetTreeDef) + "_" + strRetVal;
            } else {
                strRetVal = RowsetClassPrefix + GenerateRowsetName(rowsetDef);
            }
            return(strRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GenerateRowsetBaseClassName:                    Generates the base class name of a class attached 
        ///                                                 to a Rep++ rowset tree node.
        /// </summary>
        /// <param name="node">                             RowsetTreeDef node</param>
        /// <param name="strTypedInstRowsetClassPrefix">    Prefix for Typed instance rowset class</param>
        /// <returns>
        /// Class name
        /// </returns>
        //
        //*********************************************************
        protected string GenerateRowsetBaseClassName(RepPP.RowsetTreeDefNode node, string strTypedInstRowsetClassPrefix) {
            string  strRetVal;
            string  strRowsetName;

            strRowsetName = GenerateRowsetName(node.RowsetDef);
            strRetVal     = strTypedInstRowsetClassPrefix + strRowsetName + "." + WEB_SERV_CLASS_NAME + "." + RowsetClassPrefix + strRowsetName;
            return(strRetVal);
        }

        //*********************************************************	    
        //
	    /// <summary>
	    /// PurifyRemark:               Be sure a remark doesn't contain illegal character
	    /// </summary>
	    /// <param name="strRem">       Remark</param>
	    /// <returns>
	    /// Purified remark
	    /// </returns>
        //
        //*********************************************************
        protected string PurifyRemark(string strRem) {
            string  strRetVal;
            
            strRetVal = strRem.Replace('<', ' ').Replace('>', ' ').Replace("///", "");
            return(strRetVal);
        }

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetClass:                        Generates a class to use in Web Service to represent a rowset
	    /// </summary>
	    /// <param name="rowsetDef">                    Rep++ group</param>
	    /// <param name="iIndent">                      Indentation to use</param>
	    /// <param name="strBaseClass">                 Base class</param>
        /// <param name="strTypedInstRowsetClassPrefix">Prefix for Typed instance rowset class</param>
        /// <param name="bUseWSDataContracts">          true to use DataContract</param>
        //
        //*********************************************************
        public abstract string GenerateRowsetClass(RepPP.RowsetDef rowsetDef, int iIndent, string strBaseClass, string strTypedInstRowsetClassPrefix, bool bUseWSDataContracts);

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetTreeClass:                    Generates a class to use in Web Service to represent a rowset tree
	    /// </summary>
	    /// <param name="rowsetTreeDef">                Rep++ rowset tree definition</param>
	    /// <param name="iIndent">                      Indentation to use</param>
	    /// <param name="strRowsetTreeDefBaseClass">    RowsetTreeDef Base class</param>
	    /// <param name="strRowsetDefBaseClass">        RowsetDef Base class</param>
        /// <param name="strTypedInstRowsetClassPrefix">Prefix for Typed instance rowset class</param>
        /// <param name="bUseWSDataContracts">          true to use DataContract</param>
        //
        //*********************************************************
        public abstract string GenerateRowsetTreeClass(RepPP.RowsetTreeDef rowsetTreeDef, int iIndent, string strRowsetTreeDefBaseClass, string strRowsetDefBaseClass, string strTypedInstRowsetClassPrefix, bool bUseWSDataContracts);

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetSerializationFunctions:   Generates serialization function for the rowset
	    /// </summary>
	    /// <param name="strContainerClassName">    Container class name</param>
	    /// <param name="node">                     Rep++ rowset tree definition node</param>
	    /// <param name="rowsetDef">                Rep++ rowset definition</param>
	    /// <param name="iIndent">                  Indentation to use</param>
	    /// <returns>
	    /// Generated functions
	    /// </returns>
	    //
        //*********************************************************
        public abstract string GenerateRowsetSerializationFunctions(string strContainerClassName, RepPP.RowsetTreeDefNode node, RepPP.RowsetDef rowsetDef, int iIndent);

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetTreeSerializationFunctions:   Generates serialization function for the rowset tree
	    /// </summary>
	    /// <param name="strContainerClassName">        Container class name</param>
	    /// <param name="rowsetTreeDef">                Rep++ rowset tree</param>
	    /// <param name="iIndent">                      Indentation to use</param>
	    /// <returns>
	    /// Generated functions
	    /// </returns>
	    //
        //*********************************************************
        public abstract string GenerateRowsetTreeSerializationFunctions(string strContainerClassName, RepPP.RowsetTreeDef rowsetTreeDef, int iIndent);

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetCode:                         Generates Web services support code for the rowset
	    /// </summary>
	    /// <param name="strContainerClassName">        Container class name</param>
	    /// <param name="node">                         Rep++ rowset tree definition node</param>
	    /// <param name="rowsetDef">                    Rep++ rowset definition</param>
	    /// <param name="iIndent">                      Indentation to use</param>
	    /// <param name="strBaseClass">                 Base class</param>
        /// <param name="strTypedInstRowsetClassPrefix">Prefix for Typed instance rowset class</param>
        /// <param name="bUseWSDataContracts">          true to use DataContract</param>
	    /// <returns>
	    /// Generated code
	    /// </returns>
	    //
        //*********************************************************
        public abstract string GenerateRowsetCode(string                    strContainerClassName,
                                                 RepPP.RowsetTreeDefNode    node,
                                                 RepPP.RowsetDef            rowsetDef,
                                                 int                        iIndent,
                                                 string                     strBaseClass,
                                                 string                     strTypedInstRowsetClassPrefix,
                                                 bool                       bUseWSDataContracts);

        //*********************************************************	    
        //
	    /// <summary>
	    /// GenerateRowsetTreeCode:                     Generates Web services support code for the RowsetTreeDef
	    /// </summary>
	    /// <param name="strContainerClassName">        Container class name</param>
	    /// <param name="rowsetTreeDef">                Rep++ RowsetTreeDef</param>
	    /// <param name="iIndent">                      Indentation to use</param>
	    /// <param name="strRowsetTreeDefBaseClass">    RowsetTreeDef Base class</param>
	    /// <param name="strRowsetDefBaseClass">        RowsetDef Base class</param>
        /// <param name="strTypedInstRowsetClassPrefix">Prefix for Typed instance group class</param>
        /// <param name="bUseWSDataContracts">          true to use DataContract</param>
        /// <returns>
	    /// Generated code
	    /// </returns>
	    //
        //*********************************************************
        public abstract string GenerateRowsetTreeCode(string                strContainerClassName, 
                                                      RepPP.RowsetTreeDef   rowsetTreeDef,
                                                      int                   iIndent,
                                                      string                strRowsetTreeDefBaseClass,
                                                      string                strRowsetDefBaseClass,
                                                      string                strTypedInstRowsetClassPrefix,
                                                      bool                  bUseWSDataContracts);
	}
}
