using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.IO;
using System.Reflection;
using System.Collections;


namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
	/// <summary>
	/// Implementation of a context menu control
	/// </summary>
	//
    //********************************************************************************
	[ParseChildren(true, "Items")]
	[Designer(typeof(RepPP.Toolkit.Web.Design.ContextMenuDesign))]
	[DefaultEvent("Click")]
	[DefaultProperty("Items")]
    [ToolboxData("<{0}:ContextMenu runat=server></{0}:ContextMenu>")]
	public class ContextMenu : WebControl, INamingContainer, IPostBackEventHandler {
		#region Constants
        /// <summary>HTML representation of a seperator menu item</summary>
		private const string m_cstrHrSeparator = "<hr style='height:1px;border:solid 1px black;' />";
        /// <summary>The script that is injected to bound controls</summary>
        private const string m_cstrAttachContextMenu = "return ContextMenu_showContextMenu(event,'{0}');";
        /// <summary>The script that is injected to bound controls</summary>
        private const string m_cstrAttachContextMenuWithRef = "return ContextMenu_showContextMenu(event,'{0}', {1});";
        /// <summary>Script used (usually on body) to hide "NON AUTO HIDE" menus using the escape key</summary>
        private const string m_cstrTrapEscKey = "ContextMenu_trapESC({0})";
        /// <summary>Script used (usually on body) to hide "NON AUTO HIDE" menus using a mouse click</summary>
        private const string m_cstrHideOnClick = "document.getElementById('{0}').style.display = 'none';";
        /// <summary>script to activate a menu item</summary>
        private const string m_cstrOnMouseOver = "this.style.background = '{0}';";
        /// <summary>script to deactivate a menu item</summary>
        private const string m_cstrOnMouseOut = "this.style.background = '{0}';";
        /// <summary>script to activate a menu item</summary>
        private const string m_cstrOnMouseOverChangeClass = "this.className = '{0}';";
        /// <summary>script to deactivate a menu item</summary>
        private const string m_cstrOnMouseOutChangeClass = "this.className = '{0}';";
		#endregion

		#region Class Variables
        /// <summary>The list of menu items</summary>
		ContextMenuItemCollection       m_Items;
        /// <summary>The list of bound controls (automates event handeling)</summary>
        ArrayList                       m_boundControls;
        /// <summary>a js reference to the element that invoked the ContextMenu</summary>
        string                          m_strCallerReference;
		#endregion

		#region Events
		/// <summary>Event that fires whenever an item with a non empty command argument is clicked</summary>
		public event CommandEventHandler Click;
		#endregion

		#region Properties
        //********************************************************************************
        //
        /// <summary>
        /// Items :                 Gets the collection of the menu items
        /// </summary>
        //
        //********************************************************************************
        [Category("Data")]
        [DefaultValue(null)]
        [NotifyParentProperty(true)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		[PersistenceMode(PersistenceMode.InnerDefaultProperty)]
		[Description("Gets the collection of the menu items")]
		public virtual ContextMenuItemCollection Items {
			get {
                if (m_Items == null) {
                    m_Items = new ContextMenuItemCollection();
                }
				return(m_Items);
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// BoundControls :               Gets the collection of controls for 
        ///                               which the context menu should be displayed 
        /// </summary>
        //
        //********************************************************************************
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		[Description("Gets the collection of controls for which the context menu should be displayed")]
		public virtual ArrayList BoundControls {
			get {
                if (m_boundControls == null) {
                    m_boundControls = new ArrayList();
                }
				return(m_boundControls);
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// RolloverColor :               Gets and sets the background color when 
        ///                               the mouse hovers over the menu item
        /// </summary>
        //
        //********************************************************************************
		[Description("Gets and sets the background color when the mouse hovers over the menu item")]
		public virtual Color RolloverColor {
			get {
                Color   colorRetVal;
				object  o           = ViewState["RolloverColor"];

                if (o == null) {
                    colorRetVal = Color.Gold;
                } else {
                    colorRetVal = (Color) o;
                }
				return(colorRetVal); 
			}
			set {
                ViewState["RolloverColor"] = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// RolloverClass :               Gets and sets the css class name of when 
        ///                               the mouse hovers over the menu item
        /// </summary>
        //
        //********************************************************************************
		[Description("Gets and sets the css class name of when the mouse hovers over the menu item")]
		public virtual string RolloverClass {
			get {
                string  strRetVal;
				object  o           = ViewState["RolloverClass"];

                if (o == null) {
                    strRetVal = string.Empty;
                } else {
                    strRetVal = o.ToString();
                }
				return(strRetVal); 
			}
			set {
                ViewState["RolloverClass"] = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// CellPadding :               Gets or Sets the pixels around each menu item 
        /// </summary>
        //
        //********************************************************************************
		[Description("The space in pixels around each menu item")]
		public virtual int CellPadding {
			get {
				object  o       = ViewState["CellPadding"];
                int     iRetVal;

                if (o == null) {
                    iRetVal = 2;
                } else {
                    iRetVal = (int)o;
                }
				return(iRetVal);
			}
			set {
				ViewState["CellPadding"] = value;
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// AutoHide :                  whether the context menu should be 
        ///                             dismissed as the user moves out
        /// </summary>
        //
        //********************************************************************************
        [Description("Indicates whether the context menu should be dismissed as the user moves out")]
        public virtual bool AutoHide {
            get {
                object  o       = ViewState["AutoHide"];
                bool    bRetVal;

                if (o == null) {
                    bRetVal = true;
                } else {
                    bRetVal = (bool) o;
                }
                return(bRetVal);
            }
            set {
                ViewState["AutoHide"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CallerReference :           a javascript reference to the element that invoked
        ///                             the context menu
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Browsable(true)]
        [Description("a javascript reference to the element that invoked the context menu e.g. this.parentNode")]
        public virtual string CallerReference {
            get {
                return(m_strCallerReference);
            }
            set {
                m_strCallerReference = value;
            }
        }

        #endregion

		#region Methods
        //********************************************************************************
        //
        /// <summary>
        /// GetMenuOpenScript:           Returns the Javascript code to attach the
        ///                             context menu to a HTML element
        /// </summary>
        //
        //********************************************************************************
        public string GetMenuOpenScript() {
            string strRetVal;

            if (CallerReference == null ||CallerReference == string.Empty) {
                strRetVal = String.Format(m_cstrAttachContextMenu, 
                                          Controls[0].ClientID);
            } else {
                strRetVal = String.Format(m_cstrAttachContextMenuWithRef,
                                          Controls[0].ClientID,
                                          CallerReference);
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetEscReference:            Returns the Javascript code to dismiss 
        ///                             the context menu when the user hits ESC
        /// </summary>
        //
        //********************************************************************************
		public string GetEscReference() {
			return(String.Format(m_cstrTrapEscKey, Controls[0].ClientID));
		}

        //********************************************************************************
        //
        /// <summary>
        /// GetOnClickReference:        Returns the Javascript code to dismiss
        ///                             the context menu when the user clicks outside the menu
        /// </summary>
        //
        //********************************************************************************
		public string GetOnClickReference() {
			return(String.Format(m_cstrHideOnClick, Controls[0].ClientID));
		}

		#endregion

		#region Rendering
        //********************************************************************************
        //
        /// <summary>
        /// EnsureDefaultStyles:        Sets the standard style of the control
        /// </summary>
        //
        //********************************************************************************
        private void EnsureDefaultStyles() {
            Style styleBase     =  ControlStyle;

            if (styleBase.CssClass == string.Empty          &&
                styleBase.BorderStyle == BorderStyle.NotSet &&
                styleBase.BorderColor.IsEmpty               &&
                styleBase.BorderWidth.IsEmpty               &&
                styleBase.BackColor.IsEmpty                 &&
                styleBase.ForeColor.IsEmpty                 &&
                styleBase.Font.Names.Length == 0            &&
                styleBase.Font.Size.IsEmpty) {
                styleBase.BorderStyle   = BorderStyle.Outset;
                styleBase.BorderWidth   = Unit.Pixel(2);
                styleBase.BorderColor   = Color.FromName("#C3DAF9");
                styleBase.BackColor     = Color.FromName("#C3DAF9");
                styleBase.ForeColor     = Color.Black;
                styleBase.Font.Names    = new string[]{"arial", "helvetica", "sans-serif"};
                styleBase.Font.Size     = FontUnit.Point(8);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateChildControls :          Overridden to build the UI of the control
        /// </summary>
        //
        //********************************************************************************
		protected override void CreateChildControls()
		{
            System.Web.UI.WebControls.Image imgMenuItem;
            HtmlGenericControl              ctlDIV;
			Table                           tblMenu;
            TableRow                        rowMenuItem;
            TableCell                       celMenuItemContainer;
            TableCell                       celImageContainer;
            LiteralControl                  lblMenuItem;

            Controls.Clear();
			ctlDIV                      = new HtmlGenericControl("div");
			ctlDIV.ID                   = "div";
			ctlDIV.Style["Display"]     = "none";
            ctlDIV.Style["z-index"]     = "999";
            ctlDIV.Style["position"]    = "absolute";
            ctlDIV.Style.Remove("left");
            ctlDIV.Style.Remove("top");
            if (AutoHide) {
                ctlDIV.Attributes["onmouseout"] = "OnMouseLeft(event, this)";
            }
            Controls.Add(ctlDIV);
            tblMenu = new Table();
			tblMenu.ApplyStyle(ControlStyle);
            tblMenu.CellSpacing = 0;
            tblMenu.CellPadding = CellPadding;
			ctlDIV.Controls.Add(tblMenu);
			foreach(ContextMenuItem item in Items) {
				rowMenuItem = new TableRow();
				tblMenu.Rows.Add(rowMenuItem);
                celImageContainer = new TableCell();
                rowMenuItem.Cells.Add(celImageContainer);
                celMenuItemContainer = new TableCell();
                celMenuItemContainer.Wrap = false;
                rowMenuItem.Cells.Add(celMenuItemContainer);
                if((item.Text == String.Empty) || (item.Text == null)) {
                    celImageContainer.ColumnSpan = 2;
					celImageContainer.Controls.Add(new LiteralControl(ContextMenu.m_cstrHrSeparator));
                    rowMenuItem.Cells.Remove(celMenuItemContainer);
				} else {
                    if (RolloverClass != string.Empty) {
					    rowMenuItem.Attributes["onmouseover"]= String.Format(ContextMenu.m_cstrOnMouseOverChangeClass, RolloverClass);
    					rowMenuItem.Attributes["onmouseout"] = String.Format(ContextMenu.m_cstrOnMouseOutChangeClass, "");
                    } else {
					    rowMenuItem.Attributes["onmouseover"]= String.Format(ContextMenu.m_cstrOnMouseOver, ColorTranslator.ToHtml(RolloverColor));
    					rowMenuItem.Attributes["onmouseout"] = String.Format(ContextMenu.m_cstrOnMouseOut, ColorTranslator.ToHtml(BackColor));
                    }
                    rowMenuItem.ToolTip                  = item.Tooltip;
                    lblMenuItem                          = new LiteralControl(item.Text);
                    celMenuItemContainer.Controls.Add(lblMenuItem);
                    if ((item.CommandName != null) && (item.CommandName != string.Empty)){
                        rowMenuItem.Attributes["onclick"] = WebUtil.GetPostBackEventReference(Page, this, item.CommandName);
                    } else {
                        if (item.JavaScriptCommand.IndexOf("caller") != -1 &&
                            CallerReference != null                                 &&
                            CallerReference != string.Empty                             ){
                            item.JavaScriptCommand = item.JavaScriptCommand.Replace("caller", "ContextMenu_GlobalCallerTracker");
                        }
                        rowMenuItem.Attributes["onclick"] = GetOnClickReference() + item.JavaScriptCommand;
                    }
                    if ((item.ImageURL != null) && (item.ImageURL != string.Empty)) {
                        imgMenuItem          = new System.Web.UI.WebControls.Image();
                        imgMenuItem.ImageUrl = item.ImageURL;
                        celImageContainer.Controls.Add(imgMenuItem);
                    }
                    if(!item.Visible){
                        rowMenuItem.Style["display"] = "none";
                    }
                }
			}
			foreach(Control ctl in BoundControls){
				if (ctl is WebControl) {
					((WebControl)ctl).Attributes["onclick"] = GetMenuOpenScript();
				} else if (ctl is HtmlControl) {
					((HtmlControl)ctl).Attributes["onclick"] = GetMenuOpenScript();
				}
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// Render :                    Overridden to render the UI of the control
        ///                             and uses it to initialize the base TableItemStyle
        /// </summary>
        /// <param name="writer">      The html text writer</param>
        //
        //********************************************************************************
		protected override void Render(HtmlTextWriter writer) {
            EnsureDefaultStyles();
			EnsureChildControls();
			PrepareControlForRendering();
			RenderContents(writer);
		}
        
        //********************************************************************************
        //
        /// <summary>
        /// EnsureChildControls :       Overridden to ensure proper refresh and display
        ///                             in design mode
        /// </summary>
        //
        //********************************************************************************
        protected override void EnsureChildControls() {
            base.EnsureChildControls ();
            if (Controls.Count == 0) {
                CreateChildControls();
            }
        }
	
        //********************************************************************************
        //
        /// <summary>
        /// PrepareControlForRendering :          Applies styles to the control components
        ///                                       immediately before rendering
        /// </summary>
        //
        //********************************************************************************
		protected virtual void PrepareControlForRendering() {
            HtmlGenericControl  ctlDIV;
            Table               tblMenu;
            TableRow            rowMenuItem;
            TableCell           celMenuItemContainer;
            LinkButton          btnMenuItem;

            if (Controls.Count == 1) {
                ctlDIV  = (HtmlGenericControl)Controls[0];
                tblMenu = (Table) ctlDIV.Controls[0];
                tblMenu.ApplyStyle(ControlStyle);
                for(int i = 0 ; i < tblMenu.Rows.Count; i++) {
                    rowMenuItem          = tblMenu.Rows[i];
                    celMenuItemContainer = rowMenuItem.Cells[rowMenuItem.Cells.Count - 1];
                    if (celMenuItemContainer.Controls.Count > 0) {
                        btnMenuItem = (celMenuItemContainer.Controls[0] as LinkButton);
                        if (btnMenuItem != null) {
                            btnMenuItem.ForeColor = ControlStyle.ForeColor;
                            btnMenuItem.Style["text-decoration"] = "none";
                        }
                    }
                }
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// OnPreRender:                    Overridden to register the client script. The 
        ///                                 script is included once per page even in the 
        ///                                 presence of multiple ContextMenu objects
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            base.OnPreRender(e);
            EmbedScriptCode();
        }

        //********************************************************************************
        //
        /// <summary>
        /// EmbedScriptCode :          Insert the script code needed to refresh the UI
        /// </summary>
        //
        //********************************************************************************
		private void EmbedScriptCode() {
            Assembly        assembly;
            string          strScript;
            StreamReader    reader;

            if (!WebUtil.IsClientScriptBlockRegistered(Page, "ContextMenuHelper")) {
                assembly  = Assembly.GetExecutingAssembly();
                reader    = new StreamReader(assembly.GetManifestResourceStream("WebContextMenuScript.js"));
                strScript = reader.ReadToEnd();
                reader.Close();
                WebUtil.RegisterClientScriptBlock(Page, "ContextMenuHelper", strScript);
            }
		}
		#endregion

		#region Event-related Members
        //********************************************************************************
        //
        /// <summary>
        /// RaisePostBackEvent:         Implements IPostBackEventHandler.RaisePostBackEvent
        ///                             enables this server control to process an event 
        ///                             raised when a form is posted to the server
        /// </summary>
        /// <param name="CommandName">  Event Command Name</param>
        //
        //********************************************************************************
        public void RaisePostBackEvent(String CommandName) {
            OnClick(new CommandEventArgs(CommandName, string.Empty));
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnClick :                   Fires the Click event to the host page
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
		protected virtual void OnClick(CommandEventArgs e){
            if (Click != null){
                Click(this, e);
            }
		}

        #endregion
	}//class ContextMenu
}//namespace