using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.Drawing.Design;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Add the follosing functions to the ASP.NET Grid:
	///     Can be smartly binded to the DataSourceRowset (AutoSize, AutoTitle etc.)
	///     Generates Javascript and template row to create row dynamicaly on the front end
	///     Adds somes virtual functions to add attributes to the table, row or cell.
	/// </summary>
	//
    //********************************************************************************
	[Designer(typeof(DataGridRowset.DataGridRowsetDesigner))]
	public class DataGridRowset : System.Web.UI.WebControls.DataGrid, IPostBackEventHandler {
        /// <summary>
        /// Automatic generation mode for editable columns
        /// </summary>
        public enum AutoGeneratedEditColumnsE {
            /// <summary>Don't generate editable columns </summary>
            None                = 0,
            /// <summary>Generated editable columns only for user defined flag fields </summary>
            UserFlagOnly        = 1,
            /// <summary>Generated editable columns only for selectable fields </summary>
            SelectableOnly      = 2,
            /// <summary>Generated editable columns for all visible fields </summary>
            All                 = 3
        }
        /// <summary>
        /// Control type fpr editable columns
        /// </summary>
        public enum EditColumnTypeE {
            /// <summary>Field text box </summary>
            FieldTextBox        = 0,
            /// <summary>Field control type</summary>
            FieldControlType    = 1
        }
	    /// <summary>RowsetDef control utility class</summary>
	    protected ControlUtil                   m_controlUtil;
        /// <summary>Array of datagrid columns</summary>
        protected DataGridColumn[]              m_arrColumns              = null;
        /// <summary>The name of the column to be used to sort the datagrid</summary>
        protected string                        m_strSortColumn           = null;
        /// <summary>The direction of the datagrid sort ("ASC or DESC")</summary>
        protected string                        m_strSortDirection        = null;
        /// <summary>Indicates if the current request is a page change</summary>
        protected bool                          m_bIsAnExplicitPageChange = false;
        /// <summary>Select item command name</summary>
        public const string                     SELECT_ITEM_CMD           = "_SELECTITEM";
        /// <summary>Indicates whether or not to generate the onclick script to raise the itemselected evcent</summary>
        protected bool                          bGenOnClickScript         = true;
        /// <summary>True to indicate that the Rowset of the datasource was auto associated</summary>
        protected bool                          m_bAutoAssociated         = false;
        /// <summary>Raised when a user clicks on an item in teh datagrid</summary>
        [Browsable(true)]
        [Category("Action")]
        [Description("Event raised when a user clicks on an item in the datagrid")] 
        public event CommandEventHandler        ItemSelected;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
		public DataGridRowset() {
		    m_controlUtil  = new ControlUtil(this, new WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:                    Dispose the object
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_controlUtil != null) {
                m_controlUtil.Dispose();
                m_controlUtil = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetNewPageIndex:                Set the new page index
	    /// </summary>
	    /// <param name="iNewPageIndex">    New page index</param>
	    //
        //********************************************************************************
		protected virtual void SetNewPageIndex(int iNewPageIndex) {
            CurrentPageIndex = iNewPageIndex;
	        DataBind();
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPageIndexChanged:             Called when the index page has been changed
	    /// </summary>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
        protected override void OnPageIndexChanged(DataGridPageChangedEventArgs e) {
            m_bIsAnExplicitPageChange = true;
            SetNewPageIndex(e.NewPageIndex);
            base.OnPageIndexChanged(e);
        }

        //********************************************************************************
        //
		/// <summary>
		/// RaisePostBackEvent:     Called to raise an event
		/// </summary>
		//
        //********************************************************************************
        public virtual void RaisePostBackEvent(string strEventArgument) {
            string strCmdArgs;
            
            if (strEventArgument.ToUpper().StartsWith(SELECT_ITEM_CMD + "=")) {
                strCmdArgs = strEventArgument.Substring(SELECT_ITEM_CMD.Length + 1);
                OnItemSelected(new CommandEventArgs(SELECT_ITEM_CMD, strCmdArgs));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnItemSelected:             Function used to raise the "ItemSelected" event
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnItemSelected(CommandEventArgs e) {
            if (ItemSelected != null) {
                ItemSelected(this, e);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// ResolveRowsetDefFromDataSource: Finds the Rep++ rowset definition attached to the datasource.
		///                                 The datasource can come from a DataSourceRowset
		///                                 or from a DataView if the designer creates a sample one.
		/// </summary>
		/// <param name="rowset">           Rowset attached to the datasource if any</param>
		/// <returns>
		/// Rep++ rowset definition or NULL if none
		/// </returns>
		//
        //********************************************************************************
		protected RowsetDef ResolveRowsetDefFromDataSource(out Rowset rowset) {
            return(ResolveRowsetDefFromDataSource(out rowset, InternalDataSource));
        }

        //********************************************************************************
        //
		/// <summary>
		/// ResolveRowsetDefFromDataSource: Finds the Rep++ rowset definition attached to the datasource.
		///                                 The datasource can come from a DataSourceRowset
		///                                 or from a DataView if the designer creates a sample one.
		/// </summary>
		/// <param name="rowset">           Rowset attached to the datasource if any</param>
        /// <param name="oDataSource">      Data Source object</param>
		/// <returns>
		/// Rep++ rowset definition or NULL if none
		/// </returns>
		//
        //********************************************************************************
		protected RowsetDef ResolveRowsetDefFromDataSource(out Rowset rowset, object oDataSource) {
            RowsetDef               rowsetDefRetVal     = null;
            DataSourceRowset        rowsetDataSource    = null;
            DataView                dataView;
            string                  strName;

            rowsetDataSource = oDataSource as DataSourceRowset;
            if (rowsetDataSource != null) {
                rowsetDefRetVal = rowsetDataSource.RowsetDef;
                rowset          = rowsetDataSource.Rowset;
                if (rowset == null || m_bAutoAssociated) {
                    rowsetDataSource.AutoAssociateRowset(null);
                    rowset            = rowsetDataSource.Rowset;
                    m_bAutoAssociated = true;
                    OnDataSourceViewChanged(this, EventArgs.Empty);
                }
            } else {
                rowset   = null;
                dataView = oDataSource as DataView;
                if (dataView != null) {
                    strName = dataView.Table.TableName;
                    if (strName != null && strName.Length != 0 && Application != null) {
                        rowsetDefRetVal = Application.RowsetDefs[strName];
                    }
                }
            }
            return(rowsetDefRetVal);
		}

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_controlUtil != null) {
                m_controlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AutoGenerateOnClickScript:         True to generate OnClick script
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Bindable(true)]
        [DefaultValue(true)]
        [Description("True to generate the OnClick script to raise the ItemSelected event")]
        public virtual bool AutoGenerateOnClickScript {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["AutoGenerateOnClickScript"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AutoGenerateOnClickScript"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_controlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// AutoColumnAlign:            true to automaticaly align some columns (money)
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(null)]
        [Description("Automatically align columns")]
        public virtual bool AutoColumnAlign {
            get {
                bool    bRetVal;
                object  objAuto;
                
                objAuto = ViewState["AutoColumnAlign"];
                if (objAuto != null) {
                    bRetVal = (bool)objAuto;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AutoColumnAlign"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AutoColumnSize:         true to automaticaly size the columns
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(null)]
        [Description("Automatically set the column size")]
        public virtual bool AutoColumnSize {
            get {
                bool    bRetVal;
                object  objAuto;
                
                objAuto = ViewState["AutoColumnSize"];
                if (objAuto != null) {
                    bRetVal = (bool)objAuto;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["AutoColumnSize"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// MinColumnSize:          Minimum column size for automatic sizing
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(10)]
        [Description("Minimum size for automatically size columns")]
        public virtual int MinColumnSize {
            get {
                int     iRetVal;
                object  objAuto;
                
                objAuto = ViewState["MinColumnSize"];
                if (objAuto != null) {
                    iRetVal = (int)objAuto;
                } else {
                    iRetVal = 10;
                }
                return(iRetVal);
            }
            set {
                ViewState["MinColumnSize"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// MaxColumnSize:          Maximum column size for automatic sizing
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(20)]
        [Description("Maximum size for automatically size columns")]
        public virtual int MaxColumnSize {
            get {
                int     iRetVal;
                object  objAuto;
                
                objAuto = ViewState["MaxColumnSize"];
                if (objAuto != null) {
                    iRetVal = (int)objAuto;
                } else {
                    iRetVal = 20;
                }
                return(iRetVal);
            }
            set {
                ViewState["MaxColumnSize"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AutoGeneratedEditColumns:       Automatically generation of editable columns mode     
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(null)]
        [Description("Automatically generation of editable columns mode")]
        public virtual AutoGeneratedEditColumnsE AutoGeneratedEditColumns {
            get {
                AutoGeneratedEditColumnsE   eRetVal;
                object                      objAuto;
                
                objAuto = ViewState["AutoGeneratedEditColumns"];
                if (objAuto != null) {
                    eRetVal = (AutoGeneratedEditColumnsE)objAuto;
                } else {
                    eRetVal = AutoGeneratedEditColumnsE.None;
                }
                return(eRetVal);
            }
            set {
                ViewState["AutoGeneratedEditColumns"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EditColumnType:       Control type for editable columns
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(null)]
        [Description("Control type for editable columns")]
        public virtual EditColumnTypeE EditColumnType {
            get {
                EditColumnTypeE     eRetVal;
                object              objAuto;
                
                objAuto = ViewState["EditColumnType"];
                if (objAuto != null) {
                    eRetVal = (EditColumnTypeE)objAuto;
                } else {
                    eRetVal = EditColumnTypeE.FieldTextBox;
                }
                return(eRetVal);
            }
            set {
                ViewState["EditColumnType"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllLinesEditable:         True to allow all lines to be editable 
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(false)]
        [Description("Allow all lines to be editable")]
        public virtual bool AllLinesEditable {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["AllLinesEditable"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = false;
                }
                return(bRetVal);
            }
            set {
                ViewState["AllLinesEditable"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveSortInCookie:         True to save/load the sort expression to/from a cookie
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [Bindable(true)]
        [DefaultValue(true)]
        [Description("True to save/load the sort expression to/from a cookie")]
        public virtual bool SaveSortInCookie {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["SaveSortInCookie"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["SaveSortInCookie"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataKeyField:           Field representing the key. Not supported. The
        ///                         key fields are match to the group one.
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        public override string DataKeyField {
            get {
                return("");
            }
            set {
                // Do nothing...
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SortAscImageUrl:          The image that will appear in the header of the
        ///                           column of the ascending sort field 
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Category("Appearance")]
        [Description("The image that will appear in the header of the column of the ascending sort field")]
        public virtual string SortAscImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["SortAscImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = string.Empty;
                }
                return(strRetVal);
            }
            set {
                ViewState["SortAscImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SortDescImageUrl:         The image that will appear in the header of the
        ///                           column of the descending sort field 
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(UrlEditor),typeof(UITypeEditor))]
        [Category("Appearance")]
        [Description("The image that will appear in the header of the column of the descending sort field")]
        public virtual string SortDescImageUrl {
            get {
                string  strRetVal;
                object  objAuto;
                
                objAuto = ViewState["SortDescImageUrl"];
                if (objAuto != null) {
                    strRetVal = (string)objAuto;
                } else {
                    strRetVal = string.Empty;
                }
                return(strRetVal);
            }
            set {
                ViewState["SortDescImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FixupPageIndex:     If the data source has been changed, the current page index can
        ///                     be out of range. Ensure to fix it up.
        /// </summary>
        //
        //********************************************************************************
        private void FixupPageIndex(Rowset rowset) {
            int     iLineCount;
            int     iMaxPageIndex;
            
            if (AllowPaging) {
                iLineCount      = rowset.UndelLineCount;
                iMaxPageIndex   = (iLineCount == 0) ? 0 : (iLineCount - 1) / PageSize;
                if (CurrentPageIndex > iMaxPageIndex) {
                    CurrentPageIndex = iMaxPageIndex;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EnsurePageVisibility:     If the items in the group instance change their position
        ///                           make sure that the page of the current item is visible
        /// </summary>
        //
        //********************************************************************************
        protected void EnsurePageVisibility(Rowset rowset) {
            int     iCurLine;
            int     iPageNb;
            
            if (AllowPaging) {
                iCurLine = rowset.CurrentUndelLine;
                iPageNb  = iCurLine / PageSize;
                if (iPageNb != CurrentPageIndex && iPageNb < PageCount) {
                    //TODO: can we eliminate one of the possible 3 Databinds 
                    SetNewPageIndex(iPageNb);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// InternalDataSource:         Datasource attached to the rowset
        /// </summary>
        //
        //********************************************************************************
        public object InternalDataSource {
            get {
                object oRetVal;

                oRetVal = DataSource;
                if (oRetVal == null) {
                    oRetVal = ConvertIDToDataSource();
                }
                return (oRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:         Datasource attached to the rowset
        /// </summary>
        //
        //********************************************************************************
        public override object DataSource {
            get {
                return(base.DataSource);
            }
            set {
                Rowset   rowset;
                
                if (base.DataSource != value) {
                    m_bAutoAssociated = false;
                }
                base.DataSource = value;
                if (value != null) {
                    ResolveRowsetDefFromDataSource(out rowset);
                    if (rowset != null) {
                        FixupPageIndex(rowset);
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsGridPostBack:         Indicates if the postback was triggered by one of 
        ///                         the child controls of the grid
        /// </summary>
        //
        //********************************************************************************
        protected virtual bool IsGridPostBack {
            get {
                return (Page.Request.Params["__EVENTTARGET"] != null &&
                        ID != null                                   &&
                        Page.Request.Params["__EVENTTARGET"].StartsWith(ID));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnSortCommand:      Called when the sort occurs
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnSortCommand(DataGridSortCommandEventArgs e) {
            string                  strDirection       = " ASC";
            
            if (m_strSortColumn != null                  &&
                m_strSortColumn.Equals(e.SortExpression) &&
                m_strSortDirection != null               &&
                m_strSortDirection.Equals(" ASC")){
                strDirection = " DESC";
            }
            m_strSortColumn     = e.SortExpression;
            m_strSortDirection  = strDirection;
            DataBind();
            base.OnSortCommand(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDeleteCommand:        Called when an item in the datagrid is deleted.
        /// </summary>
        /// <param name="e">        Event parameter</param>
        //
        //********************************************************************************
        protected override void OnDeleteCommand(DataGridCommandEventArgs e) {
            DataSourceRowset rowsetDataSource;
            
            rowsetDataSource = InternalDataSource as DataSourceRowset;
            if (rowsetDataSource != null && rowsetDataSource.AllowRemove) {
                rowsetDataSource.RemoveAt(e.Item.ItemIndex);
                DataBind();
            }
            base.OnDeleteCommand (e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ApplySortExpression:      applies/re-applies the sort expression to the current 
        ///                           DataSource
        /// </summary>
        //
        //********************************************************************************
        protected virtual bool ApplySortExpression( bool bDataBinding) {
            DataSourceRowset    rowsetDataSource;
            string              strSortExpression;
            bool                bRetVal             = false;
            FieldDef            fldDef;
            
            if (AllowSorting) {
                if (bDataBinding) {
                    rowsetDataSource = InternalDataSource as DataSourceRowset;
                    if (rowsetDataSource != null) {
                        strSortExpression = m_strSortColumn + m_strSortDirection;
                        fldDef               = rowsetDataSource.RowsetDef.FieldDefs[m_strSortColumn];
                        if (m_strSortColumn != null    && 
                            m_strSortDirection != null &&
                            fldDef != null             &&
                            fldDef.IsVisible) {
                            rowsetDataSource.Sort = strSortExpression;
                            SaveSortInformation(m_strSortColumn, m_strSortDirection );
                            bRetVal = true;
                        }
                    }
                } else {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnInit:             Called when control is initialized
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnInit(EventArgs e) {
            if (!DesignMode && AllowSorting && !Page.IsPostBack) {
                if (GetSortInformation(out m_strSortColumn, out m_strSortDirection)) {
                    ApplySortExpression(false);
                }
            }
            base.OnInit (e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Converts the DataSourceID to a DataSource
        /// </summary>
        //
        //********************************************************************************
        private DataSourceRowset ConvertIDToDataSource() {
            Control             ctlContainer;
            Control             ctlFound;
            RowsetDataSourceCtl repDataSource       = null;
            DataSourceRowset    dsRowset            = null;
            string              strDataSourceID;
            Rowset              rowset;

            if (!DesignMode) {
                strDataSourceID = base.DataSourceID;
                if (!string.IsNullOrEmpty(strDataSourceID)) {
                    ctlContainer = this.Parent;
                    while (ctlContainer != null && repDataSource == null) {
                        ctlFound = ctlContainer.FindControl(strDataSourceID);
                        if (ctlFound != null){
                            if (ctlFound is RowsetDataSourceCtl) {
                                repDataSource = (RowsetDataSourceCtl)ctlFound;
                            } else {
                                throw new Exception(strDataSourceID + " is not a valid RowsetDataSourceCtl");
                            }
                        }
                        ctlContainer  = ctlContainer.Parent;
                    }
                    if (repDataSource == null) {
                        throw new Exception("Cannot find RowsetDataSourceCtl: " + strDataSourceID);
                    }
                    dsRowset = repDataSource.DataSource;
                    if(dsRowset != null) {
                        ResolveRowsetDefFromDataSource(out rowset, dsRowset);
                        if (rowset != null) {
                            FixupPageIndex(rowset);
                        }
                    }
                }
            }
            return(dsRowset);
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadViewState:              Used to restore the sort after the view state has been loaded
        /// </summary>
        /// <param name="savedState">   Saved Data</param>
        //
        //********************************************************************************
        protected override void LoadViewState(object savedState) {
            base.LoadViewState(savedState);
            if (!DesignMode && AllowSorting && Page.IsPostBack) {
                if (GetSortInformation(out m_strSortColumn, out m_strSortDirection)) {
                    ApplySortExpression(false);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Called when the databinding occurs
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnDataBinding(EventArgs e) {

            DisposeFldControls(Controls);
            base.OnDataBinding(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DisposeFldControls:      When we databind more than once during the same request
        ///                          Controls.Clear() does not call Dispose() on the Rep++
        ///                          Field controls for editable columns. This method
        ///                          corrects the  problem   
        /// </summary>
        /// <param name="arrCtls">   The controls container to be searched recursively</param>
        //
        //********************************************************************************
        private void DisposeFldControls(ControlCollection arrCtls) {
            foreach (Control ctl in arrCtls) {
                if (ctl is FldControlUtil.IFldControl) {
                    ctl.Dispose();
                } else if (ctl.Controls.Count > 0) {
                    DisposeFldControls(ctl.Controls);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ComputeColSize:             Compute the size of a column
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition</param>
        /// <param name="fldDef">       Field definition</param>
        /// <param name="col">          Datagrid column</param>
        /// <param name="iMinColSize">  Minimum column size</param>
        /// <param name="iMaxColSize">  Maximum column size</param>
        /// <returns>
        /// Size in characters
        /// </returns>
        //
        //********************************************************************************
        protected virtual int ComputeColSize(RowsetDef rowsetDef, FieldDef fldDef, DataGridColumn col, int iMinColSize, int iMaxColSize) {
            int             iRetVal;
            int             iPromptSize;
            int             iValSize;
            BoundColumn     boundCol;
            ButtonColumn    buttonCol;
            HyperLinkColumn hyperLinkCol;

            if (fldDef != null) {
                iPromptSize = fldDef.Prompt.Length;
                iValSize    = fldDef.MaxSize;
                iRetVal     = Math.Max(iPromptSize, iValSize);
                iRetVal     = Math.Max(iMinColSize, iRetVal);
                iRetVal     = Math.Min(iMaxColSize, iRetVal);
            } else if (col != null) {
                if (col is BoundColumn) {
                    boundCol    = (BoundColumn)col;
                    iRetVal     = (boundCol.HeaderText == null) ? 10 : boundCol.HeaderText.Length;
                } else if (col is ButtonColumn) {
                    buttonCol = (ButtonColumn)col;
                    iRetVal   = (buttonCol.Text == null) ? 10 : buttonCol.Text.Length;
                } else if (col is HyperLinkColumn) {
                    hyperLinkCol = (HyperLinkColumn)col;
                    iRetVal      = (hyperLinkCol.Text == null) ? 10 : hyperLinkCol.Text.Length;
                } else {
                    iRetVal      = (col.HeaderText == null) ? 10 : col.HeaderText.Length;
                }
            } else {
                iRetVal = 10;
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetAutoSize:                Set the column size
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition</param>
        /// <param name="fldDef">       Field definition</param>
        /// <param name="col">          Datagrid column</param>
        /// <param name="cell">         Cell</param>
        /// <param name="iTotalSize">   Total size of the fields</param>
        /// <param name="iMinColSize">  Minimum column size (as set by the property)</param>
        /// <param name="iMaxColSize">  Maximum column size (as set by the property)</param>
        //
        //********************************************************************************
        protected virtual void SetAutoSize(RowsetDef rowsetDef, FieldDef fldDef, DataGridColumn col, TableCell cell, int iTotalSize, int iMinColSize, int iMaxColSize) {
            int     iColSize;
            double  dPer;
            
            iColSize    = ComputeColSize(rowsetDef, fldDef, col, iMinColSize, iMaxColSize);
            dPer        = ((double)iColSize) / ((double)(iTotalSize)) * 99.9;
            if (cell.Width.Type == UnitType.Pixel && cell.Width.Value == 0) {
                cell.Width  = new Unit(dPer, UnitType.Percentage);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetAutoAlign:               Set the alignment of a cell
        /// </summary>
        /// <param name="fldDef">       Field definition attached to the column</param>
        /// <param name="itemType">     Item type</param>
        /// <param name="cell">         Cell</param>
        //
        //********************************************************************************
        protected virtual void SetAutoAlign(FieldDef fldDef, ListItemType itemType,  TableCell cell) {
            if ((fldDef.OutputMask != null && fldDef.OutputMask != String.Empty) ||
                fldDef.Precision != 0 ||
                fldDef.Type == FieldType.sdFieldMoney) {
                if (cell.HorizontalAlign == HorizontalAlign.NotSet) {
                    cell.HorizontalAlign = HorizontalAlign.Right;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ComputeTotalColSize:        Compute the size of all fields
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition</param>
        /// <param name="cols">         Collection of datagrid columns</param>
        /// <returns>
        /// Total field size in characters
        /// </returns>
        //
        //********************************************************************************
        protected virtual int ComputeTotalColSize(RowsetDef rowsetDef, DataGridColumnCollection cols) {
            int         iRetVal      = 0;
            int         iMinColSize;
            int         iMaxColSize;
            FieldDef    fldDef = null;
            
            iMinColSize = MinColumnSize;
            iMaxColSize = MaxColumnSize;
            foreach (DataGridColumn col in cols) {
                if (rowsetDef != null) {
                    fldDef = GetColumnField(rowsetDef.FieldDefs, col, -1);
                }
                iRetVal += ComputeColSize(rowsetDef, fldDef, col, iMinColSize, iMaxColSize);
            }
            if (AutoGenerateColumns && rowsetDef != null) {
                foreach (FieldDef fldDef2 in rowsetDef.FieldDefs) {
                    iRetVal += ComputeColSize(rowsetDef, fldDef2, null, iMinColSize, iMaxColSize);
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillKeyList:                Fill the key list with the one from the attached rowset
        /// </summary>
        /// <param name="dataSource">   Source attached to the grid</param>
        //
        //********************************************************************************
        protected virtual void FillKeyList(DataSourceRowset dataSource) {
            DataRowRowset   row;
            ArrayList       arrKeys;
            FieldDef[]      arrPKFldDef;
            string[]        arrPKVal;
            Rowset          rowset;
            int             iLineIndex;
            int             iCount;
            
            arrKeys = this.DataKeysArray;
            arrKeys.Clear();
            rowset = dataSource.Rowset;
            if (rowset != null) {
                iCount = 0;
                foreach (FieldDef fldDefPK in dataSource.RowsetDef.FieldDefs) {
                    if (fldDefPK.IsPrimaryKey) {
                        iCount++;
                    }
                }
                if (iCount != 0) {
                    arrPKFldDef = new FieldDef[iCount];
                    arrPKVal    = new string[iCount];
                    iCount      = 0;
                    foreach (FieldDef fldDefPK in dataSource.RowsetDef.FieldDefs) {
                        if (fldDefPK.IsPrimaryKey) {
                            arrPKFldDef[iCount++] = fldDefPK;
                        }
                    }
                    foreach(DataGridItem item in Items) {
                        row = dataSource[item.DataSetIndex] as DataRowRowset;
                        if (row != null) {
                            iLineIndex = rowset.LineFromBookmark(row.Bookmark);
                            iCount     = 0;
                            foreach (RepPP.FieldDef fldDefPK in arrPKFldDef) {
                                arrPKVal[iCount++] = rowset.GetFieldValue(fldDefPK, iLineIndex, true);
                            }
                            arrKeys.Add(WebUtil.EncodeStringArray(arrPKVal));
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateControlHierarchy:         Creates the grid control hierarchy
        /// </summary>
        /// <param name="bUseDataSource">   true to use the data source</param>
        //
        //********************************************************************************
        protected override void CreateControlHierarchy(bool bUseDataSource) {
            DataSourceRowset dataSource;

            base.CreateControlHierarchy(bUseDataSource);
            if (bUseDataSource) {
                dataSource = InternalDataSource as DataSourceRowset;
                if (dataSource != null) {
                    FillKeyList(dataSource);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddTableAttributes:         Add attributes to the table
        /// </summary>
        /// <param name="tbl">          Table</param>
        /// <param name="rowset">       Rowset attached to the grid if any</param>
        /// <param name="rowsetDef">    Rowset definition attached to the grid if any</param>
        //
        //********************************************************************************
        protected virtual void AddTableAttributes(Table tbl, Rowset rowset, RowsetDef rowsetDef) {
            string                  strKey;
            int                     iSelectedIndex;
            string                  strStyle;
            string                  strItemStyle;
            string                  strAlternateItemStyle;
            string                  strSelectedItemClass;
            string                  strItemClass;
            string                  strAlternateItemClass;
            
            iSelectedIndex = (rowset == null) ? -1 : rowset.CurrentLine;
            strKey         = (iSelectedIndex == -1) ? String.Empty : WebUtil.EncodeStringArray(Util.BuildPrimaryKey(rowset.RowsetDef, false));
            if (strKey == null) {
                strKey = String.Empty;
            }
            tbl.Attributes.Add("CurrentKey", strKey);
            strStyle             = WebUtil.RenderStyleInString(this, SelectedItemStyle);
            strSelectedItemClass = SelectedItemStyle.CssClass;
            tbl.Attributes.Add("SelectedItemStyle", strStyle);
            tbl.Attributes.Add("SelectedItemClass", strSelectedItemClass);
            strItemStyle          = WebUtil.RenderStyleInString(this, ItemStyle);
            strAlternateItemStyle = WebUtil.RenderStyleInString(this, AlternatingItemStyle);
            strItemClass          = ItemStyle.CssClass;
            strAlternateItemClass = AlternatingItemStyle.CssClass;
            tbl.Attributes.Add("OddItemStyle",  (ShowHeader) ? strItemStyle : strAlternateItemStyle);
            tbl.Attributes.Add("EvenItemStyle", (ShowHeader) ? strAlternateItemStyle : strItemStyle);
            tbl.Attributes.Add("OddItemClass",  (ShowHeader) ? strItemClass : strAlternateItemClass);
            tbl.Attributes.Add("EvenItemClass", (ShowHeader) ? strAlternateItemClass : strItemClass);
            tbl.Attributes.Add("PageCount", PageCount.ToString());
            tbl.Attributes.Add("PageIndex", CurrentPageIndex.ToString());
            tbl.Attributes.Add("PageSize", PageSize.ToString());
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddRowAttributes:       Adds attributes to rows
        /// </summary>
        /// <param name="item">     Datagrid item</param>
        /// <param name="rowset">   Rowset attached to the grid if any</param>
        /// <param name="rowsetDef">Rowset definition attached to the grid if any</param>
        //
        //********************************************************************************
        protected virtual void AddRowAttributes(DataGridItem item, Rowset rowset, RowsetDef rowsetDef) {
            string          strPK;
            int             iItemIndex;
            
            if (!DesignMode) {
                switch(item.ItemType) {
                case ListItemType.AlternatingItem:
                case ListItemType.Item:
                case ListItemType.SelectedItem:
                    if (rowsetDef != null) {
                        iItemIndex = item.ItemIndex;
                        if (iItemIndex != -1) {
                            if (iItemIndex < DataKeys.Count) {
                                strPK = DataKeys[iItemIndex] as string;
                            } else {
                                strPK = String.Empty; 
                            }
                            if (strPK != null) {
                                item.Attributes.Add("RowKey", strPK);
                            }              
                        }
                    }
                    break;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetCellOnClickScript:       Get cell Onclick script
        /// </summary>
        /// <param name="strRowKey">    Row key of the cell</param>
        /// <returns>
        /// OnClick script
        /// </returns>
        //
        //********************************************************************************
        protected virtual string GetCellOnClickScript(string strRowKey) {
            return(WebUtil.GetPostBackEventReference(Page, this, SELECT_ITEM_CMD + "=" + strRowKey));
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddCellAttributes:      Adds attributes to cells
        /// </summary>
        /// <param name="cell">     Cell</param>
        /// <param name="row">      Datagrid item (row)</param>
        /// <param name="col">      Column attached to the cell</param>
        /// <param name="rowset">   Rowset attached to the grid if any</param>
        /// <param name="rowsetDef">Rowset definition attached to the datagrid if any</param>
        /// <param name="fldDef">   FieldDef attached to the cell if any</param>
        //
        //********************************************************************************
        protected virtual void AddCellAttributes(TableCell cell, DataGridItem row, DataGridColumn col, Rowset rowset, RowsetDef rowsetDef, FieldDef fldDef) {
            if (fldDef != null) {
                cell.Attributes.Add("FieldName", fldDef.Name);
                if (AutoGenerateOnClickScript && !DesignMode) {
                    if (!(col is TemplateColumn) && (col is BoundColumn)) {
                        switch(row.ItemType) {
                        case ListItemType.AlternatingItem:
                        case ListItemType.Item:
                        case ListItemType.SelectedItem:
                            cell.Attributes.Add("OnClick", GetCellOnClickScript(row.Attributes["RowKey"]));
                            break;
                        default:
                            break;
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetColumnField:             Gets the field attached to the column if any
        /// </summary>
        /// <param name="fldDefs">      Array of field definitions</param>
        /// <param name="col">          Column</param>
        /// <param name="iFldIndex">    Field index if col is null</param>
        /// <returns>
        /// Field or null if none
        /// </returns>
        //
        //********************************************************************************
        private FieldDef GetColumnField(FieldDefs fldDefs, DataGridColumn col, int iFldIndex) {
            FieldDef    fldDefRetVal;
            string      strFldName;

            if (col == null) {
                fldDefRetVal = fldDefs[iFldIndex];
            } else if (col is BoundColumn) {
                strFldName = ((BoundColumn)col).DataField;
                fldDefRetVal  = (strFldName != null && strFldName != String.Empty) ? fldDefs[strFldName] : null;
            } else if (col is TemplateColumn) {
                if ((col as TemplateColumn).ItemTemplate is TemplateFldControl) {
                    strFldName = ((col as TemplateColumn).ItemTemplate as TemplateFldControl).FieldName;
                    fldDefRetVal  = (strFldName != null && strFldName != String.Empty) ? fldDefs[strFldName] : null;
                } else {
                    fldDefRetVal = null;
                }
            } else {
                fldDefRetVal = null;
            }
            return(fldDefRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateTemplateRow:              Creates a template row which can be used to create a new row
        ///                                 on the client side.
        /// </summary>
        /// <param name="cols">             Array of columns</param>
        /// <param name="dataSource">       Rowset data source</param>
        //
        //********************************************************************************
        protected virtual void CreateTemplateRow(DataGridColumn[] cols, DataSourceRowset dataSource) {
            DataGridItem    item;
            Table           tbl;
            Rowset          rowset;
            RowsetDef       rowsetDef;
            FieldDef        fldDef;
            DataGridColumn  dataGridCol;
            int             iIndex;
            int             iColCount;
            
            if (dataSource != null) {
                tbl     = new Table();
                tbl.ID  = "Template";
                tbl.Style.Add("Display", "None");
                item = CreateItem(-1, -1, ListItemType.Item);
                InitializeItem(item, cols);
                tbl.Rows.Add(item);
                this.Controls.Add(tbl);
                rowset = dataSource.Rowset;
                rowsetDef   = dataSource.RowsetDef;
                AddRowAttributes(item, rowset, rowsetDef);
                iColCount = cols.Length;
                iIndex    = 0;
                foreach (TableCell cell in item.Cells) {
                    dataGridCol = (iIndex < iColCount) ? cols[iIndex] : null;
                    fldDef = GetColumnField(rowsetDef.FieldDefs, dataGridCol, iIndex - iColCount);
                    if (fldDef != null) {
                        if (cell.Controls.Count == 0) {
                            cell.Text = "";
                        }
                    }
                    AddCellAttributes(cell, item, cols[iIndex], rowset, rowsetDef, fldDef);
                    iIndex++;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsColumnEditable:       Check if the column can be editable
        /// </summary>
        /// <param name="fldDef">   Attached field</param>
        /// <returns>
        /// True:           The column is editable
        /// False:          The column is not editable
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool IsColumnEditable(FieldDef fldDef) {
            bool    bRetVal = false;

            if (fldDef.IsVisible) {
                switch (AutoGeneratedEditColumns) {
                case AutoGeneratedEditColumnsE.None:
                    bRetVal = false;
                    break;
                case AutoGeneratedEditColumnsE.All:
                    bRetVal = true;
                    break;
                case AutoGeneratedEditColumnsE.SelectableOnly:
                    bRetVal = fldDef.IsSelectable;
                    break;
                case AutoGeneratedEditColumnsE.UserFlagOnly:
                    bRetVal = fldDef.IsUserFlagOn;
                    break;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateColumnSet:                Creates a column set. We intercept this function so we
        ///                                 can generates a templare row and generate javascript. This
        ///                                 Javascript can be used to add a row in the datagrid at the 
        ///                                 client side.
        /// </summary>
        /// <param name="dataSource">       Data source</param>
        /// <param name="bUseDataSource">   true if using a data source</param>
        /// <returns>
        /// Column set
        /// </returns>
        //
        //********************************************************************************
        protected override ArrayList CreateColumnSet(PagedDataSource dataSource, bool bUseDataSource) {
            ArrayList                       arrRetVal;
            RepPP.FieldDefs                 fldDefs;
            RepPP.FieldDef                  fldDef;
            DataSourceRowset                dsRowset;
            DataGridColumn                  col;
            TemplateColumn                  colTemplate;
            int                             iColCount;
            WebUtil.FieldWebControlTypeE    eFldCtlType;
            
            if (AllowSorting) {
                ApplySortExpression(bUseDataSource);
            }
            arrRetVal = base.CreateColumnSet(dataSource, bUseDataSource);
            dsRowset  = InternalDataSource as DataSourceRowset;
            if (AutoGeneratedEditColumns !=  AutoGeneratedEditColumnsE.None && dsRowset != null) {
                fldDefs = dsRowset.RowsetDef.FieldDefs;
                for (iColCount = 0; iColCount < arrRetVal.Count; iColCount++) {
                    col = (DataGridColumn)arrRetVal[iColCount];
                    fldDef = GetColumnField(fldDefs, col, iColCount);  
                    if (fldDef != null && IsColumnEditable(fldDef)) {
                        if (EditColumnType == EditColumnTypeE.FieldTextBox) {
                            eFldCtlType = WebUtil.FieldWebControlTypeE.FldTextBox;
                        } else {
                            eFldCtlType = WebUtil.GetFieldControlType(fldDef);
                        }
                        colTemplate = new TemplateColumn();
                        if (AllLinesEditable) {
                            colTemplate.ItemTemplate = new TemplateFldControl(fldDef, eFldCtlType);
                        } else {
                            colTemplate.ItemTemplate     = new TemplateFldControl(fldDef, WebUtil.FieldWebControlTypeE.FldValue);
                            colTemplate.EditItemTemplate = new TemplateFldControl(fldDef, eFldCtlType);
                        }
                        arrRetVal[iColCount] = colTemplate;
                    }
                }
            }
            if (dataSource != null) {
                m_arrColumns = new DataGridColumn[arrRetVal.Count];
                arrRetVal.CopyTo(m_arrColumns);
                InitColumnHeaders();
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InitColumnHeaders:              Sets the headers of the bound columns
        /// </summary>
        //
        //********************************************************************************
        protected virtual void InitColumnHeaders() {
            RowsetDef   rowsetDef;
            Rowset      rowset;
            FieldDefs   fldDefs;
            FieldDef    fldDef;
            string      strSortColumn;
            string      strSortDirection;
            int         iColCount = 0;
            bool        bDisplayImages;

            bDisplayImages   = (!SortAscImageUrl.Equals(string.Empty)) && (!SortDescImageUrl.Equals(string.Empty));
            strSortColumn    = (m_strSortColumn == null) ? string.Empty : m_strSortColumn;
            strSortDirection = (m_strSortDirection == null) ? string.Empty : m_strSortDirection;
            rowsetDef        = ResolveRowsetDefFromDataSource(out rowset);
            if (rowsetDef != null) {
                fldDefs = rowsetDef.FieldDefs;
                foreach(DataGridColumn col in m_arrColumns){
                    fldDef = GetColumnField(fldDefs, col, iColCount++);   
                    if (fldDef != null && fldDef.Prompt != string.Empty){
                        col.HeaderText = fldDef.Prompt;
                        if (bDisplayImages) {
                            if (strSortColumn.Equals(col.SortExpression)) {
                                if (strSortDirection.Equals(" ASC")){
                                    col.HeaderText += " <img src='" + SortAscImageUrl + "' border=0 align='absmiddle'>";
                                } else {
                                    col.HeaderText += " <img src='" + SortDescImageUrl + "' border=0 align='absmiddle'>";
                                }
                            }
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnPreRender:            Add javascript before rendering occurs
	    /// </summary>
	    /// <param name="e">        Event arguments</param>
	    //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            const string        strKey = "WebGrpDataGridScript_js";
            StreamReader        reader;
            RowsetDef           rowsetDef;
            Rowset              rowset;
            DataRowRowset       rowsetRow;
            DataSourceRowset    rowsetDataSource;
            IntPtr              iSelectedBmk;
            int                 iCurLine;
            int                 iSelectedIndex;

            base.OnPreRender(e);
            WebUtil.RegisterEncodeFunctions(Page);
            if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                reader = new StreamReader(typeof(DataGridRowset).Assembly.GetManifestResourceStream("WebDataGridRowsetScript.js"));
                WebUtil.RegisterClientScriptBlock(Page,
                                                  strKey, 
                                                  String.Format(reader.ReadToEnd(), ID));
            }
            rowsetDataSource = InternalDataSource as DataSourceRowset;
            rowsetDef        = ResolveRowsetDefFromDataSource(out rowset);
            if (rowset != null && rowsetDataSource != null) {
                if (!m_bIsAnExplicitPageChange){
                    EnsurePageVisibility(rowset);
                }
                iSelectedIndex  = -1;
                iCurLine        = rowset.CurrentLine;
                iSelectedBmk    = (iCurLine == -1) ? (IntPtr)(-1) : rowset.LineBookmark(iCurLine);
                foreach (DataGridItem item in Items) {
                    rowsetRow = rowsetDataSource[item.DataSetIndex] as DataRowRowset;
                    if (rowsetRow != null && rowsetRow.Bookmark == iSelectedBmk) {
                        iSelectedIndex = item.ItemIndex;
                        break;
                    }
                }
                SelectedIndex = iSelectedIndex;
            }

        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                     Render the control. 
        /// </summary>
        /// <param name="writer">       Stream in which the function render the HTML</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            Table           tbl;
            DataGridItem    item;
            DataGridColumn  dataGridCol;
            int             iIndex;
            Rowset          rowset;
            RowsetDef       rowsetDef;
            FieldDefs       fldDefs;
            FieldDef        fldDef;
            int             iMinColSize;
            int             iMaxColSize;
            int             iTotalSize;
            int             iColCount;
            bool            bAutoSize;
            bool            bAutoAlign;
            
            rowsetDef = ResolveRowsetDefFromDataSource(out rowset);
            if (rowsetDef == null) {
                base.Render(writer);
            } else {
                EnsureChildControls();
                PrepareControlHierarchy();
                bAutoAlign  = AutoColumnAlign;
                bAutoSize   = AutoColumnSize;
                iMinColSize = MinColumnSize;
                iMaxColSize = MaxColumnSize;
                fldDefs     = rowsetDef.FieldDefs;
                iTotalSize  = ComputeTotalColSize(rowsetDef, Columns);
                tbl         = Controls[0] as Table;
                AddTableAttributes(tbl, rowset, rowsetDef);
                if (tbl != null) {
                    foreach (TableRow row in tbl.Rows) {
                        item = row as DataGridItem;
                        AddRowAttributes(item, rowset, rowsetDef);
                        iColCount  = (m_arrColumns != null) ? m_arrColumns.Length : Columns.Count;
                        if (item != null) {
                            iIndex = 0;
                            foreach (TableCell cell in item.Cells) {
                                if (m_arrColumns != null) {
                                    dataGridCol = (iIndex < iColCount) ? m_arrColumns[iIndex] : null;
                                } else {
                                    dataGridCol = (iIndex < iColCount) ? Columns[iIndex] : null;
                                }
                                fldDef = GetColumnField(fldDefs, dataGridCol, iIndex - iColCount);
                                iIndex++;
                                if (fldDef != null) {
                                    if (bAutoAlign) {
                                        SetAutoAlign(fldDef, item.ItemType, cell);
                                    }
                                }
                                AddCellAttributes(cell, item, dataGridCol, rowset, rowsetDef, fldDef);
                                if (bAutoSize) {
                                    SetAutoSize(rowsetDef, fldDef, dataGridCol, cell, iTotalSize, iMinColSize, iMaxColSize);
                                }
                            }
                        }
                    }
                }
                if (m_arrColumns != null) {
                    CreateTemplateRow(m_arrColumns, InternalDataSource as DataSourceRowset);
                }
                base.RenderContents(writer);
            }
        }


        //********************************************************************************
        //
        /// <summary>
        /// GetSortInformation:                     Gets the sort information from a
        ///                                         cookie or view state (if it exists)
        /// </summary>
        /// <param name="strSortColumn">            Column to sort</param>
        /// <param name="strSortDirection">         Direction of the sort</param>
        /// <returns>
        /// True if the sort info is found, false otherwise
        /// </returns>
        //
        //********************************************************************************
        protected bool GetSortInformation(out string strSortColumn, out string strSortDirection) {
            HttpCookie  cookie;
            bool        bRetVal   = false;
            string      strKey;

            if (SaveSortInCookie) {
                strKey = "RepPP:Sort:" + Page.Server.MapPath(ID);
                cookie = Page.Request.Cookies[strKey];
                if ( cookie != null ) {
                    strSortColumn       = cookie.Values["SortColumn"];
                    strSortDirection    = cookie.Values["SortDirection"];
                    bRetVal             = true;
                } else {
                    strSortColumn       = null;
                    strSortDirection    = null;
                }
            } else {
                strSortColumn       = (string)ViewState["SortColumn"];
                strSortDirection    = (string)ViewState["SortDirection"];
                if (strSortColumn != null && strSortDirection != null) {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveSortInformation:                    Saves the sort information in a cookie/view state.
        /// </summary>
        /// <param name="strSortColumn">            Column to sort</param>
        /// <param name="strSortDirection">         Direction of the sort</param>
        //
        //********************************************************************************
        protected void SaveSortInformation(string strSortColumn, string strSortDirection) {
            HttpCookie cookie;
            string     strKey;

            if (SaveSortInCookie) {
                strKey                  = "RepPP:Sort:" + Page.Server.MapPath(ID);
                cookie                  = Page.Response.Cookies[strKey];
                cookie["SortColumn"]    = strSortColumn;
                cookie["SortDirection"] = strSortDirection;
                cookie.Expires          = DateTime.MaxValue;
                Page.Response.AppendCookie(cookie);
            } else if (EnableViewState) {
                ViewState["SortColumn"]     = strSortColumn;
                ViewState["SortDirection"]  = strSortDirection;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataGridRowsetDesigner:  Designer for the DataGridRowset
        /// </summary>
        //
        //********************************************************************************
        public class DataGridRowsetDesigner : System.Web.UI.Design.WebControls.DataGridDesigner {
            //********************************************************************************
            //
            /// <summary>
            /// GetDesignTimeHtml:          Gets the design time HTML. The base function creates a sample
            ///                             data source (a DataView on a DataTable) using the selected
            ///                             data source as a template. This sample data source is used to
            ///                             show the datagrid in design mode. We override this function so
            ///                             we can pickup the group from the selected datasource and save
            ///                             it as the table name in the DataTable. We will be able to use
            ///                             it later to find the rowset definition attached to the DataView.
            /// </summary>
            /// <returns>
            /// Design time HTML
            /// </returns>
            //
            //********************************************************************************
            public override string GetDesignTimeHtml() {
                string          strRetVal = null;
                int             iMinRowCount;
                bool            bDummyDataSource;
                IEnumerable     dataSourceRetVal;
                IEnumerable     dataSourceSelected;
                DataGridRowset  dataGrid;
                RowsetDef       rowsetDef;
                string          strDataSourceID;
                Control         ctlDataSource;
                string          strErrors = string.Empty;

                dataGrid = Component as DataGridRowset;
                if (dataGrid.AllowPaging && dataGrid.PageSize != 0) {
                    iMinRowCount = Math.Min(dataGrid.PageSize + 1, 101);
                } else {
                    iMinRowCount = 5;
                }
                dataSourceSelected = GetResolvedSelectedDataSource();
                if (dataSourceSelected == null) {
                    strDataSourceID = base.DataSourceID;
                    if (!string.IsNullOrEmpty(strDataSourceID)) {
                        ctlDataSource = FindControl(base.Component.Site, 
                                                    (Control)base.Component, 
                                                    strDataSourceID);
                        if (ctlDataSource == null) {
                            strErrors = "Cannot find RowsetDataSourceCtl: " + strDataSourceID;
                        } else if(ctlDataSource is RowsetDataSourceCtl) {
                            dataSourceSelected = ((RowsetDataSourceCtl)ctlDataSource).DataSource;
                        } else {
                            strErrors = strDataSourceID + " is not a RowsetDataSourceCtl";
                        }
                    }
                }
                if (dataSourceSelected is DataSourceRowset) {
                    rowsetDef = ((DataSourceRowset)dataSourceSelected).RowsetDef;
                    if (rowsetDef != null) {
                        dataSourceRetVal = GetDesignTimeDataSource(dataSourceSelected, iMinRowCount, out bDummyDataSource);
                        if (dataSourceRetVal is DataView) {
                            ((DataView)dataSourceRetVal).Table.TableName = rowsetDef.Name;
                        }
                    }
                }
                strRetVal = base.GetDesignTimeHtml();
                if (!string.IsNullOrEmpty(strErrors)){
                    strRetVal += string.Format("<span style=\"color: Red; font-weight:bold\">{0}</span>", strErrors);
                }
                return(strRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// 
            /// </summary>
            /// <param name="serviceProvider">  Service provider</param>
            /// <param name="control">          Container</param>
            /// <param name="strCtlIdToFind">   Name of the control to find</param>
            /// <returns></returns>
            //
            //********************************************************************************
            internal static Control FindControl(IServiceProvider    serviceProvider, 
                                                Control             control, 
                                                string              strCtlIdToFind) {
                IDesignerHost host;
                IContainer    container;
                Control       ctlRetVal = null;

                if (string.IsNullOrEmpty(strCtlIdToFind)) {
                    throw new ArgumentNullException("strCtlIdToFind");
                }
                if (serviceProvider != null) {
                    host = (IDesignerHost)serviceProvider.GetService(typeof(IDesignerHost));
                    if (host != null) {
                        container = host.Container;
                        if (container != null) {
                            ctlRetVal = (container.Components[strCtlIdToFind] as Control);
                        }
                    }
                }
                return(ctlRetVal);
            }
        } // DataGridRowsetDesigner
    } // Class DataGridRowset
} // Namespace
