using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.IO;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;

namespace RepPP.Toolkit.Web {
	/// <summary>
	/// Summary description for WebUtil.
	/// </summary>
	public class WebUtil : Util {
        /// <summary>Field web control type</summary>
        public enum FieldWebControlTypeE {
            /// <summary>Textbox </summary>
            FldTextBox          = 0,
            /// <summary>Calendar </summary>
            FldCalendar         = 1,
            /// <summary>Checkbox list </summary>
            FldCheckBoxList     = 2,
            /// <summary>Dropdown list </summary>
            FldDropDownList     = 3,
            /// <summary>Label </summary>
            FldLabel            = 4,
            /// <summary>Listbox </summary>
            FldListBox          = 5,
            /// <summary>Radio button list </summary>
            FldRadioButtonList  = 6,
            /// <summary>Label attached to a field value </summary>
            FldValue            = 7
        }

        //********************************************************************************
        //
        /// <summary>
        /// EncodeString:                   Encode a string so it doesn't contains the specified characters
        /// </summary>
        /// <param name="str">              String array to encode</param>
        /// <param name="arrForbiddenChar"> Array of forbidden character. No more than 9 forbidden characters are allowed and '@' cannot be one of them</param>
        /// <returns>
        /// Resulting string
        /// </returns>
        //
        //********************************************************************************
        static public string EncodeString(string str, char[] arrForbiddenChar) {
            if (arrForbiddenChar.Length > 9) {
                throw new ApplicationException("No more than 9 forbidden characters are allowed");
            }
            str = str.Replace("@", "@0");
            for (int iIndex = 0; iIndex < arrForbiddenChar.Length; iIndex++) {
                str = str.Replace(arrForbiddenChar[iIndex].ToString(), "@" + iIndex.ToString());
            }
            return(str);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DecodeString:                   Decode an encoded string
        /// </summary>
        /// <param name="str">              String array to decode</param>
        /// <param name="arrForbiddenChar"> Array of forbidden character. No more than 9 forbidden characters are allowed and '@' cannot be one of them</param>
        /// <returns>
        /// Resulting string
        /// </returns>
        //
        //********************************************************************************
        static public string DecodeString(string str, char[] arrForbiddenChar) {
            if (arrForbiddenChar.Length > 9) {
                throw new ApplicationException("No more than 9 forbidden characters are allowed");
            }
            for (int iIndex = 0; iIndex < arrForbiddenChar.Length; iIndex++) {
                str = str.Replace("@" + iIndex.ToString(), arrForbiddenChar[iIndex].ToString());
            }
            str = str.Replace("@0", "@");
            return(str);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EncodeStringArray:              Encode a string array into a string which can be passed in HTML
        /// </summary>
        /// <param name="arrStr">           String array to encode</param>
        /// <param name="arrForbiddenChar"> Array of forbidden character. No more than 9 forbidden characters are allowed and '@' cannot be one of them
        ///                                 The first forbidden character is used as the array separator.</param>
        /// <returns>
        /// Resulting string
        /// </returns>
        //
        //********************************************************************************
        static public string EncodeStringArray(string[] arrStr, char[] arrForbiddenChar) {
            string[]    arrTmp;
            int         iIndex;

            arrTmp  = new string[arrStr.Length];
            iIndex  = 0;
            foreach (string str in arrStr) {
                arrTmp[iIndex++] = EncodeString(str, arrForbiddenChar);
            }
            return(String.Join(arrForbiddenChar[0].ToString(), arrTmp));
        }

        //********************************************************************************
        //
        /// <summary>
        /// DecodeStringArray:              Decode a string coming from HTML into a string array
        /// </summary>
        /// <param name="strVal">           String to decode</param>
        /// <param name="arrForbiddenChar"> Array of forbidden character. No more than 9 forbidden characters are allowed and '@' cannot be one of them
        ///                                 The first forbidden character is used as the array separator.</param>
        /// <returns>
        /// Resulting string array
        /// </returns>
        //
        //********************************************************************************
        static public string[] DecodeStringArray(string strVal, char[] arrForbiddenChar) {
            string[]    arrRetVal;
            int         iIndex;
            
            arrRetVal = strVal.Split(arrForbiddenChar[0]);
            iIndex    = 0;
            foreach (string str in arrRetVal) {
                arrRetVal[iIndex++] = DecodeString(str, arrForbiddenChar);
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EncodeStringArray:      Encode a string array into a string which can be passed in HTML
        /// </summary>
        /// <param name="arrStr">   String array to encode</param>
        /// <returns>
        /// Resulting string
        /// </returns>
        //
        //********************************************************************************
        static public string EncodeStringArray(string[] arrStr) {
            return(EncodeStringArray(arrStr, new char[] { ':', ',', ';' }));
        }

        //********************************************************************************
        //
        /// <summary>
        /// DecodeStringArray:      Decode a string coming from HTML into a string array
        /// </summary>
        /// <param name="strVal">   String to decode</param>
        /// <returns>
        /// Resulting string array
        /// </returns>
        //
        //********************************************************************************
        static public string[] DecodeStringArray(string strVal) {
            return(DecodeStringArray(strVal, new char[] { ':', ',', ';' }));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetDecodeStringArrayJSFunction: Gets the javascript function which decode a string to an
        ///                                 string array
        /// </summary>
        /// <returns>
        /// Javascript function
        /// </returns>
        //
        //********************************************************************************
        static public string GetDecodeStringArrayJSFunction() {
            return(
                   "function DecodeStringArray(strVal) {\r\n" +
                   "    var arrRetVal;\r\n" + 
                   "    var iIndex;\r\n" +
                   "\r\n" +
                   "    arrRetVal = strVal.split(\":\");\r\n" +
                   "    for (iIndex = 0; iIndex < arrRetVal.length; iIndex++) {\r\n" +
                   "        arrRetVal[iIndex] = arrRetVal[iIndex].replace(/@1/, \":\").replace(/@2/, \",\").replace(/@3/, \";\").replace(/@0/, \"@\");\r\n" +
                   "    }\r\n" +
                   "    return(arrRetVal);\r\n" +
                   "}\r\n\r\n");
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetEncodeStringArrayJSFunction: Gets the javascript function which encode an array to a
        ///                                 string so it can be passed in HTML
        /// </summary>
        /// <returns>
        /// Javascript function
        /// </returns>
        //
        //********************************************************************************
        static public string GetEncodeStringArrayJSFunction() {
            return(
                   "function EncodeStringArray(arrVal) {\r\n" +
                   "    var arrTmp;\r\n" +
                   "    var iIndex;\r\n" +
                   "\r\n" +
                   "    arrTmp = new Array();\r\n" +
                   "    for (iIndex = 0; iIndex < arrVal.length; iIndex++) {\r\n" +
                   "        arrTmp[iIndex] = arrTmp[iIndex].replace(/@/, \"@0\").replace(/:/,\"@1\").replace(/,/,\"@2\").replace(/;/,\"@3\");\r\n" +
                   "    }\r\n" +
                   "    return(arrTmp.join(\":\"));\r\n" +
                   "}\r\n\r\n");
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsClientScriptBlockRegistered:  Check if the script routine is registred
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strKey">           Key</param>
        /// <returns>
        /// true if present, false if not
        /// </returns>
        //
        //********************************************************************************
        static public bool IsClientScriptBlockRegistered(System.Web.UI.Page page, string strKey) {
            bool    bRetVal;
            bRetVal = page.ClientScript.IsClientScriptBlockRegistered(strKey);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterClientScriptBlock:      Regsiter a script routine
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strKey">           Key</param>
        /// <param name="strCode">          Code to register</param>
        /// <returns>
        /// true if present, false if not
        /// </returns>
        //
        //********************************************************************************
        static public void RegisterClientScriptBlock(System.Web.UI.Page page, string strKey, string strCode) {

            page.ClientScript.RegisterClientScriptBlock(typeof(Util),
                                                        strKey,
                                                        strCode);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterArrayDeclaration:      Register an array declaration routine
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="arrayName">        Name of the array declaration</param>
        /// <param name="arrayValue">       Array declaration text</param>
        //
        //********************************************************************************
        static public void RegisterArrayDeclaration(System.Web.UI.Page page, string arrayName, string arrayValue) {

            page.ClientScript.RegisterArrayDeclaration(arrayName, arrayValue);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterStartupScript:          Regsiter a startup script routine
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strKey">           Key</param>
        /// <param name="strCode">          Code to register</param>
        /// <returns>
        /// true if present, false if not
        /// </returns>
        //
        //********************************************************************************
        static public void RegisterStartupScript(System.Web.UI.Page page, string strKey, string strCode) {

            page.ClientScript.RegisterStartupScript(typeof(Util),
                                                    strKey,
                                                    strCode);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetClientValidatedPostback:     Gets the postback function that will trigger the validation
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strArg">           Argument</param>
        /// <param name="ctl">              Control which trigger the postback</param>
        /// <returns>
        /// Postback function
        /// </returns>
        //
        //********************************************************************************
        static public string GetClientValidatedPostback(Page page, Control ctl, string strArg) {
            string strRetVal;
            string strPostBackEventReference;

            strPostBackEventReference = GetPostBackEventReference(page, ctl, strArg);
            strRetVal                 = "{if (typeof(Page_ClientValidate) != 'function' ||  Page_ClientValidate()) " + 
                                        strPostBackEventReference +
                                        "} ";
            return (strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetPostBackEventReference:      Gets the postack function
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strArg">           Argument</param>
        /// <param name="ctl">              Control which trigger the postback</param>
        /// <returns>
        /// Postback function
        /// </returns>
        //
        //********************************************************************************
        static public string GetPostBackEventReference(Page page, Control ctl, string strArg) {
            string strRetVal;

            strRetVal = page.ClientScript.GetPostBackEventReference(ctl, strArg);
            return (strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetPostBackClientHyperlink:     Gets the postack hyperlink function
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strArg">           Argument</param>
        /// <param name="ctl">              Control which trigger the postback</param>
        /// <returns>
        /// Postback function
        /// </returns>
        //
        //********************************************************************************
        static public string GetPostBackClientHyperlink(Page page, Control ctl, string strArg) {
            string strRetVal;

            strRetVal = page.ClientScript.GetPostBackClientHyperlink(ctl, strArg);
            return (strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterHiddenField:            Register a hidden field
        /// </summary>
        /// <param name="page">             Page containing the script</param>
        /// <param name="strFldName">       Field name</param>
        /// <param name="strFldVal">        Field value</param>
        //
        //********************************************************************************
        static public void RegisterHiddenField(Page page, string strFldName, string strFldVal) {
            page.ClientScript.RegisterHiddenField(strFldName, strFldVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterEncodeFunctions:    Register in a page the javascript functions used to encode/decode string array
        /// </summary>
        //
        //********************************************************************************
        static public void RegisterEncodeFunctions(System.Web.UI.Page page) {
            const string    strKey = "EncodeDecodeStringArray";
            string          strFunc;
            
			if (!IsClientScriptBlockRegistered(page, strKey)) {
                strFunc = "<script language='javascript'>\r\n<!--\r\n" +
                          GetEncodeStringArrayJSFunction() +
                          GetDecodeStringArrayJSFunction() +
                          "-->\r\n</script>\r\n";
				RegisterClientScriptBlock(page, strKey, strFunc);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// RenderStyleInString:    Render a style as a string
	    /// </summary>
	    /// <param name="ctl">      Web control</param>
	    /// <param name="style">    Style to render</param>
	    //
        //********************************************************************************
		static public string RenderStyleInString(WebControl ctl, Style style) {
            string                  strRetVal;
            PseudoHtmlTextWriter    htmlWriter;
            
            htmlWriter = new PseudoHtmlTextWriter();
            style.AddAttributesToRender(htmlWriter, ctl);
            htmlWriter.RenderBeginTag(HtmlTextWriterTag.A);
            strRetVal = htmlWriter.StyleText;
		    return(strRetVal);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// FindFieldValidator:     Find the field validator control
	    /// </summary>
	    /// <param name="page">     Page containing the control</param>
	    /// <param name="fldDef">   FieldDef</param>
	    //
        //********************************************************************************
        static public FldValidator FindFieldValidator(Page page, FieldDef fldDef) {
            FldValidator    validatorRetVal = null;
            FldFullName     fldFullName;
            Control         ctl;

            foreach (BaseValidator validator in page.Validators) {
                if (validator is FldValidator) {
                    ctl = page.FindControl(validator.ControlToValidate);
                    if (ctl != null && ctl is FldControlUtil.IFldControl) {
                        fldFullName = ((FldControlUtil.IFldControl)ctl).FieldFullName;
                        if (fldFullName != null                     &&
                            fldFullName.RowsetName == fldDef.RowsetDef.Name &&
                            fldFullName.FieldName == fldDef.Name) {
                            validatorRetVal = (validator as FldValidator);
                            break;
                        }
                    }
                }
            }
            return(validatorRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// FindControlUsingType:       Find a web control using its type
	    /// </summary>
	    /// <param name="ctls">         Collection of web controlsr</param>
	    /// <param name="typeCtl">      Type of the control to find</param>
	    //
        //********************************************************************************
        static public Control FindControlUsingType(ControlCollection ctls, Type typeCtl) {
            Control ctlRetVal = null;

            foreach (Control ctl in ctls) {
                if (ctl.GetType() == typeCtl) {
                    ctlRetVal = ctl;
                    break;
                } else {
                    ctlRetVal = FindControlUsingType(ctl.Controls, typeCtl);
                    if (ctlRetVal != null) {
                        break;
                    }
                }
            }
            return(ctlRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FindControlUsingID:       Find a web control using its type
        /// </summary>
        /// <param name="ctls">         Collection of web controlsr</param>
        /// <param name="strCtlID">     ID of the control</param>
        //
        //********************************************************************************
        static public Control FindControlUsingID(ControlCollection ctls, String strCtlID)
        {
            Control ctlRetVal = null;

            foreach (Control ctl in ctls) {
                if (ctl.ID == strCtlID) {
                    ctlRetVal = ctl;
                    break;
                } else {
                    ctlRetVal = FindControlUsingID(ctl.Controls, strCtlID);
                    if (ctlRetVal != null) {
                        break;
                    }
                }
            }
            return (ctlRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// FindAllFldControlsID:       Find all field web controls
	    /// </summary>
	    /// <param name="ctls">         Collection of web controlsr</param>
	    /// <param name="arrCtlID">     Array of control ID</param>
	    //
        //********************************************************************************
        static public void FindAllFldControlsID(ControlCollection ctls, ref ArrayList arrCtlID) {
            foreach (Control ctl in ctls) {
                if (ctl.Visible && ctl is FldControlUtil.IFldControl && ctl.ClientID != String.Empty && ctl.ClientID != null) {
                    arrCtlID.Add(ctl.ClientID);
                }
                FindAllFldControlsID(ctl.Controls, ref arrCtlID);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// GetFieldControlType:        Get the field web control type which can represent the field.
	    /// </summary>
	    /// <param name="fldDef">       FieldDef</param>
	    /// <returns>
	    /// Field web control type
	    /// </returns>
	    //
        //********************************************************************************
        static public FieldWebControlTypeE GetFieldControlType(RepPP.FieldDef fldDef) {
            FieldWebControlTypeE eRetVal;

            switch (fldDef.ControlType) {
            case CtlType.sdCtlCheck:
                eRetVal = FieldWebControlTypeE.FldCheckBoxList;
                break;
            case CtlType.sdCtlCombo:
                eRetVal = FieldWebControlTypeE.FldDropDownList;
                break;
            case CtlType.sdCtlList:
                eRetVal = FieldWebControlTypeE.FldListBox;
                break;
            case CtlType.sdCtlRadio:
                eRetVal = FieldWebControlTypeE.FldRadioButtonList;
                break;
            case CtlType.sdCtlStatic:
                eRetVal = FieldWebControlTypeE.FldLabel;
                break;
            default:
                eRetVal = FieldWebControlTypeE.FldTextBox;
                break;
            }
            return(eRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateFldControl:           Create a field web control instance
        /// </summary>
        /// <param name="eFldCtlType">  Control type</param>
        /// <returns>
        /// Field web control
        /// </returns>
	    //
        //********************************************************************************
        static public Control CreateFldControl(FieldWebControlTypeE eFldCtlType) {
            Control ctlRetVal;

            switch (eFldCtlType) {
            case FieldWebControlTypeE.FldTextBox:
                ctlRetVal = new FldTextBox();
                break;
            case WebUtil.FieldWebControlTypeE.FldCheckBoxList:
                ctlRetVal = new FldCheckBoxList();
                break;
            case FieldWebControlTypeE.FldDropDownList:
                ctlRetVal = new FldDropDownList();
                break;
            case FieldWebControlTypeE.FldValue:
                ctlRetVal = new FldValue();
                break;
            case FieldWebControlTypeE.FldListBox:
                ctlRetVal = new FldListBox();
                break;
            case FieldWebControlTypeE.FldRadioButtonList:
                ctlRetVal = new FldRadioButtonList();
                break;
            case FieldWebControlTypeE.FldCalendar:
                ctlRetVal = new FldCalendar();
                break;
            case FieldWebControlTypeE.FldLabel:
                ctlRetVal = new FldLabel();
                break;
            default :
                ctlRetVal = null;
                break;
            }
            return(ctlRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsWebDesignMode:                The definite indicator of DesignMode in WebForms
        /// </summary>
        //
        //********************************************************************************
        public static bool IsWebDesignMode() {
            return (System.Web.HttpContext.Current == null);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// PseudoHtmlTextWriter:       Pseudo Html text writer use to build the style attributes
	    /// </summary>
	    //
        //********************************************************************************
        private class PseudoHtmlTextWriter : HtmlTextWriter {
            private     string      m_strStyle;

            //********************************************************************************
            //
	        /// <summary>
	        /// ctor:                   Class constructor
	        /// </summary>
	        //
            //********************************************************************************
            public PseudoHtmlTextWriter() : base(new System.IO.StringWriter()) {
                m_strStyle = "";
            }

            //********************************************************************************
            //
	        /// <summary>
	        /// OnStyleAttributeRender: Called each time an attributes is rendered
	        /// </summary>
            /// <param name="name">     Name of the attribute</param>
            /// <param name="value">    Value of the attribute</param>
            /// <param name="key">      Key of the attribute</param>
            /// <returns>
            /// true if succeed, false if not
            /// </returns>
	        //
            //********************************************************************************
            protected override bool OnStyleAttributeRender(string name, string value, HtmlTextWriterStyle key) {
                if (m_strStyle != String.Empty) {
                    m_strStyle += " ";
                }
                m_strStyle = m_strStyle + name.ToUpper() + ":" + value + ";";
                return base.OnStyleAttributeRender (name, value, key);
            }

            //********************************************************************************
            //
	        /// <summary>
	        /// StyleText:              Result text of the style attributes. Set after the attributes
	        ///                         are rendered.
	        /// </summary>
	        //
            //********************************************************************************
            public string StyleText {
                get {
                    return(m_strStyle);
                }
            }
        } // Class PseudoHtmlTextWriter
        
	}
}
