using System;
using System.Collections.Generic;
using System.Text;
using RepPP;

namespace RepPP.Toolkit.Window.Ext {
    /// <summary>Implement a comparaison object</summary>
    public class DeltaObject : IDisposable {
        /// <summary>Type of differences</summary>
        public enum DiffTypeE {
            /// <summary>The component are the same</summary>
            Same             = 0,
            /// <summary>The component exist only in the first source</summary>
            ExistInSrc1Only  = 1,
            /// <summary>The component exist only in the second source</summary>
            ExistInSrc2Only  = 2,
            /// <summary>The component exist in both source but is different in each</summary>
            Different        = 3,
            /// <summary>The component exist in both source but value(s) tag(s) as MustNotCompare is/are different in each</summary>
            Equivalent       = 4
        };
        
        public class ProgressEventArgs : System.EventArgs {
            public  RowsetDef       RowsetDef;
            public  Rowset          Rowset;
            public  DiffTypeE       DifferenceType;
            public  int             ActualCount;
            public  bool            Cancel;
        };
        
        public delegate void            delProgressEvent(object sender, ProgressEventArgs e);
        
        public event delProgressEvent   ProgressEvent;
        
        /// <summary>Unique ID used to create RowsetTreeDef and RowsetDef</summary>
        static private int              m_iUniqueId = 1;
        /// <summary>Rep++ Application object</summary>
        private RepPP.Application       m_app;
        /// <summary>RowsetTreeDef used to define first data source</summary>
        protected RowsetTreeDef         m_rstdSrc1;
        /// <summary>RowsetTreeDef used to define second data source</summary>
        protected RowsetTreeDef         m_rstdSrc2;
        /// <summary>RowsetTree used to hold first data source</summary>
        protected RowsetTree            m_rstSrc1;
        /// <summary>RowsetTree used to hold second data source</summary>
        protected RowsetTree            m_rstSrc2;
        /// <summary>Repository connection</summary>
        private Connection              m_connRep;
        /// <summary>First source connection</summary>
        private Connection              m_connSrc1;
        /// <summary>true if the first connection has been opened by us</summary>
        private bool                    m_bOwningConn1;
        /// <summary>Second source connection</summary>
        private Connection              m_connSrc2;
        /// <summary>true if the second connection has been opened by us</summary>
        private bool                    m_bOwningConn2;
        /// <summary>Root of the list of different items</summary>
        private DeltaNode               m_deltaNodeRoot;
        /// <summary>Stack of the primary key</summary>
        protected Stack<PrimaryKeyItem> m_stackPrimaryKeyItem;
        /// <summary>List of created RowsetDefs</summary>
        private List<RowsetDef>         m_arrCreatedRowsetDefs;
        /// <summary>Starting point of the comparaison</summary>
        protected RowsetTreeDefNode     m_nodeStartRoot;

        //*********************************************************     
        //
        /// <summary>
        /// Create a DeltaObject object
        /// </summary>
        /// <param name="rstdSrc">   RowsetTreeDef defining data we want to compare</param>
        //  
        //*********************************************************     
        public DeltaObject(RowsetTreeDef rstdSrc) {
            m_app                   = rstdSrc.Application;
            m_arrCreatedRowsetDefs  = new List<RowsetDef>(128);
            m_connRep               = m_app.RepositoryConnection;
            m_connSrc1              = m_app.RepositoryConnection;
            m_connSrc2              = m_app.RepositoryConnection;
            m_bOwningConn1          = false;
            m_bOwningConn2          = false;
            m_rstdSrc1              = rstdSrc;
            m_rstdSrc2              = CloneRowsetTreeDef(m_rstdSrc1);
            m_rstSrc1               = m_rstdSrc1.RowsetTrees.Add();
            m_rstSrc2               = m_rstdSrc2.RowsetTrees.Add();
            m_deltaNodeRoot         = new DeltaNode(null, null, DiffTypeE.Same);
            m_stackPrimaryKeyItem   = new Stack<PrimaryKeyItem>(32);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose the object
        /// </summary>
        //  
        //*********************************************************     
        public void Dispose() {
            if (m_rstSrc2 != null) {
                m_rstSrc2.Delete();
                m_rstSrc2 = null;
            }
            if (m_rstSrc1 != null) {
                m_rstSrc1.Delete();
                m_rstSrc1 = null;
            }
            if (m_rstdSrc2 != null) {
                m_rstdSrc2.Delete();
                m_rstdSrc2 = null;
            }
            if (m_arrCreatedRowsetDefs != null) {
                foreach (RowsetDef rsd in m_arrCreatedRowsetDefs) {
                    rsd.Delete();
                }
                m_arrCreatedRowsetDefs = null;
            }
            if (m_bOwningConn1 && m_connSrc1 != null) {
                m_connSrc1.Close();
            }
            m_connSrc1 = null;
            if (m_bOwningConn2 && m_connSrc2 != null) {
                m_connSrc2.Close();
            }
            m_connSrc2 = null;
        }

        //*********************************************************     
        //
        /// <summary>
        /// Trigger the ProgressEvent
        /// </summary>
        /// <param name="e">    Event argument</param>
        //  
        //*********************************************************     
        protected virtual void OnProgressEvent(ProgressEventArgs e) {
            if (ProgressEvent != null) {
                ProgressEvent(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Trigger the ProgressEvent
        /// </summary>
        /// <param name="rsd">      RowsetDef</param>
        /// <param name="rs">       RowsetDef instance</param>
        /// <param name="eDiffType">Type of difference</param>
        //  
        //*********************************************************     
        private void OnProgressEvent(ProgressEventArgs e, RowsetDef rsd, Rowset rs, DiffTypeE eDiffType) {
            e.RowsetDef         = rsd;
            e.Rowset            = rs;
            e.DifferenceType    = eDiffType;
            OnProgressEvent(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Clone a Node and all its descendant
        /// </summary>
        /// <param name="nodeSrc">      RowsetTreeDefNode to clone</param>
        /// <param name="rstdNew">      New RowsetTreeDef receiving the new node</param>
        /// <param name="nodeFather">   Father node of the new one</param>
        //  
        //*********************************************************     
        private void CloneNode(RowsetTreeDefNode nodeSrc, RowsetTreeDef rstdNew, RowsetTreeDefNode nodeFather) {
            RowsetDef           rsdNew;
            RowsetTreeDefNode   nodeNew;
            
            rsdNew  = nodeSrc.RowsetDef.Clone("$DeltaGrp" + (m_iUniqueId++).ToString(), nodeSrc.RowsetDef.TableName, true);
            m_arrCreatedRowsetDefs.Add(rsdNew);
            nodeNew = nodeSrc.Clone(rstdNew, nodeFather, rsdNew, false);
            foreach (RowsetTreeDefNode nodeSrcChild in nodeSrc.Nodes) {
                CloneNode(nodeSrcChild, rstdNew, nodeNew);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Clone a RowsetTreeDef renaming the RowsetTreeDef and all contained RowsetDefs
        /// </summary>
        /// <param name="crdSrc">       RowsetTreeDef to clone</param>
        /// <returns>
        /// New cloned RowsetTreeDef
        /// </returns>
        //  
        //*********************************************************     
        private RowsetTreeDef CloneRowsetTreeDef(RowsetTreeDef rstdSrc) {
            RowsetTreeDef    rstdNew;
            
            rstdNew = rstdSrc.Clone("$DeltaCrd" + (m_iUniqueId++).ToString(), false);
            CloneNode(rstdSrc.Nodes[0], rstdNew, null);
            return(rstdNew);
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef used to define first data source
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDef Source1RowsetTreeDef {
            get {
                return(m_rstdSrc1);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef used to define second data source
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDef Source2RowsetTreeDef {
            get {
                return(m_rstdSrc2);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree used to hold first data source
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTree Source1RowsetTree {
            get {
                return(m_rstSrc1);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree used to hold second data source
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTree Source2RowsetTree {
            get {
                return(m_rstSrc2);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Connection used by repository 
        /// </summary>
        //  
        //*********************************************************     
        public Connection RepositoryConnection {
            get {
                return (m_connRep);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Connection used to read the first data source
        /// </summary>
        //  
        //*********************************************************     
        public Connection Source1Connection {
            get {
                return(m_connSrc1);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Connection used to read the second data source
        /// </summary>
        //  
        //*********************************************************     
        public Connection Source2Connection {
            get {
                return(m_connSrc2);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Associate a new connection to source 1
        /// </summary>
        /// <param name="strConnName">  Name of the connection</param>
        //  
        //*********************************************************     
        public bool OpenSource1Connection(string strConnName, string strPassword) {
            Connection  conn;
            bool        bOwningConn;

            if (m_app.RepositoryConnection.Name == strConnName) {
                conn        = m_app.RepositoryConnection;
                bOwningConn = false;
            } else {
                conn        = m_app.Connections.Open(strConnName, strPassword);
                bOwningConn = true;
            }
            if (conn != null) {
                if (m_bOwningConn1) {
                    m_connSrc1.Close();
                }
                m_connSrc1     = conn;
                m_bOwningConn1 = bOwningConn;
            }
            return(conn != null);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Associate a new connection to source 2
        /// </summary>
        /// <param name="strConnName">  Name of the connection</param>
        //  
        //*********************************************************     
        public bool OpenSource2Connection(string strConnName, string strPassword) {
            Connection  conn;
            bool        bOwningConn;

            if (m_app.RepositoryConnection.Name == strConnName) {
                conn        = m_app.RepositoryConnection;
                bOwningConn = false;
            } else {
                conn        = m_app.Connections.Open(strConnName, strPassword);
                bOwningConn = true;
            }
            if (conn != null) {
                if (m_bOwningConn2) {
                    m_connSrc2.Close();
                }
                m_connSrc2     = conn;
                m_bOwningConn2 = bOwningConn;
            }
            return(conn != null);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Find the node in the second source which correspond to the node in the first source.
        /// The RowsetTreeDef being clone, the name are all differents.
        /// </summary>
        /// <param name="nodeSrc1"> Node source 1</param>
        //  
        //*********************************************************     
        public RowsetTreeDefNode FindSource2Node(RowsetTreeDefNode nodeSrc1) {
            RowsetTreeDefNode   nodeRetVal;
            RowsetTreeDefNode   nodeSrc1Parent;
            RowsetTreeDefNode   nodeSrc2Parent;
            
            nodeSrc1Parent = nodeSrc1.Parent as RowsetTreeDefNode;
            if (nodeSrc1Parent == null) {
                nodeRetVal     = Source2RowsetTreeDef.Nodes[0];
            } else {
                nodeSrc2Parent = FindSource2Node(nodeSrc1Parent);
                nodeRetVal     = nodeSrc2Parent.Nodes[nodeSrc1.ChildId];
            }
            return(nodeRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Add the primary key for this RowsetDef only to the WHERE clause
        /// </summary>
        /// <param name="strWhere">WHERE clause to be modified</param>
        /// <param name="rsd">the group whose PK will be added to the WHERE clause</param>
        /// <returns>
        /// The modified WHERE clause
        /// </returns>
        //  
        //*********************************************************     
        protected virtual string AddPrimaryKeyClauseToWhere(string strWhere, RowsetDef rsd) {
            string  strRetVal   = "";
            int     iFldPKCount = 0;

            if (strWhere.Length == 0 ) {
                strRetVal = "WHERE ";
            } else {
                strRetVal = strWhere;
            }
            foreach (FieldDef fldDef in rsd.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    if (iFldPKCount > 0 || strWhere.Length > 0) {
                        strRetVal += " AND ";
                    }
                    strRetVal += fldDef.ColPhysName + "=:" + rsd.Name + "." + fldDef.Name;
                    iFldPKCount++;
                }
            }
            return(strRetVal);
        }        

        //*********************************************************     
        //
        /// <summary>
        /// Add the primary key for this RowsetDef to the select command
        /// </summary>
        /// <param name="strSqlCmd">Sql command to be modified</param>
        /// <param name="rsd">the RowsetDef whose PK will be added to the select command</param>
        /// <returns>
        /// The modified sql command
        /// </returns>
        //  
        //*********************************************************     
        protected virtual string AddPrimaryKeyClauseToSelectCommand(string strSqlCmd, RowsetDef rsd) {
            string  strRetVal;
            Tool    tool;
            
            tool      = m_app.Tool;
            strRetVal = tool.SetSqlWhere(strSqlCmd, AddPrimaryKeyClauseToWhere(tool.GetSqlWhere(strSqlCmd), rsd));
            return(strRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Add a component which is different in the difference tree
        /// </summary>
        /// <param name="rsSrc">        Rowset containing the guilty component</param>
        /// <param name="eDiffType">    Type of difference</param>
        /// <returns>
        /// The newly created delta node
        /// </returns>
        //  
        //*********************************************************     
        protected virtual DeltaNode AddGuiltyComponent(Rowset rsSrc, DiffTypeE eDiffType) {
            PrimaryKeyItem[]    arrPrimaryKeyItem;
            int                 iCount;
            DeltaNode           deltaNode;
            DeltaNode           deltaNodeParent;
            
            arrPrimaryKeyItem = m_stackPrimaryKeyItem.ToArray();
            iCount            = arrPrimaryKeyItem.Length;
            deltaNodeParent   = m_deltaNodeRoot;
            for (int iIndex = iCount - 1; iIndex > 0; iIndex--) {
                if (!deltaNodeParent.Child.TryGetValue(arrPrimaryKeyItem[iIndex], out deltaNode)) {
                    deltaNode = new DeltaNode(deltaNodeParent, arrPrimaryKeyItem[iIndex], DiffTypeE.Same);
                }
                deltaNodeParent = deltaNode;
            }
            deltaNode = new DeltaNode(deltaNodeParent, arrPrimaryKeyItem[0], eDiffType);
            return(deltaNode);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Build the sql command for the card hierarchies
        /// </summary>
        //  
        //*********************************************************     
        protected virtual void BuildSqlCommand() {
            Source1RowsetTreeDef.BuildSqlCommand(true);
            Source2RowsetTreeDef.BuildSqlCommand(true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Checks if the specified field must be compared
        /// </summary>
        /// <param name="fldDef">  FieldDef to check</param>
        /// <returns>
        /// true if must be compared
        /// </returns>
        //  
        //*********************************************************     
        protected virtual bool MustCompareField(FieldDef fldDef) {
            return(fldDef.IsDatabaseLinked && !fldDef.IsPrimaryKey && !fldDef.IsUserFlagOn);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Checks if the data contains in the 2 fields is the same
        /// </summary>
        /// <param name="fldSrc1">   First field</param>
        /// <param name="fldSrc2">   Second field</param>
        /// <returns>
        /// true if equivalent
        /// </returns>
        //  
        //*********************************************************     
        protected virtual bool IsFieldsDataEqual(Field fldSrc1, Field fldSrc2) {
            return(fldSrc1.Value == fldSrc2.Value);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Compare if the data of the two lines are identical, equivalent or equal.
        /// </summary>
        /// <param name="giSrc1">   Rowset of the first source</param>
        /// <param name="giSrc2">   Rowset of the second source</param>
        /// <returns>
        /// Line's status
        /// </returns>
        //  
        //*********************************************************     
        protected virtual DiffTypeE CompareLinesData(Rowset rsSrc1, Rowset rsSrc2) {
            DiffTypeE   eDiffType = DiffTypeE.Same;
            Field       fldSrc2;

            foreach (Field fldSrc1 in rsSrc1.Fields) {
                fldSrc2 = rsSrc2.Fields[fldSrc1.FieldDef.Name];
                if (!IsFieldsDataEqual(fldSrc1, fldSrc2)) {
                    if (MustCompareField(fldSrc1.FieldDef)) {
                        eDiffType = DiffTypeE.Different;
                        break;
                    } else {
                        eDiffType = DiffTypeE.Equivalent;
                    }
                }
            }
            return(eDiffType);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Checks if we must compare this kind of item
        /// </summary>
        /// <param name="rsd">          RowsetDef holding this item</param>
        /// <returns>
        /// true if yes, false if not
        /// </returns>
        //  
        //*********************************************************     
        protected virtual bool DoWeCompareThisKindOfItem(RepPP.RowsetDef rsd) {
            return(true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Checks if we must compare this specific item
        /// </summary>
        /// <param name="rs">           Rowset containing the item</param>
        /// <returns>
        /// true if yes, false if not
        /// </returns>
        //  
        //*********************************************************     
        protected virtual bool DoWeCompareThisItem(RepPP.Rowset rs) {
            return(true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Compare data of the child of the specified rowsets
        /// </summary>
        /// <param name="rsFather1">    Father Rowset for source 1</param>
        /// <param name="rsFather2">    Father Rowset for source 2</param>
        /// <param name="e">            Event arguments</param>
        /// <param name="iDiffCount">   Difference counts</param>
        //  
        //*********************************************************     
        protected virtual void CompareChildNode(Rowset rsFather1, Rowset rsFather2, ProgressEventArgs e, ref int iDiffCount) {
            Rowset   rsSrc2Child;
            int      iIndex = 0;
            int      iCount = rsFather1.ChildRowsets.Count;
            Rowset   rsSrc1Child;
            
            while (iIndex < iCount && e.Cancel == false) {
                rsSrc1Child = rsFather1.ChildRowsets[iIndex];
                rsSrc2Child = rsFather2.ChildRowsets[rsSrc1Child.RowsetTreeDefNode.ChildId];
                CompareNode(rsSrc1Child, rsSrc2Child, e, ref iDiffCount);
                iIndex++;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Compare data specified by the card groups
        /// </summary>
        /// <param name="rsSrc1">       Rowset for source 1</param>
        /// <param name="rsSrc2">       Rowset for source 2</param>
        /// <param name="e">            Event arguments</param>
        /// <param name="iDiffCount">   Difference counts</param>
        //  
        //*********************************************************     
        protected void CompareNode(Rowset rsSrc1, Rowset rsSrc2, ProgressEventArgs e, ref int iDiffCount) {
            string                          strSrc1SqlCmd;
            string                          strSrc2SqlCmd;
            SqlCursor                       sqlCursorSrc1;
            SqlCursor                       sqlCursorSrc2;
            Connection                      connSrc1;
            Connection                      connSrc2;
            RowsetTreeDefNode               nodeSrc1;
            RowsetTreeDefNode               nodeSrc2;
            Dictionary<PrimaryKeyItem, int> hashSrc1PK;
            PrimaryKeyItem                  primaryKeyItem;

            if (DoWeCompareThisKindOfItem(rsSrc1.RowsetDef)) {
                nodeSrc1      = rsSrc1.RowsetTreeDefNode;
                nodeSrc2      = rsSrc2.RowsetTreeDefNode;
                strSrc1SqlCmd = nodeSrc1.SelectSqlCommand;
                strSrc2SqlCmd = nodeSrc2.SelectSqlCommand;
                strSrc2SqlCmd = AddPrimaryKeyClauseToSelectCommand(strSrc2SqlCmd, nodeSrc1.RowsetDef);
                connSrc1      = Source1Connection;
                connSrc2      = Source2Connection;
                hashSrc1PK    = new Dictionary<PrimaryKeyItem,int>(8192);
                if (connSrc1.Execute(strSrc1SqlCmd, SqlOptimization.sdOptFull, out sqlCursorSrc1) != 0) {
                    throw new System.ApplicationException(connSrc1.ErrorMessage);
                } else {
                    using(sqlCursorSrc1) {
                        while (sqlCursorSrc1.Next() == 0 && e.Cancel == false) {
                            e.ActualCount++;
                            if (connSrc2.Execute(strSrc2SqlCmd, SqlOptimization.sdOptFull, out sqlCursorSrc2) != 0) {
                                throw new System.ApplicationException(connSrc2.ErrorMessage);
                            } else {
                                using (sqlCursorSrc2) {
                                    if (DoWeCompareThisItem(rsSrc1)) {
                                        primaryKeyItem = new PrimaryKeyItem(rsSrc1.RowsetDef.Name, rsSrc1);
                                        hashSrc1PK.Add(primaryKeyItem, 0);
                                        m_stackPrimaryKeyItem.Push(primaryKeyItem);
                                        if (sqlCursorSrc2.Next() == 0) {
                                            CompareVertRowSets(rsSrc1, rsSrc2, e, ref iDiffCount);
                                            CompareChildNode(rsSrc1, rsSrc2, e, ref iDiffCount);
                                        } else {
                                            iDiffCount++;
                                            AddGuiltyComponent(rsSrc1, DiffTypeE.ExistInSrc1Only);
                                            OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc1, DiffTypeE.ExistInSrc1Only);
                                        }
                                        m_stackPrimaryKeyItem.Pop();
                                    }
                                }
                            }
                        }
                    }
                }
                strSrc2SqlCmd = nodeSrc2.SelectSqlCommand;
                if (connSrc2.Execute(strSrc2SqlCmd, SqlOptimization.sdOptFull, out sqlCursorSrc2) != 0) {
                    throw new System.ApplicationException(connSrc2.ErrorMessage);
                } else {
                    using(sqlCursorSrc2) {
                        while (sqlCursorSrc2.Next() == 0 && e.Cancel == false) {
                            e.ActualCount++;
                            if (DoWeCompareThisItem(rsSrc2)) {
                                primaryKeyItem = new PrimaryKeyItem(rsSrc1.RowsetDef.Name, rsSrc2);
                                if (!hashSrc1PK.ContainsKey(primaryKeyItem)) {
                                    m_stackPrimaryKeyItem.Push(primaryKeyItem);
                                    iDiffCount++;
                                    AddGuiltyComponent(rsSrc2, DiffTypeE.ExistInSrc2Only);
                                    m_stackPrimaryKeyItem.Pop();
                                    OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc2, DiffTypeE.ExistInSrc2Only);
                                } else {
                                    OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc2, DiffTypeE.Same);
                                }
                            }
                        }
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Compare two vertical rowsets
        /// </summary>
        /// <param name="rsSrc1">       Rowset for source 1</param>
        /// <param name="rsSrc2">       Rowset for source 2</param>
        /// <param name="e">            Event arguments</param>
        /// <param name="iDiffCount">   Difference counts</param>
        //  
        //*********************************************************
        protected void CompareVertRowSets(Rowset rsSrc1, Rowset rsSrc2, ProgressEventArgs e, ref int iDiffCount) {
            DiffTypeE eDiffType;

            eDiffType = CompareLinesData(rsSrc1, rsSrc2);
            switch (eDiffType) {
            case DiffTypeE.Different:
                iDiffCount++;
                AddGuiltyComponent(rsSrc1, DiffTypeE.Different);
                OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc1, DiffTypeE.Different);
                break;
            case DiffTypeE.Equivalent:
                AddGuiltyComponent(Source1RowsetTree.RootRowset, DiffTypeE.Equivalent);
                OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc1, DiffTypeE.Equivalent);
                break;
            case DiffTypeE.Same:
                OnProgressEvent(e, rsSrc1.RowsetDef, rsSrc1, DiffTypeE.Same);
                break;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Build the WHERE clause of all ancester plus the current card group
        /// </summary>
        /// <param name="node">           RowsetTreeDefNode</param>
        /// <returns>
        /// WHERE clause without the WHERE
        /// </returns>
        //  
        //*********************************************************     
        private string BuildWhereClause(RowsetTreeDefNode node) {
            string  strRetVal = "";
            
            if (node.Parent != null) {
                strRetVal = BuildWhereClause(node.Parent as RepPP.RowsetTreeDefNode);
            }
            foreach (FieldDef fldDef in node.RowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    if  (strRetVal != "") {
                        strRetVal += " AND ";
                    }
                    strRetVal += fldDef.ColPhysName + " = :" + fldDef.RowsetDef.Name + "." + fldDef.Name;
                }
            }
            return(strRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Push ancester primary keys to the stack
        /// </summary>
        /// <param name="nodeSrc1"> RowsetTreeDefNode to be used as root of source 1</param>
        /// <param name="nodeSrc1"> RowsetTreeDefNode to be used as root of source 2</param>
        /// <returns>
        /// Nb of primary key pushed
        /// </returns>
        //  
        //*********************************************************     
        private int PushAncesterKey(RowsetTreeDefNode nodeSrc1, RowsetTreeDefNode nodeSrc2) {
            int                     iRetVal = 0;
            Rowset                  rsSrc1;
            Rowset                  rsSrc2;
            RowsetTreeDefNode       nodeSrc1Parent;
            RowsetTreeDefNode       nodeSrc2Parent;
            DoublePrimaryKeyItem    primaryKeyItem;
            
            rsSrc1          = nodeSrc1.RowsetDef.ActiveRowset;
            rsSrc2          = nodeSrc2.RowsetDef.ActiveRowset;
            nodeSrc1Parent  = nodeSrc1.Parent as RowsetTreeDefNode;
            nodeSrc2Parent  = nodeSrc2.Parent as RowsetTreeDefNode;
            if (nodeSrc1Parent != null && nodeSrc2Parent != null) {
                iRetVal = PushAncesterKey(nodeSrc1Parent, nodeSrc2Parent);
            }
            primaryKeyItem = new DoublePrimaryKeyItem(rsSrc1.RowsetDef.Name, rsSrc1, rsSrc2);
            m_stackPrimaryKeyItem.Push(primaryKeyItem);
            return(iRetVal + 1);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Find the delta between the two sources
        /// </summary>
        /// <param name="nodeSrc1"> RowsetTreeDefNode to be used as root</param>
        /// <returns>
        /// Nb of compared components
        /// </returns>
        //  
        //*********************************************************     
        public virtual int FindDelta(RowsetTreeDefNode nodeSrc1, out int iDiffCount) {
            int                             iRetVal = 0;
            int                             iFatherCount;
            RowsetTreeDefNode               nodeSrc2;
            ProgressEventArgs               e;

            iDiffCount      = 0;
            e               = new ProgressEventArgs();
            e.ActualCount   = 0;
            e.Cancel        = false;
            m_nodeStartRoot = nodeSrc1;
            nodeSrc2        = FindSource2Node(nodeSrc1);
            iFatherCount    = PushAncesterKey(nodeSrc1, nodeSrc2);
            m_deltaNodeRoot.Child.Clear();
            Source1RowsetTree.RootRowset.Select(true);
            Source2RowsetTree.RootRowset.Select(true);
            BuildSqlCommand();
            // Compare root item            
            e.ActualCount++;
            CompareVertRowSets(Source1RowsetTree.RootRowset, Source2RowsetTree.RootRowset, e, ref iDiffCount);
            // Compare all children
            CompareChildNode(nodeSrc1.RowsetDef.ActiveRowset,
                             nodeSrc2.RowsetDef.ActiveRowset,
                             e,
                             ref iDiffCount);
            OnProgressEvent(e, null, null, DiffTypeE.Same);
            iRetVal = e.ActualCount;
            while (iFatherCount != 0) {
                iFatherCount--;
                m_stackPrimaryKeyItem.Pop();
            }
            return(iRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Set the primary key value
        /// </summary>
        /// <param name="deltaNode">        Delta Node</param>
        /// <param name="giSrc">            Rowset of this node</param>
        /// <param name="bSecondSourcePK">  Use the second source primary key if true and two primary key specified</param>
        //  
        //*********************************************************     
        private void SetPrimaryKey(DeltaNode deltaNode, Rowset rsSrc, bool bSecondSourcePK) {
            int                     iIndex;
            PrimaryKeyItem          primaryKey;
            DoublePrimaryKeyItem    doublePrimaryKey;
            
            if (deltaNode.Parent != m_deltaNodeRoot) {
                SetPrimaryKey(deltaNode.Parent, rsSrc.FatherRowset, bSecondSourcePK);
            }
            primaryKey = deltaNode.PrimaryKeyItem;
            if (bSecondSourcePK && primaryKey is DoublePrimaryKeyItem) {
                doublePrimaryKey = (DoublePrimaryKeyItem)primaryKey;
                primaryKey       = doublePrimaryKey.SecondPrimaryKeyItem;
            }
            iIndex = 0;
            foreach (Field fld in rsSrc.Fields) {
                if (fld.FieldDef.IsPrimaryKey) {
                    fld.Value = primaryKey[iIndex++];
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Read the data of the specified node
        /// </summary>
        /// <param name="conn">             Connection to use</param>
        /// <param name="deltaNode">        Delta Node</param>
        /// <param name="rsSrc">            Rowset of this node</param>
        /// <param name="bSecondSourcePK">  Use the second source primary key if true and two primary key specified</param>
        /// <returns>
        /// true if succeed, false if not.
        /// </returns>
        //  
        //*********************************************************     
        private bool ReadData(Connection conn, DeltaNode deltaNode, Rowset rsSrc, bool bSecondSourcePK) {
            bool        bRetVal;
            string      strSqlCmd;
            SqlCursor   cur;
            
            SetPrimaryKey(deltaNode, rsSrc, bSecondSourcePK);
            strSqlCmd = rsSrc.RowsetTreeDefNode.SelectSqlCommand;
            strSqlCmd = AddPrimaryKeyClauseToSelectCommand(strSqlCmd, rsSrc.RowsetTreeDefNode.RowsetDef);
            if (conn.Execute(strSqlCmd, SqlOptimization.sdOptNone, out cur) != 0) {
                throw new System.ApplicationException(conn.ErrorMessage);
            }
            using(cur) {
                bRetVal = (cur.Next() == 0);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Read the data of the specified node
        /// </summary>
        /// <param name="deltaNode">    Delta Node</param>
        /// <returns>
        /// true if succeed, false if not.
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool ReadData(DeltaNode deltaNode) {
            bool                bRetVal;
            RowsetTreeDefNode   nodeSrc1;
            RowsetTreeDefNode   nodeSrc2;
            Rowset              rsSrc1;
            Rowset              rsSrc2;
            
            nodeSrc1    = m_rstdSrc1.FindNode(deltaNode.PrimaryKeyItem.m_strRowsetDefName);
            nodeSrc2    = FindSource2Node(nodeSrc1);
            rsSrc1      = m_rstSrc1.GetCurrentRowset(nodeSrc1);
            rsSrc2      = m_rstSrc2.GetCurrentRowset(nodeSrc2);
            switch(deltaNode.DifferenceType) {
            case DiffTypeE.ExistInSrc1Only:
                bRetVal = ReadData(m_connSrc1, deltaNode, rsSrc1, false);
                rsSrc2.Empty();
                break;
            case DiffTypeE.ExistInSrc2Only:
                rsSrc1.Empty();
                bRetVal = ReadData(m_connSrc2, deltaNode, rsSrc2, true);
                break;
            default:
                bRetVal = ReadData(m_connSrc1, deltaNode, rsSrc1, false) &&
                          ReadData(m_connSrc2, deltaNode, rsSrc2, true);
                break;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Root delta node
        /// </summary>
        //  
        //*********************************************************     
        public DeltaNode RootDeltaNode {
            get {
                return(m_deltaNodeRoot);
            }
        }
        
        /// <summary>Class holding a primary key value of a component</summary>
        public class PrimaryKeyItem : List<string>, IEquatable<PrimaryKeyItem> {
            /// <summary>RowsetDef identifying the type of the component</summary>
            public string   m_strRowsetDefName;
            /// <summary>First Genral purpose tag</summary>
            public object   m_objTag1 = null;
            /// <summary>Second Genral purpose tag</summary>
            public object   m_objTag2 = null;

            //*********************************************************     
            //
            /// <summary>
            /// Create a primary key item
            /// </summary>
            /// <param name="strRowsetDefName"> Name of the RowsetDef</param>
            /// <param name="gi">               RowsetDef instance containing the value of the key</param>
            //  
            //*********************************************************     
            public PrimaryKeyItem(string strRowsetDefName, Rowset gi) : base(4) {
                m_strRowsetDefName    = strRowsetDefName;
                foreach (Field fld in gi.Fields) {
                    if (fld.FieldDef.IsPrimaryKey) {
                        Add(fld.Value);
                    }
                }
            }

            //*********************************************************     
            //
            /// <summary>
            /// Return a hash code of the structure
            /// </summary>
            /// <returns>
            /// Hash code
            /// </returns>
            //  
            //*********************************************************     
            public override int GetHashCode() {
                int iRetVal = m_strRowsetDefName.GetHashCode();
                
                foreach (string str in this) {
                    iRetVal += str.GetHashCode();
                }
                return(iRetVal);
            }

            //*********************************************************     
            //
            /// <summary>
            /// Determine if two primary keys are the same
            /// </summary>
            /// <returns>
            /// true if yes
            /// </returns>
            //  
            //*********************************************************     
            public bool Equals(PrimaryKeyItem other) {
                bool    bRetVal;
                int     iCount;
                
                iCount = Count;
                bRetVal = (m_strRowsetDefName == other.m_strRowsetDefName && iCount == other.Count);
                if (bRetVal) {
                    for (int iIndex = 0; iIndex < iCount; iIndex++) {
                        if (this[iIndex] != other[iIndex]) {
                            bRetVal = false;
                            break;
                        }
                    }
                }
                return(bRetVal);
            }
        }; // Class PrimaryKeyItem
        
        /// <summary>Class holding the primary key value of the two source component</summary>
        public class DoublePrimaryKeyItem : PrimaryKeyItem {
            /// <summary>Primary key of the second source component</summary>
            PrimaryKeyItem  m_pkItemSecond;

            //*********************************************************     
            //
            /// <summary>
            /// Create a primary key item
            /// </summary>
            /// <param name="strRowsetDefName"> Name of the RowsetDef</param>
            /// <param name="rs1">               Rowset containing the value of the key</param>
            /// <param name="rs2">               Rowset containing the value of the key</param>
            //  
            //*********************************************************     
            public DoublePrimaryKeyItem(string strRowsetDefName, RepPP.Rowset rs1, RepPP.Rowset rs2) : base(strRowsetDefName, rs1) {
                m_pkItemSecond = new PrimaryKeyItem(strRowsetDefName, rs2);
            }

            //*********************************************************     
            //
            /// <summary>
            /// Primary key of the second source
            /// </summary>
            //  
            //*********************************************************     
            public PrimaryKeyItem SecondPrimaryKeyItem {
                get {
                    return(m_pkItemSecond);
                }
            }
        }; // Class DoublePrimaryKeyItem
        
        /// <summary>Delta node</summary>
        public class DeltaNode {
            /// <summary>Type of difference</summary>
            private DiffTypeE                               m_eDiffType;
            /// <summary>Parent of the item</summary>
            private DeltaNode                               m_deltaNodeParent;
            /// <summary>List of childs</summary>
            private Dictionary<PrimaryKeyItem,DeltaNode>    m_hashNodeChilds;
            /// <summary>List of childs</summary>
            private PrimaryKeyItem                          m_primKeyItem;

            //*********************************************************     
            //
            /// <summary>
            /// Create a delta node
            /// </summary>
            /// <param name="primaryKeyItem">   Primary key of the item</param>
            /// <param name="eDiffType">        Difference type</param>
            //  
            //*********************************************************     
            public DeltaNode(DeltaNode deltaNodeParent, PrimaryKeyItem primaryKeyItem, DiffTypeE eDiffType) {
                m_eDiffType       = eDiffType;
                m_deltaNodeParent = deltaNodeParent;
                m_primKeyItem     = primaryKeyItem;
                m_hashNodeChilds  = new Dictionary<PrimaryKeyItem,DeltaNode>(256);
                if (deltaNodeParent != null) {
                    deltaNodeParent.Child.Add(primaryKeyItem, this);
                }
            }

            //*********************************************************     
            //
            /// <summary>
            /// Parent of the delta node
            /// </summary>
            //  
            //*********************************************************     
            public DeltaNode Parent {
                get {
                    return(m_deltaNodeParent);
                }
            }

            //*********************************************************     
            //
            /// <summary>
            /// Child of the delta node
            /// </summary>
            //  
            //*********************************************************     
            public Dictionary<PrimaryKeyItem,DeltaNode> Child {
                get {
                    return(m_hashNodeChilds);
                }
            }

            //*********************************************************     
            //
            /// <summary>
            /// Type of difference
            /// </summary>
            //  
            //*********************************************************     
            public DiffTypeE DifferenceType {
                get {
                    return(m_eDiffType);
                }
            }

            //*********************************************************     
            //
            /// <summary>
            /// Primary key item
            /// </summary>
            //  
            //*********************************************************     
            public PrimaryKeyItem PrimaryKeyItem {
                get {
                    return(m_primKeyItem);
                }
            }
        } // Class DifferentItem
    } // Class DeltaObject
} // Namespace
