using DevComponents.DotNetBar;
using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Drawing;

namespace RepPP.Toolkit.Window.Ext {
    
    //**************************************************************************
    //
    /// <summary>
    /// TabControlExt:			        Tab Control Extended :
    ///                                 Supports simple creation tab method
    ///                                 instead heavy DevComponents code. Also
    ///                                 Close tab and Navigation are directly
    ///                                 integrated.
    /// </summary>
    //
    //**************************************************************************
    public class TabControlExt : TabControl  {

        /// <summary>Occurs after selected tab changed</summary>
        public  event EventHandler                              AfterSelectedTabChanged;
        /// <summary>Flag during tab selection</summary>
        private bool                                            m_bSelectingTab;
        /// <summary>Context menu</summary>
        protected System.Windows.Forms.ContextMenu              m_ctxMenu;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public TabControlExt() : base() {
            System.Windows.Forms.MenuItem       miClose;

            m_bSelectingTab                 = false;
            m_ctxMenu                       = new System.Windows.Forms.ContextMenu();
            miClose                         = m_ctxMenu.MenuItems.Add("&Close");
            miClose.Name                    = "miClose";
            miClose.Click                  += new EventHandler(miClose_Click);
            this.ColorScheme.TabBackground  = System.Drawing.SystemColors.Control;
            this.ColorScheme.TabBackground2 = System.Drawing.SystemColors.Control;
            this.SelectedTabChanged        += new TabStrip.SelectedTabChangedEventHandler(TabControlExt_SelectedTabChanged);
        }

        //******************************************************************
        //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            System.Windows.Forms.MenuItem   miClose;

            if (bDisposing) {
                if (m_ctxMenu != null) {
                    miClose = m_ctxMenu.MenuItems["miClose"];
                    if (miClose != null) {
                        miClose.Click -= new EventHandler(miClose_Click);
                    }
                }
                if (TabStrip != null) {
                    TabStrip.MouseUp -= new System.Windows.Forms.MouseEventHandler(TabStrip_MouseUp);
                }
                if (m_ctxMenu != null) {
                    m_ctxMenu.Dispose();
                    m_ctxMenu = null;
                }
                this.SelectedTabChanged -= new TabStrip.SelectedTabChangedEventHandler(TabControlExt_SelectedTabChanged);
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
		/// <summary>
		/// Gets the context menu associated to the tab
		/// </summary>
        //
        //******************************************************************
        public System.Windows.Forms.ContextMenu TabContextMenu {
            get {
                return(m_ctxMenu);
            }
        }

        //******************************************************************
        //
		/// <summary>
        /// OnCreateControl:            Occurs when the control is created
		/// </summary>
        //
        //******************************************************************
        protected override void OnCreateControl() {
            TabStrip.MouseUp += new System.Windows.Forms.MouseEventHandler(TabStrip_MouseUp);
            base.OnCreateControl();
        }

        //******************************************************************
        //
		/// <summary>
        /// TabStrip_MouseUp:           Occurs when the mouse is Up after click
		/// </summary>
        /// <param name="sender">       Sender</param>
        /// <param name="e">            Event arguments</param>
        //
        //******************************************************************
        private void TabStrip_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
            TabStripTabChangedEventArgs tabEvent;
            TabItem                     tab;
            Point                       pt;
            Boolean                     bCanShow = true;

            if (e.Button == System.Windows.Forms.MouseButtons.Right) {
                pt  = new Point(e.X, e.Y);
                tab = GetTabAtPoint(pt);
                if (tab != null) {
                    if (tab != SelectedTab) {
                        tabEvent = new TabStripTabChangedEventArgs(SelectedTab, tab);
                        InvokeSelectedTabChanged(tabEvent);
                        bCanShow = !tabEvent.Cancel;
                        if (bCanShow) {
                            SelectedTab = tab;
                        }
                    }
                    if (bCanShow) {
                        m_ctxMenu.Show(TabStrip, pt);
                    }
                }
            }
        }

        //******************************************************************
        //
		/// <summary>
        /// miClose_Click:              Occurs when the mouse is Up after click
		/// </summary>
        /// <param name="sender">       Sender</param>
        /// <param name="e">            Event arguments</param>
        //
        //******************************************************************
        protected void miClose_Click(object sender, EventArgs e) {
            if (SelectedTab != null) {
                InvokeTabItemClose(new TabStripActionEventArgs());
            }            
        }

        //******************************************************************
        //
		/// <summary>
        /// GetTabAtPoint:              Get Tab At point
		/// </summary>
        /// <param name="pt">Point to get Tab</param>
        /// <returns>
        /// Return the tab at point if found otherwise null
        /// </returns>
        //
        //******************************************************************
        private TabItem GetTabAtPoint(Point pt) {
            TabItem     tabRetVal = null;
            Rectangle   rect;

            foreach (TabItem tab in Tabs) {
                if (tab.Visible) {
                    rect = tab.DisplayRectangle;
                    if (pt.X >= rect.Left && pt.X <= rect.Right && pt.Y >= rect.Top && pt.Y <= rect.Bottom) {
                        tabRetVal = tab;
                        break;
                    }
                }
            }
            return(tabRetVal);
        }

        //******************************************************************
        //
		/// <summary>
        /// CreateNewTab:               Create new tab
		/// </summary>
        /// <param name="strTabName">   Tab name</param>
        //
        //******************************************************************
        public TabItemExt CreateNewTab(String strTabName) {
            TabItemExt tabItemExtended;

            tabItemExtended      = CreateTabFactory();
            tabItemExtended.Text = strTabName;
            return(tabItemExtended);
        }

        //******************************************************************
        //
		/// <summary>
        /// CreateTabFactory:           Tab factory
		/// </summary>
        //
        //******************************************************************
        protected virtual TabItemExt CreateTabFactory() {
            return(new TabItemExt(this));
        }

        //******************************************************************
        //
		/// <summary>
        /// TabControlExt_SelectedTabChanged: 
        ///                             Occurs when tab item is changed
		/// </summary>
        /// <param name="sender">       Sender</param>
        /// <param name="e">            Tab Changed event arguments</param>
        //
        //******************************************************************
        private void TabControlExt_SelectedTabChanged(object sender, TabStripTabChangedEventArgs e) {
            m_bSelectingTab = true;
        }

        //******************************************************************
        //
		/// <summary>
        /// OnAfterSelectedTabChanged:  Occurs after tab item changed
		/// </summary>
        /// <param name="e">            Tab Changed event arguments</param>
        //
        //******************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]       
        public void OnAfterSelectedTabChanged(EventArgs e) {
            if (m_bSelectingTab && AfterSelectedTabChanged != null) {
                AfterSelectedTabChanged(this, e);
            }
            m_bSelectingTab = false;
        }

        //******************************************************************
        //
		/// <summary>
        /// InvokeTabItemClose:        Tab close invokation
		/// </summary>
        /// <param name="e">           Tab close event arguments</param>
        //
        //******************************************************************
        protected override void InvokeTabItemClose(TabStripActionEventArgs e) {
            TabStripTabChangedEventArgs     tabEvent;
            TabItem                         tiToDelete;
            TabItem                         tiToSelect = null;

            if (this.Tabs.Count > 1) {
                tiToDelete = this.SelectedTab;
                foreach (TabItem tab in Tabs) {
                    if (tab == tiToDelete) {
                        break;
                    }
                    if (tab.Visible) {
                        tiToSelect = tab;
                    }
                }
                if (tiToSelect == null) {
                    tiToSelect = Tabs[0];
                }
                tabEvent = new TabStripTabChangedEventArgs(SelectedTab, tiToSelect);
                InvokeSelectedTabChanged(tabEvent);
                if (!tabEvent.Cancel) {
                    base.InvokeTabItemClose(e);
                    if (!e.Cancel) {
                        SelectedTab        = tiToSelect;
                        tiToDelete.Visible = false;
                        this.Tabs.Remove(tiToDelete);
                        tiToDelete.Dispose();
                        tiToDelete = null;
                    }
                }
            }
        }

    } // End TabControlExt class

    //**************************************************************************
    //
    /// <summary>
    /// TabControlPanelExt:			    Tab Control Panel Extended :
    ///                                 Informs Tab Contol that tab containing
    ///                                 panel is changed
    /// </summary>
    //
    //**************************************************************************
    public class TabControlPanelExt : DevComponents.DotNetBar.TabControlPanel {

        //******************************************************************
        //
        /// <summary>
        /// Select:                     Occurs each time panel get focus
        /// </summary>
        /// <param name="directed">     true to specify the direction of 
        ///                             the control to select; otherwise, false.</param>
        /// <param name="forward">      true to move forward in the tab 
        ///                             order; false to move backward in the tab order.</param>                        
        //
        //******************************************************************
        protected override void Select(bool directed, bool forward) {
            TabControlExt           tabCtrlExt;

            if (TabItem != null && TabItem.Parent.Parent is TabControlExt) {
                tabCtrlExt = TabItem.Parent.Parent as TabControlExt;
                tabCtrlExt.OnAfterSelectedTabChanged(EventArgs.Empty);
            }
            base.Select(directed, forward);
        }

    } // TabControlPanelExt class

    //**************************************************************************
    //
    /// <summary>
    /// TabItemExt:			            Tab Item Extended :
    ///                                 Create automatically the panel attached
    ///                                 to its tab.
    /// </summary>
    //
    //**************************************************************************
    public class TabItemExt : DevComponents.DotNetBar.TabItem {
        /// <summary>Panel attached to tab control</summary>
        private TabControlPanelExt              m_backPanel;
        /// <summary>Tag Object attached to the tab</summary>
        private Object                          m_tag;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public TabItemExt()
            : base() {
            m_tag               = null;
            m_backPanel         = new TabControlPanelExt();
            m_backPanel.Padding = new System.Windows.Forms.Padding(6);
            m_backPanel.Dock    = System.Windows.Forms.DockStyle.Fill;
            m_backPanel.TabItem = this;
            m_backPanel.SendToBack();
            this.AttachedControl = m_backPanel;
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        /// <param name="tabControlParent">  Tab Control parent</param>
        //
        //******************************************************************
        public TabItemExt(DevComponents.DotNetBar.TabControl tabControlParent)
            : this() {
            tabControlParent.ApplyDefaultPanelStyle(m_backPanel);
            tabControlParent.Tabs.Add(this);
            tabControlParent.Controls.Add(m_backPanel);
            tabControlParent.RecalcLayout();
        }

        //******************************************************************
        //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_backPanel != null) {
                    m_backPanel.Dispose();
                    m_backPanel = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// Gets or sets the object that contains data about the control.
        /// </summary>
        //
        //******************************************************************
        public Object Tag {
            get {
                return(m_tag);
            }
            set {
                m_tag = value;
            }
        }

    } // End TabItemExt class

} // End namespace 
