using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using DevComponents.DotNetBar;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window.Ext {
    
	public class frmBaseMain : frmManagedForm {
        /// <summary>
        /// Enum for configuration saving mode
        /// </summary>
        public enum ConfigSaveModeE {
            /// <summary>Manual</summary>
            CSM_Manual      =   0, 
            /// <summary>Ask</summary>
            CSM_Ask         =   1,
            /// <summary>Auto</summary>
            CSM_Auto        =   2
        };

        private BarControlTextTranslator                    m_bartxtTranslator;       // Menu translation object
        private ConfigSaveModeE                             m_eConfigSaveMode;        // Configuration saving mode

        //******************************************************************        
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        //
        //******************************************************************        
        public frmBaseMain() {
            m_bartxtTranslator = null;
        }

        //******************************************************************        
        //
        /// <summary>
        /// InitV:				            Initialize the class
        /// </summary>
        /// <param name="app">              Rep++ Application object</param>
        /// <param name="arrTopForm">       Array of top form</param>
        /// <param name="frmActiveForm">    Active form</param>
        /// <param name="strCfgFileName">   Configuration file name</param>
        /// <param name="strAtomName">      Form atom name</param>
        //
        //******************************************************************        
        protected virtual void InitV(RepPP.Application      app, 
                                     ArrayList              arrTopForm, 
                                     Form                   frmActiveForm,
                                     string                 strCfgFileName,
                                     string                 strAtomName) {
            base.InitV(app, arrTopForm, frmActiveForm);
            m_eConfigSaveMode = ConfigSaveModeE.CSM_Auto;
            if (strCfgFileName != null) {
                m_configHelper = new BarUIConfigurationHelper(strCfgFileName);
            }
            m_bartxtTranslator = new BarControlTextTranslator(m_app, new string[] {strAtomName});
            foreach(RowsetTreeDef rstd in m_app.RowsetTreeDefs) {
                rstd.BuildSqlCommand(true);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitToolsCmd:               Initialize the tool cmd. This function can
        ///                             be called by the managed form to build their
        ///                             tool menu.
        /// </summary>
        /// <param name="miTools">      Tools menu</param>
        /// <param name="barTools">     Bar containing tools</param>
        /// <param name="cmdHolder">    Command holder</param>
        //
        //******************************************************************
        public virtual void InitToolsCmd(ButtonItem miTools, Bar barTools, CmdHolder cmdHolder) {
        }

        //******************************************************************        
        //
        /// <summary>
        /// ChildFormClosing:                   Called by the managed child form when they are closing
        /// </summary>
        /// <param name="frmChild">             Child form closing</param>
        //
        //******************************************************************        
        public virtual void ChildFormClosing(frmManagedForm frmChild) {
            if (ManagedFormListArray.Count == 0) {
                Close();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ConfigurationSaveMode:  Choose the mode for automatic configuration saving
        /// </summary>
        //
        //******************************************************************
        public ConfigSaveModeE ConfigurationSaveMode {
            get {
                return(m_eConfigSaveMode);
            }
            set {
                m_eConfigSaveMode = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// BaseControlTextTranslator:  Control text translator                 
        /// </summary>
        //
        //******************************************************************
        public virtual BarControlTextTranslator BaseControlTextTranslator {
            get {
                return(m_bartxtTranslator);
            }
            set {
                m_bartxtTranslator = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveUIConfigv:          Save UI configuration
        /// </summary>
        //
        //******************************************************************
        private void SaveUIConfigv() {
            bool    bSave;
            string  strText;
            
            if (m_app != null && m_configHelper != null) {
                switch(m_eConfigSaveMode) {
                case ConfigSaveModeE.CSM_Manual:
                    bSave = false;
                    break;
                case ConfigSaveModeE.CSM_Ask:
                    strText = m_app.GetAtomText("AskToSaveUI", RepPP.AtomType.sdAtomMsg);
                    bSave   = (MessageBox.Show(this, strText, Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes);
                    break;
                case ConfigSaveModeE.CSM_Auto:
                default:
                    bSave = true;
                    break;
                }
                if (bSave) {
                    m_configHelper.SaveToFile();
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnClosed:         Called when the form is closed 
        /// </summary>
        /// <param name="e">  Event arguments</param>
        //
        //******************************************************************
        protected override void OnClosed(System.EventArgs e) {
            SaveUIConfigv();
            base.OnClosed(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadState:                      Load the form state.
        ///                                 Implementing RepPPToolkit.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
        public override void LoadState(UIConfigurationHelper configHelper, XmlNode node) {
            string  strConfigSaveMode;
            
            base.LoadState(configHelper, node);
            strConfigSaveMode = configHelper.GetNodeAttribute(node, "CONFIGSAVEMODE");
            if (strConfigSaveMode != null && strConfigSaveMode.Length != 0) {
                try {
                    m_eConfigSaveMode = (ConfigSaveModeE)Int32.Parse(strConfigSaveMode);
                } catch {
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveState:                      Save the form state
        ///                                 Implementing RepPPToolkit.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
        public override void SaveState(UIConfigurationHelper configHelper, XmlNode node) {
            base.SaveState(configHelper, node);
            configHelper.SetNodeAttribute(node, "CONFIGSAVEMODE", ((int)m_eConfigSaveMode).ToString());
        }
	} // Class frmBaseMain
} // Namespace