using System;
using System.Collections;

namespace RepPP.Toolkit.Window {
    /// <summary>
    /// Summary description for Cmd.
    /// </summary>
    public class Cmd : IDisposable {
        /// <summary>Command name</summary>
        public string                 m_strCmdName;
        /// <summary>Delegate to call the command</summary>
        public CmdHolder.OnCommand    m_del;
        /// <summary>Command state</summary>
        public CmdHolder.CmdStateE    m_eState;
        /// <summary>Command state was forced</summary>
        public bool                   m_bForcedState;
        /// <summary>Array of triggers</summary>
        public ArrayList              m_arrTrigger;
        /// <summary>OnSelect delegate</summary>
        public EventHandler           m_delOnSelect;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="strCmdName">   Unique command name</param>
        /// <param name="del">          Delegate to execute the command</param>
        /// <param name="eState">       Original state of the command</param>
        //
        //*********************************************************
        public Cmd(string strCmdName, CmdHolder.OnCommand del, CmdHolder.CmdStateE eState) {
            m_strCmdName    = strCmdName;
            m_del           = del;
            m_eState        = eState;
            m_arrTrigger    = new ArrayList();
            m_delOnSelect   = new EventHandler(OnSelect);
            m_bForcedState  = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose:                    Clean-up class
        /// </summary>
        //
        //*********************************************************
        public virtual void Dispose() {
            while (m_arrTrigger.Count != 0) {
                RemoveTrigger(m_arrTrigger[0]);
            }
            m_delOnSelect = null;
            m_del         = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// Name:                       Command name
        /// </summary>
        //
        //*********************************************************
        public string Name {
            get {
                return(m_strCmdName);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// State:                      Command state
        /// </summary>
        //
        //*********************************************************
        public CmdHolder.CmdStateE State {
            get {
                return(m_eState);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AddTrigger:                 Add triggers to the command
        /// </summary>
        /// <param name="arrTrigger">   Array of trigger for the command</param>
        //
        //*********************************************************
        public void AddTrigger(object[] arrTrigger) {
            foreach (object obj in arrTrigger) {
                if (m_arrTrigger.IndexOf(obj) == -1) {
                    m_arrTrigger.Add(obj);
                    SetState(obj, m_eState);
                    AddHandler(obj);
                } else {
                    SetState(obj, m_eState);
                }
            }	                
        }

        //*********************************************************
        //
        /// <summary>
        /// RemoveTrigger:              Remove a trigger to the command
        /// </summary>
        /// <param name="obj">          Trigger to remove of the command</param>
        //
        //*********************************************************
        public virtual void RemoveTrigger(object obj) {
            if (obj is System.Windows.Forms.MenuItem) {
                ((System.Windows.Forms.MenuItem)obj).Click -= m_delOnSelect;
            } else if (obj is System.Windows.Forms.Button) {
                ((System.Windows.Forms.Button)obj).Click    -= m_delOnSelect;
            } else if (obj is System.Windows.Forms.ToolStripMenuItem) {
                ((System.Windows.Forms.ToolStripMenuItem)obj).Click -= m_delOnSelect;
            } else if (obj is System.Windows.Forms.ToolStripItem) {
                ((System.Windows.Forms.ToolStripItem)obj).Click -= m_delOnSelect;
            }
            m_arrTrigger.Remove(obj);
        }

        //*********************************************************
        //
        /// <summary>
        /// AddHandlerv:                Add event handler for the trigger
        /// </summary>
        /// <param name="obj">          Trigger object</param>
        //
        //*********************************************************
        protected virtual void AddHandler(object obj) {
            if (obj is System.Windows.Forms.MenuItem) {
                ((System.Windows.Forms.MenuItem)obj).Click += m_delOnSelect;
            } else if (obj is System.Windows.Forms.Button) {
                ((System.Windows.Forms.Button)obj).Click += m_delOnSelect;
            } else if (obj is System.Windows.Forms.ToolStripMenuItem) {
                ((System.Windows.Forms.ToolStripMenuItem)obj).Click += m_delOnSelect;
            } else if (obj is System.Windows.Forms.ToolStripItem) {
                ((System.Windows.Forms.ToolStripItem)obj).Click += m_delOnSelect;
            } 
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSelect:                   Called when a trigger is selected
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Argument</param>
        //
        //*********************************************************
        private void OnSelect(object sender, EventArgs e) {
            if (m_eState == CmdHolder.CmdStateE.Enabled) {
                m_del();    // Call the command delegate.
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetState:                   Set the state of a trigger
        /// </summary>
        /// <param name="obj">          Trigger object</param>
        /// <param name="eState">       Trigger state</param>
        //
        //*********************************************************
        public void SetState(object obj, CmdHolder.CmdStateE eState) {
            Type                                type;
            System.Reflection.PropertyInfo      pinfoVisible;
            System.Reflection.PropertyInfo      pinfoEnabled;
            System.Windows.Forms.Control        ctl;
            System.Windows.Forms.ToolBarButton  btnTb;

            if (obj is System.Windows.Forms.Control) {
                ctl = obj as System.Windows.Forms.Control;
                switch(eState) {
                case CmdHolder.CmdStateE.Hidden:
                    ctl.Visible = false;
                    break;
                case CmdHolder.CmdStateE.Disabled:
                    ctl.Visible = true;
                    ctl.Enabled = false;
                    break;
                case CmdHolder.CmdStateE.Enabled:
                    ctl.Visible = true;
                    ctl.Enabled = true;
                    break;
                case CmdHolder.CmdStateE.None:
                    ctl.Visible = true;
                    break;
                default:
                    break;
                }
            } else if (obj is System.Windows.Forms.ToolBarButton) {
                btnTb = obj as System.Windows.Forms.ToolBarButton;
                switch(eState) {
                case CmdHolder.CmdStateE.Hidden:
                    btnTb.Visible = false;
                    break;
                case CmdHolder.CmdStateE.Disabled:
                    btnTb.Visible = true;
                    btnTb.Enabled = false;
                    break;
                case CmdHolder.CmdStateE.Enabled:
                    btnTb.Visible = true;
                    btnTb.Enabled = true;
                    break;
                case CmdHolder.CmdStateE.None:
                    btnTb.Visible = true;
                    break;
                default:
                    break;
                }
            } else {
                try {
                    type = obj.GetType();
                    pinfoVisible = type.GetProperty("Visible", typeof(bool));
                    pinfoEnabled = type.GetProperty("Enabled", typeof(bool));
                    switch(eState) {
                    case CmdHolder.CmdStateE.Hidden:
                        pinfoVisible.SetValue(obj, false, null);
                        break;
                    case CmdHolder.CmdStateE.Disabled:
                        pinfoVisible.SetValue(obj, true, null);
                        pinfoEnabled.SetValue(obj, false, null);
                        break;
                    case CmdHolder.CmdStateE.Enabled:
                        pinfoVisible.SetValue(obj, true, null);
                        pinfoEnabled.SetValue(obj, true, null);
                        break;
                    case CmdHolder.CmdStateE.None:
                        pinfoVisible.SetValue(obj, true, null);
                        break;
                    default:
                        break;
                    }
                } catch (System.Exception) {
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetState:                   Set the state of the command
        /// </summary>
        /// <param name="eState">       New command state</param>
        //
        //*********************************************************
        public void SetState(CmdHolder.CmdStateE eState) {
            if (!m_bForcedState) {
                SetStateInternal(eState);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetForcedState:             Set the forced state of the command
        /// </summary>
        /// <param name="eState">       New command state</param>
        //
        //*********************************************************
        public void SetForcedState(CmdHolder.CmdStateE eState) {
            SetStateInternal(eState);
            m_bForcedState = true;
        }

        //*********************************************************
        //
        /// <summary>
        /// ClearForcedState:           Clears the forced state of the command
        /// </summary>
        //
        //*********************************************************
        public void ClearForcedState() {
            m_bForcedState = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// SetStateInternal:           Set the state of the command
        /// </summary>
        /// <param name="eState">       New command state</param>
        //
        //*********************************************************
        private void SetStateInternal(CmdHolder.CmdStateE eState) {
            if (eState != m_eState) {
                foreach (object obj in m_arrTrigger) {
                    SetState(obj, eState);
                }
                m_eState = eState;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetText:                    Set the text of a trigger
        /// </summary>
        /// <param name="obj">          Trigger</param>
        /// <param name="strText">      Text to use</param>
        //
        //*********************************************************
        public void SetText(object obj, string strText) {
            Type                            type;
            System.Reflection.PropertyInfo  pinfoText;
	    
            type      = obj.GetType();
            pinfoText = type.GetProperty("Text", typeof(string));
            pinfoText.SetValue(obj, strText, null);
        }
    }
}
