using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D; 

namespace RepPP.Toolkit.Window {

    //*********************************************************	    
    //
    /// <summary>
    /// Implement a collapsible GroupBox control.
    /// </summary>
    //
    //*********************************************************	
    [ToolboxBitmap(typeof(CollapsibleGroupBox))]
    public class CollapsibleGroupBox : GroupBox {

		private Color       m_borderColor           = Color.FromArgb(191, 184, 191);
		private Color       m_textColor             = Color.FromArgb(0, 0, 0);
        private const int   m_collapsedHeight       = 20;
        private Rectangle   m_toggleRect            = new Rectangle(8, 2, 11, 11);
        private Boolean     m_collapsed             = false;
        private Boolean     m_bResizingFromCollapse = false;
        private Size        m_FullSize              = Size.Empty;

        /// <summary>Fired when the Collapse Toggle button is pressed</summary>
        public delegate void CollapseBoxClickedEventHandler(object sender);
        /// <summary>Event fired when the Collapse Toggle button is pressed</summary>
        public event CollapseBoxClickedEventHandler CollapseBoxClickedEvent;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************        
        public CollapsibleGroupBox() {
            InitializeComponent();
        }

        //*********************************************************
        //
        /// <summary>
        /// FullHeight:                 Full non-collapsed height
        /// </summary>
        //
        //*********************************************************        
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int FullHeight {
            get { 
                return(m_FullSize.Height); 
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// IsCollapsed:               Checks if the group box is collapsed
        /// </summary>
        //
        //*********************************************************        
        [DefaultValue(false), Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool IsCollapsed {
            get { 
                return (m_collapsed); 
            }
            set {
                if (value != m_collapsed) {
                    m_collapsed = value;
                    if (!value) {
                        // Expand
                        this.Size = m_FullSize;
                    } else {
                        // Collapse
                        m_bResizingFromCollapse = true;
                        if (m_FullSize == Size.Empty) {
                            m_FullSize = this.Size;
                        }
                        this.Height             = m_collapsedHeight;
                        m_bResizingFromCollapse = false;
                    }
                    foreach (Control c in Controls){
                        c.Visible = !value;
                    }
                    Invalidate();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CollapsedHeight:            Collapsed height
        /// </summary>
        //
        //*********************************************************        
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int CollapsedHeight {
            get { 
                return(m_collapsedHeight); 
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnMouseUp:                  Called to trigger the MouseUp event
        /// </summary>
        /// <param name="e">            event argument</param>
        //
        //*********************************************************        
        protected override void OnMouseUp(MouseEventArgs e) {
            if (m_toggleRect.Contains(e.X, e.Y)) {
                ToggleCollapsed();
            } else {
                base.OnMouseUp(e);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnPaint:                    Called to trigger the Paint event
        /// </summary>
        /// <param name="e">            event argument</param>
        //
        //*********************************************************        
        protected override void OnPaint(PaintEventArgs e) {
            HandleResize();
            DrawGroupBox(e.Graphics);
            DrawToggleButton(e.Graphics);
        }

        //*********************************************************
        //
        /// <summary>
        /// DrawGroupBox:               Draws the groupbox
        /// </summary>
        /// <param name="g">            Graphics object</param>
        //
        //*********************************************************        
        private void DrawGroupBox(Graphics g) {
			Rectangle       bounds;
            StringFormat    sf;
            int             iTextPos;
            int             iTextSize;
            int             iEndPos;

            // Get windows to draw the GroupBox
			bounds = new Rectangle(ClientRectangle.X, 
                                   ClientRectangle.Y + 6, 
                                   ClientRectangle.Width, 
                                   ClientRectangle.Height - 6);
			// Text Formating positioning & Size
			sf         = new StringFormat();
			iTextPos   = (bounds.X + 8) + m_toggleRect.Width + 2;
			iTextSize  = (int)g.MeasureString(Text, this.Font).Width;
			iTextSize  = iTextSize < 1 ? 1 : iTextSize;
			iEndPos    = iTextPos + iTextSize + 1;

			// Draw the Border
			using (Pen p = new Pen(m_borderColor)) {
				DrawRoundRectangle(g, p, 0, 6, Width-1, Height-7, 5F);
            }
			// Draw a line to cover the GroupBox border where the text will sit
			g.FillRectangle(SystemBrushes.Control, 8, 2, iEndPos-8, 12);
			// Draw the GroupBox text
			using (SolidBrush drawBrush = new SolidBrush(ForeColor)){
				g.DrawString(Text, this.Font, drawBrush, iTextPos, 0);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DrawRoundRectangle:         Draws a round rectangle
        /// </summary>
        /// <param name="g">            Graphics object</param>
		/// <param name="pen">          Pen</param>
		/// <param name="x">            X</param>
		/// <param name="y">            Y</param>
		/// <param name="width">        Width</param>
		/// <param name="height">       Height</param>
		/// <param name="radius">       Radius</param>
        //
        //*********************************************************        
        private void DrawRoundRectangle(Graphics g, Pen pen, float x, float y, float width, float height, float radius) 
		{ 
			RectangleF   rectangle; 
			GraphicsPath path;
 
            rectangle   = new RectangleF(x, y, width, height);
            path        = this.GetRoundedRect(rectangle, radius); 
			g.DrawPath(pen, path); 
		} 

        //*********************************************************
        //
        /// <summary>
        /// GetRoundedRect:             Gets a rounded rectangle
        /// </summary>
        //
        //*********************************************************        
        private GraphicsPath GetRoundedRect(RectangleF baseRect, float radius) 
		{
            float           diameter;
            SizeF           sizeF;
            RectangleF      arc;
            GraphicsPath    path;

			if( radius <= 0.0F ) { 
			    // if corner radius is less than or equal to zero, 
			    // return the original rectangle 
				path = new GraphicsPath(); 
				path.AddRectangle(baseRect); 
				path.CloseFigure(); 
			} else if( radius>=(Math.Min(baseRect.Width, baseRect.Height))/2.0) { 
			    // if the corner radius is greater than or equal to 
			    // half the width, or height (whichever is shorter) 
			    // then return a capsule instead of a lozenge 
				path = GetCapsule( baseRect );
            } else {
			    // create the arc for the rectangle sides and declare 
			    // a graphics path object for the drawing 
                diameter = radius * 2.0F; 
			    sizeF    = new SizeF( diameter, diameter );
			    arc      = new RectangleF( baseRect.Location, sizeF ); 
			    path     = new System.Drawing.Drawing2D.GraphicsPath(); 
			    // top left arc 
			    path.AddArc( arc, 180, 90 ); 
			    // top right arc 
			    arc.X = baseRect.Right-diameter; 
			    path.AddArc( arc, 270, 90 ); 
			    // bottom right arc 
			    arc.Y = baseRect.Bottom-diameter; 
			    path.AddArc( arc, 0, 90 ); 
			    // bottom left arc
			    arc.X = baseRect.Left;     
			    path.AddArc( arc, 90, 90 );     
			    path.CloseFigure(); 
            }
            return path;
        }

        //*********************************************************
        //
        /// <summary>
        /// GetCapsule:                 Gets a capsule
        /// </summary>
        //
        //*********************************************************        
		private GraphicsPath GetCapsule( RectangleF baseRect ) 
		{ 
			float        diameter; 
			RectangleF   arc; 
			GraphicsPath path = new GraphicsPath(); 
            SizeF        sizeF;

            try { 
				if(baseRect.Width > baseRect.Height) { 
					// return horizontal capsule 
					diameter = baseRect.Height; 
					sizeF    = new SizeF(diameter, diameter);
					arc      = new RectangleF(baseRect.Location, sizeF); 
					path.AddArc( arc, 90, 180); 
					arc.X = baseRect.Right-diameter; 
					path.AddArc( arc, 270, 180); 
				} else if(baseRect.Width < baseRect.Height) { 
					// return vertical capsule 
					diameter = baseRect.Width;
					sizeF    = new SizeF(diameter, diameter);
					arc      = new RectangleF( baseRect.Location, sizeF ); 
					path.AddArc( arc, 180, 180 ); 
					arc.Y = baseRect.Bottom-diameter; 
					path.AddArc( arc, 0, 180 ); 
				} else { 
					// return circle 
					path.AddEllipse( baseRect ); 
				}
			} catch(Exception) {
				path.AddEllipse( baseRect ); 
			} finally { 
				path.CloseFigure(); 
			} 
			return(path); 
		} 

        //*********************************************************
        //
        /// <summary>
        /// DrawToggleButton:           Draws the toggle button
        /// </summary>
        /// <param name="g">            Graphics object</param>
        //
        //*********************************************************        
        private void DrawToggleButton(Graphics g){
            Bitmap bmpPlus;
            Bitmap bmpMinus;

            if(IsCollapsed) {
                bmpPlus  = new Bitmap(GetType().Assembly.GetManifestResourceStream("RepPP.Toolkit.Window.CollapsibleGroupBox.Plus.bmp"));
                g.DrawImage(bmpPlus, m_toggleRect);
            } else {
                bmpMinus = new Bitmap(GetType().Assembly.GetManifestResourceStream("RepPP.Toolkit.Window.CollapsibleGroupBox.Minus.bmp"));
                g.DrawImage(bmpMinus, m_toggleRect);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ToggleCollapsed:           Collapse/Expand the groupbox
        /// </summary>
        //
        //*********************************************************        
        public void ToggleCollapsed() {
            IsCollapsed = !IsCollapsed;

            if (CollapseBoxClickedEvent != null) {
                CollapseBoxClickedEvent(this);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// HandleResize:           Manages the control resize
        /// </summary>
        //
        //*********************************************************        
        void HandleResize() {
            if (!m_bResizingFromCollapse && !m_collapsed) {
                m_FullSize = this.Size;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose:                    Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">    true if managed resources should be disposed; otherwise, false.</param>
        //
        //*********************************************************        
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Component Designer generated code
        /// <summary> 
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;


        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(CollapsibleGroupBox));
            this.SuspendLayout();
            // 
            // CollapsibleGroupBox
            // 
            this.Name = "CollapsibleGroupBox";
            this.ResumeLayout(false);

        }

        #endregion
    }
}
