using System;
using System.Collections;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// Controls text translator using atoms
	/// </summary>
	public class ControlTextTranslator : ControlTextTranslatorBase {

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="app">          Rep++ application object</param>
        /// <param name="arrAtomName">  Array of atoms</param>
        //
        //*********************************************************        
		public ControlTextTranslator(RepPP.IApplication app, string [] arrAtomName)	: base (app, arrAtomName) {
		}

        //*********************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="fatherTranslator"> Father translator</param>
        //
        //*********************************************************        
		public ControlTextTranslator(RepPP.IApplication       app, 
                                     string []                arrAtomName, 
                                     ControlTextTranslator    fatherTranslator) : base(app, arrAtomName, fatherTranslator) { 
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateControlTextTranslator:    Creates a new control text translator using the same type as
        ///                                 the father.
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="fatherTranslator"> Father translator</param>
        /// <returns>
        /// New object
        /// </returns>
        //
        //*********************************************************        
		public static ControlTextTranslator CreateControlTextTranslator(RepPP.IApplication       app, 
                                                                        string []                arrAtomName, 
                                                                        ControlTextTranslator    fatherTranslator) {
            ControlTextTranslator               transRetVal;
            System.Reflection.ConstructorInfo   ctorInfo;
            
            if (fatherTranslator == null) {
                transRetVal = new ControlTextTranslator(app, arrAtomName);
            } else {
                ctorInfo = fatherTranslator.GetType().GetConstructor(new Type[] {typeof(RepPP.IApplication),
                                                                                 typeof(string[]),
                                                                                 typeof(ControlTextTranslator)});
                transRetVal = (ctorInfo == null) ? new ControlTextTranslator(app, arrAtomName, fatherTranslator) :
                                                   ctorInfo.Invoke(new object[] {app, arrAtomName, fatherTranslator} ) as ControlTextTranslator;
            }
            return(transRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsObjectContainsComponent:  Check if the object can contains other objects
        /// </summary>
        /// <returns>
        /// True:
        /// False:
        /// </returns>
        //
        //*********************************************************        
        protected virtual bool IsObjectContainsComponent(object obj) {
            bool    bRetVal;
            
            if (obj is System.Windows.Forms.Form) {
                bRetVal = true;
            } else if (obj is System.Windows.Forms.ToolBar) {
                bRetVal = true;
            } else if (obj is System.Windows.Forms.MainMenu) {
                bRetVal = true;
            } else if (obj is System.Windows.Forms.MenuStrip) {
                bRetVal = true;
            } else if (obj is System.Windows.Forms.ToolStrip) {
                bRetVal = true;
            } else {
                bRetVal = false;
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// SetTooltipText:                 Set the tooltip property of an object
        /// </summary>
        /// <param name="obj">              Object</param>
        /// <param name="strTooltipText">   Tooltip text</param>
        /// <param name="toolTip">          ToolTip Component</param>
        //
        //*********************************************************                        
        protected virtual void SetTooltipText(Object obj, string strTooltipText, ToolTip toolTip) {
            if (obj is ToolBarButton) {
                ((ToolBarButton)obj).ToolTipText = strTooltipText;
            } else if (obj is System.Windows.Forms.ToolStripMenuItem) {
                ((ToolStripMenuItem)obj).ToolTipText = strTooltipText;
            } else if (obj is System.Windows.Forms.ToolStripItem) {
                ((ToolStripItem)obj).ToolTipText = strTooltipText;
            } else if (obj is System.Windows.Forms.MenuItem) {
                // Set the tooltip
            } else if (toolTip != null && obj is Control) {
                toolTip.SetToolTip((Control)obj, strTooltipText);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetText:                    Set the text property of an object
        /// </summary>
        /// <param name="obj">          Object</param>
        /// <param name="strItemText">  Text</param>
        //
        //*********************************************************
        protected virtual void SetText(Object obj, string strItemText) {
            if (obj is Control) {
                ((Control)obj).Text = strItemText;
            } else if (obj is MenuItem) {
                ((MenuItem)obj).Text = strItemText;
            } else if (obj is ToolStripMenuItem) {
                ((ToolStripMenuItem)obj).Text = strItemText;
            } else if (obj is ToolStripItem) {
                ((ToolStripItem)obj).Text = strItemText;
            } else if (obj is ToolBarButton) {
                ((ToolBarButton)obj).Text = strItemText;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetComponentInfo:          Set component info (Text, Tooltip)
        /// </summary>
        /// <param name="obj">         Object</param>
        /// <param name="strObjName">  Object name</param>
        /// <param name="toolTip">     ToolTip Component</param>
        //
        //*********************************************************
        protected virtual void SetComponentInfo(object obj, string strObjName, ToolTip toolTip) {
            string  strItemText;
            
            strItemText = GetItemText(strObjName, "Text");
            if (strItemText != null) {
                SetText(obj, strItemText);
            }
            else if (strObjName.StartsWith("_")) {
                strItemText = GetItemText(strObjName.Substring(1, strObjName.Length - 1), "Text");
                if (strItemText != null) {
                    SetText(obj, strItemText);
                }
            }
            strItemText = GetItemText(strObjName, "Tooltip");
            if (strItemText != null) {
                SetTooltipText(obj, strItemText, toolTip);
            }
            else if (strObjName.StartsWith("_")) {
                strItemText = GetItemText(strObjName.Substring(1, strObjName.Length - 1), "Tooltip");
                if (strItemText != null) {
                    SetTooltipText(obj, strItemText, toolTip);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EnumComponents:         Enumarate components of an object
        /// </summary>
        /// <param name="owner">    Object</param>
        /// <param name="toolTip">  ToolTip Component</param>
        //
        //*********************************************************        
        private void EnumComponents(object owner, ref ToolTip toolTip) {
		    System.Type                     typeOwner;
		    System.Reflection.MemberInfo[]  arrMemberInfo;
		    System.Reflection.FieldInfo     fieldInfo;
		    object                          obj;
		    
		    typeOwner       = owner.GetType();
		    while (typeOwner != null) {
		        if (String.Compare(typeOwner.Assembly.GetName(false).Name, 0, "System", 0, 6, true) == 0) {
		            typeOwner = null;
		        } else {
		            arrMemberInfo   = typeOwner.GetMembers(System.Reflection.BindingFlags.GetField |
		                                                   System.Reflection.BindingFlags.NonPublic |
		                                                   System.Reflection.BindingFlags.Instance |
		                                                   System.Reflection.BindingFlags.Public |
		                                                   System.Reflection.BindingFlags.DeclaredOnly);
                    foreach (System.Reflection.MemberInfo memberInfo in arrMemberInfo) {
                        fieldInfo = typeOwner.GetField(memberInfo.Name,
	                                                System.Reflection.BindingFlags.NonPublic |
		                                            System.Reflection.BindingFlags.Instance |
		                                            System.Reflection.BindingFlags.Public);
                        if (fieldInfo != null) {
                            // TODO: Filter value type....
                            obj = fieldInfo.GetValue(owner);
                            if (obj != null) {
                                if (obj is ToolTip) {
                                    toolTip = obj as ToolTip;
                                } else if (!(obj is Control)) {
                                    SetComponentInfo(obj, fieldInfo.Name,toolTip);
                                }
                            }
                        }
                    }
                    typeOwner = typeOwner.BaseType;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EnumToolStrip:              Enumarate the tool strip items
        /// </summary>
        /// <param name="toolStrip">    Tool strip item</param>
        /// <param name="toolTip">      ToolTip Component</param>
        /// 
        //
        //*********************************************************                
        private void EnumToolStrip(ToolStrip toolStrip, ToolTip toolTip) {
            foreach (ToolStripItem item in toolStrip.Items) {
                if (item is ToolStripMenuItem) {
                    EnumToolStripMenuItems(item as ToolStripMenuItem, toolTip);
                } else if (item is ToolStripItem) {
                    SetComponentInfo(item, item.Name, toolTip);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EnumToolStripMenuItems:     Enumarate the strip menu items
        /// </summary>
        /// <param name="mnuItem">      Strip menu item</param>
        /// <param name="toolTip">      ToolTip Component</param>
        //
        //*********************************************************                
        private void EnumToolStripMenuItems(ToolStripMenuItem mnuItem, ToolTip toolTip) {
            SetComponentInfo(mnuItem, mnuItem.Name, toolTip);
            foreach(ToolStripItem mnuItemChild in mnuItem.DropDownItems) {
                if (mnuItemChild is ToolStripMenuItem) {
                    EnumToolStripMenuItems(mnuItemChild as ToolStripMenuItem, toolTip);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EnumControls:           Enumarate controls of an object
        /// </summary>
        /// <param name="ctl">      Object</param>
        /// <param name="toolTip">  ToolTip Component</param>
        //
        //*********************************************************                
        private void EnumControls(Control ctl, ToolTip toolTip) {
            SetComponentInfo(ctl, ctl.Name, toolTip);
            if (ctl is ToolStrip) {
                EnumToolStrip(ctl as ToolStrip, toolTip);
            }
            if (IsObjectContainsComponent(ctl)) {
                EnumComponents(ctl, ref toolTip);
            }
            foreach (Control ctlChild in ctl.Controls) {
                EnumControls(ctlChild, toolTip);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// TranslateControl:     Translate the control and it's child
        /// </summary>
        /// <param name="ctl">    Control</param>
        //
        //*********************************************************		
		public virtual void TranslateControl(Control ctl) {
            if (m_hash.Keys.Count != 0 || m_fatherTranslator != null) {
                EnumControls(ctl, null);
            }
#if REPDEBUG 
            if (m_hashUsed.Count != 0) {
                System.Diagnostics.Debug.WriteLine(String.Format("Translation: Unused keys in list {0} in this control father {1} and its children during the phase of automatic translation", m_strAtomListName, ctl.Name));
                foreach (String strKey in m_hashUsed.Keys) {
                    System.Diagnostics.Debug.WriteLine(String.Format("\t{0}", strKey));
                }
                m_hashUsed.Clear();
            }
#endif
		}
	}
}
