//*********************************************************
//
// DataGridColumn:              Implement an editor for the Datagrid
//
// Author:                      JFO
//
// (c) 2002 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Data;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
    /// <summary>
    /// Implementation of an editor for the datagrid
    /// </summary>
    //
    //*********************************************************
    public class DataGridColumn : DataGridTextBoxColumn {
        /// <summary>Type of column</summary>
        public enum ColStyleE {
            /// <summary>Standard column</summary>
            CS_STD      = 0,
            /// <summary>Combo box column</summary>
            CS_COMBO    = 1,
            /// <summary>DateTime picker column</summary>
            CS_DATE     = 2,
        };

        /// <summary>Base control</summary>
        private IDataGridColumn         m_base;
        /// <summary>Column style</summary>
        private ColStyleE               m_eStyle;
        /// <summary>Attached field</summary>
        private RepPP.FieldDef          m_fldDef;
        /// <summary>Father datagrid</summary>
        private GridDataTable           m_father;

        //*********************************************************
        //
        /// <summary>
        /// ctor for a datagrid column
        /// </summary>
        /// <param name="prop">     Property descriptor</param>
        /// <param name="fldDef">   Attached field definition</param>
        /// <param name="father">   SCDataGrid to attach</param>
        /// <param name="eStyle">   Style of column:
        ///                         CS_STD      Standard (edit)
        ///                         CS_COMBO    Combo box
        ///                         CS_DATE     Date/time picker</param>
        // 
        //*********************************************************
        public DataGridColumn(PropertyDescriptor prop, RepPP.FieldDef fldDef, GridDataTable father, ColStyleE eStyle) : base(prop) {
            m_base      = null;
            m_fldDef    = fldDef;
            m_father    = father;
            ColStyle    = eStyle;
        }

        //*********************************************************
        //
        /// <summary>
        /// Style of the column
        /// </summary>
        //
        //*********************************************************
        public ColStyleE ColStyle {
            get {
                return(m_eStyle);
            }
            set {
                if (m_eStyle != value) {
                    if (m_base != null) {
                        if (m_father.AttachedDataGrid != null) {
                            m_base.UnsetDataGridInColumn(m_father.AttachedDataGrid);
                        }
                        m_base.Dispose();
                        m_base = null;
                    }
                    switch(value) {
                    case ColStyleE.CS_DATE:
                        m_base = new DataGridDateTimeColumn(m_fldDef, m_father, this);
                        break;
                    case ColStyleE.CS_COMBO:
                        m_base = new DataGridComboBoxColumn(m_fldDef, m_father, this);
                        break;
                    default:
                        break;
                    }
                    m_eStyle = value;
                    if (m_father.AttachedDataGrid != null) {
                        if (m_base != null) {
                            m_base.SetDataGridInColumn(m_father.AttachedDataGrid);
                        }
                        m_father.AttachedDataGrid.Refresh();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Adds the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is added</param>
        //
        //*********************************************************
        protected override void SetDataGridInColumn(DataGrid dataGrid) {
            if (m_base != null) {
                m_base.SetDataGridInColumn(dataGrid);
            } else {
                base.SetDataGridInColumn(dataGrid);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Start the column edition
        /// </summary>
        /// <param name="dataSource">       Current data source</param>
        /// <param name="iRowNum">          Row number</param>
        /// <param name="bounds">           Position of the cell</param>
        /// <param name="bReadOnly">        true for read only column</param>
        /// <param name="strInstantText">   ???</param>
        /// <param name="bCellIsVisible">   true if cell is visible, false if not.</param>
        //
        //*********************************************************
        protected override void Edit(CurrencyManager    dataSource, 
                                     int                iRowNum, 
                                     Rectangle          bounds,
                                     bool               bReadOnly, 
                                     string             strInstantText, 
                                     bool               bCellIsVisible) {
            if (m_base != null) {
                m_base.Edit(dataSource, iRowNum, bounds, bReadOnly, strInstantText, bCellIsVisible);
            } else {
                TextBox.MaxLength       = m_fldDef.MaxSize;
                TextBox.CharacterCasing = (m_fldDef.Type == RepPP.FieldType.sdFieldUpperString) ?
                                           System.Windows.Forms.CharacterCasing.Upper :
                                           System.Windows.Forms.CharacterCasing.Normal;
                base.Edit(dataSource, iRowNum, bounds, bReadOnly, strInstantText, bCellIsVisible);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Abort the edit
        /// </summary>
        /// <param name="iRowNum">  Row number</param>
        //
        //*********************************************************
        protected override void Abort(int iRowNum) {
            if (m_base != null) {
                m_base.Abort(iRowNum);
            } else {
                base.Abort(iRowNum);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Commit the edit
        /// </summary>
        /// <param name="dataSource">   Current data source</param>
        /// <param name="iRowNum">      Row number</param>
        /// <returns></returns>
        //
        //*********************************************************
        protected override bool Commit(CurrencyManager dataSource, int iRowNum) {
            bool    bRetVal;

            bRetVal = (m_base != null) ? m_base.Commit(dataSource, iRowNum) : base.Commit(dataSource, iRowNum);
            return(bRetVal);
        }            

        //*********************************************************
        //
        /// <summary>
        /// Close the combo
        /// </summary>
        //
        //*********************************************************
        protected override void ConcedeFocus() {
            if (m_base != null) {
                m_base.ConcedeFocus();
            } else {
                base.ConcedeFocus();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </summary>
        //
        //*********************************************************
        public bool ListDescription {
            get {
                bool    bRetVal;
                
                bRetVal = (m_base != null) ? m_base.GetListDescription() : true;
                return(bRetVal);
            }
            set {
                if (m_base != null) {
                    m_base.SetListDescription(value);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// true to display the choice description in paint.
        /// false to display the code in paint
        /// </summary>
        //
        //*********************************************************
        public bool ShowDescription {
            get {
                bool    bRetVal;
                
                bRetVal = (m_base != null) ? m_base.GetShowDescription() : true;
                return(bRetVal);
            }
            set {
                if (m_base != null) {
                    m_base.SetShowDescription(value);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the minimum height 
        /// </summary>
        /// <returns>
        /// Object size
        /// </returns>
        //
        //*********************************************************
        protected override int GetMinimumHeight() {
            int iRetVal;

            if (m_base != null) {
                iRetVal = m_base.GetMinimumHeight();
            } else {
                iRetVal = base.GetMinimumHeight();
            }
            return(iRetVal);
        }                                                                                                      

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred height
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object height
        /// </returns>
        //
        //*********************************************************
        protected override int GetPreferredHeight(Graphics  g,
                                                  object    value) {
            int     iRetVal;

            if (m_base != null) {
                iRetVal = m_base.GetPreferredHeight(g, value);
            } else {
                iRetVal = base.GetPreferredHeight(g, value);
            }
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred size of the combo
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object size
        /// </returns>
        //  
        //*********************************************************
        protected override Size GetPreferredSize(Graphics   g,
                                                 object     value) {
            Size    sizeRetVal;

            if (m_base != null) {
                sizeRetVal = m_base.GetPreferredSize(g, value);
            } else {
                sizeRetVal = base.GetPreferredSize(g, value);
            }
            return(sizeRetVal);
        }                                                                                                      

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="backBrush">    Background brush</param>
        /// <param name="foreBrush">    Foreground brush</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        protected override void Paint(Graphics          gr,
                                      Rectangle         bounds,
                                      CurrencyManager   source,
                                      int               iRowNum,
                                      Brush             backBrush,
                                      Brush             foreBrush,
                                      bool              bAlignToRight) {
            if (m_base == null || !m_base.Paint(gr, bounds, source, iRowNum, backBrush, foreBrush, bAlignToRight)) {
                base.Paint(gr, bounds, source, iRowNum, backBrush, foreBrush, bAlignToRight);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        protected override void Paint(Graphics         gr,
                                      Rectangle        bounds,
                                      CurrencyManager  source,
                                      int              iRowNum,
                                      bool             bAlignToRight) {
            if (m_base == null || !m_base.Paint(gr, bounds, source, iRowNum, bAlignToRight)) {
                base.Paint(gr, bounds, source, iRowNum, bAlignToRight);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        //
        //*********************************************************
        protected override void Paint(Graphics          gr,
                                      Rectangle         bounds,
                                      CurrencyManager   source,
                                      int               iRowNum) {
            if (m_base == null || !m_base.Paint(gr, bounds, source, iRowNum, false)) {
                base.Paint(gr, bounds, source, iRowNum, false);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Start editing the column
        /// </summary>
        /// <param name="ctl">  Control</param>
        //
        //*********************************************************
        public new void ColumnStartedEditing(Control ctl) {
            base.ColumnStartedEditing(ctl);
        }

        //*********************************************************
        //
        /// <summary>
        /// Set the value of the column
        /// </summary>
        /// <param name="dataSource">   Data source</param>
        /// <param name="iRowNum">      Row index</param>
        /// <param name="strValue">     Value</param>
        //
        //*********************************************************
        public new void SetColumnValueAtRow(CurrencyManager dataSource, int iRowNum, Object strValue) {
            base.SetColumnValueAtRow(dataSource, iRowNum, strValue);
        }        

        //*********************************************************
        //
        /// <summary>
        /// Get the value of the column
        /// </summary>
        /// <param name="dataSource">   Data source</param>
        /// <param name="iRowNum">      Row index</param>
        /// <returns>
        /// Value
        /// </returns>
        //
        //*********************************************************
        public new Object GetColumnValueAtRow(CurrencyManager dataSource, int iRowNum) {
            return(base.GetColumnValueAtRow(dataSource, iRowNum));
        }        
    } // SCDataGridColumn

    //*********************************************************
    //
    /// <summary>
    /// Interface implement by all controls which want to be hosted in the grid
    /// </summary>
    //
    //*********************************************************
    public interface IDataGridColumn {
        //*********************************************************
        //
        /// <summary>
        /// Dispose the control
        /// </summary>
        //
        //*********************************************************
        void    Dispose();
        //*********************************************************
        //
        /// <summary>
        /// Adds the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is added</param>
        //
        //*********************************************************
        void    SetDataGridInColumn(DataGrid dataGrid);
        //*********************************************************
        //
        /// <summary>
        /// Removes the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is removed</param>
        //
        //*********************************************************
        void    UnsetDataGridInColumn(DataGrid dataGrid);
        //*********************************************************
        //
        /// <summary>
        /// Start the column edition
        /// </summary>
        /// <param name="dataSource">       Current data source</param>
        /// <param name="iRowNum">          Row number</param>
        /// <param name="bounds">           Position of the cell</param>
        /// <param name="bReadOnly">        true for read only column</param>
        /// <param name="strInstantText">   ???</param>
        /// <param name="bCellIsVisible">   true if cell is visible, false if not.</param>
        //
        //*********************************************************
        void    Edit(CurrencyManager dataSource, int iRowNum, Rectangle bounds, bool bReadOnly, string strInstantText, bool bCellIsVisible);
        //*********************************************************
        //
        /// <summary>
        /// Abort the edit
        /// </summary>
        /// <param name="iRowNum">  Row number</param>
        //
        //*********************************************************
        void    Abort(int iRowNum);
        //*********************************************************
        //
        /// <summary>
        /// Commit the edit
        /// </summary>
        /// <param name="dataSource">   Current data source</param>
        /// <param name="iRowNum">      Row number</param>
        /// <returns></returns>
        //
        //*********************************************************
        bool    Commit(CurrencyManager dataSource, int iRowNum);
        //*********************************************************
        //
        /// <summary>
        /// Close the combo
        /// </summary>
        //
        //*********************************************************
        void    ConcedeFocus();
        //*********************************************************
        //
        /// <summary>
        /// Returns if the choice description is shown in the combo list
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        bool    GetListDescription();
        //*********************************************************
        //
        /// <summary>
        /// Determine if the choice description is shown in the combo list
        /// </summary>
        //
        //*********************************************************
        void    SetListDescription(bool b);
        //*********************************************************
        //
        /// <summary>
        /// Returns if the choice description is shown in the grid
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        bool    GetShowDescription();
        //*********************************************************
        //
        /// <summary>
        /// Determine if the choice description is shown in the grid
        /// </summary>
        //
        //*********************************************************
        void    SetShowDescription(bool b);
        //*********************************************************
        //
        /// <summary>
        /// Gets the minimum height 
        /// </summary>
        /// <returns>
        /// Object size
        /// </returns>
        //
        //*********************************************************
        int     GetMinimumHeight();
        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred height
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object height
        /// </returns>
        //
        //*********************************************************
        int     GetPreferredHeight(Graphics  g, object    value);
        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred size of the combo
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object size
        /// </returns>
        //  
        //*********************************************************
        Size    GetPreferredSize(Graphics g, object value);
        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="backBrush">    Background brush</param>
        /// <param name="foreBrush">    Foreground brush</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        bool    Paint(Graphics gr, Rectangle bounds, CurrencyManager source, int iRowNum, Brush backBrush, Brush foreBrush, bool bAlignToRight);
        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        bool    Paint(Graphics gr, Rectangle bounds, CurrencyManager source, int iRowNum, bool bAlignToRight);
        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        //
        //*********************************************************
        bool    Paint(Graphics gr, Rectangle bounds, CurrencyManager source, int iRowNum);
    }; // SCIDataGridColumn
} // Namespace
