//*********************************************************
//
// DataGridDateTimeColumn:	    Implement a date/time editor for the Datagrid
//
// Author:						JFO
//
// (c) 2002 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Data;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
    /// <summary>
    /// Datetime implementation for the datagrid
    /// </summary>
    //
    //*********************************************************
    public class DataGridDateTimeColumn : IDataGridColumn {
        //*********************************************************
        //
        /// <summary>
        /// Datetime super-classing to intercept the OnSelectionChangeCommited
        /// </summary>
        //
        //*********************************************************
        private class SCDateTimePicker : NullableDateTimePicker {
            /// <summary>true if edition is started</summary>
            public  bool                        m_bStartEditing;
            /// <summary>true if calendar is dropped down</summary>
            public  bool                        m_bDroppedDown;
            /// <summary>Column editor owning the combo box</summary>
            public  DataGridDateTimeColumn      m_father;

            //*********************************************************
            //
            /// <summary>
            /// Called when a new value is selected.
            /// Intercepted to indicate to the grid when column is edited
            /// </summary>
            /// <param name="e">    Event argument</param>
            //
            //*********************************************************
            protected override void OnTextChanged(EventArgs e) {
                if (!m_bStartEditing) {
                    m_bStartEditing = true;
                    m_father.m_gridColStyle.ColumnStartedEditing(this);
                }
                base.OnTextChanged(e);
            }

            //*********************************************************
            //
            /// <summary>
            /// Called when the calendar is dropped down
            /// </summary>
            /// <param name="e">    Event argument</param>
            //
            //*********************************************************
            protected override void OnDropDown(EventArgs e) {
                m_bDroppedDown = true;
                base.OnDropDown(e);
            }

            //*********************************************************
            //
            /// <summary>
            /// Called when the dropdown calendar is closed
            /// </summary>
            /// <param name="e">    Event argument</param>
            //
            //*********************************************************
            protected override void OnCloseUp(EventArgs e) {
                m_bDroppedDown = false;
                base.OnCloseUp(e);
            }

            //*********************************************************
            //
            // In:
            //  keyData:            
            // Out:
            /// <summary>
            /// Determine if the key is processed by the control
            /// Intercepted to accept Alt-DownArrow for poping up selection.
            /// </summary>
            /// <param name="keyData">  Key data</param>
            /// <returns>
            /// true:               Input key
            /// false:              Process by the message loop
            /// </returns>
            //
            //*********************************************************
            protected override bool IsInputKey(Keys keyData) {
                bool    bRetVal;
                
                switch(keyData) {
                case Keys.Down | Keys.Alt:
                case Keys.Up | Keys.Alt:
                    bRetVal = true;
                    break;
                default:
                    bRetVal = base.IsInputKey(keyData);
                    break;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            /// <summary>
            /// Override to correct what seems to be a bug in .NET
            /// When the control is selected via m_combo.Select() in Edit() and
            /// the edit is called because of a TAB key, the KEYUP message cause
            /// the control to be unselected.
            /// </summary>
            /// <param name="m">    Message</param>
            /// <returns>
            /// true:               Message processed by the control
            /// false:              Not processed by the control
            /// </returns>
            //
            //*********************************************************
            protected override bool ProcessKeyMessage(ref Message m) {
                bool    bRetVal  = false;
                bool    bCallBase = true;

                switch(m.Msg) {
                case 256:   // WM_KEYDOWN
                case 257:   // WM_KEYUP
                    if (m.Msg == 257 && (int)m.WParam == 9) {
                        // When the control is selected via m_combo.Select() in Edit() and
                        // the edit is called because of a TAB key, the KEYUP message cause
                        // the control to be unselected.
                        bRetVal   = true;
                        bCallBase = false;
                    } else {
                        switch((Keys)(int)m.WParam) {
                        case Keys.Tab:
                        case Keys.Tab | Keys.Shift:
                            if (m_bDroppedDown) {
                                // Seems the control doesn't close on TAB out.... MSFT know why!
                                SendKeys.Send("%{UP}");
                            }
                            break;                            
                        case Keys.Down:
                        case Keys.Up:
                        case Keys.Right:
                        case Keys.Left:
                        case Keys.Home:
                        case Keys.End:
                        case Keys.PageUp:
                        case Keys.PageDown:
                        case Keys.Control | Keys.PageUp:
                        case Keys.Control | Keys.PageDown:
                            if (m_bDroppedDown) {
                                bCallBase = false;
                            }
                            break;
                        default:
                            break;
                        }
                    }
                    break;
                default:        
                    break;
                }
                if (bCallBase) {
                    bRetVal = base.ProcessKeyMessage(ref m);
                }
                return(bRetVal);
            }
        }
        /// <summary>Grid column style</summary>
        private DataGridColumn                  m_gridColStyle;
        /// <summary>Grid data table</summary>
        private GridDataTable                   m_gridDataTable;
        /// <summary>Date time control</summary>
        private SCDateTimePicker                m_dateTimePicker;
        /// <summary>Attached field</summary>
        private RepPP.FieldDef                  m_fldDef;
        /// <summary>true if combo has been activated (must be commited)</summary>
        private bool                            m_bActive;

        //*********************************************************
        //
        /// <summary>
        /// Datagrid date time control ctor
        /// </summary>
        /// <param name="fldDef">       Attached field</param>
        /// <param name="gridDataTable">Grid data table</param>
        /// <param name="gridColStyle"> Grid column style</param>
        //
        //*********************************************************
        public DataGridDateTimeColumn(RepPP.FieldDef fldDef, GridDataTable gridDataTable, DataGridColumn gridColStyle) {
            string                                  strDateSep;
            string                                  strTimeSep;
            string                                  strDateFormat;
            string                                  strTimeFormat;
            System.Globalization.DateTimeFormatInfo info;
            
            m_gridDataTable           = gridDataTable;
            m_gridColStyle            = gridColStyle;
            m_bActive                 = false;
            m_fldDef                  = fldDef;
            m_dateTimePicker          = new SCDateTimePicker();
            m_dateTimePicker.m_father = this;
            info                      = System.Globalization.DateTimeFormatInfo.CurrentInfo;
            strDateSep                = info.DateSeparator;
            strTimeSep                = info.TimeSeparator;
            strDateFormat             = info.ShortDatePattern;
            strTimeFormat             = "HH" + strTimeSep + "mm" + strTimeSep + "ss";
            if (strDateFormat.Substring(0, 1) == "d") {
                strDateFormat = "dd" + strDateSep + "MM" + strDateSep + "yyyy";
            } else if (strDateFormat.Substring(0, 1) == "M" || strDateFormat.Substring(0, 1) == "m") {
                strDateFormat = "MM" + strDateSep + "dd" + strDateSep + "yyyy";
            } else {
                strDateFormat = "yyyy" + strDateSep + "MM" + strDateSep + "dd";
            }
            switch(m_fldDef.Type) {
            case RepPP.FieldType.sdFieldDate:
                m_dateTimePicker.CustomFormat = strDateFormat;
                break;
            case RepPP.FieldType.sdFieldTime:
                m_dateTimePicker.CustomFormat = strTimeFormat;
                break;
            default:
                m_dateTimePicker.CustomFormat = strDateFormat + " " + strTimeFormat;
                break;
            }
            m_dateTimePicker.Format     = System.Windows.Forms.DateTimePickerFormat.Custom;
            m_dateTimePicker.Location   = new System.Drawing.Point(32, 512);
            m_dateTimePicker.Name       = "dateTimePicker";
            m_dateTimePicker.Size       = new System.Drawing.Size(136, 20);
            m_dateTimePicker.TabIndex   = 1;
            m_dateTimePicker.TabStop    = true;
            m_dateTimePicker.Visible    = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose the control
        /// </summary>
        //
        //*********************************************************
        public void Dispose() {
            m_dateTimePicker.Dispose();
            m_dateTimePicker = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// Fills the control with the list of values 
        /// </summary>
        //
        //*********************************************************
        private void FillCtlv() {
            m_dateTimePicker.m_bStartEditing = true;
            if (m_fldDef.IsNull) {
                m_dateTimePicker.IsNull = true;
            } else {
                m_dateTimePicker.IsNull = false;
                m_dateTimePicker.Text   = m_fldDef.Value.ToString();
            }
            m_dateTimePicker.m_bStartEditing = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Adds the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is added</param>
        //
        //*********************************************************
        public virtual void SetDataGridInColumn(DataGrid dataGrid) {
            if (dataGrid.Controls.GetChildIndex(m_dateTimePicker, false) == -1) {
                dataGrid.Controls.Add(m_dateTimePicker);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Removes the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is removed</param>
        //
        //*********************************************************
        public virtual void UnsetDataGridInColumn(DataGrid dataGrid) {
            if (dataGrid.Controls.GetChildIndex(m_dateTimePicker, false) != -1) {
                dataGrid.Controls.Remove(m_dateTimePicker);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Start the column edition
        /// </summary>
        /// <param name="dataSource">       Current data source</param>
        /// <param name="iRowNum">          Row number</param>
        /// <param name="bounds">           Position of the cell</param>
        /// <param name="bReadOnly">        true for read only column</param>
        /// <param name="strInstantText">   ???</param>
        /// <param name="bCellIsVisible">   true if cell is visible, false if not.</param>
        //
        //*********************************************************
        public virtual void Edit(CurrencyManager    dataSource, 
                                 int                iRowNum, 
                                 Rectangle          bounds,
                                 bool               bReadOnly, 
                                 string             strInstantText, 
                                 bool               bCellIsVisible) {

            if (!bReadOnly && m_fldDef.IsSelectable) {
                if (!m_bActive) {
                    FillCtlv();
                    m_bActive = true;
                }
                if (!m_dateTimePicker.Visible) {
                    m_dateTimePicker.Bounds  = bounds;
                    m_dateTimePicker.Visible = true;
                    m_dateTimePicker.Select();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Abort the edit
        /// </summary>
        /// <param name="iRowNum">  Row number</param>
        //
        //*********************************************************
        public virtual void Abort(int iRowNum) {
            if (m_bActive) {
                if (m_dateTimePicker.Visible) {
                    m_dateTimePicker.Visible = false;
                }
                m_bActive = false;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Commit the edit
        /// </summary>
        /// <param name="dataSource">   Current data source</param>
        /// <param name="iRowNum">      Row number</param>
        /// <returns></returns>
        //
        //*********************************************************
        public virtual bool Commit(CurrencyManager dataSource, int iRowNum) {
            bool                bRetVal;
            string              strValue = "";
            BindingManagerBase  bnd;

            if (m_bActive) {
                strValue = m_dateTimePicker.IsNull ? String.Empty : m_dateTimePicker.Value.ToString(m_dateTimePicker.CustomFormat);
                if (strValue != m_fldDef.Value.ToString()) {
                    //
                    //  Bug from Microsoft???
                    //  Sometimes, the current position of the source is different from iRowNum
                    //  Cause a problem... so just skip it...
                    //
                    bnd = m_gridDataTable.AttachedDataGrid.BindingContext[m_gridDataTable];
                    if (bnd == null || bnd.Position == iRowNum) {
                        m_gridColStyle.SetColumnValueAtRow(dataSource, iRowNum, strValue);
                    }
                }
                m_dateTimePicker.Visible = false;
                m_bActive                = false;
                bRetVal                  = true;
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }            

        //*********************************************************
        //
        /// <summary>
        /// Close the combo
        /// </summary>
        //
        //*********************************************************
        public virtual void ConcedeFocus() {
            m_dateTimePicker.Visible = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Returns if the choice description is shown in the combo list
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        public bool GetListDescription() {
            return(true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Determine if the choice description is shown in the combo list
        /// </summary>
        //
        //*********************************************************
        public void SetListDescription(bool bValue) {
        }

        //*********************************************************
        //
        /// <summary>
        /// Returns if the choice description is shown in the grid
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        public bool GetShowDescription() {
            return(true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Determine if the choice description is shown in the grid
        /// </summary>
        //
        //*********************************************************
        public void SetShowDescription(bool bValue) {
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the minimum height 
        /// </summary>
        /// <returns>
        /// Object size
        /// </returns>
        //
        //*********************************************************
        public virtual int GetMinimumHeight() {
            return(m_dateTimePicker.Height);
        }                                                                                                      

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred height
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object height
        /// </returns>
        //
        //*********************************************************
        public virtual int GetPreferredHeight(Graphics g, object value) {
            return(m_dateTimePicker.Height);
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred size of the combo
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object size
        /// </returns>
        //  
        //*********************************************************
        public virtual Size GetPreferredSize(Graphics g, object value) {
            Size    size;

            size = new Size(0, 0);
            return(size);
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="backBrush">    Background brush</param>
        /// <param name="foreBrush">    Foreground brush</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics          gr,
                                  Rectangle         bounds,
                                  CurrencyManager   source,
                                  int               iRowNum,
                                  Brush             backBrush,
                                  Brush             foreBrush,
                                  bool              bAlignToRight) {
            return(false);
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics         gr,
                                  Rectangle        bounds,
                                  CurrencyManager  source,
                                  int              iRowNum,
                                  bool             bAlignToRight) {
            return(false);
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics          gr,
                                  Rectangle         bounds,
                                  CurrencyManager   source,
                                  int               iRowNum) {
            return(false);
        }
    } // SCDataGridDateTimeColumn
} // Namespace
