﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.ComponentModel;

namespace RepPP.Toolkit.Window {

    /// <summary>
    /// 
    /// </summary>
    public abstract class DataGridViewRowsetBase : DataGridView, IUIConfiguration {

        /// <summary>Autofill the grid if true</summary>
        protected Boolean                                 m_bAutoFill;

        /// <summary>WHERE clause used by the autofill grid</summary>
        protected String                                  m_strWhereClause;   

        /// <summary>Event occurs when the column infos has been set</summary>
        [Description("Occurs when the datagridview set the infos of a new column")]
        [Category("Action")]        
        public event DataGridViewColumnEventHandler     SetColumnInfos;

        //********************************************************************************
        //
        /// <summary>
        /// Get the Attached rowset to the datagridview
        /// </summary>
        //
        //********************************************************************************
        public abstract RepPP.Rowset Rowset {
            get; set;       
        }

        //********************************************************************************
        //
        /// <summary>
        /// Get the Attached Rowset to the datagridview by its internal property
        /// </summary>
        //
        //********************************************************************************
        protected virtual RepPP.Rowset InnerRowset {
            get {
                RepPP.Rowset     rsGrid;
                RepPP.Rowset     rsRetVal = null;

                rsGrid = Rowset;
                if (rsGrid != null && !rsGrid.IsDetached) {
                    rsRetVal = rsGrid;
                }
                return(rsRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AutoFill:               Use to auto-fill the data source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Auto fill the data source")]
        [DefaultValue(false)]
		public virtual bool AutoFill {
		    get {
		        return(m_bAutoFill);
		    }
		    set {
		        m_bAutoFill = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// WhereClause:            WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("WHERE Clause use to auto-fill the grid")]
        [DefaultValue("")]
		public virtual string WhereClause {
		    get {
		        return(m_strWhereClause);
		    }
		    set {
		        m_strWhereClause = (value == null) ? "" : value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// OnSetColumnInfos:   Occurs when the column infos has been set
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        public void OnSetColumnInfos(DataGridViewColumnEventArgs e) {
            if (SetColumnInfos != null) {
                SetColumnInfos(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetColumnInfo:                  Set the information of a column attached to a field
        /// </summary>
        /// <param name="col">              Column for which the information is set</param>
        /// <param name="fldDef">           Field definition attached to the column</param>
        /// <param name="bShowDescription"> true if show description instead of code</param>
        //
        //********************************************************************************
        protected virtual void SetColumnInfo(DataGridViewColumn col, RepPP.FieldDef fldDef, bool bShowDescription) {
            RepPP.ChoiceList            choiceList;
            DataGridViewComboBoxColumn  cbItem;
            String                      strChoice;
            
            if (col is DataGridViewComboBoxColumn) {
                cbItem     = (DataGridViewComboBoxColumn)col;
                choiceList = fldDef.ChoiceList;
                cbItem.Items.Clear();
                for (int iIndex = 0; iIndex < choiceList.Count; iIndex++) {
                    if (bShowDescription) {
                        strChoice = choiceList.GetDesc(iIndex);
                        if (strChoice == null || strChoice == String.Empty) {
                            strChoice = choiceList.GetCode(iIndex);
                        }
                    } else {
                        strChoice = choiceList.GetCode(iIndex);
                    }
                    cbItem.Items.Add(strChoice);
                }
            } else if (fldDef.Type == RepPP.FieldType.sdFieldMoney) {
                col.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            }
            if (col is DataGridViewTextBoxColumn) {
                ((DataGridViewTextBoxColumn)col).MaxInputLength = fldDef.MaxSize;
            }
            col.ReadOnly   = !fldDef.IsSelectable || fldDef.IsPrimaryKey;
            col.Visible    = fldDef.IsVisible;
            col.HeaderText = fldDef.Prompt;
            OnSetColumnInfos(new DataGridViewColumnEventArgs(col));
        }

        //********************************************************************************
        //
        /// <summary>
        /// FindColumnIndex:                Finds the index of a column using the name of a field
        /// </summary>
        /// <param name="fldDef">           Field definition</param>
        /// <returns>
        /// Index of the column or -1 if not found.
        /// </returns>
        //
        //********************************************************************************
        protected virtual int FindColumnIndex(FieldDef fldDef) {
            DataGridViewColumn  col;
            int                 iRetVal = -1;

            col = Columns[fldDef.Name];
            if (col != null) {
                iRetVal = col.Index;
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetRowsetLineFromRowIndex:       Finds the index of a line in the rowset
        ///                                 using the index of a row in the grid
        /// </summary>
        /// <param name="iRowIndex">        the index of the row</param>
        /// <returns>
        /// Index of the line or -1 if not found.
        /// </returns>
        //
        //********************************************************************************
        protected virtual int GetRowsetLineFromRowIndex(int iRowIndex){
            return(iRowIndex);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidateField:                  Valide the field
        /// </summary>
        /// <param name="fld">Field to validate</param>
        /// <param name="strError">error text if error</param>
        /// <returns>
        /// True if error otherwise false
        /// </returns>
        //
        //********************************************************************************
        private Boolean ValidateField(RepPP.Field fld, out String strError) {
            String      strFldError;
            Boolean     bRetVal;

            if (fld.FieldDef.Validate(out strFldError) == 0) {
                strError = String.Empty;
                bRetVal  = true;
            } else {
                strError = String.Format("{0}: {1}", fld.FieldDef.Prompt, strFldError);
                bRetVal  = false;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidatingCell:                 Validating the cell in editing 
        /// </summary>
        /// <param name="cell">cell to validate</param>
        /// <returns>
        /// True if valid otherwise false
        /// </returns>
        //
        //********************************************************************************
        protected Boolean ValidatingCell(DataGridViewCell cell) {
            RepPP.Rowset        rsGrid;  
            RepPP.Field         fldCell;
            DataGridViewRow     row;
            DataGridViewColumn  col;
            String              strError;
            int                 iLinePos;
            Boolean             bRetVal     = false;

            rsGrid = InnerRowset;
            row    = cell.OwningRow;
            if (rsGrid != null && rsGrid.LineCount != 0 && row != null && !row.IsNewRow) {
                iLinePos = GetRowsetLineFromRowIndex(row.Index);
                if (iLinePos != -1) {
                    col      = cell.OwningColumn;
                    fldCell  = rsGrid.Fields[col.Name];
                    ClearError(cell);
                    rsGrid.SaveLinePos();
                    try {
                        rsGrid.SelectLine(iLinePos, true, false);
                        if (fldCell != null) {
                            if (ValidateField(fldCell, out strError)) {
                                bRetVal = true;
                            } else {
                                cell.ErrorText = strError;
                            }
                        }
                    } finally {
                        rsGrid.RestoreLinePos(true);
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidatingRow:                 Validating the row in editing 
        /// </summary>
        /// <param name="row">Row to validate</param>
        /// <returns>
        /// True if valid otherwise false
        /// </returns>
        //
        //********************************************************************************
        protected Boolean ValidatingRow(DataGridViewRow row) {
            RepPP.Rowset    rsGrid;            
            String          strError;
            String          strFldError;
            int             iLinePos;
            int             iColIndex;
            Boolean         bRetVal     = true;

            rsGrid = InnerRowset;
            if (rsGrid != null && rsGrid.LineCount != 0 && !row.IsNewRow) {
                iLinePos = GetRowsetLineFromRowIndex(row.Index);
                if (iLinePos != -1) {
                    if (rsGrid.ThisLineState(iLinePos) != LineState.sdLineDeleted) {
                        rsGrid.RowsetDef.SaveActiveRowset();
                        rsGrid.SaveLinePos();
                        try {
                            rsGrid.Select(false, false);
                            rsGrid.SelectLine(iLinePos, true, false);
                            strError = String.Empty;
                            ClearError(row);
                            foreach (Field fld in rsGrid.Fields) {
                                if (fld.FieldDef.Validate(out strFldError) != 0) {
                                    iColIndex = FindColumnIndex(fld.FieldDef);
                                    if (iColIndex != -1) {
                                        row.Cells[iColIndex].ErrorText = strFldError;
                                    }
                                    if (strError != String.Empty) {
                                        strError += "\r\n";
                                    }
                                    strError += fld.FieldDef.Prompt + ": " + strFldError;
                                    bRetVal = false;
                                }
                            }
                            if (!bRetVal) {
                                row.ErrorText = strError;
                            } else if (rsGrid.ValidActiveLine(true) != 0) {
                                bRetVal = false;
                            }
                        } finally {
                            rsGrid.RestoreLinePos(true);
                            rsGrid.RowsetDef.RestoreActiveRowset(true);
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidateRow:                    Validate the current row
        /// </summary>
        /// <returns>
        /// True if current Row is valid
        /// </returns>
        //
        //******************************************************************************** 
        protected virtual Boolean ValidateRow() {
            RepPP.Rowset                        rsGrid;
            DataGridViewCellCancelEventArgs     dgCellCancelArgs;
            Boolean                             bRetVal             = false;

            rsGrid = InnerRowset;
            if (InnerRowset != null && InnerRowset.LineCount != 0) {
                dgCellCancelArgs = new DataGridViewCellCancelEventArgs(CurrentCell.ColumnIndex, CurrentRow.Index);
                OnRowValidating(dgCellCancelArgs);
                bRetVal = !dgCellCancelArgs.Cancel;
            }
             return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetCell:                        Gets the Cell at the location passed
        /// </summary>
        /// <param name="iRowIndex">Row index to get the cell</param>
        /// <param name="iColumnIndex">Column index to get the cell</param>
        /// <returns>
        /// Cell found at the location 
        /// </returns>
        //
        //******************************************************************************** 
        public virtual DataGridViewCell GetCell(int iRowIndex, int iColumnIndex) {
            DataGridViewCell cellRetVal;

            if (iRowIndex < 0 || iRowIndex >= Rows.Count) {
                throw new ArgumentException("Row Index was out of range. Must be non-negative and less than the size of the row collection");
            }
            if (iColumnIndex < 0 || iColumnIndex >= Columns.Count) {
                throw new ArgumentException("Column Index was out of range. Must be non-negative and less than the size of the column collection");
            }
            cellRetVal = Rows[iRowIndex].Cells[iColumnIndex];
            return(cellRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ClearError:                     Clear pending error of the cell
        /// </summary>
        /// <param name="cell">Cell to clear its error</param>
        //
        //******************************************************************************** 
        public virtual void ClearError(DataGridViewCell cell)  {
            cell.ErrorText = String.Empty;
        }

        //********************************************************************************
        //
        /// <summary>
        /// ClearError:                     Clear pending errors of the row
        /// </summary>
        /// <param name="row">Row to clear its errors</param>
        //
        //******************************************************************************** 
        public virtual void ClearError(DataGridViewRow row)  {
            foreach (DataGridViewCell cell in row.Cells) {
                ClearError(cell);
            }
            row.ErrorText = String.Empty;
        }

        //********************************************************************************
        //
        /// <summary>
        /// ClearError:                     Clear All pending errors
        /// </summary>
        //
        //******************************************************************************** 
        public virtual void ClearError()  {
            foreach (DataGridViewRow row in Rows) {
                ClearError(row);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadState:                      Load the control state
        /// </summary>
        /// <param name="helper">           Configuration helper</param>
        /// <param name="node">             Node containing the information</param>
        //
        //********************************************************************************
        public virtual void LoadState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            XmlNode             nodeColStyle;
            String              strVersion;
            String              strColWidth;
            String              strDisplayIndex;
            int                 iWidth;
            int                 iDisplayIndex;
            
            strVersion = helper.GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                foreach (DataGridViewColumn col in Columns) {
                    nodeColStyle = helper.FindNode(node, "col" + col.Name);
                    strColWidth  = helper.GetNodeAttribute(nodeColStyle, "WIDTH");
                    if (strColWidth != null && Int32.TryParse(strColWidth, out iWidth)) {
                        col.Width = iWidth;
                    }
                    strDisplayIndex = helper.GetNodeAttribute(nodeColStyle, "DISPLAYINDEX");
                    if (strDisplayIndex != null && Int32.TryParse(strDisplayIndex, out iDisplayIndex)) {
                        col.DisplayIndex = iDisplayIndex;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveState:                      Save the control state
        /// </summary>
        /// <param name="helper">           Configuration helper</param>
        /// <param name="node">             Node where to save the information</param>
        //
        //********************************************************************************
        public virtual void SaveState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            XmlNode nodeColStyle;

            node.RemoveAll();
            helper.SetNodeAttribute(node, "VERSION", "1");            
            foreach (DataGridViewColumn col in Columns) {
                nodeColStyle = helper.FindNode(node, "col" + col.Name);
                helper.SetNodeAttribute(nodeColStyle, "WIDTH", col.Width.ToString());
                helper.SetNodeAttribute(nodeColStyle, "DISPLAYINDEX", col.DisplayIndex.ToString());
            }
        }

    }//End class DataGridViewRowsetBase
}//End namespace
