﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;

namespace RepPP.Toolkit.Window {

    //********************************************************************************
    //
    /// <summary>
    /// Datagridview virtual rowset
    /// </summary>
    //
    //********************************************************************************
    public class DataGridViewVirtualRowset : DataGridViewRowsetBase, RowsetControlUtil.IRowsetControl, ILineListHandler {

	    /// <summary>Embedded utility class for control attached to rowset</summary>
	    protected RowsetControlUtil                 m_rowsetControlUtil;

        /// <summary>Indicates if any cell is dirty state</summary>
        private Boolean                             m_bDirtyState;

        /// <summary>Contains the last cell value before editing</summary>
        private Object                              m_lastValue;

        /// <summary>Triggers the LineSelectedInList event</summary>
        public event LineSelectedInListEventHandler LineSelectedInList;

        //********************************************************************************
        //
        /// <summary>
        /// DataGridViewRowsetVirtualColumnEventHandler:
        ///                         Occurs when a new column is built from its definition
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        public delegate void DataGridViewRowsetVirtualColumnEventHandler(object sender, DataGridViewRowsetVirtualColumnEventArgs e);

        /// <summary></summary>
        [Description("Occurs when the virtual datagridview adding a new column")]
        [Category("Action")]
        public event DataGridViewRowsetVirtualColumnEventHandler AddingVirtualColumn;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        //
        //********************************************************************************
        public DataGridViewVirtualRowset() : base() {
            base.VirtualMode    = true;
            m_bDirtyState       = false;
            m_lastValue         = null;
            m_rowsetControlUtil = new RowsetControlUtil(this, new WndControlUtilHelper(), false);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        /// <param name="rowset">Rowset binded to grid</param>
        //
        //********************************************************************************
        public DataGridViewVirtualRowset(RepPP.Rowset rowset) : this() {
            if (rowset != null) {
                if (RowsetDef == null) {
                    RowsetDef = rowset.RowsetDef;
                    Rowset    = rowset;
                } else {
                    Rowset = rowset; // Will trigger an exception
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:            Dispose the control
        /// </summary>
        //
        //********************************************************************************
        protected override void Dispose(Boolean bDisposing) {
            if (bDisposing) {
                if (m_rowsetControlUtil != null) {
                    m_rowsetControlUtil.Dispose();
                    m_rowsetControlUtil = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets a value indicating whether you have provided your own data-management
        /// operations for the System.Windows.Forms.DataGridView control.
        /// </summary>
        //
        //********************************************************************************        
        [Browsable(false)]
        [DefaultValue(true)]
        public new Boolean VirtualMode {
            get {
                return(base.VirtualMode);
            }
            set {
                throw new NotImplementedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetName:                  Name of the group use to fill the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.RowsetControlDesigner+RowsetNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [DefaultValue("")]
        [Description("Name of the rowset attached to the grid")]
		public virtual string RowsetName {
		    get {
		        return(m_rowsetControlUtil.RowsetName);
		    }
		    set {
                if (m_rowsetControlUtil.RowsetName != value) {
                    m_rowsetControlUtil.RowsetName = value;
                    InitColumns();
                }
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// RowsetDef:              Attached RowsetDef to grid
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        [Description("RowsetDef attached to the grid")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public RepPP.RowsetDef RowsetDef {
            get {
                RepPP.RowsetDef     rsDefRetVal;

                rsDefRetVal = (m_rowsetControlUtil == null) ? null : m_rowsetControlUtil.RowsetDef;
                return(rsDefRetVal);
            }
            set {
                if (m_rowsetControlUtil.RowsetDef != value) {
                    Rowset                        = null;
                    m_rowsetControlUtil.RowsetDef = value;
                    InitColumns();
                }                
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_rowsetControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Rowset:                 Attached Rowset to grid
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
        [Description("Rowset Attached to its grid")]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public override RepPP.Rowset Rowset {
            get {
                RepPP.Rowset    rsRetVal;

                rsRetVal = (m_rowsetControlUtil == null) ? null : m_rowsetControlUtil.Rowset;
                return(rsRetVal);
            }
            set {
                RepPP.RowsetDef rowsetDef;

                if (m_rowsetControlUtil.Rowset != value) {
                    rowsetDef = RowsetDef;
                    if (rowsetDef == null) {
                        throw new System.ApplicationException("We can not attach a rowset without an existing rowsetDef binded to grid !");
                    }
                    if (rowsetDef != value.RowsetDef) {
                        throw new System.ApplicationException("This rowset has not the same definition its RowsetDef !");
                    }
                    m_rowsetControlUtil.Rowset = value;
                    InitGrid();
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:                             Ignore the setting of the datasource
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public new object DataSource {
            get {
                return(base.DataSource);
            }
            set {
                base.DataSource = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataSourceChanged:                     Ignore the setting of the datasource
        /// </summary>
        //
        //********************************************************************************
        protected override void OnDataSourceChanged(EventArgs e) {
            if (DataSource != null) {
                DataSource = null;
                throw new InvalidOperationException("We can't set a DataSource in context of DataGridViewVirtualRowset");
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnColumnVirtualAdding:     Occurs when a new virtual column is adding to 
        ///                            the columns
        /// </summary>
        //
        //********************************************************************************
        protected virtual void OnColumnVirtualAdding(DataGridViewRowsetVirtualColumnEventArgs e) {
            if (AddingVirtualColumn != null) {
                AddingVirtualColumn(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildColumnsFromDef:       Build columns using the rowset definition
        /// </summary>
        //
        //********************************************************************************
        protected virtual void BuildColumnsFromDef() {
            RepPP.RowsetDef                             rowsetDef;
            RepPP.FieldDef                              fldDef;
            DataGridViewRowsetVirtualColumnEventArgs    e;
            DataGridViewColumnCollection                cols;
            DataGridViewColumn                          col;
            int                                         iFieldCount;

            cols = Columns;
            cols.Clear();
            rowsetDef = m_rowsetControlUtil.RowsetDef;
            if (rowsetDef != null) {
                iFieldCount = rowsetDef.FieldDefs.Count;
                for (int i = 0; i < iFieldCount; i++) {
                    fldDef = rowsetDef.FieldDefs[i];
                    switch(fldDef.Type) {
                    case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                    case RepPP.FieldType.sdFieldSimpleNumChoice:
                        col = new DataGridViewComboBoxColumn();
                        break;
                    case RepPP.FieldType.sdFieldMultiAlphaChoice:
                    case RepPP.FieldType.sdFieldMultiNumChoice:
                        col = new DataGridViewCheckBoxColumn();
                        break;
                    case RepPP.FieldType.sdFieldDate:
                    case RepPP.FieldType.sdFieldDateTime:
                    case RepPP.FieldType.sdFieldTime:
                        col = new DataGridViewTextBoxColumn(); // TODO Create DateTime...
                        break;
                    default:
                        col = new DataGridViewTextBoxColumn();
                        break;
                    }
                    e = new DataGridViewRowsetVirtualColumnEventArgs(col, fldDef);
                    OnColumnVirtualAdding(e);
                    col                     = e.Column;
                    col.Name                = fldDef.Name;
                    col.DataPropertyName    = fldDef.Name;
                    SetColumnInfo(col, fldDef, true);
                    cols.Add(col);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetRowCount:        Initialize the row count for the virtual mode
        /// </summary>
        //
        //********************************************************************************
        private void SetRowCount(int iRowCount) {
            if (AllowUserToAddRows) {
                iRowCount++;
            }
            if (RowCount != iRowCount) {
                RowCount = iRowCount;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// InitColumns:        Initialize the columns of the grid
        /// </summary>
        //
        //********************************************************************************
        private void InitColumns() {
            BuildColumnsFromDef();
            SetRowCount(0);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InitGrid:           Initialize the grid
        /// </summary>
        //
        //********************************************************************************
        protected void InitGrid() {
            int         iRowNeeded;

            iRowNeeded = (InnerRowset == null) ? 0 : InnerRowset.UndelLineCount;
            SetRowCount(iRowNeeded);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetValue:           Gets the value from the rowset using the cardinal cell 
        /// </summary>
        /// <param name="iColIndex">Column's position where get the value</param>
        /// <param name="iRowPos">Row's position where get the value</param>
        //
        //********************************************************************************
        private String GetValue(int iColIndex, int iRowPos) {
            RepPP.Rowset            rsGrid;
            RepPP.Field             fld;
            DataGridViewColumn      col;
            String                  strRetVal   = String.Empty;
            int                     iLinePos;

            rsGrid = InnerRowset;
            if (rsGrid != null && iRowPos < rsGrid.UndelLineCount) {
                if (iColIndex < Columns.Count) {
                    iLinePos  = GetRowsetLineFromRowIndex(iRowPos);
                    if (iLinePos != -1) {
                        col = Columns[iColIndex];
                        fld = rsGrid.Fields[col.Name];
                        if (fld != null) {
                            strRetVal = fld.GetValue(iLinePos, true);
                        }
                    }
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellValueNeeded:      Occurs when the cell value is needed
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnCellValueNeeded(DataGridViewCellValueEventArgs e) {
            RepPP.Rowset    rsGrid;

            rsGrid = InnerRowset;
            if (rsGrid != null && rsGrid.UndelLineCount != 0) {
                e.Value = GetValue(e.ColumnIndex, e.RowIndex);
            }
            base.OnCellValueNeeded(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetValue:           Sets the value to the rowset using the cardinal cell 
        /// </summary>
        /// <param name="strValue">Value to set to the rowset</param>
        /// <param name="iColIndex">Column's position where the value is getting</param>
        /// <param name="iRowPos">Row's position where the value is getting</param>
        //
        //********************************************************************************
        private void SetValue(String strValue, int iColIndex, int iRowPos) {
            RepPP.Rowset            rsGrid;
            RepPP.Field             fld;
            DataGridViewColumn      col;
            int                     iLinePos;

            rsGrid = InnerRowset;
            if (rsGrid != null && iRowPos < rsGrid.UndelLineCount) {
                if (iColIndex < Columns.Count) {
                    iLinePos = GetRowsetLineFromRowIndex(iRowPos);
                    if (iLinePos != -1) {
                        col = Columns[iColIndex];
                        fld = rsGrid.Fields[col.Name];
                        if (fld != null) {
                            fld.SetValue(iLinePos, true, strValue);
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellValuePushed:      Occurs when the cell value is pushed to the rowset
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnCellValuePushed(DataGridViewCellValueEventArgs e) {
            RepPP.Rowset    rsGrid;
            String          strValue;            

            rsGrid = InnerRowset;
            if (rsGrid != null && rsGrid.UndelLineCount != 0) {
                strValue = e.Value as String;
                SetValue(strValue, e.ColumnIndex, e.RowIndex);
            }
            base.OnCellValuePushed(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnUserDeletingRow:      Occurs when the user is deleting a a row
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnUserDeletingRow(DataGridViewRowCancelEventArgs e) {
            int         iOldLine;
            int         iNewLine = -1;
            Boolean     bSuccess;

            iOldLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
            base.OnUserDeletingRow(e);
            if (!e.Cancel) {
                bSuccess = DeleteRow(e.Row);
                if (bSuccess) {
                    if (CurrentRow != null) {
                        iNewLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
                    }
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, iNewLine));
                }
                e.Cancel = !bSuccess;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnRowDirtyStateNeeded:  Occurs when the row need the dirty state
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnRowDirtyStateNeeded(QuestionEventArgs e) {
            e.Response = m_bDirtyState;
            base.OnRowDirtyStateNeeded(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetCurrentCellAddressCore:     Occurs when a cell in row has been selected
        /// </summary>
        /// <param name="columnIndex">Column index selected</param>
        /// <param name="rowIndex">Row index selected</param>
        /// <param name="setAnchorCellAddress">True to make the new current cell the anchor cell for 
        /// a subsequent multicell selection using the SHIFT key otherwise False</param>
        /// <param name="validateCurrentCell">Indicates if we should valide the current cell</param>
        /// <param name="throughMouseClick">True if the current cell is being set as a result of 
        /// a mouse click otherwise False</param>
        /// <returns>
        /// True if the current cell (row) was successfully selected otherwise False
        /// </returns>
        //
        //********************************************************************************
        protected override bool SetCurrentCellAddressCore(int columnIndex, int rowIndex, bool setAnchorCellAddress, bool validateCurrentCell, bool throughMouseClick) {
            RepPP.Rowset        rsGrid;
            int                 iLinePos;
            Boolean             bRetVal  = true;

            rsGrid = InnerRowset;
            if (rsGrid != null) {
                iLinePos = GetRowsetLineFromRowIndex(rowIndex);
                if (rowIndex == -1 || SelectLine(iLinePos)) {
                    bRetVal = base.SetCurrentCellAddressCore(columnIndex, rowIndex, setAnchorCellAddress, validateCurrentCell, throughMouseClick);
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellBeginEdit:        Occurs when the cell is begin edit
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnCellBeginEdit(DataGridViewCellCancelEventArgs e) {
            RepPP.Rowset        rsGrid;

            base.OnCellBeginEdit(e);
            if (!e.Cancel) {
                m_bDirtyState = true;
                if (Rows[e.RowIndex].IsNewRow) {
                    rsGrid = InnerRowset;
                    rsGrid.AddBlankLine(true);
                }
                m_lastValue = CurrentCell.Value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellEndEdit:          Occurs when the cell is End edit
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnCellEndEdit(DataGridViewCellEventArgs e) {
            m_bDirtyState = false;
            ValidatingCell(GetCell(e.RowIndex, e.ColumnIndex));
            base.OnCellEndEdit(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCancelRowEdit:        Occurs when the row editing is cancelled
        /// </summary>
        /// <param name="e">Event argument</param>
        //
        //********************************************************************************
        protected override void OnCancelRowEdit(QuestionEventArgs e) {
            CurrentCell.Value = m_lastValue;
            ValidatingRow(CurrentRow);
            base.OnCancelRowEdit(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// MoveRow:                         Move the current row to the new row index
        /// </summary>
        /// <param name="iRowIndexDest">The new row index</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public Boolean MoveRow(int iRowIndexDest) {
            RepPP.Rowset        rsGrid;
            DataGridViewRow     row;
            int                 iLinePos;
            int                 iLinePosDest;
            int                 iMoveDirection;
            Boolean             bRetVal     = false;

            row = CurrentRow;
            if (!row.IsNewRow) {
                rsGrid = InnerRowset;                
                if (rsGrid != null && rsGrid.UndelLineCount != 0) {
                    iLinePosDest = GetRowsetLineFromRowIndex(iRowIndexDest);
                    if (iLinePosDest != -1) {
                        iLinePos       = GetRowsetLineFromRowIndex(row.Index);
                        iMoveDirection = (iLinePosDest <= iLinePos) ? -1 : 0;
                        rsGrid.SelectUndelLine(iLinePos, true, false);
                        rsGrid.MoveLineTo(iLinePosDest + iMoveDirection);
                        row          = Rows[iLinePosDest];
                        row.Selected = true;
                        CurrentCell  = row.Cells[0];
                        OnLineSelectedInList(new LineSelectedInListEventArgs(iLinePos, iLinePosDest));
                        Refresh();
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectLine:                    Select a row from the rowset
        /// </summary>
        /// <param name="iLinePos">        Line position in Rowset</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public Boolean SelectLine(int iLinePos) {
            RepPP.Rowset        rsGrid;
            DataGridViewRow     row;
            int                 iOldLine        = -1;
            Boolean             bRetVal         = true;

            rsGrid = InnerRowset;
            if (rsGrid != null && iLinePos != -1) {
                if (iLinePos < rsGrid.UndelLineCount && iLinePos != rsGrid.CurrentUndelLine) {
                    if (CurrentRow != null) {
                        iOldLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
                        bRetVal  = ValidatingRow(CurrentRow);
                    }
                    if (bRetVal) {
                        bRetVal  = (rsGrid.SelectUndelLine(iLinePos, true, true) == 0);
                        if (bRetVal) {
                            row          = Rows[iLinePos];
                            row.Selected = true;
                            CurrentCell  = row.Cells[0];
                            OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, iLinePos));
                        }
                    }
                }
            } else {
                bRetVal = false;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InsertLine:                     Insert a row
        /// </summary>
        /// <param name="bAtEnd">           True to insert at the End of the grid otherwise 
        ///                                 the new line will insert at the current position</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public Boolean InsertLine(bool bAtEnd) {
            RepPP.Rowset        rsGrid;
            int                 iLinePos;
            int                 iCurrentLine;
            Boolean             bRetVal     = false;

            rsGrid = InnerRowset;
            if (rsGrid != null) {
                iLinePos = (bAtEnd) ? RowCount - 1 : GetRowsetLineFromRowIndex(CurrentRow.Index);
                bRetVal = SelectLine(iLinePos);
                if (bRetVal && rsGrid.AddBlankLine(bAtEnd) == 0) {
                    RowCount++;
                    iCurrentLine                = rsGrid.CurrentUndelLine;
                    CurrentCell                 = Rows[iCurrentLine].Cells[0];
                    Rows[iCurrentLine].Selected = true;
                    BeginEdit(true);
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iLinePos, iCurrentLine));
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateLine:                  Duplicate a row
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public Boolean DuplicateLine() {
            RepPP.Rowset        rsGrid;
            DataGridViewRow     row;
            int                 iLinePos;
            Boolean             bRetVal     = false;

            row = CurrentRow;
            if (!row.IsNewRow) {
                rsGrid = InnerRowset;                
                if (rsGrid != null && rsGrid.UndelLineCount != 0) {
                    iLinePos = GetRowsetLineFromRowIndex(row.Index);
                    if (iLinePos != -1) {
                        rsGrid.SelectUndelLine(iLinePos, true, false);
                        if (rsGrid.DuplicateLine() == 0) {
                            RowCount++;
                            OnLineSelectedInList(new LineSelectedInListEventArgs(iLinePos, rsGrid.CurrentUndelLine));
                            bRetVal = true;
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteRow:              Delete the row
        /// </summary>
        /// <param name="row">Row to delete</param>
        /// <returns>
        /// True if succeed otherwise false
        /// </returns>
        //
        //********************************************************************************
        private Boolean DeleteRow(DataGridViewRow row) {
            RepPP.Rowset        rsGrid;
            IntPtr              hBookmark;
            int                 iLinePos;
            Boolean             bRetVal     = false;

            if (!row.IsNewRow) {
                rsGrid = InnerRowset;
                if (rsGrid != null && rsGrid.UndelLineCount != 0) {
                    iLinePos = GetRowsetLineFromRowIndex(row.Index);
                    if (iLinePos != -1) {
                        if (rsGrid.SelectUndelLine(iLinePos, true, true) == 0) {
                            hBookmark = rsGrid.LineBookmark(iLinePos);
                            if (rsGrid.DeleteLine() == 0) {
                                ClearError(row);
                                bRetVal = true;
                            }
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteCurrentLine:          Delete the current row of the DataGridView
        /// </summary>
        /// <returns>
        /// True if succeed otherwise false
        /// </returns>
        //
        //********************************************************************************
        public Boolean DeleteCurrentLine() {
            RepPP.Rowset    rsGrid;    
            int             iOldLine;
            int             iNewLine    = -1;
            Boolean         bRetVal     = false;


            if (CurrentRow != null) {
                rsGrid = InnerRowset;
                if (rsGrid != null) {
                    iOldLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
                    bRetVal  = DeleteRow(CurrentRow);
                    if (bRetVal) {
                        RowCount--;
                        if (CurrentRow != null) {
                            iNewLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
                            rsGrid.SelectUndelLine(iNewLine, true, true);
                        }
                        OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, iNewLine));
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
        /// CreateRowset:          Create an autofill rowset
		/// </summary>
        //
        //********************************************************************************
        private void CreateRowset() {
            RepPP.RowsetDef     rowsetDef;
            RepPP.Rowset        rsAutoFill;
            String              strSelect;
            String              strDummy;            
            
            if (RowsetDef != null) {
                if (Rowset != null) {
                    Rowset.Dispose();                
                }
                rowsetDef = RowsetDef;
                rowsetDef.BuildSqlCommand("", out strSelect, out strDummy, out strDummy, out strDummy);
                strSelect  = Application.Tool.SetSqlWhere(strSelect, m_strWhereClause);
                rsAutoFill = rowsetDef.Rowsets.Create(RowsetType.sdHorizontal);
                if (rsAutoFill.ReadFromDb(strSelect) == 0) {
                    Rowset = rsAutoFill;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCreateControl:        Occurs when the control
        /// </summary>
        //
        //********************************************************************************
        protected override void OnCreateControl() {
            if (!DesignMode) {
                InitColumns();
                if (m_bAutoFill) {
                    CreateRowset();
                } else {
                    InitGrid();
                }
            }
            base.OnCreateControl();
        }

        //********************************************************************************
        //
        /// <summary>
        /// CommitEdit:                     Commit the edit
        /// </summary>
        //
        //********************************************************************************
        public void CommitEdit() {
            CommitEdit(DataGridViewDataErrorContexts.Commit);
            if (CurrentRow != null) {
                ValidatingRow(CurrentRow);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AssociateRowset:         Associate a rowset with the grid
        /// </summary>
        /// <param name="rowset">Rowset to associate</param>
        //
        //********************************************************************************
        public void AssociateRowset(Rowset rowset) {
            Rowset = rowset;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLineSelectedInList:           Triggers the LineSelectedInList event
        /// </summary>
        /// <param name="e">                Event Argument</param>
        //
        //********************************************************************************
        public void OnLineSelectedInList(LineSelectedInListEventArgs e) {
            if (LineSelectedInList != null) {
                LineSelectedInList(this, e);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            InitColumns();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is resetted
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public void TriggerComponentChange() {
            if (m_rowsetControlUtil != null) {
                m_rowsetControlUtil.TriggerComponentChange();
            }            
        }

        //********************************************************************************
        //
        /// <summary>
        /// Event argument when the virtual rowset datagridview add a new column 
        /// from its definition
        /// </summary>
        //
        //********************************************************************************
        public class DataGridViewRowsetVirtualColumnEventArgs {
            /// <summary>Field Definition containing the definition of the Column</summary>
            private RepPP.FieldDef     m_fldDef;

            /// <summary>Column built from its definition</summary>
            private DataGridViewColumn m_col;

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor 
            /// </summary>
            /// <param name="column">New column built from its definition</param>
            /// <param name="fldDef">Field containing the definition of the Column</param>
            //
            //********************************************************************************
            public DataGridViewRowsetVirtualColumnEventArgs(DataGridViewColumn column, RepPP.FieldDef fldDef) {
                m_col    = column;
                m_fldDef = fldDef;
            }

            //********************************************************************************
            //
            /// <summary>
            /// Gets/Sets the new column built from its definition
            /// </summary>
            //
            //********************************************************************************
            public DataGridViewColumn Column { 
                get {
                    return(m_col);
                }
                set {
                    m_col.Dispose();
                    m_col = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// Gets the Field Definition containing the definition of the Column
            /// </summary>
            //
            //********************************************************************************            
            public RepPP.FieldDef fldDef {
                get {
                    return(m_fldDef);
                }
            }
        }//End class DataGridViewRowsetVirtualColumnEventArgs   

        //********************************************************************************
        //
        /// <summary>
        /// Image Column without the bug of the native DataGridViewImageColumn
        /// Bug who display a 'X' image on the new Row instead display a picture
        /// </summary>
        //
        //********************************************************************************
        public class DataGridViewVirtualRowsetImageColumn : DataGridViewImageColumn {

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor
            /// </summary>
            //
            //********************************************************************************
            public DataGridViewVirtualRowsetImageColumn()
                : base() {
                InitColumn();
            }            

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor
            /// </summary>
            /// <param name="bValueIsIcon">Flag to indicate that the Value property of cells 
            /// in this column will be set to values of type Icon if true 
            /// otherwise false for type Image.</param>
            //
            //********************************************************************************
            public DataGridViewVirtualRowsetImageColumn(Boolean bValueIsIcon) 
                : base(bValueIsIcon) {
                InitColumn();
            }

            //********************************************************************************
            //
            /// <summary>
            /// InitColumn:             Init the properties of this column
            /// </summary>
            //
            //********************************************************************************
            private void InitColumn() {
                DefaultCellStyle.NullValue = null;
                CellTemplate               = new DataGridViewVirtualRowsetImageCell();
            }

            //********************************************************************************
            //
            /// <summary>
            /// Image Cell without default image association
            /// </summary>
            //
            //********************************************************************************
            private class DataGridViewVirtualRowsetImageCell : DataGridViewImageCell {

                //********************************************************************************
                //
                /// <summary>
                /// Gets the default value that is used when creating a new row
                /// </summary>
                ///             
                //********************************************************************************
                public override object DefaultNewRowValue {
                    get {
                        return(null);
                    }
                }

            }//End class DataGridViewVirtualRowsetImageCell    
        
        }//End class DataGridViewVirtualRowsetImageColumn

    }//End class DataGridViewVirtualRowset
}//End namespace
