//*********************************************************
//
// DynamicForm          Handle dynamic form creation
//
// Author:		        Jacques Fournier
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using RepPP.Toolkit;

namespace RepPP.Toolkit.Window {
    /// <summary>
    /// Handle dynamic form creation
    /// </summary>
    public class DynamicForm {
        /// <summary>Options for form generation</summary>
        public enum OptionE {
            /// <summary>No option</summary>
            OPT_NONE                            = 0,
            /// <summary>Only add field attached to the main table</summary>
            OPT_ONLY_MAIN_TBL_FIELD             = 1,
            /// <summary>Field are always selectable</summary>
            OPT_ALWAYS_SELECTABLE               = 2,
            /// <summary>Replace radio by a combo box</summary>
            OPT_NO_RADIO                        = 4,
            /// <summary>Replace check by listbox</summary>
            OPT_NO_CHECK                        = 8,
            /// <summary>Add a combo box with operators</summary>
            OPT_ADD_OPERATOR_COMBO              = 16,
            /// <summary>Add an additional invisible field for BETWEEN clause</summary>
            OPT_ADD_BETWEEN_FIELD               = 32,
            /// <summary>Don't add fields which are user flsg</summary>
            OPT_NO_USER_FLAG_FIELD              = 64
        };
        /// <summary>
        /// Default control height
        /// </summary>
        private const int   DEF_CTL_HEIGHT      =   14;
        /// <summary>
        /// Horizontal space between controls
        /// </summary>
        private const int   HORZ_MARGE_SPACE    =   4;
        /// <summary>
        /// Vertical space between controls
        /// </summary>
        private const int   VERT_MARGE_SPACE    =   4;
        /// <summary>
        /// Size of the bitmap for the combobox, radio or check
        /// </summary>
        private const int   BUTTON_BITMAP_SIZE  =   12;

        //*********************************************************	    
        //
        /// <summary>
        /// ToPixelX:               Convert our logical unit in pixel
        /// </summary>
        /// <param name="gr">       Graphic object</param>
        /// <param name="x">        X position to translate</param>
        /// <param name="font">     font used</param>
        /// <returns>
        /// Position in pixels
        /// </returns>
        //
        //*********************************************************	    
        static private int ToPixelX(System.Drawing.Graphics gr, int x, System.Drawing.Font font)
        {
            return((int)(gr.MeasureString("H", font).Width * x / 4));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ToPixelY:               Convert our logical unit in pixel
        /// </summary>
        /// <param name="gr">       Graphic object</param>
        /// <param name="y">        Y position to translate</param>
        /// <param name="font">     font used</param>
        /// <returns>
        /// Position in pixels
        /// </returns>
        //
        //*********************************************************	    
        static private int ToPixelY(System.Drawing.Graphics gr, int y, System.Drawing.Font font)
        {
            return((int)(font.GetHeight(gr) * y / 8));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ComputeMaxPromptSizei:  Compute the maximum prompt size
        /// </summary>
        /// <param name="gr">       Graphic object</param>
        /// <param name="rowsetDef">Rowset definition to use as template</param>
        /// <param name="font">     Font to use</param>
        //
        //*********************************************************	    
        static private int ComputeMaxPromptSizei(System.Drawing.Graphics gr, RepPP.RowsetDef rowsetDef, System.Drawing.Font font)
        {
            int                     iRetVal = 0;
            int                     iPromptSize;
            
            foreach (RepPP.FieldDef fldDef in rowsetDef.FieldDefs) {
                iPromptSize = (int)gr.MeasureString(fldDef.Prompt, font).Width;
                if (iPromptSize > iRetVal) {
                    iRetVal = iPromptSize;
                }
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ComputeMaxChoiceSizei:      Compute the maximum choice size
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="choiceList">   Choice list</param>
        /// <param name="font">         Font to use</param>
        /// <param name="bCodeToo">     true to add the code and tab between code and description</param>
        //
        //*********************************************************	    
        static private int ComputeMaxChoiceSizei(System.Drawing.Graphics gr, RepPP.ChoiceList choiceList, System.Drawing.Font font, bool bCodeToo)
        {
            int                     iRetVal = 0;
            int                     iChoiceSize;
            int                     iChoiceCount;
            string                  strCode;
            string                  strDesc;
            string                  str;
            
            iChoiceCount = choiceList.Count;
            for (int i = 0; i < iChoiceCount; i++) {
                strCode = choiceList.GetCode(i);
                strDesc = choiceList.GetDesc(i);
                if (bCodeToo) {
                    if (strDesc == null || strDesc.Length == 0) {
                        str = strCode;
                    } else {
                        str = strCode + "\t" + strDesc;
                    }
                } else {
                    str = (strDesc == null || strDesc.Length == 0) ? strCode : strDesc;
                }
                iChoiceSize = (int)gr.MeasureString(str, font).Width;
                if (iChoiceSize > iRetVal) {
                    iRetVal = iChoiceSize;
                }
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateFieldCtl:                     Create a field control in the specified container.
        /// </summary>
        /// <param name="ctl">                  Control in which to add the controls</param>
        /// <param name="fldDef">               Field definition attached to the control to create</param>
        /// <param name="eOption">              Generation option</param>
        /// <param name="gr">                   Graphic object</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iYPos">                Y position of the control</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        /// <param name="iMaxFieldXSize">       Maximum width of the field control in pixel</param>
        /// <param name="iButtonXSize">         Width of the button</param>
        /// <param name="iStaticYSize">         Height of static control</param>
        /// <param name="iYMargin">             Y margin</param>
        /// <param name="iXMargin">             X margin</param>
        /// <param name="iComboBoxWidth">       Width of the combo box</param>
        /// <param name="iSpaceAfterCombo">     Space after the combo box</param>
        /// <param name="iMaxPromptSize">       Maximum size of the prompt in pixel</param>
        /// <param name="statCtl">              Returned static control if one created</param>
        /// <param name="fldCtl">               Returned the field control if one created</param>
        /// <param name="comboCtl">             Returned combobox control if one created</param>
        /// <param name="iFieldYSize">          Height of the created control</param>
        //
        //*********************************************************	    
        static private void CreateFieldCtl(System.Windows.Forms.Control  ctl,
                                           RepPP.FieldDef                fldDef,
                                           OptionE                       eOption,
                                           Graphics                      gr,
                                           System.Drawing.Font           font, 
                                           int                           iYPos,
                                           int                           iMaxFieldSizeInChr,
                                           int                           iMaxFieldXSize,
                                           int                           iButtonXSize,
                                           int                           iStaticYSize,
                                           int                           iYMargin,
                                           int                           iXMargin,
                                           int                           iComboBoxWidth,
                                           int                           iSpaceAfterCombo,
                                           int                           iMaxPromptSize,
                                           out RepPP.RepPPStat           statCtl,
                                           out RepPP.RepPPField          fldCtl,
                                           out ComboBox                  comboCtl,
                                           out int                       iFieldYSize) {
            bool                bBetweenField;
            RepPP.CtlType       eCtlType;
            RepPP.FieldType     eFldType;
            int                 iFieldXSize;
            int                 iFieldXPos;
            int                 iTmpSize;
            int                 iMaxFieldXSizeDiv2;
            int                 iFieldAdjustXPos;
            int                 iStaticXPos;

            fldCtl              = null;
            statCtl             = null;
            comboCtl            = null;
            iMaxFieldXSizeDiv2  = iMaxFieldXSize / 2 - 1;
            iStaticXPos         = iXMargin;
            iFieldXPos          = iStaticXPos + iMaxPromptSize + iXMargin;
            bBetweenField       = (fldDef.Name[0] == '_');
            eCtlType            = fldDef.ControlType;
            eFldType            = fldDef.Type;
            switch(eCtlType) {
            case RepPP.CtlType.sdCtlCheck:
                if ((eOption & OptionE.OPT_NO_CHECK) != 0) {
                    if (eFldType == FieldType.sdFieldMultiAlphaChoice || eFldType == FieldType.sdFieldMultiNumChoice) {
                        eCtlType = RepPP.CtlType.sdCtlList;
                    } else {
                        eCtlType = RepPP.CtlType.sdCtlCombo;
                    }
                }
                break;
            case RepPP.CtlType.sdCtlRadio:
                if ((eOption & OptionE.OPT_NO_RADIO) != 0) {
                    if (eFldType == FieldType.sdFieldMultiAlphaChoice || eFldType == FieldType.sdFieldMultiNumChoice) {
                        eCtlType = RepPP.CtlType.sdCtlList;
                    } else {
                        eCtlType = RepPP.CtlType.sdCtlCombo;
                    }
                }
                break;
            case RepPP.CtlType.sdCtlEdit:
                if (fldDef.ChoiceList.Count > 0) {
                    eCtlType = RepPP.CtlType.sdCtlCombo;
                }
                break;
            }
            iFieldXSize = 0;
            switch(eCtlType) {
            case RepPP.CtlType.sdCtlCheck:
            case RepPP.CtlType.sdCtlRadio:
                iFieldXSize = ComputeMaxChoiceSizei(gr, fldDef.ChoiceList, font, false) + iButtonXSize;
                iFieldYSize = (font.Height + iYMargin) * fldDef.ChoiceList.Count - iYMargin;
                break;
            case RepPP.CtlType.sdCtlCombo:
                iFieldXSize = ComputeMaxChoiceSizei(gr, fldDef.ChoiceList, font, true) + iButtonXSize;
                iFieldYSize = iStaticYSize;
                break;
            case RepPP.CtlType.sdCtlList:
                iFieldXSize = ComputeMaxChoiceSizei(gr, fldDef.ChoiceList, font, true) + iButtonXSize;
                iFieldYSize = (iStaticYSize + iYMargin) * 4 - iYMargin;
                break;
            case RepPP.CtlType.sdCtlFmt:
            case RepPP.CtlType.sdCtlFmtDate:
            case RepPP.CtlType.sdCtlMChr:
            case RepPP.CtlType.sdCtlMLine:
                iFieldYSize = iStaticYSize;
                break;
            case RepPP.CtlType.sdCtlEdit:
            default:
                eCtlType    = RepPP.CtlType.sdCtlEdit;
                iFieldYSize = (fldDef.Type == RepPP.FieldType.sdFieldMemo) ? (iStaticYSize + iYMargin) * 4 - iYMargin : iStaticYSize;
                break;
            }
            if (iFieldXSize == 0) {
                iTmpSize    = (fldDef.MaxSize < iMaxFieldSizeInChr) ? fldDef.MaxSize : iMaxFieldSizeInChr;
                iFieldXSize = (int)gr.MeasureString("M".PadRight(iTmpSize, 'M'), font).Width;
            }
            if (bBetweenField) {
                if (iFieldXSize > iMaxFieldXSizeDiv2) {
                    iFieldXSize = iMaxFieldXSizeDiv2;
                }
                iFieldAdjustXPos  = iFieldXPos + 2;
                if ((eOption & OptionE.OPT_ADD_OPERATOR_COMBO) != 0) {
                    iFieldAdjustXPos += iComboBoxWidth + iSpaceAfterCombo + iFieldXSize;
                }
            } else {
                if (iFieldXSize > iMaxFieldXSize) {
                    iFieldXSize = iMaxFieldXSize;
                }
                statCtl                 = new RepPP.RepPPStat();
                statCtl.AtomName        = "";
                statCtl.AtomType        = RepPP.AtomType.sdAtomField;
                statCtl.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                statCtl.ControlType     = RepPP.CtlType.sdCtlStatic;
                statCtl.FieldName       = fldDef.Name;
                statCtl.Font            = font;
                statCtl.ForeColor       = System.Drawing.Color.Black;
                statCtl.RowsetName       = fldDef.RowsetDef.Name;
                statCtl.Location        = new System.Drawing.Point(iStaticXPos, iYPos);
                statCtl.Name            = "sta" + fldDef.Name;
                statCtl.Prefix          = false;
                statCtl.Size            = new System.Drawing.Size(iMaxPromptSize, iStaticYSize);
                statCtl.TabIndex        = 0;
                statCtl.TabStop         = false;
                statCtl.TextAlign       = System.Windows.Forms.HorizontalAlignment.Left;
                ctl.Controls.Add(statCtl);
                if ((eOption & OptionE.OPT_ADD_OPERATOR_COMBO) != 0) {
                    comboCtl            = new ComboBox();
                    comboCtl.Name       = "combo" + fldDef.Name;
                    comboCtl.Location   = new System.Drawing.Point(iFieldXPos, iYPos);
                    comboCtl.Size       = new System.Drawing.Size(iComboBoxWidth, iFieldYSize);
                    comboCtl.TabIndex   = fldDef.LogicalPos;
                    comboCtl.TabStop    = true;
                    iFieldAdjustXPos    = iFieldXPos + iComboBoxWidth + iSpaceAfterCombo;
                    ctl.Controls.Add(comboCtl);
                } else {
                    iFieldAdjustXPos = iFieldXPos;
                }
            }
            fldCtl                  = new RepPP.RepPPField();
            fldCtl.AutoHScroll      = true;
            fldCtl.AutoVScroll      = false;
            fldCtl.BackColor        = System.Drawing.SystemColors.Window;
            fldCtl.BorderStyle      = System.Windows.Forms.BorderStyle.Fixed3D;
            fldCtl.ComboBoxStyle    = System.Windows.Forms.ComboBoxStyle.DropDownList;
            fldCtl.ControlType      = eCtlType;
            fldCtl.FieldName        = fldDef.Name;
            fldCtl.Font             = font;
            fldCtl.ForeColor        = System.Drawing.SystemColors.ControlText;
            fldCtl.RowsetName       = fldDef.RowsetDef.Name;
            fldCtl.HideSelection    = true;
            fldCtl.LeftText         = false;
            fldCtl.Location         = new System.Drawing.Point(iFieldAdjustXPos, iYPos);
            fldCtl.Multiline        = (fldDef.Type == RepPP.FieldType.sdFieldMemo) ? true : false;
            fldCtl.Name             = "fldDef" + fldDef.Name;
            fldCtl.Placement        = RepPP.Placement.sdTopToBottomWrap;
            fldCtl.ScrollBars       = (eCtlType == CtlType.sdCtlList) ? ScrollBars.Vertical : ScrollBars.None;
            fldCtl.Size             = new System.Drawing.Size(iFieldXSize, iFieldYSize);
            fldCtl.TabIndex         = fldDef.LogicalPos + 1;
            fldCtl.TabStop          = true;
            fldCtl.TextAlign        = System.Windows.Forms.HorizontalAlignment.Left;
            fldCtl.Visible          = fldDef.IsVisible;
            if (eCtlType == CtlType.sdCtlCombo && fldDef.ControlType == CtlType.sdCtlEdit) {
                fldCtl.ComboBoxStyle = ComboBoxStyle.DropDown;
            }
            ctl.Controls.Add(fldCtl);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFields:                 Create field controls in the specified container.
        /// </summary>
        /// <param name="ctl">                  Control in which to add the controls</param>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="arrFieldDef">          Array of field definitions to use to create the controls</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        /// <param name="iComboBoxWidthInChr">  Width of combobox if option is selected</param>
        /// <param name="iSpaceAfterComboInChr">Space after combo in characters</param>
        /// <param name="eOption">              Generation option</param>
        /// <param name="iMaxXPos">             Returned the maximum right edge of a field</param>
        /// <param name="listCombo">            Array of combo box generated if option specified</param>
        /// <param name="listStat">             Array of created static fields</param>
        /// <param name="listField">            Array of created fields</param>
        /// <param name="listBetweenField">     Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        static private void CreateRowsetFields(System.Windows.Forms.Control ctl,
                                             RepPP.RowsetDef                rowsetDef,
                                             ArrayList                      arrFieldDef,
                                             System.Drawing.Font            font, 
                                             int                            iMaxPromptSizeInChr,
                                             int                            iMaxFieldSizeInChr,
                                             int                            iComboBoxWidthInChr,
                                             int                            iSpaceAfterComboInChr,
                                             OptionE                        eOption,
                                             out int                        iMaxXPos,
                                             ArrayList                      listCombo,
                                             ArrayList                      listStat,
                                             ArrayList                      listField,
                                             ArrayList                      listBetweenField)
        {
            int                 iMaxPromptSize;
            Graphics            gr;
            RepPP.RepPPStat     statCtl;
            RepPP.RepPPField    fldCtl;
            ComboBox            comboCtl;
            int                 iYPos;
            int                 iYOldPos;
            int                 iStaticYSize;
            int                 iFieldYSize;
            int                 iMaxFieldXSize;
            int                 iXMargin;
            int                 iYMargin;
            int                 iButtonXSize;   // Use for combo, radio or check
            int                 iComboBoxWidth;
            int                 iSpaceAfterCombo;
            int                 iTmpSize;
            bool                bBetweenField;
            
            if (font == null) {
                font = ctl.Font;
            }
            gr               = ctl.CreateGraphics();
            iXMargin         = ToPixelX(gr, HORZ_MARGE_SPACE, font);
            iYMargin         = ToPixelY(gr, VERT_MARGE_SPACE, font);
            iTmpSize         = (iMaxPromptSizeInChr == 0) ? 0 : (int)gr.MeasureString("M".PadRight(iMaxPromptSizeInChr, 'M'), font).Width;
            iMaxPromptSize   = ComputeMaxPromptSizei(gr, rowsetDef, font);
            iComboBoxWidth   = (iComboBoxWidthInChr == 0) ? 0 : (int)gr.MeasureString("M".PadRight(iComboBoxWidthInChr, 'M'), font).Width;
            iSpaceAfterCombo = (iSpaceAfterComboInChr == 0) ? 0 : (int)gr.MeasureString("M".PadRight(iSpaceAfterComboInChr, 'M'), font).Width;
            if (iMaxPromptSize > iTmpSize) {
                iMaxPromptSize = iTmpSize;
            }
            iMaxFieldXSize      = (iMaxFieldSizeInChr == 0) ? 0 : (int)gr.MeasureString("M".PadRight(iMaxFieldSizeInChr, 'M'), font).Width;
            iStaticYSize        = ToPixelY(gr, DEF_CTL_HEIGHT, font); 
            iYPos               = iYMargin;
            iButtonXSize        = ToPixelX(gr, BUTTON_BITMAP_SIZE, font);
            iMaxXPos            = 2 * iXMargin + iMaxPromptSize + iComboBoxWidth + iSpaceAfterCombo + iMaxFieldXSize;
            iYOldPos            = iYPos;
            foreach (RepPP.FieldDef fldDef in arrFieldDef) {
                bBetweenField = (fldDef.Name[0] == '_');                
                CreateFieldCtl(ctl,
                               fldDef,
                               eOption,
                               gr,
                               font, 
                               (bBetweenField) ? iYOldPos : iYPos,
                               iMaxPromptSizeInChr,
                               iMaxFieldXSize,
                               iButtonXSize,
                               iStaticYSize,
                               iYMargin,
                               iXMargin,
                               iComboBoxWidth,
                               iSpaceAfterCombo,
                               iMaxPromptSize,
                               out statCtl,
                               out fldCtl,
                               out comboCtl,
                               out iFieldYSize);
			    if (statCtl != null) {
			        listStat.Add(statCtl);
			    }
                if (comboCtl != null) {
                    listCombo.Add(comboCtl);
                }
                iYOldPos  = iYPos;
                if (bBetweenField) {
                    listBetweenField.Add(fldCtl);
                } else {
                    listField.Add(fldCtl);
                    iYPos    += iFieldYSize + iYMargin;
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFields:                 Create field controls in the specified container.
        /// </summary>
        /// <param name="ctl">                  Control in which to add the controls</param>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="arrFieldDef">          Array of field definition to use to create the controls</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        /// <param name="iComboBoxWidthInChr">  Width of combobox if option is selected</param>
        /// <param name="iSpaceAfterComboInChr">Space after combo in characters</param>
        /// <param name="eOption">              Generation option</param>
        /// <param name="iMaxXPos">             Returned the maximum right edge of a field</param>
        /// <param name="arrComboBox">          Array of combo box generated if option specified</param>
        /// <param name="arrStatCtl">           Array of created static fields</param>
        /// <param name="arrFieldCtl">          Array of created fields</param>
        /// <param name="arrBetweenFieldCtl">   Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        static public void CreateRowsetFields(System.Windows.Forms.Control   ctl,
                                              RepPP.RowsetDef                rowsetDef,
                                              ArrayList                      arrFieldDef,
                                              System.Drawing.Font            font, 
                                              int                            iMaxPromptSizeInChr,
                                              int                            iMaxFieldSizeInChr,
                                              int                            iComboBoxWidthInChr,
                                              int                            iSpaceAfterComboInChr,
                                              OptionE                        eOption,
                                              out int                        iMaxXPos,
                                              out ComboBox[]                 arrComboBox,
                                              out RepPP.RepPPStat[]          arrStatCtl,
                                              out RepPP.RepPPField[]         arrFieldCtl,
                                              out RepPP.RepPPField[]         arrBetweenFieldCtl)
        {
            ArrayList           listCombo;
            ArrayList           listStat;
            ArrayList           listField;
            ArrayList           listBetweenField;
            
            listCombo       = new ArrayList(rowsetDef.FieldDefs.Count);
            listStat        = new ArrayList(rowsetDef.FieldDefs.Count);
            listField       = new ArrayList(rowsetDef.FieldDefs.Count);
            listBetweenField= new ArrayList(rowsetDef.FieldDefs.Count);
            CreateRowsetFields(ctl,
                              rowsetDef,
                              arrFieldDef,
                              font, 
                              iMaxPromptSizeInChr,
                              iMaxFieldSizeInChr,
                              iComboBoxWidthInChr,
                              iSpaceAfterComboInChr,
                              eOption,
                              out iMaxXPos,
                              listCombo,
                              listStat,
                              listField,
                              listBetweenField);
            arrComboBox         = (ComboBox[])listCombo.ToArray(typeof(ComboBox));
            arrStatCtl          = (RepPP.RepPPStat[])listStat.ToArray(typeof(RepPP.RepPPStat));
            arrFieldCtl         = (RepPP.RepPPField[])listField.ToArray(typeof(RepPP.RepPPField));
            arrBetweenFieldCtl  = (RepPP.RepPPField[])listBetweenField.ToArray(typeof(RepPP.RepPPField));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFields:                 Create field controls in the specified container.
        /// </summary>
        /// <param name="ctl">                  Control in which to add the controls</param>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        /// <param name="iComboBoxWidthInChr">  Width of combobox if option is selected</param>
        /// <param name="iSpaceAfterComboInChr">Space after combo in characters</param>
        /// <param name="eOption">              Generation option</param>
        /// <param name="iMaxXPos">             Returned the maximum right edge of a field</param>
        /// <param name="arrComboBox">          Array of combo box generated if option specified</param>
        /// <param name="arrStatCtl">           Array of created static fields</param>
        /// <param name="arrFieldCtl">          Array of created fields</param>
        /// <param name="arrBetweenFieldCtl">   Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        static public void CreateRowsetFields(System.Windows.Forms.Control   ctl,
                                             RepPP.RowsetDef                rowsetDef,
                                             System.Drawing.Font            font, 
                                             int                            iMaxPromptSizeInChr,
                                             int                            iMaxFieldSizeInChr,
                                             int                            iComboBoxWidthInChr,
                                             int                            iSpaceAfterComboInChr,
                                             OptionE                        eOption,
                                             out int                        iMaxXPos,
                                             out ComboBox[]                 arrComboBox,
                                             out RepPP.RepPPStat[]          arrStatCtl,
                                             out RepPP.RepPPField[]         arrFieldCtl,
                                             out RepPP.RepPPField[]         arrBetweenFieldCtl)
        {
            bool        bBetweenField;
            ArrayList   arrFieldDef;
            
            arrFieldDef = new ArrayList(rowsetDef.FieldDefs.Count);
            foreach (RepPP.FieldDef fldDef in rowsetDef.FieldDefs) {
                bBetweenField = (fldDef.Name[0] == '_');
                if (fldDef.IsVisible || 
                    ((eOption & OptionE.OPT_ADD_BETWEEN_FIELD) != 0 && bBetweenField)) {
                    if (fldDef.IsDatabaseLinked ||
                        (eOption & OptionE.OPT_ONLY_MAIN_TBL_FIELD) == 0) {
                        if (!fldDef.IsUserFlagOn ||
                            (eOption & OptionE.OPT_NO_USER_FLAG_FIELD) == 0) {
                            arrFieldDef.Add(fldDef);
                        }
			        }
                }
            }
            CreateRowsetFields(ctl,
                              rowsetDef,
                              arrFieldDef,
                              font, 
                              iMaxPromptSizeInChr,
                              iMaxFieldSizeInChr,
                              iComboBoxWidthInChr,
                              iSpaceAfterComboInChr,
                              eOption,
                              out iMaxXPos,
                              out arrComboBox,
                              out arrStatCtl,
                              out arrFieldCtl,
                              out arrBetweenFieldCtl);

        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFieldsForDSearch:       Create field controls in the specified containers for direct search.
        /// </summary>
        /// <param name="ctlPanel1">            Panels containing the direct search field</param>
        /// <param name="fldDSearch">           Field definition use for direct search</param>
        /// <param name="ctlPanel2">            Panels containing all the other fields</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        /// <param name="iComboBoxWidthInChr">  Width of combobox if option is selected</param>
        /// <param name="iSpaceAfterComboInChr">Space after combo in characters</param>
        /// <param name="eOption">              Generation option</param>
        /// <param name="iMaxXPos">             Returned the maximum right edge of a field</param>
        /// <param name="arrComboBox">          Array of combo box generated if option specified</param>
        /// <param name="arrStatCtl">           Array of created static fields</param>
        /// <param name="arrFieldCtl">          Array of created fields</param>
        /// <param name="arrBetweenFieldCtl">   Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        static public void CreateRowsetFieldsForDSearch(System.Windows.Forms.Control   ctlPanel1,
                                                        RepPP.FieldDef                 fldDSearch,
                                                        System.Windows.Forms.Control   ctlPanel2,
                                                        System.Drawing.Font            font, 
                                                        int                            iMaxPromptSizeInChr,
                                                        int                            iMaxFieldSizeInChr,
                                                        int                            iComboBoxWidthInChr,
                                                        int                            iSpaceAfterComboInChr,
                                                        OptionE                        eOption,
                                                        out int                        iMaxXPos,
                                                        out ComboBox[]                 arrComboBox,
                                                        out RepPP.RepPPStat[]          arrStatCtl,
                                                        out RepPP.RepPPField[]         arrFieldCtl,
                                                        out RepPP.RepPPField[]         arrBetweenFieldCtl)
        {
            bool                bBetweenField;
            ArrayList           arrFieldDef;
            ArrayList           arrFieldDSearch;
            RepPP.FieldDef      fldDef;
            RepPP.RowsetDef     rowsetDef;
            int                 iFldCount;
            int                 iIndex;
            ArrayList           listCombo;
            ArrayList           listStat;
            ArrayList           listField;
            ArrayList           listBetweenField;
            
            rowsetDef           = fldDSearch.RowsetDef;
            iFldCount           = rowsetDef.FieldDefs.Count;
            listCombo           = new ArrayList(iFldCount);
            listStat            = new ArrayList(iFldCount);
            listField           = new ArrayList(iFldCount);
            listBetweenField    = new ArrayList(iFldCount);
            arrFieldDSearch     = new ArrayList(2);
            arrFieldDef         = new ArrayList(iFldCount * 2 - 2);
            iIndex              = 0;
            while (iIndex < iFldCount) {
                fldDef = rowsetDef.FieldDefs[iIndex];
                if (fldDef == fldDSearch) {
                    arrFieldDSearch.Add(fldDef);
                    if (iIndex + 1 < iFldCount) {
                        fldDef             = rowsetDef.FieldDefs[iIndex+1];
                        bBetweenField   = (fldDef.Name[0] == '_');
                        if ((eOption & OptionE.OPT_ADD_BETWEEN_FIELD) != 0 && bBetweenField) {
                            arrFieldDSearch.Add(fldDef);
                            iIndex++;
                        }
                    }
                } else {
                    bBetweenField = (fldDef.Name[0] == '_');
                    if (fldDef.IsVisible || 
                        ((eOption & OptionE.OPT_ADD_BETWEEN_FIELD) != 0 && bBetweenField)) {
                        if (fldDef.IsDatabaseLinked ||
                            (eOption & OptionE.OPT_ONLY_MAIN_TBL_FIELD) == 0) {
                            if (!fldDef.IsUserFlagOn ||
                                (eOption & OptionE.OPT_NO_USER_FLAG_FIELD) == 0) {
                                arrFieldDef.Add(fldDef);
                            }
			            }
                    }
                }
                iIndex++;    
            }
            CreateRowsetFields(ctlPanel1,
                               rowsetDef,
                               arrFieldDSearch,
                               font, 
                               iMaxPromptSizeInChr,
                               iMaxFieldSizeInChr,
                               iComboBoxWidthInChr,
                               iSpaceAfterComboInChr,
                               eOption,
                               out iMaxXPos,
                               listCombo,
                               listStat,
                               listField,
                               listBetweenField);
            CreateRowsetFields(ctlPanel2,
                               rowsetDef,
                               arrFieldDef,
                               font, 
                               iMaxPromptSizeInChr,
                               iMaxFieldSizeInChr,
                               iComboBoxWidthInChr,
                               iSpaceAfterComboInChr,
                               eOption,
                               out iMaxXPos,
                               listCombo,
                               listStat,
                               listField,
                               listBetweenField);

            arrComboBox         = (ComboBox[])listCombo.ToArray(typeof(ComboBox));
            arrStatCtl          = (RepPP.RepPPStat[])listStat.ToArray(typeof(RepPP.RepPPStat));
            arrFieldCtl         = (RepPP.RepPPField[])listField.ToArray(typeof(RepPP.RepPPField));
            arrBetweenFieldCtl  = (RepPP.RepPPField[])listBetweenField.ToArray(typeof(RepPP.RepPPField));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFields:                 Create field controls in the specified container.
        /// </summary>
        /// <param name="ctl">                  Control in which to add the controls</param>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="font">                 Font to use</param>
        /// <param name="iMaxPromptSizeInChr">  Maximum prompt size in characters</param>
        /// <param name="iMaxFieldSizeInChr">   Maximum field size in characters</param>
        //
        //*********************************************************	    
        static public void CreateRowsetFields(System.Windows.Forms.Control ctl, RepPP.RowsetDef rowsetDef, System.Drawing.Font font, int iMaxPromptSizeInChr, int iMaxFieldSizeInChr)
        {
            ComboBox[]          arrDummy;
            RepPP.RepPPStat[]   arrDummy2;
            RepPP.RepPPField[]  arrDummy3;
            RepPP.RepPPField[]  arrDummy4;
            int                 iDummy;
            
            CreateRowsetFields(ctl, rowsetDef, font, iMaxPromptSizeInChr, iMaxFieldSizeInChr, 0, 0, OptionE.OPT_NONE, out iDummy, out arrDummy, out arrDummy2, out arrDummy3, out arrDummy4);
        }
    }
}