using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //******************************************************************
    //
    /// <summary>
    /// Create a field combobox which supports images
    /// </summary>
    //
    //******************************************************************
    public class FldComboBoxImage : RepPP.Toolkit.Window.FldComboBox {
        /// <summary>Delegate for the SetItemInfo event</summary>
        public delegate void SetItemInfoHandler(object sender, SetItemInfoEventArgs e);
        /// <summary>Event sent to set the item info (label, bitmap, color, mark)</summary>
        public event SetItemInfoHandler SetItemInfo;
        /// <summary>ImageList contains all Displayable images</summary>
        private ImageList               m_imageList;
        /// <summary>Set to show only Description</summary>
        private bool                    m_bShowOnlyDescription;
        /// <summary>Background color to be able to support enabled/disabled states</summary>
        private Color                   m_colorBackground;
        /// <summary>Text color to be able to support enabled/disabled states</summary>
        private Color                   m_colorText;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                             Class constructor
        /// </summary>
        //
        //******************************************************************
        public FldComboBoxImage() {
            m_colorBackground      = BackColor;
            m_colorText            = ForeColor;
            m_imageList            = null;
            m_bShowOnlyDescription = false;
            this.DrawMode          = DrawMode.OwnerDrawFixed;
            this.DropDownStyle     = ComboBoxStyle.DropDownList;
        }

        //*********************************************************
        //
        /// <summary>
        /// ImageList :             Contains all Displayable images
        /// </summary>
        //
        //*********************************************************
        [Category("Behavior")]
        [DefaultValue("")]
        public ImageList ImageList {
            get {
                return (m_imageList);
            }
            set {
                m_imageList = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ShowOnlyDescription :   Show Only the description if
        ///                         set 
        /// </summary>
        //
        //*********************************************************
        [Category("Behavior")]
        [DefaultValue(false)]
        [Description("Only Description will be displayed if set.")]
        public bool ShowOnlyDescription {
            get {
                return (m_bShowOnlyDescription);
            }
            set {
                m_bShowOnlyDescription = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSetItemInfo:              Trigger the SetItemInfo event
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        protected virtual void OnSetItemInfo(SetItemInfoEventArgs e) {
            if (SetItemInfo != null) {
                SetItemInfo(this, e);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnDrawItem                  Occurs when customized drawing process
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //********************************************************* 
        protected override void OnDrawItem(DrawItemEventArgs e) {
            ImageComboItem  item;
            Image           image;
            Bitmap          bitmap;
            Color           forecolor;
            Font            font;
            bool            bHighLightSelected;
            int             iWidth;

            if (!m_bFilling) {
                bHighLightSelected = e.BackColor.Equals(Color.FromKnownColor(KnownColor.Highlight));
                this.BackColor = this.Enabled ? m_colorBackground : SystemColors.InactiveBorder;
                if (!this.Enabled) {
                    forecolor = SystemColors.InactiveCaption;
                } else if (bHighLightSelected) {
                    forecolor = m_colorBackground;
                } else {
                    forecolor = m_colorText;
                }
                e.DrawBackground();
                e.DrawFocusRectangle();
                iWidth = (this.ImageList != null) ? this.ImageList.ImageSize.Width : 0;
                if (e.Index < 0) {
                    e.Graphics.DrawString(this.Text, e.Font, new SolidBrush(forecolor), e.Bounds.Left + iWidth, e.Bounds.Top);
                } else {
                    if (this.Items[e.Index] is RepPP.Toolkit.Window.FldComboBoxImage.ImageComboItem) {
                        item = (RepPP.Toolkit.Window.FldComboBoxImage.ImageComboItem)this.Items[e.Index];
                        font = item.Mark ? new Font(e.Font, FontStyle.Bold) : e.Font;
                        if (!item.ForeColor.Equals(Color.FromKnownColor(KnownColor.Transparent)) && !bHighLightSelected && this.Enabled) {
                            forecolor = item.ForeColor;
                        }
                        if (item.ImageIndex != -1 && this.ImageList != null) {
                            if (this.Enabled) {
                                this.ImageList.Draw(e.Graphics, e.Bounds.Left, e.Bounds.Top, item.ImageIndex);
                            } else {
                                image = this.ImageList.Images[item.ImageIndex];
                                bitmap = ConvertImageToGreyScale(image);
                                e.Graphics.DrawImage(bitmap, e.Bounds.Left, e.Bounds.Top);
                            }
                        }
                        e.Graphics.DrawString(item.Text, font, new SolidBrush(forecolor), e.Bounds.Left + iWidth, e.Bounds.Top);
                    } else {
                        e.Graphics.DrawString(this.Items[e.Index].ToString(), e.Font, new SolidBrush(forecolor), e.Bounds.Left + iWidth, e.Bounds.Top);
                    }
                }
            }
            base.OnDrawItem(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// BuildListItemsv:        Build the combobox list items
        /// </summary>
        /// <param name="bForce">   Force the building of the list items</param>
        //
        //*********************************************************
        protected override void BuildListItems(bool bForce) {
            SetItemInfoEventArgs    itemInfoEventArgs;
            ImageComboItem          imageComboItem;
            RepPP.FieldDef          fldDef;
            RepPP.ChoiceList        lst;
            Graphics                gr;
            string                  strCode;
            string                  strDesc;
            string                  strValue;
            string                  strChoiceAns;
            int                     iIndex;
            int                     iMaxCodeWidth;
            int                     iSpaceWidth;
            int                     iCodeWidth;
            int                     iCount;
            int                     iIndexPos;

            if (bForce || this.Items.Count == 0) {
                fldDef = m_fldControlUtil.FieldDef;
                this.Items.Clear();
                if (fldDef != null && fldDef.IsChoiceAvailable == true) {
                    lst = fldDef.ChoiceList;
                    iCount = lst.Count;
                    gr = CreateGraphics();
                    iSpaceWidth = (int)gr.MeasureString(" ", Font).Width;
                    iMaxCodeWidth = GetMaxCodeWidth(lst) + iSpaceWidth * 4;
                    iIndex = 0;
                    iIndexPos = 0;
                    do {
                        iIndex = lst.ExtractChoice(iIndex, out strValue, out strCode, out strDesc);
                        if (iIndex != -1) {
                            imageComboItem = new ImageComboItem();
                            if (strValue.Equals(String.Empty)) {
                                imageComboItem.Value = strCode;
                            } else {
                                imageComboItem.Value = strValue;
                            }
                            if (strDesc != String.Empty && !ShowOnlyDescription) {
                                iCodeWidth = (int)gr.MeasureString(strCode, Font).Width;
                                strChoiceAns = strCode.PadRight(strCode.Length + (iMaxCodeWidth - iCodeWidth) / iSpaceWidth);
                                strChoiceAns = strChoiceAns + strDesc;
                            } else {
                                strChoiceAns = (strDesc != string.Empty && ShowOnlyDescription) ? strDesc : strCode;
                            }
                            imageComboItem.Text = strChoiceAns;
                            OnSetItemInfo(itemInfoEventArgs = new SetItemInfoEventArgs(imageComboItem, iIndexPos));
                            if (!itemInfoEventArgs.Cancel) {
                                this.Items.Add(imageComboItem);
                            }
                            iIndexPos++;
                        }
                    } while (iIndex != -1);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ConvertPictureToGreyScale :            Convertit le dessin original du 
        ///                                        tab Control en gris.
        /// </summary>
        //
        //********************************************************************************
        private Bitmap ConvertImageToGreyScale(Image image) {
            Bitmap  imgGreyScale = null;
            Color   cCurColor;
            Byte    byteAverage;

            if (image != null) {
                imgGreyScale = new Bitmap(image);
                for (int iX = 0; iX < imgGreyScale.Width; iX++) {
                    for (int iY = 0; iY < imgGreyScale.Height; iY++) {
                        cCurColor = imgGreyScale.GetPixel(iX, iY);
                        byteAverage = (byte)((cCurColor.R + cCurColor.G + cCurColor.B) / 3);
                        imgGreyScale.SetPixel(iX, iY, Color.FromArgb(byteAverage, byteAverage, byteAverage));
                    }
                }
                imgGreyScale.MakeTransparent(Color.Black);
            }
            return(imgGreyScale);
        }

        //*********************************************************	    
        /// <summary>
        /// Implement Event arguments for the SetItemInfo event.
        /// </summary>
        //*********************************************************	  
        public class ImageComboItem : ListItem {
            /// <summary>Item Image Index</summary>
            private int     m_iImageindex;
            /// <summary>Mark Text to bold</summary>
            private bool    m_bMark;
            /// <summary>Text ForeColor</summary>
            private Color   m_colForeColor;

            //******************************************************************
            //
            /// <summary>
            /// ctor:                             Class constructor
            /// </summary>
            //
            //******************************************************************
            public ImageComboItem() {
                m_iImageindex = -1;
                m_bMark = false;
                this.ForeColor = Color.FromKnownColor(KnownColor.Transparent);
            }

            //******************************************************************
            //
            /// <summary>
            /// ctor:                             Class constructor
            /// </summary>
            /// <param name="strText">            Item Text</param>
            //
            //******************************************************************
            public ImageComboItem(string strText)
                : this() {
                this.Text = strText;
            }

            //******************************************************************
            //
            /// <summary>
            /// ctor:                             Class constructor
            /// </summary>
            /// <param name="strText">            Item Text</param>
            /// <param name="iImageIndex">        Item Image Index</param>
            //
            //******************************************************************
            public ImageComboItem(string strText, int iImageIndex)
                : this(strText) {
                m_iImageindex = iImageIndex;
            }

            //******************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="strText">              Item Text</param>
            /// <param name="iImageIndex">          Item Image Index</param>
            /// <param name="bMark">                Mark Item to bold</param>
            //
            //******************************************************************
            public ImageComboItem(string strText, int iImageIndex, bool bMark)
                : this(strText, iImageIndex) {
                m_bMark = bMark;
            }

            //******************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="strText">              Item Text</param>
            /// <param name="iImageIndex">          Item Image Index</param>
            /// <param name="bMark">                Mark Item to bold</param>
            /// <param name="colForeColor">         Item Text ForeColor</param>
            //
            //******************************************************************
            public ImageComboItem(string strText, int iImageIndex, bool bMark, Color colForeColor)
                : this(strText, iImageIndex, bMark) {
                this.ForeColor = colForeColor;
            }

            //*********************************************************
            //
            /// <summary>
            /// ImageIndex :                        Item Image Index
            /// </summary>
            //
            //*********************************************************
            public int ImageIndex {
                get {
                    return m_iImageindex;
                }
                set {
                    m_iImageindex = value;
                }
            }

            //*********************************************************
            //
            /// <summary>
            /// Mark :                              Mark Item Text to
            ///                                     Bold
            /// </summary>
            //
            //*********************************************************
            public bool Mark {
                get {
                    return m_bMark;
                }
                set {
                    m_bMark = value;
                }
            }

            //*********************************************************
            //
            /// <summary>
            /// ForeColor :                         ForeColor Text 
            ///                                     to display
            /// </summary>
            //
            //*********************************************************
            public Color ForeColor {
                get {
                    return m_colForeColor;
                }
                set {
                    m_colForeColor = value;
                }
            }

            //*********************************************************
            //
            /// <summary>
            /// ToString :                          Convert Item to text
            /// </summary>
            //
            //*********************************************************
            public override string ToString() {
                return (this.Text);
            }

        } // End ImageComboItem class

        //*********************************************************	    
        /// <summary>
        /// Implement Event arguments for the SetItemInfo event.
        /// </summary>
        //*********************************************************	    
        public class SetItemInfoEventArgs : System.EventArgs {
            /// <summary>New Item</summary>
            private ImageComboItem m_item;
            /// <summary>ChoiceList index position</summary>
            private int m_iChoiceListIndexPos;
            /// <summary>Cancelling Add Item</summary>
            private bool m_bCancel;

            //******************************************************************
            //
            /// <summary>
            /// ctor:                             Class constructor
            /// </summary>
            /// <param name="item">               ImageComboBox item</param>
            /// <param name="iChoiceListIndexPos">ChoiceList index position</param>
            //
            //******************************************************************
            public SetItemInfoEventArgs(ImageComboItem item, int iChoiceListIndexPos) {
                m_item = item;
                m_bCancel = false;
                m_iChoiceListIndexPos = iChoiceListIndexPos;
            }

            //******************************************************************
            //
            /// <summary>
            /// Item:                       ImageComboBox item
            /// </summary>
            //
            //******************************************************************
            public ImageComboItem Item {
                get {
                    return (m_item);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// ChoiceListIndexPosition:    ChoiceList index Position
            /// </summary>
            //
            //******************************************************************
            public int ChoiceListIndexPosition {
                get {
                    return (m_iChoiceListIndexPos);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Cancel:                     Cancelling Add Item
            /// </summary>
            //
            //******************************************************************
            public bool Cancel {
                get {
                    return (m_bCancel);
                }
                set {
                    m_bCancel = value;
                }
            }

        } // End SetItemInfoEventArgs class
    } // End FldImageComboBox Class
} // End Namespace