using System;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;

namespace RepPP.Toolkit.Window {
    //*********************************************************
	/// <summary>
	/// Connect the error reporter object to the Rep++ Field
	/// </summary>
    //*********************************************************
	public class FldErrorProvider : ErrorProvider {
        /// <summary>Controls in error</summary>
	    private ArrayList           m_arrCtlInError;
        /// <summary>Error alignment</summary>
	    private ErrorIconAlignment  m_eAlignment;
        /// <summary>Icon padding</summary>
	    private int                 m_iIconPadding;

        //*********************************************************
        //
		/// <summary>
		/// ctor:                       Class constructor
		/// </summary>
		/// <param name="eAlignment">   Error alignment</param>
		/// <param name="iIconPadding">  Icon padding</param>
		//
        //*********************************************************
		public FldErrorProvider(ErrorIconAlignment eAlignment, int iIconPadding) : this() {
		    Init(eAlignment, iIconPadding);    
		}

        //*********************************************************
        //
		/// <summary>
		/// ctor:                       Class constructor
		/// </summary>
		//
        //*********************************************************
        public FldErrorProvider() : base() {
            Init(ErrorIconAlignment.MiddleLeft, 2);
        }

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="parentControl">The container of the control 
        ///                             to monitor for errors</param>
        //
        //*********************************************************
        public FldErrorProvider(ContainerControl parentControl) : base(parentControl) {
            Init(ErrorIconAlignment.MiddleLeft, 2);
        }

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="container">    The IContainer to monitor 
        ///                             for errors.</param>
        //
        //*********************************************************
        public FldErrorProvider(IContainer container) : base(container) {
            Init(ErrorIconAlignment.MiddleLeft, 2);
        }

        //*********************************************************
        //
        /// <summary>
        /// Init:                       Initializes the component
        /// </summary>
		/// <param name="eAlignment">   Error alignment</param>
		/// <param name="iIconPadding"> Icon padding</param>am>
        //
        //*********************************************************
        public void Init(ErrorIconAlignment eAlignment, int iIconPadding) {
            m_arrCtlInError = new ArrayList();
            m_eAlignment    = eAlignment;
		    m_iIconPadding  = iIconPadding;
        } 

        //*********************************************************
        //
		/// <summary>
		/// IconPadding:             Distance between the icon and the field
		/// </summary>
		//
        //*********************************************************
        [DefaultValue(2)]
        [Description("Padding between control and Error Icon")]
        [Category("Layout")]
		public int IconPadding {
		    get {
		        return(m_iIconPadding);
		    }
		    set {
		        m_iIconPadding = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// ErrorIconAlignment:     Alignment of the error icon
        /// </summary>
        //
        //*********************************************************
        [DefaultValue(ErrorIconAlignment.MiddleLeft)]
        [Description("Alignment where should appear the Error Icon")]
        [Category("Layout")]
		public ErrorIconAlignment ErrorIconAlignment {
		    get {
		        return(m_eAlignment);
		    }
		    set {
		        m_eAlignment = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// AddError:               Add an error text to a control
        /// </summary>
        /// <param name="ctl">      Control for which we add the error</param>
        /// <param name="strError"> Error text</param>
        //
        //*********************************************************
		public virtual void AddError(Control ctl, string strError) {
            SelectTabPage(ctl);
            SetError(ctl, strError);
		    SetIconAlignment(ctl, m_eAlignment);
		    SetIconPadding(ctl, m_iIconPadding);
		    m_arrCtlInError.Add(ctl);
		}

        //*********************************************************
        //
        /// <summary>
        /// SelectTabPage:          Selects the tab page containing the control
        /// </summary>
        /// <param name="ctl">      Control for which we add the error</param>
        //
        //*********************************************************
        private static void SelectTabPage(Control ctl) {
		    Control     ctlParent;
            TabControl  tabParent;

            ctlParent = ctl.Parent;
            while (ctlParent != null) {
                if (ctlParent is TabPage) {
                    tabParent = ctlParent.Parent as TabControl;
                    if (tabParent != null && tabParent.SelectedTab != ctlParent) {
                        tabParent.SelectedTab = (TabPage)ctlParent;
                    }
                    break;
                }
                ctlParent = ctlParent.Parent;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AddError:                       Add an error text to a field control attached to the specified field
        /// </summary>
        /// <param name="ctlContainer">     Form or other container containing the field control</param>
        /// <param name="fldDef">           FieldDef</param>
        /// <param name="strError">         Error text</param>
        //
        //*********************************************************
		public virtual void AddError(Control ctlContainer, RepPP.FieldDef fldDef, string strError) {
		    foreach (Control ctl in ctlContainer.Controls) {
                if (IsControlAttachedToFieldB(ctl, fldDef)) {
                    if (!ctl.Visible && ctl.Tag is Control) {
                        AddError((Control)(ctl.Tag), strError);
                    } else {
		                AddError(ctl, strError);
                    }
		        }
		        AddError(ctl, fldDef, strError);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// IsControlAttachedToFieldB:  Check if the control is a rep++ field or a toolit 
        ///                             rep++ field and is attached to the field
        /// </summary>
        /// <param name="ctl">          The control</param>
        /// <param name="fldDef">       The field definition</param>
        /// <returns>
        /// True:                       The control is attached to the field
        /// Flase:                      The control is not attached to the field
        /// </returns>
        //
        //*********************************************************
        protected virtual bool IsControlAttachedToFieldB(Control ctl, RepPP.FieldDef fldDef) {
            bool bRetVal = false;

            if (ctl is RepPP.RepPPField) {
                bRetVal = ((ctl as RepPP.RepPPField).FieldDef == fldDef);
            } else if (ctl is FldCheckBox) {
                bRetVal = ((ctl as FldCheckBox).FieldDef == fldDef);
            } else if (ctl is FldComboBox) {
                bRetVal = ((ctl as FldComboBox).FieldDef == fldDef);
            } else if (ctl is FldDateTimePicker) {
                bRetVal = ((ctl as FldDateTimePicker).FieldDef == fldDef);
            } else if (ctl is FldListBox) {
                bRetVal = ((ctl as FldListBox).FieldDef == fldDef);
            } else if (ctl is FldMonthCalendar) {
                bRetVal = ((ctl as FldMonthCalendar).FieldDef == fldDef);
            } else if (ctl is FldTextBox) {
                bRetVal = ((ctl as FldTextBox).FieldDef == fldDef);
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// ResetError:                     Reset all the error shown using AddError
        /// </summary>
        //
        //*********************************************************
		public virtual void Reset() {
		    foreach (Control ctl in m_arrCtlInError) {
		        SetError(ctl, "");
		    }
		    m_arrCtlInError.Clear();
		}
	}//End class FldErrorProvider
}//End namespace
