using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using RepPP.Toolkit;

namespace RepPP.Toolkit.Window {
    /// <summary>Implement Field Fill Indicator associated to control</summary>
    public partial class IndicatorFieldFilled : Component, FldControlUtil.IFldControl {
        /// <summary>FieldDef control utility class</summary>
        protected FldControlUtil    m_fldControlUtil;
        /// <summary>Control binded to indicator</summary>
        private Control             m_ctrlBinded;
        /// <summary>Original label from control</summary>
        private String              m_strOriginalLabel;
        /// <summary>Event before Label is modified</summary>
        [Category("Behavior")]
        [Description("Occurs before control label is modified")]
        public event BeforeCtrlLabelModifiedEventHandler BeforeCtrlLabelModified;
        /// <summary>
        /// BeforeCtrlLabelModified :       Occurs before control label is modified
        /// </summary>
        /// <param name="sender">              Sender object</param>
        /// <param name="e">                   Event arguments</param>
        public delegate void BeforeCtrlLabelModifiedEventHandler(object sender, BeforeCtrlModifiedEventArgs e);

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //
        //******************************************************************
        public IndicatorFieldFilled() {
            InitializeComponent();
            Init();
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="container">            Component container</param>
        //
        //******************************************************************
        public IndicatorFieldFilled(IContainer container) {
            container.Add(this);

            InitializeComponent();
            Init();
        }

        //********************************************************************************
        //
        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="bDisposing">true if managed resources should be disposed; 
        ///                          otherwise, false.</param>
        //
        //********************************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing && (components != null)) {
                if (m_fldControlUtil != null) {
                    m_fldControlUtil.Application.FldChange -= new RepPP.FldChangeEventHandler(appFldChange);
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
                components.Dispose();
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Init:              Initialize the control utility
        /// </summary>
        //
        //********************************************************************************
        private void Init() {
            m_fldControlUtil                        = new FldControlUtil(this, new WndControlUtilHelper(), false);
            m_fldControlUtil.Application.FldChange += new RepPP.FldChangeEventHandler(appFldChange);
            SetIndicator();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
        public FldFullName FieldFullName {
            get {
                return(m_fldControlUtil.FieldFullName);
            }
            set {
                m_fldControlUtil.FieldFullName = value;
                SetIndicator();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindedControl:              Control binded to indicator
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [DefaultValue(null)]
        [Category("Data")]
        [Description("Control binded to indicator Field Filled")]
        public Control BindedControl {
            get {
                return(m_ctrlBinded);
            }
            set {
                m_ctrlBinded = value;
                SetIndicator();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public RepPP.Application Application {
            get {
                return(m_fldControlUtil.Application);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public RepPP.FieldDef Field {
            get {
                return(m_fldControlUtil.FieldDef);
            }
            set {
                m_fldControlUtil.FieldDef = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// appFldChange : Occurs when field value change
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        private void appFldChange(object sender, RepPP.FieldGenEventArgs e) {
            if (m_fldControlUtil != null) {
                if (m_fldControlUtil.FieldDef != null && m_fldControlUtil.FieldDef == e.FieldDef) {
                    SetIndicator();
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetIndicator:               Set Indicator state to control
        /// </summary>
        //
        //********************************************************************************
        private void SetIndicator() {
            BeforeCtrlModifiedEventArgs beforeCtrlLblModEventArgs;
            RepPP.FieldDef              fld;
            String                      strNewLabel;
            Boolean                     bIsFilled;

            if (!DesignMode && m_ctrlBinded != null && m_ctrlBinded.Text != string.Empty) {
                if (String.IsNullOrEmpty(m_strOriginalLabel)) {
                    m_strOriginalLabel = m_ctrlBinded.Text;
                }
                if (m_fldControlUtil != null) {
                    fld = m_fldControlUtil.FieldDef;
                    if (fld != null && fld.RowsetDef.ActiveRowset != null) {
                        bIsFilled = !fld.RowsetDef.ActiveRowset.Fields[fld.Name].IsNull();
                        strNewLabel = (bIsFilled) ? String.Format("{0}...", m_strOriginalLabel) : m_strOriginalLabel;
                        beforeCtrlLblModEventArgs = new BeforeCtrlModifiedEventArgs(m_ctrlBinded, bIsFilled, m_strOriginalLabel, strNewLabel);
                        if (BeforeCtrlLabelModified != null) {
                            BeforeCtrlLabelModified(this, beforeCtrlLblModEventArgs);
                        }
                        if (!beforeCtrlLblModEventArgs.Cancel) {
                            m_ctrlBinded.Text = beforeCtrlLblModEventArgs.NewLabel;
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RepPPReInit:        Called when Rep++ is reinitialized in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReInit() {
            //Do nothing, I'm a component !
        }

        //********************************************************************************
        //
        /// <summary>
        /// RepPPReset:         Called when Rep++ is reset in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReset() {
            //Do nothing, I'm a component !
        }

        //********************************************************************************
        //
        /// <summary>
        /// Before the Control is modified Event Args
        /// </summary>
        //
        //********************************************************************************    
        public class BeforeCtrlModifiedEventArgs : CancelEventArgs {
            /// <summary>Control label to be modified</summary>
            Control     m_ctrl;
            /// <summary>Original label</summary>
            String      m_strOriginalLabel;
            /// <summary>New label</summary>
            String      m_strNewLabel;
            /// <summary>Indicate if Field is filled</summary>
            Boolean     m_bIsFilled;

            //******************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            //
            //******************************************************************
            public BeforeCtrlModifiedEventArgs(Control ctrl, Boolean bIsFilled, String strOriginalLabel, String strNewLabel) {
                m_ctrl             = ctrl;
                m_bIsFilled        = bIsFilled;
                m_strOriginalLabel = strOriginalLabel;
                m_strNewLabel      = strNewLabel;
            }

            //******************************************************************
            //
            /// <summary>
            /// Control label to be modified
            /// </summary>
            //
            //******************************************************************
            public Control ControlModified {
                get {
                    return(m_ctrl);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Indicate if Field is Filled
            /// </summary>
            //
            //******************************************************************
            public Boolean IsFilled {
                get {
                    return(m_bIsFilled);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Original label from Control
            /// </summary>
            //
            //******************************************************************
            public String OriginalLabel {
                get {
                    return(m_strOriginalLabel);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// New label to be appliqued to control
            /// </summary>
            //
            //******************************************************************
            public String NewLabel {
                get {
                    return(m_strNewLabel);
                }
                set {
                    m_strNewLabel = value;
                }
            }

        }// End class BeforeCtrlFldModifiedEventArgs
    }//End class IndicatorFieldFilled      
}//End namespace
