using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// ListViewBase: Create a list view around a group
	/// </summary>
	[ToolboxItem(false)]
	public class ListViewBase : ListViewUIExt {
	    /// <summary>
	    /// Method use to size a column
	    /// </summary>
	    public enum SizeMethodE {
	        /// <summary>
	        /// Use the maximum of the prompt size and value size
	        /// </summary>
	        Auto            = 0,
	        /// <summary>
	        /// Use the prompt size
	        /// </summary>
	        PromptSize      = 1,
	        /// <summary>
	        /// Use the field maximum size
	        /// </summary>
	        MaxValueSize    = 2,
	        /// <summary>
	        /// Use a fix size
	        /// </summary>
	        FixSize         = 3
	    };
        /// <summary>
        /// List of fields that override the UseDescription property 
        /// </summary>
        public static Dictionary<string, bool> UseDescriptionOverride = new Dictionary<string, bool>();
        /// <summary>
        /// Delegate for the SetItemInfo event
        /// </summary>
        public  delegate void               SetItemInfoHandler(object sender, SetItemInfoEventArgs e);
        /// <summary>
        /// Event sent to set the item info (bitmap, tooltip...)
        /// </summary>
        public  event SetItemInfoHandler    SetItemInfo;
        /// <summary>
        /// Rep++ rowset definition attached to the listview
        /// </summary>
        protected RepPP.RowsetDef           m_rowsetDef;
        /// <summary>
        /// Method uses to size the columns
        /// </summary>
        protected SizeMethodE               m_eSizeMethod;
        /// <summary>
        /// Minimum column size in characters
        /// </summary>
        protected int                       m_iMinColSize;
        /// <summary>
        /// Maximum column size in characters
        /// </summary>
        protected int                       m_iMaxColSize;
        /// <summary>
        /// Creates column for field with user flag on if true
        /// </summary>
        protected bool                      m_bShowUserFlagFld;
        /// <summary>
        /// Button use to move up a line in the listview
        /// </summary>
        protected Button                    m_butUp;
        /// <summary>
        /// Button use to move down a line in the listview
        /// </summary>
        protected Button                    m_butDown;
        /// <summary>
        /// Allow drag and drop in the same listview
        /// </summary>
        protected bool                      m_bMoveAllowed;
        /// <summary>
        /// Handler for the button UP click event
        /// </summary>
        protected EventHandler              m_delOnButUp;
        /// <summary>
        /// Handler for the button DOWN click event
        /// </summary>
        protected EventHandler              m_delOnButDown;         // Event handler for button Down click
        /// <summary>
        /// Indicates if the desciption for a choice field is displayed in the listview
        /// </summary>
        protected bool                      m_bUseDescription;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
        public ListViewBase() {
            m_rowsetDef         = null;
            m_eSizeMethod       = SizeMethodE.Auto;
            m_iMinColSize       = 5;
            m_iMaxColSize       = 40;
            m_bShowUserFlagFld  = true;
            m_bUseDescription   = true;
            m_bMoveAllowed      = false;
            m_butUp             = null;
            m_butDown           = null;
            m_delOnButUp        = null;
            m_delOnButDown      = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose:                    Disposing the object
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC</param>
        //
        //*********************************************************
        protected override void Dispose(bool bDisposing)
        {
            if (bDisposing) {
                if (m_delOnButUp != null && m_butUp != null) {
                    m_butUp.Click -= m_delOnButUp;
                    m_delOnButUp   = null;
                }
                if (m_delOnButDown != null && m_butDown != null) {
                    m_butDown.Click -= m_delOnButDown;
                    m_delOnButDown   = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsUserFlagFieldShown:       Determine if the field with user flag on must
        ///                             be included in the list.
        /// </summary>
        //
        //*********************************************************
		[Description("Determines if the field with user flag on are to be included in the list."), Category("Behavior")]
        public bool IsUserFlagFieldShown {
            get {
                return(m_bShowUserFlagFld);
            }
            set {
                m_bShowUserFlagFld = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// UseDescription:         Indicates if the desciption for 
        ///                         a choice field is displayed in 
        ///                         the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Indicates if the desciption for a choice field is displayed in the listview."), Category("Behavior")]
        public bool UseDescription {
            get {
                return(m_bUseDescription);
            }
            set {
                m_bUseDescription = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetUpDownButtonStatev:  Set the button state
        /// </summary>
        //
        //*********************************************************
        private void SetUpDownButtonStatev()
        {            
            if (m_butUp != null) {
                m_butUp.Enabled = (SelectedItems.Count != 0 && SelectedItems[0].Index != 0);
            }
            if (m_butDown != null) {
                m_butDown.Enabled = (SelectedItems.Count != 0 && SelectedItems[0].Index != (Items.Count - 1));
            }                
        }

        //*********************************************************
        //
        /// <summary>
        /// UpButton:                   Button to move UP the selected item
        /// </summary>
        //
        //*********************************************************
		[Description("Button use to move up an item in the listview."), Browsable(false) ]
        public Button UpButton {
            get {
                return(m_butUp);
            }
            set {
                if (m_butUp != value) {
                    if (m_delOnButUp != null && m_butUp != null) {
                        m_butUp.Click -= m_delOnButUp;
                        m_delOnButUp   = null;
                    }
                    m_butUp = value;
                    if (m_butUp != null) {
                        m_delOnButUp   = new EventHandler(OnButUpClick);
                        m_butUp.Click += m_delOnButUp;
                        SetUpDownButtonStatev();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DownButton:                 Button to move DOWN the selected item
        /// </summary>
        //
        //*********************************************************
		[Description("Button use to move down an item in the listview."), Browsable(false) ]
        public Button DownButton {
            get {
                return(m_butDown);
            }
            set {
                if (m_butDown != value) {
                    if (m_delOnButDown != null && m_butDown != null) {
                        m_butDown.Click -= m_delOnButDown;
                        m_delOnButDown   = null;
                    }
                    m_butDown = value;
                    if (m_butDown != null) {
                        m_delOnButDown   = new EventHandler(OnButDownClick);
                        m_butDown.Click += m_delOnButDown;
                        SetUpDownButtonStatev();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoMoveAllowed:            true to allow automatic move through
        ///                             drag and drop
        /// </summary>
        //
        //*********************************************************
		[Description("Determine if the user can move a line by drag and drop."), Category("Behavior") ]
        public bool AutoMoveAllowed {
            get {
                return(m_bMoveAllowed);
            }
            set {
                m_bMoveAllowed = value;
                AllowDrop      = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SizeMethod:                 Method use to define the column size
        /// </summary>
        //
        //*********************************************************
		[Description("Determine the method use to auto-size the column."), Category("Behavior") ]
        public SizeMethodE SizeMethod {
            get {
                return(m_eSizeMethod);
            }
            set {
                m_eSizeMethod = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// MinColumnSize:              Minimum size of a column in characters
        /// </summary>
        //
        //*********************************************************
		[Description("Minimum column size for auto-size."), Category("Behavior") ]
        public int MinColumnSize {
            get {
                return(m_iMinColSize);
            }
            set {
                m_iMinColSize = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// MaxColumnSize:              Maximum size of a column in characters
        /// </summary>
        //
        //*********************************************************
		[Description("Maximum column size for auto-size."), Category("Behavior") ]
        public int MaxColumnSize {
            get {
                return(m_iMaxColSize);
            }
            set {
                m_iMaxColSize = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// IsFieldShown:           
        /// </summary>
        //
        //*********************************************************
        protected bool IsFieldShown(RepPP.FieldDef fldDef) {
            bool    bRetVal;
            
            bRetVal = fldDef.IsVisible && (!fldDef.IsUserFlagOn || m_bShowUserFlagFld);
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// InitColumn:                 Build the list of columns
        /// </summary>
        //
        //*********************************************************
        private void InitColumn() {
			int                     iNbCol = 0;
			System.Drawing.Graphics gr;
			int                     iFldSize;
			int                     iPromptSize;
			int                     iSize;
			string                  str;
            ColumnHeader            col;

            Columns.Clear();
            if (m_rowsetDef != null) {
                gr = CreateGraphics();
			    foreach(RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
			        if (IsFieldShown(fldDef)) {
			            iFldSize    = fldDef.MaxSize;
			            iPromptSize = fldDef.Prompt.Length;
			            switch(m_eSizeMethod) {
			            case SizeMethodE.FixSize:
			                iSize = m_iMaxColSize;
			                break;
			            case SizeMethodE.MaxValueSize:
			                iSize = iFldSize;
			                break;
			            case SizeMethodE.PromptSize:
			                iSize = iPromptSize;
			                break;
			            default:
                            iSize = (iFldSize > iPromptSize) ? iFldSize : iPromptSize;
                            break;
			            }
			            if (iSize < m_iMinColSize) {
			                iSize = m_iMinColSize;
			            }
			            if (iSize > m_iMaxColSize) {
			                iSize = m_iMaxColSize;
			            }
			            str         = "N".PadRight(iSize + 1, 'N');
                        iSize       = (int)gr.MeasureString(str, Font).Width;
				        col         = Columns.Add(fldDef.Prompt, iSize, (fldDef.Type == RepPP.FieldType.sdFieldMoney) ?
				                                                     HorizontalAlignment.Right : HorizontalAlignment.Left);
                        col.Name    = fldDef.Name;

				        iNbCol++;
				    }
			    }
			    gr.Dispose();
			}
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetDef:                      Rowset definition assign to the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Rowset definition associates to the listview."), Browsable(false) ]
        public virtual RepPP.RowsetDef RowsetDef {
            get {
                return(m_rowsetDef);
            }
            set {
                if (m_rowsetDef != value) {
                    m_rowsetDef   = value;
                    InitColumn();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSetItemInfo:              Trigger the SetItemInfo event
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        protected virtual void OnSetItemInfo(SetItemInfoEventArgs e) {
            if (SetItemInfo != null) {
                SetItemInfo(this, e);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateListViewItem:         Creates a new listview item. Can be override to creates derived class
        /// </summary>
        /// <param name="iLineIndex">   Index of the group instance line index</param>
        /// <returns>
        /// New listview item
        /// </returns>
        //
        //*********************************************************
        protected virtual ListViewItem CreateListViewItem(int iLineIndex) {
            return(new ListViewItem());
        }

        //*********************************************************
        //
        /// <summary>
        /// FillUnknownField:           Called when a field is not found in the source
        /// </summary>
        /// <param name="fldDef">          FieldDef not found in group instance</param>
        /// <param name="rowset">        Rowset use to fill the values</param>
        /// <returns>
        /// Return the field value
        /// </returns>
        //
        //*********************************************************
        protected virtual string FillUnknownField(RepPP.FieldDef fldDef, RepPP.Rowset rowset) {
            return("");
        }

        //*********************************************************
        //
        /// <summary>
        /// GetFieldValue:              Get the field value
        /// </summary>
        /// <param name="rowset">        Rowset</param>
        /// <param name="fldDef">       Field definition to use</param>
        /// <param name="iLineIndex">   Line index</param>
        /// <returns>
        /// String value
        /// </returns>
        //
        //*********************************************************
        private string GetFieldValue(RepPP.Rowset rowset, RepPP.FieldDef fldDef, int iLineIndex) {
            string      strRetVal;
            int         iChoiceIndex;
            bool        bUseDesc;
            string      strName;

            bUseDesc = m_bUseDescription;
            if (UseDescriptionOverride.Count > 0) {
                strName = fldDef.Name; 
                if (UseDescriptionOverride.ContainsKey(strName)) {
                    bUseDesc = UseDescriptionOverride[strName];
                } else {
                    strName = fldDef.RowsetDef.Name + "." + strName; 
                    if (UseDescriptionOverride.ContainsKey(strName)) {
                        bUseDesc = UseDescriptionOverride[strName];
                    }
                }
            }
            if (bUseDesc && fldDef.IsChoiceAvailable) {
                iChoiceIndex = fldDef.ChoiceList.FindValue(rowset.GetFieldValue(fldDef, iLineIndex, false));
                if (iChoiceIndex == -1) {
                    strRetVal = rowset.GetFieldValue(fldDef, iLineIndex, true);
                } else {
                    strRetVal = fldDef.ChoiceList.GetDesc(iChoiceIndex);
                    if (strRetVal.Length == 0) {
                        strRetVal = rowset.GetFieldValue(fldDef, iLineIndex, true);
                    }
                }
            } else {
                strRetVal = rowset.GetFieldValue(fldDef, iLineIndex, true);
            }
            return(strRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateFldArray:                 Creates a field array
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="arrFldDef">        Array of field definitions to use</param>
        /// <param name="arrMatchingFld">   Array of mathching fields to use</param>
        //
        //*********************************************************
        private void CreateFldArray(RepPP.Rowset rowset, out RepPP.FieldDef[] arrFldDef, out RepPP.FieldDef[] arrMatchingFld) {
            ArrayList       arrListFld;
            ArrayList       arrListMatchingFld;
            RepPP.FieldDef  fldMatch;

            arrListFld = new ArrayList(m_rowsetDef.FieldDefs.Count);
            if (rowset.RowsetDef == m_rowsetDef) {
                arrListMatchingFld  = null;
                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                    if (IsFieldShown(fldDef)) {
                        arrListFld.Add(fldDef);
                    }
                }
            } else {
                arrListMatchingFld  = new ArrayList(m_rowsetDef.FieldDefs.Count);
                foreach (RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                    if (IsFieldShown(fldDef)) {
                        arrListFld.Add(fldDef);
                        fldMatch = rowset.RowsetDef.FieldDefs[fldDef.Name];
                        arrListMatchingFld.Add(fldMatch);
                    }
                }
            }
            arrFldDef = new RepPP.FieldDef[arrListFld.Count];
            arrListFld.CopyTo(arrFldDef);
            if (arrListMatchingFld == null) {
                arrMatchingFld = null;
            } else {
                arrMatchingFld = new RepPP.FieldDef[arrListMatchingFld.Count];
                arrListMatchingFld.CopyTo(arrMatchingFld);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateItemFromRowset:           Creates an item from the value of the current line of a rowset
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="iLineIndex">       Line index</param>
        /// <param name="arrFldDef">        Array of fields to use</param>
        /// <param name="arrMatchingFld">   Array of mathching fields to use</param>
        /// <returns>
        /// Return the new list view item
        /// </returns>
        //
        //*********************************************************
        private ListViewItem CreateItemFromRowset(RepPP.Rowset rowset, int iLineIndex, RepPP.FieldDef[] arrFldDef, RepPP.FieldDef[] arrMatchingFld) {
            ListViewItem                    itemRetVal;
            string                          strVal;
            int                             iCount;
            RepPP.FieldDef                  fldMatch;
            ListViewItem.ListViewSubItem    lvSubItem;
            string                          strName;
            
            itemRetVal = CreateListViewItem(iLineIndex);
            iCount     = 0;
            if (arrMatchingFld == null) {
                foreach (RepPP.FieldDef fldDef in arrFldDef) {
                    strVal   = GetFieldValue(rowset, fldDef, iLineIndex);
                    if (iCount == 0) {
                        itemRetVal.SubItems[0].Text = strVal;
                        itemRetVal.SubItems[0].Name = fldDef.Name;
                    } else {
                        lvSubItem       = itemRetVal.SubItems.Add(strVal);
                        lvSubItem.Name  = fldDef.Name;
                    }
                    iCount++;
                }
            } else {
                foreach (RepPP.FieldDef fldDef in arrFldDef) {
                    fldMatch = arrMatchingFld[iCount];
                    if (fldMatch == null) {
                        strVal  = FillUnknownField(fldDef, rowset);
                        strName = fldDef.Name; 
                    } else {
                        strVal  = GetFieldValue(rowset, fldMatch, iLineIndex);
                        strName = fldMatch.Name;
                    }
                    if (iCount == 0) {
                        itemRetVal.SubItems[0].Text = strVal;
                        itemRetVal.SubItems[0].Name = strName;
                    } else {
                        lvSubItem       = itemRetVal.SubItems.Add(strVal);
                        lvSubItem.Name  = strName;
                    }
                    iCount++;
                }
            }
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateItemFromRowset:           Creates an item from the value of the current line of a rowset
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="iLineIndex">       Line index</param>
        /// <returns>
        /// Return the new list view item
        /// </returns>
        //
        //*********************************************************
        private ListViewItem CreateItemFromRowset(RepPP.Rowset rowset, int iLineIndex) {
            RepPP.FieldDef[]   arrFldDef;
            RepPP.FieldDef[]   arrMatchingFld;

            CreateFldArray(rowset, out arrFldDef, out arrMatchingFld);
            return(CreateItemFromRowset(rowset, iLineIndex, arrFldDef, arrMatchingFld));
        }

        //*********************************************************
        //
        /// <summary>
        /// InternalFill:                   Fill the listview with the value of the rowset
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="bDispDelLine">     True to display deleted lines in the listview</param>
        //
        //*********************************************************
        protected void InternalFill(RepPP.Rowset rowset, bool bDispDelLine) {
            int                 iLineCount;
            int                 iLineIndex;
            int                 iIndex;
            RepPP.FieldDef[]    arrFldDef;
            RepPP.FieldDef[]    arrMatchingFld;
            ListViewItem[]      listViewItems;

            CreateFldArray(rowset, out arrFldDef, out arrMatchingFld);
            BeginUpdate();
            Items.Clear();
            if (rowset != null) {
                if (rowset.RowsetDef != m_rowsetDef) {
                    throw new System.ApplicationException("Rowset must be associated with the listview group");
                }
                iLineCount    = rowset.LineCount;
                listViewItems = new ListViewItem[bDispDelLine ? iLineCount : rowset.UndelLineCount];
                iIndex        = 0;
                for (iLineIndex = 0; iLineIndex < iLineCount; iLineIndex++) {
                    if (bDispDelLine || (rowset.ThisLineState(iLineIndex) != RepPP.LineState.sdLineDeleted)) {
                        listViewItems[iIndex++] = CreateItemFromRowset(rowset, iLineIndex , arrFldDef, arrMatchingFld);
                    }
                }
            } else {
                listViewItems = new ListViewItem[0];
            }
            Items.AddRange(listViewItems);
            foreach (ListViewItem item in listViewItems) {
                OnSetItemInfo(new SetItemInfoEventArgs(item, true));
            }
            EndUpdate();
        }

        //*********************************************************
        //
        /// <summary>
        /// InsertItem:             Insert an item in the list
        /// </summary>
        /// <param name="iIndex">   Index where to add it in the list. -1 to append</param>
        /// <param name="rowset">   Rowset containing the value. Can be from a different group.</param>
        /// <returns>
        /// New item inserted
        /// </returns>
        //
        //*********************************************************
        protected ListViewItem InternalInsertItem(int iIndex, RepPP.Rowset rowset) {
            ListViewItem    itemRetVal;

            if (rowset == null) {
                throw new System.ApplicationException("Bad parameter");
            }
            if (m_rowsetDef == null) {
                throw new System.ApplicationException("RowsetDef not initialized");
            }
            if (rowset.Type == RepPP.RowsetType.sdHorizontal && rowset.LineCount == 0) {
                throw new System.ApplicationException("Rowset empty");
            }
            itemRetVal = CreateItemFromRowset(rowset, rowset.CurrentLine);
            if (iIndex == -1) {
                Items.Add(itemRetVal);
            } else {
                Items.Insert(iIndex, itemRetVal);
            }
            OnSetItemInfo(new SetItemInfoEventArgs(itemRetVal, true));
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// UpdateItem:             Update an item in the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to update</param>
        /// <param name="rowset">   Rowset containing the value. Can be from a different rowset definition.</param>
        //
        //*********************************************************
        protected void InternalUpdateItem(int iIndex, RepPP.Rowset rowset) {
            ListViewItem        item;
            ListViewLineItem    itemTmp;
            string              strVal;
            int                 iLineIndex;
            int                 iCount;
            RepPP.FieldDef      fldMatch;
            RepPP.FieldDef[]    arrFldDef;
            RepPP.FieldDef[]    arrMatchingFld;

            if (rowset == null) {
                throw new System.ApplicationException("Bad parameter");
            }
            if (m_rowsetDef == null) {
                throw new System.ApplicationException("RowsetDef not initialized");
            }
            if (rowset.Type == RepPP.RowsetType.sdHorizontal && rowset.LineCount == 0) {
                throw new System.ApplicationException("Rowset empty");
            }
            if (iIndex < 0 || iIndex >= Items.Count) {
                throw new System.ApplicationException("Bad index");
            }
            iLineIndex  = rowset.CurrentLine;
            item        = Items[iIndex];
            itemTmp     = item as ListViewLineItem;
            if (itemTmp != null) {
                itemTmp.LineBookmark = rowset.LineBookmark(rowset.CurrentLine);
            }
            CreateFldArray(rowset, out arrFldDef, out arrMatchingFld);
            iCount  = 0;
            foreach (RepPP.FieldDef fldDef in arrFldDef) {
                if (arrMatchingFld == null) {
                    strVal = GetFieldValue(rowset, fldDef, iLineIndex);
                } else {
                    fldMatch = arrMatchingFld[iCount];
                    strVal   = (fldMatch == null) ? FillUnknownField(fldDef, rowset) : GetFieldValue(rowset, fldMatch, iLineIndex);
                }
                item.SubItems[iCount].Text = strVal;
                iCount++;
            }
            OnSetItemInfo(new SetItemInfoEventArgs(item, false));
        }

        //*********************************************************
        //
        /// <summary>
        /// InternalDeleteItem:     Delete an item from the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to update</param>
        //
        //*********************************************************
        protected void InternalDeleteItem(int iIndex) {
            Items.RemoveAt(iIndex);
        }

        //******************************************************************
        //
        /// <summary>
        /// MoveItem:                   Move an item after the specified position
        /// </summary>
        /// <param name="item">         Item to move</param>
        /// <param name="itemAfter">    Item after which to move the item or null for first position</param>
        //
        //******************************************************************
        public virtual void MoveItem(ListViewItem item, ListViewItem itemAfter)
        {
            if (item != itemAfter) {
                Items.Remove(item);
                Items.Insert((itemAfter == null) ? 0 : itemAfter.Index + 1, item);
                SetUpDownButtonStatev();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnItemDrag:         Called when a drag is about to begin
        /// </summary>
        /// <param name="e">    Drag parameter</param>
        //
        //******************************************************************
        protected override void OnItemDrag(ItemDragEventArgs e)
        {
            DataObject  dataObject;
            
            base.OnItemDrag(e);
            if (m_bMoveAllowed) {
                if (e.Button == MouseButtons.Left) {
                    dataObject = new DataObject();
                    dataObject.SetData(typeof(ListViewItem), e.Item);
                    DoDragDrop(dataObject, DragDropEffects.Move);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragOver:         Called when the object is dragged over
        /// </summary>
        /// <param name="e">    Drag parameter</param>
        //
        //******************************************************************
        protected override void OnDragOver(DragEventArgs e)
        {
            ListViewItem    item;
            object          obj;
            
            base.OnDragOver(e);
            if (m_bMoveAllowed) {
                obj  = e.Data.GetData(typeof(ListViewItem));
                item = obj as ListViewItem;
                if (item == null || item.ListView != this) {
                    e.Effect = DragDropEffects.None;
                } else {
                    e.Effect = DragDropEffects.Move;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragDrop:         Called when the drag object is dropped
        /// </summary>
        /// <param name="e">    Drag parameter</param>
        //
        //******************************************************************
        protected override void OnDragDrop(DragEventArgs e)
        {
            ListViewItem    itemToMove;
            ListViewItem    itemPos;
            ListViewItem    itemFirst;
            object          obj;
            Point           pt;
            Rectangle       rect;
            
            base.OnDragDrop(e);
            if (m_bMoveAllowed) {
                obj         = e.Data.GetData(typeof(ListViewItem));
                itemToMove  = obj as ListViewItem;
                if (itemToMove != null && itemToMove.ListView == this) {
                    itemFirst   = TopItem;
                    if (itemFirst != null) {
                        rect        = GetItemRect(itemFirst.Index);
                        pt          = PointToClient(new Point(e.X, e.Y));
                        pt.X        = rect.Left;
                        pt.Y       -= rect.Height / 2;
                        if (pt.Y <= rect.Top) {
                            itemPos = null;
                        } else {
                            itemPos = GetItemAt(pt.X, pt.Y);
                            if (itemPos == null) {
                                itemPos = Items[Items.Count - 1];
                            }
                        }
                        MoveItem(itemToMove, itemPos);
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragDrop:         Called when the drag object is dropped
        /// </summary>
        /// <param name="e">    Drag parameter</param>
        //
        //******************************************************************
        protected override void OnQueryContinueDrag(QueryContinueDragEventArgs e)
        {
            if (e.EscapePressed) {
                e.Action = DragAction.Cancel;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnGiveFeedback:     Called when a drop operation is to be done
        /// </summary>
        /// <param name="e">    Drag parameter</param>
        //
        //******************************************************************
        protected override void OnGiveFeedback(GiveFeedbackEventArgs e)
        {
            base.OnGiveFeedback(e);
            if (m_bMoveAllowed) {
                e.UseDefaultCursors = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAfterSelect:      Called after a new item has been selected
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //******************************************************************
        protected override void OnAfterSelect(AfterSelectEventArgs e) {
            base.OnAfterSelect(e);
            SetUpDownButtonStatev();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnButUpClick:           Called when the button up is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //******************************************************************
        private void OnButUpClick(object sender, EventArgs e) {
            ListViewItem    item;
            ListViewItem    itemAfter;
            
            item = SelectedItem;
            if (item != null && item.Index != 0) {
                itemAfter = (item.Index < 2) ? null : Items[item.Index-2];
                MoveItem(item, itemAfter);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnButDownClick:         Called when the button down is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //******************************************************************
        private void OnButDownClick(object sender, EventArgs e) {
            ListViewItem    item;
            ListViewItem    itemAfter;
            
            item = SelectedItem;
            if (item != null && item.Index != (Items.Count - 1)) {
                itemAfter = Items[item.Index+1];
                MoveItem(item, itemAfter);
            }
        }

        //*********************************************************	    
        /// <summary>
        /// Implement arguments for the SetItemInfo event.
        /// </summary>
        //*********************************************************	    
        public class SetItemInfoEventArgs : System.EventArgs {
            /// <summary>Item modified or added</summary>
            private ListViewItem    m_item;
            /// <summary>true if new item</summary>
            private bool            m_bIsNewItem;

            //******************************************************************
            //
            /// <summary>
            /// ctor:                       Class constructor
            /// </summary>
            /// <param name="item">         Listview item</param>
            /// <param name="bIsNewItem">   true if it's a new item</param>
            //
            //******************************************************************
            public SetItemInfoEventArgs(ListViewItem item, bool bIsNewItem) {
                m_item          = item;
                m_bIsNewItem    = bIsNewItem;
            }

            //******************************************************************
            //
            /// <summary>
            /// Item:                       Listview item
            /// </summary>
            //
            //******************************************************************
            public ListViewItem Item {
                get {
                    return(m_item);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// IsNewItem:                  true if item is new
            /// </summary>
            //
            //******************************************************************
            public bool IsNewItem {
                get {
                    return(m_bIsNewItem);
                }
            }
        } // SetItemInfoEventArgs class
    } // class
} // namespace
