using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Xml;
using System.Drawing;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// ListViewRowset: Create a list view around a rowset
	/// </summary>
	[ToolboxItem(true)]
	[ToolboxBitmap(typeof(ListViewRowset),"SCListView.bmp")]
	public class ListViewRowset : ListViewBase, ISelectionBufferUI {
        /// <summary>
        /// Rep++ Rowset attached to the listview
        /// </summary>
        protected RepPP.Rowset          m_rowset;
        /// <summary>
        /// FieldDef which must be autorenum after a move
        /// </summary>
        protected RepPP.FieldDef        m_fldAutoRenum;
        /// <summary>
        /// Increment for the auto-renum
        /// </summary>
        protected int                   m_iAutoInc;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
        public ListViewRowset() {
            m_rowset        = null;
            m_fldAutoRenum  = null;
            m_iAutoInc      = 10;
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetDef:                      RowsetDef assign to the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Rowset definition associates to the listview."), Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden) ]
        public override RepPP.RowsetDef RowsetDef {
            get {
                return(base.RowsetDef);
            }
            set {
                if (m_rowsetDef != value) {
                    m_rowset = null;
                    base.RowsetDef = value;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Rowset:              Rowset assign to the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Rowset associates to the listview."), Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden) ]
        public virtual RepPP.Rowset Rowset {
            get {
                return(m_rowset);
            }
            set {
                m_rowset = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumField:             FieldDef use to auto-renum the line after a move
        /// </summary>
        //
        //*********************************************************
 	    [Description("Field use to auto-renum the line after a move."), Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden) ]
        public RepPP.FieldDef AutoRenumField {
            get {
                return(m_fldAutoRenum);
            }
            set {
                m_fldAutoRenum = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumIncrement:         Value use to auto-renum the lines after a move
        /// </summary>
        //
        //*********************************************************
		[Description("Value use to auto-renum the lines after a move."), Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden) ]
        public int AutoRenumIncrement {
            get {
                return(m_iAutoInc);
            }
            set {
                m_iAutoInc = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateListViewItem:         Creates a new listview item. Can be override to creates derived class
        /// </summary>
        /// <param name="iLineIndex">   Line index</param>
        /// <returns>
        /// New listview item
        /// </returns>
        //
        //*********************************************************
        protected override ListViewItem CreateListViewItem(int iLineIndex) {
            return(new ListViewLineItem(m_rowset, iLineIndex));
        }

        //*********************************************************
        //
        /// <summary>
        /// Fill:                           Fill the listview with the value of the rowset
        /// </summary>
        /// <param name="rowset">           Rowset to use to fill the listview</param>
        //
        //*********************************************************
        public virtual void Fill(RepPP.Rowset rowset) {
            if (m_rowsetDef == null) {
                RowsetDef = rowset.RowsetDef;
            }
            m_rowset = rowset;
            InternalFill(m_rowset, true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Fill:                           Fill the listview with the value of the rowset
        /// </summary>
        /// <param name="rowset">           Rowset to use to fill the listview</param>
        /// <param name="bDispDelLine">     True to display deleted lines in the listview</param>
        //
        //*********************************************************
        public virtual void Fill(RepPP.Rowset rowset, bool bDispDelLine) {
            if (m_rowsetDef == null) {
                RowsetDef = rowset.RowsetDef;
            }
            m_rowset = rowset;
            InternalFill(m_rowset, bDispDelLine);
        }

        //*********************************************************
        //
        /// <summary>
        /// FindItemFromLineBookmark:   Find an item using the line bookmark
        /// </summary>
        /// <param name="iLineBmk">     Line bookmark</param>
        /// <returns>
        /// item or null if not found
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewLineItem FindItemFromLineBookmark(IntPtr iLineBmk) {
            ListViewLineItem    itemRetVal = null;
            
            foreach (ListViewItem item in Items) {
                if (item is ListViewLineItem) {
                    itemRetVal = item as ListViewLineItem;
                    if (itemRetVal.LineBookmark == iLineBmk) {
                        break;
                    } else {
                        itemRetVal = null;
                    }
                }
            }
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// GetItemKey:                 Gets the currently selected item unique key
        /// </summary>
        /// <returns>
        /// Item key or null if none
        /// </returns>
        //
        //*********************************************************
        public string GetItemKey() {
            string  strRetVal;
            
            if (m_rowset == null || m_rowset.CurrentLine == -1) {
                strRetVal = null;
            } else {
                strRetVal = "";
                foreach (RepPP.FieldDef fldDef in m_rowset.RowsetDef.FieldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        if (strRetVal.Length != 0) {
                            strRetVal += "\0";
                        }
                        strRetVal += fldDef.Value;
                    }
                }
            }
            return(strRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// FindItemFromKey:            Find the item from the specified key
        /// </summary>
        /// <param name="strItemKey">   Item key</param>
        /// <returns>
        /// Item related to the key or null if not found
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewLineItem FindItemFromKey(string strItemKey) {
            ListViewLineItem    itemRetVal = null;
            int                 iCount;
            int                 iPos;
            string[]            arrKey;
            object[]            arrObj;
            
            if (strItemKey != null && strItemKey.Length != 0) {
                arrKey = strItemKey.Split('\0');
                iCount = arrKey.Length;
                arrObj = new object[iCount * 2];
                iPos   = 0;
                foreach (RepPP.FieldDef fldDef in m_rowset.RowsetDef.FieldDefs) {
                    if (fldDef.IsPrimaryKey) {
                        if (iPos == iCount) {
                            throw new System.ApplicationException("Key item overflow!");
                        }
                        arrObj[iPos*2]      = fldDef;
                        arrObj[iPos*2+1]    = arrKey[iPos];
                        iPos++;
                    }
                }
                if (iPos != iCount) {
                    throw new System.ApplicationException("Key item mismatch");
                }
                if (m_rowset.FindValue(false, arrObj) == 0) {
                    itemRetVal = FindItemFromLineBookmark(m_rowset.LineBookmark(m_rowset.CurrentLine));
                }
            }
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// SelectItemFromKey:          Select the item from the specified key
        /// </summary>
        /// <param name="strItemKey">   Item key</param>
        /// <returns>
        /// true if a selection occurs, false if not.
        /// </returns>
        //
        //*********************************************************
        public virtual bool SelectItemFromKey(string strItemKey) {
            bool                bRetVal;
            ListViewLineItem    item;
            
            item = FindItemFromKey(strItemKey);
            if (item != null) {
                SelectedItem  = item;
                bRetVal       = true;
            } else {
                bRetVal       = false;
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// InsertItem:             Insert an item in the list
        /// </summary>
        /// <param name="iIndex">   Index where to add it in the list. -1 to append</param>
        /// <returns>
        /// New item inserted
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewItem InsertItem(int iIndex) {
            return(InternalInsertItem(iIndex, m_rowset));
        }

        //*********************************************************
        //
        /// <summary>
        /// UpdateItem:             Update an item in the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to change</param>
        //
        //*********************************************************
        public virtual void UpdateItem(int iIndex) {
            InternalUpdateItem(iIndex, m_rowset);
        }

        //*********************************************************
        //
        /// <summary>
        /// DeleteItem:             Delete an item from the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to update</param>
        //
        //*********************************************************
        public void DeleteItem(int iIndex) {
            InternalDeleteItem(iIndex);
        }

        //******************************************************************
        //
        /// <summary>
        /// AutoRenum:                  Autorenum the lines after a move
        /// </summary>
        //
        //******************************************************************
        protected virtual void AutoRenum()
        {
            IntPtr              iCurBmk;
            int                 iCurLine;
            IntPtr              iLineBmk;
            RepPP.LineState     eLineState;
            int                 iVal;
            string              strVal;
            ListViewLineItem    gitem;
            
            if (m_fldAutoRenum != null && m_rowset != null && m_rowset.LineCount != 0) {
                iCurBmk = m_rowset.LineBookmark(m_rowset.CurrentLine);
                iVal    = m_iAutoInc;
                // Be careful, line count can change because of changes in PK
                for (int iIndex = 0; iIndex < m_rowset.LineCount; iIndex++) {
                    if (m_rowset.ThisLineState(iIndex) != RepPP.LineState.sdLineDeleted) {
                        m_rowset.SelectLine(iIndex, false, false);
                        iLineBmk = m_rowset.LineBookmark(iIndex);
                        gitem    = FindItemFromLineBookmark(iLineBmk);
                        strVal = iVal.ToString();
                        if (m_fldAutoRenum.Value != strVal) {
                            m_fldAutoRenum.Value = strVal;
                            eLineState         = m_rowset.LineState; // Force the state change of the line.
                            iLineBmk           = m_rowset.LineBookmark(iIndex);
                            gitem.LineBookmark = iLineBmk;
                            RefreshItemInSelectionBuffer(m_rowset);
                        }
                        iVal += m_iAutoInc;
                    }
                }
                iCurLine = m_rowset.LineFromBookmark(iCurBmk);
                if (iCurLine != -1 && iCurLine != m_rowset.CurrentLine) {
                    m_rowset.SelectLine(iCurLine, false, false);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MoveItem:                   Move an item after the specified position
        /// </summary>
        /// <param name="item">         Item to move</param>
        /// <param name="itemAfter">    Item after which the item is move or null for the first pos</param>
        //
        //******************************************************************
        public override void MoveItem(ListViewItem item, ListViewItem itemAfter)
        {
            int                 iItemLine;
            int                 iItemAfterLine;
            ListViewLineItem    gitem;
            ListViewLineItem    gitemAfter;

            base.MoveItem(item, itemAfter);
            if (item != itemAfter) {
                gitem       = item as ListViewLineItem;
                gitemAfter  = itemAfter as ListViewLineItem;
                if (gitem != null) {
                    iItemLine       = m_rowset.LineFromBookmark(gitem.LineBookmark);
                    iItemAfterLine  = (gitemAfter == null) ? -1 : m_rowset.LineFromBookmark(gitemAfter.LineBookmark);
                    if (m_rowset.CurrentLine != iItemLine) {
                        m_rowset.SelectLine(iItemLine);
                    }
                    if (iItemLine != (iItemAfterLine + 1)) {
                        m_rowset.MoveLineTo(iItemAfterLine);
                        AutoRenum();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAfterSelect:              Called after selecting an item
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        protected override void OnAfterSelect(AfterSelectEventArgs e) {
            ListViewLineItem                item;
            int                             iLinePos;
            
            if (SelectedItems.Count != 0) {
                if (SelectedItems[0] is ListViewLineItem) {
                    item     = SelectedItems[0] as ListViewLineItem;
                    iLinePos = m_rowset.LineFromBookmark(item.LineBookmark);
                    if (iLinePos != -1) {
                        if (iLinePos != m_rowset.CurrentLine) {
                            m_rowset.SelectLine(iLinePos, true, true);
                        }
                    }
                }
            }
            base.OnAfterSelect(e);
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:   Add a new item to the selection buffer
        /// </summary>
        /// <param name="ginstSB">      Rowset of the selection buffer</param>
        /// <param name="bDuplicate">   true to duplicate, false to insert new one</param>
        //
        //******************************************************************
        public virtual void AddItemToSelectionBuffer(RepPP.Rowset ginstSB, bool bDuplicate)  // implement ISelectionBufferUI
        { 
            ListViewItem    item;

            item          = InsertItem(-1);
            SelectedItem  = item;
            item.EnsureVisible();
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh an item
        /// </summary>
        /// <param name="ginstData">        Rowset use to refresh the data</param>
        //
        //******************************************************************
        public virtual void RefreshItemInSelectionBuffer(RepPP.Rowset ginstData) // implement ISelectionBufferUI
        {
            RepPP.Rowset        rowset;
            IntPtr              iLineBmk;
            ListViewLineItem    item;
            int                 iLine;
            
            rowset = Rowset;
            if (rowset != null && !rowset.IsDetached && ginstData != null && !ginstData.IsDetached) {
                if (rowset.RowsetDef != ginstData.RowsetDef) {
                    rowset.RowsetDef.CopyFieldsFrom(ginstData.RowsetDef, RepPP.CopyFieldOpt.sdCopyState);
                }
                iLine = rowset.CurrentLine;
                if (iLine >= 0) {
                    iLineBmk = rowset.LineBookmark(iLine);
                    if (iLineBmk != (IntPtr)(-1)) {
                        item = FindItemFromLineBookmark(iLineBmk);
                        if (item == null) {
                            item = SelectedItem as ListViewLineItem;
                        }
                        if (item != null) {
                            UpdateItem(item.Index);
                        }
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete an item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to select another row</param>
        //
        //******************************************************************
        public virtual void DeleteItemFromSelectionBuffer(bool bReselect) // implement ISelectionBufferUI
        {
            ListViewItem        item;
            int                 iIndex;
            
            item   = (SelectedItem != null) ? SelectedItem : m_selectionInfo.m_itemCurrent;
            iIndex = item.Index;
            Rowset.DeleteLine();
            DeleteItem(iIndex);
            if (bReselect) {
                if (iIndex >= Items.Count) {
                    iIndex = Items.Count - 1;
                }
                if (iIndex != -1) {
                    SelectedItem = Items[iIndex];
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetItemCount:                   Gets the number of item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetItemCount()      // implement ISelectionBufferUI
        {
            return(Items.Count);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItem:                 Gets the current item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetCurrentItem()    // implement ISelectionBufferUI
        {
            int     iRetVal;

            iRetVal = (SelectedItems.Count == 0) ? -1 : SelectedItems[0].Index;
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select the specified item
        /// </summary>
        //
        //******************************************************************
        public virtual bool SelectItem(int iPos)   // implement ISelectionBufferUI
        {
            bool         bRetVal;
            ListViewItem item;

            if (iPos < 0) {
                bRetVal = false;
            } else if (iPos >= Items.Count) {
                bRetVal = false;
            } else {
                item         = Items[iPos];
                SelectedItem = item;
                item.EnsureVisible();
                bRetVal              = true;
            }
            return(bRetVal);
        }
	} // class
} // namespace
