using System;
using System.Collections;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //*********************************************************	    
    //
    /// <summary>
    /// Implement the base class of a tree node attached to a TreeViewRowsetTree 
    /// </summary>
    // 
    //*********************************************************	    
    public class TreeNodeRowset : TreeNodeBase {
        /// <summary>Internal class use to keep information about recursion</summary>
        private class SCRecInfo {
            /// <summary>Recursive rowset or null if none</summary>
            public RepPP.RowsetDef                          m_rowsetDefRecursive;
            /// <summary>WHERE clause to make the recursion</summary>
            public string                                   m_strRecWhereClause;
            /// <summary>ORDER BY to order the row of a recursive call</summary>
            public string                                   m_strOrderBy;
            /// <summary>SELECT sql command for 2 levels recursion</summary>
            public string                                   m_strSelectFor2Level;
            /// <summary>Array of child field name if recursion on 2 levels</summary>
            public string[]                                 m_arrChildFldName;
            /// <summary>RowsetTree created if this node is recursive</summary>
            public RepPP.RowsetTree                         m_rowsetTreeRec;
        };

        private static Hashtable                        m_hashRowsetDefInfo;
        /// <summary>Rowset attached to this node.</summary>
        private RepPP.Rowset                            m_rowset;
        /// <summary>Bookmark uniquely identifying the line attached to this node.</summary>
        private IntPtr                                  m_iLineBmk;
        /// <summary>Information about the recursion if this node is recursive (null if not)</summary>
        private TreeNodeRowset.SCRecInfo                m_recInfo;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="treeFather">       Father treeview</param>
        /// <param name="rowset">            Rowset to attach to the node</param>
        /// <param name="iLineIndex">       Line index</param>
        /// <param name="childNodeFactory"> Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public TreeNodeRowset(TreeViewRowsetTree                         treeFather,
                             RepPP.Rowset                           rowset,
                             int                                    iLineIndex,
                             TreeNodeRowset.IRowsetChildNodeFactory childNodeFactory) : base(childNodeFactory)
        {
            m_rowset            = rowset;
            m_iLineBmk          = rowset.LineBookmark(iLineIndex);
            m_recInfo           = null;
            Init(treeFather, rowset.ChildRowsets.Count != 0); 
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        //
        //*********************************************************	    
        public TreeNodeRowset() 
        {
            m_rowset            = null;
            m_iLineBmk          = (IntPtr)0;
            m_recInfo           = null;
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                Free the node
        /// </summary>
        //
        //*********************************************************	    
        public override void Dispose()
        {
            if (m_recInfo != null) {
                if (m_recInfo.m_rowsetTreeRec != null) {
                    m_recInfo.m_rowsetTreeRec.Delete();
                    m_recInfo.m_rowsetTreeRec = null;
                }
                m_recInfo = null;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Clone:                      Clone the node and all its descendant
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override object Clone() {
            TreeNodeRowset  nodeRetVal;
            SCRecInfo       recInfo;
            RowsetTree      rowsetTreeNew;

            nodeRetVal              = base.Clone() as TreeNodeRowset;
            nodeRetVal.m_rowset     = m_rowset;
            nodeRetVal.m_iLineBmk   = m_iLineBmk;
            if (m_recInfo == null) {
                nodeRetVal.m_recInfo            = null;
            } else {
                recInfo                         = new SCRecInfo();
                recInfo.m_rowsetDefRecursive    = m_recInfo.m_rowsetDefRecursive;
                recInfo.m_strRecWhereClause     = m_recInfo.m_strRecWhereClause;
                recInfo.m_strOrderBy            = m_recInfo.m_strOrderBy;
                recInfo.m_strSelectFor2Level    = m_recInfo.m_strSelectFor2Level;
                recInfo.m_arrChildFldName       = m_recInfo.m_arrChildFldName;
                if (m_recInfo.m_rowsetTreeRec == null) {
                    recInfo.m_rowsetTreeRec  = null;
                } else {
                    rowsetTreeNew = m_recInfo.m_rowsetTreeRec.RowsetTreeDef.RowsetTrees.Add();
                    rowsetTreeNew.RootRowset.CopyFrom(m_recInfo.m_rowsetTreeRec.RootRowset, true);
                    recInfo.m_rowsetTreeRec  = rowsetTreeNew;
                }
            }
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SpiceSelectCmd: Spice the SELECT cmd to execute a join between the child and father table. This
        ///                 spicing of the SELECT cmd need to be done for recursion using 2 groups
        /// </summary>
        //
        // Transform SELECT P1,P2,S1,S2 FROM T1 into
        //           SELECT A.P1,A.P2,S1,S2 FROM T1 A,T2 B WHERE A.P1=B.X1 AND A.P2=B.X2 AND B.P1=:T1.P1 AND B.P2=:T2.P2
        //           Where  Pn is primary key of the root card group
        //                  Xn is the child foreign key
        //                  T2 is the table name of the specified card group...
        //
        //*********************************************************	    
        private void SpiceSelectCmd() {
            RepPP.RowsetDef rowsetDef;
            RepPP.FieldDefs fldDefsT1;
            RepPP.FieldDefs fldDefsT2;
            RepPP.FieldDef  fldDefT2;
            RepPP.Tool      tool;
            string          strWhere;
            string          strWhere2;
            string          strSqlCmd;
            string          strOriOrderBy;
            string          strTableName = "";
            int             iPKCount;
            int             iPos;

            tool            = m_rowset.Application.Tool;            
            rowsetDef       = m_rowset.RowsetDef;
            fldDefsT1       = rowsetDef.FieldDefs;
            fldDefsT2       = m_recInfo.m_rowsetDefRecursive.FieldDefs;
            iPKCount        = 0;
            strSqlCmd       = m_rowset.RowsetTreeDefNode.SelectSqlCommand;
            strWhere2       = "";
            strOriOrderBy   = tool.GetSqlOrderBy(strSqlCmd);
            foreach (RepPP.FieldDef fldDef in fldDefsT1) {
                if (fldDef.IsPrimaryKey) {
                    iPKCount++;
                    strWhere2 += " AND B." + fldDef.ColName + "=:" + rowsetDef.Name + "." + fldDef.Name;
                    if (StringUtility.StrFindKeyword(strSqlCmd,
                                                     0,
                                                     out iPos,
                                                     fldDef.ColName,
                                                     StringUtility.SearchTypeE.SK_NON_ALNUM_AFTER  |
                                                     StringUtility.SearchTypeE.SK_NON_ALNUM_BEFORE |
                                                     StringUtility.SearchTypeE.SK_IGNORE_CASE      |
                                                     StringUtility.SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE) != -1) {
                                                     strSqlCmd = strSqlCmd.Substring(0, iPos) + "A." + strSqlCmd.Substring(iPos);
                    }
                    if (StringUtility.StrFindKeyword(strOriOrderBy,
                                                     0,
                                                     out iPos,
                                                     fldDef.ColName,
                                                     StringUtility.SearchTypeE.SK_NON_ALNUM_AFTER  |
                                                     StringUtility.SearchTypeE.SK_NON_ALNUM_BEFORE |
                                                     StringUtility.SearchTypeE.SK_IGNORE_CASE      |
                                                     StringUtility.SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE) != -1) {
                                                     strOriOrderBy = strOriOrderBy.Substring(0, iPos) + "A." + strOriOrderBy.Substring(iPos);
                    }
                }
            }
            if (iPKCount != m_recInfo.m_arrChildFldName.Length) {
                throw new System.ApplicationException("Bad list of child fields. Array must match the primary key");
            }
            strWhere = " WHERE ";
            iPKCount = 0;
            foreach (RepPP.FieldDef fldDefT1 in fldDefsT1) {
                if (fldDefT1.IsPrimaryKey) {
                    fldDefT2 = fldDefsT2[m_recInfo.m_arrChildFldName[iPKCount]];
                    if (fldDefT2 == null) {
                        throw new System.ApplicationException("Field '" + m_recInfo.m_arrChildFldName[iPKCount] +
                                                              "' not found!");
                    }
                    if (iPKCount != 0) {
                        strWhere += " AND ";
                    }
                    strWhere += "A." + fldDefT1.ColName + "=B." + fldDefT2.ColName;
                    strTableName = fldDefT2.TableName;
                    iPKCount++;
                }
            }
            m_recInfo.m_strSelectFor2Level = tool.SetSqlWhere(strSqlCmd, "A, " + strTableName + " B " + strWhere + strWhere2);
            m_recInfo.m_strSelectFor2Level = tool.SetSqlOrderBy(m_recInfo.m_strSelectFor2Level, (m_recInfo.m_strOrderBy != null) ? m_recInfo.m_strOrderBy : strOriOrderBy);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetRecursive:                       Transform this node into a recursive one
        /// </summary>
        /// <param name="rowsetDefRecursive">   RowsetDef which make the recursion. NULL for the root.</param>
        /// <param name="strRecWhereClause">    WHERE clause use to make the recursion</param>
        /// <param name="strOrderBy">           ORDER BY clause or null if none</param>
        /// <param name="arrChildFldName">      List of child field which make the link</param>
        //
        //*********************************************************	    
        private void SetRecursive(RepPP.RowsetDef   rowsetDefRecursive,
                                  string            strRecWhereClause,
                                  string            strOrderBy,
                                  string[]          arrChildFldName) {
            if (m_recInfo != null) {
                throw new System.ApplicationException("The recursion is already set for this node!");
            }
            try {
                if (m_rowset.RowsetTree == null) {
                    throw new System.ApplicationException("The attached rowset of a node must be part of a rowset tree to be able to be recursive");
                } else if (m_rowset.RowsetTree.RootRowset != m_rowset) {
                    throw new System.ApplicationException("Only a node attached to the root rowset of a rowset tree can be set as recursive");
                }
                m_recInfo                      = new SCRecInfo();
                m_recInfo.m_strSelectFor2Level = "";
                m_recInfo.m_strRecWhereClause  = strRecWhereClause;
                m_recInfo.m_strOrderBy         = strOrderBy;
                m_recInfo.m_arrChildFldName    = arrChildFldName;
                m_recInfo.m_rowsetTreeRec      = null;
                if (rowsetDefRecursive == null || rowsetDefRecursive == m_rowset.RowsetDef) {
                    m_recInfo.m_rowsetDefRecursive = m_rowset.RowsetDef;
                } else {
                    foreach (RepPP.RowsetTreeDefNode node in m_rowset.RowsetTree.RowsetTreeDef.Nodes[0].Nodes) {
                        if (node.RowsetDef == rowsetDefRecursive) {
                            m_recInfo.m_rowsetDefRecursive = rowsetDefRecursive;
                            break;
                        }
                    }
                    if (m_recInfo.m_rowsetDefRecursive == null) {
                        throw new System.ApplicationException("The rowset definition making the recursion must be the root rowset or a direct descendant");
                    }
                    if (arrChildFldName == null) {
                        throw new System.ApplicationException("You must specify a list of fields which contains the link in the child rowset to the father primary key");
                    }
                    SpiceSelectCmd();
                }
            } catch {
                m_recInfo = null;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetRecursive1Level:             Transform this node into a recursive one. The recursion must
        ///                                 occurs within the root rowset.
        /// </summary>
        /// <param name="strRecWhereClause">WHERE clause use to make the recursion</param>
        /// <param name="strOrderBy">       ORDER BY clause or null if none</param>
        //
        //*********************************************************	    
        public void SetRecursive1Level(string   strRecWhereClause,
                                       string   strOrderBy) {

            SetRecursive(null,
                         strRecWhereClause,
                         strOrderBy,
                         null);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetRecursive2Level:                 Transform this node into a recursive one. The recursion is done
        ///                                     between the root rowset and the specified one (which must be a direct
        ///                                     descendant of the root one).
        /// </summary>
        /// <param name="rowsetDefRecursive">   Rowset definition which make the recursion.</param>
        /// <param name="strOrderBy">           ORDER BY clause or null if none</param>
        /// <param name="arrChildFldName">      List of child field which make the link</param>
        //
        //*********************************************************	    
        public void SetRecursive2Level(RepPP.RowsetDef  rowsetDefRecursive,
                                       string           strOrderBy,
                                       string[]         arrChildFldName) {
            SetRecursive(rowsetDefRecursive,
                         "",
                         strOrderBy,
                         arrChildFldName);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Rowset:      Rowset attached to this node
        /// </summary>
        //
        //*********************************************************	    
        public override RepPP.Rowset Rowset {
            get {
                return(m_rowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReassociateGInst            Reassociate the node to a new rowset
        /// </summary>
        /// <param name="rowsetNew">    New rowset to associate</param>
        /// <param name="iNewLine">     New line of the item (-1 if didn't change)</param>
        //
        //******************************************************************
        public virtual void ReassociateGInst(Rowset rowsetNew, int iNewLine) {
            Rowset          rowsetOld;
            Rowset          rowsetChildOld;
            Rowset          rowsetChildNew;
            TreeNodeRowset  nodeChild;
            int             iLine;

            rowsetOld   = Rowset;
            iLine       = (iNewLine != -1) ? iNewLine : rowsetOld.LineFromBookmark(m_iLineBmk);
            if (rowsetOld.ChildRowsets.Count != 0) {
                foreach (TreeNode nodeTmp in Nodes) {
                    nodeChild = nodeTmp as TreeNodeRowset;
                    if (nodeChild != null) {
                        rowsetChildOld = nodeChild.Rowset;
                        if (rowsetChildOld != null && rowsetChildOld.FatherRowset == rowsetOld) {
                            rowsetChildNew = rowsetNew.GetChild(rowsetChildOld.RowsetTreeDefNode.ChildId, iLine, false, false);
                            nodeChild.ReassociateGInst(rowsetChildNew, -1);
                        }
                    }
                }
            }
            m_iLineBmk = rowsetNew.LineBookmark(iLine);
            m_rowset   = rowsetNew;
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindChildRowset:            Find a child rowset if any
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition</param>
        /// <returns>
        /// null or the found rowset
        /// </returns>
        //
        //*********************************************************	    
        public override RepPP.Rowset FindChildRowset(RepPP.RowsetDef rowsetDef)
        {
            RepPP.Rowset rowsetRetVal = null;
            
            if (m_rowset != null) {
                rowsetRetVal = m_rowset.ChildRowsets.Find(rowsetDef.Name);
            }
            return(rowsetRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// LineBookmark:       Bookmark identifying the group instance line attached to this node.
        /// </summary>
        //
        //*********************************************************	    
        public IntPtr LineBookmark {
            get {
                return(m_iLineBmk);
            }
            set {
                m_iLineBmk = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// IsRecursive:            true if the node is recursive, no if not.
        /// </summary>
        //
        //*********************************************************	    
        public override bool IsRecursive
        {
            get {
                return(m_recInfo != null);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///  SpiceSelectCommand     Returned the spiced SQL command if any
        /// </summary>
        //
        //*********************************************************	    
        public string SpiceSelectCommand
        {
            get {
                return((m_recInfo != null) ? m_recInfo.m_strSelectFor2Level : null);
            }
            set {
                if (m_recInfo == null) {
                    throw new System.ApplicationException("You must first set the node recursive!");
                }
                m_recInfo.m_strSelectFor2Level = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// RecRowsetTree:          Returned the rowset tree created by this node to hold
        ///                         the recursive child rowset.
        /// </summary>
        //
        //*********************************************************	    
        public RepPP.RowsetTree RecRowsetTree
        {
            get {
                RowsetTree  rowsetTreeRetVal;

                if (m_recInfo == null) {
                    rowsetTreeRetVal = null;
                } else {
                    if (m_recInfo.m_rowsetTreeRec == null && m_rowset != null) {
                        m_recInfo.m_rowsetTreeRec = m_rowset.RowsetTree.RowsetTreeDef.RowsetTrees.Add();
                    }
                    rowsetTreeRetVal = m_recInfo.m_rowsetTreeRec;
                }
                return(rowsetTreeRetVal);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateNode:                     Creates a node
        /// </summary>
        /// <param name="treeFather">       Father treeview</param>
        /// <param name="rowset">           Rowset</param>
        /// <param name="iLineIndex">       Line index</param>
        /// <param name="childNodeFactory"> Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public static TreeNodeRowset CreateNode(TreeViewRowsetTree                      treeFather,
                                               RepPP.Rowset                             rowset,
                                               int                                      iLineIndex,
                                               TreeNodeRowset.IRowsetChildNodeFactory   childNodeFactory)
        {
            TreeNodeRowset                                  node = null;
            TreeViewRowsetTree.BeforeNodeCreatedEventArgs   e;

            e = new TreeViewRowsetTree.BeforeNodeCreatedEventArgs(rowset, iLineIndex);
            if (treeFather != null) {
                treeFather.OnBeforeNodeCreated(e);
            }
            if (!e.Cancel) {
                node = (childNodeFactory == null) ? new TreeNodeRowset(treeFather, rowset, iLineIndex, null) :
                                                    childNodeFactory.CreateRowsetNode(treeFather,
                                                                                     rowset,
                                                                                     iLineIndex,
                                                                                     childNodeFactory);
                if (treeFather != null) {
                    treeFather.OnAfterNodeCreated(new TreeViewRowsetTree.AfterNodeCreatedEventArgs(node));
                }
            }
            return(node);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FillNodes:                      Fill the nodes with the content of the group instance
        /// </summary>
        /// <param name="treeFather">       Father treeview</param>
        /// <param name="nodes">            Nodes collections</param>
        /// <param name="rowset">           Rowset</param>
        /// <param name="childNodeFactory"> Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public static void FillNodes(TreeViewRowsetTree                     treeFather,
                                     TreeNodeCollection                     nodes,
                                     RepPP.Rowset                           rowset,
                                     TreeNodeRowset.IRowsetChildNodeFactory childNodeFactory)
        {
            int             iLineCount;
            int             iLineIndex;
            ArrayList       nodesToAdd;
            TreeNodeBase    node;
            
            treeFather.BeginUpdate();
            try {
                iLineCount = rowset.LineCount;
                nodesToAdd = new ArrayList(rowset.UndelLineCount);
                for (iLineIndex = 0; iLineIndex < iLineCount; iLineIndex++) {
                    if (rowset.ThisLineState(iLineIndex) != LineState.sdLineDeleted) {
                        node = CreateNode(treeFather, rowset, iLineIndex, childNodeFactory);
                        if (node != null) {
                            nodesToAdd.Add(node);
                        }
                    }
                }
                nodes.AddRange((TreeNodeBase[])nodesToAdd.ToArray(typeof(TreeNodeBase)));
            } finally {
                treeFather.EndUpdate();
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FillOneChildNodes:              Fill the nodes with the content of the group instance
        /// </summary>
        /// <param name="treeFather">       Father treeview</param>
        /// <param name="rowset">           Rowset</param>
        /// <param name="childNodeFactory"> Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public virtual void FillOneChildNodes(TreeViewRowsetTree                        treeFather,
                                              RepPP.Rowset                              rowset,
                                              TreeNodeRowset.IRowsetChildNodeFactory    childNodeFactory)
        {
            FillNodes(treeFather, Nodes, rowset, childNodeFactory);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ReadChildren:           Read node's children
        /// </summary>
        //
        //*********************************************************	    
        public override void ReadChildren()
        {
            RepPP.RowsetTree                            rowsetTreeRec;
            RepPP.Rowset                                rowsetChild;
            int                                         iAttachedLine;
            TreeViewRowsetTree.BeforeChildReadEventArgs beforeArg;
            TreeViewRowsetTree.AfterChildReadEventArgs  afterArg;
            string                                      strOldSelectCmd;
            
            if (!IsChildrenRead) {
                IsChildrenRead  = true;
                // Remove the 'Dummy' node created by the base class so the '+' appear
                Nodes.Clear();
                beforeArg       = new TreeViewRowsetTree.BeforeChildReadEventArgs(this);
                OnBeforeChildRead(beforeArg);
                if (!beforeArg.Cancel) {
                    iAttachedLine   = m_rowset.LineFromBookmark(m_iLineBmk);
                    if (iAttachedLine != -1) {
                        m_rowset.RowsetDef.SaveActiveRowset();
                        m_rowset.SaveLinePos();
                        try {
                            m_rowset.Select(false, false);
                            m_rowset.SelectLine(iAttachedLine, false, false);
                            if (m_recInfo != null) {
                                // Recursive 1 level: the group point to itself
                                strOldSelectCmd = m_rowset.RowsetTreeDefNode.SelectSqlCommand;
                                try {
                                    if (m_recInfo.m_rowsetDefRecursive == m_rowset.RowsetDef) {
                                        m_rowset.RowsetTreeDefNode.SetSelectSqlWhere(m_recInfo.m_strRecWhereClause);
                                    } else {
                                        m_rowset.RowsetTreeDefNode.SelectSqlCommand = m_recInfo.m_strSelectFor2Level;
                                    }
                                    if (m_recInfo.m_strOrderBy != null) {
                                        m_rowset.RowsetTreeDefNode.SetSelectSqlOrderBy(m_recInfo.m_strOrderBy);
                                    }
                                    rowsetTreeRec = RecRowsetTree;
                                    m_rowset.Select(false, false);
                                    m_recInfo.m_rowsetTreeRec.ReadFromDb();
                                } finally {
                                    m_rowset.RowsetTreeDefNode.SelectSqlCommand = strOldSelectCmd;
                                }
                                FillOneChildNodes(TreeView as TreeViewRowsetTree, m_recInfo.m_rowsetTreeRec.RootRowset, ChildNodeFactory);
                            }
                            foreach (RepPP.RowsetTreeDefNode node in m_rowset.RowsetTreeDefNode.Nodes) {
                                if (m_recInfo == null || node.RowsetDef != m_recInfo.m_rowsetDefRecursive) {
                                    rowsetChild = m_rowset.ChildRowsets[node.ChildId];
                                    FillOneChildNodes(TreeView as TreeViewRowsetTree, rowsetChild, ChildNodeFactory);
                                }
                            }
                        } finally {
                            m_rowset.RestoreLinePos(false);
                            m_rowset.RowsetDef.RestoreActiveRowset(false);
                        }
                    }
                    afterArg = new TreeViewRowsetTree.AfterChildReadEventArgs(this);
                    OnAfterChildRead(afterArg);
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetNodeInfo:                Set the node information
        /// </summary>
        /// <param name="treeFather">   Father treeview</param>
        //
        //*********************************************************	    
        protected override void SetNodeInfo(TreeViewRowsetTree treeFather) {
            RepPP.FieldDef[]    arrFldDefs;
            ArrayList           arrList;
            RepPP.RowsetDef     rowsetDef;
            RepPP.FieldDefs     fldDefs;
            int                 iLineIndex;
            bool                bPKInList;
            string              strUserText = null;
            
            if (m_hashRowsetDefInfo == null) {
                m_hashRowsetDefInfo = new Hashtable(10);
            }
            if (m_rowset != null) {
                rowsetDef = m_rowset.RowsetDef;
                if (m_hashRowsetDefInfo.ContainsKey(rowsetDef)) {
                    arrFldDefs = m_hashRowsetDefInfo[rowsetDef] as RepPP.FieldDef[];
                } else {
                    arrList     = new ArrayList();
                    fldDefs     = m_rowset.RowsetDef.FieldDefs;
                    bPKInList   = false;
                    foreach(RepPP.FieldDef fldDef in m_rowset.RowsetDef.FieldDefs) {
                        if (fldDef.IsPrimaryKey && arrList.Count == 0) {
                            arrList.Add(fldDef);
                            bPKInList = true;
                        }
                        if (fldDef.IsUserFlagOn) {
                            if (bPKInList) {
                                arrList.Clear();
                                bPKInList = false;
                            }
                            arrList.Add(fldDef);
                        }
                    }
                    if (arrList.Count == 0) {
                        arrList.Add(fldDefs[0]);
                    }
                    arrFldDefs = new RepPP.FieldDef[arrList.Count];
                    arrList.CopyTo(arrFldDefs);
                    m_hashRowsetDefInfo.Add(rowsetDef, arrFldDefs);
                }
                iLineIndex  = m_rowset.LineFromBookmark(m_iLineBmk);
                strUserText = null;
                foreach (RepPP.FieldDef fldDef in arrFldDefs) {
                    if (strUserText == null) {
                        strUserText = m_rowset.GetFieldValue(fldDef, iLineIndex, true);
                    } else {
                        strUserText += " " + m_rowset.GetFieldValue(fldDef, iLineIndex, true);
                    }
                }
                Text    = strUserText;
                Tooltip = strUserText;
                base.SetNodeInfo(treeFather);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetNodeKey:             Gets the node unique identifier.
        /// </summary>
        /// <returns>
        /// Key uniquely identifying the node
        ///</returns>
        //
        //*********************************************************	    
        public override string GetNodeKey()
        {
            string      strRetVal;
            int         iAttachedLine;
            
            strRetVal     = "ROWSET:";
            iAttachedLine = m_rowset.LineFromBookmark(m_iLineBmk);
            if (iAttachedLine != -1) {
                m_rowset.RowsetDef.SaveActiveRowset();
                m_rowset.SaveLinePos();
                try {
                    m_rowset.Select(false, false);
                    if (iAttachedLine != m_rowset.CurrentLine) {
                        m_rowset.SelectLine(iAttachedLine, false, false);
                    }
                    foreach (RepPP.FieldDef fldDef in m_rowset.RowsetDef.FieldDefs) {
                        if (fldDef.IsPrimaryKey) {
                            if (strRetVal.Length != 0) {
                                strRetVal += ",";
                            }
                            strRetVal += fldDef.RowsetDef.Name + "." + fldDef.Name + "," + fldDef.Value;
                        }
                    }
                } finally {
                    m_rowset.RestoreLinePos(false);
                    m_rowset.RowsetDef.RestoreActiveRowset(false);
                }
            }
            return(strRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SelectAttachedObject:       Select the object attached to the node
        /// </summary>
        //
        //*********************************************************	    
        public override void SelectAttachedObject()
        {
            int             iLineIndex;
            
            base.SelectAttachedObject();
            if (m_rowset != m_rowset.ActiveRowset) {
                m_rowset.Select(false, true);
            }
            if (m_iLineBmk != (IntPtr)(-1)) {
                iLineIndex = m_rowset.LineFromBookmark(m_iLineBmk);
                if (iLineIndex != -1) {
                    m_rowset.SelectLine(iLineIndex, false, true);
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingCount:        Gets the number of sibling
        /// </summary>
        /// <returns>
        /// Number of nodes of the same type.
        /// </returns>
        //
        //*********************************************************	    
        public override int GetSiblingCount()
        {
            int                 iRetVal = 0;
            RepPP.Rowset        rowset;
            TreeNodeRowset      nodeSibling;
            TreeNodeCollection  nodes;
            
            rowset      = Rowset;
            nodes       = (Parent == null) ? TreeView.Nodes : Parent.Nodes;
            iRetVal     = 0;
            foreach (TreeNode nodeTmp in nodes) {
                nodeSibling = nodeTmp as TreeNodeRowset;
                if (nodeSibling != null && nodeSibling.Rowset == rowset) {
                    iRetVal++;
                }
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingPos:          Gets the position in the sibling
        /// </summary>
        /// <returns>
        /// Position
        /// </returns>
        //
        //*********************************************************	    
        public override int GetSiblingPos()
        {
            int                 iRetVal;
            TreeNodeCollection  nodes;
            TreeNodeRowset      nodeSibling;
            RepPP.Rowset        rowset;
            
            rowset      = Rowset;
            nodes       = (Parent == null) ? TreeView.Nodes : Parent.Nodes;
            iRetVal     = 0;
            foreach (TreeNode nodeTmp in nodes) {
                if (nodeTmp == this) {
                    break;
                }
                nodeSibling = nodeTmp as TreeNodeRowset;
                if (nodeSibling != null && nodeSibling.Rowset == rowset) {
                    iRetVal++;
                }
            }
            return(iRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindSiblingNode:        Find a sibling node using a position
        /// </summary>
        /// <param name="iPos">     Position (0..n)</param>
        /// <returns>
        /// null or the specified Node
        /// </returns>
        //
        //*********************************************************	    
        public override TreeNodeBase FindSiblingNode(int iPos)
        {
            TreeNodeBase        nodeRetVal = null;
            TreeNodeCollection  nodes;
            TreeNodeRowset      nodeSibling;
            RepPP.Rowset        rowset;
            
            rowset      = Rowset;
            nodes       = (Parent == null) ? TreeView.Nodes : Parent.Nodes;
            foreach (TreeNode nodeTmp in nodes) {
                nodeSibling = nodeTmp as TreeNodeRowset;
                if (nodeSibling != null && nodeSibling.Rowset == rowset) {
                    if (iPos == 0) {
                        nodeRetVal = nodeSibling;
                        break;
                    } else {
                        iPos--;
                    }
                }
            }
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// RowsetDef instance node factory interface
        /// </summary>
        //
        //*********************************************************	    
        public interface IRowsetChildNodeFactory {
            //*********************************************************	    
            //
            /// <summary>
            /// CreateRowsetNode:               Creates a rowset node
            /// </summary>
            /// <param name="treeFather">       Father treeview</param>
            /// <param name="rowset">           Rowset attached to the node</param>
            /// <param name="iLineIndex">       Line index</param>
            /// <param name="childNodeFactory"> Factory to use for children</param>
            /// <returns>
            /// The new node
            /// </returns>
            //
            //*********************************************************	    
            TreeNodeRowset CreateRowsetNode(TreeViewRowsetTree treeFather, RepPP.Rowset rowset, int iLineIndex, IRowsetChildNodeFactory childNodeFactory);
        }
    }
}
