using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Xml;

namespace RepPP.Toolkit.Window {

    //*********************************************************	    
    //
    /// <summary>
    /// Implement a tree view attached containing TreeNodeBase or derived node.
    /// </summary>
    //
    //*********************************************************	
    [ToolboxBitmap(typeof(TreeViewRowsetTree),"TreeViewRowsetTree.bmp")]
    public class TreeViewRowsetTree : TreeView, ISelectionBufferUI, IUIConfiguration {
        /// <summary>Kind of drop allowed on a node</summary>
        public enum AllowedDropMode {
            /// <summary>No drop allowed</summary>
            NoDrop,
            /// <summary>Drop allowed. A recursive item will includes another one (ex. folder will contain a folder)</summary>
            RootDrop,
            /// <summary>Drop allowed. An item will have a new father (ex. a file will be moved to a new folder)</summary>
            ChildDrop
        };

        /// <summary>
        /// Delegates for AfterNodeMoved events
        /// </summary>
        public delegate void                            AfterNodeMovedEventHandler(object sender, AfterNodeMovedEventArgs e);
        /// <summary>
        /// Delegates for BeforeNodeMoved events
        /// </summary>
        public delegate void                            BeforeNodeMovedEventHandler(object sender, BeforeNodeMovedEventArgs e);
        /// <summary>
        /// Delegates for AfterNodeCopied events
        /// </summary>
        public delegate void                            AfterNodeCopiedEventHandler(object sender, AfterNodeCopiedEventArgs e);
        /// <summary>
        /// Delegates for BeforeNodeCopied events
        /// </summary>
        public delegate void                            BeforeNodeCopiedEventHandler(object sender, BeforeNodeCopiedEventArgs e);
        /// <summary>
        /// Delegate for BeforeNodeCreated events
        /// </summary>
        public delegate void                            BeforeNodeCreatedEventHandler(object sender, BeforeNodeCreatedEventArgs e);
        /// <summary>
        /// Delegate for AfterNodeCreated events
        /// </summary>
        public delegate void                            AfterNodeCreatedEventHandler(object sender, AfterNodeCreatedEventArgs e);
        /// <summary>
        /// Occurs after a node has been moved with a drag and drop operation
        /// </summary>
        public event AfterNodeMovedEventHandler         AfterNodeMoved;
        /// <summary>
        /// Occurs before a node is moved with a drag and drop operation
        /// </summary>
        public event BeforeNodeMovedEventHandler        BeforeNodeMoved;
        /// <summary>
        /// Occurs after a node has been copied with a drag and drop operation
        /// </summary>
        public event AfterNodeCopiedEventHandler        AfterNodeCopied;
        /// <summary>
        /// Occurs before a is copied with a drag and drop operation
        /// </summary>
        public event BeforeNodeCopiedEventHandler       BeforeNodeCopied;
        /// <summary>
        /// Delegates for BeforeChildRead events
        /// </summary>
        public delegate void                            BeforeChildReadEventHandler(object sender, BeforeChildReadEventArgs e);
        /// <summary>
        /// Delegates for AfterChildRead events
        /// </summary>
        public delegate void                            AfterChildReadEventHandler(object sender, AfterChildReadEventArgs e);
        /// <summary>
        /// Delegates for SetNodeInfo events
        /// </summary>
        public delegate void                            SetNodeInfoHandler(object sender, SetNodeInfoEventArgs e);
        /// <summary>
        /// Occurs before the child node is read.
        /// </summary>
        public event BeforeChildReadEventHandler        BeforeChildRead;
        /// <summary>
        /// Occurs after the child node is read.
        /// </summary>
        public event AfterChildReadEventHandler         AfterChildRead;
        /// <summary>
        /// Occurs when a new node need to set its info
        /// </summary>
        public event SetNodeInfoHandler                 SetNodeInfo;
        /// <summary>
        /// Occurs before a node is created
        /// </summary>
        public event BeforeNodeCreatedEventHandler      BeforeNodeCreated;
        /// <summary>
        /// Occurs after a node is created
        /// </summary>
        public event AfterNodeCreatedEventHandler       AfterNodeCreated;
        /// <summary>
        /// Rowset attached to the child nodes if any.
        /// </summary>
        protected RepPP.Rowset                          m_rowsetChildNode;
        /// <summary>
        /// true if the tree is in filling mode, false if not.
        /// </summary>
        protected bool                                  m_bFilling;
        /// <summary>
        /// Tooltip object attached to the treeview
        /// </summary>
        private ToolTip                                 m_toolTip;
        /// <summary>
        /// Factory for the child node use to fill the first level of the tree
        /// </summary>
        private TreeNodeRowset.IRowsetChildNodeFactory  m_childNodeFactory;
        /// <summary>
        /// Configuration helper use to serialize the UI configuration
        /// </summary>
        private UIConfigurationHelper                   m_configHelper;
        /// <summary>
        /// Node use to load the UI configuration
        /// </summary>
        private XmlNode                                 m_nodeSerialization;
        /// <summary>
        /// True if the drop can be done on this node
        /// </summary>
        private bool                                    m_bDragOnTarget;
        /// <summary>
        /// True to manage the drag and drop automatically 
        /// </summary>
        private bool                                    m_bAutoDragDrop;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        //
        //*********************************************************	    
        public TreeViewRowsetTree() : base() {
            m_rowsetChildNode   = null;
            m_bFilling          = false;
            m_toolTip           = null;
            m_childNodeFactory  = null;
            m_nodeSerialization = null;
            m_bAutoDragDrop     = false;
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the treeview
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC</param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing)
        {
            if (bDisposing) {
                m_bFilling = true;
                Empty();
                m_bFilling = false;
            }
            base.Dispose(bDisposing);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeChildRead:      Trigger the event before the child nodes are read
        /// </summary>
        /// <param name="e">        Event argument</param>        
        //
        //*********************************************************	    
        public virtual void OnBeforeChildRead(BeforeChildReadEventArgs e)
        {
            if (BeforeChildRead != null) {
                BeforeChildRead(this, e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnAfterChildRead:       Trigger the event after the child nodes are read
        /// </summary>
        /// <param name="e">        Event argument</param>        
        //
        //*********************************************************	    
        public virtual void OnAfterChildRead(AfterChildReadEventArgs e)
        {
            if (AfterChildRead != null) {
                AfterChildRead(this, e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnSetNodeInfo:          Trigger the SetNodeInfo event. Use this event to change the node appearance.
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************	    
        public virtual void OnSetNodeInfo(SetNodeInfoEventArgs e)
        {
            if (SetNodeInfo != null) {
                SetNodeInfo(this, e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeNodeCreated:        Trigger the BeforeNodeCreated event
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************	    
        public virtual void OnBeforeNodeCreated(BeforeNodeCreatedEventArgs e)
        {
            if (BeforeNodeCreated != null) {
                BeforeNodeCreated(this, e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnAfterNodeCreated:     Trigger the AfterNodeCreated event
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************	    
        public virtual void OnAfterNodeCreated(AfterNodeCreatedEventArgs e)
        {
            if (AfterNodeCreated != null) {
                AfterNodeCreated(this, e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ChildNodeRowset:         Gets the rowset attached to the child nodes
        /// </summary>
        //
        //*********************************************************	    
		[Description("Rowset use to fill the root nodes"), Browsable(false)]
        public RepPP.Rowset ChildNodeRowset
        {
            get {
                return(m_rowsetChildNode);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindChildRowset:        Find the rowset of the specified child which can fill
        ///                         the Nodes of the treeview.
        /// </summary>
        /// <param name="rowsetDef">Rowset definition</param>
        /// <returns>
        /// null or the specified rowset
        /// </returns>
        //
        //*********************************************************	    
        public virtual RepPP.Rowset FindChildRowset(RepPP.RowsetDef rowsetDef)
        {
            RepPP.Rowset rowsetRetVal = null;
            
            if (m_rowsetChildNode != null) {
                rowsetRetVal = (m_rowsetChildNode.RowsetDef == rowsetDef) ? m_rowsetChildNode : null;
            }
            return(rowsetRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FillNodes:                      Fill the nodes with the content of the rowset
        /// </summary>
        /// <param name="rowsetChildNode">  Rowset attached the child node</param>
        /// <param name="childNodeFactory"> Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public virtual void FillNodes(RepPP.Rowset                              rowsetChildNode,
                                      TreeNodeRowset.IRowsetChildNodeFactory    childNodeFactory)
        {
            BeforeChildReadEventArgs    beforeArg;
            AfterChildReadEventArgs     afterArg;
            
            m_bFilling = true;
            BeginUpdate();
            try {
                Empty();
                m_childNodeFactory = childNodeFactory;
                m_rowsetChildNode  = rowsetChildNode;
                beforeArg          = new BeforeChildReadEventArgs(null);
                afterArg           = new AfterChildReadEventArgs(null);
                OnBeforeChildRead(beforeArg);
                if (!beforeArg.Cancel) {
                    if (rowsetChildNode != null) {
                        TreeNodeRowset.FillNodes(this, Nodes, rowsetChildNode, childNodeFactory);
                    }
                    OnAfterChildRead(afterArg);
                }
            } finally {
                m_bFilling = false;
                EndUpdate();
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ToolTip:                    Attached tooltip
        /// </summary>
        //
        //*********************************************************	    
		[Description("Tooltip control attached to the treeview"), Browsable(false)]
        public ToolTip ToolTip
        {
            get {
                return(m_toolTip);
            }
            set {
                m_toolTip = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// EmptyNodes:             Empty nodes 
        /// </summary>
        /// <param name="nodes">    Nodes to empty</param>
        //
        //*********************************************************
        private void EmptyNodes(TreeNodeCollection nodes)
        {
            TreeNodeBase    treeNode;
            
            foreach (TreeNode node in nodes) {
                EmptyNodes(node.Nodes);
                treeNode = node as TreeNodeBase;
                if (treeNode != null) {
                    treeNode.Dispose();
                }
            }            
        }

        //*********************************************************
        //
        /// <summary>
        /// Empty:              Empty the treeview 
        /// </summary>
        // 
        //*********************************************************
        public void Empty()
        {
            BeginUpdate();
            EmptyNodes(Nodes);
            Nodes.Clear();
            EndUpdate();
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetNodeFullKey:             Gets the node unique identifier.
        /// </summary>
        /// <param name="node">         Node</param>
        /// <returns>
        /// Key uniquely identifying the node
        ///</returns>
        //
        //*********************************************************	    
        public virtual string GetNodeFullKey(TreeNodeBase node)
        {
            string  strRetVal = "";
            
            if (node != null) {
                strRetVal = node.GetNodeKey();
                node      = node.Parent as TreeNodeBase;
                while (node != null) {
                    strRetVal = node.GetNodeKey() + "\0" + strRetVal;
                    node      = node.Parent as TreeNodeBase;
                }
            }
            return(strRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSelectedNodeFullKey: Gets the key representing the currently selected node
        /// </summary>
        /// <returns>
        /// Key of the selected node.
        /// </returns>
        //
        //*********************************************************	    
        public string GetCurrentlySelectedNodeFullKey()
        {
            string          strRetVal = "";
            TreeNode        nodeSelected;
            TreeNodeBase    node;
            
            nodeSelected = SelectedNode;
            while (nodeSelected != null && !(nodeSelected is TreeNodeBase)) {
                nodeSelected = nodeSelected.Parent;
            }
            if (nodeSelected != null) {
                node      = nodeSelected as TreeNodeBase;
                strRetVal = GetNodeFullKey(node);
            }
            return(strRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindNodeUsingRowset:    Find a nodes with the specified line bookmark
        /// </summary>
        /// <param name="nodes">    Nodes in which to do the search</param>
        /// <param name="rowset">   Rowset to search</param>
        /// <param name="iBmk">     Bookmark to search</param>
        /// <param name="bRec">     True for a recursive search</param>
        /// <returns>
        /// Node or null if not found
        /// </returns>
        //
        //*********************************************************	    
        private TreeNodeRowset FindNodeUsingRowset(TreeNodeCollection nodes, RepPP.Rowset rowset, IntPtr iBmk, bool bRec) {
            TreeNodeRowset   nodeRetVal = null;
            TreeNodeRowset   treeNode;
            
            foreach (TreeNode node in nodes) {
                treeNode = node as TreeNodeRowset;
                if (treeNode != null) {
                    if (treeNode.Rowset == rowset &&
                        treeNode.LineBookmark  == iBmk) {
                        nodeRetVal = treeNode;
                        break;
                    }
                }
                if (bRec) {
                    nodeRetVal = FindNodeUsingRowset(node.Nodes, rowset, iBmk, true);
                    if (nodeRetVal != null) {
                        break;
                    }
                }
            }
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindNodeUsingRowset:        Find a nodes recursively using the rowset and the current line bookmark
        /// </summary>
        /// <param name="rowset">       Rowset to find</param>
        /// <returns>
        /// Node or null if not found
        /// </returns>
        //
        //*********************************************************	    
        public TreeNodeRowset FindNodeUsingRowset(RepPP.Rowset rowset)
        {
            TreeNodeRowset  nodeRetVal = null;
            int             iCurLine;
            IntPtr          iLineBmk;
            
            iCurLine = rowset.CurrentLine;
            if (iCurLine != -1) {
                iLineBmk   = rowset.LineBookmark(rowset.CurrentLine);
                nodeRetVal = FindNodeUsingRowset(Nodes, rowset, iLineBmk, true);
            }
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindNodeUsingFullKey:       Finds a node in a tree using the specified node full key
        /// </summary>
        /// <param name="strFullKey">   Primary key of node tree</param>
        /// <param name="nodeFound">    If not found, the last father which match. If found, the node.</param>
        /// <returns>
        /// true if found, false if not.
        /// </returns>
        //
        //*********************************************************	    
        public virtual bool FindNodeUsingFullKey(string strFullKey, out TreeNodeBase nodeFound)
        {
            bool                bRetVal = true;
            string[]            arrKey;
            TreeNodeCollection  nodes;
            TreeNodeBase        nodeBase;
            
            nodeFound = null;
            arrKey    = strFullKey.Split('\0');
            nodes     = Nodes;
            foreach (string strKey in arrKey) {
                nodeBase = TreeNodeBase.FindNodeUsingKey(nodes, strKey);
                if (nodeBase != null) {
                    nodeFound = nodeBase;
                    nodes     = nodeBase.Nodes;
                    nodeFound.Expand();
                } else {
                    bRetVal = false;
                    break;
                }
            }
            return(bRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ShowCurrentLine:            Select the node corresponding to the specified full key.
        /// </summary>
        /// <param name="strFullKey">   Full key identifying the node.</param>
        /// <returns>
        /// true if the full key has been found, false if only a part of the key has been found
        /// </returns>
        //
        //*********************************************************	    
        public virtual bool ShowCurrentLine(string strFullKey)
        {
            bool            bRetVal;
            TreeNodeBase    nodeBase;
            
            bRetVal = FindNodeUsingFullKey(strFullKey, out nodeBase);
            SelectedNode = nodeBase;
            return(bRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Refresh:                    Refresh the content of the treeview 
        /// </summary>
        /// <param name="strFullKey">   Full key representing the current node position</param>
        //
        //*********************************************************	    
        public virtual void Refresh(string strFullKey)
        {
            Empty();
            FillNodes(m_rowsetChildNode, m_childNodeFactory);
            base.Refresh();
            ShowCurrentLine(strFullKey);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Refresh:            Refresh the content of the treeview 
        /// </summary>
        //
        //*********************************************************	    
        public override void Refresh()
        {
            string      strFullKey;
            
            strFullKey = GetCurrentlySelectedNodeFullKey();
            Refresh(strFullKey);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnMouseMove:        Called when a mouse movement happens on the control
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************	    
        protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
        {
            TreeNodeBase    node;
            
            if (m_toolTip != null) {
                node = GetNodeAt(e.X, e.Y) as TreeNodeBase;
                if (node != null) {
                    if (m_toolTip.GetToolTip(this) != node.Tooltip) {
                        m_toolTip.SetToolTip(this, node.Tooltip);
                    }
                } else {
                    m_toolTip.SetToolTip(this, "");
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeExpand:     Force the filling of the child node before expanding it.
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************	    
        protected override void OnBeforeExpand(System.Windows.Forms.TreeViewCancelEventArgs e)
        {
            TreeNodeBase    nodeBase;

            if (!m_bFilling) {
                nodeBase = e.Node as TreeNodeBase;
                if (nodeBase != null) {
                    if (!nodeBase.IsChildrenRead) {
                        nodeBase.ReadChildren();
                    }
                }
                base.OnBeforeExpand(e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnAfterSelect:      Force the selection of the attached rowset and line 
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************	    
        protected override void OnAfterSelect(System.Windows.Forms.TreeViewEventArgs e)
        {
            TreeNodeBase    treeNode;
            TreeNode        node;
		    
            if (!m_bFilling) {                
                node = e.Node;
                while (node != null && !(node is TreeNodeBase)) {
                    node = node.Parent;
                }
                if (node is TreeNodeBase) {
                    treeNode = (TreeNodeBase)node;
                    treeNode.SelectAttachedObject();
                }
                base.OnAfterSelect(e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeSelect:     Trigger the BeforeSelect event 
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************	
        protected override void OnBeforeSelect(TreeViewCancelEventArgs e) {
            if (!m_bFilling) {
                base.OnBeforeSelect(e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingCount:        Gets the count of node of the same type as the specified one
        /// </summary>
        /// <param name="node">     Node</param>
        //
        //*********************************************************	    
        public int GetSiblingCount(TreeNodeBase node)
        {
            return(node.GetSiblingCount());                    
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingPos:          Gets the position of a node in its same type and sibling one 
        /// </summary>
        /// <param name="node">     Node</param>
        //
        //*********************************************************	    
        public int GetSiblingPos(TreeNodeBase node)
        {
            return(node.GetSiblingPos());
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindSiblingNode:        Find a sibling node using a position
        /// </summary>
        /// <param name="node">     Node</param>
        /// <param name="iPos">     Position (0..n)</param>
        /// <returns>
        /// Sibling node or null if not found
        /// </returns>
        //
        //*********************************************************	    
        public TreeNodeBase FindSiblingNode(TreeNodeBase node, int iPos) {
            return(node.FindSiblingNode(iPos));
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:   Add a new item to the selection buffer
        /// </summary>
        /// <param name="rowsetSB">     Rowset of the selection buffer</param>
        /// <param name="bDuplicate">   true to duplicate, false to insert new one</param>
        //
        //******************************************************************
        public virtual void AddItemToSelectionBuffer(RepPP.Rowset rowsetSB, bool bDuplicate) {
            TreeNodeBase                            nodeNew = null;
            TreeNodeBase                            nodeCurrent;
            TreeNode                                nodeParent;
            TreeNodeBase                            nodeParentBase;
            RepPP.Rowset                            rowsetNode;
            TreeNodeRowset.IRowsetChildNodeFactory  childNodeFactory;
            bool                                    bAddNode;
            IntPtr                                  iLineBmk;

            if (SelectedNode == null ||
                SelectedNode is TreeNodeBase) {
                nodeCurrent = SelectedNode as TreeNodeBase;
                rowsetNode   = (nodeCurrent == null) ? ChildNodeRowset : nodeCurrent.Rowset;
                if (bDuplicate) {
                    foreach(RepPP.Rowset rowsetChild in rowsetNode.ChildRowsets) {
                        rowsetChild.Empty();
                    }
                }
                if (rowsetNode == rowsetSB) {
                    nodeParent          = (nodeCurrent == null) ? null : nodeCurrent.Parent;
                    nodeParentBase      = nodeParent as TreeNodeBase;
                    childNodeFactory    = (nodeParentBase == null) ? m_childNodeFactory : nodeParentBase.ChildNodeFactory;
                    nodeNew             = TreeNodeRowset.CreateNode(this, rowsetNode, rowsetNode.CurrentLine, childNodeFactory);
                    if (nodeNew != null) {
                        if (nodeParent == null) {
                            Nodes.Add(nodeNew);
                        } else {
                            nodeParent.Nodes.Add(nodeNew);
                        }
                    }
                } else {
                    bAddNode = true;
                    if (nodeCurrent != null) {
                        if (nodeCurrent.IsChildrenRead == false) {
                            nodeCurrent.Expand();
                            // Don't add the new node in this case. The expand did it
                            bAddNode   = false;
                            iLineBmk   = rowsetSB.LineBookmark(rowsetSB.CurrentLine);
                            nodeNew    = FindNodeUsingRowset(nodeCurrent.Nodes, rowsetSB, iLineBmk, false);
                        } else {
                            rowsetNode = nodeCurrent.FindChildRowset(rowsetSB.RowsetDef);
                        }
                    } else {
                        rowsetNode = FindChildRowset(rowsetSB.RowsetDef);
                    }
                    if (bAddNode) {
                        if (rowsetNode == rowsetSB) {
                            childNodeFactory = (nodeCurrent == null) ? m_childNodeFactory : nodeCurrent.ChildNodeFactory;
                            nodeNew          = TreeNodeRowset.CreateNode(this, rowsetNode, rowsetNode.CurrentLine, childNodeFactory);
                            if (nodeNew != null) {
                                if (nodeCurrent == null) {
                                    Nodes.Add(nodeNew);
                                } else {
                                    nodeCurrent.Nodes.Add(nodeNew);
                                }
                            }
                        } else {
                            MessageBox.Show("Don't know where to insert the item in the treeview... RowsetDef = '" +
                                            rowsetSB.RowsetDef.Name + "'");
                        }
                    }
                }
                SelectedNode = nodeNew;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh an item
        /// </summary>
        /// <param name="rowsetData">       Rowset use to refresh the data</param>
        //
        //******************************************************************
        public virtual void RefreshItemInSelectionBuffer(RepPP.Rowset rowsetData)
        {
            TreeNodeBase    nodeCurrent;
            RepPP.Rowset    rowset;
            
            nodeCurrent = SelectedNode as TreeNodeBase;
            if (nodeCurrent != null) {
                //nodeCurrent.SetNodeInfo();
                rowset = nodeCurrent.Rowset;
                if (rowset.RowsetDef != rowsetData.RowsetDef) {
                    rowset.RowsetDef.CopyFieldsFrom(rowsetData.RowsetDef, RepPP.CopyFieldOpt.sdCopyState);
                }
                nodeCurrent.SetNodeInfo();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete an item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to select another row</param>
        //
        //******************************************************************
        public virtual void DeleteItemFromSelectionBuffer(bool bReselect)
        {
            TreeNode            node;
            TreeNodeCollection  nodes;
            int                 iIndex;
            RepPP.Rowset        rowset;
            
            node = SelectedNode;
            if (node is TreeNodeBase) {
                if (node is TreeNodeRowset) {
                    rowset = ((TreeNodeRowset)node).Rowset;
                    // delete the current line in the selection buffer rowset
                    if (rowset != null) {
                        rowset.LineState = RepPP.LineState.sdLineNew;
                        rowset.DeleteLine();
                    }
                }
                nodes      = (node.Parent == null) ? Nodes : node.Parent.Nodes;
                iIndex     = node.Index;
                m_bFilling = true;
                node.Remove();
                m_bFilling = false;
                if (bReselect) {
                    if (iIndex >= nodes.Count) {
                        iIndex = nodes.Count - 1;
                    }
                    if (iIndex != -1) {
                        SelectedNode = null;
                        SelectedNode = nodes[iIndex];
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetItemCount:                   Gets the number of item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetItemCount()
        {
            int             iRetVal;
            TreeNode        node;
            TreeNodeBase    nodeCurrent;

            node        = SelectedNode;
            nodeCurrent = node as TreeNodeBase;
            iRetVal     = (nodeCurrent != null) ? GetSiblingCount(nodeCurrent) : 0;
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItem:                 Gets the current item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetCurrentItem()
        {
            int                 iRetVal;
            TreeNode            node;
            TreeNodeBase        nodeCurrent;

            node        = SelectedNode;
            nodeCurrent = node as TreeNodeBase;
            iRetVal     = (nodeCurrent != null) ? GetSiblingPos(nodeCurrent) : -1;
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select the specified item
        /// </summary>
        //
        //******************************************************************
        public virtual bool SelectItem(int iPos)
        {
            bool            bRetVal;
            TreeNode        node;
            TreeNodeBase    nodeCurrent;
            TreeNodeBase    nodeNew;

            if (iPos < 0) {
                bRetVal = false;
            } else {
                node        = SelectedNode;
                nodeCurrent = node as TreeNodeBase;
                if (nodeCurrent != null) {
                    nodeNew = FindSiblingNode(nodeCurrent, iPos);
                    if (nodeNew == null || nodeNew == nodeCurrent) {
                        bRetVal = false;
                    } else {
                        SelectedNode = nodeNew;
                        bRetVal      = true;
                    }
                } else {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ConfigurationHelper:        Returns the configuration helper used to load the UI config
        /// </summary>
        //
        //******************************************************************
        public UIConfigurationHelper ConfigurationHelper {
            get {
                return(m_configHelper);
            }
            set {
                m_configHelper = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// UIConfigurationNode:        Node containing the treeview configuration
        /// </summary>
        //
        //******************************************************************
        public XmlNode UIConfigurationNode {
            get {
                return(m_nodeSerialization);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadState               Load the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        /// <param name="nodes">    Collection of nodes to load</param>
        //
        //******************************************************************
        private void LoadState(UIConfigurationHelper helper, System.Xml.XmlNode node, TreeNodeCollection nodes) {
            TreeNodeBase    nodeBase;
            
            foreach (TreeNode treeNode in nodes) {
                nodeBase = treeNode as TreeNodeBase;
                if (nodeBase != null) {
                    nodeBase.LoadState(helper, node);
                }
                LoadState(helper, node, treeNode.Nodes);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadState               Load the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public void LoadState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            m_configHelper      = helper;
            m_nodeSerialization = node.Clone();
            LoadState(helper, node, Nodes);
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveState               Save the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        /// <param name="nodes">    Collection of nodes to load</param>
        //
        //******************************************************************
        private void SaveState(UIConfigurationHelper helper, System.Xml.XmlNode node, TreeNodeCollection nodes) {
            TreeNodeBase    nodeBase;
            
            foreach (TreeNode treeNode in nodes) {
                nodeBase = treeNode as TreeNodeBase;
                if (nodeBase != null) {
                    nodeBase.SaveState(helper, node);
                }
                SaveState(helper, node, treeNode.Nodes);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveState               Save the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public void SaveState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            if (m_nodeSerialization != null) {
                node.InnerXml = m_nodeSerialization.InnerXml;
            }
            SaveState(helper, node, Nodes);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAfterNodeMoved        Send the AfterNodeMoved event
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        protected virtual void OnAfterNodeMoved(AfterNodeMovedEventArgs e) {
            if (AfterNodeMoved != null) {
                AfterNodeMoved(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnBeforeNodeMoved       Send the BeforeNodeMoved event
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        protected virtual void OnBeforeNodeMoved(BeforeNodeMovedEventArgs e) {
            if (BeforeNodeMoved != null) {
                BeforeNodeMoved(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAfterNodeCopied       Send the AfterNodeCopied event
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        protected virtual void OnAfterNodeCopied(AfterNodeCopiedEventArgs e) {
            if (AfterNodeCopied != null) {
                AfterNodeCopied(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnBeforeNodeCopied      Send the BeforeNodeCopied event
        /// </summary>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        protected virtual void OnBeforeNodeCopied(BeforeNodeCopiedEventArgs e) {
            if (BeforeNodeCopied != null) {
                BeforeNodeCopied(this, e);
            }
        }
        


        //******************************************************************
        //
        /// <summary>
        /// CanDropThere:                   Checks if the a drop can occurs
        /// </summary>
        /// <param name="nodeToDrop">       Node to drop</param>
        /// <param name="nodeNewFather">    Destination node</param>
        /// <returns>
        /// NoDrop                          Drop not allowed
        /// RootDrop                        Drop allowed. Drop an item to another item of the same type (recursive drop like folder to a new folder)
        /// ChildDrop                       Move a child to a new father (file to a new folder)
        /// </returns>
        //
        //******************************************************************
        public virtual AllowedDropMode CanDropThere(TreeNode nodeToDrop, TreeNode nodeNewFather) {
            AllowedDropMode eRetVal;
            TreeNodeRowset  nodeRowsetToDrop;
            TreeNodeRowset  nodeRowsetNewFather;
            Rowset          rowsetToDrop;
            Rowset          rowsetNewFather;
            RowsetTree      rowsetTreeToDrop;
            RowsetTree      rowsetTreeNewFather;

            nodeRowsetToDrop = nodeToDrop as TreeNodeRowset;
            if (nodeRowsetToDrop == null) {
                eRetVal  = AllowedDropMode.NoDrop;
                // Cannot move to the same parent node or if moving to one of the node child
            } else if (nodeRowsetToDrop.Parent == nodeNewFather || nodeRowsetToDrop == nodeNewFather || nodeRowsetToDrop.IsNodeAncestorOf(nodeNewFather)) {
                eRetVal  = AllowedDropMode.NoDrop;
            } else {
                rowsetToDrop     = nodeRowsetToDrop.Rowset;
                rowsetTreeToDrop = (rowsetToDrop == null) ? null : rowsetToDrop.RowsetTree;
                // Node must be a TreeViewNodeGInst and attached to a rowset which is part of a rowset tree
                if (rowsetTreeToDrop == null) {
                    eRetVal      = AllowedDropMode.NoDrop;
                } else if (nodeNewFather == null) {
                    rowsetNewFather     = ChildNodeRowset;
                    rowsetTreeNewFather = (rowsetNewFather == null) ? null : rowsetNewFather.RowsetTree;
                    // Treeview must be attached to the root rowset of a rowset tree
                    // The node to move must be a root of the same card.
                    if (nodeRowsetToDrop.IsRecursive                            &&
                        rowsetTreeNewFather                   != null           &&
                        rowsetTreeNewFather.RootRowset      == rowsetNewFather  &&
                        rowsetNewFather.RowsetTreeDefNode   == rowsetToDrop.RowsetTreeDefNode) {
                        eRetVal = AllowedDropMode.RootDrop;
                    } else {
                        eRetVal = AllowedDropMode.NoDrop;
                    }
                } else {
                    nodeRowsetNewFather     = nodeNewFather as TreeNodeRowset;
                    rowsetNewFather         = (nodeRowsetNewFather == null) ? null : nodeRowsetNewFather.Rowset;
                    rowsetTreeNewFather     = (rowsetNewFather == null) ? null : rowsetNewFather.RowsetTree;
                    // Node must be a TreeViewNodeGInst and attached to a rowset part of a rowset tree
                    if (rowsetTreeNewFather == null) {
                        eRetVal = AllowedDropMode.NoDrop;
                    } else {
                        // If moving to a node attached to the our parent card group
                        if (rowsetToDrop.RowsetTreeDefNode.Parent == rowsetNewFather.RowsetTreeDefNode) {
                            eRetVal = AllowedDropMode.ChildDrop;
                        } else {
                            if (nodeRowsetNewFather.IsRecursive                 && 
                                rowsetTreeToDrop.RootRowset    == rowsetToDrop  &&
                                rowsetToDrop.RowsetTreeDefNode == rowsetNewFather.RowsetTreeDefNode) {
                                eRetVal = AllowedDropMode.RootDrop;
                            } else {
                                eRetVal = AllowedDropMode.NoDrop;
                            }
                        }
                    }
                }
            }
            return(eRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// DropNode:                       Drop a node to a new location
        /// </summary>
        /// <param name="nodeToDrop">       Node to drop</param>
        /// <param name="nodeNewFather">    Destination node</param>
        /// <param name="bCopy">            true to copy, false to move</param>
        /// <returns>
        /// NoDrop                          Event cancel the operation
        /// RootDrop                        Drop allowed. Drop an item to another item of the same type (recursive drop like folder to a new folder)
        /// ChildDrop                       Move a child to a new father (file to a new folder)
        /// </returns>
        //
        //******************************************************************
        public virtual AllowedDropMode DropNode(TreeNodeRowset nodeToDrop, TreeNodeRowset nodeNewFather, bool bCopy) {
            bool                        bCancel;
            AllowedDropMode             eDropMode;
            Rowset                      rowsetToDrop;
            Rowset                      rowsetNewFather;
            Rowset                      rowsetNew;
            TreeNodeRowset              nodeOri;
            TreeNode                    nodeOldFather;
            int                         iLine;
            AfterNodeMovedEventArgs     afterNodeMovedEventArgs;
            BeforeNodeMovedEventArgs    beforeNodeMovedEventArgs;
            AfterNodeCopiedEventArgs    afterNodeCopiedEventArgs;
            BeforeNodeCopiedEventArgs   beforeNodeCopiedEventArgs;
            
            BeginUpdate();
            try {
                if (bCopy) {
                    beforeNodeCopiedEventArgs = new BeforeNodeCopiedEventArgs(nodeToDrop, nodeNewFather);
                    OnBeforeNodeCopied(beforeNodeCopiedEventArgs);
                    bCancel = beforeNodeCopiedEventArgs.Cancel;
                } else {
                    beforeNodeMovedEventArgs = new BeforeNodeMovedEventArgs(nodeToDrop, nodeNewFather);
                    OnBeforeNodeMoved(beforeNodeMovedEventArgs);
                    bCancel = beforeNodeMovedEventArgs.Cancel;
                }
                if (bCancel) {
                    eDropMode = AllowedDropMode.NoDrop;
                } else {
                    rowsetToDrop  = nodeToDrop.Rowset;
                    eDropMode     = CanDropThere(nodeToDrop, nodeNewFather);
                    switch(eDropMode) {
                    case AllowedDropMode.ChildDrop:
                        rowsetNewFather = nodeNewFather.Rowset;
                        rowsetNew       = rowsetNewFather.GetChild(rowsetToDrop.RowsetTreeDefNode.ChildId, rowsetNewFather.LineFromBookmark(nodeNewFather.LineBookmark), true, false);
                        break;
                    case AllowedDropMode.RootDrop:
                        if (nodeNewFather == null) {
                            rowsetNew = ChildNodeRowset;
                        } else {
                            nodeNewFather.Expand();
                            rowsetNew = nodeNewFather.RecRowsetTree.RootRowset;
                        }
                        break;
                    default:
                        throw new System.ApplicationException("Drop is not allowed on this node");
                    }
                    rowsetNew.RowsetDef.SaveActiveRowset();
                    rowsetNew.SaveLinePos();
                    try {
                        nodeOri       = nodeToDrop;
                        nodeOldFather = nodeOri.Parent;
                        iLine         = rowsetToDrop.LineFromBookmark(nodeToDrop.LineBookmark);
                        rowsetToDrop.SelectLine(iLine, false, false);
                        rowsetNew.AddBlankLine(true);
                        rowsetNew.CopyLineFrom(rowsetToDrop, CopyFieldOpt.sdCopyState, true, false);
                        rowsetNew.LineState = LineState.sdLineChanged;
                        if (bCopy) {
                            nodeToDrop = nodeToDrop.Clone() as TreeNodeRowset;
                            nodeToDrop.ReassociateGInst(rowsetNew, rowsetNew.CurrentLine);
                        } else {
                            nodeToDrop.ReassociateGInst(rowsetNew, rowsetNew.CurrentLine);
                            rowsetToDrop.RemoveLine();
                            nodeToDrop.Remove();
                        }
                        if (nodeNewFather == null) {
                            Nodes.Add(nodeToDrop);
                        } else {
                            nodeNewFather.Nodes.Add(nodeToDrop);
                        }
                    } finally {
                        rowsetNew.RestoreLinePos(false);
                        rowsetNew.RowsetDef.RestoreActiveRowset(true);
                    }
                    if (bCopy) {
                        afterNodeCopiedEventArgs = new AfterNodeCopiedEventArgs(nodeOri, nodeToDrop);
                        OnAfterNodeCopied(afterNodeCopiedEventArgs);
                    } else {
                        afterNodeMovedEventArgs = new AfterNodeMovedEventArgs(nodeToDrop, nodeOldFather);
                        OnAfterNodeMoved(afterNodeMovedEventArgs);
                    }
                }
            } finally {
                EndUpdate();
            }
            return(eDropMode);
        }

        //******************************************************************
        //
        /// <summary>
        /// DragOnTarget:                   Returns if the drag is on target
        /// </summary>
        //
        //******************************************************************
        protected virtual bool DragOnTarget {
            get {
                return(m_bDragOnTarget);
            }
            set {
                m_bDragOnTarget = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AutoDragDrop:   Indicates if the drag and Drop is managed automatically
        /// </summary>
        //
        //******************************************************************
        [Description("Indicates whether or not the drag & Drop is managed automatically"),
        DefaultValue(false),
        Category("Behavior")]
        public bool AutoDragDrop {
            get {
                return(m_bAutoDragDrop);
            }
            set {
                m_bAutoDragDrop = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnItemDrag:                     Called to start the drag operation on an item
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //******************************************************************
        protected override void OnItemDrag(ItemDragEventArgs e) {
            base.OnItemDrag(e);
            if (AutoDragDrop) {
                DragOnTarget  = false;
                if (e.Button == MouseButtons.Left) {
                    DoDragDrop(e.Item, DragDropEffects.Move);
                } else if (e.Button == MouseButtons.Right) {
                    DoDragDrop(e.Item, DragDropEffects.Copy);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragEnter:                    Called when the mouse enter the window of the treeview window and drag is on
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //******************************************************************
        protected override void OnDragEnter(DragEventArgs e) {
            base.OnDragEnter(e);
            if (AutoDragDrop) {
                e.Effect = e.AllowedEffect;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragOver:                     Called when the mouse move on the treeview window and drag is on
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //******************************************************************
        protected override void OnDragOver(DragEventArgs e) {
            Point           ptTarget;
            TreeNode        nodeTarget;
            TreeNodeRowset  nodeDragged;
            
            base.OnDragOver(e);
            if (AutoDragDrop) {
                ptTarget    = PointToClient(new Point(e.X, e.Y));
                nodeTarget  = GetNodeAt(ptTarget);
                nodeDragged = (TreeNodeRowset)e.Data.GetData(typeof(TreeNodeRowset));
                if (nodeDragged != null && CanDropThere(nodeDragged, nodeTarget) != AllowedDropMode.NoDrop) {
                    DragOnTarget = true;
                } else {
                    DragOnTarget = false;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnGiveFeedback:                 Give feedback on a drag operation
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //******************************************************************
        protected override void OnGiveFeedback(GiveFeedbackEventArgs e) {
            base.OnGiveFeedback(e);
            if (AutoDragDrop) {
                if (DragOnTarget) {
                    e.UseDefaultCursors = true;
                } else {
                    e.UseDefaultCursors = false;
                    Cursor.Current      = Cursors.No;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDragDrop:                     Called when the drag occurs
        /// </summary>
        /// <param name="e">                Event argument</param>
        //
        //******************************************************************
        protected override void OnDragDrop(DragEventArgs e) {
            Point           ptTarget;
            TreeNode        nodeTarget;
            TreeNodeRowset  nodeDragged;
            AllowedDropMode eDropMode;
                
 	        base.OnDragDrop(e);
            if (AutoDragDrop) {
                ptTarget    = PointToClient(new Point(e.X, e.Y));
                nodeTarget  = GetNodeAt(ptTarget);
                nodeDragged = (TreeNodeRowset)e.Data.GetData(typeof(TreeNodeRowset));
                if (nodeDragged != null) {
                    if (CanDropThere(nodeDragged, nodeTarget) != AllowedDropMode.NoDrop) {
                        eDropMode = DropNode(nodeDragged as TreeNodeRowset, nodeTarget as TreeNodeRowset, e.Effect == DragDropEffects.Copy);
                        if (eDropMode != AllowedDropMode.NoDrop && nodeTarget != null) {
                            nodeTarget.Expand();
                        }
                    }
                }
            }
        }
 
        //******************************************************************
        //
        /// <summary>
        /// BeforeChildReadEventArgs:   Arguments for BeforeChildRead event
        /// </summary>
        //
        //******************************************************************
        public class BeforeChildReadEventArgs : System.ComponentModel.CancelEventArgs {
            /// <summary>
            /// Node of the event
            /// </summary>
            private TreeNodeBase    m_node;

            //******************************************************************
            //
            /// <summary>
            /// BeforeChildReadEventArgs    Class ctor
            /// </summary>
            /// <param name="node">         Node to attach to the event</param>
            //
            //******************************************************************
            public BeforeChildReadEventArgs(TreeNodeBase node) {
                m_node = node;
            }

            //******************************************************************
            //
            /// <summary>
            /// Node:                       Node attached to the event
            /// </summary>
            //
            //******************************************************************
            public TreeNodeBase Node {
                get {
                    return(m_node);
                }
            }
        };

        //******************************************************************
        //
        /// <summary>
        /// AfterChildReadEventArgs:    Arguments for AfterChildRead event
        /// </summary>
        //
        //******************************************************************
        public class AfterChildReadEventArgs : System.EventArgs {
            /// <summary>
            /// Node of the event
            /// </summary>
            private TreeNodeBase    m_node;

            //******************************************************************
            //
            /// <summary>
            /// AfterChildReadEventArgs     Class ctor
            /// </summary>
            /// <param name="node">         Node to attach to the event</param>
            //
            //******************************************************************
            public AfterChildReadEventArgs(TreeNodeBase node) {
                m_node = node;
            }

            //******************************************************************
            //
            /// <summary>
            /// Node:                       Node attached to the event
            /// </summary>
            //
            //******************************************************************
            public TreeNodeBase Node {
                get {
                    return(m_node);
                }
            }
        } // Class TreeNodeBase
        
        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the BeforNodeCreated event.
        /// </summary>
        //
        //*********************************************************	    
        public class BeforeNodeCreatedEventArgs : System.EventArgs {
            /// <summary>
            /// Rowset of the node
            /// </summary>
            private Rowset              m_rowset;
            /// <summary>
            /// Line index in the rowset
            /// </summary>
            private int                 m_iLineIndex;
            /// <summary>
            /// indicates whether or not to cancel the creation of the node
            /// </summary>
            private bool                m_bCancel;

            //******************************************************************
            //
            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="rowset">            Group isntance of the node</param>
            /// <param name="iLineIndex">       Line index in the rowset</param>
            //
            //******************************************************************
            public BeforeNodeCreatedEventArgs(Rowset rowset, int iLineIndex) {
                m_rowset        = rowset;
                m_iLineIndex    = iLineIndex;
                m_bCancel       = false;
            }

            //******************************************************************
            //
            /// <summary>
            /// Rowset:          Rowset of the node
            /// </summary>
            //
            //******************************************************************
            public Rowset Rowset {
                get {
                    return(m_rowset);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// LineIndex:              Line index in the rowset of the node
            /// </summary>
            //
            //******************************************************************
            public int LineIndex {
                get {
                    return(m_iLineIndex);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Cancel:              Indicates whether or not to cancel the creation
            /// </summary>
            //
            //******************************************************************
            public bool Cancel {
                get {
                    return(m_bCancel);
                }
                set {
                    m_bCancel = value;
                }
            }
        } // Class BeforeNodeCreatedEventArgs

        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the AfterNodeCreated event.
        /// </summary>
        //
        //*********************************************************	    
        public class AfterNodeCreatedEventArgs : System.EventArgs {
            /// <summary>
            /// The new created node 
            /// </summary>
            private TreeNodeRowset        m_node;

            //******************************************************************
            //
            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="node">         Node to attach to the event</param>
            //
            //******************************************************************
            public AfterNodeCreatedEventArgs(TreeNodeRowset node) {
                m_node = node;
            }

            //******************************************************************
            //
            /// <summary>
            /// Node:                       Node attached to the event
            /// </summary>
            //
            //******************************************************************
            public TreeNodeRowset Node {
                get {
                    return(m_node);
                }
            }
        } // Class AfterNodeCreatedEventArgs

        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the SetNodeInfo event.
        /// </summary>
        //
        //*********************************************************	    
        public class SetNodeInfoEventArgs : System.EventArgs {
            /// <summary>
            /// Node to set the info
            /// </summary>
            private TreeNodeBase        m_node;

            //******************************************************************
            //
            /// <summary>
            /// SetNodeInfoEventArgs        Class ctor
            /// </summary>
            /// <param name="node">         Node to attach to the event</param>
            //
            //******************************************************************
            public SetNodeInfoEventArgs(TreeNodeBase node) {
                m_node = node;
            }

            //******************************************************************
            //
            /// <summary>
            /// Node:                       Node attached to the event
            /// </summary>
            //
            //******************************************************************
            public TreeNodeBase Node {
                get {
                    return(m_node);
                }
            }
        } // Class SetNodeInfoEventArgs

        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the AfterNodeMoved event
        /// </summary>
        //
        //*********************************************************	    
        public class AfterNodeMovedEventArgs : System.EventArgs {
            /// <summary>Old father node</summary>
            public TreeNode     OldFatherNode;
            /// <summary>Node being moved</summary>
            public TreeNode     MovedNode;

            //******************************************************************
            //
            /// <summary>
            /// AfterNodeMovedEventArgs         Class ctor
            /// </summary>
            /// <param name="nodeMoved">        Node being moved</param>
            /// <param name="nodeOldFather">    Node receiving the moving one</param>
            //
            //******************************************************************
            public AfterNodeMovedEventArgs(TreeNode nodeMoved, TreeNode nodeOldFather) {
                OldFatherNode   = nodeOldFather;
                MovedNode       = nodeMoved;
            }
        } // Class AfterNodeMovedEventArgs

        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the BeforeNodeMoved event
        /// </summary>
        //
        //*********************************************************	    
        public class BeforeNodeMovedEventArgs {
            /// <summary>Node receiving the moving one</summary>
            public TreeNode     NewFatherNode;
            /// <summary>Node being moved</summary>
            public TreeNode     MovedNode;
            /// <summary>true to cancel the operation</summary>
            public bool         Cancel;

            //******************************************************************
            //
            /// <summary>
            /// BeforeNodeMovedEventArgs        Class ctor
            /// </summary>
            /// <param name="nodeMoved">        Node being moved</param>
            /// <param name="nodeNewFather">    Node receiving the moving one</param>
            /// 
            //******************************************************************
            public BeforeNodeMovedEventArgs(TreeNode nodeMoved, TreeNode nodeNewFather) {
                MovedNode       = nodeMoved;
                NewFatherNode   = nodeNewFather;
                Cancel          = false;
            }
        } // Class BeforeNodeMovedEventArgs

                
        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the AfterNodeCopied event
        /// </summary>
        //
        //*********************************************************	    
        public class AfterNodeCopiedEventArgs : System.EventArgs {
            /// <summary>Node receiving the moving one</summary>
            public TreeNode     OriginalNode;
            /// <summary>Node being moved</summary>
            public TreeNode     NewNode;

            //******************************************************************
            //
            /// <summary>
            /// AfterNodeCopiedEventArgs        Class ctor
            /// </summary>
            /// <param name="nodeOri">          Original node</param>
            /// <param name="nodeNew">          Node being moved</param>
            //
            //******************************************************************
            public AfterNodeCopiedEventArgs(TreeNode nodeOri, TreeNode nodeNew) {
                OriginalNode    = nodeOri;
                NewNode         = nodeNew;
            }
        } // Class AfterNodeCopiedEventArgs
                
        //*********************************************************	    
        //
        /// <summary>
        /// Implement arguments for the BeforeNodeCopied event
        /// </summary>
        //
        //*********************************************************	    
        public class BeforeNodeCopiedEventArgs : System.EventArgs {
            /// <summary>Node receiving the moving one</summary>
            public TreeNode     OriginalNode;
            /// <summary>Node receiving the moving one</summary>
            public TreeNode     NewFatherNode;
            /// <summary>true to cancel the operation</summary>
            public bool         Cancel;

            //******************************************************************
            //
            /// <summary>
            /// BeforeNodeCopiedEventArgs       Class ctor
            /// </summary>
            /// <param name="nodeOri">          Original node</param>
            /// <param name="nodeNewFather">    New father node</param>
            //
            //******************************************************************
            public BeforeNodeCopiedEventArgs(TreeNode nodeOri, TreeNode nodeNewFather) {
                OriginalNode    = nodeOri;
                NewFatherNode   = nodeNewFather;
                Cancel          = false;
            }
        } // Class BeforeNodeCopiedEventArgs
    } // Class TreeViewRowsetTree
} // Namespace RepPPToolkit
