using System;
using System.Drawing;
using System.Collections;
using System.Xml;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    /// <summary>
    /// Define the interface implement by objects which can auto-save their UI configuration
    /// </summary>
	public interface IUIConfiguration {
        //*********************************************************
        //
        /// <summary>
        /// LoadState:                  Load the state of the UI item.
        /// </summary>
        /// <param name="helper">       Helper object</param>
        /// <param name="node">         Node containing the information</param>
	    //
        //*********************************************************
	    void    LoadState(UIConfigurationHelper helper, XmlNode node);
        //*********************************************************
        //
        /// <summary>
        /// SaveState:                  Save the state of the UI item.
        /// </summary>
        /// <param name="helper">       Helper object</param>
        /// <param name="node">         Node where to save the information</param>
	    //
        //*********************************************************
	    void    SaveState(UIConfigurationHelper helper, XmlNode node);
	}; // End interface
	
	/// <summary>
	/// UI Configuration class helper
	/// </summary>
	public class UIConfigurationHelper {
	    private class RegistredForm {
            public  Point   m_ptsNorLocation;   // Form location when normal
            public  Size	m_sizeNorSize;      // Form size when normal
	    };
	    
	    private string      m_strFileName;          // Filename containing the configuration
        private bool        m_bDoNotLoadWindowState;// Flag that prevents setting the window state
	    private XmlDocument m_xmlDoc;               // XML document containing the configuration
	    private Form        m_frmFather;            // Father form
	    private Hashtable   m_hashRegistredForm;    // List of registred form

        //*********************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    //
        //*********************************************************
        private UIConfigurationHelper(string strFileName, System.IO.TextReader textReader) {
            XmlNode node;

            m_strFileName           = strFileName;
            m_bDoNotLoadWindowState = false;
            m_frmFather             = null;
            m_hashRegistredForm     = new Hashtable();
            m_xmlDoc                = new System.Xml.XmlDocument();
            if (textReader != null) {
                LoadFromStream(textReader);
            }
            if (strFileName != null && strFileName != "") {
                LoadFromFile();
            }
            if (m_xmlDoc.ChildNodes.Count == 0) {
				node = m_xmlDoc.CreateNode(XmlNodeType.Element, "REP", "");
                if (node != null) {
					m_xmlDoc.AppendChild(node);
				}
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    /// <param name="strFileName">  File name associated with the configuration</param>
	    //
        //*********************************************************
        public UIConfigurationHelper(string strFileName) : this(strFileName, null) {
        }

        //*********************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    /// <param name="reader">       Text reader</param>
	    //
        //*********************************************************
        public UIConfigurationHelper(System.IO.TextReader reader) : this(null, reader) {
        }

        //*********************************************************
        //
        /// <summary>
        /// DoNotLoadWindowState:        Flag indicating whether the
        ///                              Configuration Helper loads
        ///                              the window state or not
        /// </summary>
        /// <remarks>
        /// Setting the WindowState will force the display of the 
        /// form and may result in flikering while the form is 
        /// being resized.
        /// </remarks>
        //
        //*********************************************************
        public bool DoNotLoadWindowState {
            get {
                return(m_bDoNotLoadWindowState);
            }
            set {
                m_bDoNotLoadWindowState = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// XmlDocument:                XML document containing the configuration
        /// </summary>
        //
        //*********************************************************
        public XmlDocument XmlDocument {
            get {
                return(m_xmlDoc);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AttachedForm:               Form attached to the UI configuration
        /// </summary>
        //
        //*********************************************************
        public Form AttachedForm {
            get {
                return(m_frmFather);
            }
            set {
                if (m_frmFather != value) {
                    if (m_frmFather != null) {
                        m_frmFather.Closed -= new System.EventHandler(m_frmFather_Closed);
                    }
                    m_frmFather = value;
                    if (m_frmFather != null) {
                        m_frmFather.Closed += new System.EventHandler(m_frmFather_Closed);
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RegisterForm:               Register a form
        /// </summary>
        /// <param name="frm">          Form to register</param>
        //
        //*********************************************************
        public virtual void RegisterForm(Form frm) {
            RegistredForm   regForm;
            
            regForm                  = new RegistredForm();
            regForm.m_ptsNorLocation = frm.Location;
            regForm.m_sizeNorSize    = frm.Size;
            frm.Load                += new EventHandler(RegistredForm_Load);
            frm.Closing             += new System.ComponentModel.CancelEventHandler(RegistredForm_Closing);
            frm.Closed              += new EventHandler(RegistredForm_Closed);
            frm.LocationChanged     += new EventHandler(RegistredForm_PosSizeChanged);
            frm.Resize              += new EventHandler(RegistredForm_PosSizeChanged);
            m_hashRegistredForm.Add(frm, regForm);
        }

        //*********************************************************
        //
        /// <summary>
        /// UnregisterForm:             Unregister a form
        /// </summary>
        //
        //*********************************************************
        protected virtual void UnregisterForm(Form frm) {
            m_hashRegistredForm.Remove(frm);
            frm.Load                -= new EventHandler(RegistredForm_Load);
            frm.Closing             -= new System.ComponentModel.CancelEventHandler(RegistredForm_Closing);
            frm.Closed              -= new EventHandler(RegistredForm_Closed);
            frm.LocationChanged     -= new EventHandler(RegistredForm_PosSizeChanged);
            frm.Resize              -= new EventHandler(RegistredForm_PosSizeChanged);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFromFile:               Load the configuration from the file
	    /// </summary>
	    //
        //*********************************************************
        private void LoadFromFile() {
            try {
                if (System.IO.File.Exists(m_strFileName)) {
                    m_xmlDoc.Load(m_strFileName);
                }
            } catch(System.Exception) {
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFromStream:             Load the configuration from a stream
	    /// </summary>
        /// <param name="reader">       Reader</param>
	    //
        //*********************************************************
        private void LoadFromStream(System.IO.TextReader reader) {
            try {
                m_xmlDoc.Load(reader);
            } catch(System.Exception) {
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SaveToFile:                 Save the configuration to the file
	    /// </summary>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
	    //
        //*********************************************************
        public bool SaveToFile() {
            bool    bRetVal = true;
            
            try {
                m_xmlDoc.Save(m_strFileName);
            } catch(System.Exception) {
                bRetVal  = false;
            }
            return(bRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SaveToWriter:               Save the configuration to a writer
	    /// </summary>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
	    //
        //*********************************************************
        public bool SaveToWriter(System.IO.TextWriter writer) {
            bool    bRetVal = true;
            
            try {
                m_xmlDoc.Save(writer);
            } catch(System.Exception) {
                bRetVal  = false;
            }
            return(bRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// GetNodeAttribute:           Gets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
        /// <returns>
        /// Attribute or null if not found
        /// </returns>
	    //
        //*********************************************************
        public string GetNodeAttribute(XmlNode node, string strName) {
            string      strRetVal = null;
            
            strName = strName.ToUpper();
            foreach (XmlAttribute attr in node.Attributes) {
                if (attr.Name.ToUpper() == strName) {
                    strRetVal = attr.Value;
                    break;
                }
            }
            return(strRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// GetNodeAttribute:           Gets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
	    /// <param name="bDefaultValue">Default value</param>
        /// <returns>
        /// Attribute or default value if not found
        /// </returns>
	    //
        //*********************************************************
        public bool GetNodeAttribute(XmlNode node, string strName, bool bDefaultValue) {
            bool    bRetVal = bDefaultValue;
            string  str;
            
            str = GetNodeAttribute(node, strName);
            if (str != null && str.Length != 0) {
                try {
                    bRetVal = (Int32.Parse(str) != 0);
                } catch {
                }
            }
            return(bRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// GetNodeAttribute:           Gets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
	    /// <param name="iDefaultValue">Default value</param>
        /// <returns>
        /// Attribute or default value if not found
        /// </returns>
	    //
        //*********************************************************
        public int GetNodeAttribute(XmlNode node, string strName, int iDefaultValue) {
            int     iRetVal = iDefaultValue;
            string  str;
            
            str = GetNodeAttribute(node, strName);
            if (str != null && str.Length != 0) {
                try {
                    iRetVal = Int32.Parse(str);
                } catch {
                }
            }
            return(iRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SetNodeAttribute:           Sets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
	    /// <param name="strVal">       Attribute value to set</param>
	    //
        //*********************************************************
        public void SetNodeAttribute(XmlNode node, string strName, string strVal) {
            bool            bFound = false;
            XmlAttribute    attrNew;
            
            strName = strName.ToUpper();
            foreach (XmlAttribute attr in node.Attributes) {
                if (attr.Name.ToUpper() == strName) {
                    attr.Value = strVal;
                    bFound     = true;
                    break;
                }
            }
            if (!bFound) {
                attrNew       = m_xmlDoc.CreateAttribute(strName);
                attrNew.Value = strVal;
                node.Attributes.Append(attrNew);
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SetNodeAttribute:           Sets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
	    /// <param name="bVal">         Attribute value to set</param>
	    //
        //*********************************************************
        public void SetNodeAttribute(XmlNode node, string strName, bool bVal) {
            SetNodeAttribute(node, strName, bVal ? "1" : "0");
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SetNodeAttribute:           Sets a node attribute
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Attribute name to find</param>
	    /// <param name="iVal">         Attribute value to set</param>
	    //
        //*********************************************************
        public void SetNodeAttribute(XmlNode node, string strName, int iVal) {
            SetNodeAttribute(node, strName, iVal.ToString());
        }

        //*********************************************************
        //
	    /// <summary>
	    /// FindNode:                   Find a node
	    /// </summary>
	    /// <param name="node">         Node in which to search the child. null for root</param>
	    /// <param name="strName">      Node name to find</param>
        /// <returns>
        /// Node or null if document is not loaded.
        /// </returns>
	    //
        //*********************************************************
        public XmlNode FindNode(XmlNode node, string strName) {
            XmlNode     nodeRetVal = null;
            XmlNodeList nodeList;
            
            if (m_xmlDoc != null) {
                strName  = strName.ToUpper();
                nodeList = (node == null) ? m_xmlDoc.DocumentElement.ChildNodes : node.ChildNodes;
                foreach (XmlNode nodeChild in nodeList) {
                    if (nodeChild.Name.ToUpper() == strName) {
                        nodeRetVal = nodeChild;
                        break;
                    }
                }
                if (nodeRetVal == null) {
                    nodeRetVal = m_xmlDoc.CreateNode(XmlNodeType.Element, strName, "");
                    if (node == null) {
                        m_xmlDoc.DocumentElement.AppendChild(nodeRetVal);
                    } else {
                        node.AppendChild(nodeRetVal);
                    }
                }
            }
            return(nodeRetVal);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// FindNodeUsingAttribute:             Find a node using an attribute value
	    /// </summary>
	    /// <param name="node">                 Node in which to search the child. null for root</param>
	    /// <param name="strName">              Node name to find</param>
	    /// <param name="strAttrName">          Attribute name to find</param>
	    /// <param name="strAttrValue">         Attribute value</param>
	    /// <param name="bCreateIfNotFound">    true to create if not found</param>
        /// <returns>
        /// Node or null if document is not loaded.
        /// </returns>
	    //
        //*********************************************************
        public XmlNode FindNodeUsingAttribute(XmlNode   node, 
                                              string    strName, 
                                              string    strAttrName,
                                              string    strAttrValue, 
                                              bool      bCreateIfNotFound) {
            XmlNode     nodeRetVal = null;
            XmlNodeList nodeList;
            
            if (m_xmlDoc != null) {
                strName  = strName.ToUpper();
                nodeList = (node == null) ? m_xmlDoc.DocumentElement.ChildNodes : node.ChildNodes;
                foreach (XmlNode nodeChild in nodeList) {
                    if (nodeChild.Name.ToUpper() == strName &&
                        GetNodeAttribute(nodeChild, strAttrName) == strAttrValue) {
                        nodeRetVal = nodeChild;
                        break;
                    }
                }
                if (nodeRetVal == null && bCreateIfNotFound) {
                    nodeRetVal = m_xmlDoc.CreateNode(XmlNodeType.Element, strName, "");
                    SetNodeAttribute(nodeRetVal, strAttrName, strAttrValue);
                    if (node == null) {
                        m_xmlDoc.DocumentElement.AppendChild(nodeRetVal);
                    } else {
                        node.AppendChild(nodeRetVal);
                    }
                }
            }
            return(nodeRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadGridState:              Load the state of a datagrid
        /// </summary>
        /// <param name="node">         Node in which to get the information</param>
        /// <param name="dataGrid">     Datagrid</param>
        //
        //*********************************************************
        private void LoadGridState(XmlNode node, DataGrid dataGrid) {
            string              strVersion;
            XmlNode             nodeTblStyle;
            XmlNode             nodeColStyle;
            string              strColWidth;
            
            strVersion = GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                foreach (DataGridTableStyle styleTbl in dataGrid.TableStyles) {
                    nodeTblStyle = FindNode(node, styleTbl.MappingName);
                    foreach (DataGridColumnStyle styleCol in styleTbl.GridColumnStyles) {
                        nodeColStyle = FindNode(nodeTblStyle, styleCol.MappingName);
                        strColWidth  = GetNodeAttribute(nodeColStyle, "WIDTH");
                        if (strColWidth != null) {
                            try {
                                styleCol.Width = Int32.Parse(strColWidth);
                            } catch(System.Exception) {
                            }
                        }
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadPanelState:             Load the state of a panel
        /// </summary>
        /// <param name="node">         Node in which to get the information</param>
        /// <param name="panel">        Panel</param>
        //
        //*********************************************************
        private void LoadPanelState(XmlNode node, Panel panel) {
            string  strVersion;
            
            strVersion = GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                panel.Left   = GetNodeAttribute(node, "LEFT", panel.Left);
                panel.Top    = GetNodeAttribute(node, "TOP", panel.Top);
                panel.Width  = GetNodeAttribute(node, "WIDTH", panel.Width);
                panel.Height = GetNodeAttribute(node, "HEIGHT", panel.Height);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadSplitContainerState:        Load the state of a split container
        /// </summary>
        /// <param name="node">             Node in which to get the information</param>
        /// <param name="splitContainer">   Split Container</param>
        //
        //*********************************************************
        private void LoadSplitContainerState(XmlNode node, SplitContainer splitContainer) {
            string  strVersion;
            
            strVersion = GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                splitContainer.BorderStyle      = (BorderStyle)GetNodeAttribute(node, "BORDERSTYLE",       (int)splitContainer.BorderStyle);
                splitContainer.Top              = GetNodeAttribute(node, "TOP",               splitContainer.Top);
                splitContainer.Left             = GetNodeAttribute(node, "LEFT",              splitContainer.Left);
                splitContainer.Panel1Collapsed  = GetNodeAttribute(node, "PANEL1COLLAPSED",   splitContainer.Panel1Collapsed);
                splitContainer.Panel2Collapsed  = GetNodeAttribute(node, "PANEL2COLLAPSED",   splitContainer.Panel2Collapsed);
                splitContainer.Width            = GetNodeAttribute(node, "WIDTH",             splitContainer.Width);
                splitContainer.Height           = GetNodeAttribute(node, "HEIGHT",            splitContainer.Height);
                splitContainer.SplitterDistance = GetNodeAttribute(node, "SPLITTERDISTANCE",  splitContainer.SplitterDistance);
                splitContainer.Dock             = (DockStyle)GetNodeAttribute(node, "Dock",              (int)splitContainer.Dock);
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadControlState:           Load the state of a control
	    /// </summary>
	    /// <param name="node">         Father node</param>
	    /// <param name="ctl">          Form to load the state</param>
	    //
        //*********************************************************
        public virtual void LoadControlState(XmlNode node, System.Windows.Forms.Control ctl) {
            IUIConfiguration    uiConfig;
            XmlNode             nodeChild;

            if (IsControlSerializable(ctl)) {
                nodeChild = FindNode(node, "CTL" + ctl.Name);
                if (ctl is IUIConfiguration) {
                    uiConfig  = (IUIConfiguration)ctl;
                    uiConfig.LoadState(this, nodeChild);
                } else if (ctl is DataGrid) {
                    LoadGridState(nodeChild, (DataGrid)ctl);
                } else if (ctl is Panel 
                           && !(ctl.Parent is SplitContainer)
                          ) {
                    LoadPanelState(nodeChild, (Panel)ctl);
                } else if (ctl is SplitContainer) {
                    LoadSplitContainerState(nodeChild, (SplitContainer)ctl);
                }
                foreach (System.Windows.Forms.Control ctlChild in ctl.Controls) {
                    LoadControlState(nodeChild, ctlChild);
                }
            }
        }

        //*********************************************************
        //
	    /// <summary>
        /// FindFormNode:                   Find the xml node for a 
        ///                                 form 
	    /// </summary>
        /// <param name="frm">Form where to find node</param>
        /// <returns>
        /// Node found (if the node doesn't exist, a default node is created)
        /// </returns>
	    //
        //*********************************************************
        public virtual XmlNode FindFormNode(System.Windows.Forms.Form frm) {
            XmlNode             nodeRetVal;           

            nodeRetVal = FindNode(null, "FORM" + frm.Name);
            return(nodeRetVal);
        }    

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFormWindowState:            Load the window state of a form 
	    /// </summary>
	    /// <param name="frm">              Form to load the state</param>
	    //
        //*********************************************************
        public virtual void LoadFormWindowState(System.Windows.Forms.Form frm) {
            XmlNode             node;

            node = FindFormNode(frm);
            LoadFormWindowState(frm, node);
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFormWindowState:            Load the window state of a form 
	    /// </summary>
	    /// <param name="frm">              Form to load the state</param>
	    /// <param name="node">             Node containing the form position state</param>
	    //
        //*********************************************************
        protected virtual void LoadFormWindowState(System.Windows.Forms.Form frm, XmlNode node) {
            string          strVal;
            FormWindowState eState = FormWindowState.Normal;

            try {
                strVal = GetNodeAttribute(node, "WINDOWSTATE");
                if (strVal != null && strVal != String.Empty) {
                    eState = (FormWindowState)Int32.Parse(strVal);
                    if (frm.WindowState != eState) {
                        frm.WindowState = eState;
                    }
                }
            } catch(System.Exception) {
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFormPosState:               Load the state of a form position
	    /// </summary>
	    /// <param name="frm">              Form to load the state</param>
	    /// <param name="node">             Node containing the form position state</param>
	    /// <param name="ptsNorLocation">   Location of the window when in its normal state</param>
	    /// <param name="sizeNorSize">      Size of the window when in its normal state</param>
	    //
        //*********************************************************
        public virtual void LoadFormPosState(System.Windows.Forms.Form frm, XmlNode node, out System.Drawing.Point ptsNorLocation, out System.Drawing.Size sizeNorSize) {
            string          strVal;
            Rectangle       rectWorkingArea;
            int             iLeft   = 0;
            int             iTop    = 0;
            int             iWidth  = 100;
            int             iHeight = 100;

            ptsNorLocation = frm.Location;
            sizeNorSize    = frm.Size;
            try {
                strVal = GetNodeAttribute(node, "LEFT");
                if (strVal != null) {
                    iLeft = Int32.Parse(strVal);
                }
                strVal = GetNodeAttribute(node, "TOP");
                if (strVal != null) {
                    iTop = Int32.Parse(strVal);
                }
                strVal = GetNodeAttribute(node, "WIDTH");
                if (strVal != null) {
                    iWidth = Int32.Parse(strVal);
                }
                strVal = GetNodeAttribute(node, "HEIGHT");
                if (strVal != null) {
                    iHeight = Int32.Parse(strVal);
                }
            } catch(System.Exception) {
            }
            rectWorkingArea = SystemInformation.WorkingArea;
            if (!rectWorkingArea.Contains(iLeft, iTop)){
                iLeft = 0;
                iTop  = 0;
            }
            if (iWidth > rectWorkingArea.Width) {
                iWidth = rectWorkingArea.Width;                
            }
            if (iHeight > rectWorkingArea.Height) {
                iHeight = rectWorkingArea.Height;                
            }
            ptsNorLocation  = frm.Location;
            sizeNorSize     = frm.Size;
            frm.Left        = iLeft;
            frm.Top         = iTop;
            frm.Width       = iWidth;
            frm.Height      = iHeight;
            ptsNorLocation  = frm.Location;
            sizeNorSize     = frm.Size;
            if (!DoNotLoadWindowState) {
                LoadFormWindowState(frm, node);
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadFormState:                  Load the state of a form
	    /// </summary>
	    /// <param name="frm">              Form to load the state</param>
	    //
        //*********************************************************
        public virtual void LoadFormState(System.Windows.Forms.Form frm) {
            XmlNode             node;           
            IUIConfiguration    uiConfig;
            string              strVersion;
            RegistredForm       regForm;

            node = FindNode(null, "FORM" + frm.Name);
            if (m_hashRegistredForm.ContainsKey(frm)) {
                regForm     = m_hashRegistredForm[frm] as RegistredForm;
                strVersion  = GetNodeAttribute(node, "VERSION");
                if (strVersion != null && strVersion == "1") {
                    LoadFormPosState(frm, node, out regForm.m_ptsNorLocation, out regForm.m_sizeNorSize);
                }
            }
            if (frm is IUIConfiguration) {
                uiConfig = (IUIConfiguration)frm;
                uiConfig.LoadState(this, node);
            }
            foreach (System.Windows.Forms.Control ctl in frm.Controls) {
                LoadControlState(node, ctl);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SaveGridState:              Save the state of a datagrid
        /// </summary>
        /// <param name="node">         Node in which to save the information</param>
        /// <param name="dataGrid">     Datagrid</param>
        //
        //*********************************************************
        private void SaveGridState(XmlNode node, DataGrid dataGrid) {
            XmlNode             nodeTblStyle;
            XmlNode             nodeColStyle;

            node.RemoveAll();
            SetNodeAttribute(node, "VERSION", "1");
            foreach (DataGridTableStyle styleTbl in dataGrid.TableStyles) {
                nodeTblStyle = FindNode(node, styleTbl.MappingName);
                foreach (DataGridColumnStyle styleCol in styleTbl.GridColumnStyles) {
                    nodeColStyle = FindNode(nodeTblStyle, styleCol.MappingName);
                    SetNodeAttribute(nodeColStyle, "WIDTH", styleCol.Width.ToString());
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SavePanelState:             Save the state of a panel
        /// </summary>
        /// <param name="node">         Node in which to save the information</param>
        /// <param name="panel">        Panel</param>
        //
        //*********************************************************
        private void SavePanelState(XmlNode node, Panel panel) {
            node.RemoveAll();
            SetNodeAttribute(node, "VERSION", "1");
            SetNodeAttribute(node, "LEFT", panel.Left);
            SetNodeAttribute(node, "TOP", panel.Top);
            SetNodeAttribute(node, "WIDTH", panel.Width);
            SetNodeAttribute(node, "HEIGHT", panel.Height);
        }

        //*********************************************************
        //
        /// <summary>
        /// SaveSplitContainerState:        Save the state of a split container
        /// </summary>
        /// <param name="node">             Node in which to save the information</param>
        /// <param name="splitContainer">   Split Container</param>
        //
        //*********************************************************
        private void SaveSplitContainerState(XmlNode node, SplitContainer splitContainer) {
            node.RemoveAll();
            SetNodeAttribute(node, "VERSION",           "1");
            SetNodeAttribute(node, "BORDERSTYLE",       (int)splitContainer.BorderStyle);
            SetNodeAttribute(node, "Dock",              (int)splitContainer.Dock);
            SetNodeAttribute(node, "TOP",               splitContainer.Top);
            SetNodeAttribute(node, "LEFT",              splitContainer.Left);
            SetNodeAttribute(node, "PANEL1MINSIZE",     splitContainer.Panel1MinSize);
            SetNodeAttribute(node, "PANEL1COLLAPSED",   splitContainer.Panel1Collapsed);
            SetNodeAttribute(node, "PANEL2MINSIZE",     splitContainer.Panel2MinSize);
            SetNodeAttribute(node, "PANEL2COLLAPSED",   splitContainer.Panel2Collapsed);
            SetNodeAttribute(node, "WIDTH",             splitContainer.Width);
            SetNodeAttribute(node, "HEIGHT",            splitContainer.Height);
            SetNodeAttribute(node, "SPLITTERDISTANCE",  splitContainer.SplitterDistance);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsControlSerializable:      Check if the control is serializable
        /// </summary>
        /// <param name="ctl">          The control</param>
        /// <returns>
        /// True:   Serializable control
        /// False:  Not a serializable control
        /// </returns>
        //
        //*********************************************************
        protected virtual bool IsControlSerializable(System.Windows.Forms.Control ctl) {
            return (ctl is IUIConfiguration || 
                    ctl is DataGrid         ||
                    ctl is Panel            ||
                    ctl.Controls.Count > 0); 
        }

        //*********************************************************
        //
	    /// <summary>
        /// FindCtrlNode:                   Find the xml node for a 
        ///                                 control 
	    /// </summary>
        /// <param name="parentNode">Parent Node where to find the child node</param>
        /// <param name="ctl">Control where to find node</param>
        /// <returns>
        /// Node found (if the node doesn't exist, a default node is created)
        /// </returns>
	    //
        //*********************************************************
        public virtual XmlNode FindCtrlNode(XmlNode parentNode, System.Windows.Forms.Control ctl) {
            XmlNode             nodeRetVal;

            nodeRetVal = FindNode(parentNode, "CTL" + ctl.Name);
            return(nodeRetVal);
        } 

        //*********************************************************
        //
	    /// <summary>
	    /// SaveControlState:           Load the state of a control
	    /// </summary>
	    /// <param name="node">         Father node</param>
	    /// <param name="ctl">          Form to load the state</param>
	    //
        //*********************************************************
        public virtual void SaveControlState(XmlNode node, System.Windows.Forms.Control ctl) {
            IUIConfiguration    uiConfig;
            XmlNode             nodeChild = null;
            
             if (IsControlSerializable(ctl)) {
                 nodeChild = FindCtrlNode(node, ctl);
                if (ctl is IUIConfiguration) {
                    uiConfig  = (IUIConfiguration)ctl;
                    uiConfig.SaveState(this, nodeChild);
                } else if (ctl is DataGrid) {
                    SaveGridState(nodeChild, (DataGrid)ctl);
                } else if (ctl is Panel
                            && !(ctl.Parent is SplitContainer)
                          ) {
                    SavePanelState(nodeChild, (Panel)ctl);
                } else if (ctl is SplitContainer) {
                    SaveSplitContainerState(nodeChild, (SplitContainer)ctl);
                }
                foreach (System.Windows.Forms.Control ctlChild in ctl.Controls) {
                    SaveControlState(nodeChild, ctlChild);
                }
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SaveFormPosState:               Save the state of a form position
	    /// </summary>
	    /// <param name="frm">              Form to load the state</param>
	    /// <param name="node">             Node containing the form position state</param>
	    /// <param name="ptsNorLocation">   Location of the window when in its normal state</param>
	    /// <param name="sizeNorSize">      Size of the window when in its normal state</param>
	    //
        //*********************************************************
        public virtual void SaveFormPosState(System.Windows.Forms.Form frm, XmlNode node, System.Drawing.Point ptsNorLocation, System.Drawing.Size sizeNorSize) {
            SetNodeAttribute(node, "WINDOWSTATE", ((int)frm.WindowState).ToString());
            SetNodeAttribute(node, "WIDTH", sizeNorSize.Width.ToString());
            SetNodeAttribute(node, "HEIGHT",sizeNorSize.Height.ToString());
            SetNodeAttribute(node, "LEFT",  ptsNorLocation.X.ToString());
            SetNodeAttribute(node, "TOP",   ptsNorLocation.Y.ToString());
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SaveFormState:              Save the state of a form
	    /// </summary>
	    /// <param name="frm">          Form to save the state</param>
	    //
        //*********************************************************
        public virtual void SaveFormState(System.Windows.Forms.Form frm) {
            XmlNode             node;
            IUIConfiguration    uiConfig;
            RegistredForm       regForm;
            
            node = FindNode(null, "FORM" + frm.Name);
            node.RemoveAll();
            if (m_hashRegistredForm.ContainsKey(frm)) {
                regForm = m_hashRegistredForm[frm] as RegistredForm;
                SetNodeAttribute(node, "VERSION", "1");
                SaveFormPosState(frm, node, regForm.m_ptsNorLocation, regForm.m_sizeNorSize);
            }
            if (frm is IUIConfiguration) {
                uiConfig = (IUIConfiguration)frm;
                uiConfig.SaveState(this, node);
            }
            foreach (System.Windows.Forms.Control ctl in frm.Controls) {
                SaveControlState(node, ctl);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RegistredForm_Load:         Called when the registred form is loaded
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void RegistredForm_Load(object sender, EventArgs e) {
            LoadFormState(sender as Form);
        }

        //*********************************************************
        //
        /// <summary>
        /// RegistredForm_PosSizeChanged:   Called when a registred form position or size is changing
        /// </summary>
        /// <param name="sender">           Sender object</param>
        /// <param name="e">                Event argument</param>
        //
        //*********************************************************
        private void RegistredForm_PosSizeChanged(object sender, EventArgs e) {
            Form            frm;
            RegistredForm   regForm;
            
            frm = sender as Form;
            if (frm.WindowState == FormWindowState.Normal && m_hashRegistredForm.ContainsKey(frm)) {
                regForm                     = m_hashRegistredForm[frm] as RegistredForm;
                regForm.m_ptsNorLocation    = frm.Location;
                regForm.m_sizeNorSize       = frm.Size;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RegistredForm_Closing:      Called when the registred form is closing
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void RegistredForm_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
            if (!e.Cancel) {
                SaveFormState(sender as Form);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RegistredForm_Closed:       Called when the registred form is closed
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void RegistredForm_Closed(object sender, System.EventArgs e) {
            UnregisterForm(sender as Form);
        }

        //*********************************************************
        //
        /// <summary>
        /// m_frmFather_Closed:         Called when the attached form is closed
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void m_frmFather_Closed(object sender, System.EventArgs e) {
            SaveToFile();
        }
    }; // End class
};  // End namespace